

CREATE OR REPLACE FUNCTION fLogin() 
RETURNS VARCHAR AS 
$BODY$
DECLARE
	_xregid		VARCHAR;
	_xregdesc	VARCHAR;
	_xregval	VARCHAR;
	_count		INT;
BEGIN
	SELECT INTO _xregid, _xregval xregid, xregval FROM xreg WHERE xregid='TRIAL';

	_count = CAST(_xregval AS INT);
	_count = _count+1;

	UPDATE xreg SET xregval=CAST(_count AS VARCHAR) WHERE xregid='TRIAL';

	RETURN _xregval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




DROP AGGREGATE SUM (TEXT);

CREATE OR REPLACE FUNCTION fNoteCat(_text1 TEXT, _text2 TEXT) 
RETURNS TEXT AS
$BODY$
DECLARE
	_retval		TEXT;
BEGIN
	IF ((_text1 IS NULL OR _text1='') AND (_text2 IS NULL OR _text2='')) THEN
		_retval:='';
	ELSIF (_text1 IS NOT NULL AND (_text2 IS NULL OR _text2='')) THEN
		_retval:=_text1;
	ELSIF ((_text1 IS NULL OR _text1='') AND _text2 IS NOT NULL) THEN
		_retval:=_text2;
	ELSIF ((_text1 IS NOT NULL AND _text1<>'') AND (_text2 IS NOT NULL AND _text2<>'')) THEN
		_retval:=_text1||E'\n'||_text2;
	ELSE
		_retval:='';
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE AGGREGATE sum (BASETYPE=text, SFUNC=fnotecat, STYPE=text, INITCOND='');




CREATE OR REPLACE FUNCTION fQtyXDesc (
	_ItemID		VARCHAR, 
	_QtyX		NUMERIC(15,4)
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_UnitX		VARCHAR(5);
	_ConvX		NUMERIC(15,4);
	_tmpQty		NUMERIC(15,4);

	_isMinus	BOOLEAN;
	_UseQtyX	BOOLEAN;

	_RetVal 	VARCHAR;
BEGIN

	SELECT INTO _UnitX, _ConvX, _useQtyX unitx, convx, useqtyx
	FROM item WHERE itemid=_ItemID;

	IF NOT FOUND OR _UseQtyX=FALSE THEN
		_RetVal:=CAST(0 as VARCHAR);
		RETURN _RetVal;
	END IF;

	IF _QtyX<0 THEN
		_isMinus:=true;
	END IF;

	_RetVal='';
	_tmpQty := abs(_QtyX);
	_RetVal:=CAST(_tmpQty as integer)||' ' || _UnitX;

	IF _isMinus=true THEN
		_RetVal:='-' || _RetVal;
	END IF;
	RETURN _RetVal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPriceLog() 
RETURNS SETOF TRIGGER AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret 			SPStatus%ROWTYPE;

	_oldPrice1		NUMERIC(19,4);
	_oldDiscExp1		VARCHAR;
	_oldPrice2		NUMERIC(19,4);
	_oldDiscExp2		VARCHAR;
	_oldPrice3		NUMERIC(19,4);
	_oldDiscExp3		VARCHAR;

BEGIN
	_retval:=(0, 'OK');

	IF (TG_OP = 'INSERT') THEN
		_oldPrice1:=0;
		_oldDiscExp1:=NULL;
		_oldPrice2:=0;
		_oldDiscExp2:=NULL;
		_oldPrice3:=0;
		_oldDiscExp3:=NULL;
	ELSIF (TG_OP = 'UPDATE') THEN
		_oldPrice1:= OLD.price1;
		_oldDiscExp1:= OLD.disc1exp;
		_oldPrice2:= OLD.price2;
		_oldDiscExp2:= OLD.disc2exp;
		_oldPrice3:= OLD.price3;		
		_oldDiscExp3:= OLD.disc3exp;
	END IF;

	INSERT INTO pricelog (pricelogdate, itemid, usrid, prclvlid, crcid, price1old, disc1expold, price1new, disc1expnew, price2old, disc2expold, price2new, disc2expnew, price3old, disc3expold, price3new, disc3expnew)
		VALUES (CURRENT_TIMESTAMP, NEW.itemid, NEW.usrid, NEW.prclvlid, NEW.crcid, _oldPrice1, _oldDiscExp1, NEW.price1, NEW.disc1exp, _oldPrice2, _oldDiscExp2, NEW.price2, NEW.disc2exp, _oldPrice3, _oldDiscExp3, NEW.price3, NEW.disc3exp);
	IF (NOT FOUND) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Gagal Buat Price Log (spPriceLog)';
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;

	RETURN NEW;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

DROP TRIGGER pricelog ON prc;
CREATE TRIGGER pricelog AFTER INSERT OR UPDATE ON prc FOR EACH ROW EXECUTE PROCEDURE spPriceLog();



CREATE OR REPLACE FUNCTION spCons_Item_Sisa(
	__ConsNo 		VARCHAR, 
	__ConsDNo 		INT,
	__ItemID		VARCHAR, 
	__PID			VARCHAR,
	__Qty1			NUMERIC
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE
	_spret 			SPStatus%ROWTYPE;
	_retval			SPStatus%ROWTYPE;

	_Active			BOOLEAN;
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_ConsDPID		VARCHAR;
	_qty1_SellIn		NUMERIC(15,4);	-- konversi dari Qty-Unit
	_qty1_SellOut		NUMERIC(15,4); -- from consd
	_qty1_CRet		NUMERIC(15,4); -- from consd
	_qty1_tolerance		NUMERIC(15,4);
	_qty1_sisa		NUMERIC(15,4);

	_qty1_to_order_now	NUMERIC(15,4);
	_isstock		BOOLEAN;
BEGIN
	SELECT INTO 
		_Active 
		Active 
	FROM Cons WHERE ConsNo=__ConsNo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Sell In #% tidak ditemukan (spCons_Item_Sisa)', __ConsNo;
	END IF;

	IF (_Active=FALSE) THEN
		RAISE EXCEPTION 'Status Sell In #% tidak-Aktif (spCons_Item_Sisa)', __ConsNo;
	END IF;

	SELECT INTO 
		_Qty, _Unit, _ConsDPID, _Qty1_SellOut, _Qty1_CRet
		QtySellIn, Unit, PID, QtySellOut, QtyCRet
	FROM ConsD
	WHERE 
	ConsNo=__ConsNo AND ConsDNo=__ConsDNo
	AND ItemID=__ItemID;

	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Sell In #%, detail ke-% tidak ditemukan! (spCons_Item_Sisa)', __ConsNo, __ConsDNo;
	END IF;

	_qty1_SellIn:=fQtyZToQty1(__ItemID, _Qty, _Unit);
	_qty1_sisa:=_qty1_SellIn-_Qty1_SellOut-_Qty1_CRet;

	RETURN _qty1_sisa;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCRet_New(_CRetNo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CustID			VARCHAR(15);
	_CRetDate		DATE;
	_Total			NUMERIC(19,4);
	_CashID			VARCHAR(15);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;

	_CRetDNo		SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_Conv			NUMERIC(15,4);
	_QtyX			NUMERIC(15,4);
	_Cost			NUMERIC(19,4);
	_ItemType		VARCHAR(5);

	_StockANo		VARCHAR(15);
	_StockANote		VARCHAR(50);
	_StockADNo		SMALLINT;

	_CRetQty		NUMERIC(15,4);
	_CRetQtyX		NUMERIC(15,4);
	_ItemsCost		NUMERIC(19,4);
	_QtySisa		NUMERIC(15,4);
	_Qty1			NUMERIC(15,4);

	_CashAmt		NUMERIC(19,4);
	_CashANote		VARCHAR(50);

	_IsContra		BOOLEAN;
	_regval			VARCHAR(30);

	_RcvNo			VARCHAR(15);
	_CrcID			VARCHAR(5);

	_SpaidNo		INT2;
	_SpaidAmt		NUMERIC(19,4);
	_Bayar			NUMERIC(19,4);

	_Freight		DECIMAL(19,4);
	_Others			DECIMAL(19,4);
	_Disc			DECIMAL(19,4);
	_Count1			INT;
	_Count2			INT;
	_JnlNo1			VARCHAR(15);
	_JnlNote		VARCHAR;
	_ListPrice		DECIMAL(19,4);
	_DebtAmt		DECIMAL(19,4);
	_ExcRate		DECIMAL(19,4);
	_DebtNote		VARCHAR(50);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
   	_DeptID     		VARCHAR(5);
   	_PrjID      		VARCHAR(5);

	cCRetD			REFCURSOR;
	cCRetD2			REFCURSOR;
	cCRet			REFCURSOR;
	cCRetAcc		REFCURSOR;
	
	_AccAsset		VARCHAR(10);
	_AccDisc		VARCHAR(10);
	_AccInv			VARCHAR(10);
	_AccCOGS		VARCHAR(10);
	_AccFreight		VARCHAR(10);
	_AccCRet		VARCHAR(10);
	_AccAmt			DECIMAL(19,4);    
	_MTaxAmt		DECIMAL(19,4);  
	_Round			INT;
	

	_ConsNo			VARCHAR;
	_ConsDNo		SMALLINT;

	_unit1			VARCHAR;
	_unit2			VARCHAR;
	_unit3			VARCHAR;
	_conv2			DECIMAL;
	_conv3			DECIMAL;
	_listpriceAsli		DECIMAL;
	_QtyAsli		DECIMAL;
	_UnitAsli		VARCHAR;

	cCons			REFCURSOR;
	cTax			REFCURSOR;

	_isDraft		BOOLEAN;
	_LockDate		DATE;

	_ItemIDCheck		VARCHAR;
	_CRetDTaxAmt		NUMERIC(19,4);
	_CRetDTaxAmtForex	NUMERIC(19,4);
	_AccCRetTax		VARCHAR;
	_TotalTax		NUMERIC(19,4);
	_SelisihTax		NUMERIC(19,4);
	_i			INT;
	_Subtotal		NUMERIC(19,4);
	_SelisihBasePrice	NUMERIC(19,4);
	_TotalBasePrice		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_CRetStatusYes		VARCHAR(1);

	_BTotal			NUMERIC(19,4); -- base currency total
	_BSubtotal		NUMERIC(19,4); -- base currency subtotal
	_MSubtotal		NUMERIC(19,4);
	_MBFTaxAmt		NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt		NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt		NUMERIC(19,4);
	_TotalTaxAmt		NUMERIC(19,4);
	_BTotalTaxAmt		NUMERIC(19,4);
	_TotalDiscAmt		NUMERIC(19,4);
	_TotalDisc2Amt		NUMERIC(19,4);
	_DSubtotal		NUMERIC(19,4);
	_DBSubtotal		NUMERIC(19,4);
	
	_AccRateLR		VARCHAR; -- akun laba/rugi selisih kurs
	_RateLR			NUMERIC(19,4); -- nilai laba/rugi selisih kurs
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_CRetStatusYes:='Y';


	SELECT INTO _isDraft, _PerID, _BranchID, _CustID, _CRetDate, _BTotal, _Total, _CrcID, _CashID, 
		_CrtBy, _CrtDate, _UpdBy, _UpdDate, _ConsNo, _Freight, _MBDiscAmt, _Disc, _AccDisc, _AccFreight, 
		_ExcRate, _MBFTaxAmt, _MTaxAmt, _Others, _IsContra, _BSubtotal, _SubTotal, _FisRate

		isdraft, perid, branchid, custid, cretdate, basetotal, total, crcid, cashid, 
		crtby, crtdate, updby, upddate, consno, freight, basediscamt, discamt, accsdisc, accfreight, 
		excrate, basftaxamt, taxamt, others, iscontra, basesubtotal, subtotal, fisrate
	FROM cret
	WHERE cretno=_CRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Retur Konsinyasi #'||_CRetNo||' tidak ditemukan (spCRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CRET';
	IF (_LockDate IS NOT NULL AND _LockDate>_CRetDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Retur Konsinyasi Melebihi Batas LockDate! (spCRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_ItemsCost:=0;

	OPEN cCRetD FOR
		SELECT d.CRetDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, i.ItemType, d.ConsNo, d.ConsDNo
		FROM CRetD d JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.CRetNo=_CRetNo;

	_StockADNo=0;
	_i:=0;
	LOOP
		FETCH cCRetD INTO _CRetDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ItemType, _ConsNo, _ConsDNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _QtySisa * FROM spCons_Item_Sisa(_ConsNo, _ConsDNo, _ItemID, _PID, _Qty);
		_Qty1:=fQtyZToQty1(_ItemID, _Qty, _Unit);

		IF (_Qty1>_QtySisa) THEN
			_retval.ErrNo:=50056;
			_retval.ErrMsg:='Item:'|| _ItemID ||', Qty Retur > Qty Sisa (spCret_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		UPDATE CONSD SET QtyCRet=QtyCRet+_Qty1 WHERE ItemID=_ItemID AND PID=_PID AND ConsNo=_ConsNo AND ConsDNo=_ConsDNo;

		_i:=_i+1;

		IF (_i=1) THEN		-- generate StockA header
			_StockANo:=fTransNoGet('STOCKA');
			_StockANote='Retur Konsinyasi '||fBPName(_CustID)||':'||_CRetNo;
			SELECT INTO _spret * FROM spStockA_New(
				_StockANo,_CustID, 'CRET', _CRetDate, 'O', 'CRET', _CRetNo, _StockANote, TRUE, 
				_CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID								
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;		
		END IF;			

		_StockADNo=_StockADNo+1;

		SELECT INTO _Cost cost FROM ConsD WHERE ItemID=_ItemID AND ConsNo=_ConsNo AND ConsDNo=_ConsDNo;

		_ItemsCost=_ItemsCost+(_Qty*_Cost);

		IF (_Cost IS NULL) THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg:='Error menghitung HPP Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spCRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spStockAD_New(
			_CRetDate, 'CRET', _StockANo, _StockADNo, _WhID, 
			_ItemID, _PID, _Qty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, TRUE);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_Conv:=fItemUnit_Conv(_ItemID, _Unit);

		UPDATE CRetD SET Cost=_Cost, Conv=_Conv WHERE CRetNo=_CRetNo AND CRetDNo=_CRetDNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50010;
			_retval.ErrMsg:='Error UPDATE CRetD:Cost (spCRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	END LOOP;
	CLOSE cCRetD;

	_IsContra=FALSE;

	UPDATE CRet SET CostTot=_ItemsCost, IsContra=_IsContra WHERE CRetNo=_CRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50010;
		_retval.ErrMsg:='Error UPDATE CRet:CostTot (spCRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

--- Update ConsStatus
	OPEN cCRetD FOR
		SELECT DISTINCT ConsNo FROM CRetD 
		WHERE CretNo=_CretNo AND (ConsNo IS NOT NULL AND ConsNo<>'');
	LOOP
		FETCH cCRetD INTO _ConsNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spSetConsStatus(_ConsNo);
		IF (_spret.ErrNo<>0) THEN
			RAISE EXCEPTION '%', _spret.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cCRetD;

/***************************** AKUNTANSI **************************************/
	IF isAccInstalled() THEN
		_JnlNo1:=fTransNoGet('JURNAL');
		_JnlNote:= 'Retur Konsinyasi '||fBPName(_CustID)||':'||_CRetNo;
		SELECT INTO _Round * FROM fReg('ROUND');
		IF (_Round IS NULL) THEN
			_Round:=0;
		END IF;

		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo1, 'CRET', _CRetDate, 'CRET', _CRetNo,
			_JnlNote, TRUE, TRUE, CAST(0 AS NUMERIC(19,4)),
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _Excrate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_AccAsset:=fBpAcc(_CustID, _CrcID, 'AR');

		_Count1:=0;
		_Count2:=0;

		SELECT INTO 
			_TotalBasePrice 
			Round(Sum(Qty*(BasePrice-COALESCE(DiscAmt,0))*_ExcRate),_Round) 
		FROM CRetD WHERE CRetNo=_CRetNo AND (fItemAcc(ItemID, 'CRET') IS NOT NULL AND fItemAcc(ItemID, 'CRET')<>'');

		_SelisihBasePrice:=(_Subtotal*_ExcRate)-_TotalBasePrice;
		_i:=0;
		--******************* JURNAL CRet: DEBET *************************--
		OPEN cCRetAcc FOR
			SELECT fItemAcc(itemid, 'INV'), deptid, prjid, SUM(basesubtotal), SUM(subtotal) 
			FROM cretd 
			WHERE cretno=_CRetNo 
			GROUP BY fItemAcc(itemid, 'INV'), deptid, prjid;
		LOOP
			FETCH cCRetAcc INTO _AccCRet, _DeptID, _PrjID, _DBSubtotal, _DSubtotal;
			EXIT WHEN NOT FOUND;

			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccCRet, 'D', _DBSubtotal, _DeptID, _PrjID, _DSubtotal);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cCRetAcc;


		_Disc:=COALESCE(_Disc, 0);
		IF (_Disc<>0) THEN
			IF (_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('SALEDISC');
			END IF;

			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccDisc, 'D', _MBDiscAmt*-1, NULL, NULL,_Disc*-1);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		SELECT INTO _TotalTax Round(sum(d.taxamt*Qty*_ExcRate),_Round) 
		FROM CRetD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.CRetNo=_CRetNo and d.taxid IS NOT NULL;

		_SelisihTax:=(_MTaxAmt*_ExcRate)-_TotalTax;
		_i:=0;

		/*POSTING TAX DI GRUP IN SESUAI PAJAK TIAP2 ITEM*/
		OPEN cTax FOR
			SELECT SUM(basftotaltaxamt), SUM(totaltaxamt), accsaletax, deptid, prjid
			FROM cretd d JOIN tax ON d.taxid=tax.taxid
			WHERE d.cretno=_CRetNo and d.taxid IS NOT NULL
			GROUP BY accsaletax, deptid, prjid;
		LOOP
			FETCH cTax INTO _CRetDTaxAmt, _CRetDTaxAmtForex, _AccCRetTax, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			_Count1:=_Count1+1;	
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccCRetTax, 'D', _CRetDTaxAmt, _DeptID, _PrjID, _CRetDTaxAmtForex);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;

		/** ==================== LABA RUGI SELISIH KURS ================================ */
		_RateLR:=(_BTotal+_MBDiscAmt)-(_BSubtotal+_MBFTaxAmt);
		IF (_RateLR!=0) THEN
			_Count1:=_Count1+1;
			_AccRateLR:=fAccDefault('PKKURS');
			IF (_AccRateLR IS NULL OR _AccRateLR='') THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Akun PK Selisih Kurs belum diset.'||E'\n'||'Silahkan disetting pada form Setting Akun';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccRateLR, 'D', _RateLR, _DeptID, _PrjID, NULL, _RateLR/_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;	

		IF (_Freight<>0) THEN
			IF (_AccFreight IS NULL) THEN
				_AccFreight:=fAccDefault('SALEFREIGHT');
			END IF;
			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccFreight, 'D', _Freight*_ExcRate, NULL, NULL, _Freight);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		IF (_Others<>0) THEN
			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, fAccDefault('PKROUND'), 'D', _Others*_ExcRate, NULL, NULL,_Others);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		/******************* JURNAL CRet: KREDIT ***************************/
		_Count1:=_Count1+1;		
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, fAccDefault('ARCONS'), 'K', _BTotal, _DeptID, _PrjID, _Total);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		/*---------- execute spJnlNew --------------*/
		SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	END IF;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCRet_Void (
	_CRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_CustID		VARCHAR(15);
	_ConsNo		VARCHAR(15);
	_ConsDNo	INT;
	_IsContra	BOOLEAN;
	_JnlNo		VARCHAR(15);
	_regval		VARCHAR(30);

	_itemid		VARCHAR(20);
	_qty		NUMERIC(15,4);
	_qtyx		NUMERIC(15,4);
	_unit		VARCHAR(5);
	_qtycret	NUMERIC(15,4);
	cCretD		refcursor;

	_DeliNo		VARCHAR(15);
	cCRetDeli	REFCURSOR;

	_CRetStatus	VARCHAR(1);
	_PID		VARCHAR;
	
	_isDraft	BOOLEAN;
	_CRetDate	DATE;
	_LockDate	DATE;
BEGIN
	SELECT into _CustID, _IsContra, _ConsNo, _CRetStatus, _CRetDate
		    CustID, IsContra, ConsNo, CRetStatus, CRetDate
	FROM CRet
	WHERE CRetNo=_CRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Retur/Memo-Piutang #'||_CRetNo||' tidak ditemukan (spCRet_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CRET';
	IF(_LockDate is not null AND _LockDate>_CRetDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Retur Melebihi Batas LockDate!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** CEK ISDRAFT */
	SELECT INTO _isDraft IsDraft FROM CRet WHERE CRetNo=_CRetNo;
	IF NOT FOUND THEN
		_isDraft:=FALSE;
	END IF;

	IF (_isDraft IS NULL) THEN
		_isDraft:=FALSE;
	END IF;

	IF (_isDraft=FALSE) THEN /** MULAI ISDRAFT */

		SELECT into _spret * FROM spStockA_Void_byRef('CRET', _CRetNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_ConsNo IS NOT NULL) THEN	
			/** do nothing, khan saat new nggak bikin bpa */
		ELSE		
			SELECT into _spret * FROM spBPA_Void_byRef('CRET', _CretNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		OPEN cCretD FOR
			SELECT itemid, qty, qtyx, PID, unit, consno, consdno from cretd where cretno=_CRetNo;
			LOOP
				FETCH cCretD INTO _itemid, _qty, _qtyx, _PID, _unit, _ConsNo, _ConsDNo;
				EXIT WHEN NOT FOUND;
					_qtycret:=fQtyZToQty1(_itemid,_qty,_unit);
					UPDATE consd set qtycret=qtycret-_qtycret, qtyxcret=qtyxcret-_qtyx WHERE consno=_ConsNo AND consdno=_ConsDNo AND itemid=_itemid AND PID=_PID;
			END LOOP;
		CLOSE cCretD;

		OPEN cCRetD FOR
			SELECT DISTINCT ConsNo FROM CRetD 
			WHERE CretNo=_CretNo AND (ConsNo IS NOT NULL AND ConsNo<>'');
		LOOP
			FETCH cCRetD INTO _ConsNo;
			EXIT WHEN NOT FOUND;

			SELECT INTO _spret * FROM spSetConsStatus(_ConsNo);
			IF (_spret.ErrNo<>0) THEN
				RAISE EXCEPTION '%', _spret.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cCRetD;

	END IF; /** END IF ISDRAFT */

	IF (isAccInstalled() AND _isDraft=FALSE) THEN
		SELECT into _spret * from spJnl_Void_byRef('CRET',_CRetNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM CRetD WHERE CRetNo=_CRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Delete CRetD (spCRet_Void)';
	END IF;
	
	DELETE FROM CRet WHERE CRetNo=_CRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error Delete CRet (spCRet_Void)';
	END IF;	

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSetConsStatus(__Consno VARCHAR)
RETURNS SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	sisa 		NUMERIC(19,4);
	_ConsDNo 	VARCHAR;
	_ConsNo	VARCHAR;
	_Qty1_O		NUMERIC(15,4);		/** Sell In **/
	_Qty1_D		NUMERIC(15,4);		/** Sell Out **/
	_Qty1_B		NUMERIC(15,4);		/** Retur **/

	_Qty1_R1	NUMERIC(15,4);		/** R1 = Sell In - Sell Out ==> untuk menentukan ConsStatus **/
	_Qty1_R2	NUMERIC(15,4);		/** R2 = Sell IN - Sell Out - Retur ==> untuk menentukan Active **/

	_Expired	BOOLEAN;
	_IsFull		BOOLEAN;
	_IsPartial	BOOLEAN;
	_IsClear	BOOLEAN;
	_Active_orig	BOOLEAN;
	_Active		BOOLEAN;
	_ConsStatus	VARCHAR(1);
	cConsD		REFCURSOR;
BEGIN
	

	IF (__Consno IS NULL OR __Consno='') THEN	-- utk melayani SP lain yg memang gak nge-check valuenya, kita ngalahin ngecheck disini
		_retval:=(0,'OK');
		RETURN _retval;
	END IF;


	SELECT into _Active_orig Active from CONS WHERE ConsNo=__Consno;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Konsinyasi tidak ditemukan % (spCons_ChNUp_Status)',__Consno;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_IsFull=true;			/** default ConsStatus=FULL ***/
	_IsPartial=false;		/** default ConsStatus=FULL ***/
	_IsClear=true;			/** default CLOSED **/
	_Active=true;			/** default OPEN **/

	OPEN cConsD FOR
	SELECT ConsDNo, fQtyZToQty1(ItemID, QtySellIn, Unit), QtySellOut, QtyCRet from consd WHERE ConsNo=__Consno;
	LOOP
		FETCH cConsD INTO _ConsDNo, _Qty1_O, _Qty1_D, _Qty1_B;
		EXIT WHEN NOT FOUND;

		IF(_Qty1_O<0) THEN
			_Qty1_O:=_Qty1_O*(-1);
		END IF;

		IF(_Qty1_D<0) THEN
			_Qty1_D:=_Qty1_D*(-1);
		END IF;

		IF(_Qty1_B<0) THEN
			_Qty1_B:=_Qty1_B*(-1);
		END IF;

		_Qty1_R1 = _Qty1_O - _Qty1_D;
		_Qty1_R2 = _Qty1_O - _Qty1_D - _Qty1_B;

		IF (_Qty1_R1>0) THEN
			_IsFull:=false;
		END IF;

		IF (_Qty1_D>0) THEN
			_IsPartial:=true;
		END IF;

		IF (_Qty1_R2>0) THEN
			_IsClear:=false;
		END IF;
	END LOOP;
	CLOSE cConsD;

	IF (_IsFull=true) THEN _ConsStatus:='F';
	ELSIF ((_IsFull=false) AND (_IsPartial=true)) THEN _ConsStatus:='P';
	ELSIF ((_IsFull=false) AND (_IsPartial=false)) THEN _ConsStatus:='N';
	END IF;

	IF (_IsClear=true) THEN 
		_Active:=false;
	END IF;

	UPDATE Cons SET ConsStatus=_ConsStatus, Active=_Active WHERE ConsNo=__Consno;


_retval=(0, 'OK');
RETURN _retval;

END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION isItemUsedInTrans (
	__ItemID	VARCHAR,
	__isStock	BOOLEAN
) RETURNS BOOLEAN AS 
$BODY$
DECLARE
	_count		INT;
	_retval		BOOLEAN;
	_isStock	BOOLEAN;
BEGIN
	_retval:=FALSE;
	_isStock=COALESCE(__isStock,FALSE);

	IF (NOT(_retval) AND _isStock=TRUE) THEN

		SELECT INTO _count COUNT(*) FROM stockad WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
		END IF;

	ELSIF (NOT(_retval) AND _isStock=FALSE) THEN

		SELECT INTO _count COUNT(*) FROM prcvd WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM purcd WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM pretd WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM delid WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM saled WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM sretd WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM mfgmd WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM mfgpd WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM consd WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

		SELECT INTO _count COUNT(*) FROM cretd WHERE itemid=__ItemID;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
			RETURN _retval;
		END IF;

	END IF;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spUpdateReg () 
RETURNS TRIGGER AS
$BODY$
DECLARE
	_RegValue		reg.regval%TYPE;
	_CompRegValue		reg.regval%TYPE;

	_IntRegValue		INT2;
	_IntCompRegValue	INT2;
BEGIN
	IF (NEW.regid='ROUND') THEN
		_RegValue:=COALESCE(NEW.regval, '0');
		_IntRegValue:=CAST(_RegValue AS INT2);

		SELECT INTO _CompRegValue regval FROM reg WHERE regid='ROUND_TAX';
		IF NOT FOUND THEN
			_CompRegValue:='0';		
		END IF;
		_CompRegValue:=COALESCE(_CompRegValue, '0');
		_IntCompRegValue:=CAST(_CompRegValue AS INT2);

		IF (_IntRegValue<_IntCompRegValue) THEN
			RAISE EXCEPTION 'Nilai system setting#ROUND harus lebih dari atau sama dengan nilai system setting#ROUND_TAX ! (spUpdateReg)';
			RETURN OLD;
		END IF;
	ELSIF (NEW.regid='ROUND_TAX') THEN
		_RegValue:=COALESCE(NEW.regval, '0');
		_IntRegValue:=CAST(_RegValue AS INT2);

		SELECT INTO _CompRegValue regval FROM reg WHERE regid='ROUND';
		IF NOT FOUND THEN
			_CompRegValue:='0';		
		END IF;
		_CompRegValue:=COALESCE(_CompRegValue, '0');
		_IntCompRegValue:=CAST(_CompRegValue AS INT2);

		IF (_IntRegValue>_IntCompRegValue) THEN
			RAISE EXCEPTION 'Nilai system setting#ROUND_TAX harus kurang dari atau sama dengan nilai system setting#ROUND ! (spUpdateReg)';
			RETURN OLD;
		END IF;
	END IF;

	RETURN NEW;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

DROP TRIGGER updatereg ON reg;
CREATE TRIGGER updatereg BEFORE UPDATE ON reg FOR EACH ROW EXECUTE PROCEDURE spUpdateReg();




CREATE OR REPLACE FUNCTION fIsContainDeptPrj (
	_Type		VARCHAR,
	_RefNo		VARCHAR, 
	_DeptId		VARCHAR, 
	_PrjId		VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;
	_TransNo	VARCHAR;
BEGIN
	_retval:=FALSE;

	IF (_Type='PAY') THEN
		IF (_DeptId IS NULL AND _PrjId IS NULL) THEN
			_retval:=TRUE;
		ELSIF (_DeptId IS NULL AND _PrjId IS NOT NULL) THEN
			SELECT INTO _TransNo payno FROM ppaid WHERE payno=_RefNo AND deptid IS NULL AND prjid=_PrjId;
			IF FOUND THEN
				_retval:=TRUE;
			END IF;
		ELSIF (_DeptId IS NOT NULL AND _PrjId IS NULL) THEN
			SELECT INTO _TransNo payno FROM ppaid WHERE payno=_RefNo AND deptid=_DeptId AND prjid IS NULL;
			IF FOUND THEN
				_retval:=TRUE;
			END IF;
		ELSE
			SELECT INTO _TransNo payno FROM ppaid WHERE payno=_RefNo AND deptid=_DeptId AND prjid=_PrjId;
			IF FOUND THEN
				_retval:=TRUE;
			END IF; 
		END IF;
	ELSIF (_Type='RCV') THEN
		IF (_DeptId IS NULL AND _PrjId IS NULL) THEN
			_retval:=TRUE;
		ELSIF (_DeptId IS NULL AND _PrjId IS NOT NULL) THEN
			SELECT INTO _TransNo rcvno FROM spaid WHERE rcvno=_RefNo AND deptid IS NULL AND prjid=_PrjId;
			IF FOUND THEN
				_retval:=TRUE;
			END IF;
		ELSIF (_DeptId IS NOT NULL AND _PrjId IS NULL) THEN
			SELECT INTO _TransNo rcvno FROM spaid WHERE rcvno=_RefNo AND deptid=_DeptId AND prjid IS NULL;
			IF FOUND THEN
				_retval:=TRUE;
			END IF;
		ELSE
			SELECT INTO _TransNo rcvno FROM spaid WHERE rcvno=_RefNo AND deptid=_DeptId AND prjid=_PrjId;
			IF FOUND THEN
				_retval:=TRUE;
			END IF; 
		END IF;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fNeracaKey (
	_AccNo		VARCHAR,
	_UpAccNo	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	IF (_UpAccNo='*') THEN
		_retval:=_AccNo||':'||_UpAccNo;
	ELSE
		_retval:=_UpAccNo||':'||_AccNo;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tRptNeraca CASCADE;
CREATE TYPE tRptNeraca AS (
	acctype			VARCHAR, 
	acctypedesc		VARCHAR,
	acctype2		VARCHAR, 
	acctype2desc		VARCHAR, 
	acctype1b		VARCHAR,
	acctype1bdesc		VARCHAR, 
	neracakey		VARCHAR, 
	accno			VARCHAR, 
	upaccno			VARCHAR, 
	accname			VARCHAR, 
	accbal			NUMERIC(19,4), 
	accbalold		NUMERIC(19,4),
	accbalnonharta		NUMERIC(19,4),
	accbalnonhartaold	NUMERIC(19,4), 
	accdk			VARCHAR, 
	isheader		BOOLEAN
);


CREATE OR REPLACE FUNCTION fRptNeraca(
	_StartDate	DATE,
	_EndDate	DATE,
	_StartDateOld	DATE,
	_EndDateOld	DATE,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS SETOF tRptNeraca AS
$BODY$
DECLARE
	_retval 			tRptNeraca%ROWTYPE;

	cAcc 				REFCURSOR;
	cAccType1 			REFCURSOR;

	_StartDatePer 			DATE;
	_PerNo				INT;

	_BulanPer 			INTEGER;
	_TahunPer 			INTEGER;
	_Bulan 				INTEGER;
	_Tahun 				INTEGER;

	_StartDatePrev 			DATE;
	_EndDatePrev 			DATE;

	_StartDatePerNow 		DATE;
	_StartDatePerOld 		DATE;

	_AccType2 			VARCHAR;
	_AccType2Desc 			VARCHAR;
	_AccType1 			VARCHAR;
	_AccType1Desc 			VARCHAR;
	_AccType1B			VARCHAR;
	_AccType1BDesc 			VARCHAR;
	_AccNo 				VARCHAR;

	_UpAccNo			VARCHAR;
	_AccName 			VARCHAR;
	_AccDK 				VARCHAR;
	_IsHeader			BOOLEAN;

	_AccBal 			NUMERIC(19,4);
	_AccBalPrev			NUMERIC(19,4);
	_AccBalOld 			NUMERIC(19,4);
	_AccBalOldPrev			NUMERIC(19,4);
	inc				INTEGER;
	_Condition 			BOOLEAN;
	_JurnalTutupTahunDate		DATE;
	_JurnalTutupTahunDateOld	DATE;
BEGIN
	SELECT INTO _StartDatePer, _PerNo perstart, perno 
	FROM per 
	WHERE active IS TRUE;

	_BulanPer:=DATE_PART('month', _StartDatePer);
	_TahunPer:=DATE_PART('year', _StartDatePer);
	_Bulan:=DATE_PART('month', _StartDate);
	_Tahun:=DATE_PART('year', _StartDate);

	SELECT INTO _StartDatePrev, _EndDatePrev perstart, perend 
	FROM per 
	WHERE perno=_PerNo-1 AND perend IS NOT NULL;

	SELECT INTO _StartDatePerNow perstart FROM per WHERE perid=fGetPer(_StartDate);
	IF(_StartDateOld IS NOT NULL) THEN
		SELECT INTO _StartDatePerOld perstart FROM per WHERE perid=fGetPer(_StartDateOld);
	END IF;

	SELECT INTO _JurnalTutupTahunDate MAX(jnldate) 
	FROM jnl 
	WHERE reftype='CLSY' AND jnldate>_StartDate;

	IF (_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
		SELECT INTO _JurnalTutupTahunDateOld MAX(jnldate) 
		FROM jnl 
		WHERE reftype='CLSY' AND jnldate>_StartDateOld;
	END IF;

	OPEN cAccType1 FOR
		SELECT at2.acctype2, at2.acctype2desc, at2.acctype, at.acctypedesc, at2.acctype1b, at1.acctype1bdesc, a.accno
		FROM acctype2 at2
		LEFT JOIN acctype1 at ON at2.acctype=at.acctype
		LEFT JOIN acctype1b at1 ON at2.acctype1b=at1.acctype1b
		LEFT JOIN acc a ON a.acctype2=at2.acctype2 AND a.upaccno='*'
		WHERE at.nr='N'
		ORDER BY at2.acctype2, at.acctype, at1.acctype1b, a.accno;
	LOOP
		FETCH cAccType1 INTO _AccType2, _AccType2Desc, _AccType1, _AccType1Desc, _AccType1B, _AccType1BDesc, _AccNo;
		EXIT WHEN NOT FOUND;
		
		IF (_AccNo IS NULL OR _AccNo='') THEN
			_retval.acctype:=_AccType1; 
			_retval.acctypedesc:=_AccType1Desc;
			_retval.acctype2:=_AccType2; 
			_retval.acctype2desc:=_AccType2Desc; 
			_retval.acctype1b:=_AccType1B;	
			_retval.acctype1bdesc:=_AccType1BDesc; 
			_retval.neracakey:=NULL; 
			_retval.accno:=NULL; 
			_retval.upaccno:=NULL; 
			_retval.accname:=NULL; 
			_retval.accbal:=0; 
			_retval.accbalold:=0;
			_retval.accbalnonharta:=0;
			_retval.accbalnonhartaold:=0; 
			_retval.accdk:=NULL; 
			_retval.isheader:=FALSE;

			RETURN NEXT _retval;
		ELSE
			OPEN cAcc FOR
				SELECT a.accno, a.upaccno, a.accname, a.accdk, a.isheader
				FROM acc a 
				WHERE (a.accno=_AccNo OR a.upaccno=_AccNo)
				ORDER BY a.accno;
			LOOP
				FETCH cAcc INTO _AccNo, _UpAccNo, _AccName, _AccDK, _IsHeader;
				EXIT WHEN NOT FOUND;
				
				IF(_AccNo<>fAccDefault('LABATHNJLN')) THEN
					_Condition:=FALSE;
					_AccBal:=fAccBal_StartDate(_Accno, _EndDate, _BranchID, _DeptID, _PrjID);
					_AccBalPrev:=fAccBal_StartDate(_Accno, _StartDate-1, _BranchID, _DeptID, _PrjID);
					_AccBalPrev:=COALESCE(_AccBalPrev, 0);

					IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
						_AccBalOld:=fAccBal_StartDate(_Accno, _EndDateOld, _BranchID, _DeptID, _PrjID);
						_AccBalOldPrev:=fAccBal_StartDate(_Accno, _EndDateOld, _BranchID, _DeptID, _PrjID);
						_AccBalOldPrev:=COALESCE(_AccBalOldPrev, 0);
					END IF;

					IF(_AccNo<>fAccDefault('LABATAHAN')) THEN
						_Condition:=true;
					END IF;

					IF(_AccNo=fAccDefault('LABATAHAN') AND _AccBal<>0) THEN
						IF (_JurnalTutupTahunDate IS NOT NULL) THEN
							IF (_StartDate<_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
								_AccBal:=0;
							END IF;
						END IF;

						IF (_JurnalTutupTahunDateOld IS NOT NULL) THEN
							IF (_StartDateOld<_JurnalTutupTahunDateOld AND _EndDateOld>_JurnalTutupTahunDateOld) THEN
								_AccBalOld:=0;
							END IF;
						END IF;

						_Condition:=true;
					END IF;
				
					IF(_StartDate IS NOT NULL AND _EndDate IS NOT NULL AND _AccNo=fAccDefault('LABATAHAN') AND _AccBalOld<>0) THEN
						_Condition:=true;
					END IF;

					IF (_Condition=TRUE) THEN
						_retval.acctype:=_AccType1; 
						_retval.acctypedesc:=_AccType1Desc;
						_retval.acctype2:=_AccType2; 
						_retval.acctype2desc:=_AccType2Desc; 
						_retval.acctype1b:=_AccType1B;	
						_retval.acctype1bdesc:=_AccType1BDesc; 
						_retval.neracakey:=fNeracaKey(_AccNo, _UpAccNo); 
						
						IF (_UpAccNo IS NOT NULL AND LENGTH(_UpAccNo)>0 AND _UpAccNo!='*') THEN
							_AccNo:='   '||_AccNo;
							_AccName:='   '||_AccName;
						END IF;

						_retval.accno:=_AccNo; 
						_retval.upaccno:=_UpAccNo; 
						_retval.accname:=_AccName; 
--						_retval.accbal:=COALESCE(_AccBal-_AccBalPrev, 0);
						_retval.accbal:=COALESCE(_AccBal, 0);
--						_retval.accbalold:=COALESCE(_AccBalOld-_AccBalOldPrev, 0);
						_retval.accbalold:=COALESCE(_AccBalOld, 0);
						_retval.accdk:=_AccDK; 
						_retval.isheader:=_IsHeader;

						IF(_AccType1<>'100') THEN
--							_retval.accbalnonharta:=COALESCE(_AccBal-_AccBalPrev, 0);
							_retval.accbalnonharta:=COALESCE(_AccBal, 0);
--							_retval.accbalnonhartaold:=COALESCE(_AccBalOld-_AccBalOldPrev, 0);
							_retval.accbalnonhartaold:=COALESCE(_AccBalOld, 0);
						ELSE
							_retval.accbalnonharta:=0;
							_retval.accbalnonhartaold:=0;
						END IF;
					ELSE
						CONTINUE;
					END IF;
				ELSE
					SELECT INTO _JurnalTutupTahunDate MAX(jnldate) 
					FROM jnl 
					WHERE reftype='CLSY' AND jnldate>_StartDate;
					IF (_JurnalTutupTahunDate IS NOT NULL AND _StartDate>=_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
						SELECT INTO _AccBal lababersih 
						FROM fProfitLoss(_JurnalTutupTahunDate, _EndDate, _BranchID, _DeptID, _PrjID);
					ELSE
						SELECT INTO _AccBal lababersih 
						FROM fProfitLoss(_StartDate, _EndDate, _BranchID, _DeptID, _PrjID);
					END IF;

					IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
						SELECT INTO _JurnalTutupTahunDateOld MAX(jnldate) 
						FROM jnl 
						WHERE reftype='CLSY' AND jnldate>_StartDateOld;
						IF (_JurnalTutupTahunDateOld IS NOT NULL AND _StartDateOld>=_JurnalTutupTahunDateOld AND _EndDateOld>_JurnalTutupTahunDateOld) THEN
							SELECT INTO _AccBalOld lababersih 
							FROM fProfitLoss(_JurnalTutupTahunDateOld, _EndDateOld, _BranchID, _DeptID, _PrjID);
						ELSE
							SELECT INTO _AccBalOld lababersih 
							FROM fProfitLoss(_StartDatePerOld, _EndDateOld, _BranchID, _DeptID, _PrjID);
						END IF;
					END IF;

					IF(_AccBal<>0) THEN
						_retval.acctype:=_AccType1; 
						_retval.acctypedesc:=_AccType1Desc;
						_retval.acctype2:=_AccType2; 
						_retval.acctype2desc:=_AccType2Desc; 
						_retval.acctype1b:=_AccType1B;	
						_retval.acctype1bdesc:=_AccType1BDesc; 
						_retval.neracakey:=fNeracaKey(_AccNo, _UpAccNo); 
						_retval.accno:=_AccNo; 
						_retval.upaccno:=_UpAccNo; 
						_retval.accname:=_AccName; 
						_retval.accbal:=COALESCE(_AccBal, 0); 
						_retval.accbalold:=COALESCE(_AccBalOld, 0);
						_retval.accbalnonharta:=COALESCE(_AccBal, 0);
						_retval.accbalnonhartaold:=COALESCE(_AccBalOld, 0);
						_retval.accdk:=_AccDK; 
						_retval.isheader:=_IsHeader;
					ELSE
						_retval.acctype:=_AccType1; 
						_retval.acctypedesc:=_AccType1Desc;
						_retval.acctype2:=_AccType2; 
						_retval.acctype2desc:=_AccType2Desc; 
						_retval.acctype1b:=_AccType1B;	
						_retval.acctype1bdesc:=_AccType1BDesc; 
						_retval.neracakey:=NULL; 
						_retval.accno:=NULL; 
						_retval.upaccno:=NULL; 
						_retval.accname:=NULL; 
						_retval.accbal:=0; 
						_retval.accbalold:=0;
						_retval.accbalnonharta:=0;
						_retval.accbalnonhartaold:=0; 
						_retval.accdk:=NULL; 
						_retval.isheader:=FALSE;
					END IF;
				END IF;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cAcc;
		END IF;
	END LOOP;	
	CLOSE cAccType1;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spInsertAccount() 
RETURNS TRIGGER AS
$BODY$
DECLARE
	_AccType1B	VARCHAR;
BEGIN
	IF (NEW.upaccno IS NULL) THEN
		NEW.upaccno:='*';
	END IF;

	IF (NEW.upaccno='*') THEN
		NEW.acclevel=1;
	ELSE
		NEW.acclevel:=2;	
	END IF;

	SELECT INTO _AccType1B acctype1b FROM acctype2 WHERE acctype2=NEW.acctype2;
	IF FOUND THEN
		NEW.acctype1b:=_AccType1B;
	END IF;

	RETURN NEW;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

DROP TRIGGER insertaccount ON acc;
CREATE TRIGGER insertaccount BEFORE INSERT ON acc FOR EACH ROW EXECUTE PROCEDURE spInsertAccount();




CREATE OR REPLACE FUNCTION spUpdateAccount()
RETURNS TRIGGER AS
$BODY$
DECLARE
	_AccNo		VARCHAR;
	_JnlNo		VARCHAR;
BEGIN
	IF (New.upaccno!=OLD.upaccno) THEN
		SELECT INTO _JnlNo jnlno FROM lgr WHERE accno=OLD.accno LIMIT 1;
		IF FOUND THEN
			RAISE EXCEPTION '%', 'Sudah ada transaksi yang menggunakan akun ini !'||E'\n'||'Tidak diperkenankan mengganti akun header ! (spUpdateAccount)';
			RETURN OLD;		
		ELSE
			RETURN NEW;		
		END IF;
	ELSE
		IF (NEW.isheader!=OLD.isheader AND NEW.isheader=FALSE) THEN
			SELECT INTO _AccNo accno FROM acc WHERE upaccno=OLD.accno LIMIT 1;
			IF FOUND THEN
				RAISE EXCEPTION 'Akun %', OLD.accno||' sudah digunakan sebagai header oleh akun lain ! (spUpdateAccount)';			
				RETURN OLD;
			ELSE
				RETURN NEW;
			END IF;
		ELSIF (NEW.isheader!=OLD.isheader AND NEW.isheader=TRUE) THEN
			SELECT INTO _JnlNo jnlno FROM lgr WHERE accno=OLD.accno LIMIT 1;
			IF FOUND THEN
				RAISE EXCEPTION '%', 'Sudah ada transaksi yang menggunakan akun ini !'||E'\n'||'Tidak diperkenankan menjadikan akun ini sebagai akun header ! (spCheckAccount)';
				RETURN OLD;
			ELSE 
				RETURN NEW;		
			END IF;
		ELSE
			RETURN NEW;
		END IF;
	END IF;	 
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

DROP TRIGGER updateaccount ON acc;
CREATE TRIGGER updateaccount BEFORE UPDATE ON acc FOR EACH ROW EXECUTE PROCEDURE spUpdateAccount();




CREATE OR REPLACE FUNCTION fIsUsePPn (
	_SaleNo	VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;

	_ItemID		VARCHAR;
BEGIN
	_retval:=FALSE;

	SELECT INTO _ItemID itemid FROM saled WHERE saleno=_SaleNo AND taxid='PPN' LIMIT 1;
	IF FOUND THEN
		_retval:=TRUE;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAccTrans (
	_TransType	VARCHAR,
	_TransNo	VARCHAR,
	_AccType	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE 
	_retval		VARCHAR;
BEGIN
	IF (_TransType='PURC') THEN
		IF (_AccType='AP') THEN
			SELECT INTO _retval accap FROM purc WHERE purcno=_TransNo;
		END IF;
	ELSIF (_TransType='SALE') THEN
		IF (_AccType='AR') THEN
			SELECT INTO _retval accar FROM sale WHERE saleno=_TransNo;
		END IF;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fValidatePID (
	_PID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	IF (_PID IS NULL) THEN
		_retval:='';
	ELSE
		_retval:=_PID;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fProfitSale (
	_SaleNo		VARCHAR
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_ItemProfit	NUMERIC(19,4);
	_DiscAmt	NUMERIC(19,4);
	_retval		NUMERIC(19,4);
BEGIN
	SELECT INTO 
		_ItemProfit 
		SUM(sd.qty*(sd.baseprice-sd.discamt-(sd.cost/s.excrate))) 
	FROM saled sd
	JOIN sale s ON s.saleno=sd.saleno
	WHERE sd.saleno=_SaleNo;

	SELECT INTO 
		_DiscAmt
		discamt
	FROM sale 
	WHERE saleno=_SaleNo;

	_retval:=_ItemProfit-_DiscAmt;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fProfitDeli (
	_DeliNo VARCHAR
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	SELECT INTO _retval sum(qty*(baseprice-discamt-cost)) 
	FROM DeliD 
	WHERE DeliNo=_DeliNo;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fMainAddr(_BPID VARCHAR)
RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval Addr FROM Addr WHERE bpid=_BPID ORDER BY addrno LIMIT 1;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fMainAddrCity (
	_BPID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval c.cityname FROM addr a JOIN city c ON c.cityid=a.cityid WHERE a.bpid=_BPID ORDER BY a.addrno LIMIT 1;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fMainContName (
	_BPID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval contname FROM cont WHERE bpid=_BpID ORDER BY contid LIMIT 1;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fMainContPhone (
	_BPID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	
BEGIN
	SELECT INTO _retval phone FROM cont WHERE bpid=_BPID ORDER BY contid LIMIT 1;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



/**

	Reads BASE CURRENCY
	
**/

CREATE OR REPLACE FUNCTION fCrcDefault()
RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN

	SELECT INTO _retval	crcid
	FROM cmp;


/**
	-- tidak baca dari Crc:isDefault karena rawan di edit

	SELECT INTO 
		_retval
		crcid
	FROM crc WHERE isDefault=TRUE;
**/

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fusrprv (
	_usrid	VARCHAR
) RETURNS SETOF tusrprv AS
$BODY$
DECLARE
	_retval	tUsrPrv%ROWTYPE;
	_found	VARCHAR;
	cObj	CURSOR IS
		SELECT grpprv.objid, acstype, (fIntToBool(SUM(fBoolToInt(acsenable))) AND obj.enabled) AS enabled,acsval
		FROM grpprv 
		JOIN obj ON obj.objid=grpprv.objid
		WHERE grpid IN (SELECT grpid FROM usrgrp WHERE usrid=_usrid)
		GROUP BY grpprv.objid, acstype,obj.enabled,acsval
		ORDER BY grpprv.objid, acstype,acsval;
BEGIN
	OPEN cObj;
	LOOP 
		FETCH cObj INTO _retval;
		EXIT WHEN NOT FOUND;

		return NEXT _retval;

	END LOOP;
	CLOSE cObj;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fReg (
	_RegID	VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_tmp	VARCHAR(30);
BEGIN
	_tmp:=NULL;

	SELECT INTO _tmp RegVal FROM Reg WHERE RegID=_RegID;

	RETURN _tmp;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fParseDate(
	_Date		DATE,
	_Format		VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;

	_Day		INT2;
	_Month		INT2;
	_Year		INT2;

	_TmpFormat	VARCHAR;
	_SeqChar	VARCHAR;
	_LastChar	VARCHAR(1);
	_CurrChar	VARCHAR(1);

	_SDay		VARCHAR;
	_SMonth		VARCHAR;
	_SYear		VARCHAR;

	_CDay		VARCHAR(1);
	_CMonth		VARCHAR(1);
	_CYear		VARCHAR(1);

	_Counter	INT2;
	_FLength	INT2;
	_SeqLength	INT2;

	_ProceedLast	BOOLEAN;
	_LastSeparator	VARCHAR;
BEGIN
	_retval:='';
	_SeqChar:='';
	_LastChar:='';
	_LastSeparator:='';
	_CDay:='d';
	_CMonth:='m';
	_CYear:='y';
	_TmpFormat:=LOWER(_Format);
	_FLength:=LENGTH(_Format);

	_Day:=DATE_PART('day', _Date);
	_SDay:=CAST(_Day AS VARCHAR);
	
	_Month:=DATE_PART('month', _Date);
	_SMonth:=CAST(_Month AS VARCHAR);

	_Year:=DATE_PART('year', _Date);
	_SYear:=CAST(_Year AS VARCHAR);

	_Counter:=1;
	WHILE (_Counter<=_FLength) LOOP
		_CurrChar:=SUBSTRING(_TmpFormat, _Counter, 1);

		IF (_CurrChar!=_CDay AND _CurrChar!=_CMonth AND _CurrChar!=_CYear) THEN
			IF (LENGTH(_SeqChar)=0) THEN
				_retval:=_retval||_CurrChar;			
			ELSE
				_LastSeparator:=_LastSeparator||_CurrChar;
			END IF;

			IF (_Counter=_FLength AND LENGTH(_SeqChar)>0) THEN
				IF (_LastChar=_CDay) THEN
					IF (_SeqLength=1 OR _SeqLength>2) THEN
						_retval:=_retval||_SDay;
					ELSE
						IF (_Day<10) THEN
							_retval:=_retval||'0'||_SDay;
						ELSE
							_retval:=_retval||_SDay;
						END IF;
					END IF;
				ELSIF (_LastChar=_CMonth) THEN
					IF (_SeqLength=1 OR _SeqLength>2) THEN
						_retval:=_retval||_SMonth;
					ELSE
						IF (_Month<10) THEN
							_retval:=_retval||'0'||_SMonth;
						ELSE
							_retval:=_retval||_SMonth;
						END IF;
					END IF;
				ELSIF (_LastChar=_CYear) THEN
					IF ((_SeqLength>=1 AND _SeqLength<=3) OR _SeqLength>4) THEN
						_retval:=_retval||SUBSTRING(_SYear, 3, 2);
					ELSE
						_retval:=_retval||_SYear;
					END IF;
				END IF;
				
				_retval:=_retval||_LastSeparator;
				_LastSeparator:='';
			END IF;
		ELSE
			IF (_Counter=_FLength) THEN
				IF (_LastChar=_CurrChar) THEN
					_LastChar:=_CurrChar;
					_SeqChar:=_SeqChar||_CurrChar;
					_ProceedLast:=FALSE;
				ELSE
					_ProceedLast:=TRUE;
				END IF;

				IF (_LastChar=_CDay) THEN
					IF (_SeqLength=1 OR _SeqLength>2) THEN
						_retval:=_retval||_SDay;
					ELSE
						IF (_Day<10) THEN
							_retval:=_retval||'0'||_SDay;
						ELSE
							_retval:=_retval||_SDay;
						END IF;
					END IF;
				ELSIF (_LastChar=_CMonth) THEN
					IF (_SeqLength=1 OR _SeqLength>2) THEN
						_retval:=_retval||_SMonth;
					ELSE
						IF (_Month<10) THEN
							_retval:=_retval||'0'||_SMonth;
						ELSE
							_retval:=_retval||_SMonth;
						END IF;
					END IF;
				ELSIF (_LastChar=_CYear) THEN
					IF ((_SeqLength>=1 AND _SeqLength<=3) OR _SeqLength>4) THEN
						_retval:=_retval||SUBSTRING(_SYear, 3, 2);
					ELSE
						_retval:=_retval||_SYear;
					END IF;
				END IF;
				
				_retval:=_retval||_LastSeparator;
				_LastSeparator:='';

				_SeqChar:='';
				_LastChar:=_CurrChar;
				_SeqChar:=_SeqChar||_CurrChar;

				IF (_ProceedLast=TRUE) THEN
					IF (_LastChar=_CDay) THEN
						IF (_SeqLength=1 OR _SeqLength>2) THEN
							_retval:=_retval||_SDay;
						ELSE
							IF (_Day<10) THEN
								_retval:=_retval||'0'||_SDay;
							ELSE
								_retval:=_retval||_SDay;
							END IF;
						END IF;
					ELSIF (_LastChar=_CMonth) THEN
						IF (_SeqLength=1 OR _SeqLength>2) THEN
							_retval:=_retval||_SMonth;
						ELSE
							IF (_Month<10) THEN
								_retval:=_retval||'0'||_SMonth;
							ELSE
								_retval:=_retval||_SMonth;
							END IF;
						END IF;
					ELSIF (_LastChar=_CYear) THEN
						IF ((_SeqLength>=1 AND _SeqLength<=3) OR _SeqLength>4) THEN
							_retval:=_retval||SUBSTRING(_SYear, 3, 2);
						ELSE
							_retval:=_retval||_SYear;
						END IF;
					END IF;

					_retval:=_retval||_LastSeparator;
					_LastSeparator:='';
				END IF;
			ELSE
				IF (_LastChar!=_CurrChar) THEN
					_SeqLength:=LENGTH(_SeqChar);

					IF (_SeqLength!=0) THEN
						IF (_LastChar=_CDay) THEN
							IF (_SeqLength=1 OR _SeqLength>2) THEN
								_retval:=_retval||_SDay;
							ELSE
								IF (_Day<10) THEN
									_retval:=_retval||'0'||_SDay;
								ELSE
									_retval:=_retval||_SDay;
								END IF;
							END IF;
						ELSIF (_LastChar=_CMonth) THEN
							IF (_SeqLength=1 OR _SeqLength>2) THEN
								_retval:=_retval||_SMonth;
							ELSE
								IF (_Month<10) THEN
									_retval:=_retval||'0'||_SMonth;
								ELSE
									_retval:=_retval||_SMonth;
								END IF;
							END IF;
						ELSIF (_LastChar=_CYear) THEN
							IF ((_SeqLength>=1 AND _SeqLength<=3) OR _SeqLength>4) THEN
								_retval:=_retval||SUBSTRING(_SYear, 3, 2);
							ELSE
								_retval:=_retval||_SYear;
							END IF;
						END IF;

						_retval:=_retval||_LastSeparator;
						_LastSeparator:='';

						_SeqChar:='';
					END IF;

					_LastChar:=_CurrChar;
					_SeqChar:=_SeqChar||_CurrChar;
				ELSE
					_LastChar:=_CurrChar;
					_SeqChar:=_SeqChar||_CurrChar;
				END IF;
			END IF;
		END IF;
		_Counter:=_Counter+1;
	END LOOP;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*
	fFormat
*/


CREATE OR REPLACE FUNCTION fFormat (
	_Input	VARCHAR,
	_Date	DATE
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	_Now		TIMESTAMP;
	_Year		VARCHAR;
	_Month		VARCHAR;
	
/*
	_date		VARCHAR;
	_time		VARCHAR;
*/
	
	_Tmp		VARCHAR;
BEGIN
	_Tmp:=_Input;
	IF (_Date IS NOT NULL) THEN
		_Now:=_Date;
	ELSE
		_Now:=now();
	END IF;

	_Year:=SUBSTRING(CAST(DATE_PART('year', _Now) AS VARCHAR), 3, 2);
	_Month:=CAST(DATE_PART('month', _Now) AS VARCHAR);
	IF (LENGTH(_Month)<2) THEN _Month:='0'||_Month; END IF;

	IF (_Tmp='#YY#') THEN
		_Tmp:=REPLACE(_Tmp, '#YY#', _Year);
	ELSIF (_Tmp='#MM#') THEN
		_Tmp:=REPLACE(_Tmp, '#MM#', _Month);
	ELSIF (_Tmp='#MMYY#') THEN
		_Tmp:=REPLACE(_Tmp, '#MMYY#', _Month||_Year);	
	ELSIF (_Tmp='#YYMM#') THEN
		_Tmp:=REPLACE(_Tmp, '#YYMM#', _Year||_Month);
	ELSE
		_Tmp:=fParseDate(CAST(_Now AS DATE), _Input);
	END IF;
	
	_retval:=_Tmp;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fFormat (
	_Input	VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval fFormat(_Input, NULL);

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/*
	spTransNoGet
*/

CREATE OR REPLACE FUNCTION fTransNoGet (
	_NoID		VARCHAR,
	_Date		DATE,
	_BranchId	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval			VARCHAR;
		
	_TransCode		VARCHAR;
	_SiteCode		VARCHAR;
	_PeriodeCode		VARCHAR;
	_CurNo			INT4;
	_StartNo		INTEGER;
	_NoLen			INT2;
	_PeriodeCode_Parsed	VARCHAR;
	_IndexOfPlus		INT;
	_SiteCode_Parse		CHAR(1);
	_NCountPId		VARCHAR;
BEGIN
	LOCK TABLE ncount IN EXCLUSIVE MODE;
	/**  */

	SELECT INTO _CurNo, _NoLen, _TransCode, _SiteCode, _PeriodeCode, _StartNo
		curno, nolen, transcode, sitecode, periodecode, startno
	FROM ncount 
	WHERE noid=_NoID;
	IF NOT FOUND THEN
		RETURN NULL;
	END IF;

	IF (_SiteCode IS NOT NULL) THEN
		_IndexOfPlus:=POSITION('+' IN _SiteCode);
		IF (_IndexOfPlus > 0) THEN
			IF (_IndexOfPlus = 1) THEN
				_SiteCode_Parse:=SUBSTR(_SiteCode, 2, 1);
			ELSE
				_SiteCode_Parse:=SUBSTR(_SiteCode, 1, 1);
			END IF;
		END IF;

		IF (_SiteCode_Parse = 'B') THEN
			_SiteCode:=_BranchId;
		ELSIF (_IndexOfPlus>0) THEN
			_SiteCode:='';
		END IF;
	END IF;

	IF (_PeriodeCode IS NOT NULL) THEN
		_PeriodeCode_Parsed:=fFormat(_PeriodeCode, _Date);
		_NCountPId:=_PeriodeCode_Parsed;

		IF (_IndexOfPlus > 0 AND _SiteCode_Parse='B' AND _BranchId IS NOT NULL AND _BranchId!='') THEN
			IF (_IndexOfPlus = 1) THEN
				_NCountPId:=_BranchId || _PeriodeCode_Parsed;
			ELSE
				_NCountPId:=_PeriodeCode_Parsed || _BranchId;
			END IF;
		END IF;

		SELECT INTO _CurNo curno FROM ncountp WHERE noid=_NoID AND periodecode=_NCountPId;
		IF FOUND THEN
			IF (_CurNo IS NULL) THEN 
				_CurNo:=0; 
			END IF;

			_CurNo:=_CurNo+1;
			UPDATE ncountp SET curno=_CurNo WHERE noid=_NoID AND periodecode=_NCountPId;
		ELSE
			IF (_StartNo IS NULL) THEN 
				_StartNo:=1; 
			END IF;

			_CurNo:=_StartNo;
			INSERT INTO ncountp(noid, periodecode, curno) VALUES (_NoID, _NCountPId, _CurNo);
		END IF;
	ELSIF (_IndexOfPlus > 0 AND _SiteCode_Parse='B' AND _BranchId IS NOT NULL AND _BranchId!='') THEN
		SELECT INTO _CurNo curno FROM ncountp WHERE noid=_NoID AND periodecode=_BranchId;
		IF FOUND THEN
			IF (_CurNo IS NULL) THEN 
				_CurNo:=0; 
			END IF;

			_CurNo:=_CurNo+1;
			UPDATE ncountp SET curno=_CurNo WHERE noid=_NoID AND periodecode=_BranchId;
		ELSE
			IF (_StartNo IS NULL) THEN 
				_StartNo:=1; 
			END IF;

			_CurNo:=_StartNo;
			INSERT INTO ncountp(noid, periodecode, curno) VALUES (_NoID, _BranchId, _CurNo);
		END IF;
	ELSE
		IF (_CurNo IS NULL) THEN 
			_CurNo:=0; 
		END IF;

		_CurNo:=_CurNo+1;
		UPDATE ncount SET curno=_CurNo WHERE noid=_NoID;
	END IF;

	_retval:=TRIM(CAST(_CurNo AS VARCHAR));
	WHILE (char_length(_retval)<_NoLen) LOOP
		_retval:='0' || _retval;
	END LOOP;

	IF (_IndexOfPlus = 1) THEN
		_retval:=COALESCE(_SiteCode,'') || COALESCE(_TransCode, '') || COALESCE(_PeriodeCode_Parsed, '') || _retval;
	ELSE
		_retval:=COALESCE(_TransCode,'') || COALESCE(_SiteCode, '') || COALESCE(_PeriodeCode_Parsed, '') || _retval;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fTransNoGet(
	_NoID		VARCHAR,
	_BranchId	VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval fTransNoGet(_NoID, NULL, _BranchId);

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fTransNoGet(
	_NoID 	VARCHAR,
	_Date 	DATE
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval fTransNoGet(_NoID, _Date, NULL);

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fTransNoGet(_NoID VARCHAR) 
RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval fTransNoGet(_NoID, NULL, NULL);

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: fbooltoint(value bool)*/

/*-- DROP FUNCTION fbooltoint(value bool);*/

CREATE OR REPLACE FUNCTION fbooltoint (
	value	BOOLEAN
) RETURNS INT2 AS
$BODY$
DECLARE
	_retval INT2;
BEGIN
	IF value=true THEN
		_retval := 1;
	END IF;
	IF value=false THEN
		_retval := 0;
	END IF;
	RETURN(_retval);
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: finttobool(value int8)*/

/*-- DROP FUNCTION finttobool(value int8);*/

CREATE OR REPLACE FUNCTION finttobool (
	value	INT8
) RETURNS BOOLEAN AS
$BODY$
BEGIN
	IF value>=1 THEN
		RETURN(true);
	END IF;

	IF value=0 THEN
		RETURN(false);
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItemUsePID (
	__ItemID	VARCHAR
) RETURNS BOOLEAN AS 
$BODY$
DECLARE
	_retval	BOOLEAN;
BEGIN
	SELECT INTO _retval usePID FROM Item WHERE ItemID=__ItemID;
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/**
	__TaxID	= 'PPN'
	__Amt	= ListPrice yang mau dihitung
			  agar lebih presisi
	
			misal Harga 1000
			Qty = 100
			_Amt=100,000

	__isTaxInc = BOOLEAN
			apakah included Tax

	__Mode
		0 = TaxAmt
		1 = BasePrice (khusus isTaxInc==TRUE)
			atau nilai Amt asli (untuk isTax FALSE)


Testing data:
=============

DELETE FROM Tax WHERE TaxID LIKE 'A%';
INSERT INTO Tax VALUES ('A1', '122001', '220001', 'Test Tax', '1.23%+2.342345%');
INSERT INTO Tax VALUES ('A2', '122001', '220001', 'Test Tax', '5%+6%+7%');
INSERT INTO Tax VALUES ('E1', '122001', '220001', 'Test Error Parse Tax', '5%+6+7%');

select fTaxCalc('A1', 1000, FALSE, 0);
select fTaxCalc('E1', 1000, FALSE, 0);
select fTaxCalc('PPN', 1.5*1000*9300, TRUE, 0);


Samples :
=========
Hitung Pajak
--------------
SELECT fTaxCalc('PPN', 1000*(1000-fDiscCalc('5%',1000))*9300, TRUE, TRUE, 0);
SELECT fTaxCalc('PPN', 1000*(1000-fDiscCalc('5%',1000))*9300, TRUE, FALSE, 0);
SELECT fTaxCalc('PPN', 1000*(1000-fDiscCalc('5%',1000))*9300, FALSE, FALSE, 0);


Hitung SubTotal
--------------
SELECT fTaxCalc('PPN', 1000*(1000-fDiscCalc('5%',1000))*9300, TRUE, TRUE, 1);
SELECT fTaxCalc('PPN', 1000*(1000-fDiscCalc('5%',1000))*9300, TRUE, FALSE, 1);
SELECT fTaxCalc('PPN', 1000*(1000-fDiscCalc('5%',1000))*9300, FALSE, FALSE, 1);

**/

 DROP FUNCTION fTaxCalc(VARCHAR, DECIMAL, BOOLEAN, INT);

CREATE OR REPLACE FUNCTION fTaxCalc(
	__TaxID		VARCHAR,
	__Amt		DECIMAL(19,4),
	__isTaxed	BOOLEAN,
	__isTaxInc	BOOLEAN,
	__Mode		INT
) RETURNS DECIMAL(19,4) AS 
$BODY$
DECLARE
	_retval		DECIMAL(19,4);
	_cur		refcursor;

	_TaxExp		VARCHAR;	-- remaining Expression
	_CurExp		VARCHAR;	-- current/chunked expression
	_TmpExp		VARCHAR;

	_pos_plus	INT;
	_pos_end	INT;
	_pos_percent	INT;
	_i		INT;
	_pos2		INT;
	_ExpDec		DECIMAL(19,4);

	_SubTotal	DECIMAL(19,4);
	_CurTaxAmt	DECIMAL(19,4);
	_AccTaxAmt	DECIMAL(19,4);
	_BasePrice	DECIMAL(19,4);
	_ErrMsg		VARCHAR;
BEGIN
	_SubTotal:=__Amt;

	IF (__isTaxed IS NULL OR __isTaxed=FALSE OR __TaxID IS NULL) THEN
		IF (__Mode=0) THEN
			_retval:=0;
			RETURN _retval;
		ELSIF (__Mode=1) THEN
			_retval:=_SubTotal;
			RETURN _retval;
		END IF;
	END IF;

	SELECT INTO _TaxExp TaxExp FROM Tax WHERE TaxID=__TaxID;

	IF NOT FOUND THEN
		_retval:=(50001, 'TaxID '||__TaxID||' tidak ditemukan (fTaxCalc)');
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_AccTaxAmt:=0;
	_CurTaxAmt:=0;
	_i:=0;
	LOOP
		EXIT WHEN LENGTH(_TaxExp)=0 OR (_i>5);

		_i:=_i+1;

		_pos_plus:=POSITION('+' IN _TaxExp);


		IF (_pos_plus>0) THEN
			_CurExp:=SUBSTRING(_TaxExp, 1, _pos_plus-1);
			_TaxExp:=SUBSTRING(_TaxExp, _pos_plus+1, LENGTH(_TaxExp));
		ELSE
			_CurExp:=SUBSTRING(_TaxExp, 1, LENGTH(_TaxExp));
			_TaxExp:='';
		END IF;

		-- Parse Current Expression into Decimal
		_pos_percent:=POSITION('%' IN _CurExp);
		IF (_pos_percent>0) THEN
			_TmpExp:=SUBSTRING(_CurExp, 1, _pos_percent-1);
			_ExpDec:=CAST(_TmpExp AS DECIMAL(19,4));
--			RAISE NOTICE '		Decimal %', _ExpDec;
		ELSE
			_ErrMsg:='Error Parsing Expression ['||_CurExp||']';
			RAISE EXCEPTION '%', _ErrMsg;
		END IF;

		-- Calc Current Tax
		IF (__isTaxInc) THEN
			_CurTaxAmt:=(_ExpDec*_SubTotal)/(100+_ExpDec);
		ELSE
			_CurTaxAmt:=(_ExpDec*_SubTotal)/100;
		END IF;

		IF (_i=1 AND __Mode=1) THEN
			IF (__isTaxInc=TRUE) THEN
				_BasePrice:=(100*_SubTotal)/(_ExpDec+100);	-- gak berlaku recursive karena nested
			ELSE
				_BasePrice:=_SubTotal;	-- gak berlaku recursive karena nested
			END IF;
		END IF;

--		RAISE NOTICE '		SubTotal  %', _SubTotal;
--		RAISE NOTICE '		CurTaxAmt %', _CurTaxAmt;

		-- Accumulate to TaxAmtAcc
		_SubTotal:=_SubTotal+_CurTaxAmt;
		_AccTaxAmt:=_AccTaxAmt+_CurTaxAmt;
	END LOOP;

	IF (__Mode=1) THEN
		_retval:=_BasePrice;
	ELSE
		_retval:=_AccTaxAmt;
	END IF;

	RETURN _retval;
END
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



/***

SELECT fDiscCalc('2%+500', 1000);
SELECT fDiscCalc('2%+3%', 1000);
SELECT fDiscCalc('500+300', 1000);
SELECT fDiscCalc('100+2%+3%', 1000);

***/


-- DROP FUNCTION fDiscCalc(varchar, decimal);

CREATE OR REPLACE FUNCTION fDiscCalc (
	__DiscExp	VARCHAR,
	__Amt		DECIMAL(19,4)
) RETURNS DECIMAL(19,4) AS 
$BODY$
DECLARE
	_i		INT;
	_DiscExp	VARCHAR;
	_CurExp		VARCHAR;
	_TmpExp		VARCHAR;
	_ExpDec		DECIMAL(19,4);
	_DiscAmt	DECIMAL(19,4);
	_DiscAmtAccum	DECIMAL(19,4);	-- akumulasi (total) discamt
	_Amt		DECIMAL(19,4);

	_pos_plus	INT;
	_pos_percent	INT;
	_isDebug	BOOLEAN;
BEGIN
	_isDebug:=FALSE;

	_DiscExp:=__DiscExp;
	_DiscAmtAccum:=0;
	_Amt:=__Amt;
	_i:=0;

	IF (_DiscExp IS NULL OR LENGTH(_DiscExp)=0 OR _Amt<=0) THEN		
		RETURN _DiscAmtAccum;
	END IF;

	LOOP
		EXIT WHEN LENGTH(_DiscExp)=0 OR (_i>5);

		_i:=_i+1;

IF (_isDebug) THEN RAISE NOTICE 'Loop ke-%', _i; END IF;

		_pos_plus:=POSITION('+' IN _DiscExp);

		IF (_pos_plus>0) THEN
IF (_isDebug) THEN RAISE NOTICE '		Examining-A: %', _DiscExp; END IF;
			_CurExp:=SUBSTRING(_DiscExp, 1, _pos_plus-1);
			_DiscExp:=SUBSTRING(_DiscExp, _pos_plus+1, LENGTH(_DiscExp));
IF (_isDebug) THEN RAISE NOTICE '		DiscExp now: %', _DiscExp; END IF;
		ELSE
IF (_isDebug) THEN RAISE NOTICE '		Examining-B: %', _DiscExp; END IF;
			_CurExp:=SUBSTRING(_DiscExp, 1, LENGTH(_DiscExp));
			_DiscExp:='';
		END IF;
IF (_isDebug) THEN RAISE NOTICE '		CurExp: %', _CurExp; END IF;



		_pos_percent:=POSITION('%' IN _CurExp);
		IF (_pos_percent>0) THEN
			_TmpExp:=SUBSTRING(_CurExp, 1, _pos_percent-1);
			_ExpDec:=CAST(_TmpExp AS DECIMAL(19,4));
			_DiscAmt:=(_Amt*_ExpDec/100);
IF (_isDebug) THEN RAISE NOTICE '		Percent % ', _ExpDec; END IF;
		ELSE	-- Number Literal
			_TmpExp:=SUBSTRING(_CurExp, 1, LENGTH(_CurExp));
			_DiscAmt:=CAST(_TmpExp AS DECIMAL(19,4));
IF (_isDebug) THEN RAISE NOTICE '		Literal %', _DiscAmt; END IF;
		END IF;
IF (_isDebug) THEN RAISE NOTICE '		DiscAmt %', _DiscAmt; END IF;

		_Amt:=_Amt-_DiscAmt;
		_DiscAmtAccum:=_DiscAmtAccum+_DiscAmt;

IF (_isDebug) THEN RAISE NOTICE 'DiscAmt= % , RunningTotal: %', _DiscAmt, _Amt; END IF;
		

	END LOOP;

	RETURN _DiscAmtAccum;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*-- DROP FUNCTION iif(bool, varchar, varchar);*/

CREATE OR REPLACE FUNCTION iif(tes bool, val1 varchar, val2 varchar)
  RETURNS VARCHAR AS
$BODY$
begin
	IF (tes) THEN
		RETURN(val1);
	ELSE
		RETURN(val2);
	END IF;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
/*
	spTransNoGet
*/


CREATE OR REPLACE FUNCTION spTransNoGet (
	_NoID		VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_TransCode	VARCHAR;
	_curno		INT4;
	_nolen		INT2;
	_retval		VARCHAR;
BEGIN
	SELECT INTO _CurNo, _NoLen, _TransCode CurNo, NoLen, TransCode FROM NCount WHERE NoID=_NoID;

	UPDATE NCount SET CurNo=CurNo+1 WHERE NoID=_NoID;

	_retval:=TRIM(CAST(_curno+1 AS VARCHAR));
	WHILE (char_length(_retval)<_nolen) LOOP
		_retval:='0' || _retval;
	END LOOP;

	_retval:=_TransCode || _retval;
	RETURN(_retval);
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spLogin (
	_UserID		VARCHAR,
	_Password	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_found		VARCHAR;
BEGIN
	_retval.ErrNo:=1;
	_retval.ErrMsg:='Login Failed !';

	SELECT INTO _found usrid FROM usr WHERE (usrid=_UserID AND (passwd IS NULL OR passwd='' OR passwd=_Password) AND disabled='f');
	IF FOUND THEN
		UPDATE usr SET 
			lastloginfrom=host(inet_client_addr()),lastlogintime=now()
		WHERE UsrID=_UserID;
		_retval:=(0, 'OK');
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fZeroFill (
	_No	DECIMAL,
	_Len	INT
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_transcode	VARCHAR;
	_sitecode	VARCHAR;
	_curno		INT4;
	_nolen		INT2;
	_retval		VARCHAR;
BEGIN
	_retval:=TRIM(CAST(_No AS VARCHAR));
	WHILE (char_length(_retval)<_Len) LOOP
		_retval:='0' || _retval;
	END LOOP;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBPName (
	_BPID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	_mycrc		VARCHAR;
BEGIN
	SELECT INTO _retval	BPName
	FROM bp WHERE bpid=_BPID;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fEmpName (
	_EMPID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	_mycrc		VARCHAR;
BEGIN
	SELECT INTO _retval	EMPNAME
	FROM emp WHERE empid=_EMPID;
	IF _retval IS NULL THEN
		_retval:='';
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPrcLvlID (
	_BPID	VARCHAR
) RETURNS VARCHAR(3) AS 
$BODY$
DECLARE
	_PrcLvlID	VARCHAR(3);
BEGIN
	IF (_BPID IS NOT NULL) THEN
		SELECT INTO _PrcLvlID PrcLvlID FROM bp
		WHERE BPID=_BPID;
	END IF;

	IF (_BPID IS NULL OR _PrcLvlID IS NULL) THEN
		SELECT INTO _PrcLvlID MIN(PrcLvlID) FROM PrcLvl; 
	END IF;

	RETURN _PrcLvlID;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPriceExt (
	_ItemID		VARCHAR,
	_Qty		DECIMAL(15,4),
	_Unit		VARCHAR(5),
	_BPID		VARCHAR
) RETURNS SETOF TPriceExt AS
$BODY$
DECLARE
	_retval		TPriceExt%ROWTYPE;

	_Price1		TPriceExt;
	_Price2		TPriceExt;
	_Price3		TPriceExt;

	_LastSaleNo	VARCHAR;

	_Unit1		VARCHAR;
	_Unit2		VARCHAR;
	_Conv2		DECIMAL(15,4);
	_Unit3		VARCHAR;
	_Conv3		DECIMAL(15,4);

	cSaleD		REFCURSOR;
	_PrcLvlID	VARCHAR(3);
BEGIN
	SELECT INTO
		_Unit1, _Unit2, _Conv2, _Unit3, _Conv3
		Unit1, Unit2, Conv2, Unit3, Conv3
	FROM Item
	WHERE ItemID=_ItemID;

	IF NOT FOUND THEN
		RETURN;
	END IF;

	_PrcLvlID:=fPrcLvlID(_BPID);	/** get cust pricelevel or default if cust is null **/	

	SELECT INTO 
		_Price1.ListPrice, _Price1.DiscExp, _Price2.ListPrice, _Price2.DiscExp,  _Price3.ListPrice, _Price3.DiscExp
		Price1, Disc1Exp, Price2, Disc2Exp, Price3, Disc3Exp
	FROM Prc WHERE ItemID=_ItemID AND PrcLvlID=_PrcLvlID;

	IF (_Unit=_Unit3) THEN
		_retval:=_Price3;
	ELSIF (_Unit=_Unit2) THEN
		_retval:=_Price2;
	ELSE
		_retval:=_Price1;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSale_LastPrice (
	_ItemID		VARCHAR,
	_Qty		DECIMAL(15,4),
	_Unit		VARCHAR(5),
	_BPID		VARCHAR
) RETURNS SETOF TPriceExt AS
$BODY$
DECLARE
	_retval		TPriceExt%ROWTYPE;
	_LastSaleNo	VARCHAR;
	cSaleD		REFCURSOR;
	_found		BOOLEAN;
BEGIN

	IF (_BPID IS NOT NULL) THEN
		SELECT INTO _LastSaleNo MAX(h.SaleNo) FROM SaleD d JOIN Sale h ON d.SaleNo=h.SaleNo WHERE h.CustID=_BPID AND d.ItemID=_ItemID;
		_found=FOUND;
	ELSE
		_found=FALSE;
	END IF;
	

	IF (_found) THEN
		SELECT INTO
		_retval.BasePrice, _retval.ListPrice, _retval.DiscExp, _retval.DiscAmt, _retval.TaxID, _retval.TaxAmt 
		BasePrice, ListPrice, DiscExp, DiscAmt, TaxID, TaxAmt
		FROM SaleD 
		WHERE SaleNo=_LastSaleNo AND ItemID=_ItemID;

		_found=FOUND;
	END IF;

	IF (NOT _found) THEN
		SELECT INTO _retval * FROM fPriceExt(_ItemID, _Qty, _Unit, _BPID);
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPurc_LastPrice (
	_ItemID		VARCHAR,
	_Qty		DECIMAL(15,4),
	_Unit		VARCHAR(5),
	_BPID		VARCHAR
) RETURNS SETOF TPriceExt AS
$BODY$
DECLARE
	_retval		TPriceExt%ROWTYPE;
	_LastPurcNo	VARCHAR;
	cPurcD		REFCURSOR;
	_found		BOOLEAN;
BEGIN

	IF (_BPID IS NOT NULL) THEN
		SELECT INTO _LastPurcNo MAX(h.PurcNo) FROM PurcD d JOIN Purc h ON d.PurcNo=h.PurcNo WHERE h.VendorID=_BPID AND d.ItemID=_ItemID;
		_found=FOUND;

		/** alternatif: jika pembelian dari supplier ybs  tidak ditemukan, cari data pembelian dari supplier lain 
		* ...
		* ...
		**/
	ELSE
		_found=FALSE;
	END IF;

	IF (_found) THEN
		SELECT INTO
		_retval.BasePrice, _retval.ListPrice, _retval.DiscExp, _retval.DiscAmt, _retval.TaxID, _retval.TaxAmt 
		BasePrice, ListPrice, DiscExp, DiscAmt, TaxID, TaxAmt
		FROM PurcD
		WHERE PurcNo=_LastPurcNo AND ItemID=_ItemID;

		_found=FOUND;
	END IF;

	IF (NOT _found) THEN
		RETURN;		/** jika tidak ditemukan RETURN nothing **/

		/** ALTERNATIF-2 : jika tidak ditemukan RETURN nothing
		_retval:=(0, 0, NULL, 0, NULL, 0);
		**/
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fRandom (
	_range	INT
) RETURNS INT AS 
$BODY$
BEGIN
	RETURN(CAST((RANDOM()*_range) AS INT));
END
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSaleTotalMonth (
	_SaleDate	DATE
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(15,4);
	_Month		INT;
	_Year		INT;

	_TotalBal	NUMERIC(15,4);
	cBal		REFCURSOR;
BEGIN
	/** inisialisasi */
	_Month:=date_part('month',_SaleDate);
	_Year:=date_part('year',_SaleDate);

	SELECT INTO _retval COALESCE(SUM(total),0) FROM sale 
	WHERE date_part('month',saledate)=_Month AND date_part('year',saledate)=_Year;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPIDCheck(
	_ItemID		VARCHAR(20),
	_WhID		VARCHAR(3),
	_PIDStart	VARCHAR(25),
	_PIDEnd		VARCHAR(25)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_Qty		DECIMAL(15,4);
	
	_start		DECIMAL(25);
	_end		DECIMAL(25);
	_i		DECIMAL(25);
	_PIDtmp		VARCHAR(25);
	_len		INT;
BEGIN
	_retval:=(0, 'OK');
	_len:=LENGTH(_PIDStart);

	IF (_PIDEnd IS NULL OR _PIDEnd=_PIDStart) THEN
		SELECT INTO _Qty Qty FROM Stock WHERE ItemID=_ItemID AND ((_WhID IS NULL AND Qty>0) OR WhID=_WhID) AND PID=_PIDStart;
		IF NOT FOUND THEN
			_retval:=(1, 'Item:'||_ItemID||', Gd:'||COALESCE(_WhID, '<kosong>')||', PID:'||COALESCE(_PIDStart,'<kosong>')||' tidak ditemukan ! (spPIDCheck)');
		END IF;

		IF (_Qty=0) THEN
			_retval:=(2, 'Item:'||_ItemID||', Gd:'||COALESCE(_WhID, '<kosong>')||', PID:'||COALESCE(_PIDStart,'<kosong>')||' Stock Qty=0 ! (spPIDCheck)');
		END IF;
	ELSE
		_start=CAST(_PIDStart AS DECIMAL(25));
		_end=CAST(_PIDEnd AS DECIMAL(25));
		_i:=_start;

		LOOP
			_PIDtmp:=fZeroFill(_i, _len);

/*--			SELECT INTO _Qty Qty FROM Stock WHERE ItemID=_ItemID AND ((_WhID IS NULL AND Qty>0) OR WhID=_WhID) AND PID=_PIDtmp;*/
		SELECT INTO _Qty Qty FROM Stock WHERE ItemID=_ItemID AND PID=_PIDtmp AND (WhID=_WhID);
/*--SELECT INTO _Qty Qty FROM Stock WHERE ItemID=_ItemID AND PID=_PIDtmp AND ((_WhID IS NULL AND Qty>0) OR WhID=_WhID);*/

			IF (NOT FOUND) THEN
				_retval:=(1, 'Item:'||_ItemID||', Gd:'||COALESCE(_WhID, '<kosong>')||', PID:'||COALESCE(_PIDtmp,'<kosong>')||' tidak ditemukan ! (spPIDCheck)');
			END IF;

			IF (_Qty=0) THEN
				_retval:=(2, 'Item:'||_ItemID||', Gd:'||COALESCE(_WhID, '<kosong>')||', PID:'||COALESCE(_PIDtmp,'<kosong>')||' Stock Qty=0 ! (spPIDCheck)');
			END IF;

			EXIT WHEN (_i=_end OR _retval.ErrNo<>0);
			_i:=_i+1;
		END LOOP;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spBPBal_Update (
	_BPID		VARCHAR(15),
	_TDate		DATE,
	_TAmount	DECIMAL(19,4),
	_RefType	VARCHAR(5),
	_CrcID		VARCHAR(5),
	_RefNo VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_Month		INT;
	_PYear		INT;
	_fPYear		INTEGER;
	_BegBal		DECIMAL(19,4);
	cFutureBPBal	REFCURSOR;
	_found		VARCHAR;
	_DC		VARCHAR(1);

BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error UPDATE BPA:Balance (spBPBal_Update)';

	_Month=EXTRACT(month FROM _TDate);
	_PYear=EXTRACT(year FROM _TDate);

	IF(_RefType='FCN') THEN
		SELECT INTO _DC DC FROM FCN WHERE FCNNO=_RefNo;
	END IF;

	RAISE NOTICE '%', _RefType||' RefNo:'||_RefNo;
	
	IF ((_RefType='SALE') OR (_RefType='RCV') OR (_RefType='SRET') OR ((_RefType='FCN') and (_DC='D'))) THEN
		UPDATE BPAcc SET ARBalance=ARBalance+_TAmount WHERE BPID=_BPID AND CrcID=_CrcID;

		SELECT INTO _found BPID FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
		IF NOT FOUND THEN
			SELECT INTO _BegBal (BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04+BPDBal05+BPDBal06+BPDBal07+BPDBal08+BPDBal09+BPDBal10+BPDBal11+BPDBal12)
			FROM BPDBal
			WHERE BPID=_BPID
			AND PYear=(SELECT MAX(PYear) FROM BPDBal WHERE BPID=_BPID AND PYear<_PYear) AND CrcID=_CrcID;
			
			IF NOT FOUND THEN
				_BegBal:=0;
			END IF;

			INSERT INTO BPDBal(BPID, PYear, BPDBal00,CrcID) VALUES (_BPID, _PYear, _BegBal,_CrcID);
		END IF;

		
		IF (_Month=1) THEN	
		UPDATE BPDBal SET BPDBal01=BPDBal01+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
		
		ELSIF (_Month=2) THEN	
		UPDATE BPDBal SET BPDBal02=BPDBal02+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;

		ELSIF (_Month=3) THEN	
		UPDATE BPDBal SET BPDBal03=BPDBal03+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=4) THEN	
		UPDATE BPDBal SET BPDBal04=BPDBal04+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
		
		ELSIF (_Month=5) THEN	
		UPDATE BPDBal SET BPDBal05=BPDBal05+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=6) THEN	
		UPDATE BPDBal SET BPDBal06=BPDBal06+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=7) THEN
		UPDATE BPDBal SET BPDBal07=BPDBal07+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=8) THEN
		UPDATE BPDBal SET BPDBal08=BPDBal08+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=9) THEN	
		UPDATE BPDBal SET BPDBal09=BPDBal09+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=10) THEN
		UPDATE BPDBal SET BPDBal10=BPDBal10+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=11) THEN
		UPDATE BPDBal SET BPDBal11=BPDBal11+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=12) THEN
		UPDATE BPDBal SET BPDBal12=BPDBal12+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;

		END IF;

		OPEN cFutureBPBal FOR
			SELECT PYear FROM BPDBal WHERE BPID=_BPID AND PYear>_PYear AND crcID=_CrcID;

		LOOP
			FETCH cFutureBPBal INTO _fPYear ;
			IF NOT FOUND THEN
				EXIT;
			END IF;
			
			UPDATE BPDBal SET BPDBal00=BPDBal00+_TAmount WHERE BPID=_BPID AND PYear=_fPYear AND crcID=_CrcID;
			
			
		END LOOP;
		CLOSE cFutureBPBal;

	ELSIF ((_RefType='PURC') OR (_RefType='PAY') OR (_RefType='PRET') OR ((_RefType='FCN') and (_DC='C'))) THEN
		UPDATE BPAcc SET APBalance=APBalance+_TAmount WHERE BPID=_BPID AND CrcID=_CrcID;
		
		SELECT INTO _found BPID FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
		IF NOT FOUND THEN
			SELECT INTO _BegBal (BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04+BPCBal05+BPCBal06+BPCBal07+BPCBal08+BPCBal09+BPCBal10+BPCBal11+BPCBal12)
			FROM BPCBal
			WHERE BPID=_BPID
			AND PYear=(SELECT MAX(PYear) FROM BPCBal WHERE BPID=_BPID AND PYear<_PYear) AND CrcID=_CrcID;
			
			IF NOT FOUND THEN
				_BegBal:=0;
			END IF;
			
			INSERT INTO BPCBal(BPID, PYear, BPCBal00,CrcID) VALUES (_BPID, _PYear, _BegBal,_CrcID);
		END IF;

		
		IF (_Month=1) THEN	
		UPDATE BPCBal SET BPCBal01=BPCBal01+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
		
		ELSIF (_Month=2) THEN	
		UPDATE BPCBal SET BPCBal02=BPCBal02+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;

		ELSIF (_Month=3) THEN	
		UPDATE BPCBal SET BPCBal03=BPCBal03+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=4) THEN	
		UPDATE BPCBal SET BPCBal04=BPCBal04+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
		
		ELSIF (_Month=5) THEN	
		UPDATE BPCBal SET BPCBal05=BPCBal05+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=6) THEN	
		UPDATE BPCBal SET BPCBal06=BPCBal06+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=7) THEN
		UPDATE BPCBal SET BPCBal07=BPCBal07+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=8) THEN
		UPDATE BPCBal SET BPCBal08=BPCBal08+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=9) THEN	
		UPDATE BPCBal SET BPCBal09=BPCBal09+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=10) THEN
		UPDATE BPCBal SET BPCBal10=BPCBal10+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=11) THEN
		UPDATE BPCBal SET BPCBal11=BPCBal11+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			
		ELSIF (_Month=12) THEN
		UPDATE BPCBal SET BPCBal12=BPCBal12+_TAmount WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;

		END IF;

		OPEN cFutureBPBal FOR
			SELECT PYear FROM BPCBal WHERE BPID=_BPID AND PYear>_PYear and crcid=_crcID;

		LOOP
			FETCH cFutureBPBal INTO _fPYear ;
			IF NOT FOUND THEN
				EXIT;
			END IF;
			UPDATE BPCBal SET BPCBal00=BPCBal00+_TAmount WHERE BPID=_BPID AND PYear=_fPYear AND CrcID=_CrcID;
		
		END LOOP;
		CLOSE cFutureBPBal;
	END IF;
	

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spBPA_New(
	_BPID		VARCHAR(15),
	_BPAType	VARCHAR(1),
	_CrcID		VARCHAR(5),
	_BPADate	DATE,
	_RefType 	varchar(5),
	_RefNo 		varchar(15),
	_BPAAmt		DECIMAL(19,4),
	_BPANote	VARCHAR(255),
	_isAutoGen	BOOLEAN,
	_CrtBy		VARCHAR(10),
	_CrtDate	TIMESTAMP,
	_UpdBy		VARCHAR(10),
	_UpdDate	TIMESTAMP,
	_PerID		VARCHAR(5),
	_BranchID	VARCHAR(10),
	_DeptID		VARCHAR(5),
	_PrjID		VARCHAR(5),
	_ExcRate	NUMERIC(19,4),
	_FisRate	NUMERIC(19,4)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_BPANo	VARCHAR;
	_retval	SPStatus%ROWTYPE;
	_spret	SPStatus%ROWTYPE;
	_Count	INT;
BEGIN
	SELECT INTO _BPANo * from fTransNoGet('BPA');

	IF (_BPANo IS NULL) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error fTransNoGet(BPA) (spBPA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_BPID IS NULL) THEN
		RAISE EXCEPTION 'BPID tidak boleh NULL! (spBPA_New)';
	END IF;

	SELECT INTO _Count Count(*) FROM BPAcc WHERE BPID=_BPID AND CrcID=_CrcID;

	IF (_Count IS NULL OR _Count=0) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Mata Uang: '||_CrcID||' untuk Mitra Bisnis: '||_BPID||' belum di buat !! Silahkan edit Mitra Bisnis dan pilih tab Hutang/Piutang untuk menambahkan Mata Uang (spBPA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	INSERT INTO BPA(
		BPANo,CrcID, BPADate, BPAType, BPID, 
		DC, RefType, RefNo, BPAAmt, BPANote, 
		isAutoGen, CrtBy, CrtDate, UpdBy, UpdDate,
		PerID, BranchID, DeptID, PrjID, ExcRate, FisRate
	)
	VALUES (
		_BPANo, _CrcID, _BPADate, _BPAType, _BPID, 
		_BPAType, _RefType, _RefNo, _BPAAmt, _BPANote, 
		_isAutoGen, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
		_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate
	);

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error INSERT BPA (spBPA_New)'||_BPANo;

	SELECT INTO _spret * FROM spBPBal_Update(_BPID, _BPADate, _BPAAmt,_RefType,_CrcID,_RefNo);

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION  spBPA_Void (
	_BPANo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_BPID		VARCHAR(15);
	_BPAAmt		DECIMAL(19,4);
	_tmpBPAAmt	DECIMAL(19,4);
	
	_BPADate	DATE;
	_RefType	VARCHAR(5);
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_RefNo		VARCHAR;
	_CrcID		VARCHAR;

BEGIN
	_retval:=(50001, 'General ERROR spBPA_Void');

	SELECT INTO _BPADate, _BPID, _BPAAmt, _RefType, _RefNo, _CrcID BPADate, BPID, BPAAmt, RefType,RefNo,CrcID FROM BPA WHERE BPANo=_BPANo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='BPANo ' || _BPANo || ' not found (spBPA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=1032;
	_retval.ErrMsg:='Error EXEC spBal_Update (spBPA_New)';

	_tmpBPAAmt:=_BPAAmt*(-1);
	SELECT INTO _spret * FROM spBPBal_Update(_BPID, _BPADate, _tmpBPAAmt, _RefType, _CrcID, _RefNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;			
	END IF;


	_retval.ErrNo:=1033;
	_retval.ErrMsg:='Error DELETE BPA (spBPA_New)';
	DELETE FROM BPA WHERE BPANo=_BPANo;

	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function spBPA_Void_byRef (
	_RefType	VARCHAR,
	_RefNo		VARCHAR
) RETURNS SETOF SPSTATUS AS 
$BODY$
DECLARE	
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_BPANo		VARCHAR(15);
	_CrcID		VARCHAR(5);
	_BPADate	DATE;
	_BPID		VARCHAR(15);
	_BPAAmt		NUMERIC(19,4);
	_tmpBPAAmt	NUMERIC(19,4);

BEGIN
	_retval:=(50001, 'Error spBPA_Void_byRef');

	SELECT into _BPANo, _CrcID, _BPID, _BPADate, _BPAAmt 
		    BPANo, CrcID, BPID, BPADate, BPAAmt
	FROM BPA WHERE RefType=_RefType AND RefNo=_RefNo;

	IF NOT FOUND THEN
		/*
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Transaksi Hutang untuk '||_RefType||' #'||_RefNo||' tidak ditemukan! (spBPA_Void_byRef)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		*/
		_retval:=(0, 'No Transaction found!');
		RETURN NEXT _retval;
	ELSE
		_tmpBPAAmt=_BPAAmt*(-1);

		SELECT into _spret * FROM spBPBal_Update (_BPID, _BPADate, _tmpBPAAmt, _RefType, _CrcID, _RefNo);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		DELETE FROM BPA WHERE BPANo=_BPANo;
		
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCash_Remove(_CashID VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_RefType	VARCHAR(5);
	_RefNo		VARCHAR(15);
	
BEGIN
	_retval:=(0, 'OK');

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE CashBal (spCash_Remove)';
	DELETE FROM CashBal WHERE CashID=_CashID;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE CashPrv (spCash_Remove)';
	DELETE FROM CashPrv WHERE CashID=_CashID;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error DELETE CashA (spCash_Remove)';
	DELETE FROM CashA WHERE CashID=_CashID;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE Cash (spCash_Remove)';
	DELETE FROM Cash WHERE CashID=_CashID;

	_retval:=(0, 'DELETE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



/*
	
	Revision:
	Created 27/11/06 23:45 david
*/


CREATE OR REPLACE FUNCTION spCashBal_Update(
	_CashID		VARCHAR(15),
	_CashADate	DATE,
	_Amount		NUMERIC(19,4)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_month			INT4;
	_year			INT4;
	_fyear			INT4;
	_BegBal			NUMERIC(19,4);

	_found			VARCHAR;
	cFutureBal		CURSOR (_cyear INT4) IS SELECT PYear FROM CashBal WHERE CashID=_CashID AND PYear>_cyear;
BEGIN
	SELECT INTO _month DATE_PART('month', _CashADate);
	SELECT INTO _year DATE_PART('year', _CashADate);

	/*--BEGIN*/
		_retval.ErrNo:=1051;
		_retval.ErrMsg:='Error UPDATE Cash:Balance (spCashBal_Update)'||_CashID;

		UPDATE Cash SET CashBal=CashBal+_Amount WHERE CashID=_CashID;		/* UPDATE saldo di master Cash */
		
		IF NOT FOUND THEN
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		/* cek apakah saldo utk tahun ini sudah ada */
				
		SELECT INTO _found CashID FROM CashBal WHERE CashID=_CashID AND PYear=_year;
		IF NOT FOUND THEN
			SELECT INTO _BegBal (CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06+CashBal07+CashBal08+CashBal09+CashBal10+CashBal11+CashBal12)
			FROM CashBal WHERE CashID=_CashID 
			AND PYear=(SELECT MAX(PYear) FROM CashBal WHERE CashID=_CashID AND PYear<_year);

			IF NOT FOUND THEN
				_BegBal:=0;
			END IF;

			_retval.ErrNo:=1012;
			_retval.ErrMsg:='Error creating CASHBal for ' || _CashID || ' : Year ' || CAST(_year AS VARCHAR) || ' (spBPABal_Update)';
			INSERT INTO CashBal(CashID, PYear, CashBal00) VALUES (_CashID, _year, _BegBal);
			IF NOT FOUND THEN
				RAISE EXCEPTION '%', _retval.ErrMsg;					
			END IF;

		END IF;

		_retval.ErrNo:=1013;
		_retval.ErrMsg:='Error UPDATING CashBal:month FOR ' || _CASHID || ' : Month ' || CAST(_month AS VARCHAR) || ' (spBPABal_Update)';
		
		IF (_month=1) THEN 
			UPDATE CashBal SET CashBal01=CashBal01+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=2) THEN 
			UPDATE CashBal SET CashBal02=CashBal02+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=3) THEN 
			UPDATE CashBal SET CashBal03=CashBal03+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=4) THEN 
			UPDATE CashBal SET CashBal04=CashBal04+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=5) THEN 
			UPDATE CashBal SET CashBal05=CashBal05+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=6) THEN 
			UPDATE CashBal SET CashBal06=CashBal06+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=7) THEN 
			UPDATE CashBal SET CashBal07=CashBal07+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=8) THEN 
			UPDATE CashBal SET CashBal08=CashBal08+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=9) THEN 
			UPDATE CashBal SET CashBal09=CashBal09+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=10) THEN 
			UPDATE CashBal SET CashBal10=CashBal10+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=11) THEN 
			UPDATE CashBal SET CashBal11=CashBal11+_Amount WHERE CashID=_CashID AND PYear=_year;
		ELSIF (_month=12) THEN 
			UPDATE CashBal SET CashBal12=CashBal12+_Amount WHERE CashID=_CashID AND PYear=_year;
		END IF;

		IF NOT FOUND THEN /*-- CEK JIKA UPDATE GAGAL RAISE EXCEPTION*/
			RAISE EXCEPTION '%', _retval.ErrMsg;					
		END IF;


		OPEN cFutureBal(_year);

		LOOP
			FETCH cFutureBal INTO _fyear;
			IF NOT FOUND THEN
				EXIT;
			END IF;
			
			_retval.ErrNo:=1014;
			_retval.ErrMsg:='Error UPDATE future CashBal (spCashBal_Update)';
			UPDATE CashBal SET CashBal00=CashBal00+_Amount WHERE CashID=_CashID AND PYear=_fyear;
			IF NOT FOUND THEN
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		END LOOP;

		CLOSE cFutureBal;

	_retval:=(0, 'OK');
	/*--EXCEPTION*/
	/*--	WHEN OTHERS THEN NULL;*/
	/*--END;*/
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCashA_New (
	__CashID	VARCHAR,
	__BPID		VARCHAR,
	__CashAType 	VARCHAR,
	__CashADate	DATE,
	__InOut		VARCHAR,
	__RefType	VARCHAR,
	__RefNo		VARCHAR,
	__Amount	NUMERIC,
	__CashANote	VARCHAR,
	__IsAutoGen	BOOLEAN,
	__CrtBy		VARCHAR,
	__CrtDate	TIMESTAMP,
	__UpdBy		VARCHAR,
	__UpdDate	TIMESTAMP,

	_PerID		VARCHAR(5),
	_BranchID	VARCHAR(10),
	_DeptID		VARCHAR(5),
	_PrjID		VARCHAR(5),

	_Insert		BOOLEAN,
	_CrcID		VARCHAR,
	_ExcRate	NUMERIC(19,4),
	_FisRate	NUMERIC(19,4)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	
	_Month			SMALLINT;
	_PYear			INTEGER;
	_fPYear			INTEGER;
	_BegBal			DECIMAL(19,4);

	_CashID			VARCHAR(15);
	_BPID			VARCHAR(15);
	_CashAType 		VARCHAR(5);
	_CashADate		DATE;
	_InOut			VARCHAR(1);
	_RefType		VARCHAR(5);
	_RefNo			VARCHAR(15);
	_Amount			DECIMAL(19,4);
	_CashANote		VARCHAR;
	_IsAutoGen		BOOLEAN;
	_CrtBy			VARCHAR(10);
	_CrtDate		TIMESTAMP;
	_UpdBy			VARCHAR(10);
	_UpdDate		TIMESTAMP;

	_CashANo		VARCHAR(15);

	_isAutoReconcileIn	BOOLEAN;
	_isAutoReconcileOut	BOOLEAN;
	_isReconciled		BOOLEAN;
BEGIN
	_CashANo:=NULL;

	IF (_Insert=TRUE AND __Amount IS NULL) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error, jumlah Uang belum anda masukkan (kosong) (spCashA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_Insert=FALSE) THEN
		SELECT INTO _CashID, _BPID, _CashAType, _CashADate, _InOut, _RefType,
			_RefNo, _Amount, _CashANote, _IsAutoGen, _CrtBy, _CrtDate,
			_UpdBy, _UpdDate, _isReconciled
			CashID, BPID, CashAType, CashADate, InOut, RefType,
			RefNo, Amount, CashANote, IsAutoGen, CrtBy, CrtDate,
			UpdBy, UpdDate, isreconciled
		FROM CashA WHERE CashANo=_CashANo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Transaksi-Kas No:'||COALESCE(_CashANo, '<kosong>')||' tidak ditemukan ! (spCashA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		-- karena nggak diinsert apakah perlu di check & update isReconciled disini ?
		-- takutnya di program tidak dicek & tidak di set
	END IF;

	-- get AutoReconcile flag
	SELECT INTO 
		_isAutoReconcileIn, _isAutoReconcileOut 
		isAutoReconcileIn, isAutoReconcileOut
	FROM Cash WHERE cashid=__CashID;
	IF NOT FOUND THEN
		_isAutoReconcileIn:=TRUE;
		_isAutoReconcileOut:=TRUE;
	END IF;
	-- Cek direction : In (receive cash)  ...vs... Out (make payment)

	IF (__InOut='I') THEN
		_isReconciled:=_isAutoReconcileIn;
	ELSIF (__InOut='O') THEN
		_isReconciled:=_isAutoReconcileOut;
	END IF;

	-- jika set begbal cash/bank, langsung posting aja gak usah check isautoreconcile
	IF (__RefType='CADJ' AND __CashAType='BEG') THEN
		_isReconciled:=TRUE;
	END IF;

	IF (_Insert=TRUE) THEN
		_CashANo:=fTransNoGet('CASHA');

		IF (_CashANo IS NULL) THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Error in spTransNoGet (spCashA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		INSERT INTO CashA (CashANo, CashID, BPID, CashAType, CashADate,
			InOut, RefType, RefNo, Amount, CashANote,
			IsAutoGen, CrtBy, CrtDate, UpdBy, UpdDate,
			PerID, BranchID, DeptID, PrjID, isreconciled, CrcID, ExcRate, FisRate
			)
		VALUES (_CashANo, __CashID, __BPID, __CashAType, __CashADate,
			__InOut, __RefType, __RefNo, __Amount, __CashANote,
			__IsAutoGen, __CrtBy, __CrtDate, __UpdBy, __UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _isReconciled, _CrcID, _ExcRate, _FisRate
			);
	ELSE	-- bukan INSERT ?
		-- Update isReconcile flag 
		UPDATE CashA SET isReconciled=_isReconciled WHERE CashANo=_CashANo;
	END IF;

	IF (_isReconciled=TRUE) THEN
		SELECT INTO _spret * FROM spCashBal_Update(__CashID, __CashADate, __Amount);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spCashA_New (
	__CashID	VARCHAR,
	__BPID		VARCHAR,
	__CashAType 	VARCHAR,
	__CashADate	DATE,
	__InOut		VARCHAR,
	__RefType	VARCHAR,
	__RefNo		VARCHAR,
	__Amount	NUMERIC,
	__CashANote	VARCHAR,
	__IsAutoGen	BOOLEAN,
	__CrtBy		VARCHAR,
	__CrtDate	TIMESTAMP,
	__UpdBy		VARCHAR,
	__UpdDate	TIMESTAMP,

	_PerID		VARCHAR(5),
	_BranchID	VARCHAR(10),
	_DeptID		VARCHAR(5),
	_PrjID		VARCHAR(5),
	_CrcID		VARCHAR,
	_ExcRate	NUMERIC(19,4),
	_FisRate	NUMERIC(19,4)
)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spCashA_New(__CashID, __BPID, __CashAType, __CashADate, __InOut, __RefType, __RefNo, __Amount, __CashANote, __IsAutoGen, 
		__CrtBy, __CrtDate, __UpdBy, __UpdDate, _PerID, _BranchID, _DeptID, _PrjID, TRUE, _CrcID, _ExcRate, _FisRate);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCashA_Void(
	_CashANo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_JnlNo			VARCHAR(15);
	_CashID			VARCHAR(15);
	_Amount			DECIMAL(19,4);
	_tmpAmount		DECIMAL(19,4);
	_CashADate		DATE;
	_LockDate		DATE;
	
	cAcc 			REFCURSOR;

	_isAutoReconcileIn	BOOLEAN;
	_isAutoReconcileOut	BOOLEAN;
	_isReconciled		BOOLEAN;
BEGIN
	_retval:=(50001, 'GENERAL ERROR (spCashA_Void)');

	SELECT INTO _CashID, _CashADate, _Amount, _isReconciled cashID, CashADate, Amount, isreconciled FROM CashA WHERE CashANo=_CashANo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='CashANo ' || _CashANo || ' not found (spCashA_Void)';			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CADJ';
	IF(_LockDate is not null AND _LockDate>_CashADate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penyesuaian Kas Melebihi Batas LockDate!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_tmpAmount:=_Amount*(-1);
	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error EXEC spCashBal_Update (spCashA_Void)';
	
	IF (_isReconciled=TRUE) THEN	
		SELECT INTO _spret * FROM spCashBal_Update(_CashID, _CashADate, _tmpAmount);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF isAccInstalled() THEN
			SELECT into _spret * from spJnl_Void_byRef('CADJ',_CashANo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END IF;
	
	DELETE FROM CashA where CashANo=_CashANo;
	
	_retval:=(0, 'OK'); 
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCashA_Void_byRef (
	__RefType	VARCHAR,
	__RefNo		VARCHAR,
	__CashID	VARCHAR,
	__CashANo	VARCHAR
) RETURNS SETOF spStatus AS 
$BODY$
DECLARE
	_retval			spStatus%ROWTYPE;
	spretSpCashBal_Update	spStatus%ROWTYPE;

	_CashID			VARCHAR(15);
	_CashANo		VARCHAR;
	_CashADate		DATE;
	_Amount			DECIMAL(19,4);
	_tmpAmount		DECIMAL(19,4);

	cCashA			REFCURSOR;
	_isReconciled		BOOLEAN;
BEGIN
	_retval:=(50001, 'GENERAL ERROR spCashA_Void_byRef');
	
	IF (__CashANo IS NOT NULL) THEN
		OPEN cCashA FOR 
		SELECT CashANo, CashID, CashADate, Amount, isReconciled 
		FROM CashA 
		WHERE CashANo=__CashANo;
	ELSE
		IF (__CashID IS NOT NULL) THEN
			OPEN cCashA FOR 
			SELECT CashANo, CashID, CashADate, Amount, isReconciled 
			FROM CashA 
			WHERE CashID=__CashID AND RefType=__RefType AND RefNo=__RefNo;
		ELSE
			OPEN cCashA FOR
			SELECT CashANo, CashID, CashADate, Amount, isReconciled 
			FROM CashA 
			WHERE RefType=__RefType AND RefNo=__RefNo;
		END IF;
	END IF;

	LOOP
		FETCH cCashA INTO _CashANo, _CashID, _CashADate, _Amount, _isReconciled;
		EXIT WHEN NOT FOUND;

		IF (_isReconciled=TRUE) THEN
			_tmpAmount:=_Amount*(-1);
			SELECT INTO spretSpCashBal_Update * FROM spCashBal_Update(_CashID, _CashADate, _tmpAmount);

			IF spretSpCashBal_Update.ErrNo<>0 THEN
				_retval.ErrNo:=spretSpCashBal_Update.ErrNo;
				_retval.ErrMsg:=spretSpCashBal_Update.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		DELETE FROM CashA WHERE CashANo=_CashANo;

	END LOOP;
	CLOSE cCashA;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockLog(
	__StockLogDate		DATE,
	__Itemid		VARCHAR,
	__WhID			VARCHAR,
	__PID			VARCHAR,
	__QtyNew		NUMERIC(15,4),
	__CostNew		NUMERIC(19,4),
	__SubtotalNew	NUMERIC(19,4),
	__UpdQty		NUMERIC(15,4),
	__UpdCost		NUMERIC(19,4),
	__UpdSubtotal	NUMERIC(19,4),
	__QtyOld		NUMERIC(15,4),
	__CostOld		NUMERIC(19,4),
	__SubtotalOld	NUMERIC(19,4),
	__RefType		VARCHAR,
	__RefNo			VARCHAR
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret 			SPStatus%ROWTYPE;

	--_StockLogID			BIGINT;
	--_CurrentCost	NUMERIC(19,4);

	--_Itemid			VARCHAR;
	--_WhID			VARCHAR;
	--_PID			VARCHAR;
	--_BatchID		BIGINT;
	--_Qty			NUMERIC(15,4);
	--_Cost			NUMERIC(19,4);
	--_Subtotal		NUMERIC(19,4);
	--_QtyOld			NUMERIC(15,4);
	--_CostOld		NUMERIC(19,4);
	--_SubtotalOld	NUMERIC(19,4);
	--_RefType		VARCHAR;
	--_RefNo			VARCHAR;
BEGIN
	_retval:=(0, 'OK');

	--SELECT INTO _CostID, _CurrentCost costid, cost 
	--FROM costlog WHERE costdate=__CostDate AND itemid=__ItemID AND whid=__WhID AND pid=__PID;

	--IF (_CostID IS NULL) THEN 
	INSERT INTO stocklog (stocklogdate, itemid, whid, pid, qtynew, costnew, subtotalnew, updqty, updcost, updsubtotal, qtyold, costold, subtotalold, reftype, refno)
		VALUES (__StockLogDate, __ItemID, __WhID, __PID, __QtyNew, __CostNew, __SubtotalNew, __UpdQty, __UpdCost, __UpdSubtotal, __QtyOld, __CostOld, __SubtotalOld, __RefType, __RefNo);
	IF (NOT FOUND) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Gagal Buat Cost Log (spCostLog)';
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;
	--ELSE
	--	IF (_CurrentCost!=__Cost) THEN
	--		INSERT INTO costlog (costdate, itemid, whid, pid, batchid, qty, cost, subtotal, qtyold, costold, subtotalold, reftype, refno)
	--			VALUES (__CostDate, __ItemID, __WhID, __PID, __BatchID, __Qty, __Cost, __Subtotal, __QtyOld, __CostOld, __SubtotalOld, __RefType, __RefNo);
	--	ELSE
	--		UPDATE costlog SET qty=__Qty, cost=__Cost, subtotal=__Subtotal, 
	--			qtyold=__QtyOld, costold=__CostOld, subtotalold=__SubtotalOld, 
	--			reftype=__RefType, refno=__RefNo
	--		WHERE costid=_CostID;  
	--	END IF;
	--END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockBal_Update(
	__ItemID		VARCHAR(20),
	__WhID			VARCHAR(3),
	__PID			VARCHAR(25),
	__Qty			DECIMAL(15,4),
	__Unit			VARCHAR(5),
	__Conv			DECIMAL(15,4),
	__QtyX			DECIMAL(15,4),
	__Cost			DECIMAL(19,4),
	__StockADate		DATE,
	__StockAType		VARCHAR(5),
	__RefNo			VARCHAR,
	__SubTotal		DECIMAL(15,4)
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_IsStock		BOOLEAN;
	_curQty			NUMERIC(15,4);
	_curQtyX		NUMERIC(15,4);
	_curAvgCost		NUMERIC(19,4);
	_newQty			NUMERIC(15,4);
	_newQtyX		NUMERIC(15,4);
	_newAvgCost		NUMERIC(19,4);
	_IsEmpty		BOOLEAN;
	_Cost1			NUMERIC(19,4);
	_Qty_Unit1		NUMERIC(15,4);
	_Month			INT2;
	_PYear			INT4;
	_fPYear			INT4;
	_BegBal			DECIMAL(15,4);
	_Cost			DECIMAL(19,4);
	_PID			VARCHAR(25);
	_StockGuard		BOOLEAN;

	_OpDate			DATE;
	_OpEmpID		VARCHAR;
	_OpUsrID		VARCHAR;
	_OpNote			VARCHAR;

	_Count			INT;

	cFutureStockBal		REFCURSOR;
	_usePID			BOOLEAN;
	_isPIDUnique		BOOLEAN;
	_regval			VARCHAR;
	_PrcvIType		VARCHAR;
	_IsUpdate		BOOLEAN;
	_InOut			VARCHAR;
	_curSubtotal		NUMERIC(19,4);
	_newSubtotal		NUMERIC(19,4);
	_DeliIType		VARCHAR;
	_IsUpdateDeli		BOOLEAN;

	_sadjmtd		VARCHAR;
BEGIN
	_retval:=(0, 'OK');
	-- copy variabel yg mungkin butuh di replace valuenya 
	_Cost:=__Cost;
	_PID:=__PID;

	SELECT INTO 
		_IsStock, _usePID, _isPIDUnique  
		IsStock, usePID, isPIDUnique 
	FROM Item WHERE ItemID=__ItemID;
		
		
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error, Item:'||COALESCE(__ItemID, '<kosong>')||' tidak ditemukan (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_IsStock=FALSE) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error, Item:'||COALESCE(__ItemID, '<kosong>')||' TIDAK di-SET untuk diStock (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (__StockADate IS NULL) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Tanggal transaksi Stock NULL! (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	-- use PID ? check ---
	IF (_usePID) THEN
		IF (_PID IS NULL OR _PID='') THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg:='PID untuk Item:'||COALESCE(__ItemID, '<kosong>')||' tidak boleh kosong (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;				
		END IF;
	ELSE	-- NOT usePID ?
		IF (_PID IS NULL) THEN	-- replace NULL -> '' (empty string)
			_PID:='';
		ELSIF (_PID<>'') THEN	-- tidak pakai PID tapi ngeyel di kasih? tolak !
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Item:'||COALESCE(__ItemID, '<kosong>')||' tidak memerlukan PID (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	--- Get Current Stock ---
	SELECT INTO 
			_curQty, _curQtyX, _curAvgCost, _curSubtotal
			Qty, QtyX, AvgCost, (CASE WHEN qty=0 THEN 0 ELSE COALESCE(SubTotal,0) END)
	FROM Stock 
	WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID;

	IF NOT FOUND THEN
		
		IF (__Qty<0 AND _usePID) THEN  -- jika pengeluaran-barang(sale/pret,stockout) dan Item/PID tidak ditemukan
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Stock untuk Item:'||COALESCE(__ItemID, '<kosong>')||', PID:'||COALESCE(_PID,'<kosong>')||' Gd:'||COALESCE(__WhID,'<kosong>')||' tidak ditemukan (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- Purchase and Not yet Stocked ? -> Create Stock, isi default 0*/
		INSERT INTO Stock(ItemID, WhID, PID) VALUES (__ItemID, __WhID, _PID);

		IF NOT FOUND THEN
			_retval.ErrNo=50001;
			_retval.ErrMsg:='Error membuat kartu stok untuk Item:'||COALESCE(__ItemID, '<kosong>')||', PID:'||COALESCE(_PID, '<kosong>')||', Gd:'||COALESCE(__WhID, '<kosong>')||' (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		--- belum punya kartu stock ? buatin baru! ---
		_curQty:=0;
		_curQtyX:=0;
		_curAvgCost:=0;	
		_curSubtotal:=0;

	END IF;

	--*** IF @Cost ommitted, use last read AverageCost instead! 
	IF (_Cost IS NULL) THEN 
		_Cost:=_curAvgCost;
	END IF;

/***************** Hitung Qty Baru **********************************/

	-- convert ke Qty1 (qty satuan terkecil)
	_Qty_Unit1:=__Qty*__Conv;

	IF (_Qty_Unit1 IS NULL) THEN
		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error Konversi Item:'||COALESCE(__ItemID, '<kosong>')||' Unit:'||COALESCE(_Unit, '<kosong>')||' (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_newQty:=_curQty+_Qty_Unit1;
	_newQtyX:=_curQtyX+__QtyX;

/***************** EOF Hitung Qty Baru **********************************/


/***************** Check PID Unique *************************************/
	IF (_newQty<0) THEN
		IF (_usePID AND _isPIDUnique) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Stock Item:'||COALESCE(__ItemID, '<kosong>')||' tidak mencukupi (atau kosong) untuk transaksi ini (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;	
		ELSE
			_IsEmpty:=true;
		END IF;
	ELSE
		_IsEmpty:=false;
	END IF;

	IF (_usePID=true AND _isPIDUnique=true AND _newQty>1) THEN
		_retval.ErrNo:=50005;
		_retval.ErrMsg:='PID untuk Item:' ||COALESCE(__ItemID, '<kosong>') || ' harus unik, Untuk PID:'||COALESCE(_PID, '<kosong>')||' Qty tidak boleh lebih dari 1 (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
/***************** Check PID Unique *************************************/

/****************************************************************
	HITUNG AVERAGE HPP baru jika:

	- Transaksi Pembelian/Mutasi-masuk (Qty_Unit1>0)
	AND Cost<>AvgCost (ada perbedaan cost)
	AND Qty(newQty) akhir diatas 0

****************************************************************/
	IF (__Conv=0 OR __Qty=0) THEN
		_Cost1:=0;
	ELSE
		IF (__SubTotal IS NOT NULL) THEN  
			_Cost1:=__SubTotal/__Qty/__Conv;		-- Untuk meningkatkan presisi, hitung dari SubTotal
		ELSE
			_Cost1:=_Cost/__Conv;
		END IF;
	END IF;

	--*** Lacak ke detail transaksi untuk menentukan perlu UPDATE Hpp atau nggak 
	IF (__StockAType='PRCV') THEN		-- KLO PRCV YANG TIPE NYA RETUR NGGAK UPDATE HPP...MANGKANNYA INI CARI TIPE NYA PRCV NYA DULU
		SELECT INTO _PRCVITYPE PRCVITYPE FROM PRCV WHERE PRCVNO=__RefNo;
		IF(_PRCVITYPE='PRET') THEN
			_IsUpdate:=FALSE;
		ELSE
			_IsUpdate:=TRUE;
		END IF;
	ELSEIF(__StockAType='DELI') THEN 	-- kalau DELI YANG TIPE NYA RETUR UPDATE HPP...MANGKANNYA INI CARI TIPE NYA DELI NYA DULU
		SELECT INTO _DELIITYPE DELIITYPE FROM DELI WHERE DELINO=__RefNo;
		IF(_DELIITYPE='SRET') THEN
			_IsUpdateDeli:=TRUE;
		ELSE
			_IsUpdateDeli:=FALSE;
		END IF;
	ELSEIF(__StockAType='STTR') THEN 	--  STTR ? CARI INOUT NYA , KLO TIPE NYA OUTPUT GAK UPDATE HPP
		SELECT INTO _InOut inout FROM STOCKA WHERE StockAType='STTR' AND StockANo=__RefNo;

	ELSEIF(__StockAType='SADJ') THEN  	-- CARI METODE NYA SADJ , KLO METODE NYA RPL LANGSUNG NIMPUK I HPP, KLO METODENYA RD UPDATE HPP, KLO METODENYA PL G UBAH HPP
		SELECT INTO _SAdjMtd SAdjMtd FROM Sadj WHERE SadjNO=__RefNo;
	END IF;

	IF (
		(
			(__StockAType='PURC') 
			OR (__StockAType='PRCV' AND _IsUpdate=TRUE) 
			OR (__StockAType='BEG') 
			OR (__StockAType='STTR') 
			OR (__StockAType='SRET') 
			OR (__StockAType='PRET') 
			OR (__StockAType='MFG') 
			OR (__StockAType='DELI' AND _IsUpdateDeli=TRUE) 
			OR (__StockAType='SADJ' AND _SADJMtd='RD') 
			OR (__StockAType='SIN')
		) 
		AND (_Qty_Unit1<>0) AND (_Cost1<>_curAvgCost))
	THEN
/*******************************************************************************
						Hitung AvgCost baru
*******************************************************************************/

			IF (__SubTotal IS NOT NULL) THEN
				_newSubtotal:=_curSubTotal+__SubTotal;
			ELSE
				_newSubtotal:=_curSubTotal+(__Qty*_Cost);
			END IF;

			IF (_newQty = 0) THEN
				_newAvgCost:=_curAvgCost;
			ELSE
				_newAvgCost:=_newSubtotal/_newQty;
			END IF;

	ELSIF((__StockAType='SADJ' AND _SADJMtd='RPL')) THEN
		_newAvgCost:=_Cost;
		_newSubtotal:=_newQty*_newAvgCost;
	ELSE
		IF (__SubTotal IS NOT NULL) THEN
			_newSubtotal:=_curSubTotal+__SubTotal;
		ELSE
			_newSubtotal:=_curSubTotal+(__Qty*_Cost);
		END IF;

		IF (_newQty = 0) THEN
			_newAvgCost:=_curAvgCost;
		ELSE
			_newAvgCost:=_newSubtotal/_newQty;
		END IF;

	END IF;

	---** LAST VALIDATION before committed to STOCK to check for NULL values for field AvgCost & Qty 
	IF (_newSubtotal IS NULL OR _newAvgCost IS NULL) THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error try to UPDATE Stock cost with NULL for Item:'||COALESCE(__ItemID, '<kosong>')||' (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

/****************  STOCK_GUARD ALWAY ON !!! *************************
	SELECT INTO _regval regval FROM reg WHERE regid='STOCK_GUARD';
	_StockGuard:=(_regval='1' OR UPPER(_regval)='TRUE');
**/
	_StockGuard:=TRUE;

	IF((_StockGuard) AND (_newQty<0)) THEN
		_retval.ErrNo:=50005;
		_retval.ErrMsg:='Stock ItemID:'||COALESCE(__ItemID, '<kosong>')||', Wh:'||COALESCE(__WhID, '<kosong>')||', PID:'||COALESCE(_PID, '<kosong>')||' Qty tidak boleh minus (spStockBal_Update)!!';
		_retval.ErrMsg:=_retval.ErrMsg||E'\n'||'Qty:'||_newQty;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_newSubTotal<0) THEN
		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Total nilai Stock ItemID:'||COALESCE(__ItemID, '<kosong>')||', Wh:'||COALESCE(__WhID, '<kosong>')||', PID:'||COALESCE(_PID, '<kosong>')||' tidak boleh minus (spStockBal_Update)!!';
		_retval.ErrMsg:=_retval.ErrMsg||E'\n'||'SubTotal:'||_newSubTotal;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

/******** UPDATE Stock *********/
	UPDATE Stock SET 
		Qty=_newQty, 
		QtyX=_newQtyX, 
		AvgCost=_newAvgCost, 
		SubTotal=_newSubTotal
	WHERE 
		ItemID=__ItemID AND WhID=__WhID AND PID=_PID;


	/** ===================== RUNNING COST LOGGING ============================================= */
	SELECT INTO _spret * FROM spStockLog(__StockADate, __ItemID, __WhID, __PID, 
			_newQty, _newAvgCost, _newSubTotal, 
			_Qty_Unit1, _Cost1, __SubTotal, 
			_curQty, _curAvgCost, _curSubTotal,
			__StockAType, __RefNo);
	IF (_spret.ErrNo!=0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_PYear:=date_part('year', __StockADate);
	_Month:=date_part('month',  __StockADate);

	/*** Check Current Year StockBal is EXIST!
		**** NO ? -> create one *****/
	IF NOT EXISTS (SELECT 1 FROM StockBal WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear) THEN 
		SELECT INTO _BegBal (StockBal00+StockBal01+StockBal02+StockBal03+StockBal04+StockBal05+StockBal06+StockBal07+StockBal08+StockBal09+StockBal10+StockBal11+StockBal12)
		FROM StockBal WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID 
		AND PYear=(SELECT MAX(PYear) FROM StockBal WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear<_PYear);

		/*** Jika belum ada Saldo Tahun sebelumnya, SaldoAwal:=0 ***/
		IF NOT FOUND THEN
			 _BegBal:=0;
		END IF;

		INSERT INTO StockBal(ItemID, WhID, PID, PYear, StockBal00) 
		VALUES (__ItemID, __WhID, _PID, _PYear, _BegBal);

		_retval.ErrMsg:='Error INSERT StockBal for Year:'||CAST(_PYear AS VARCHAR(4))||', ItemID:'||__ItemID||', Gd:'||__WhID||', PID:'||_PID||' (spStockBal_Update)';
			
		IF NOT FOUND THEN
			_retval.ErrMsg:='Error INSERT StockBal for Year:'||CAST(_PYear AS VARCHAR(4))||', ItemID:'||__ItemID||', Gd:'||__WhID||', PID:'||_PID||' (spStockBal_Update)';
		END IF;
	END IF;

	IF (_Month=1) THEN
		UPDATE StockBal SET StockBal01=StockBal01+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=2) THEN
		UPDATE StockBal SET StockBal02=StockBal02+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=3) THEN
		UPDATE StockBal SET StockBal03=StockBal03+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=4) THEN
		UPDATE StockBal SET StockBal04=StockBal04+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=5) THEN
		UPDATE StockBal SET StockBal05=StockBal05+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=6) THEN
		UPDATE StockBal SET StockBal06=StockBal06+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=7) THEN
		UPDATE StockBal SET StockBal07=StockBal07+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=8) THEN
		UPDATE StockBal SET StockBal08=StockBal08+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=9) THEN
		UPDATE StockBal SET StockBal09=StockBal09+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=10) THEN
		UPDATE StockBal SET StockBal10=StockBal10+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=11) THEN
		UPDATE StockBal SET StockBal11=StockBal11+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=12) THEN
		UPDATE StockBal SET StockBal12=StockBal12+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	END IF;

	IF NOT FOUND THEN
		_retval.ErrNo:=50016;
		_retval.ErrMsg:='Error UPDATE StockBal for Year:'||COALESCE(CAST(_PYear AS VARCHAR(4)), '<kosong>')||' Month:'||COALESCE(CAST(_Month AS VARCHAR(2)), '<kosong>')||' (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** UPDATE future stockbal **/
	OPEN cFutureStockBal FOR
		SELECT PYear FROM StockBal WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear>_PYear;
	LOOP
		FETCH cFutureStockBal INTO _fPYear;
		IF NOT FOUND THEN
			EXIT;
		END IF;

		UPDATE StockBal SET StockBal00=StockBal00+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_fPYear;
		IF NOT FOUND THEN
			_retval.ErrNo:='50017';
			_retval.ErrMsg:='Error UPDATE future StockBal:StockBal00 for Year:'||COALESCE(CAST(_fPYear AS VARCHAR(4)), '<kosong>')||' (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cFutureStockBal;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD-1 ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spStockBal_Update(
	__ItemID		VARCHAR(20),
	__WhID			VARCHAR(3),
	__PID			VARCHAR(25),
	__Qty			DECIMAL(15,4),
	__Unit			VARCHAR(5),
	__Conv			DECIMAL(15,4),
	__QtyX			DECIMAL(15,4),
	__Cost			DECIMAL(19,4),
	__StockADate	DATE,
	__StockAType	VARCHAR(5)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_spret			SPStatus%ROWTYPE;
	_retval			SPStatus%ROWTYPE;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _spret * FROM spStockBal_Update(
		__ItemID,
		__WhID,
		__PID,
		__Qty,
		__Unit,
		__Conv,
		__QtyX,
		__Cost,
		__StockADate,
		__StockAType,
		CAST(NULL AS VARCHAR),
		CAST(NULL AS DECIMAL(15,4))
	);

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=_spret;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD-2 ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spStockBal_Update(
	__ItemID		VARCHAR(20),
	__WhID			VARCHAR(3),
	__PID			VARCHAR(25),
	__Qty			DECIMAL(15,4),
	__Unit			VARCHAR(5),
	__Conv			DECIMAL(15,4),
	__QtyX			DECIMAL(15,4),
	__Cost			DECIMAL(19,4),
	__StockADate		DATE,
	__StockAType		VARCHAR(5),
	__RefNo			VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_spret			SPStatus%ROWTYPE;
	_retval			SPStatus%ROWTYPE;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _spret * FROM spStockBal_Update(
		__ItemID,
		__WhID,
		__PID,
		__Qty,
		__Unit,
		__Conv,
		__QtyX,
		__Cost,
		__StockADate,
		__StockAType,
		__RefNo,
		CAST(NULL AS DECIMAL(15,4))
	);

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=_spret;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


 

CREATE OR REPLACE FUNCTION spStockBal_Update(
	__ItemID	VARCHAR(20),
	__WhID		VARCHAR(3),
	__PID		VARCHAR(25),
	__Qty		DECIMAL(15,4),
	__Unit		VARCHAR(5),
	__Conv		DECIMAL(15,4),
	__QtyX		DECIMAL(15,4),
	__Cost		DECIMAL(19,4),
	__StockADate	DATE,
	__StockAType	VARCHAR(5),
	__isNew		BOOLEAN
)
RETURNS SETOF spstatus AS
$BODY$

DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_IsStock	BOOLEAN;
	
	_curQty		NUMERIC(15,4);
	_curQtyX	NUMERIC(15,4);
	_curAvgCost	NUMERIC(19,4);

	_newQty		NUMERIC(15,4);
	_newQtyX	NUMERIC(15,4);
	_newAvgCost	NUMERIC(19,4);		
	
	_IsEmpty	BOOLEAN;
	_Cost1		NUMERIC(19,4);		
	_Qty_Unit1	NUMERIC(15,4);
	_Month		INT2;
	_PYear		INT4;
	_fPYear		INT4;
	_BegBal		NUMERIC(15,4);
	_Cost		NUMERIC(19,4);
	_PID		VARCHAR(25);
	_StockGuard	BOOLEAN;

	_OpDate		DATE;	
	_OpEmpID	VARCHAR;
	_OpUsrID	VARCHAR;
	_OpNote		VARCHAR;

	_Count		INT;
	
	cFutureStockBal REFCURSOR;
	_usePID		BOOLEAN;
	_isPIDUnique	BOOLEAN;
	_regval		VARCHAR;

	/** variable khusus untuk hitung hpp */
	_hppQty		NUMERIC(15,4); /** buat nampung hasil perhitungan qty (qty min dianggap nol) */
	_hppCost	NUMERIC(19,4); /** buat nampung hasil perhitungan cost (cost min dianggap nol) */
	_hppSubtotal	NUMERIC(19,4); /** buat nampung hasil perhitungan qty*cost (subtot min dianggap nol) */
BEGIN
	/** inisialisasi variable hpp */
	_hppQty:=0;
	_hppCost:=0;
	_hppSubtotal:=0;

	/** copy variabel yg mungkin butuh di replace valuenya **/
	_Cost:=__Cost;
	_PID:=__PID;

	/** praduga bersalah **/
	_retval.ErrNo:=1;
	_retval.ErrMsg:='General Error in spStockBal_Update';
		
	SELECT INTO 
		_IsStock, _usePID, _isPIDUnique  
		IsStock, usePID, isPIDUnique 
	FROM Item WHERE ItemID=__ItemID;
		
		
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error, Item:'||__ItemID||' tidak ditemukan (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
				
	IF (_IsStock=FALSE) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error, Item:'||__ItemID||' TIDAK di-SET untuk diStock (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** PID can not be NULL, fix it if applicable **/
	IF (_usePID) THEN
		IF (_PID IS NULL OR _PID='') THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg:='PID untuk Item:'||__ItemID||' tidak boleh kosong (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;				
		END IF;
	ELSE
		/** jika tidak pakai PID ... ya cingcay laaaa :) ... kasiii daaah **/
		IF (_PID IS NULL) THEN
			_PID:='';
		ELSIF (_PID<>'') THEN
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Item:'||__ItemID||' tidak memerlukan PID (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		END IF;			
	END IF;

		
	/*** Get Current Stock ***/
	SELECT INTO 
			_curQty, _curQtyX, _curAvgCost, _hppQty, _hppCost, _hppSubtotal
		   Qty, QtyX, AvgCost, fPoZ(Qty), fPoZ(AvgCost), fPoZ((CASE WHEN qty=0 THEN 0 ELSE COALESCE(SubTotal,0) END))
	FROM Stock 
	WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID;
/* DEBUG
RAISE NOTICE 'ItemID:%', __ItemID;
RAISE NOTICE 'WhID:%', __WhID;
RAISE NOTICE 'PID:%', __PID;
RAISE NOTICE 'curQty:%', _curQty;
RAISE NOTICE 'curAvgCost:%', _curAvgCost;
**/

		
	IF NOT FOUND THEN
	
		IF (__Qty<0 AND _usePID) THEN  /*** jika penjualan dan Item/PID tidak ditemukan ***/
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Stock untuk Item:'||__ItemID||', PID:'||_PID||' Gd:'||__WhID||' tidak ditemukan (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
				
		/* Purchase and Not yet Stocked ? -> Create Stock, isi default 0*/
		INSERT INTO Stock(ItemID, WhID, PID) VALUES (__ItemID, __WhID, _PID);

		IF NOT FOUND THEN
			_retval.ErrNo=50001;
			_retval.ErrMsg:='Error membuat kartu stok untuk Item:'||__ItemID||', PID:'||_PID||', Gd:'||__WhID||' (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
			
		/** anak baru ? kasian deh ... kasih data awal dunk! **/
		_curQty:=0;
		_curQtyX:=0;
		_curAvgCost:=0;		
		
		_hppQty:=0;
		_hppCost:=0;		
		_hppSubtotal:=0;						
	END IF;

		
	/*** IF @Cost ommitted, use last read AverageCost instead! ***/
	IF (_Cost IS NULL) THEN 
		--_Cost:=_curAvgCost;
		_Cost:=_hppCost;
	END IF;

	/** convert ke Qty1 (qty dasar) ***/
	_Qty_Unit1:=__Qty*__Conv;

	IF (_Qty_Unit1 IS NULL) THEN
		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error Konversi Item:'||__ItemID||' Unit:'||_Unit||' (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_newQty:=_curQty+_Qty_Unit1;
	_newQtyX:=_curQtyX+__QtyX;

	_hppQty:=_hppQty+_Qty_Unit1;
	
	/** reserved utk update "Stock:isEmpty"
	***
	IF (_newQty<=0) THEN
		_IsEmpty:=true;							
	ELSE
		_IsEmpty:=false;
	END IF;
	**/
		

	IF (_newQty<0) THEN
		IF (_usePID AND _isPIDUnique) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Stock Item:'||__ItemID||' tidak mencukupi (atau kosong) untuk transaksi ini (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		ELSE
			_IsEmpty:=true;							
		END IF;
	ELSE
		_IsEmpty:=false;
	END IF;

	/** DEBUG		
	RAISE NOTICE 'newQty:%', _newQty;
	*/

	IF _usePID=true AND _isPIDUnique=true AND _newQty>1 THEN
		_retval.ErrNo:=50005;
		_retval.ErrMsg:='PID untuk Item:' || __ItemID || ' harus unik, PID:'|| _PID ||' yang anda masukkan sudah ada (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
		
	/****************************************************************
		HITUNG AVERAGE HPP baru jika:
		- Transaksi Pembelian/Mutasi (Qty_Unit1>0)
		- Cost<>AvgCost (ada perbedaan cost)
		- Qty(newQty) akhir diatas 0

		BUG: 
			IF (@newQty<>0) -- AND (@Cost1<>@curAvgCost)

		FIXED 2006/04/16 22:52
	****************************************************************/
	IF __Conv=0 THEN
		_Cost1:=0;
	ELSE
		_Cost1:=_Cost/__Conv;
	END IF;

	IF (__isNew=TRUE) THEN			
		IF (__Qty>0 AND _newQty<>0) THEN		/*** Hitung AvgCost baru ***/			
			--_newAvgCost:=((_curQty*_curAvgCost)+(__Qty*_Cost))/_newQty;
			_newAvgCost:=((_hppQty*_hppCost)+(__Qty*_Cost))/_newQty;
		ELSE
			--_newAvgCost:=_curAvgCost;	/*** Jika @newQty<=0 THEN update avgcost dengan cost terbaru ***/
			_newAvgCost:=_hppCost;
		END IF;
	ELSE
		IF (__Qty>0) THEN				/*** Hitung AvgCost baru ***/			
			--_newAvgCost:=_curAvgCost;	/*** Jika @newQty<=0 THEN update avgcost dengan cost terbaru ***/
			_newAvgCost:=_hppCost;
		ELSIF (_newQty<>0) THEN
			--_newAvgCost:=((_curQty*_curAvgCost)+(__Qty*_Cost))/_newQty;
			_newAvgCost:=((_hppQty*_hppCost)+(__Qty*_Cost))/_newQty;
		ELSE
			--_newAvgCost:=_curAvgCost;	/*** Jika @newQty<=0 THEN update avgcost dengan cost terbaru ***/
			_newAvgCost:=_hppCost;
		END IF;
	END IF;
		
	/**** LAST VALIDATION before committed to STOCK
	 **** to check for NULL values for field AvgCost & Qty ***/
	IF _newAvgCost IS NULL THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error try to UPDATE AvgCost with NULL for Item:'||__ItemID||' (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/******** UPDATE Stock *********/

	_retval.ErrMsg:='Error UPDATE Stock! '||'ItemID:'||__ItemID||', Wh:'||__WhID||', PID:'||_PID||' (spStockBal_Update)';	

	/*Cek Reg ngambil STOCK_GUARD*/
	SELECT INTO _regval * FROM fReg('STOCK_GUARD');
	_StockGuard:=(_regval='1' OR UPPER(_regval)='TRUE');

	IF((_StockGuard) AND (_newQty<0)) THEN
		_retval.ErrNo:=50005;
		_retval.ErrMsg:='Stock ItemID:'||__ItemID||', Wh:'||__WhID||', PID:'||_PID||' Qty tidak boleh minus (spStockBal_Update)!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_OpDate:=__StockADate;

	IF (__StockAType='SADJ') THEN
		SELECT INTO _OpEmpID,_OpUsrID,_Count s.EmpID,s.CrtBy,Count(*) 
		FROM StockA s,StockAD d
		WHERE s.StockANo=d.StockANo AND d.ItemID=__ItemID AND d.whid=__WhID AND StockAType='SADJ' AND s.StockADate=__StockADate AND d.qty=__Qty AND s.stockanote=('Penyesuaian Stock, emp :'||s.EmpID||','||s.CrtBy||' tgl '||_OpDate)
		GROUP BY s.EmpID,S.CrtBy;
	END IF;		


	UPDATE Stock SET Qty=_newQty, QtyX=_newQtyX, AvgCost=_newAvgCost/*--, IsEmpty=_IsEmpty*/
	WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID;

	IF (__StockAType='SADJ' AND _Count=1)THEN
		UPDATE Stock SET lopdate=_OpDate, Lopempid=_OpEmpID,Lopusrid=_OpUsrID
		WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID; 
	END IF;

	/** ===================== RUNNING COST LOGGING ============================================= */
	SELECT INTO _spret * FROM spCostLog(__StockADate, __ItemID, __WhID, __PID, NULL, 
			_newQty, _newAvgCost, _newSubTotal, 
			_Qty_Unit1, _Cost1, __SubTotal, 
			_curQty, _curAvgCost, _curSubTotal,
			__StockAType, __RefNo);
	IF (_spret.ErrNo!=0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;	

	SELECT INTO _PYear CAST(date_part('year', __StockADate) AS VARCHAR);
	SELECT INTO _Month CAST(date_part('month',  __StockADate ) AS VARCHAR);

	/*** Check Current Year StockBal is EXIST!
	**** NO ? -> create one *****/

	IF NOT EXISTS (SELECT 1 FROM StockBal WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear) THEN 
		SELECT INTO _BegBal (StockBal00+StockBal01+StockBal02+StockBal03+StockBal04+StockBal05+StockBal06+StockBal07+StockBal08+StockBal09+StockBal10+StockBal11+StockBal12)
		FROM StockBal WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID 
		AND PYear=(SELECT MAX(PYear) FROM StockBal WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear<_PYear);

		/*** Jika belum ada Saldo Tahun sebelumnya, SaldoAwal:=0 ***/
		IF NOT FOUND THEN
 			_BegBal:=0;
		END IF;

		INSERT INTO StockBal(ItemID, WhID, PID, PYear, StockBal00) 
		VALUES (__ItemID, __WhID, _PID, _PYear, _BegBal);

		_retval.ErrMsg:='Error INSERT StockBal for Year:'||CAST(_PYear AS VARCHAR(4))||', ItemID:'||__ItemID||', Gd:'||__WhID||', PID:'||_PID||' (spStockBal_Update)';
			
		IF NOT FOUND THEN
			_retval.ErrMsg:='Error INSERT StockBal for Year:'||CAST(_PYear AS VARCHAR(4))||', ItemID:'||__ItemID||', Gd:'||__WhID||', PID:'||_PID||' (spStockBal_Update)';
		END IF;
	END IF;

	IF (_Month=1) THEN
		UPDATE StockBal SET StockBal01=StockBal01+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=2) THEN
		UPDATE StockBal SET StockBal02=StockBal02+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=3) THEN
		UPDATE StockBal SET StockBal03=StockBal03+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=4) THEN
		UPDATE StockBal SET StockBal04=StockBal04+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=5) THEN
		UPDATE StockBal SET StockBal05=StockBal05+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=6) THEN
		UPDATE StockBal SET StockBal06=StockBal06+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=7) THEN
		UPDATE StockBal SET StockBal07=StockBal07+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=8) THEN
		UPDATE StockBal SET StockBal08=StockBal08+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=9) THEN
		UPDATE StockBal SET StockBal09=StockBal09+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=10) THEN
		UPDATE StockBal SET StockBal10=StockBal10+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=11) THEN
		UPDATE StockBal SET StockBal11=StockBal11+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	ELSIF (_Month=12) THEN
		UPDATE StockBal SET StockBal12=StockBal12+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_PYear;

	END IF;

	IF NOT FOUND THEN
		_retval.ErrNo:=50016;
		_retval.ErrMsg:='Error UPDATE StockBal for Year:'||CAST(_PYear AS VARCHAR(4))||' Month:'||CAST(_Month AS VARCHAR(2))||' (spStockBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	OPEN cFutureStockBal FOR
		SELECT PYear FROM StockBal WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear>_PYear;

	LOOP
		FETCH cFutureStockBal INTO _fPYear;
		IF NOT FOUND THEN
			EXIT;
		END IF;

		UPDATE StockBal SET StockBal00=StockBal00+_Qty_Unit1 WHERE ItemID=__ItemID AND WhID=__WhID AND PID=_PID AND PYear=_fPYear;
		IF NOT FOUND THEN
			_retval.ErrNo:='50017';
			_retval.ErrMsg:='Error UPDATE future StockBal:StockBal00 for Year:'||CAST(_fPYear AS VARCHAR(4))||' (spStockBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;		
		END LOOP;
	CLOSE cFutureStockBal;
	
	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/** test data ****

select * from spStockBal_Update('001002', '01', '001', 1, 'PCS', 1, 0, 4444, '2008-06-16', 'SADJ');

*/





CREATE OR REPLACE FUNCTION spStockA_New(
	__StockANo	VARCHAR(15),
	_BPID		VARCHAR(15),
	_StockAType	VARCHAR(5),
	_StockADate	DATE,
	_InOut		VARCHAR(1),
	_RefType	VARCHAR(5),
	_RefNo		VARCHAR(15),
	_StockANote	VARCHAR(255),
	_IsAutoGen	BOOLEAN,
	_CrtBy		VARCHAR(10),
	_CrtDate	TIMESTAMP,
	_UpdBy		VARCHAR(10),
	_UpdDate	TIMESTAMP,

	_PerID		VARCHAR(5),
	_BranchID	VARCHAR(10),
	_DeptID		VARCHAR(5),
	_PrjID		VARCHAR(5)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_StockANo	VARCHAR(15);
BEGIN

	_StockANo:=__StockANo;
	IF (_StockANo IS NULL) THEN
		_StockANo:=fTransNoGet('SADJ');
		IF (_StockANo IS NULL) THEN
			_retval.ErrNo:=2;
			_retval.ErrMsg:='ERROR getting StockANo (spStockA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	INSERT INTO StockA
		(StockANo, BPID, StockAType, StockADate, InOut, 
		RefType, RefNo, StockANote, StockAStatus, IsAutoGen, 
		CrtBy, CrtDate, UpdBy, UpdDate,
		PerID, BranchID, DeptID, PrjID
	)
	VALUES 
		(_StockANo, _BPID, _StockAType, _StockADate, _InOut,
		_RefType, _RefNo, _StockANote, NULL, _IsAutoGen,
		_CrtBy, _CrtDate, _UpdBy, _UpdDate,
		_PerID, _BranchID, _DeptID, _PrjID
	);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION spStockA_New(
	__StockANo	VARCHAR(15),
	_BPID		VARCHAR(15),
	_StockAType	VARCHAR(5),
	_StockADate	DATE,
	_InOut		VARCHAR(1),
	_RefType	VARCHAR(5),
	_RefNo		VARCHAR(15),
	_StockANote	VARCHAR(255),
	_IsAutoGen	BOOLEAN,
	_CrtBy		VARCHAR(10),
	_CrtDate	TIMESTAMP,
	_UpdBy		VARCHAR(10),
	_UpdDate	TIMESTAMP,

	_PerID		VARCHAR(5),
	_BranchID	VARCHAR(10),
	_DeptID		VARCHAR(5),
	_PrjID		VARCHAR(5),
	_EmpID		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_StockANo	VARCHAR(15);
BEGIN

	_StockANo:=__StockANo;
	IF (_StockANo IS NULL) THEN
		_StockANo:=fTransNoGet('SADJ');
		IF (_StockANo IS NULL) THEN
			_retval.ErrNo:=2;
			_retval.ErrMsg:='ERROR getting StockANo (spStockA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	INSERT INTO StockA
		(StockANo, BPID, StockAType, StockADate, InOut, 
		RefType, RefNo, StockANote, StockAStatus, IsAutoGen, 
		CrtBy, CrtDate, UpdBy, UpdDate,
		PerID, BranchID, DeptID, PrjID, EmpID
	)
	VALUES 
		(_StockANo, _BPID, _StockAType, _StockADate, _InOut,
		_RefType, _RefNo, _StockANote, NULL, _IsAutoGen,
		_CrtBy, _CrtDate, _UpdBy, _UpdDate,
		_PerID, _BranchID, _DeptID, _PrjID, _EmpID
	);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockA_Void_byRef(
	_RefType 	VARCHAR(5),
	_RefNo 		VARCHAR(15))
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
	_StockANo 	VARCHAR;
	cStockA 	REFCURSOR;
BEGIN
	
	OPEN cStockA FOR
		SELECT StockANo FROM StockA WHERE RefType=_RefType AND RefNo=_RefNo;
	LOOP
		FETCH cStockA INTO _StockANo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spStockA_Void(_StockANo);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;		
	END LOOP;
	CLOSE cStockA;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;





/**
 *	update status preq menjadi N
 */
DROP FUNCTION spPReq_New (VARCHAR);
CREATE OR REPLACE FUNCTION spPReq_New (
	_P_PReqNo		preq.preqno%TYPE
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_isDraft	BOOLEAN;
BEGIN
	_isDraft:=FALSE;

	SELECT INTO _IsDraft isdraft 
	FROM preq
	WHERE preqno=_P_PReqNo;
	IF NOT FOUND THEN
		_retval.ErrNo=1000002;
		_retval.ErrMsg:=_retval.ErrNo||E'|Permintaan Pembelian:'||COALESCE(_P_PReqNo, '<kosong>')||E' tidak ditemukan ! (spPReq_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		RETURN NEXT _retval;
		RETURN;
	END IF;

	UPDATE preq SET preqstatus='N' WHERE preqno=_P_PReqNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




/**
 *	hapus preq, namun cek jika sudah diimport ke PO
 */
DROP FUNCTION spPReq_Void(VARCHAR);
CREATE OR REPLACE FUNCTION spPReq_Void (
	_P_PReqNo		preq.preqno%TYPE
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_Active		BOOLEAN;
	_isDraft	BOOLEAN;

	_NumberOfPO	INT;
	_PONos		VARCHAR;
BEGIN
	_isDraft:=FALSE;

	SELECT INTO _Active, _IsDraft 
		active, isdraft 
	FROM preq
	WHERE preqno=_P_PReqNo;
	IF NOT FOUND THEN
		_retval.ErrNo=1000002;
		_retval.ErrMsg:=_retval.ErrNo||E'|Permintaan Pembelian:'||COALESCE(_P_PReqNo, '<kosong>')||E' tidak ditemukan ! (spPReq_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM preqd WHERE preqno=_P_PReqNo;
		DELETE FROM preq WHERE preqno=_P_PReqNo;

		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _NumberOfPO, _PONos 
		COUNT(*), SUM(DISTINCT pono)
	FROM pod
	WHERE preqno=_P_PReqNo;
	IF (FOUND AND (_NumberOfPO>0 OR _Active=FALSE)) THEN
		_retval.ErrNo=1000003;
		_retval.ErrMsg=_retval.ErrNo||E'|Permintaan Pembelian ini sudah dipakai dalam transaksi Order Pembelian:\n'||COALESCE(_PONos, '<kosong>')||E'\nVoid terlebih dahulu Order Pembelian tersebut ! (spPReq_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	DELETE FROM preqd WHERE preqno=_P_PReqNo;
	DELETE FROM preq WHERE preqno=_P_PReqNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPReq_Item_Order(
	_P_PReqNo	preq.preqno%TYPE,
	_P_PReqDNo	preqd.preqdno%TYPE,
	_P_ItemId	item.itemid%TYPE,
	_P_PId		stock.pid%TYPE,
	_P_Qty1		NUMERIC(15,4)
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE
	_spret 			SPStatus%ROWTYPE;

	_Active			BOOLEAN;
	_Qty			NUMERIC(15,4);
	_Unit			item.unit1%TYPE;
	_PReqDPID		stock.pid%TYPE;
	_qty1_requested		NUMERIC(15,4);	-- konversi dari Qty-Unit
	_qty1_ordered		NUMERIC(15,4);	-- from preqd
	_qty1_reject		NUMERIC(15,4);	-- from preqd
	_qty1_tolerance		NUMERIC(15,4);
	_qty1_outstanding	NUMERIC(15,4);

	_qty1_to_order_now	NUMERIC(15,4);
	_isstock		BOOLEAN;
BEGIN
	SELECT INTO _Active active 
	FROM preq WHERE preqno=_P_PReqNo;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Permintaan Pembelian:% tidak ditemukan ! (spPReq_Item_Order)', _P_PReqNo;
	END IF;

	IF (_Active=FALSE) THEN
		RAISE EXCEPTION 'Status Permintaan-Pembelian #% tidak-Aktif (spPReq_Item_Order)', _P_PReqNo;
	END IF;
	
	SELECT INTO _Qty, _Unit, _PReqDPID, _Qty1_Ordered, _Qty1_Reject
		qty, unit, pid, qtyordered, qtyreject
	FROM preqd
	WHERE preqno=_P_PReqNo AND preqdno=_P_PReqDNo
	AND itemid=_P_ItemId;
	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Permintaan-Pembelian #%, detail ke-% tidak ditemukan! (spPReq_Item_Order)', _P_PReqNo, _P_PReqDNo;
	END IF;


	_qty1_requested:=fQtyZToQty1(_P_ItemId, _Qty, _Unit);
	_qty1_outstanding:=_qty1_requested-_Qty1_Ordered-_Qty1_Reject;
	_qty1_tolerance:=0;

	IF (_P_Qty1>(_qty1_outstanding+_qty1_tolerance)) THEN
		RAISE EXCEPTION 'Item:%, PID:%. Qty yang diterima melebihi Qty Outstanding+toleransi! (spPRcv_Item_Order)', _P_ItemId, _P_PId;
		_qty1_to_order_now:=0;
		RETURN 0;	
	ELSE
		_qty1_to_order_now:=_P_Qty1;
	END IF;

	-- update preqd
	UPDATE preqd SET qtyordered=qtyordered+_qty1_to_order_now
	WHERE preqno=_P_PReqNo AND preqdno=_P_PReqDNo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Error Update PReq:QtyOrdered #%/% (spPReq_Item_Order)', _P_PReqNo, _P_PReqDNo;
	END IF;

	RETURN _qty1_to_order_now;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPReq_Item_Cancel(
	__PReqNo 	VARCHAR, 
	__PReqDNo 	INT,
	__ItemID 	VARCHAR, 
	__PID		VARCHAR,
	__Qty1 		NUMERIC(15,4)
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE
	_retval 		NUMERIC(15,4);

	_Active			BOOLEAN;
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_PReqDPID		VARCHAR;
	_qty1_requested		NUMERIC(15,4);	-- konversi dari Qty-Unit
	_qty1_ordered		NUMERIC(15,4); -- from preqd
	_qty1_reject		NUMERIC(15,4); -- from preqd
	_qty1_tolerance		NUMERIC(15,4);
	_qty1_outstanding	NUMERIC(15,4);

	_qty1_to_cancel_now	NUMERIC(15,4);
BEGIN
	_qty1_to_cancel_now:=0;

	SELECT INTO _Active active 
	FROM preq WHERE preqno=__PReqNo;
	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Permintaan-Pembelian #%  tidak ditemukan! (spPReq_Item_Cancel)', __PReqNo;
	END IF;

	SELECT INTO  
		_Qty, _Unit, _PReqDPID, _Qty1_Ordered, _Qty1_Reject
		qty, unit, pid, qtyordered, qtyreject 
	FROM preqd 
	WHERE preqno=__PReqNo AND preqdno=__PReqDNo;
	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Order-Pembelian #%, detail ke-% tidak ditemukan! (spPReq_Item_Cancel)', __PReqNo, __PReqDNo;
	END IF;

	IF (__Qty1>_qty1_ordered) THEN
		_qty1_to_cancel_now:=_qty1_ordered;
	ELSE
		_qty1_to_cancel_now:=__Qty1;
	END IF;

	UPDATE preqd SET qtyordered=qtyordered-_qty1_to_cancel_now 
	WHERE preqno=__PReqNo AND preqdno=__PReqDNo;

	RETURN _qty1_to_cancel_now;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPReq_ChNUp_Status(__PReqNo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 		SPStatus%ROWTYPE;
	_Active			BOOLEAN;
	_PReqStatus		VARCHAR;
	_PReqDNo		INT;

	_Qty1_Requested		NUMERIC(15,4);		
	_Qty1_Ordered		NUMERIC(15,4);
	_Qty1_Reject		NUMERIC(15,4);

	_Qty1_Outstanding1	NUMERIC(15,4);
	_Qty1_Outstanding2	NUMERIC(15,4);

	_IsFull			BOOLEAN;
	_IsPartial		BOOLEAN;
	_IsClear		BOOLEAN;

	_Qty1_Req_Sum		NUMERIC(15,4);		/** Ordered Qty **/
	_Qty1_Ord_Sum		NUMERIC(15,4);		/** Delivered Qty **/
	_Qty1_Rej_Sum		NUMERIC(15,4);		/** Back-Order Qty **/	

	cPReqD	REFCURSOR;
BEGIN	
	-- utk melayani SP lain yg memang gak nge-check valuenya, kita ngalahin ngecheck disini
	IF (__PReqNo IS NULL OR __PReqNo='') THEN	
		_retval:=(0,'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _Active active FROM preq WHERE preqno=__PReqNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=12010001;
		_retval.ErrMsg:='Error Permintaan Pembelian tidak ditemukan (spPReq_ChNUp_Status)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	_IsFull:=true;			/** default PReqStatus=FULL ***/
	_IsPartial:=false;		/** default PReqStatus=FULL ***/
	_IsClear:=true;			/** default CLOSED **/
	_Active:=true;			/** default OPEN **/
	_Qty1_Req_Sum:=0;
	_Qty1_Ord_Sum:=0;
	_Qty1_Rej_Sum:=0;
	
	OPEN cPReqD FOR
		SELECT preqdno, fQtyZToQty1(itemid, qty, unit), qtyordered, qtyreject
		FROM preqd WHERE preqno=__PReqNo;
	LOOP
		FETCH cPReqD INTO _PReqDNo, _Qty1_Requested, _Qty1_Ordered, _Qty1_Reject;
		EXIT WHEN NOT FOUND;

		IF(_Qty1_Requested<0) THEN
			_Qty1_Requested:=_Qty1_Requested*(-1);
		END IF;
		_Qty1_Req_Sum:=_Qty1_Req_Sum+_Qty1_Requested;

		IF(_Qty1_Ordered<0) THEN
			_Qty1_Ordered:=_Qty1_Ordered*(-1);
		END IF;
		_Qty1_Ord_Sum:=_Qty1_Ord_Sum+_Qty1_Ordered;

		IF (_Qty1_Reject<0) THEN
			_Qty1_Reject:=_Qty1_Reject*(-1);
		END IF;
		_Qty1_Rej_Sum:=_Qty1_Rej_Sum+_Qty1_Reject;	
	END LOOP;
	CLOSE cPReqD;		

	/*_Qty1_Outstanding1 := _Qty1_Requested - _Qty1_Ordered;
	_Qty1_Outstanding2 := _Qty1_Requested - _Qty1_Ordered - _Qty1_Reject;*/

	_Qty1_Outstanding1 := _Qty1_Req_Sum - _Qty1_Ord_Sum;
	_Qty1_Outstanding2 := _Qty1_Req_Sum - _Qty1_Ord_Sum - _Qty1_Rej_Sum;

	IF (_Qty1_Outstanding1>0) THEN 			/** not full **/
		_IsFull=false;			
	END IF;
		
	IF (_Qty1_Ordered>0) THEN 			/** sudah ada sebagian yg terkirim **/
		_IsPartial=true;
	END IF;
		
	IF (_Qty1_Outstanding2>0) THEN 			/** ada sisa & belum di BO**/
		_IsClear=false;			/** jika ada sisa item yg BELUM CLEAR (belum dikirim & belum di-B.O.) **/
	END IF;
	
	-- set flags ---
/*	IF (_IsFull=true) THEN _PReqStatus='F';									
	ELSIF (_IsFull=false) AND (_IsPartial=true) THEN _PReqStatus='P';
	ELSIF (_IsFull=false) AND (_IsPartial=false) THEN _PReqStatus='N';
	END IF;*/

	IF(_Qty1_Outstanding1=0) THEN
		_PReqStatus='F';
	ELSIF(_Qty1_Outstanding1<>0 AND _Qty1_Req_Sum<>_Qty1_Outstanding1) THEN
		_PReqStatus='P';
	ELSE
		_PReqStatus='N';
	END IF;


	--- Jika sudah Clear semua, tutup PO --
/*	IF (_IsClear=true) THEN 
		_Active=false;
	END IF;*/
--RAISE EXCEPTION '%,%',_Qty1_Outstanding1,_PReqStatus;
	IF(_Qty1_Outstanding2>0 AND _PReqStatus='F') THEN
		_Active=FALSE;
	ELSIF(_Qty1_Outstanding2>0) THEN
		_Active=TRUE;
	ELSE
		_Active=FALSE;
	END IF;

	UPDATE preq SET preqstatus=_PReqStatus, active=_Active 
	WHERE preqno=__PReqNo;

	_retval=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPReq_Close (
	__PReqNo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_PReqDNo		SMALLINT;

	_ItemID			VARCHAR(20);
	_PID			VARCHAR;

	_Qty1_Requested		NUMERIC(15,4);
	_Qty1_Ordered		NUMERIC(15,4);
	_Qty1_Reject		NUMERIC(15,4);

	_Qty1_Reject_new 	NUMERIC(15,4);

	cPReqD			REFCURSOR;
BEGIN
	OPEN cPReqD FOR
		SELECT preqdno, itemid, fQtyZToQty1(itemid, qty, unit), qtyordered, qtyreject 
		FROM preqd 
		WHERE (preqno=__PReqNo);

	LOOP
		FETCH cPReqD INTO _PReqDNo, _ItemID, _Qty1_Requested, _Qty1_Ordered, _Qty1_Reject;
		EXIT WHEN NOT FOUND;

		_Qty1_Reject_new:=_Qty1_Requested-_Qty1_Ordered-_Qty1_Reject;
		IF (_Qty1_Reject_new>0) THEN	
			UPDATE preqd SET qtyreject=qtyreject+_Qty1_Reject_new
			WHERE preqno=__PReqNo AND preqdno=_PReqDNo;

			IF NOT FOUND THEN
				RAISE EXCEPTION 'Error Update PReq:QtyReject #%/%, Item:% (spPReq_Close)', __PReqNo, _PReqDNo, _ItemID;
			END IF;
		END IF;

	END LOOP;
	CLOSE cPReqD;

	SELECT INTO _spret * FROM spPReq_ChNUp_Status(__PReqNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPReq_ReOpen(
	__PReqNo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_PReqDNo	SMALLINT;
	_PReqStatus	VARCHAR;
	_ItemID		VARCHAR(20);

	_Qty1_Requested	NUMERIC(15,4);
	_Qty1_Ordered	NUMERIC(15,4);
	_Qty1_Reject	NUMERIC(15,4);
	cPReqD 		REFCURSOR;
BEGIN

	SELECT INTO _PReqStatus preqstatus FROM preq WHERE preqno=__PReqNo;
	IF (_PReqStatus='F') THEN
		_retval.ErrNo:=12010002;
		_retval.ErrMsg:='Sudah Terkirim Penuh! Permintaan Pembelian tidak dapat dibuka kembali (spPReq_ReOpen)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cPReqD FOR
		SELECT preqdno, itemid, fQtyZToQty1(itemid, qty, unit), qtyordered, qtyreject
		FROM preqd 
		WHERE preqno=__PReqNo AND (qtyreject>0);
	LOOP
		FETCH cPReqD INTO _PReqDNo, _ItemID, _Qty1_Requested, _Qty1_Ordered, _Qty1_Reject;
		EXIT WHEN NOT FOUND;

		UPDATE preqd SET QtyReject=0
		WHERE preqno=__PReqNo AND preqdno=_PReqDNo;

	END LOOP;
	CLOSE cPReqD;

	SELECT INTO _spret * FROM spPReq_ChNUp_Status (__PReqNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spretPO_ChNUp_Status.ErrNo;
		_retval.ErrMsg:=_spretPO_ChNUp_Status.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




/**
 *	update status po
 *	generate stock dan update qtypo
 *	update preq
 */
DROP FUNCTION spPO_New(VARCHAR);
CREATE OR REPLACE FUNCTION spPO_New (
	_P_PONo		po.pono%TYPE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	cData		REFCURSOR;

	_PODNo		pod.podno%TYPE;
	_ItemId		item.itemid%TYPE;
	_WhId		wh.whid%TYPE;
	_PId		stock.pid%TYPE;
	_Qty		NUMERIC(15,4);
	_Unit		item.unit1%TYPE;
	_QtyX		NUMERIC(15,4);

	_PReqNo		preq.preqno%TYPE;
	_PReqDNo	preqd.preqdno%TYPE;

	_IsStock	BOOLEAN;
	_Qty1		NUMERIC(15,4);

	_isDraft	BOOLEAN;
	_Active		BOOLEAN;
	_LastPReqNo	preq.preqno%TYPE;
BEGIN
	_Active:=TRUE;
	_isDraft:=FALSE;

	SELECT INTO _isDraft isdraft 
	FROM po WHERE pono=_P_PONo;
	IF NOT FOUND THEN
		_retval.ErrNo:=1000004;
		_retval.ErrMsg:=_retval.ErrNo||E'|Order Pembelian:'||COALESCE(_PReqNo, '<kosong>')||E' tidak ditemukan ! (spPO_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		RETURN NEXT _retval;
		RETURN;
	END IF;

/** buat kartu stock dan update qty po */
	OPEN cData FOR
		SELECT d.podno, d.itemid, d.whid, d.pid, d.qty, d.qtyx, d.unit, i.isstock
		FROM pod d
		JOIN item i ON d.itemid=i.itemid
		WHERE pono=_P_PONo;
	LOOP
		FETCH cData INTO _PODNo, _ItemId, _WhId, _PId, _Qty, _QtyX, _Unit, _isStock;
		EXIT WHEN NOT FOUND;

		IF (_PID IS NULL) THEN 
			_PID:=''; 
		END IF;

		IF (_isStock) THEN
			SELECT INTO _spret * FROM spCreateStockCard(_ItemId, _WhId, _PId);
			IF (_spret.ErrNo!=0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_Qty1:=fQtyZToQty1(_ItemId, _Qty, _Unit);
			UPDATE stock SET qtypo=qtypo+_Qty1, qtyxpo=qtyxpo+_QtyX 
			WHERE itemid=_ItemId AND whid=_WhId AND pid=_PId;
			IF NOT FOUND THEN
				_retval.ErrNo=1000006;
				_retval.ErrMsg=_retval.ErrNo||E'|Gagal update qty order pembelian pada stock\nitem:'||COALESCE(_ItemId, '<kosong>')||' di gudang:'||COALESCE(_WhId, '<kosong>')||' dengan pid:'||COALESCE(_PId, '')||'! (spPO_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cData; 

/** update qtyordered permintaan pembelian */
	OPEN cData FOR
		SELECT o.preqno, o.preqdno, o.itemid, o.pid, fQtyZToQty1(o.itemid, o.qty, o.unit), p.active
		FROM pod o
		LEFT JOIN preq p ON o.preqno=p.preqno
		WHERE o.pono=_P_PONo AND (o.preqno IS NOT NULL AND o.preqno<>'');
	LOOP
		FETCH cData INTO _PReqNo, _PReqDNo, _ItemId, _PId, _Qty1, _Active;
		EXIT WHEN NOT FOUND;

		IF (_Active=FALSE) THEN
			_retval.ErrNo:=1000007;
			_retval.ErrMsg:=_retval.ErrNo||E'|Penerimaan Pembelian:'||COALESCE(_PReqNo, '<kosong>')||' sudah tidak aktif ! (spPO_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spUpdatePReq(_PReqNo, _PReqDNo, _ItemID, _PID, _Qty1);
		IF (_spret.ErrNo!=0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_LastPReqNo IS NULL) THEN
			_LastPReqNo:=_PReqNo;
		ELSIF (_LastPReqNo!=_PReqNo) THEN
			SELECT INTO _spret * FROM spUpdatePReq_Status(_LastPReqNo);
			IF (_spret.ErrNo!=0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			_LastPReqNo:=_PReqNo;
		END IF;
	END LOOP;
	CLOSE cData;

	SELECT INTO _spret * FROM spUpdatePReq_Status(_LastPReqNo);
	IF (_spret.ErrNo!=0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	UPDATE po SET active=TRUE, postatus='N' WHERE pono=_P_PONo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




DROP FUNCTION spPO_Void(VARCHAR);
CREATE OR REPLACE FUNCTION spPO_Void (
	_P_PONo		po.pono%TYPE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	cData			REFCURSOR;

	_Active			BOOLEAN;
	_isStock		BOOLEAN;

	_PODNo			pod.podno%TYPE;
	_ItemId			item.itemid%TYPE;
	_WhId			wh.whid%TYPE;
	_PId			stock.pid%TYPE;
	_QtyX			NUMERIC(15,4);
	_IsPurc			BOOLEAN;

	_Qty1			NUMERIC(15,4);
	_PReqNo			preq.preqno%TYPE;
	_PReqDNo		preqd.preqdno%TYPE;
	_PayNo			pay.payno%TYPE;

	_isDraft		BOOLEAN;

	_NumOfPRcv		SMALLINT;
	_NumOfPurc		SMALLINT;
	_PRcvNos		VARCHAR;
	_PurcNos		VARCHAR;

	_LastPReqNo		preq.preqno%TYPE;
BEGIN
	_isDraft:=FALSE;

	SELECT INTO _Active, _isDraft
		Active, isDraft
	FROM PO WHERE PONo=_P_PONo;
	IF NOT FOUND THEN
		_retval.ErrNo:=1000004;
		_retval.ErrMsg:=_retval.ErrNo||E'|Order Pembelian:'||COALESCE(_PReqNo, '<kosong>')||E' tidak ditemukan ! (spPO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spCheckDP('PO', _P_PONo);
	IF (_spret.ErrNo!=0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM preqpo WHERE pono=_P_PONo;
		DELETE FROM pod WHERE pono=_P_PONo;
		DELETE FROM po WHERE pono=_P_PONo;
		
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _NumOfPRcv, _PRcvNos 
		COUNT(*), SUM(DISTINCT prcvno) 
	FROM prcvd WHERE pono=_P_PONo;
	IF (_NumOfPRcv>0) THEN
		_retval.ErrNo=1000003;
		_retval.ErrMsg=_retval.ErrNo||E'|Order Pembelian ini sudah dipakai dalam transaksi Penerimaan Pembelian:\n'||COALESCE(_PRcvNos, '<KOSONG>')||E'\nVoid terlebih dahulu Penerimaan Pembelian tersebut ! (spPO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _NumOfPurc, _PurcNos 
		COUNT(*), SUM(DISTINCT purcno) 
	FROM purcd WHERE pono=_P_PONo;
	IF (_NumOfPurc>0) THEN
		_retval.ErrNo=1000003;
		_retval.ErrMsg=_retval.ErrNo||E'|Order Pembelian ini sudah dipakai dalam transaksi Invoice Pembelian:\n'||COALESCE(_PurcNos, '<KOSONG>')||E'\nVoid terlebih dahulu Invoice Pembelian tersebut ! (spPO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cData FOR
		SELECT o.preqno, o.preqdno, o.whid, o.itemid, o.pid, fQtyZToQty1(o.itemid, o.qty, o.unit), p.active, i.isstock, o.qtyx
		FROM pod o
		JOIN item i ON o.itemid=i.itemid
		LEFT JOIN preq p ON o.preqno=p.preqno
		WHERE o.pono=_P_PONo AND (o.preqno IS NOT NULL AND o.preqno<>'');
	LOOP
		FETCH cData INTO _PReqNo, _PReqDNo, _WhId, _ItemId, _PId, _Qty1, _Active, _isStock, _QtyX;
		EXIT WHEN NOT FOUND;

		IF (_isStock=TRUE) THEN
			UPDATE stock SET qtypo=qtypo+(-1*_Qty1), qtyxpo=qtyxpo+(-1*_QtyX) WHERE itemid=_ItemId AND whid=_WhId AND pid=_PId;
		END IF;

		SELECT INTO _spret * FROM spUpdatePReq(_PReqNo, _PReqDNo, _ItemId, _PId, _Qty1*-1);
		IF (_spret.ErrNo!=0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_LastPReqNo IS NULL) THEN
			_LastPReqNo:=_PReqNo;
		ELSIF (_LastPReqNo!=_PReqNo) THEN
			SELECT INTO _spret * FROM spUpdatePReq_Status(_LastPReqNo);
			IF (_spret.ErrNo!=0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			_LastPReqNo:=_PReqNo;
		END IF;
	END LOOP;
	CLOSE cData;

	SELECT INTO _spret * FROM spUpdatePReq_Status(_LastPReqNo);
		IF (_spret.ErrNo!=0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	DELETE FROM preqpo WHERE pono=_P_PONo;
	DELETE FROM pod WHERE pono=_P_PONo;
	DELETE FROM po WHERE pono=_P_PONo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPurc_New(_PurcNo VARCHAR(15))
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_VendorID		VARCHAR(15);
	_VendorID2		VARCHAR(15);
	_PurcDate		DATE;
	_PurcType		VARCHAR(1);
	_PONo			VARCHAR(15);
	_TermType		VARCHAR(1);
	_Term			SMALLINT;
	_Total			DECIMAL(19,4);
	_PaidAmt		DECIMAL(19,4);
	_CashID			VARCHAR(15);
	_CashAmt		DECIMAL(19,4);
	_CashANote		VARCHAR(50);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;

	_PRcvNo			VARCHAR(15);
	_PayNo			VARCHAR(15);
	_EmpID			VARCHAR(10);
	
	_PurcDNo		SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_Qty			DECIMAL(15,4);
	_Unit			VARCHAR(5);
	_QtyX			DECIMAL(15,4);
	_BasePrice		DECIMAL(19,4);
	_ListPrice		DECIMAL(19,4);
	_DiscExp		VARCHAR(15);
	_DiscAmt		DECIMAL(19,4);
	_Disc2Amt		DECIMAL(19,4);
	_TaxID			VARCHAR(5);
	_TaxAmt			DECIMAL(19,4);
	_MTaxAmt		DECIMAL(19,4);
	_MDiscAmt		DECIMAL(19,4);
	_IsPurc			BOOLEAN;
	
	_AccAP			VARCHAR;
	_StockANo		VARCHAR(15);
	_StockADNo		SMALLINT;
	_StockANote		VARCHAR(255);
	_StockADCost		DECIMAL(19,4);	
	_StockADSubTotal	DECIMAL(19,4);	
	
	_BPAAmt 		DECIMAL(19,4);
	_BPANote		VARCHAR(50);
	
	_Qty1			DECIMAL(15,4);
	_Qty1Rcv		DECIMAL(15,4);

	_tmpConv		DECIMAL(15,4);
	_tmp			VARCHAR(5);

	/** POD	*/
	_Active 		BOOLEAN;
	_PODNo			INT;
	_PRcvDNo		INT;
	_QtyPOD			DECIMAL(15,4);
	_UnitPOD2		VARCHAR(5);
	_WhIDPOD		VARCHAR(3);
	_PID2			VARCHAR(25);
	_Qty1_O			DECIMAL(15,4);		/** Ordered Qty **/
	_Qty1_D			DECIMAL(15,4);		/** Delivered Qty **/
	_Qty1_B			DECIMAL(15,4);		/** Back-Order Qty **/
	_Qty1_R			DECIMAL(15,4);		/** Remaining Open Qty **/
	_count			INT;
	_JnlDNo			INT;
	_CrcID			VARCHAR;		

	_AllowCredit 		DECIMAL(19,4);
	_CreditLimit 		DECIMAL(19,4);
	_CountCredit 		DECIMAL(19,4);
	_CreditAmount 		DECIMAL(19,4);
	_regval			VARCHAR(30);
	_ExcRate 		DECIMAL(19,4);

	_AccNo			VARCHAR(10);
	_AccDisc		VARCHAR(10);
	_AccInv			VARCHAR(10);
	_Disc			DECIMAL(19,4);
	_Freight		DECIMAL(19,4);
	_Others			DECIMAL(19,4);
	_AccFreight		VARCHAR(10);
	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;


	/** CURSORS */
	cPurcD			REFCURSOR;
	cPO			REFCURSOR;
	cPurcD2			REFCURSOR;
	cPRcv			REFCURSOR;

	_countPRcv		SMALLINT;

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
   	_DeptID     		VARCHAR(5);
   	_PrjID      		VARCHAR(5);
	_Round			INT;
	_countBPTYPE		INT;

	_isDraft		BOOLEAN;
	_isTaxed		BOOLEAN;
	_isTaxInc		BOOLEAN;

	_JnlType		VARCHAR;

	_JnlDAmt2		DECIMAL(19,4);
	_UBPurcAmt		DECIMAL(19,4);
	_UBPrcv 		DECIMAL(19,4);
	_dkUBPrcv 		VARCHAR;
	_dkInv 			VARCHAR;
	_JnlNo2 		VARCHAR;
	_JnlNote2 		VARCHAR;

	_LockDate		DATE;
	_Subtotal		DECIMAL(19,4);
	_SubTotalExt		DECIMAL(40,20);

    	_TotalBasePrice		DECIMAL(19,4);
	_ItemIDCheck		VARCHAR;
	cPurcD1			REFCURSOR;
	cTax			REFCURSOR;
	_PurcDTaxAmt		DECIMAL(19,4);
	_AccPurcTax		VARCHAR;
	_TotalTax		DECIMAL(19,4);
	_FisRate		DECIMAL(19,4);
	_TotalBasePriceForex	DECIMAL(19,4);
	_SumQtyPrcvD		DECIMAL(19,4);
	_SumQtyPurcD		DECIMAL(19,4);
	_isStock		BOOLEAN;
	_tmp_va1		VARCHAR(1);
	
	_BTotal			NUMERIC(19,4); -- base currency total
	_BSubtotal		NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt		NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt		NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt		NUMERIC(19,4);
	_TotalTaxAmt		NUMERIC(19,4);
	_BTotalTaxAmt		NUMERIC(19,4);
	_TotalDiscAmt		NUMERIC(19,4);
	_TotalDisc2Amt		NUMERIC(19,4);
	_DSubtotal		NUMERIC(19,4);
	_DBSubtotal		NUMERIC(19,4);
	
	_AccRateLR		VARCHAR; -- akun laba/rugi selisih kurs
	_RateLR			NUMERIC(19,4); -- nilai laba/rugi selisih kurs
	_CashCrcID		VARCHAR;
	_InOut			CHAR(1);
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_InOut:='O';

	SELECT INTO
		_isDraft, _PerID, _BranchID, _VendorID, _PurcDate, _TermType, _Term, _CashID, _BTotal, _Total, 
		_PaidAmt, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _EmpID, _CrcID, _Disc, _AccDisc, _MTaxAmt, _MBFTaxAmt,  
		_AccFreight, _Freight, _ExcRate, _MBDiscAmt, _MDiscAmt, _Others, _PurcType, _AccAP, _BSubtotal, _Subtotal, _FisRate,
		_isTaxed, _isTaxInc

		isDraft, PerID, BranchID, VendorID, PurcDate, TermType, duedays, CashID, basetotal, Total, 
		PaidAmt, CrtBy, CrtDate, UpdBy, UpdDate, EmpID, crcid, DiscAmt, accpDisc, taxamt, basftaxamt, 
		accfreight, freight, excrate, basediscamt, DiscAmt, others, purctype, accap, basesubtotal, SubTotal, FisRate,
		isTaxed, TaxInc
	FROM Purc
	WHERE PurcNo=_PurcNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Pembelian #'||_PurcNo||' tidak ditemukan (spPurc_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='PURC';
	IF(_LockDate IS NOT NULL AND _LockDate>_PurcDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Pembelian Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	--** Jika Draft, langsung RETURN OK 
	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');

	SELECT INTO _countBPTYPE COUNT(*) FROM bpbptype WHERE bpid=_VendorID AND bptype='VEND';

	--------- cek apakah punya bptype supplier (untuk jaga2 ketika import hutang lwt excel) ------------
	IF _countBPTYPE=0 THEN			
		_retval.ErrNo:=50003;
		_retval.ErrMsg='BPID: '||_VendorID||' tidak terdaftar sebagai supplier (spSale_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	_Qty1Rcv:=null;
	IF ((_TermType IS NULL) AND (_CashID IS NOT NULL)) THEN
		_retval.ErrNo:=50005;
		_retval.ErrMsg:='ERROR Termin or Kas/Bank belum diset (spPurc_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	ELSEIF (_TermType='C') THEN
		IF ((_CashID IS NULL) OR (_CashID='')) THEN
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Error, Kas/Bank untuk Penjualan Tunai belum diset (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		END IF;	

		SELECT INTO _CashCrcID crcid FROM cash WHERE cashid=_CashID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' belum diset (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
 					
		IF (_CrcID!=_CashCrcID) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' tidak sama dengan mata uang pembelian (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;		
		END IF;
	ELSEIF (_TermType='R') THEN
		SELECT INTO _AllowCredit, _CreditLimit, _CreditAmount aplimitfreq, aplimitamt, apbalance FROM BPAcc WHERE BPID=_VendorID AND crcid=_CrcID;

		SELECT INTO _CountCredit count(*) FROM purc WHERE PayStatus<>'F' AND vendorid=_VendorID AND CrcID=_CrcID;

		IF (_AllowCredit=0) THEN
			_retval.ErrNo:=50005;
			_retval.ErrMsg:='Supplier '||_VendorID||' tidak diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;	
						
		ELSEIF (_AllowCredit<>-1 AND _AllowCredit<_CountCredit) THEN
			_retval.ErrNo:=50006;
			_retval.ErrMsg:='Kepada Supplier '||_VendorID||' melebihi jumlah diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		END IF;

		-- mestinya Sudah di-Check di Applikasi agar bisa diconfirmasi ke user
		IF (_CreditLimit=0) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Supplier '||_VendorID||' tidak diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		ELSEIF (_CreditLimit<>-1 AND _CreditLimit<(_CreditAmount+_Total) ) THEN
			_retval.ErrNo:=50008;
			_retval.ErrMsg:='Pembelian melebihi Limit Hutang (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		END IF;

	END IF;

	-- Hitung penggunaan PRcv	
	SELECT INTO _countPRcv COUNT(*) FROM PurcD WHERE PurcNo=_PurcNo AND (PRcvNo IS NOT NULL AND PRcvNo<>'');

/************************************************************* 
	Pembelian langsung, tidak Lewat Purchase Receive ? 
***************************************************************/
	_count:=0;
	OPEN cPurcD FOR
		SELECT 	d.PurcDNo, d.ItemID, d.WhID, d.Qty, d.Unit, d.QtyX, d.ListPrice, d.BasePrice, 
			d.DiscExp, d.DiscAmt, d.Disc2Amt, d.TaxID, d.TaxAmt, d.pid, d.DeptID, d.PrjID, 
			d.PRcvNo, d.PRcvDNo, d.PONo, d.PODNo, i.isStock, d.totaldiscamt, d.totaldisc2amt, d.basetotaltaxamt, d.basesubtotal 
		FROM PurcD d JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.PurcNo=_PurcNo;		
	LOOP
		FETCH cPurcD INTO _PurcDNo, _ItemID, _WhID, _Qty, _Unit, _QtyX, _ListPrice, _BasePrice, 
			_DiscExp, _DiscAmt, _Disc2Amt, _TaxID, _TaxAmt, _PID, _DeptID, _PrjID, _PRcvNo, _PRcvDNo, _PONo, _PODNo, _isStock, _TotalDiscAmt, _TotalDisc2Amt, _TotalTaxAmt, _DBSubtotal;
		EXIT WHEN NOT FOUND;

		_tmpConv:=fItemUnit_Conv(_ItemID, _Unit);
		IF (_tmpConv IS NULL) THEN
			_retval.ErrNo:=50055;
			_retval.ErrMsg:='Error, Konversi ItemID:' || _ItemID || ', Unit:' || _Unit || '!  (fItemUnit_Conv)(spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- Update PurcD:Conv
		UPDATE PurcD SET Conv=_tmpConv WHERE PurcNo=_PurcNo AND PurcDNo=_PurcDNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50056;
			_retval.ErrMsg:='Error UPDATE PurcD:Conv (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- generate stock hanya yg bukan dari PRcv
		IF (_isStock=TRUE AND (_PRcvNo IS NULL OR _PRcvNo='')) THEN
			_count:=_count+1;
			--** row #1 ? Generate Header first 
			IF (_count=1) THEN
				_StockADNo:=0;
				_StockANote:='Pembelian #'|| _PurcNo;

				SELECT INTO _StockANo * FROM fTransNoGet('STOCKA');
				IF (_StockANo IS NULL) THEN
					_retval.ErrNo:=3;
					_retval.ErrMsg:='ERROR getting StockANo (spPurc_New)';			
					RAISE EXCEPTION '%', _retval.ErrMsg;				
				END IF;

				SELECT INTO _spret * FROM spStockA_New(
					_StockANo, _VendorID, 'PURC', _PurcDate, 'I', 'PURC', _PurcNo, 
					_StockANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID
				);

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;			
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				_count:=_count+1;
			END IF;

			--** Generate Stock Details 
			_StockADNo:=_StockADNo+1;
			--Hitung HPP, dibedakan pakai akunting atau tidak
			--kalau pakai akunting, HPP tidak termasuk pajak
			--kalau retail ambil praktisnya aja, 
		
			-- Retail tanpa akunting ?
			_StockADCost:=_DBSubtotal/_Qty; -- tanpa dikalikan excrate karena sudah dalam base crc
			_PID:=COALESCE(_PID, '');
		
			SELECT INTO _spret * FROM spStockAD_New(
				_PurcDate, 'PURC', _StockANo, _StockADNo, _WhID, 
				_ItemID, _PID, _Qty, _Unit, _QtyX, _StockADCost, 
				_DeptID, _PrjID, true, _PurcNo, _DBSubtotal
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cPurcD;
	
/***********************************************************************************************************

							POSTING Kas/Bank atau Account-Payable

************************************************************************************************************/	
	IF (_Total=0) THEN	-- Total=0 ? set lunas!
		_tmp_va1:='F';
		UPDATE Purc SET PayStatus=_tmp_va1 WHERE PurcNo=_PurcNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50058;
			_retval.ErrMsg:='Error UPDATE Purc:PayStatus [NettAmt=0] (spPurc_New)';
			RAISE EXCEPTION '%', _reval.ErrMsg;
		END IF;

	ELSEIF (_Total>0) THEN
		--************************************** TUNAI *************************************
		IF (_TermType='C') THEN	
			UPDATE Purc SET PaidAmt=0 WHERE PurcNo=_PurcNo;
			_PayNo:=fTransNoGet('PAY');
			IF (_PayNo IS NULL) THEN
				_retval.ErrNo:=50041;
				_retval.ErrMsg:='Error getting PayNo (spPurc_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			--generate data payment tunai
			INSERT INTO pay (payno, paytype, paydate, bpid, empid, totppaid, totpayDisc, totpayamt, reftype, refno, crtby, crtdate, updby, upddate, isautogen, branchid, crcid, excrate, fisrate)
			VALUES(_PayNo, 'PURC', _PurcDate, _VendorID, _EmpId, _Total, 0, _Total, 'PURC', _PurcNo, _CrtBy, _CrtDate, _UpdBy, _UpdDate, true, _BranchID, _CrcID, _Excrate, _FisRate);
			INSERT INTO payd (payno, paydno, paydmtd, cashid, paydamt, refno, crcid, excrate, fisrate) VALUES(_PayNo, 1, 'CASH', _CashId, _Total, _PurcNo, _CrcID, _ExcRate, _FisRate);
			INSERT INTO ppaid (payno, ppaidno, reftype, refno, ppaidDiscAmt, ppaidamt, excrate, fisrate, crcid) VALUES(_PayNo, 1, 'PURC', _PurcNo, 0, _Total, _ExcRate, _FisRate, _CrcID);

			SELECT INTO _spret * FROM spPay_New(_PayNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			-- tetep harus update Purc nya lagi, sebab Pay yg reftype='PURC' tidak update Purc ???
			_tmp_va1:='F';
			UPDATE Purc SET PaidAmt=_Total, PayStatus=_tmp_va1 WHERE PurcNo=_PurcNo;
						
		--************************************** KREDIT *************************************
		ELSEIF (_TermType='R') THEN		
			_retval.ErrNo:=50051;
			_retval.ErrMsg:='Total: '||_PurcNo;
			
			SELECT INTO _AllowCredit, _CreditLimit, _CreditAmount 
				aplimitfreq, aplimitamt, apbalance 
			FROM BPAcc
			WHERE BPID=_VendorID AND crcid=_CrcID;

			SELECT INTO _CountCredit COUNT(*) FROM purc WHERE PayStatus<>'F' AND vendorid=_VendorID;

			IF (_AllowCredit=0) THEN
				_retval.ErrNo:=50005;
				_retval.ErrMsg:='Kepada Supplier '||_VendorID||' tidak diperkenankan KREDIT. Daftarkan supplier baru, lalu ulangi pembelian KREDIT (spPurc_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			ELSEIF (_AllowCredit<>-1 AND _AllowCredit<_CountCredit) THEN
				_retval.ErrNo:=50006;
				_retval.ErrMsg:='Kepada Supplier '||_VendorID||' melebihi jumlah diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurc_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;						
			END IF;

			IF (_CreditLimit=0) THEN
				_retval.ErrNo:=50007;
				_retval.ErrMsg:='Kepada supplier '||_VendorID||' tidak diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurc_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;		
			ELSEIF (_CreditLimit<>-1 AND _CreditLimit<(_CreditAmount+_Total) ) THEN
				_retval.ErrNo:=50008;
				_retval.ErrMsg:='Pembelian melebihi Limit Hutang (spPurc_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;		
			END IF;

			_tmp_va1:='N';
			UPDATE Purc SET PaidAmt=0, PayStatus=_tmp_va1 WHERE PurcNo=_PurcNo;
			_BPAAmt:=_Total*(-1);
			_BPANote:='Pembelian #'||_PurcNo;
			

			-- Posting Account-Payable
			SELECT INTO _spret * FROM spBPA_New(
						_VendorID, 'C',_CrcID, _PurcDate, 'PURC', _PurcNo, _BPAAmt, 
						_BPANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
						_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		END IF;
	END IF;

	
	--******************** Update POD:QtyReceive untuk yang pakai PO & tanpa lewat PRcv *******************************
	OPEN cPurcD2 FOR
		SELECT PONo, PODNo, ItemID, PID, fQtyZToQty1(ItemID, Qty, Unit) 
		FROM PurcD 
		WHERE PurcNo=_PurcNo AND (PONo IS NOT NULL AND PONo<>'') AND (PRcvNo IS NULL OR PRcvNo='')
		ORDER BY PurcDNo;
	LOOP
		FETCH cPurcD2 INTO _PONo, _PODNo, _ItemID, _PID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Rcv * FROM spPO_Item_Receive(_PONo, _ItemID, _PID, _Qty1, _PODNo);
	END LOOP;
	CLOSE cPurcD2;

	--********************* Update PO Status *****************************************************************
	OPEN cPO FOR
		SELECT DISTINCT PONo FROM PurcD 
		WHERE PurcNo=_PurcNo AND (PONo IS NOT NULL AND PONo<>'') AND (PRcvNo IS NULL OR PRcvNo='')
		ORDER BY PONo;
	LOOP
		FETCH cPO INTO _PONo;
		EXIT WHEN NOT FOUND;
		
		SELECT INTO _VendorID2 VendorID FROM PO WHERE PONo=_PONo;
		IF (_VendorID<>_VendorID2) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Supplier PO dan Pembelian tidak sama (spPurc_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		
		SELECT INTO _spret * FROM spPO_ChNUp_Status(_PONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cPO;
	
	------------------------ Update status Invoice PRcv --------------------
	OPEN cPRcv FOR
		SELECT DISTINCT PRcvNo FROM PurcD 
		WHERE PurcNo=_PurcNo AND (PRcvNo IS NOT NULL AND PRcvNo<>'') 
		ORDER BY PRcvNo;
	LOOP
		FETCH cPRcv INTO _PRcvNo;
		EXIT WHEN NOT FOUND;
				
		UPDATE PRcv SET isInvoiced=TRUE WHERE PRcvNo=_PRcvNo;
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cPRcv;
	

/************************************************************************************************************** 
	-----------------------------------------------------------------
	-------------------- ACCOUNTING POSTING here --------------------
	-----------------------------------------------------------------
****************************************************************************************************************/


	IF (isAccInstalled()) THEN
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlDNo:=0;

	
		IF(_PurcType='B') THEN
			_JnlNote:= 'Set saldo awal Hutang: '||fBPName(_VendorID)||':'||_PurcNo;
			_JnlType:='BEG';
		ELSE
			_JnlNote:= 'Pembelian '||fBPName(_VendorID)||':'||_PurcNo;
			_JnlType:='PURC';
		END IF;

		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, _JnlType, _PurcDate,'PURC',_PurcNo,
			_JnlNote , true, true, CAST(0 AS DECIMAL(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
			);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF _TermType='C' THEN	-- tunai
			_AccNo:=fAccCash(_CashID, _InOut);
		ELSIF _TermType='R' THEN -- kredit -> hutang
			_AccNo:=fBpAcc(_VendorID,_CrcID,'AP');
		ELSIF(_PurcType='B') THEN -- set saldo awal hutang
			_AccNo:=_AccAP;
		END IF;
		
/***************************  Jurnal yang dari Purchase Receive ? ***********************************/
		OPEN cPurcD FOR
			SELECT SUM(d.basesubtotal), SUM(d.subtotal), DeptID, PrjID
			FROM PurcD d 
			WHERE d.PurcNo=_PurcNo AND (PRcvNo IS NOT NULL AND PRcvNo<>'')
			GROUP BY DeptID,PrjID;
		LOOP
			FETCH cPurcD INTO _DBSubtotal, _DSubtotal, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			_JnlDNo:=_JnlDNo+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, fAccDefault('UBPRCV'), 'D', _DBSubtotal, _DeptID, _PrjID, _DSubtotal);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;						
		END LOOP;
		CLOSE cPurcD;

/****************** ayat jurnal yg Purc Invoice/normal (gak IMPORT DARI PRCV) **********************/

		OPEN cPurcD FOR
			SELECT SUM(d.basesubtotal), SUM(d.subtotal), fItemAcc(ItemID,'INV'), DeptID, PrjID
			FROM PurcD d 
			WHERE d.PurcNo=_PurcNo AND (PRcvNo IS NULL OR PRcvNo='')
			GROUP BY fItemAcc(itemid,'INV'), DeptID, PrjID;
		LOOP
			FETCH cPurcD INTO _DBSubtotal, _DSubtotal, _AccINV, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;
			
			_JnlDNo:=_JnlDNo+1;			
			SELECT INTO _spret * FROM spJnlD_Insert(
				_JnlNo, _JnlDNo, _AccINV, 'D', _DBSubtotal,
				_DeptID, _PrjID, _DSubTotal
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;			
		END LOOP;
		CLOSE cPurcD;

/*************** Jurnal TAX DI GRUP IN SESUAI PAJAK TIAP2 ITEM *********************/
		SELECT INTO _TotalTax SUM(d.totaltaxamt*_ExcRate) 
		FROM PurcD d JOIN tax ON d.TaxID=tax.TaxID
		WHERE d.PurcNo=_PurcNo AND d.TaxID IS NOT NULL;

		OPEN cTax FOR
			SELECT SUM(d.basftotaltaxamt), SUM(d.totaltaxamt), AccPurcTax, DeptID, PrjID
			FROM PurcD d JOIN tax ON d.TaxID=tax.TaxID
			WHERE d.PurcNo=_PurcNo AND d.TaxID IS NOT NULL
			GROUP BY AccPurcTax, DeptID, PrjID;
		LOOP
			FETCH cTax INTO _BTotalTaxAmt, _TotalTaxAmt, _AccPurcTax, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			_JnlDNo:=_JnlDNo+1;
			SELECT INTO _spret * FROM spJnlD_Insert(
				_JnlNo, _JnlDNo, _AccPurcTax,'D',
				_BTotalTaxAmt, _DeptID, _PrjID,
				_TotalTaxAmt
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;

/*************** Jurnal Purc-Discount (Header/Total) *********************/

		IF (_MDiscAmt<>0) THEN
			_JnlDNo:=_JnlDNo+1;
			IF(_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('PURCDISC');
			END IF;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccDisc, 'K', _MBDiscAmt, NULL, NULL, _MDiscAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		
		IF (_Freight<>0) THEN
			_JnlDNo:=_JnlDNo+1;
			IF (_AccFreight IS NULL) THEN
				_AccFreight:=fAccDefault('PURCFREIGHT');
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(
				_JnlNo, _JnlDNo, _AccFreight, 'D', 
				_Freight*_ExcRate, NULL, NULL, _Freight);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		IF (_Others<>0) THEN
			_JnlDNo:=_JnlDNo+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, fAccDefault('PKROUND'), 'D', ROUND(_Others*_ExcRate, _Round), null, null, _Others);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		
/** ==================== LABA RUGI SELISIH KURS ================================ */
		_RateLR:=(_BTotal+_MBDiscAmt)-(_BSubtotal+_MBFTaxAmt);
		IF (_RateLR!=0) THEN
			_JnlDNo:=_JnlDNo+1;
			_AccRateLR:=fAccDefault('PKKURS');
			IF (_AccRateLR IS NULL OR _AccRateLR='') THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Akun PK Selisih Kurs belum diset.'||E'\n'||'Silahkan disetting pada form Setting Akun';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccRateLR, 'D', _RateLR, _DeptID, _PrjID, NULL, _RateLR/_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;	

/******************  Posting Kas-Keluar / Hutang Usaha **************************/
		_JnlDNo:=_JnlDNo+1;
		SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccNo, 'K', _BTotal, null, null, _Total);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

/**^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^***/
/*************************** FINISHED GENERATE JURNAL ****************************/

/***************** Trigger Posting the Jurnal ************************************/
		
		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	END IF;	-- IF isAccInstalled()


	
_retval:=(0, 'OK');
RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function spPurc_Void (
	__PurcNo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE	
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_VendorID		VARCHAR(15);
	_PurcDate		DATE;
	_PurcType		VARCHAR(5);
	_PRcvNo			VARCHAR(15);
	_PONo			VARCHAR(15);
	_TermType		VARCHAR(1);
	_Term			SMALLINT;
	_Total			NUMERIC(19,4);
	_CashID			VARCHAR(15);
	_PayStatus		VARCHAR(1);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_PayNo			VARCHAR(15);

	_ItemID			VARCHAR(20);
	_PID			VARCHAR;
	_PurcDNo		SMALLINT;
	
	_CredAmount		NUMERIC(19,4);
	_CredNote		VARCHAR(50);

	_LastPurcNo		VARCHAR(15);

	_LCPrice		NUMERIC(19,4);
	_LCUnit			VARCHAR(5);
	_LCDiscExp		VARCHAR(15);
	_LCDiscAmt		NUMERIC(19,4);
	_LCTaxID		VARCHAR(5);
	_LCTaxAmt		NUMERIC(19,4);
	_LCDate			DATE;
	_LastVendorID		VARCHAR(15);
	_PurcNo			VARCHAR(15);
	_JnlNo			VARCHAR(15);
/** PO stuff **/
	_Qty1			NUMERIC(15,4);
	_Qty1Cancel		NUMERIC(15,4);
	_Tmp			BOOLEAN;
	cPRcvPurc 		REFCURSOR;
	cPurcD			REFCURSOR;
	cPO			REFCURSOR;
	cPurcD2			REFCURSOR;
	cAcc			REFCURSOR;
	_regval			VARCHAR(30);

	_isDraft		BOOLEAN;
	_isLast			BOOLEAN;
	
	_LockDate		DATE;
	_PODNo			INTEGER;

	_WhId			VARCHAR;
	_PRetCount		INT;
	_PRetNos		VARCHAR;
	_PayNos			VARCHAR;
BEGIN
	_retval.ErrNo:=10;
	_retval.ErrMsg:='ERROR';
 
	_PurcNo:=__PurcNo;

	SELECT INTO
			_VendorID, _PurcDate, _TermType, _Term, _CashID,
		   _Total, _PayStatus, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _isDraft

			VendorID, PurcDate, TermType, duedays, CashID,
			Total, PayStatus, CrtBy, CrtDate, UpdBy, UpdDate, isDraft
	FROM Purc WHERE PurcNo=_PurcNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Pembelian #'||_PurcNo||' tidak ditemukan (spPurc_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='PURC';
	IF(_LockDate is not null AND _LockDate>_PurcDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Pembelian Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _PRetCount, _PRetNos COUNT(*), SUM(DISTINCT pretno) FROM pretd WHERE purcno=_PurcNo;
	IF (FOUND AND _PRetCount>0) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:=E'Pembelian ini sudah dipakai dalam transaksi retur pembelian:\n'||COALESCE(_PRetNos, 'BATCH')||E'\nVoid terlebih dahulu retur pembelian tersebut (spPurc_Void)!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	IF (_Total>0) AND (_TermType='R') AND (_PayStatus<>'N') THEN
		SELECT INTO _PayNos SUM(DISTINCT payno) FROM ppaid WHERE refno=_PurcNo;
		_retval.ErrNo:=50002;
		_retval.ErrMsg:=E'Sudah ada transaksi pembayaran:\n'||COALESCE(_PayNos, 'BATCH')||E'\nVoid terlebih dahulu pembayaran tersebut ! (spPurc_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO _spret * FROM spCheckDP('PURC', _PurcNo);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM POPurc WHERE PurcNo=_PurcNo;
		DELETE FROM PurcD WHERE PurcNo=_PurcNo;
		DELETE FROM PRcvPurc WHERE PurcNo=_PurcNo;
	/*--	DELETE FROM PPaid WHERE RefType='PURC' AND RefNo=_PurcNo;*/
		DELETE FROM Purc WHERE PurcNo=_PurcNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;


	---- Check apakah sudah ada transaksi apapun setelah pembelian ini, cek per item di kartustock
	OPEN cPurcD FOR
		SELECT purcdno, itemid, whid FROM purcd WHERE purcno=_PurcNo;
	LOOP
		FETCH cPurcD INTO _PurcDNo, _ItemID, _WhId;
		EXIT WHEN NOT FOUND;

		_isLast:=isItemLastTrans(_ItemID, _WhId, 'PURC', _PurcNo);
		
		IF NOT _isLast THEN
			_retval.ErrNo:=50022;
			_retval.ErrMsg:='Pembelian tidak bisa di Edit/Void karena sudah ada transaksi baru lain.'||E'\n'||' Lakukan Penyesuaian Stock bila perlu (spPurc_Void)';
			RAISE EXCEPTION '%', _retval.ErrMsg;			
		END IF;

	END LOOP;
	CLOSE cPurcD;




	OPEN cPRcvPurc FOR
	SELECT PRcvNo FROM PRcvPurc WHERE PurcNo=_PurcNo;
	LOOP
		FETCH cPRcvPurc INTO _PRcvNo;
		EXIT WHEN NOT FOUND;
		UPDATE PRcv SET isinvoiced=false WHERE PRcvNo=_PRcvNo;
		DELETE FROM PRcvPurc WHERE PRcvNo=_PRcvNo AND PurcNo=_PurcNo;
	END LOOP;

	SELECT into _spret * FROM spStockA_Void_byRef('PURC', _PurcNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;
	CLOSE cPRcvPurc;


	IF (_TermType='C') THEN
		/** VOID Pembayaran-Auto nya */
		SELECT INTO _PayNo PayNo FROM Pay WHERE RefType='PURC' AND RefNo=_PurcNo;
		IF (_PayNo IS NOT NULL) THEN
			_retval.ErrNo:=50022;
			_retval.ErrMsg:='ERROR EXEC spPay_Void for PayNo=' || _PayNo || ' (spPurc_Void)';
			
			SELECT INTO _spret * FROM spPay_Void(_PayNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
				EXIT;
			END IF;
		END IF;
	ELSIF (_TermType='R') THEN
		SELECT INTO _spret * FROM spBPA_Void_byRef('PURC', _PurcNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;


	OPEN cPurcD2 FOR
		SELECT PONo, PODNo, ItemID, PID, fQtyZToQty1(ItemID, Qty, Unit)
		FROM PurcD
		WHERE PurcNo=_PurcNo 
		AND (PONo IS NOT NULL AND PONo<>'') 
		AND (PRcvNo IS NULL OR PRcvNo='')
		ORDER BY PurcDNo;
	LOOP
		FETCH cPurcD2 INTO _PONo, _PODNo, _ItemID, _PID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Cancel * FROM spPO_Item_Cancel(_PONo, _ItemID, _PID, _Qty1, _PODNo);
	END LOOP;
	CLOSE cPurcD2;


	OPEN cPO FOR 
		SELECT DISTINCT PONo FROM PurcD 
		WHERE PurcNo=_PurcNo 
		AND (PONo IS NOT NULL AND PONo<>'') 
		AND (PRcvNo IS NULL OR PRcvNo='')
		ORDER BY PONo;
	LOOP
		FETCH cPO INTO _PONo;
		EXIT WHEN NOT FOUND;
		
		SELECT INTO _spret * from spPO_ChNUp_Status (_PONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cPO;

	IF (isAccInstalled()) THEN
		SELECT into _spret * from spJnl_Void_byRef('PURC',_PurcNo);
		IF _spret .ErrNo<>0 THEN
			_retval.ErrNo:=_spret .ErrNo;
			_retval.ErrMsg:=_spret .ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM POPurc WHERE PurcNo=_PurcNo;
	DELETE FROM PurcD WHERE PurcNo=_PurcNo;
	DELETE FROM PRcvPurc WHERE PurcNo=_PurcNo;
	DELETE FROM Purcx WHERE PurcNo=_PurcNo;
/*--	DELETE FROM PPaid WHERE RefType='PURC' AND RefNo=_PurcNo;*/
	DELETE FROM Purc WHERE PurcNo=_PurcNo;

	_retval:=(0, 'OK'); 
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE plpgsql;




CREATE OR REPLACE FUNCTION spPRet_New (
	_PRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret 			SPStatus%ROWTYPE;
	
	_VendorID		VARCHAR(15);
	_PRetDate		DATE;
	_TermType		VARCHAR(1);
	_Total			NUMERIC(19,4);
	_CashID			VARCHAR(15);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;

	_StockANo		VARCHAR(15);
	_StockANote		VARCHAR(255);
	_StockADNo		SMALLINT;
	_StockADCost		NUMERIC(19,4);

	_PRetDNo		SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_Conv			NUMERIC(15,4);
	_QtyX			NUMERIC(15,4);
	_ListPrice		NUMERIC(19,4);
	_DiscExp		VARCHAR(15);
	_DiscAmt		NUMERIC(19,4);
	_TaxID			VARCHAR(5);
	_TaxAmt			NUMERIC(19,4);

	_PRetQty		NUMERIC(15,4);
	_PRetQtyX		NUMERIC(15,4);

	_CashAmt		NUMERIC(19,4);
	_CashANote		VARCHAR(255);
	_IsContra		BOOLEAN;
	_regval			VARCHAR(30);

	_AccNo			VARCHAR;
	_AccInv			VARCHAR;
	_AccPurcTax		VARCHAR;
	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;
	_JnlNo1			VARCHAR(15);
	_JnlNote1		VARCHAR;
	_count			INT;
	_count1			INT;
	_CrcID			VARCHAR(5);
	cPRetD			REFCURSOR;
	_DebtAmt		DECIMAL(19,4);
	_DebtNote		VARCHAR(50);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
   	_DeptID			VARCHAR(5);
   	_PrjID			VARCHAR(5);
	_MTaxAmt		DECIMAL(19,4);
	_MDiscAmt		DECIMAL(19,4);
	_Freight		DECIMAL(19,4);
	_Others			DECIMAL(19,4);
	_ExcRate		DECIMAL(19,4);
	_AccFreight		VARCHAR(10);
	_AccDisc		VARCHAR(10);
	_isUsePrcv		BOOLEAN;
	
	/** buat pretprcv **/
	cPRetPRcv		REFCURSOR;
	cPurc			REFCURSOR;
	cAcc			REFCURSOR;
	_PRcvNo			VARCHAR(15);
	_CountPRcv		INT;
	_Round			INT;
	_RoundTax		INT;
	_unit1			VARCHAR;
	_unit2			VARCHAR;
	_unit3			VARCHAR;
	_conv2			DECIMAL;
	_conv3			DECIMAL;
	_listpriceAsli		DECIMAL;
	_qtyAsli		DECIMAL;
	_UnitAsli		VARCHAR;
	_PurcNo			VARCHAR;
	
	_isDraft		BOOLEAN;
	_isStock		BOOLEAN;

	_JnlNote2		VARCHAR;
	_JnlNo2			VARCHAR;

	_UBPrcvAmt 		DECIMAL(19,4);

	_JnlDAmt 		DECIMAL;
	_LockDate		DATE;
	_PRcvNoJnlD		VARCHAR;
	_Cost			NUMERIC(19,4);
	_InvAcc			VARCHAR;
	_JnlNo3			VARCHAR;
	_JnlNote3		VARCHAR;
	_Subtotal		NUMERIC(19,4);
    	_TotalBasePrice		NUMERIC(19,4);
	cPretD1			REFCURSOR;
	_ItemIDCheck		VARCHAR;
	cTax			REFCURSOR;
	_PretDTaxAmt		NUMERIC(19,4);
	_AccPretTax		VARCHAR;
	_TotalTax		NUMERIC(19,4);
	_SubtotalTax		NUMERIC(19,4);

	_PRetStatus		VARCHAR(1);
	_PRetStatusYes		VARCHAR(1);
	_FisRate		NUMERIC(19,4);
	_StockADSubTotal	DECIMAL(19,4);

	_BTotal			NUMERIC(19,4); -- base currency total
	_BSubtotal		NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt		NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt		NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt		NUMERIC(19,4);
	_TotalTaxAmt		NUMERIC(19,4);
	_BTotalTaxAmt		NUMERIC(19,4);
	_TotalDiscAmt		NUMERIC(19,4);
	_TotalDisc2Amt		NUMERIC(19,4);
	_DSubtotal		NUMERIC(19,4);
	_DBSubtotal		NUMERIC(19,4);

	_qty_purcd		DECIMAL;
	_qty_prcvd		DECIMAL;
	_qty_pretd		DECIMAL;
	
	_AccRateLR		VARCHAR; -- akun laba/rugi selisih kurs
	_RateLR			NUMERIC(19,4); -- nilai laba/rugi selisih kurs

	_CashCrcID		VARCHAR;
	_InOut			CHAR(1);
	_PurcDNo		INT2;
	_PrcvDNo		INT2;
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_InOut:='I';
	_PRetStatusYes:='Y';
/**
_retval:=(0,'OK');
RETURN NEXT _retval;
RETURN;

**/

	SELECT INTO 
		_PerID, _BranchID, _VendorID, _CrcID, _PRetDate, _TermType, _BTotal, _Total, _CashID, _CrtBy, _CrtDate, 
		_UpdBy, _UpdDate, _MBFTaxAmt, _MTaxAmt, _ExcRate, _MBDiscAmt, _MDiscAmt, _Freight, _AccFreight, _AccDisc, _Others, 
		_IsContra, _BSubtotal, _Subtotal, _PRetStatus, _FisRate, _isDraft

   	    PerID, BranchID, VendorID, CrcID, PRetDate, TermType, basetotal, Total, CashID, CrtBy, CrtDate, 
	    UpdBy, UpdDate, basftaxamt, taxamt, ExcRate, basediscamt, discamt, freight, accfreight, accpdisc, others,
	    iscontra, basesubtotal, subtotal, pretstatus, FisRate, isDraft
	FROM PRet 
	WHERE PRetNo=_PRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Retur #'||_PRetNo||' tidak ditemukan! (spPRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');	
	_RoundTax:=COALESCE(fReg('ROUND_TAX'), '0');

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='PRET';
	IF(_LockDate IS NOT NULL AND _LockDate>_PRetDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Retur Pembelian Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF(_TermType='R' AND _IsContra=true AND (_PRetStatus<>'N')) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Retur sudah pernah dipakai pembayaran (spPret_New)';	
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;

	-- ngecek itemnya is purc apa gak!
	OPEN cPretD1 FOR
	SELECT pretd.itemid FROM pretD 
	JOIN item on pretd.itemid=item.itemid 
	WHERE ispurc=FALSE and pretno=_PretNo;
	LOOP
		FETCH cPretD1 INTO _ItemIDCheck;
		EXIT WHEN NOT FOUND;

		_retval.ErrNo:=50055;
		_retval.ErrMsg:='Error, ItemID:' || _ItemIDCheck || ' tidak di Set untuk Pembelian!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END LOOP;
	CLOSE cPretD1;

	_CountPRcv:=0;
	OPEN cPRetPRcv FOR
		SELECT DISTINCT PRcvNo FROM PRetD 
		WHERE PRetNo=_PRetNo 
		AND (PRcvNo IS NOT NULL AND PRcvNo<>'');
	LOOP
		FETCH cPRetPRcv INTO _PRcvNo;
		EXIT WHEN NOT FOUND;

		_CountPRcv:=_CountPRcv+1;
		UPDATE PRcv SET 
			isInvoiced=TRUE 
		WHERE PRcvNo=_PRcvNo;

		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Error Update PRcv#'||_PRcvNo||' (spPRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;		
	END LOOP;
	CLOSE cPRetPRcv;

	_StockADNo:=0;
	_Count:=0;
	OPEN cPRetD FOR
		SELECT d.pretdno, d.itemid, d.whid, d.pid, d.qty, d.unit, d.qtyx, d.listprice, d.discamt, 
		d.taxamt, d.deptid, d.prjid, d.prcvno, d.prcvdno, i.isstock, d.purcno, d.purcdno 
		FROM pretd d 
		JOIN item i ON d.itemid=i.itemid
		WHERE d.pretno=_PRetNo;
	LOOP
		FETCH cPRetD INTO _PRetDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ListPrice, _DiscAmt, _TaxAmt, _DeptID, _PrjID, _PRcvNo, _PRcvDNo, _isStock, _PurcNo, _PurcDNo;
		EXIT WHEN NOT FOUND;

		IF _PurcNo IS NOT NULL AND _PurcNo<>'' THEN
			SELECT INTO _qty_pretd COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM pretd WHERE itemid=_ItemID AND pid=_PID AND purcno=_PurcNo AND purcdno=_PurcDNo;
			SELECT INTO _qty_purcd COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM purcd WHERE itemid=_ItemID AND pid=_PID AND purcno=_PurcNo AND purcdno=_PurcDNo;
			IF _qty_pretd>_qty_purcd THEN
				_retval.ErrNo:=50003;
				_retval.ErrMsg:='Error : Qty Retur Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' tidak boleh lebih dari Qty pada Pembelian (spPRet_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF ((_PurcNo IS NULL OR _PurcNo='') AND _PrcvNo IS NOT NULL AND _PrcvNo<>'') THEN
			SELECT INTO _qty_pretd COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM pretd WHERE itemid=_ItemID AND pid=_PID AND prcvno=_PRcvNo AND prcvdno=_PRcvDNo;
			SELECT INTO _qty_prcvd COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM prcvd WHERE itemid=_ItemID AND pid=_PID AND prcvno=_PRcvNo AND prcvdno=_PRcvDNo;

			IF _qty_pretd>_qty_prcvd THEN
				_retval.ErrNo:=50003;
				_retval.ErrMsg:='Error : Qty Retur Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' tidak boleh lebih dari Qty pada Penerimaan Pembelian (spPRet_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		_cost:=funitsubtotal(_itemid, _whid, _pid, _unit);
		_PRetQty:=_Qty*(-1);
		_PRetQtyX:=_QtyX*(-1);

		IF (_cost IS NULL) THEN
			_retval.ErrNo:=50055;
			_retval.ErrMsg:='Error, Hitung HPP ItemID:' || _ItemID || ', Unit:' || _Unit || ' Wh:'|| _WhID || ' PID:'|| _PID || ' !  (fItemUnit_Conv)(spPRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_Conv:=fItemUnit_Conv(_ItemID, _Unit);
		IF (_Conv IS NULL) THEN
			_retval.ErrNo:=50055;
			_retval.ErrMsg:='Error, Konversi ItemID:'||_ItemID||', Unit:'||_Unit||'!  (fItemUnit_Conv)(spPRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- jika di stock dan BUKAN dari PRCV (kalau dari PRcv stock sudah dikeluarin)
		IF (_isStock=TRUE AND ((_PRcvNo IS NULL) OR (_PRcvNo=''))) THEN
			_Count:=_Count+1;

			IF(_Count=1) THEN
				SELECT INTO _StockANo * FROM fTransNoGet('STOCKA');
				IF (_StockANo IS NULL) THEN
					_retval.ErrNo:=1041;
					_retval.ErrMsg:='Error getting ID STOCKA (spSTOCKA_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				_StockANote:='ReturBeli #'||_PRetNo;
				SELECT INTO _spret * FROM spStockA_New(
					_StockANo,_VendorID, 'PRET', _PRetDate, 'O', 'PRET', _PRetNo, _StockANote, TRUE, 
					_CrtBy, _CrtDate, _UpdBy, _UpdDate, _PerID, _BranchID, _DeptID, _PrjID								
				);

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			_StockADNo:=_StockADNo+1;
			_StockADCost:=(_ListPrice-COALESCE(_DiscAmt, 0)+COALESCE(_TaxAmt, 0))*_ExcRate;
			_StockADSubTotal:=_PRetQty*(_ListPrice-COALESCE(_DiscAmt,0))*_ExcRate;

			SELECT INTO _spret * FROM spStockAD_New(_PRetDate, 'PRET', _StockANo, _StockADNo, _WhID, _ItemID, 
				_PID, _PRetQty, _Unit, _PRetQtyX, _cost, _DeptID, _PrjID, true, _PRetNo, _StockADSubTotal);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		END IF; -- isStock


		-- update HPP di PRet
		UPDATE PRetD SET 
			Conv=_Conv, Cost=_Cost 
		WHERE PRetNo=_PRetNo AND PRetDNo=_PRetDNo;

	END LOOP;
	CLOSE cPRetD;

	IF (_TermType='C') THEN
		_IsContra:=true;
	ELSE
		_IsContra:=false;
	END IF;

	-- update status isContra
	UPDATE PRet SET 
		IsContra=_IsContra 
	WHERE PRetNo=_PRetNo;

	IF (_TermType='C') THEN
		IF (_CashID IS NULL) THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Kas tidak boleh kosong (spPRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _CashCrcID crcid FROM cash WHERE cashid=_CashID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' belum diset (spPRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
 					
		IF (_CrcID!=_CashCrcID) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' tidak sama dengan mata uang pembelian (spPRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;		
		END IF;

		_CashANote:='ReturBeli #'||_PRetNo;
		_CashAmt:=_Total;

		SELECT INTO _spret * FROM spCashA_New(
			_CashID, _VendorID, 'PRET', _PRetDate, 'I', 'PRET', _PRetNo, 
	      _CashAmt, _CashANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMSg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		UPDATE PRet SET PRetStatus=_PRetStatusYes, IsContra=_IsContra WHERE PRetNo=_PRetNo;
	ELSE
           _DebtAmt:=_Total;		/** penerimaan : potong hutang (-) */
		_DebtNote:='ReturBeli #'||_PretNo;

		SELECT INTO _spret * from spBPA_New(
			_VendorID, 'P', _CrcID, _PretDate, 'PRET', _PretNo, 
			_DebtAmt, _DebtNote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate											
		);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	END IF;
	
/* ================================ Accounting ================================================= */
	IF (isAccInstalled()) THEN
		_JnlNote:= 'Retur Pembelian '||fBPName(_VendorID)||':'||_PretNo;
		_JnlNo:=fTransNoGet('JURNAL');

		_Count:=0;
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'PRET', _PretDate,'PRET',_PretNo,
			_JnlNote,true,true,cast(0 as numeric(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate							
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF _TermType='C' THEN
			_AccNo:=fAccCash(_CashID, _InOut);
		ELSE
			_AccNo:=fBpAcc(_VendorID, _CrcID, 'AP');
		END IF;

		_Count:=_Count+1;
		SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo, 'D', _BTotal, _DeptID, _PrjID, _Total);			
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
			
		SELECT INTO 
			_TotalBasePrice 
			SUM(qty*(BasePrice-COALESCE(DiscAmt,0))*_ExcRate) 
		FROM pretd 
		WHERE PRetNo=_PRetNo 
		AND (fItemAcc(ItemID, 'INV') IS NOT NULL AND fItemAcc(ItemID, 'INV')<>'');


		-- posting ayat inventory yang dari PRCV
		IF(_countPRcv>0) THEN
			SELECT INTO 
				_UBPRcvAmt
				SUM(CAST((d.BasePrice-d.DiscAmt)*d.Qty AS NUMERIC(19,4)))
			FROM PRetD d
			WHERE PRetNo=_PRetNo
			AND (PRcvNo IS NOT NULL AND PRcvNo<>'');

			_AccNo:=fAccDefault('UBPRET');

			_Count:=_Count+1;
			SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo, 'K', CAST(ROUND(_UBPRcvAmt*_ExcRate, _Round) AS NUMERIC(19,4)), _DeptID, _PrjID, _UBPRcvAmt);			
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		-- posting inventory yg BUKAN dari PRcv
		OPEN cPRetD FOR
			SELECT SUM(d.basesubtotal), SUM(d.subtotal), fItemAcc(itemid,'INV'), deptid, prjid
			FROM PretD d 
			WHERE d.PretNo=_PretNo 
			AND (d.PRcvNo IS NULL OR d.PRcvNo='')
			GROUP BY fItemAcc(itemid,'INV'), deptid, prjid;
		LOOP
			FETCH cPretD INTO _DBSubtotal, _DSubtotal, _AccINV, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;
		
			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccINV, 'K', _DBSubtotal, _DeptID, _PrjID, _DSubtotal);			
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cPRetD;
	

		SELECT INTO 
			_TotalTax 
			SUM(ROUND(d.taxamt*Qty*_ExcRate, _RoundTax))
		FROM PretD d 
		JOIN tax ON d.taxid=tax.taxid
		WHERE d.PretNo=_PretNo and d.taxid IS NOT NULL;


		-- POSTING TAX DI grup by taxid
		OPEN cTax FOR
			SELECT SUM(basftotaltaxamt), SUM(totaltaxamt), accpurctax, deptid, prjid
			FROM PretD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.PretNo=_PretNo and d.taxid is not null
			GROUP BY accpurctax, deptid, prjid;
		LOOP
			FETCH cTax INTO _BTotalTaxAmt, _TotalTaxAmt, _AccPretTax, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccPretTax, 'K', _BTotalTaxAmt, _DeptID, _PrjID, _TotalTaxAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;


/** ==================== LABA RUGI SELISIH KURS ================================ */
		_RateLR:=(_BTotal+_MBDiscAmt)-(_BSubtotal+_MBFTaxAmt);
		IF (_RateLR!=0) THEN
			_count:=_count+1;
			_AccRateLR:=fAccDefault('PKKURS');
			IF (_AccRateLR IS NULL OR _AccRateLR='') THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Akun PK Selisih Kurs belum diset.'||E'\n'||'Silahkan disetting pada form Setting Akun';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _count, _AccRateLR, 'K', _RateLR, _DeptID, _PrjID, NULL, _RateLR/_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		-- posting freight
		IF(_Freight<>0) THEN
			IF (_AccFreight IS NULL) THEN
				_AccFreight:=fAccDefault('PURCFREIGHT');
			END IF;
			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,_Count,_AccFreight,'K',_Freight*_ExcRate,_DeptID,_PrjID,_Freight);			
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		-- posting purchase discount
		IF(_MDiscAmt<>0) THEN
			IF (_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('PURCDISC');
			END IF;
			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccDisc, 'K', _MBDiscAmt*-1, _DeptID, _PrjID, _MDiscAmt*-1);			
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;


		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPRet_Void (
	_PRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
	
	_VendorID	VARCHAR(15);
	_TermType	VARCHAR(1);
	_IsContra	BOOLEAN;
	_JnlNo		VARCHAR(15);
	_regval		VARCHAR(30);

	_PRcvNo		VARCHAR(15);
	cPRetPRcv	REFCURSOR;

	_PRetStatus	VARCHAR(1);
	_isDraft	BOOLEAN;
	_PRetDate	DATE;
	_LockDate	DATE;

BEGIN
	SELECT INTO _VendorID, _TermType, _IsContra, _PRetStatus, _PRetDate	
		    VendorID, TermType, IsContra, PRetStatus, PRetDate
	FROM PRet WHERE PRetNo=_PRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Retur/Memo-Hutang #'||_PRetNo||' tidak ditemukan (spPRet_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='PRET';
	IF(_LockDate is not null AND _LockDate>_PRetDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Retur Pembelian Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF ((_TermType='R') AND (_IsContra=true)) THEN /*-- (_PRetStatus<>'N')AND ()) THEN*/
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Retur/Memo-Hutang sudah dipotongkan pembayaran Hutang, VOID dulu pembayarannya ! (spPRet_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** CEK ISDRAFT */
	SELECT INTO _isDraft IsDraft FROM PRet WHERE PRetNo=_PRetNo;
	IF NOT FOUND THEN
		_isDraft:=FALSE;
	END IF;	

	IF (_isDraft IS NULL) THEN
		_isDraft:=FALSE;
	END IF;

	IF (_isDraft=FALSE) THEN /** MULAI ISDRAFT */

	OPEN cPRetPRcv FOR
	SELECT PRcvNo FROM PRetPRcv WHERE PRetNo=_PRetNo;
	LOOP
		FETCH cPRetPRcv INTO _PRcvNo;
		EXIT WHEN NOT FOUND;
		UPDATE PRcv SET isinvoiced=false WHERE PRcvNo=_PRcvNo;
		DELETE FROM PRetPRcv WHERE PRcvNo=_PRcvNo AND PRetNo=_PRetNo;
	END LOOP;
	CLOSE cPRetPRcv;

	SELECT into _spret * FROM spStockA_Void_byRef('PRET', _PRetNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrNo;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_TermType='C') THEN
		SELECT into _spret * FROM spCashA_Void_byRef('PRET', _PRetNo,null,null);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
	ELSE
		SELECT into _spret * FROM spBPA_Void_byRef('PRET', _PretNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	END IF; /** END IF ISDRAFT */
	
	IF (isAccInstalled() AND _isDraft=FALSE) THEN
		SELECT into _spret * from spJnl_Void_byRef('PRET',_PRetNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM PRetD WHERE PRetNo=_PRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Delete PRetD (spPRet_Void)';
	END IF;
	
	DELETE FROM PRet WHERE PRetNo=_PRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error Delete PRet (spPRet_Void)';
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSO_New (
	_SONo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_spretStockCard SPStatus%ROWTYPE;
	_SOStatus 	VARCHAR;
	_SODNo 		INT;
	_ItemID 	VARCHAR;
	_PID 		VARCHAR;
	_WHID 		VARCHAR;
	_Qty 		DECIMAL(15,4);
	_qty1_delivered	DECIMAL(15,4);
	_qty1_backorder	DECIMAL(15,4);
	_Unit 		VARCHAR;
	_QtyX 		DECIMAL(15,4);
	_ListPrice 	DECIMAL(19,4);
	_DiscExp	VARCHAR;
	_DiscAmt	DECIMAL(19,4);
	_TaxID 		VARCHAR;
	_TaxAmt 	DECIMAL(19,4);
	_IsSale 	BOOLEAN;
	_qty1 		DECIMAL(15,4);
	_tmpErrNo 	INTEGER;
	_tmpErrMsg 	VARCHAR(255);
	cSOD 		REFCURSOR;

	_Active		BOOLEAN;
	_OldActive	BOOLEAN;
	_OldStatus	VARCHAR;
	_isDraft	BOOLEAN;
	_isStock	BOOLEAN;
BEGIN
	SELECT INTO 
		_SOStatus, _isDraft, _OldActive
		SOStatus, isDraft, active
	FROM SO WHERE SONo=_SONo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50000;
		_retval.ErrMsg:='Order Penjualan '|| COALESCE(_SONo, '') || ' tidak ditemukan (spSO_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_OldStatus:=_SOStatus;
	_Active:=FALSE;	
	IF (_isDraft IS NULL OR _isDraft=FALSE) THEN /** MULAI ISDRAFT */

		OPEN cSOD FOR
			SELECT 	d.SODNo, d.ItemID, d.WhID, d.PID, d.Qty, d.QtyX, d.Unit, d.QtyDelivered, 
				d.QtyBO, d.ListPrice, d.DiscExp, d.DiscAmt, d.TaxID, d.TaxAmt, i.IsSale, i.isStock
			FROM SOD d
			LEFT JOIN Item i ON d.ItemID=i.ItemID
			WHERE SONo=_SONo;
			IF NOT FOUND THEN
				_retval.ErrNo:=50001;
				_retval.ErrMsg:='Order Penjualan tidak ditemukan! (spSO_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		LOOP
			FETCH cSOD INTO _SODNo,_ItemID,_WHID,_PID,_Qty,_QtyX,_Unit,_qty1_delivered,_qty1_backorder,_ListPrice,_DiscExp,_DiscAmt,_TaxID,_TaxAmt, _IsSale, _isStock;
			EXIT WHEN NOT FOUND;
				
			IF (_PID IS NULL) THEN
				_PID:='';
			END IF;	


			IF (_isStock=TRUE) THEN
				SELECT INTO _spretStockCard * FROM spStockCard_ChNCr(_ItemID,_WHID,_PID);

				IF _spretStockCard.ErrNo<>0 THEN
					_retval.ErrNo:=_spretStockCard.ErrNo;
					_retval.ErrMsg:=_spretStockCard.ErrMsg;			
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _qty1 * FROM fQtyZToQty1(_ItemID, _Qty, _Unit);

				UPDATE Stock SET 
					QtySO=QtySO+_qty1, QtyXSO=QtyXSO+_QtyX 
				WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;

			END IF;

		END LOOP;
		CLOSE cSOD;
		
		/**
		
		Close related Sales Quote

		**/


		_Active:=TRUE;

	END IF; /** END IF ISDRAFT */

	IF (_OldActive=TRUE) THEN
		UPDATE SO SET Active=_Active, SoStatus='N' WHERE SONo=_SONo;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSO_Void (
	_SONo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_SOStatus 		VARCHAR;
	_Active 		BOOLEAN;
	_SODNo 			INT;
	_ItemID 		VARCHAR;
	_WhID 			VARCHAR;
	_PID 			VARCHAR;
	_Qty 			DECIMAL(15,4);
	_QtyDelivered 		DECIMAL(15,4);
	_QtyBO 			DECIMAL(15,4);
	_Unit 			VARCHAR;
	_QtyX 			DECIMAL(15,4);
	_ListPrice 		DECIMAL(19,4);
	_DiscExp 		VARCHAR;
	_DiscAmt 		DECIMAL(19,4);
	_TaxID 			VARCHAR;
	_TaxAmt 		DECIMAL(19,4);
	_IsSale 		BOOLEAN;
	_Qty1 			DECIMAL(15,4);
	_NumOfSODeO 		INT;
	_NumOfSODeli 		INT;
	_NumOfSOSale 		INT;
	_rowcount 		INT;
	_tmpErrNo 		INTEGER;
	_tmpErrMsg 		VARCHAR;
	_retval 		SPStatus%ROWTYPE;
	_spret 			SPStatus%ROWTYPE;
	cSOD 			REFCURSOR;
	_RcvNo 			VARCHAR;

	_isDraft		BOOLEAN;
BEGIN
	SELECT INTO 
		_Active, _SOStatus, _isDraft
		Active, SOStatus, isDraft
	FROM SO WHERE SONo=_SONo;

	IF NOT FOUND THEN
		_retval.ErrNo=50001;
		_retval.ErrMsg='OrderPenjualan tidak ditemukan! (spSO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spCheckDP('SO', _SONo);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	SELECT INTO _NumOfSODeO COUNT(*) FROM DeoD WHERE SONo=_SONo;
	IF (NOT FOUND OR _NumOfSODeO>0) THEN
		_retval.ErrNo=50001;
		_retval.ErrMsg='Error sudah ada Order-Pengiriman untuk OrderPenjualan ini (spSO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _NumOfSODeli COUNT(*) FROM DeliD WHERE SONo=_SONo;
	IF (NOT FOUND OR _NumOfSODeli>0) THEN
		_retval.ErrNo=50002;
		_retval.ErrMsg='Error sudah ada Pengiriman untuk OrderPenjualan ini (spSO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _NumOfSOSale COUNT(*) FROM SaleD WHERE SONo=_SONo;
	IF (NOT FOUND OR _NumOfSOSale>0) THEN
		_retval.ErrNo=50003;
		_retval.ErrMsg='Error sudah ada Penjualan untuk S.O. ini (spSO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RcvNo rcvno from spaid where reftype='SO' and refno=_SONo;
	IF FOUND THEN
/*
		SELECT INTO _spret * FROM spRcv_Void(_RcvNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
*/

		_retval.ErrNo=50003;
		_retval.ErrMsg='Order Penjualan ini sudah ada UangMuka #'||_RcvNo||', Void dulu penerimaannya !!(spSO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	OPEN cSOD FOR
		SELECT d.SODNo, d.ItemID, d.WhID, d.Qty, d.QtyX, d.Unit, d.QtyDelivered, d.QtyBO, 
		d.ListPrice, d.DiscExp, d.DiscAmt, d.TaxID, d.TaxAmt, i.IsSale
		FROM SOD d 
		LEFT JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.SONo=_SONo AND i.IsStock=true;

	LOOP
		FETCH cSOD INTO _SODNo, _ItemID, _WhID, _Qty, _QtyX, _Unit, _QtyDelivered, _QtyBO, _ListPrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _IsSale;

		IF NOT FOUND THEN
			EXIT;
		END IF;

			/** Hitung sisa Qty1 yg masih di-PO**/ 																					
			/** Qty1=Qty-QtyD-QtyBO (QtyBO ikut dikurangkan karena kalo PO diclosed, QtyBO ditambah, dan 				**/
			/** Stock:QtySO juga sudah di-"decrease", jadi sudah tidak perlu lagi di-decrease kembali ketika VOID PO **/

		_Qty1:=fQtyZToQty1(_ItemID, _Qty, _Unit)-_QtyBO;

		UPDATE Stock SET QtySO=QtySO-_Qty1, QtyXSO=QtyXSO-_QtyX WHERE ItemID=_ItemID AND WhID=_WhID AND PID='';

	END LOOP; 
	CLOSE cSOD;

	DELETE FROM SOD WHERE SONo=_SONo;
	DELETE FROM SO WHERE SONo=_SONo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDeli_ChNUp_Status (
	_DeliNo		VARCHAR
) RETURNS SETOF SPSTATUS AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_isFull		BOOLEAN;
	_isPartial	BOOLEAN;
	_isClear	BOOLEAN;

	_DelidNo	SMALLINT;
	_Qty		DECIMAL(15,4);
	_Qtydret	DECIMAL(15,4);
	_QtyDeli	DECIMAL(15,4);

	_Qty_sum	DECIMAL(15,4);
	_Qtydret_sum	DECIMAL(15,4);
	_QtyDeli_sum	DECIMAL(15,4);


	_Qty_R1		DECIMAL(15,4);
	_Qty_R2		DECIMAL(15,4);
	_Delistatus	VARCHAR(2);
	_Active		BOOLEAN;
	cDelid		refcursor;

BEGIN

	IF (_DeliNo IS NULL OR _DeliNo='') THEN	-- utk melayani SP lain yg memang gak nge-check valuenya, kita ngalahin ngecheck disini
		_retval:=(0,'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _delistatus delistatus from deli where delino=_delino;
	IF NOT FOUND THEN
		_retval.ErrNo:=5001;
		_retval.ErrMsg:='Error Delivery not Found (spśDeli_ChNUp_Status)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_IsFull:=true;			/** default DeliStatus=FULL ***/
	_IsPartial:=false;		/** default DeliStatus=FULL ***/
	_IsClear:=true;			/** default CLOSED **/
	_Qty_sum:=0;
	_Qtydret_sum:=0;
	_QtyDeli_sum:=0;

	OPEN cDelid FOR
		SELECT DelidNo, fQtyZtoQty1(ItemID, Qty, Unit), qtydret, qtydeli
		FROM Delid WHERE delino=_delino;
		LOOP
			FETCH cDelid INTO _DelidNo, _Qty, _Qtydret, _QtyDeli;
			EXIT WHEN NOT FOUND;

			IF(_Qty<0) THEN
				_Qty:=_Qty*(-1);
			END IF;
				_Qty_sum:=_Qty_sum+_Qty;

			IF(_Qtydret<0) THEN
				_Qtydret:=_Qtydret*(-1);
			END IF;
				_Qtydret_sum:=_Qtydret_sum+_Qtydret;

			IF (_QtyDeli<0) THEN
				_QtyDeli:=_QtyDeli*(-1);
			END IF;
				_QtyDeli_sum:=_QtyDeli_sum+_QtyDeli;			

		END LOOP;
	CLOSE cDelid;

	_Qty_R1 := _Qty_sum - _QtyDeli_sum;
	_Qty_R2 := _Qty_sum - _QtyDeli_sum - _Qtydret_sum;

	IF(_Qty_R1=0) THEN
		_DeliStatus='F';
	ELSIF(_Qty_R1<>0 AND _Qty_sum<>_Qty_R1) THEN
		_DeliStatus='P';
	ELSE
		_DeliStatus='N';
	END IF;


	UPDATE Deli SET DeliStatus=_DeliStatus WHERE DeliNo=_DeliNo;

	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDeli_Close (
	_DeliNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_SPRET 		SPStatus%ROWTYPE;

	_DelidNo	SMALLINT;
	_ItemID		VARCHAR(20);
	_WhID		VARCHAR(3);
	_Qty		NUMERIC(15,4);
	_Qtydret	NUMERIC(15,4);
	_QtyDeli	NUMERIC(15,4);
	_Qtydret_New	NUMERIC(15,4);
	cDelid		REFCURSOR;

BEGIN
	_retval:=(5000, 'ERROR (spDeli_Close)');
	
	OPEN cDelid FOR
		SELECT DelidNo, WHID, ItemID, fQtyZtoQty1(ItemID,Qty,Unit), Qtydret, QtyDeli
		FROM Delid
		WHERE DeliNo=_DeliNo AND ((fQtyZtoQty1(ItemID,Qty,Unit)-Qtydret-QtyDeli)>0);
		LOOP
			FETCH cDelid INTO _DelidNo, _WhID, _ItemID, _Qty, _Qtydret, _QtyDeli;
			EXIT WHEN NOT FOUND;

			_Qtydret_New:=_Qty-_Qtydret-_QtyDeli;

			UPDATE Delid SET Qtydret=Qtydret+_Qtydret_New
			WHERE DeliNo=_DeliNo AND DelidNo=DelidNo;
		END LOOP;
	CLOSE cDelid;
	
	SELECT INTO _spret * FROM spDeli_ChNUp_Status(_DeliNo);
	IF _spret<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'Tutup Deli Berhasil');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDeli_New (
	_DeliNo		VARCHAR
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_DeliDate	DATE;
	_CustID		VARCHAR(15);
	_CustID2	VARCHAR(15);
	_TermType	VARCHAR(1);
	_IsInvoice	BOOLEAN;
	_CrtBy		VARCHAR(10);
	_CrtDate	TIMESTAMP;
	_UpdBy		VARCHAR(10);
	_UpdDate	TIMESTAMP;
	_isSale 	BOOLEAN;
	_DeliDNo	INT2;
	_itemid		VARCHAR(20);
	_WhID		VARCHAR(3);
	_PID		VARCHAR(25);
	_qty		NUMERIC(15,4);
	_qtyx		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_ListPrice	NUMERIC(19,4);
	_DiscExp	VARCHAR(15);
	_DiscAmt	NUMERIC(19,4);
	_TaxID		VARCHAR(5);
	_TaxAmt		NUMERIC(19,4);
	_Qty1		NUMERIC(15,4);
	_Qty1Rcv	NUMERIC(15,4);

	_SONo		VARCHAR(15);
	_SODNo		INT;
	_DeONo		VARCHAR(15);
	_DeliDPID	VARCHAR;

	_StockANo	VARCHAR(15);
	_StockADNo	SMALLINT;
	_StockANote	VARCHAR(255);
	_StockADCost	DECIMAL(19,4);
	_UBDeliAmt	DECIMAL(19,4);
	_MTaxAmt	DECIMAL(19,4);
	_MDiscAmt	DECIMAL(19,4);
	_ExcRate	DECIMAL(19,4);
	_Freight	DECIMAL(19,4);
	_Total		DECIMAL(19,4);
	cDeliD		refcursor;
	cSO		refcursor;
	cDeO		refcursor;
	cDeliD2		refcursor;

	_Count		INT;
	_JnlNo		VARCHAR(15);
	_JnlNo2		VARCHAR(15);
	_JnlNote	VARCHAR;

	_Inv		VARCHAR(10);
	_AccSale	VARCHAR(10);
	_AccInv		VARCHAR(10);
	_AccUBDeliSvc	VARCHAR(10);

	_PerID		VARCHAR(5);
	_BranchID	VARCHAR(10);
   	_DeptID     	VARCHAR(5);
   	_PrjID      	VARCHAR(5);

	_DeliIType	VARCHAR(5);
	_InOut		VARCHAR(1);
	_Round		INT;

	_isDraft	BOOLEAN;
	_LockDate	DATE;

	_Cost		NUMERIC(19,4);
	_CostSubTotal	NUMERIC(19,4);

	_Conv		NUMERIC(19,4);
	_Subtotal	NUMERIC(19,4);
	_CrcID		VARCHAR;
	_FisRate	NUMERIC(19,4);
	_countSO	INT;
	_CntDeo		INT;
	_Unit1		VARCHAR;
	
	_RowCount	INT;

	_BTotal		NUMERIC(19,4); -- base currency total
	_BSubtotal	NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt	NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt	NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt	NUMERIC(19,4);
	_TotalTaxAmt	NUMERIC(19,4);
	_BTotalTaxAmt	NUMERIC(19,4);
	_TotalDiscAmt	NUMERIC(19,4);
	_TotalDisc2Amt	NUMERIC(19,4);
	_DSubtotal	NUMERIC(19,4);
	_DBSubtotal	NUMERIC(19,4);
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;	

	SELECT INTO 
		_isDraft, _PerID, _BranchID, _DeliDate, _CustID, _termtype, _IsInvoice, _CrtBy, _CrtDate, _UpdBy, _UpdDate,_MDiscAmt,_MTaxAmt,_ExcRate,_Freight,_Total,_DeliIType,_Subtotal, _CrcID, _ExcRate, _FisRate
		d.isDraft, d.PerID, d.BranchID, d.DeliDate, d.CustID, s.termtype, d.IsInvoiced, d.CrtBy, d.CrtDate, d.UpdBy, d.UpdDate, s.DiscAmt, s.TaxAmt, s.excrate, s.freight, s.total, d.deliitype, s.subtotal, s.crcid, s.excrate, s.fisrate
	FROM Deli d, DeliD dd, SO s
	WHERE d.DeliNo=dd.DeliNo AND dd.SONo=s.SONo AND d.DeliNo=_DeliNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=5000;
		_retval.ErrMsg:='Pengiriman '||_DeliNo||' tidak ditemukan! (spDeli_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='DELI';
	IF(_LockDate is not null AND _LockDate>_DeliDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Pengiriman Melebihi Batas Tanggal Kunci Transaksi ! (spDeli_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	--  cek itype untuk tentukan in/out ---
	IF (_DeliIType='DELI') THEN
		_InOut:='O';
	ELSIF (_DeliIType='SRET') THEN
		_InOut:='I';	
	END IF;

	IF (_DeliIType='DELI' AND _Subtotal<0) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Qty Pengiriman tidak boleh Negatif!! (spDeli_New)';			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_DeliIType='SRET' AND _Subtotal>0) THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Penerimaan Barang Retur Qty harus Negatif! (spDeli_New)';			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

/** ////////////////////// AKUNTANSI /////////////////////////////// **/

	IF (isAccInstalled()) THEN
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:= 'Pengiriman ' || fBPName(_CustID) ||':'||_DeliNo;
		
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'DELI', _DeliDate, 'DELI', _DeliNo,
			_JnlNote, TRUE, TRUE, CAST(0 as NUMERIC(19,4)), _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_Count:=0;


		--- Ayat Unbilled Delivery HANYA untuk item yang stocked=TRUE ---------
		SELECT INTO _RowCount COUNT(*) 
		FROM DeliD d 
		JOIN Item i ON d.ItemID=i.ItemID 
		WHERE DeliNo=_DeliNo AND i.isStock=TRUE;

		IF (_RowCount>0) THEN
			SELECT INTO 
				_UBDeliAmt 
				SUM(CAST(ROUND(fUnitCost(d.ItemID, WhID, PID, (fQtyZtoQty1(d.ItemID, Qty, Unit) - QtyDRet), fUnit1(d.ItemID)), _Round) AS NUMERIC(19,4)))
			FROM DeliD d JOIN Item i ON d.ItemID=i.ItemID 
			WHERE DeliNo=_DeliNo AND i.isStock=TRUE;

			_Count:=_Count+1;
			IF (_DeliIType='SRET') THEN
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBSRET'), 'D', _UBDeliAmt, _DeptID, _PrjID, _UBDeliAmt);
			ELSIF (_DeliIType='DELI') THEN
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBDELI'), 'D', _UBDeliAmt, _DeptID, _PrjID, _UBDeliAmt);	
			END IF;

		END IF;

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		
		--------- Ayat INVENTORY untuk Item yang stocked -------------------
		OPEN cDeliD FOR
			SELECT 
			SUM(CAST(ROUND(fUnitCost(d.ItemID, WhID, PID, (fQtyZtoQty1(d.ItemID, Qty, Unit) - QtyDRet), fUnit1(d.ItemID)), _Round) AS NUMERIC(19,4))),
			fItemAcc(d.ItemID,'INV'), d.DeptID, d.PrjID
			FROM DeliD d 
			JOIN Item i ON d.ItemID=i.ItemID
			WHERE DeliNo=_DeliNo AND i.isStock=TRUE
			GROUP BY fItemAcc(d.ItemID,'INV'), d.DeptID, d.PrjID;
		LOOP
			FETCH cDeliD INTO _StockADCost, _AccInv, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;
			_Count:=_Count+1;

			SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccInv, 'K', _StockADCost, _DeptID, _PrjID, _StockADCost/_Excrate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cDeliD;

		----------- Ayat Unbilled-Service untuk Item yang Non-Stocked --------------

		SELECT INTO _RowCount COUNT(*) FROM DeliD d JOIN Item i ON d.ItemID=i.ItemID WHERE DeliNo=_DeliNo AND i.isStock=FALSE;

		IF (_RowCount>0) THEN

			--- Ayat Unbilled Deli/SRet -------

			SELECT INTO _UBDeliAmt, _BSubtotal 
				SUM(CAST(ROUND((sd.ListPrice-sd.DiscAmt)*(fQtyZtoQty1(d.ItemID, d.Qty, d.Unit) - QtyDRet)/d.Conv, _Round) AS NUMERIC(19,4))), 
				SUM(CAST(ROUND((sd.ListPrice-sd.DiscAmt)*_ExcRate*(fQtyZtoQty1(d.ItemID, d.Qty, d.Unit) - QtyDRet)/d.Conv, _Round) AS NUMERIC(19,4)))
			FROM DeliD d 
			JOIN SOD sd ON sd.SONo=d.SONo AND sd.SODNo=d.SODNo
			JOIN Item i ON d.ItemID=i.ItemID
			WHERE d.DeliNo=_DeliNo AND i.isStock=FALSE;

			IF FOUND THEN
				_Count:=_Count+1;
				IF (_DeliIType='SRET') THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBSRET'), 'D', _BSubtotal, _DeptID, _PrjID, _UBDeliAmt);
				ELSIF (_DeliIType='DELI') THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBDELI'), 'D', _BSubtotal, _DeptID, _PrjID, _UBDeliAmt);	
				END IF;
			END IF;

			--- Ayat UB Deli Service -------
			_AccUBDeliSvc:=fAccDefault('UBDELISVC');

			OPEN cDeliD FOR
			SELECT 
				SUM(CAST(ROUND((sd.ListPrice-sd.DiscAmt)*(fQtyZtoQty1(d.ItemID, d.Qty, d.Unit) - QtyDRet)/d.Conv, _Round) AS NUMERIC(19,4))), 
				SUM(CAST(ROUND((sd.ListPrice-sd.DiscAmt)*_ExcRate*(fQtyZtoQty1(d.ItemID, d.Qty, d.Unit) - QtyDRet)/d.Conv, _Round) AS NUMERIC(19,4))), d.DeptID, d.PrjID
			FROM DeliD d 
			JOIN SOD sd ON sd.SONo=d.SONo AND sd.SODNo=d.SODNo
			JOIN Item i ON d.ItemID=i.ItemID
			WHERE d.DeliNo=_DeliNo AND i.isStock=FALSE
			GROUP BY d.DeptID, d.PrjID;
		
			LOOP
				FETCH cDeliD INTO _StockADCost, _BSubtotal, _DeptID, _PrjID;
				EXIT WHEN NOT FOUND;
				_Count:=_Count+1;

				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccUBDeliSvc, 'K', _BSubtotal, _DeptID, _PrjID, _StockADCost);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
			CLOSE cDeliD;
		END IF;


		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;	
		

	END IF;
	
	-- Update stocknya pake qtydeli ---
	OPEN cDeliD FOR
		SELECT 
			d.DeliDNo, d.ItemID, d.WhID, d.PID, fQty1ToQtyZ(d.ItemID,(fQtyZtoQty1(d.ItemID, d.qty, d.unit)-d.QtyDRet),d.unit), d.qtyx, d.Unit, 
			sd.ListPrice, sd.DiscExp, sd.DiscAmt, sd.TaxID, sd.TaxAmt, d.SONo, d.DeONo, i.isSale, d.DeptID, d.PrjID
		FROM DeliD d
		JOIN SOD sd ON sd.SONo=d.SONo AND sd.SODNo=d.SODNo
		LEFT JOIN Item i ON i.itemid=d.ItemID
		WHERE d.DeliNo=_DeliNo AND i.isStock=TRUE;

		_Count:=1;
		LOOP
			FETCH cDeliD INTO
				_DeliDNo, _itemid, _WhID, _PID, _Qty, _QtyX, _Unit,
				_ListPrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _SONo, _DeONo, _isSale, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;
			
			IF (_isSale=FALSE) THEN
				_retval.ErrNo:=50005;
				_retval.ErrMsg:='Item:' || fItemDesc(_ItemID) || '(' || _ItemID || ') tidak di-Set untuk Penjualan (spDeli_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_count=1) THEN
				_StockADNo:=0;
				_StockANote:='Pengiriman #'||_delino;

				_StockANo:=fTransNoGet('STOCKA');
				IF (_StockANo IS NULL) THEN
					_retval.ErrNo:=3;
					_retval.ErrMsg:='ERROR getting StockANo (spDeli_New)';			
					RAISE EXCEPTION '%', _retval.ErrMsg;				
				END IF;
				
				SELECT INTO _spret * FROM spStockA_New(
					_StockANo, _CustID, 'DELI', _DeliDate, 'O', 'DELI', _DeliNo, 
					_StockANote, TRUE, _CrtBy, CAST(_CrtDate AS DATE), _UpdBy, CAST(_UpdDate AS DATE),
					_PerID, _BranchID, _DeptID, _PrjID
				);

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;			
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
															
				_count:=_count+1;
			END IF;
			
			-- read current Cost SubTotal, calc unit cost
			_CostSubTotal:=fUnitCost(_ItemID,_WhID,_PID, _Qty, _Unit);
			_Conv:=fItemUnit_Conv(_ItemID, _Unit);

			IF (_Qty>0) THEN
				_Cost:=_CostSubTotal/_Qty;
			ELSE
				_Cost:=0;
			END IF;

			-- Update Cost & Conversi			
			UPDATE DeliD SET 
				Conv=_Conv, Cost=_Cost
			WHERE DeliNo=_DeliNo AND DeliDNo=_DeliDNo;

			IF NOT FOUND THEN
				_retval.ErrNo:=50056;
				_retval.ErrMsg:='Error UPDATE DeliD:Conv (spDeli_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_PID IS NULL) THEN
				_PID:='';
			END IF;
			
			_StockADNo:=_StockADNo+1;

			SELECT INTO _spret * FROM spStockAD_New(
				_DeliDate, 'DELI', _StockANo, _StockADNo, _WhID, 
				_ItemID, _PID, _Qty*(-1), _Unit, _QtyX*(-1), _Cost, _DeptID, _PrjID, TRUE, _DeliNo, _CostSubTotal*-1
			);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;			
												
		END LOOP;
	CLOSE cDeliD;


/**** Process DeliD yang Non-Stocked ****/
	OPEN cDeliD FOR
		SELECT 
			d.DeliDNo, d.ItemID, d.WhID, d.PID, fQtyZtoQty1(d.ItemID, d.qty, d.unit)-d.QtyDRet, d.qtyx, d.Unit,
			sd.ListPrice, sd.DiscExp, sd.DiscAmt, sd.TaxID, sd.TaxAmt, d.SONo, d.DeONo, i.isSale, d.DeptID, d.PrjID
		FROM DeliD d
		JOIN SOD sd ON sd.SONo=d.SONo AND sd.SODNo=d.SODNo
		LEFT JOIN Item i ON i.itemid=d.ItemID
		WHERE d.DeliNo=_DeliNo AND i.isStock=FALSE;
		_Count:=1;
	LOOP
		FETCH cDeliD INTO 
			_DeliDNo, _itemid, _WhID, _PID, _qty, _qtyx, _Unit,
			_ListPrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _SONo, _DeONo, _isSale, _DeptID, _PrjID;

		EXIT WHEN NOT FOUND;
		
		_Cost:=fUnitCost(_ItemID,_WhID,_PID, _Unit);
		_Conv:=fItemUnit_Conv(_ItemID, _Unit);

	
		UPDATE DeliD SET 
			Cost=_Cost,
			Conv=_Conv
		WHERE DeliNo=_DeliNo AND DeliDNo=_DeliDNo;

	END LOOP;
	CLOSE cDeliD;	


	--------------- UPDATE SO KLO LANGSUNG ADA SO --------------
	OPEN cDeliD2 FOR
		SELECT 
			ItemID, SONo, SODNo, PID, fQtyZtoQty1(ItemID, Qty, unit) - QtyDRet
		FROM DeliD
		WHERE 
			DeliNo=_DeliNo 
			AND (SONo IS NOT NULL AND SONo<>'');
	LOOP
		FETCH cDeliD2 INTO _ItemID, _SONo, _SODNo, _DeliDPID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Rcv * FROM spSO_Item_Delivered(_SONo, _ItemID, _DeliDPID, _Qty1, _SODNo);
		
		IF (_Qty1Rcv<_Qty1) THEN
			_retval.ErrNo:=50018;
			_retval.ErrMsg:='Item:'||_ItemID||', Qty Pengiriman > QtySO '||_Qty1Rcv||' vs '||_Qty1||' (spDeli_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cDeliD2;			

	--------------- Check & Close SO ------------------
	OPEN cSO FOR
		SELECT SONo FROM DeliD
		WHERE DeliNo=_DeliNo 
		AND (SONo IS NOT NULL AND SONo<>'')
		ORDER BY SONo;
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;
		
		SELECT INTO _CustID2 CustID FROM SO WHERE SONo=_SONo;
		IF (_CustID<>_CustID2) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Customer di Order-Penjualan vs Pengiriman tidak sama (spDeli_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spSO_ChNUp_Status(_SONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSO;

	---- Set Deo Inactive ----
	OPEN cDeO FOR
		SELECT DISTINCT(DeONo) FROM DeliD 
		WHERE DeliNo=_DeliNo 
		AND (DeliNo IS NOT NULL AND DeliNo<>'')
		ORDER BY DeONo;
	LOOP
		FETCH cDeO INTO _DeONo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _CustID2 CustID FROM DeO WHERE DeONo=_DeONo;
		IF (_CustID<>_CustID2) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Customer DeO dan Deli tidak sama (spDeli_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- change DeO status here 
		UPDATE Deo set Active=FALSE WHERE deono=_DeONo;
	END LOOP;
	CLOSE cDeO;


	------------- Update Deli status ----------
	SELECT INTO _spret * FROM spDeli_ChNUp_Status(_DeliNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDeli_ReOpen (
	_DeliNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_DelidNo	SMALLINT;
	_DeliStatus	VARCHAR;
	_ItemID		VARCHAR(20);
	_WhID		VARCHAR(3);
	_Qty		NUMERIC(15,4);
	_Qtydret	NUMERIC(15,4);
	_QtyDeli	NUMERIC(15,4);
	_Qtydret_New	NUMERIC(15,4);
	cDelid		REFCURSOR;
BEGIN
	_retval:=(5000, 'Error (spDeli_ReOpen)');

	SELECT INTO _DeliStatus delistatus FROM deli WHERE delino=_DeliNo;
	IF (_DeliStatus='F') THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Sudah Terkirim Penuh! Pengiriman tidak dapat dibuka kembali (spPReq_ReOpen)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cDelid FOR
		SELECT delidno, whid, itemid, fQtyZtoQty1(itemid, qty, unit), qtydret, qtydeli
		FROM delid
		WHERE (delino=_DeliNo) AND (qtydret>0);
	LOOP
		FETCH cDelid INTO _DelidNo, _WhID, _ItemID, _Qty, _Qtydret, _QtyDeli;
		EXIT WHEN NOT FOUND;

		_Qtydret_New:=_Qtydret;

		UPDATE delid SET qtydret=qtydret - _Qtydret_New
		WHERE delino=_DeliNo AND delidno=_DelidNo;
	END LOOP;
	CLOSE cDelid;

	SELECT INTO _spret * FROM spDeli_ChNUp_Status(_DeliNo);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	_retval:=(0, 'Buka Deli Berhasil!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spDeli_Void(_DeliNo VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_IsInvoiced	BOOLEAN;
	_JnlNo		VARCHAR(15);
	_DeONo		VARCHAR;
	
	_ItemID		VARCHAR(20);
	_DeliDPID	VARCHAR;
	_Qty1			DECIMAL(15,4);
	_SONo			VARCHAR(15);	
	_QtyResult	DECIMAL(15,4);
	
	cDeliD	refcursor;
	cSO		refcursor;
	cDeO		refcursor;
	
	_isDraft	BOOLEAN;
	_DeliDate	DATE;
	_LockDate	DATE;
	_SODNo		int;
BEGIN
	SELECT INTO 
		_IsInvoiced, _DeliDate, _isDraft 
		IsInvoiced, DeliDate, isDraft
	FROM Deli
	WHERE DeliNo=_DeliNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Delivery #'||_DeliNo||' tidak ditemukan! (spDeli_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='DELI';
	IF(_LockDate is not null AND _LockDate>_DeliDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Delivery Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_IsInvoiced=true) THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Delivery #'||_DeliNo||' ini sudah di Invoice, Void dulu Invoicenya untuk melanjutkan (spDeli_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		DELETE FROM SODeli WHERE DeliNo=_DeliNo;
		DELETE FROM DeoDeli WHERE DeliNo=_DeliNo;
		DELETE FROM DeliD WHERE DeliNo=_DeliNo;
		DELETE FROM Deli WHERE DeliNo=_DeliNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	OPEN cDeliD FOR
		SELECT SONo, SODNo, ItemID, PID, fQtyZToQty1(itemid, qty, unit)-qtydret
		FROM DeliD WHERE DeliNo=_DeliNo AND SONo IS NOT NULL AND SONo<>'';		
	LOOP
		FETCH cDeliD INTO _SONo, _SODNo, _ItemID, _DeliDPID, _Qty1;
		EXIT WHEN NOT FOUND;
	
		SELECT INTO _QtyResult * FROM spSO_Item_Cancel(_SONo, _ItemID, _DeliDPID, _Qty1, _SODNo);
		IF (_QtyResult<>_Qty1) THEN
			_retval.ErrNo:=50018;
			_retval.ErrMsg:='Error Rollback Qty SO, Item:'|| _ItemID ||'='||CAST(_QtyResult AS VARCHAR)||' Qty1:'||CAST(_Qty1 AS VARCHAR)||' (spDeli_Void)';
			RAISE EXCEPTION '%', _retval.ErrMsg;			
		END IF;
			
	END LOOP;
	CLOSE cDeliD;

	OPEN cSO FOR 
		SELECT DISTINCT SONo FROM DeliD 
		WHERE DeliNo=_DeliNo 
		AND (SONo IS NOT NULL AND SONo<>'')
		ORDER BY SONo;
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;
	
		SELECT into _spret * from spSO_ChNUp_Status (_SONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSO;



/*------------------------ UNUSED ------------------------------ */
/*	OPEN cSO FOR
		SELECT SONo FROM SODeli WHERE DeliNo=_DeliNo ORDER BY SONo;
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spSO_ChNUp_Status(_SONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSO;

	OPEN cDeO FOR
		SELECT SONO 
		FROM DeODeli LEFT JOIN SODEO ON DEODELI.DEONO=SODEO.DEONO 
		WHERE DeliNo=_DeliNo ORDER BY DEODELI.DeONo;
	LOOP
		FETCH cDeO INTO _SONo;
		EXIT WHEN NOT FOUND;
	
		SELECT INTO _spret * FROM spSO_ChNUp_Status(_SONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cDeO;*/


/*** update status DeO ****/
	OPEN cDeO FOR
		SELECT DeONo FROM Delid WHERE DeliNo=_DeliNo ORDER BY DeONo;
	LOOP
		FETCH cDeO INTO _DeONo;
		EXIT WHEN NOT FOUND;
	
	

		/** change DeO status here **/
		UPDATE deo SET active=true WHERE deono=_DeONo;
	END LOOP;
	CLOSE cDeO;


	SELECT into _spret * FROM spStockA_Void_byRef('DELI', _DeliNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	
	IF (isAccInstalled()) THEN
		SELECT into _spret * from spJnl_Void_byRef('DELI',_DeliNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM SODeli WHERE DeliNo=_DeliNo;
	DELETE FROM DeoDeli WHERE DeliNo=_DeliNo;
	DELETE FROM DeliD WHERE DeliNo=_DeliNo;
	DELETE FROM Deli WHERE DeliNo=_DeliNo;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spDeO_New(_DeONo VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_ItemID		VARCHAR(20);
	_PID		VARCHAR(25);
	_Qty1		NUMERIC(15,4);
	_Qty1Rcv	NUMERIC(15,4);

	_SONo		VARCHAR(15);
	_SODNo		INT;
	_DeoDPID	VARCHAR;
	_isDraft	BOOLEAN;

	cDeoD		refcursor;

BEGIN	

	SELECT INTO 
		_isDraft isDraft
	FROM Deo
	WHERE DeONo=_DeONo;

	IF NOT FOUND THEN
		_retval.ErrNo:=5000;
		_retval.ErrMsg:='Order Pengiriman '||_DeONo||' tidak ditemukan! (spDeO_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	--------------- UPDATE SO KLO LANGSUNG ADA SO --------------
	OPEN cDeoD FOR
		SELECT 
			ItemID, SONo, SODNo, PID, fQtyZtoQty1(ItemID, Qty, unit)
		FROM DeoD
		WHERE 
			DeONo=_DeONo 
			AND (SONo IS NOT NULL AND SONo<>'');
	LOOP
		FETCH cDeoD INTO _ItemID, _SONo, _SODNo, _DeoDPID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Rcv * FROM spSO_Item_DO(_SONo, _ItemID, _DeoDPID, _Qty1, _SODNo);
		
		IF (_Qty1Rcv<_Qty1) THEN
			_retval.ErrNo:=50018;
			_retval.ErrMsg:='Item: '||_ItemID||', Qty DeO > QtySO '||_Qty1Rcv||' vs '||_Qty1||' (spDeO_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cDeoD;			

	_retval:=(0,'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION spDeO_Void(_DeONo VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_count		INT;
	_retval		SPStatus%ROWTYPE;

	_ItemID		VARCHAR(20);
	_DeoDPID	VARCHAR;
	_Qty1		DECIMAL(15,4);
	_SONo		VARCHAR(15);
	_SODNo		int;
	_QtyResult	DECIMAL(15,4);
	_isDraft	BOOLEAN;

	cDeoD		refcursor;

BEGIN
	SELECT INTO 
		_isDraft isDraft
	FROM Deo
	WHERE DeONo=_DeONo;

	IF NOT FOUND THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Order Pengiriman #'||_DeliNo||' tidak ditemukan! (spDeO_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _count COUNT(*) FROM DeoDeli WHERE DeONo=_DeONo;
	IF _count>0 THEN
		_retval:=(1, 'Sudah ada Delivery untuk transaksi D.O ini!(spDeO_Void)');
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO _count COUNT(*) FROM SODeo WHERE DeONo=_DeONo;
	IF _count>0 THEN
		DELETE FROM SODeo WHERE DeONo=_DeONo;
		IF NOT FOUND THEN
			_retval:=(1, 'Error DELETE SoDeO (spDeO_Void)');
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	IF (_isDraft IS NULL OR _isDraft=FALSE) THEN
		OPEN cDeoD FOR
			SELECT SONo, SODNo, ItemID, PID, fQtyZToQty1(itemid, qty, unit)
			FROM DeoD WHERE DeoNo=_DeONo AND SONo IS NOT NULL AND SONo<>'';		
		LOOP
			FETCH cDeoD INTO _SONo, _SODNo, _ItemID, _DeoDPID, _Qty1;
			EXIT WHEN NOT FOUND;
		
			SELECT INTO _QtyResult * FROM spSO_Item_Cancel_DO(_SONo, _ItemID, _DeoDPID, _Qty1, _SODNo);
			IF (_QtyResult<>_Qty1) THEN
				_retval.ErrNo:=50018;
				_retval.ErrMsg:='Error Rollback Qty SO, Item:'|| _ItemID ||'='||CAST(_QtyResult AS VARCHAR)||' Qty1:'||CAST(_Qty1 AS VARCHAR)||' (spDeO_Void)';
				RAISE EXCEPTION '%', _retval.ErrMsg;			
			END IF;
				
		END LOOP;
		CLOSE cDeoD;
	END IF;


	DELETE FROM DeOD WHERE DeONo=_DeONo;
	IF NOT FOUND THEN
		_retval:=(1, 'Error DELETE DeOD (spDeO_Void)');
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	DELETE FROM DeO WHERE DeONo=_DeONo;
	IF NOT FOUND THEN
		_retval:=(1, 'Error DELETE DeO (spDeO_Void)');
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0,'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



DROP TYPE tJnlPacketInv CASCADE;
CREATE TYPE tJnlPacketInv AS (
	accno	VARCHAR,
	accamt	NUMERIC(19,4),
	deptid	VARCHAR,
	prjid	VARCHAR
);


CREATE OR REPLACE FUNCTION spJnlPacketInv(
	_SaleNo	VARCHAR
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval	tJnlPacketInv%ROWTYPE;
	
	cData	REFCURSOR;
BEGIN
	OPEN cData FOR
		SELECT accno, deptid, prjid, SUM(accamt) 
		FROM spJnlPacketInv(_SaleNo, 'SALE') 
		GROUP BY accno, deptid, prjid;
	LOOP
		FETCH cData INTO _retval.accno, _retval.deptid, _retval.prjid, _retval.accamt;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION spJnlPacketInv(
	_SaleNo	VARCHAR,
	_Type	VARCHAR
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval	tJnlPacketInv%ROWTYPE;
	
	cData	REFCURSOR;
BEGIN
	OPEN cData FOR
		SELECT accno, deptid, prjid, SUM(accamt) 
		FROM spJnlPacketInv(_SaleNo, _Type, FALSE) 
		GROUP BY accno, deptid, prjid;
	LOOP
		FETCH cData INTO _retval.accno, _retval.deptid, _retval.prjid, _retval.accamt;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spJnlPacketInv ( 
	_SaleNo		VARCHAR,
	_Type		VARCHAR,
	_CheckDeli	BOOLEAN
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval	tJnlPacketInv%ROWTYPE;
	
	cSale	REFCURSOR;
	cData	REFCURSOR;

	_WhID	VARCHAR;
	_ItemID	VARCHAR;
	_PID	VARCHAR;
	_Qty	NUMERIC(19,4);
	_Unit	VARCHAR;
	_DeptID	VARCHAR;
	_PrjID	VARCHAR;
BEGIN

	IF (_Type='SALE') THEN
		OPEN cSale FOR
			SELECT d.whid, d.itemid, d.pid, d.qty, d.unit, d.deptid, d.prjid 
			FROM saled d JOIN item i ON d.itemid=i.itemid 
			WHERE i.itemtype='PACK' AND saleno=_SaleNo
			AND ((_CheckDeli=FALSE AND (d.delino IS NULL OR d.delino='')) 
				OR (_CheckDeli=TRUE AND d.delino IS NOT NULL));
		LOOP
			FETCH cSale INTO _WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			OPEN cData FOR
				SELECT * FROM spJnlPacketInv(_WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID);
			LOOP
				FETCH cData INTO _retval;
				EXIT WHEN NOT FOUND;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cData;
		END LOOP;
		CLOSE cSale;
	ELSIF (_Type='BNS') THEN
		OPEN cSale FOR
			SELECT d.whid, d.itemid, d.pid, d.qty, d.unit, NULL, NULL 
			FROM salebns d JOIN item i ON d.itemid=i.itemid 
			WHERE i.itemtype='PACK' AND saleno=_SaleNo;
		LOOP
			FETCH cSale INTO _WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			OPEN cData FOR
				SELECT * FROM spJnlPacketInv(_WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID);
			LOOP
				FETCH cData INTO _retval;
				EXIT WHEN NOT FOUND;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cData;
		END LOOP;
		CLOSE cSale;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spJnlPacketInv_Deli ( 
	_DeliNo		deli.delino%TYPE
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval	tJnlPacketInv%ROWTYPE;
	
	cDeli	REFCURSOR;
	cData	REFCURSOR;

	_WhID	wh.whid%TYPE;
	_ItemID	item.itemid%TYPE;
	_PID	VARCHAR;
	_Qty	delid.qty%TYPE;
	_Unit	delid.unit%TYPE;
	_DeptID	dept.deptid%TYPE;
	_PrjID	prj.prjid%TYPE;
BEGIN
	OPEN cDeli FOR
		SELECT d.whid, d.itemid, d.pid, d.qty, d.unit, d.deptid, d.prjid 
		FROM delid d 
		JOIN item i ON d.itemid=i.itemid 
		WHERE i.itemtype='PACK' AND delino=_DeliNo;
	LOOP
		FETCH cDeli INTO _WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		OPEN cData FOR
			SELECT * FROM spJnlPacketInv(_WhID, _ItemID, _PID, _Qty, _Unit, _DeptID, _PrjID);
		LOOP
			FETCH cData INTO _retval;
			EXIT WHEN NOT FOUND;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cData;
	END LOOP;
	CLOSE cDeli;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spJnlPacketInv (
	__WhID		VARCHAR,
	__ItemID	VARCHAR,
	__PID		VARCHAR,
	__Qty		NUMERIC(19,4),
	__Unit		VARCHAR,
	__DeptID	VARCHAR,
	__PrjID		VARCHAR
) RETURNS SETOF tJnlPacketInv AS
$BODY$
DECLARE
	_retval		tJnlPacketInv%ROWTYPE;

	_AccNo		VARCHAR;
	_AccAmt		NUMERIC(19,4);

	_BomID		VARCHAR;
	_BomQty		NUMERIC(19,4);
	_BomUnit	VARCHAR;

	_ItemID		VARCHAR;
	_Qty		NUMERIC(19,4);
	_Unit		VARCHAR;
	_PID		VARCHAR;
	
	_BomMultiplier	NUMERIC(19,4);

	cBom		REFCURSOR;
	cData		REFCURSOR;
BEGIN

	SELECT INTO _BomID, _BomQty, _BomUnit bomid, qty, unit FROM bom WHERE itemid=__ItemID AND pid=COALESCE(__PID, '') AND active=TRUE;
	IF FOUND THEN
		_BomQty:=fQtyZToQty1(__ItemID, _BomQty, _BomUnit);
		_BomMultiplier:=fQtyZtoQty1(__ItemID, __Qty, __Unit)/_BomQty;

		OPEN cBom FOR
			SELECT d.itemid, d.qty, d.unit, COALESCE(d.pid, ''), i.accinv 
			FROM bomd d JOIN item i ON d.itemid=i.itemid  
			WHERE bomid=_BoMID;
		LOOP
			FETCH cBom INTO _ItemID, _Qty, _Unit, _PID, _AccNo;
			EXIT WHEN NOT FOUND;

			SELECT INTO _BomID bomid FROM bom WHERE itemid=_ItemID AND pid=_PID AND active=TRUE;
			IF FOUND THEN
				OPEN cData FOR
					SELECT * FROM spJnlPacketInv(__WhID, _ItemID, _PID, _Qty*_BomMultiplier, _Unit, __DeptID, __PrjID);
				LOOP
					FETCH cData INTO _retval;
					EXIT WHEN NOT FOUND;

					RETURN NEXT _retval;
				END LOOP;
				CLOSE cData;
			ELSE
				_AccAmt:=fUnitCost(_ItemID, __WhID, _PID, _Qty, _Unit);
				_retval.accno:=_AccNo;
				_retval.accamt:=_AccAmt*_BomMultiplier;
				_retval.deptid:=__DeptID;
				_retval.prjid:=__PrjID;

				RETURN NEXT _retval;
			END IF;
		END LOOP;
		CLOSE cBom;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/************
Kerangka:

	CHECK LOCK, DRAFT, VALIDASI
		check cash, credit limit

	GENERATE StockA 
		stock
		bonus
		paket

	PROSES Non-Stock


*************/



CREATE OR REPLACE FUNCTION spSale_New(_SaleNo VARCHAR(15)) 
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CustID			VARCHAR(15);
	_CustID2		VARCHAR(15);
	_SaleType		VARCHAR(3);
	_SaleDate		DATE;
	_SONo			VARCHAR(15);
	_TermType		VARCHAR(1);
	_Term			INT;
	_PoSID			VARCHAR(5);
	_PoSSesNo		VARCHAR(8);

	_isTaxed		BOOLEAN;
	_isTaxInc		BOOLEAN;

	_SubTotal		DECIMAL(19,4);
	_Total			DECIMAL(19,4);
	_MDiscAmt		NUMERIC(19,4);
	_PaidAmt		DECIMAL(19,4);
	_CrtBy			VARCHAR(10);
	_CrtDate		TIMESTAMP;
	_UpdBy			VARCHAR(10);
	_UpdDate		TIMESTAMP;
	_DeliNo			VARCHAR(15);
	_SaleDNo		INT;
	_SaleNo2		VARCHAR(15);

	_StockAType		VARCHAR(5):='SALE';
	_isInsert		BOOLEAN:=true;

	_StockANo		VARCHAR(15);
	_StockANote		VARCHAR(255);
	_StockADNo		SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_Qty			DECIMAL(15,4);
	_Unit			VARCHAR(5);
	_Conv			DECIMAL(15,4);
	_QtyX			DECIMAL(15,4);
	_BasePrice		DECIMAL(19,4);
	_ListPrice 		DECIMAL(19,4);
	_DiscExp		VARCHAR(15);
	_DiscAmt		DECIMAL(19,4);
	_TaxID			VARCHAR(5);
	_TaxAmt			DECIMAL(19,4);
	_MTaxAmt		DECIMAL(19,4);
	_Cost			DECIMAL(19,4);
	_CostSubTotal		DECIMAL(19,4);

	_ItemType		VARCHAR(5);
	_isStock		BOOLEAN;
	_BoMID			bom.bomid%TYPE;
	_Qty_Unit1		DECIMAL(15,4);
	_CashID			VARCHAR(15);
	_CashAAmt		DECIMAL(19,4);
	_CashANote		VARCHAR(255);
	_DebtNote		VARCHAR(255);
	_AllowCredit		DECIMAL(19,4);
	_CreditLimit		DECIMAL(19,4);
	_countCredit		DECIMAL(19,4);
	_CreditAmount		DECIMAL(19,4);
	_CrcID			VARCHAR(5);

	_ContBal 		DECIMAL(19,4);
	_RcvNo			VARCHAR;
	_EmpId			VARCHAR;
	_ConsNo			VARCHAR(15);
	_ConsDNo			SMALLINT;
/* SO Stuff ****/
	_Qty1 			DECIMAL(15,4);
	_Qty1Rcv 		DECIMAL(15,4);

/** Bonus **/
	_StockANoBns 		VARCHAR(15);
	_StockANoteBns		VARCHAR(255);
	_StockADNoBns		INT2;
	_ItemIDBns		VARCHAR(20);
	_WhIDBns		VARCHAR(3);
	_PIDBns			VARCHAR(25);
	_QtyBns			DECIMAL(15,4);
	_QtyXBns		DECIMAL(15,4);
	_UnitBns		VARCHAR(5);
	_CostBns		DECIMAL(19,4);
	_ExcRate		DECIMAL(19,4);

	_AccAsset		VARCHAR(10);
	_AccDisc		VARCHAR(10);
	_AccInv			VARCHAR(10);
	_AccCOGS		VARCHAR(10);
	_AccFreight		VARCHAR(10);
	_AccSale		VARCHAR(10);
	_AccAmt			DECIMAL(19,4);
	_AccAmtForex		NUMERIC(19,4);
	_AccAR			VARCHAR;

	_AccUBDeliSvc		VARCHAR(10);

	_Freight		DECIMAL(19,4);
	_Others			DECIMAL(19,4);

	_count			INT;
	_count1			INT;
	_count2			INT;
	_count3			INT;
	_count4			INT;
	_count5			INT;

	_JnlNo1			VARCHAR(15);
	_JnlNo2			VARCHAR(15);
	_JnlNo3			VARCHAR(15);
	_JnlNo4			VARCHAR(15);
	_JnlNo5			VARCHAR(15);
	_JnlNote		VARCHAR;
	_UBDeliAmt		DECIMAL(19,4);
	_UBDeliAmtForex		DECIMAL(19,4);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);
	_Round			INT;
	_DeliItemCount		INT;
	cDeliSale		REFCURSOR;
	cSaleD 			REFCURSOR;
	cSaleD_Service 		REFCURSOR;
	cSO			REFCURSOR;
	cSaleBns		REFCURSOR;
	cSaleAcc		REFCURSOR;
	cSaleCons		REFCURSOR;
	cSaleCRcv		REFCURSOR;
	cCons			REFCURSOR;
	cSaleD1			REFCURSOR;
	cTax			REFCURSOR;
	cData			REFCURSOR;

/** POS/SaleCRcv stuff ***/
	_RcvType		VARCHAR;
	_EDCID			VARCHAR;
	_CashAmt		VARCHAR;
	_ChangeAmt		VARCHAR;
	_RcvAmt			VARCHAR;

	_CardNo			VARCHAR;
	_TRefNo			VARCHAR;


	_countCons		INT;
	_countDeli		INT;
	_countNormal	INT;
	_countBonus		INT;
	_countBPTYPE	INT;

	_isDraft		BOOLEAN;
	_JnlType		VARCHAR;
	_LockDate		DATE;
	_SODNo			INT;

	_SumD			NUMERIC(19,4);
	_SumKDeli		NUMERIC(19,4);
	_SumK			NUMERIC(19,4);

	_ItemIDCheck		VARCHAR;

	_countPID		INT2;
	_SaleDTaxAmt		NUMERIC(19,4);
	_AccSaleTax		VARCHAR;
	_TotalTax		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);
	_SaleDTaxAmtForex	NUMERIC(19,4);
	_UBDeliSvcAmt		NUMERIC(19,4);
	_UBDeliSvcAmtForex	NUMERIC(19,4);
	_countSO		INT;
	_CntDeli		INT;
	_CrcIDRcvD		VARCHAR;
	_ExcRateRcvD		NUMERIC(19,4);
	_FisRateRcvD		NUMERIC(19,4);

	_ItemsCost		NUMERIC(19,4);
	_SaleTotalMonth		NUMERIC(19,4);
	_tmp_va1		VARCHAR(1);
	_Qty1_Ordered		NUMERIC(15,4);

	_BTotal			NUMERIC(19,4); -- base currency total
	_BSubtotal		NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt		NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt		NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt		NUMERIC(19,4);
	_TotalTaxAmt		NUMERIC(19,4);
	_BTotalTaxAmt		NUMERIC(19,4);
	_TotalDiscAmt		NUMERIC(19,4);
	_TotalDisc2Amt		NUMERIC(19,4);
	_DSubtotal		NUMERIC(19,4);
	_DBSubtotal		NUMERIC(19,4);

	_AccRateLR		VARCHAR; -- akun laba/rugi selisih kurs
	_RateLR			NUMERIC(19,4); -- nilai laba/rugi selisih kurs

	_CashCrcID		VARCHAR;
	_SaleBnsNo		SMALLINT;
	_AccBns			VARCHAR(10);
	_InOut			CHAR(1);
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_InOut:='I';
	_retval:=(0, 'OK');

/**	_retval:=(0, 'OK');
	RETURN NEXT _retval;
	RETURN;
*/
	SELECT INTO 
		_isDraft, _PerID, _BranchID, _CustID, _SaleType, _SaleDate, _SONo, _TermType, _Term, _PoSID, _PoSSesNo, _CashID, 
		_BSubtotal, _SubTotal, _BTotal, _Total, _PaidAmt, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _EmpId, _ConsNo, _CrcID, 
		_AccFreight, _Freight, _AccDisc, _MBFTaxAmt, _MTaxAmt, _ExcRate,_Others,_AccAR, _FisRate, _isTaxed, 
		_isTaxInc, _MBDiscAmt, _MDiscAmt 

		isDraft, PerID, BranchID, CustID, SaleType, SaleDate, SONo, TermType, DueDays, PoSID, PoSSesNo, CashID, 
		basesubtotal, SubTotal, basetotal, Total, PaidAmt, CrtBy, CrtDate, UpdBy, UpdDate, EmpID, ConsNo, CrcID, 
		AccFreight, Freight, AccSDisc, basftaxamt, TaxAmt, ExcRate, others, AccAR, FisRate, isTaxed, 
		TaxInc, basediscamt, discamt 
	FROM Sale WHERE SaleNo=_SaleNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penjualan '|| _SaleNo||' tidak ditemukan ! (spSale_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/**************** SALE LIMIT *************/
	/*SELECT INTO _SaleTotalMonth * FROM fSaleTotalMonth(_SaleDate);

	IF (_SaleTotalMonth+_Total)>100000000 THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Total penjualan anda bulan ini '|| to_char(round(_SaleTotalMonth,2),'999G999G999D99') ||' . Batas maksimal penjualan per bulan adalah 100 juta !!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;*/
	/******************************************/

/******************************************************************************************************
	CHECK LOCK, DRAFT, VALIDASI
******************************************************************************************************/

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SALE';
	IF(_LockDate IS NOT NULL AND _LockDate>_SaleDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penjualan Melebihi Batas Tanggal Kunci Transaksi '|| CAST(_LockDate AS VARCHAR) || '!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	-------- count details FROM imported --------------
	SELECT INTO _countNormal COUNT(*) FROM SaleD WHERE SaleNo=_SaleNo AND (DeliNo IS NULL OR DeliNo='') AND (ConsNo IS NULL OR ConsNo='');
	SELECT INTO _countDeli COUNT(*) FROM SaleD WHERE SaleNo=_SaleNo AND DeliNo IS NOT NULL AND DeliNo<>'';
	SELECT INTO _countCons COUNT(*) FROM SaleD WHERE SaleNo=_SaleNo AND ConsNo IS NOT NULL AND ConsNo<>'';
	SELECT INTO _countBonus COUNT(*) FROM SaleBns WHERE SaleNo=_SaleNo;
	SELECT INTO _countBPTYPE COUNT(*) FROM bpbptype WHERE bpid=_CustID AND bptype='CUST';

	--------- cek apakah punya bptype customer (untuk jaga2 ketika import piutang lwt excel) ------------
	IF _countBPTYPE=0 THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg='BPID: '||_CustID||' tidak terdaftar sebagai customer (spSale_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	--------- cek & fix POS params ------------
	IF (_PoSSesNo IS NOT NULL) THEN
		IF NOT EXISTS(SELECT 1 FROM PoSSes WHERE PoSID=_PoSID AND StartTime IS NOT NULL AND EndTime IS NULL) THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg='Kasir: '||COALESCE(_PoSID,'<kosong>')||' tidak aktif (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	---------------- cek CashID terisi utk Tunai -------------
	IF (_TermType='C') THEN
		IF((_PoSSesNo IS NOT NULL) AND (_CashID IS NULL)) THEN
			SELECT INTO _CashID CashID FROM PoS WHERE PoSID=_PoSID;

			UPDATE Sale SET CashID=_CashID WHERE SaleNo=_SaleNo;
		END IF;

		IF((_CashID IS NULL) OR (_CashID='')) THEN
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Error, Kas untuk Penjualan Tunai belum diset (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _CashCrcID crcid FROM cash WHERE cashid=_CashID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' belum diset (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_CrcID!=_CashCrcID) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' tidak sama dengan mata uang penjualan (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	--------------- Cek kredit limit -----------------
	ELSIF (_TermType='R') THEN
		SELECT INTO _AllowCredit, _CreditLimit, _CreditAmount arlimitfreq, arlimitamt, arbalance FROM BPacc WHERE BPID=_CustID AND CrcID=_CrcID;
		SELECT INTO _countCredit count(*) FROM sale WHERE PayStatus<>'F' and custID=_CustID AND CrcID=_CrcID;
		IF (_AllowCredit=0) THEN
			_retval.ErrNo:=50005;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' tidak diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSEIF (_AllowCredit<>-1 AND _AllowCredit<_countCredit) THEN
			_retval.ErrNo:=50006;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' melebihi jumlah diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- Sebaiknya sudah di-Check di Applikasi agar bisa diconfirmasi ke user
		IF (_CreditLimit=0) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' tidak diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSEIF (_CreditLimit<>-1 AND _CreditLimit<(_CreditAmount+_Total) ) THEN
			_retval.ErrNo:=50008;
			_retval.ErrMsg:='Penjualan melebihi Limit Hutang (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

-------------- Check apakah item diset untuk dijual (isSale?) --------------
	OPEN cSaleD1 FOR
		SELECT saled.itemid FROM saleD JOIN item on saled.itemid=item.itemid WHERE issale=FALSE and saleno=_SaleNo;
		LOOP
			FETCH cSaleD1 INTO _ItemIDCheck;
			EXIT WHEN NOT FOUND;

			-- Kecuali : Item Begbal('0') boleh , kalau tipe nya beginning balance!
			IF NOT(_ItemIDCheck='0' AND _SaleType='B') THEN
				_retval.ErrNo:=50055;
				_retval.ErrMsg:='Error, ItemID:' || _ItemIDCheck || ' tidak di Set untuk Penjualan!! (spSale_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
	CLOSE cSaleD1;

	----------- Deli? check Customer sinkron dgn Salenya --------------
	IF (_countDeli>0) THEN
		OPEN cDeliSale FOR
			SELECT DISTINCT DeliNo FROM SaleD WHERE SaleNo=_SaleNo AND DeliNo IS NOT NULL AND DeliNo<>'';
		LOOP
			FETCH cDeliSale INTO _DeliNo;
			EXIT WHEN NOT FOUND;

			SELECT INTO _CustID2 CustID FROM Deli WHERE DeliNo=_DeliNo;
			IF (_CustID<>_CustID2) THEN
				_retval.ErrNo:=50019;
				_retval.ErrMsg:='Customer Delivery '|| _DeliNo ||' dan Penjualan tidak sama (spSale_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			UPDATE Deli SET IsInvoiced=true WHERE delino=_DeliNo;
		END LOOP;
		CLOSE cDeliSale;
	END IF;

	_StockADNo:=0;
	_count:=0;

/******************************************************************************************************
	Proses Stock / Generate StockA

kenapa kok gak di kasih blok IF (countdeli=0)-> karena dimungkinkan di Sale campur 
antara import dari Deli sama input sendiri

******************************************************************************************************/


/**
	Proses Item biasa:GOOD atau Paket:PACK
	BUKAN dari delivery atau Konsinyasi
		karena kalau import dari kedua transaksi diatas stocknya sudah di proses
	
**/

	OPEN cSaleD FOR
		SELECT 
			d.SaleDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, i.ItemType, i.isStock,
			d.DeptID, d.PrjID
		FROM SaleD d JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.SaleNo=_SaleNo 
			AND (i.isStock=TRUE OR i.ItemType='PACK')
			AND (d.DeliNo IS NULL OR d.DeliNo='') 
			AND (d.ConsNo IS NULL OR d.ConsNo='');
	LOOP
		FETCH cSaleD INTO _SaleDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ItemType, _isStock, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		----- Row-1 ? Generate header dulu 
		IF(_count=0) THEN
			_ItemsCost:=0;

			_StockANo:=fTransNoGet('STOCKA');
			_StockANote:='Penjualan #'|| _SaleNo;
			IF (_StockANo IS NULL) THEN
				_retval.ErrNo:=50009;
				_retval.ErrMsg:='Error getting StockANo (spSale_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spStockA_New(
				_StockANo, _CustID, 'SALE', _SaleDate, 'O', 'SALE', _SaleNo, _StockANote, 
				TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID
			); 

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		END IF;

		--- Calculate Cost(HPP)
		_StockANote:='Penjualan #'||_SaleNo;

		-- Hitung HPP secara Bulk utk sejumlah Qty biar lebih presisi motong stocknya
		_CostSubTotal:=fUnitCost(_ItemID, _WhID, _PID, _Qty, _Unit);

		IF (_Qty IS NOT NULL) THEN
			_Cost:=_CostSubTotal/_Qty;
		ELSE
			_Cost:=0;
		END IF;

		IF (_Cost IS NULL) THEN
			_retval.ErrNo:='50003';
			_retval.ErrMsg:='Error menghitung HPP! Item:'||COALESCE(_ItemID,'<kosong>')||', Wh:'||COALESCE(_WhID,'<kosong>')||', PID:'||COALESCE(_PID, '<kosong>')||', Unit:'||COALESCE(_Unit, '<kosong>')||' (fUnitCost)(spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_PID IS NULL) THEN
			_PID:='';
		END IF;

		---------------- PACK: Generate Stock-Details --------------------------
		--------- Item Normal ------------salebnsno

		IF (_ItemType<>'PACK' AND _ItemType<>'SERV' AND _isStock=TRUE) THEN
			_StockADNo:=_StockADNo+1;
						
			SELECT INTO _spret * FROM spStockAD_New(
					_SaleDate, 'SALE', _StockANo, _StockADNo, 
					_WhID, _ItemID, _PID, _Qty*-1, _Unit, 
					_QtyX*-1, _Cost, _DeptID, _PrjID, true, _SaleNo, _CostSubTotal*-1);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		ELSIF (_ItemType='PACK') THEN
			SELECT INTO _BoMID bomid FROM bom WHERE itemid=_ItemID AND active=TRUE;
			/**_StockADNo:=_StockADNo+1; dicomment karena sudah ada modifikasi di spstockad_new_bom*/

			IF(_BomID IS NULL) THEN
				_retval.ErrNo:='50014';
				_retval.ErrMsg='Error tidak dapat menemukan B.o.M. untuk Item:'||_ItemID||'(spSale_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spStockAD_New_BoM(
				_SaleDate, _StockAType, _StockANo, _StockADNo, _WhID, _ItemID, _PID, 
				_Qty*-1, _Unit, _QtyX*-1, _Cost, _DeptID, _PrjID, _CrcID
			);

			_StockADNo:=_spret.ErrNo;
			
			IF _spret.ErrNo<>0 AND _spret.ErrMsg<>'OK' THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		--- Read Qty conversion
		_Conv:=fItemUnit_Conv(_ItemID, _Unit);

		IF (_Conv IS NULL) THEN
			_retval.ErrNo:='50055';
			_retval.ErrMsg:='Error, Konversi ItemID:'||COALESCE(_ItemID, '<kosong>')||', Unit:'||COALESCE(_Unit, '<kosong>')||'!  (fItemUnit_Conv)(spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		--- Update Cost & QtyConversion
		UPDATE SaleD SET 
			Cost=_Cost, 
			Conv=_Conv 
		WHERE SaleNo=_SaleNo AND SaleDNo=_SaleDNo;

		_ItemsCost:=_ItemsCost+_CostSubTotal;
		_count:=_count+1;
	END LOOP;
	CLOSE cSaleD;

/******************************************************************************************************
							PROSES NON STOCK (Jasa)
******************************************************************************************************/
	OPEN cSaleD_Service FOR
		SELECT 
			d.SaleDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, 
			BasePrice, ListPrice, d.DiscExp, d.DiscAmt, d.TaxID, d.TaxAmt, i.ItemType
		FROM SaleD d JOIN Item i ON d.ItemID=i.ItemID
		WHERE 
			d.SaleNo=_SaleNo 
			AND (
				(i.ItemType='SERV') 
				OR (i.isStock=false AND i.IsAsm=false) 
				OR (_ConsNo IS NOT NULL)
			);
	LOOP
		FETCH cSaleD_Service INTO _SaleDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _BasePrice, _ListPrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _ItemType;
		EXIT WHEN NOT FOUND;

		--- Read Qty conversion
		_Conv:=fItemUnit_Conv(_ItemID, _Unit);

		--- CDIS (Claimed-Discount) -> Cost=Price(Base-Disc)
		IF (_ItemType='CDIS') THEN
			_Cost=_BasePrice-_DiscAmt;
		ELSE
			_Cost:=fUnitCost(_ItemID, _WhID, _PID, _Unit);
		END IF;

		IF (_Cost IS NULL) THEN
			_retval.ErrNo:='50003';
			_retval.ErrMsg:='Error menghitung HPP! Item:'||COALESCE(_ItemID,'<kosong>')||', Wh:'||COALESCE(_WhID,'<kosong>')||', BATCH:'||COALESCE(_PID,'<kosong>')||', Unit:'||COALESCE(_Unit,'<kosong>')||' (fUnitCost)(spSale_New)';
		END IF;

		UPDATE SaleD SET 
			Cost=_Cost,
			Conv=_Conv
		WHERE SaleNo=_SaleNo AND SaleDNo=_SaleDNo;

	END LOOP;
	CLOSE cSaleD_Service;

/******************************************************************************************************
							PROSES BONUS
******************************************************************************************************/

	OPEN cSaleD FOR
		SELECT 
			d.SaleBnsNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, i.ItemType, i.isStock, NULL, NULL
		FROM SaleBns d JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.SaleNo=_SaleNo 
			AND (i.isStock=TRUE OR i.ItemType='PACK');
	LOOP
		FETCH cSaleD INTO _SaleDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ItemType, _isStock, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		----- Row-1 ? Generate header dulu 
		IF(_count=0) THEN
			_ItemsCost:=0;

			_StockANo:=fTransNoGet('STOCKA');
			_StockANote:='Bonus Penjualan #'|| _SaleNo;
			IF (_StockANo IS NULL) THEN
				_retval.ErrNo:=50009;
				_retval.ErrMsg:='Error getting StockANoBns (spSale_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spStockA_New(
				_StockANo, _CustID, 'SALE', _SaleDate, 'O', 'SALE', _SaleNo, _StockANote, 
				TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID
			); 

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		END IF;

		--- Calculate Cost(HPP)
		_StockANote:='Bonus Penjualan #'||_SaleNo;

		-- Hitung HPP secara Bulk utk sejumlah Qty biar lebih presisi motong stocknya
		_CostSubTotal:=fUnitCost(_ItemID, _WhID, _PID, _Qty, _Unit);

		IF (_Qty IS NOT NULL) THEN
			_Cost:=_CostSubTotal/_Qty;
		ELSE
			_Cost:=0;
		END IF;

		IF (_Cost IS NULL) THEN
			_retval.ErrNo:='50003';
			_retval.ErrMsg:='Error menghitung HPP! Item:'||COALESCE(_ItemID,'<kosong>')||', Wh:'||COALESCE(_WhID,'<kosong>')||', PID:'||COALESCE(_PID, '<kosong>')||', Unit:'||COALESCE(_Unit, '<kosong>')||' (fUnitCost)(spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_PID IS NULL) THEN
			_PID:='';
		END IF;

		---------------- PACK: Generate Stock-Details --------------------------
		--------- Item Normal ------------salebnsno

		IF (_ItemType<>'PACK' AND _ItemType<>'SERV' AND _isStock=TRUE) THEN
			_StockADNo:=_StockADNo+1;
						
			SELECT INTO _spret * FROM spStockAD_New(
					_SaleDate, 'SALE', _StockANo, _StockADNo, 
					_WhID, _ItemID, _PID, _Qty*-1, _Unit, 
					_QtyX*-1, _Cost, _DeptID, _PrjID, true, _SaleNo, _CostSubTotal*-1);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		ELSIF (_ItemType='PACK') THEN
			SELECT INTO _BoMID bomid FROM bom WHERE itemid=_ItemID AND active=TRUE;
			/**_StockADNo:=_StockADNo+1; dicomment karena sudah ada modifikasi di spstockad_new_bom*/

			IF(_BomID IS NULL) THEN
				_retval.ErrNo:='50014';
				_retval.ErrMsg='Error tidak dapat menemukan B.o.M. untuk Item:'||_ItemID||'(spSale_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spStockAD_New_BoM(
				_SaleDate, _StockAType, _StockANo, _StockADNo, _WhID, _ItemID, _PID, 
				_Qty*-1, _Unit, _QtyX*-1, _Cost, _DeptID, _PrjID
			);

			_StockADNo:=_spret.ErrNo;
			
			IF _spret.ErrNo<>0 AND _spret.ErrMsg<>'OK' THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		--- Read Qty conversion
		_Conv:=fItemUnit_Conv(_ItemID, _Unit);

		IF (_Conv IS NULL) THEN
			_retval.ErrNo:='50055';
			_retval.ErrMsg:='Error, Konversi ItemID:'||COALESCE(_ItemID, '<kosong>')||', Unit:'||COALESCE(_Unit, '<kosong>')||'!  (fItemUnit_Conv)(spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		--- Update Cost & QtyConversion
		UPDATE SaleBns SET 
			Cost=_Cost, 
			Conv=_Conv 
		WHERE SaleNo=_SaleNo AND SaleBnsNo=_SaleDNo;

		_ItemsCost:=_ItemsCost+_CostSubTotal;
		_count:=_count+1;
	END LOOP;
	CLOSE cSaleD;

/******************************************************************************************************
		PROSES PAYMENT DAN PIUTANG: 
			- Payment : Cash/Bank
			- CREDIT : Piutang
******************************************************************************************************/
	--- Tunai ? Proses Uang masuk 
	IF (_TermType='C') THEN
		_CashAmt:=_PaidAmt;
		_CashANote:='Penjualan '||COALESCE(fBPName(_CustID),'Cust : <kosong>')||':'||_SaleNo;


/*********************************************************************************************** dulu pake table Sale-Cash-Receive *************
																						OPEN cSaleCRcv FOR
																							SELECT RcvType, EDCID, CashAmt, ChangeAmt, RcvAmt, CardNo, TRefNo
																							FROM SaleCRcv 
																							WHERE SaleNo=_SaleNo;
																						LOOP
																							FETCH cSaleCRcv INTO _RcvType, _CashID, _EDCID, _CashAmt, _ChangeAmt, _RcvAmt, _CardNo, _TRefNo;
																							EXIT WHEN NOT FOUND;			

																							_CashANote='Penjualan-Tunai #'||_SaleNo;
																							IF (_RcvType='CASH') THEN
																								IF (_CashID IS NULL) THEN
																									IF (_POSSesNo IS NOT NULL) THEN
																										SELECT INTO _CashID CashID FROM PoS WHERE PoSID=_PoSID;
																									ELSE
																										_retval.ErrNo:=10001;
																										_retval.ErrMsg:='CashID IS NULL';
																										RAISE EXCEPTION '%', _retval.ErrMsg;
																									END IF;
																								END IF;

																							ELSIF (_RcvType='CASH') THEN
																								SELECT INTO _CashID CashID FROM EDC WHERE EDCID=_EDCID;

																								_CashANote='Penjualan #'||_SaleNo||', Card:'||_CardNo;
																							END IF;

																							_CashAAmt=_RcvAmt;
			
																							SELECT INTO _spret * FROM spCashA_New(
																									_CashID, _CustID, 'SALE', _SaleDate, 'I', 'SALE', _SaleNo, 
																								 _CashAAmt, _CashANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
																									_PerID, _BranchID, _DeptID, _PrjID
																							);

																							IF _spret.ErrNo<>0 THEN
																								_retval.ErrNo:=_spret.ErrNo;
																								_retval.ErrMsg:=_spret.ErrMsg;
																								RAISE EXCEPTION '%', _retval.ErrMsg;
																							END IF;

																						END LOOP;
																						CLOSE cSaleCRcv;
************************************************************************************************************************/

		-------- set dulu biar mau di generate penerimaan --------
		_tmp_va1:='N';
		UPDATE Sale SET PayStatus=_tmp_va1, PaidAmt=0 WHERE SaleNo=_SaleNo;
		
		SELECT INTO _RcvNo * FROM fTransNoGet('RCV');
		IF (_RcvNo IS NULL) THEN
			_retval.ErrNo:=50041;
			_retval.ErrMsg:='Error getting RcvNo (spRcv_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		INSERT INTO rcv(rcvno,rcvtype,rcvdate,bpid,empid,totspaid,totrcvdisc,totrcvamt,reftype,refno,crtby,crtdate,updby,upddate,isautogen,crcid,excrate,fisrate,branchid) 			
			VALUES(_RcvNo,'SALE', _SaleDate, _Custid, _EmpId, _Total, 0, _Total, 'SALE', _SaleNo, _CrtBy, _CrtDate, _UpdBy, _UpdDate,true, _CrcID, _ExcRate, _FisRate, _BranchID);

		INSERT INTO rcvd(rcvno,rcvdno, rcvdmtd, cashid,rcvdcash,rcvdamt,refno,crcid,excrate,fisrate) 
		VALUES(_RcvNo, 1, 'CASH', _CashId, _Total, _Total, _SaleNo,_CrcID, _ExcRate, _FisRate);

		INSERT INTO spaid(rcvno,spaidno, reftype, refno,spaiddiscamt,spaidamt,crcid,excrate,fisrate) 
		VALUES(_RcvNo, 1, 'SALE', _SaleNo, 0, _Total, _CrcID,_ExcRate,_FisRate);

		SELECT INTO _spret * FROM spRcv_New(_RcvNo);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmp_va1:='F';
		UPDATE Sale SET PaidAmt=_Total, PayStatus=_tmp_va1 WHERE SaleNo=_SaleNo;

	---------- CREDIT ? Proses Piutang -------------
	ELSIF (_TermType='R') THEN
		_DebtNote:='Penjualan '||fBPName(_CustID)||':'||_SaleNo;
		SELECT INTO _spret * FROM spBPA_New(
			_CustID, 'C', _CrcID, _SaleDate, 'SALE', _SaleNo, 
			_Total, _DebtNote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmp_va1:='N';
		UPDATE Sale SET PaidAmt=0, PayStatus=_tmp_va1 WHERE SaleNo=_SaleNo;

	END IF;

/******************************************************************************************************
		PROSES SO:
			- Update SO:Qty Delivered
			- Check & Update Status SO (close SO)
******************************************************************************************************/
	--- Update SO:Qty-Delivered
	OPEN cSaleD_Service FOR
		SELECT SONo, SODNo, ItemID, PID, fQtyZToQty1(ItemID, Qty, Unit)
		FROM SaleD 
		WHERE 
			SaleNo=_SaleNo 
			AND (SONo IS NOT NULL AND SONo<>'') 
			AND (DELINo IS NULL OR DeliNo='');
	LOOP
		FETCH cSaleD_Service INTO _SONo, _SODNo, _ItemID, _PID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Rcv * FROM spSO_Item_Delivered(_SONo, _ItemID, _PID, _Qty1, _SODNo);
		IF (_Qty1Rcv<_Qty1) THEN
			_retval.ErrNo:=50018;
			_retval.ErrMsg:='Item:'||COALESCE(_ItemID,'<kosong>')||', Qty Delivery > QtySO (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSaleD_Service;

	----------- SO, Update status -----------------
	OPEN cSO FOR
		SELECT DISTINCT SONo 
		FROM SaleD 
		WHERE 
			SaleNo=_SaleNo 
			AND (SONo IS NOT NULL AND SONo<>'')
			AND (DELINo IS NULL OR DeliNo='')
			ORDER BY SONo;
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _CustID2 CustID FROM SO WHERE SONo=_SONo;
		IF (_CustID<>_CustID2) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Customer pada SO '|| _SONo ||' dan pada Penjualan '|| _SaleNo ||' tidak sama (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spSO_ChNUp_Status(_SONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSO;


/******************************************************************************************************
		PROSES KONSINYASI
******************************************************************************************************/
	IF (_countCons>0) THEN
		OPEN cSaleD FOR
			SELECT 
				ItemID, fQtyZToQty1(ItemID, Qty, Unit), QtyX, ConsNo, ConsDNo, PID 
			FROM SaleD  
			WHERE 
				SaleNo=_SaleNo AND (ConsNo IS NOT NULL AND ConsNo<>'')
			ORDER BY ConsNo, SaleDNo;
		LOOP
			FETCH cSaleD INTO _ItemID, _Qty, _QtyX, _ConsNo, _ConsDNo, _PID;
			EXIT WHEN NOT FOUND;

			SELECT INTO _Qty1_Ordered SUM(fQtyZToQty1(ItemID, QtySellIn, Unit)-QtySellOut-QtyCret) FROM consd WHERE itemid=_ItemID AND pid=_PID AND consno=_ConsNo;
			IF (_Qty>_Qty1_Ordered) THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Item:'|| _ItemID ||', Qty Sell Out > Qty Sell In (spSale_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_PID IS NOT NULL AND _PID!='') THEN
				SELECT INTO _countPID COUNT(pid) FROM saled WHERE saleno=_SaleNo AND pid=_PID;
				IF (_countPID>1) THEN
					_retval.ErrNo:=50555;
					_retval.ErrMsg:='Duplicate PID#'||_PID||' (spSale_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			IF (_PID IS NULL) THEN
				UPDATE ConsD SET QtyXSellOut=QtyXSellOut+_QtyX, QtySellOut=QtySellOut+_Qty
				WHERE ConsNo=_ConsNo AND ItemID=_ItemID AND ConsDNo=_ConsDNo;
			ELSE
				UPDATE ConsD SET QtyXSellOut=QtyXSellOut+_QtyX, QtySellOut=QtySellOut+_Qty
				WHERE ConsNo=_ConsNo AND ItemID=_ItemID AND PID=_PID AND ConsDNo=_ConsDNo;
			END IF;
		END LOOP;
		CLOSE cSaleD;

		OPEN cSaleCons FOR
			SELECT DISTINCT ConsNo FROM SaleD WHERE SaleNo=_SaleNo AND ConsNo IS NOT NULL AND ConsNo<>'';
		LOOP
			FETCH cSaleCons INTO _ConsNo;
			EXIT WHEN NOT FOUND;

			SELECT INTO _spret * FROM spSetConsStatus(_ConsNo);
			IF _spret.ErrNO<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSaleCons;
	END IF;


/**************************************************************************************************
	PROSES ACCOUNTING
**************************************************************************************************/

	IF (isAccInstalled()) THEN
		_JnlNo1:=fTransNoGet('JURNAL');

		IF(_SaleType='B')THEN
			_JnlNote:='Set Saldo Awal Piutang '||fBPName(_CustID)||':'||_SaleNo;
			_JnlType:='BEG';
		ELSE
			_JnlNote:='Penjualan '||fBPName(_CustID)||':'||_SaleNo;
			_JnlType:='SALE';
		END IF;

		------- Generate Jurnal Header untuk JurnalVoucher(JV) Penjualan
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo1, _JnlType, _SaleDate,'SALE', _SAleNo,
			_JnlNote, true, true, CAST(0 AS NUMERIC(19,4)),
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		----- Generate Jurnal-Header untuk Jurnal-Voucher-HPP --------
		IF(_SaleType<>'B') THEN
			_JnlNo2:=fTransNoGet('JURNAL');
			_JnlNote:= 'HPP '||fBPName(_CustID)||':'||_SaleNo;
			SELECT INTO _spret * FROM spJnl_Insert(
				_JnlNo2, 'SALE', _SaleDate,'SALE', _SAleNo,
				_JnlNote, true, true, CAST(0 AS NUMERIC(19,4)),
				_CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		----- Generate Jurnal-Header untuk Bonus --------
		IF(_countBonus>0) THEN
			_JnlNo5:=fTransNoGet('JURNAL');
			_JnlNote:= 'Bonus '||fBPName(_CustID)||':'||_SaleNo;
			SELECT INTO _spret * FROM spJnl_Insert(
				_JnlNo5, 'SALE', _SaleDate,'SALE', _SAleNo,
				_JnlNote, true, true, CAST(0 AS NUMERIC(19,4)),
				_CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		------- Fetch Akun Asset-Lancar(KasBank/Piutang) --------
		IF(_SaleType<>'B') THEN
			IF _TermType='C' THEN	--- Kas/Bank
				_AccAsset:=fAccCash(_CashID, _InOut);
			ELSE	 --- Piutang
				_AccAsset:=fBpAcc(_CustID, _CrcID, 'AR');
			END IF;
		ELSE
			--- BegBal: Piutang
			_AccAsset:=_AccAR;
		END IF;

		_count1:=0;
		_count2:=0;
		_count5:=0;

		------- Jurnal Ayat Asset (Cash/Piutang) -------------
		_count1:=_count1+1;	
--RAISE EXCEPTION 'Total %', _Total;	
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _count1, _AccAsset, 'D', _BTotal, _DeptID, _PrjID, _Total);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-------- Jurnal Ayat Sales/Income --------------
		OPEN cSaleAcc FOR
			SELECT fItemAcc(ItemID, 'SALE'), DeptID, PrjID, SUM(basesubtotal), SUM(subtotal)
			FROM SaleD 
			WHERE SaleNo=_SaleNo 
			GROUP BY fItemAcc(ItemID, 'SALE'), DeptID, PrjID;
		LOOP
			FETCH cSaleAcc INTO _AccSale, _DeptID, _PrjID, _AccAmt, _AccAmtForex;
			EXIT WHEN NOT FOUND;

			_count1:=_count1+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _count1, _AccSale, 'K', _AccAmt, _DeptID, _PrjID, _AccAmtForex);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSaleAcc;

		/** ==================== LABA RUGI SELISIH KURS ================================ */
		_RateLR:=(_BTotal+_MBDiscAmt)-(_BSubtotal+_MBFTaxAmt);
		IF (_RateLR!=0) THEN
			_count1:=_count1+1;
			_AccRateLR:=fAccDefault('PKKURS');
			IF (_AccRateLR IS NULL OR _AccRateLR='') THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Akun PK Selisih Kurs belum diset.'||E'\n'||'Silahkan disetting pada form Setting Akun';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _count1, _AccRateLR, 'K', _RateLR, _DeptID, _PrjID, NULL, _RateLR/_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;	

		---- Jurnal Discount Header --------
		IF (_MDiscAmt<>0) THEN
			IF (_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('SALEDISC');
			END IF;

			_count1:=_count1+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _count1, _AccDisc, 'K', _MBDiscAmt*-1, NULL, NULL, _MDiscAmt*-1);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		--- Ayat Pajak , groupped by item tax per item -----
		_TotalTax:=0;
		OPEN cTax FOR
			SELECT SUM(basftotaltaxamt), SUM(totaltaxamt), AccSaleTax, DeptID, PrjID
			FROM SaleD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.SaleNo=_SaleNo	AND (d.taxid IS NOT NULL)
			GROUP BY tax.AccSaleTax, d.DeptID, d.PrjID;
		LOOP
			FETCH cTax INTO _SaleDTaxAmt, _SaleDTaxAmtForex, _AccSaleTax, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			_count1:=_count1+1;

			IF (_AccSaleTax IS NULL) THEN
				_retval.ErrNo:=504334;
				_retval.ErrMsg:='Akun Pajak Penjualan belum di set, silahkan set di master Pajak (spSale_New)';
				RETURN NEXT _retval;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _count1, _AccSaleTax, 'K', _SaleDTaxAmt, _DeptID, _PrjID, _SaleDTaxAmtForex);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;


		---- Jurnal Freight --------
		IF (_Freight<>0) THEN
			_count1:=_count1+1;
	
			IF (_AccFreight IS NULL) THEN
				_AccFreight:=fAccDefault('SALEFREIGHT');
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _count1, _AccFreight, 'K', _Freight*_ExcRate, NULL, NULL, _Freight);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		---- Jurnal biaya lain2
		IF (_Others<>0) THEN
			_count1:=_count1+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _count1, fAccDefault('PKROUND'), 'K', _Others*_ExcRate, NULL, NULL, _Others);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;


/****************************** Jurnal Voucher HPP Perpetual **************************************/

--------- Posting Jurnal Voucher HPP Penjualan Normal, (BUKAN begbal) ---------------
		IF(_SaleType<>'B') THEN
			-- Ayat Jurnal HPP untuk item stocked Normal (langsung penjualan)
			OPEN cSaleAcc FOR
				SELECT fItemAcc(d.itemid, 'COST'), d.deptid, d.prjid, 
				SUM(ROUND(fUnitCost(d.itemid, d.whid, d.pid, d.qty, d.unit), _Round))	--SUM(fUnitCost(d.ItemID, d.WhID, d.PID, d.Qty, d.Unit))
				FROM saled d 
				JOIN item i ON d.itemid=i.itemid
				WHERE saleno=_SaleNo 
				AND (i.isstock=TRUE OR i.itemtype='PACK') 
				AND (d.consno IS NULL OR d.consno='') 
				GROUP BY fItemAcc(d.itemid, 'COST'), d.deptid, d.prjid;
			LOOP
				FETCH cSaleAcc INTO _AccCOGS, _DeptID, _PrjID, _AccAmt;
				EXIT WHEN NOT FOUND;

				IF(_AccCOGS IS NOT NULL) THEN
					_count2:=_count2+1;
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _count2, _AccCOGS, 'D', _AccAmt, _DeptID, _PrjID, _AccAmt/_ExcRate);

					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;
			END LOOP;
			CLOSE cSaleAcc;

			-------- Ayat Unbilled service (gantinya HPP) untuk yg pake delivery ---------
			IF (_countDeli>0) THEN
				--- Unbilled Service
				_AccUBDeliSvc:=fAccDefault('UBDELISVC');
				SELECT INTO
					_UBDeliSvcAmtForex, _BSubtotal		--??? mestinya baca cost atau price-disc sih ?
					SUM(CAST(ROUND(qty * (listPrice - fDiscCalc(discexp, listprice)), _Round) AS NUMERIC(19,4))), 
					SUM(CAST(ROUND(qty * _ExcRate * (listPrice - fDiscCalc(discexp, listprice)), _Round) AS NUMERIC(19,4)))
				FROM saled d 
				JOIN item i ON d.itemid=i.itemid
				WHERE d.saleno=_SaleNo 
				AND i.isstock=FALSE AND i.itemtype!='PACK'
				AND (consno IS NULL OR consno='')
				AND (delino IS NOT NULL AND delino<>'');
				IF (FOUND AND _UBDeliSvcAmtForex IS NOT NULL) THEN
					_UBDeliSvcAmt:=_UBDeliSvcAmtForex*_ExcRate;

					--- ayat UBDELISVC
					_count2:=_count2+1;
					SELECT INTO _spret * 
					FROM spJnlD_Insert(_JnlNo2, _count2, _AccUBDeliSvc, 'D', _BSubtotal, _DeptID, _PrjID, _UBDeliSvcAmt);

					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

					-- ayat UBDELI nya
					_count2:=_count2+1;
					SELECT INTO _spret * 
					FROM spJnlD_Insert(_JnlNo2, _count2, fAccDefault('UBDELI'), 'K', _BSubtotal, _DeptID, _PrjID, _UBDeliSvcAmt);

					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

				END IF;

				--------- Ayat Unbilled Delivered stocked-Items ---------------
				SELECT INTO 
					_UBDeliAmt	--??? mestinya baca cost atau price sih ?
					SUM(CAST(ROUND(Qty*Cost, _Round) AS NUMERIC(19,4)))
				FROM SaleD d
				JOIN Item i ON d.ItemID=i.ItemID
				WHERE 
					Saleno=_SaleNo
					AND i.isStock=TRUE
					AND (d.DeliNo IS NOT NULL AND d.DeliNo<>'')
					AND (d.ConsNo IS NULL OR d.ConsNo='');

				IF (FOUND AND _UBDeliAmt IS NOT NULL) THEN
					_UBDeliAmtForex:=_UBDeliAmt/_ExcRate;	--ExcRate nya ambil di Deli atau di Sale ?

					-- ayat HPP sudah di post diatas

					-- ayat UBDELI, 
					_count2:=_count2+1;
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _count2, fAccDefault('UBDELI'), 'K', _UBDeliAmt, _DeptID, _PrjID, _UBDeliAmtForex);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;
			END IF;

			-- bukan dari Deli (countNormal>0)
			IF (_countNormal>0) THEN
				-------------- Ayat Inventory ----------------------------
				OPEN cSaleAcc FOR
					SELECT accno, deptid, prjid, SUM(accamt) 
					FROM (
						SELECT fItemAcc(d.ItemID, 'INV') AS accno, SUM(ROUND(fUnitCost(d.itemid, d.whid, d.pid, d.qty, d.unit), _Round)) AS accamt, 
						d.DeptID, d.PrjID 
						FROM SaleD d JOIN Item i ON d.ItemID=i.ItemID
						WHERE d.SaleNo=_SaleNo AND i.isStock=true 
						AND (d.ConsNo IS NULL OR d.ConsNo='') 
						AND (d.DeliNo IS NULL OR d.DeliNo='')
						GROUP BY fItemAcc(d.ItemID, 'INV'), d.DeptID, d.PrjID
						UNION ALL

						SELECT f.accno, ROUND(f.accamt, _Round), f.deptid, f.prjid 
						FROM spJnlPacketInv(_SaleNo) f
					) a 
					GROUP BY accno, deptid, prjid;
				LOOP
					FETCH cSaleAcc INTO _AccInv, _DeptID, _PrjID, _AccAmtForex;
					EXIT WHEN NOT FOUND;

					_AccAmt:=_AccAmtForex; -- stock sudah pasti base currency, jadi sama
					IF(_AccInv IS NOT NULL) THEN
						_count2:=_count2+1;
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _count2, _AccInv, 'K', _AccAmt, _DeptID, _PrjID, _AccAmtForex*_ExcRate);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

					END IF;

				END LOOP;
				CLOSE cSaleAcc;
			END IF;	--- IF (_countNormal>0)

			------Khusus Konsinyasi, jurnal hpp yg laku -----
			IF (_countCons>0) THEN 	---- #7589
				_JnlNo3:=fTransNoGet('JURNAL');
				_JnlNote:='HPP '||fBPName(_CustID)||':'||_SaleNo;

				_count3:=0;
				_count4:=0;
				SELECT INTO _spret * FROM spJnl_Insert(
					_JnlNo3, _JnlType, _SaleDate,'SALE', _SAleNo,
					_JnlNote, true, true, CAST(0 AS NUMERIC(19,4)),
					_CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				OPEN cCons FOR
					SELECT fItemAcc(SaleD.ItemID, 'COST'), SaleD.DeptID, SaleD.PrjID, 
					--SUM(ROUND(Qty*SaleD.Cost, _Round))
					SUM(ROUND(fUnitCost(SaleD.itemid, SaleD.whid, SaleD.pid, SaleD.qty, SaleD.unit), _Round)) AS accamt 
					FROM SaleD LEFT JOIN CONSD ON SaleD.consno=consd.consno AND SaleD.ItemID=consd.ItemID
					WHERE SaleNo=_SaleNo AND SaleD.consno IS NOT NULL
					GROUP BY fItemAcc(SaleD.ItemID, 'COST'), SaleD.DeptID, SaleD.PrjID;
					LOOP
						FETCH cCons INTO _AccCOGS,_DeptID,_PrjID,_AccAmt;
						EXIT WHEN NOT FOUND;

						IF(_AccCOGS IS NOT NULL) THEN
							_count3:=_count3+1;
							SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo3, _count3, _AccCOGS, 'D', _AccAmt, _DeptID, _PrjID,_AccAmt);
							IF _spret.ErrNo<>0 THEN
								_retval.ErrNo:=_spret.ErrNo;
								_retval.ErrMsg:=_spret.ErrMsg;
								RAISE EXCEPTION '%', _retval.ErrMsg;
							END IF;

							_count3:=_count3+1;
							SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo3, _count3, fAccDefault('ARCONS'), 'K', _AccAmt, _DeptID, _PrjID,_AccAmt);
							IF _spret.ErrNo<>0 THEN
								_retval.ErrNo:=_spret.ErrNo;
								_retval.ErrMsg:=_spret.ErrMsg;
								RAISE EXCEPTION '%', _retval.ErrMsg;
							END IF;
						END IF;
					END LOOP;
				CLOSE cCons;

			END IF;	-- #7589 IF (_countCons>0) 


			-- jurnal bonus
			IF (_countBonus>0) THEN
				-------------- Ayat Inventory ----------------------------
				OPEN cSaleAcc FOR
					SELECT accno, NULL, NULL, SUM(accamt) 
					FROM (
						SELECT fItemAcc(d.ItemID, 'INV') AS accno, SUM(ROUND(d.Qty*d.Cost, _Round)) AS accamt, NULL as deptid, NULL as prjid
						FROM SaleBns d JOIN Item i ON d.ItemID=i.ItemID
						WHERE d.SaleNo=_SaleNo AND i.isStock=true 
						GROUP BY fItemAcc(d.ItemID, 'INV')
						UNION

						SELECT f.accno, f.accamt, f.deptid, f.prjid 
						FROM spJnlPacketInv(_SaleNo, 'BNS') f
					) a 
					GROUP BY accno, deptid, prjid;
				LOOP
					FETCH cSaleAcc INTO _AccInv, _DeptID, _PrjID, _AccAmtForex;
					EXIT WHEN NOT FOUND;

					_AccAmt:=_AccAmtForex; -- stock sudah pasti base currency, jadi sama
					IF(_AccInv IS NOT NULL) THEN
						_count5:=_count5+1;
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo5, _count5, _AccInv, 'K', _AccAmt, _DeptID, _PrjID, _AccAmtForex*_ExcRate);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

						_AccBns:=fAccDefault('SALEBNS');
						_count5:=_count5+1;
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo5, _count5, _AccBns, 'D', _AccAmt, _DeptID, _PrjID, _AccAmtForex*_ExcRate);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

					END IF;

				END LOOP;
				CLOSE cSaleAcc;
			END IF;	--- IF (_countBonus>0)
		END IF;

		---------- execute voucher-1 (sales) spJnlNew --------------
		SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		----- #54389
		IF(_SaleType<>'B') THEN
			--- posting voucher stock/unbilled
			IF(_count2>0) THEN
				SELECT INTO _spret * FROM spJnl_New(_JnlNo2);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSE
				DELETE FROM jnl WHERE jnlno=_JnlNo2;
			END IF;

			--- posting bonus
			IF(_countBonus>0) THEN
				SELECT INTO _spret * FROM spJnl_New(_JnlNo5);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSE
				DELETE FROM jnl WHERE jnlno=_JnlNo5;
			END IF;

			IF(_countCons>0) THEN
				SELECT INTO _spret * FROM spJnl_New(_JnlNo3);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
/*				SELECT INTO _spret * FROM spJnl_New(_JnlNo4);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
*/
			END IF;
		END IF;	-- #54389

	END IF; -- use Accounting


	_retval:=(0, 'OK');
	RETURN NEXT _retval;

END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



/** sale new untuk POS */


CREATE OR REPLACE FUNCTION spSale_New_POS(
	_SaleNo 	VARCHAR(15)/**,
	_TotRcvAmt	NUMERIC(19,4),
	_TotDebitAmt	NUMERIC(19,4),
	_TotCreditAmt	NUMERIC(19,4),
	_TotVcrAmt	NUMERIC(19,4),
	_TotChgAmt	NUMERIC(19,4)*/
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CustID			VARCHAR(15);
	_CustID2		VARCHAR(15);
	_SaleType		VARCHAR(3);
	_SaleDate		DATE;
	_SONo			VARCHAR(15);
	_TermType		VARCHAR(1);
	_Term			INT;
	_PoSID			VARCHAR(5);
	_PoSSesNo		VARCHAR(8);

	_SubTotal		DECIMAL(19,4);
	_Total			DECIMAL(19,4);
	_PaidAmt		DECIMAL(19,4);
	_CrtBy			VARCHAR(10);
	_CrtDate		TIMESTAMP;
	_UpdBy			VARCHAR(10);
	_UpdDate		TIMESTAMP;
	_DeliNo			VARCHAR(15);
	_SaleDNo		INT;
	_SaleNo2		VARCHAR(15);

	_StockAType		VARCHAR(5):='SALE';
	_isInsert		BOOLEAN:=true;

	_StockANo		VARCHAR(15);
	_StockANote		VARCHAR(255);
	_StockADNo		SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_Qty			DECIMAL(15,4);
	_Unit			VARCHAR(5);
	_Conv			DECIMAL(15,4);
	_QtyX			DECIMAL(15,4);
	_BasePrice		DECIMAL(19,4);
	_ListPrice 		DECIMAL(19,4);
	_DiscExp		VARCHAR(15);
	_DiscAmt		DECIMAL(19,4);
	_TaxID			VARCHAR(5);
	_TaxAmt			DECIMAL(19,4);
	_MTaxAmt		DECIMAL(19,4);
	_Cost			DECIMAL(19,4);
	_ItemType		VARCHAR(5);
	_IsStock		BOOLEAN;
	_BoMID			bomd.bomid%TYPE;
	_Qty_Unit1		DECIMAL(15,4);
	_SaleQty		DECIMAL(15,4);
	_SaleQtyX		DECIMAL(15,4);
	_ItemsCost		DECIMAL(19,4);
	_NumOfSaleDeli 		INT;
	_CashID			VARCHAR(15);
	_CashAAmt		DECIMAL(19,4);
	_CashANote		VARCHAR(255);
	_DebtNote		VARCHAR(255);
	_AllowCredit		DECIMAL(19,4);
	_CreditLimit		DECIMAL(19,4);
	_CountCredit		DECIMAL(19,4);
	_CreditAmount		DECIMAL(19,4);
	_CrcID			VARCHAR(5);

	_ContBal 		DECIMAL(19,4);
	_RcvNo			VARCHAR;
	_EmpId			VARCHAR;
	_TotDisc		DECIMAL(19,4);
	_ConsNo			VARCHAR(15);
	/* SO Stuff ****/
	_Qty1 			DECIMAL(15,4);
	_Qty1Rcv 		DECIMAL(15,4);

	/** Bonus **/
	_StockANoBns 		VARCHAR(15);
	_StockANoteBns		VARCHAR(255);
	_StockADNoBns		INT2;
	_ItemIDBns		VARCHAR(20);
	_WhIDBns		VARCHAR(3);
	_PIDBns			VARCHAR(25);
	_QtyBns			DECIMAL(15,4);
	_QtyXBns		DECIMAL(15,4);
	_UnitBns		VARCHAR(5);
	_CostBns		DECIMAL(19,4);
	_ExcRate		DECIMAL(19,4);

	_AccAsset		VARCHAR(10);
	_AccDisc		VARCHAR(10);
	_AccInv			VARCHAR(10);
	_AccCOGS		VARCHAR(10);
	_AccFreight		VARCHAR(10);
	_AccSale		VARCHAR(10);
	_AccAmt			DECIMAL(19,4);
	_AccAR			VARCHAR;

	_Freight		DECIMAL(19,4);
	_Others			DECIMAL(19,4);
	_Disc			DECIMAL(19,4);
	_Count1			INT;
	_Count2			INT;
	_Count3			INT;
	_Count4			INT;
	_JnlNo1			VARCHAR(15);
	_JnlNo2			VARCHAR(15);
	_JnlNo3			VARCHAR(15);
	_JnlNo4			VARCHAR(15);
	_JnlNote		VARCHAR;
	_UBDeliAmt		DECIMAL(19,4);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID 			VARCHAR(5);
	_Round			INT;
	_RoundTax		INT;
	_DeliItemCount		INT;
	_useDeli		BOOLEAN;

	cDeliSale		REFCURSOR;
	cSaleD 			REFCURSOR;
	cSaleD2 		REFCURSOR;
	cSO 			REFCURSOR;
	cSaleBns 		REFCURSOR;
	cSaleAcc		REFCURSOR;
	cSaleCons		REFCURSOR;
	cSaleCRcv		REFCURSOR;
	cCons			REFCURSOR;

	/** POS/SaleCRcv stuff ***/
	__RcvType		VARCHAR;
	__CashID		VARCHAR;
	__EDCID			VARCHAR;
	__RcvAmt		NUMERIC(19,4);
	__CardNo		VARCHAR;
	__TRefNo		VARCHAR;

	_CountCons		INT2;
	_isDraft		BOOLEAN;
	_JnlType		VARCHAR;
	_LockDate		DATE;
	_SODNo			INT;
	_Count			INT;
	_SelisihBasePrice	NUMERIC(19,4);
	_iterLoop		INT;
	_TotalBasePrice		NUMERIC(19,4);
	_SumD			NUMERIC(19,4);
	_SumKDeli		NUMERIC(19,4);
	_SumK			NUMERIC(19,4);
	_SelisihHPP		NUMERIC(19,4);
	_CountSelisihHPP	INT;
	_ItemIDCheck		VARCHAR;
	cSaleD1			REFCURSOR;

	_CountPID		INT2;
	cTax			REFCURSOR;
	_SaleDTaxAmt		NUMERIC(19,4);
	_AccSaleTax		VARCHAR;
	_TotalTax		NUMERIC(19,4);
	_SelisihTax		NUMERIC(19,4);

	_Counter		INT2; /** counter untuk rcvd */

	_Fisrate		NUMERIC(19,4);

	_first			BOOLEAN;

	_AccUnreconciled	VARCHAR;
	_Unreconciled		NUMERIC(19,4);	
	_Reconciled		NUMERIC(19,4);
	_isAutoReconcileIn	BOOLEAN;
	_tmp_va1		VARCHAR(1);

	_EndTime		TIMESTAMP;
	_POSCashId		VARCHAR;
	_CostSubTotal		NUMERIC(19,4);
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;

/**	_retval:=(0, 'OK');
	RETURN NEXT _retval;
	RETURN;
*/

	_TotDisc:=0;
	_NumOfSaleDeli:=0;
	_Unreconciled:=0;
	_Reconciled:=0;
	/** paranoid error message */
	_retval:=(50001, 'General Error spSale_New');

	SELECT INTO 
		_PerID, _BranchID, _CustID, _SaleType, _SaleDate, _SONo, _TermType, _Term, _PoSID, _PoSSesNo, _CashID, _SubTotal, _Total, _PaidAmt, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _EmpId, _ConsNo, _CrcID, _AccFreight, _Freight, _Disc, _AccDisc, _MTaxAmt, _ExcRate,_Others,_AccAR, _Fisrate
		PerID, BranchID, CustID, SaleType, SaleDate, SONo, TermType, DueDays, PoSID, PoSSesNo, CashID, SubTotal, Total, PaidAmt, CrtBy, CrtDate, UpdBy, UpdDate, EmpID, ConsNo, CrcID, AccFreight, Freight, DiscAmt, AccSDisc, TaxAmt, ExcRate,others,accar, fisrate
	FROM Sale WHERE SaleNo=_SaleNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penjualan '|| _SaleNo||' tidak ditemukan ! (spSale_New_POS)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SALE';
	IF(_LockDate is not null AND _LockDate>_SaleDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penjualan melebihi batas tanggal kunci transaksi ! (spSale_New_POS)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _POSCashId cashid FROM pos WHERE posid=_PoSID;
	IF (_POSCashId<>_CashID) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:=E'Kas pada penjualan ini tidak sama dengan kas kasir ! (spSale_New_POS)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _isDraft IsDraft FROM Sale WHERE SaleNo=_SaleNo;
	IF NOT FOUND THEN
		_isDraft:=FALSE;
	END IF;

	IF (_isDraft IS NULL) THEN
		_isDraft:=FALSE;
	END IF;
	
	IF (_isDraft=FALSE) THEN /** MULAI ISDRAFT */

	_Round:=COALESCE(fReg('ROUND'), '0');
	_RoundTax:=COALESCE(fReg('ROUND_TAX'), '0');

	SELECT INTO _CountCons COUNT(*) FROM ConsSale WHERE SaleNo=_SaleNo;
	IF (_CountCons=0) THEN
		_ConsNo:=NULL;
	ELSE
		_ConsNo:='Ada'; /** dummy **/
	END IF;

	/** Point of Sale **/
	IF (_PoSSesNo IS NOT NULL) THEN
		IF NOT EXISTS(SELECT 1 FROM PoSSes WHERE PoSID=_PoSID AND StartTime IS NOT NULL AND EndTime IS NULL) THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg='Kasir: '||COALESCE(_PoSID,'<kosong>')||' tidak aktiv (spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _EndTime EndTime FROM posses WHERE possesno=_PoSSesNo;
		IF (_EndTime IS NOT NULL) THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg='Session POS: '||COALESCE(_PoSSesNo,'<kosong>')||' tidak aktif (spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	IF (_TermType='C') THEN
		IF((_PoSSesNo IS NOT NULL) AND (_CashID IS NULL)) THEN
			SELECT INTO _CashID CashID FROM PoS WHERE PoSID=_PoSID;

			UPDATE Sale SET CashID=_CashID WHERE SaleNo=_SaleNo;
		END IF;

		IF((_CashID IS NULL) OR (_CashID='')) THEN
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Error, Kas untuk penjualan tunai belum diset ! (spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	ELSIF (_TermType='R') THEN
		SELECT INTO _AllowCredit, _CreditLimit, _CreditAmount arlimitfreq, arlimitamt, arbalance FROM BPacc WHERE BPID=_CustID;
		SELECT INTO _CountCredit count(*) from sale where PayStatus<>'F' and custID=_CustID;
		IF (_AllowCredit=0) THEN
			_retval.ErrNo:=50005;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' tidak diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSEIF (_AllowCredit<>-1 AND _AllowCredit<_CountCredit) THEN
			_retval.ErrNo:=50006;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' melebihi jumlah diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		 /*Sudah di-Check di Applikasi agar bisa diconfirmasi ke user*/
		IF (_CreditLimit=0) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' tidak diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSEIF (_CreditLimit<>-1 AND _CreditLimit<(_CreditAmount+_Total) ) THEN
			_retval.ErrNo:=50008;
			_retval.ErrMsg:='Penjualan melebihi Limit Hutang (spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	/*ngecek itemnya is sale apa gak!!*/
	OPEN cSaleD1 FOR
		SELECT saled.itemid FROM saleD JOIN item on saled.itemid=item.itemid WHERE issale=FALSE and saleno=_SaleNo;
		LOOP
			FETCH cSaleD1 INTO _ItemIDCheck;
			EXIT WHEN NOT FOUND;

			/*Item begbal boleh pas tipe nya beginning balance!*/
			IF NOT(_ItemIDCheck='0' AND _SaleType='B') THEN
				_retval.ErrNo:=50055;
				_retval.ErrMsg:='Error, ItemID:' || _ItemIDCheck || ' tidak di Set untuk Penjualan ! (spSale_New_POS)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
	CLOSE cSaleD1;

	/* Penjualan dari D.O. (DLO) sebelumnya ? */

	OPEN cDeliSale FOR
		SELECT delino FROM DeliSale WHERE SaleNo=_SaleNo;
	LOOP
		FETCH cDeliSale INTO _DeliNo;
		EXIT WHEN NOT FOUND;
		
		SELECT INTO _CustID2 CustID FROM Deli WHERE DeliNo=_DeliNo;
		IF (_CustID<>_CustID2) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Customer Delivery dan Penjualan tidak sama (spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		UPDATE Deli SET IsInvoiced=true WHERE delino=_DeliNo;
		_NumOfSaleDeli:=_NumOfSaleDeli+1;
		
	END LOOP;
	CLOSE cDeliSale;

	SELECT INTO _DeliItemCount COUNT(*) FROM SaleD WHERE SaleNo=_SaleNo AND DeliNo IS NOT NULL AND DeliNo<>'';
	_useDeli:=(_DeliItemCount>0 OR _NumOfSaleDeli>0);

	_StockADNo:=0;
	_Count:=0;
	OPEN cSaleD FOR
		SELECT 
			d.SaleDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, i.ItemType, i.IsStock,
			d.DeptID, d.PrjID
		FROM SaleD d JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.SaleNo=_SaleNo AND (d.DeliNo IS NULL OR d.DeliNo='') AND ((i.IsStock=true) 
		OR (i.ItemType='PACK')) AND (d.ConsNo IS NULL OR d.ConsNo='');
		/*Item biasa:GOOD, Paket:PACK*/
	LOOP
		FETCH cSaleD INTO _SaleDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ItemType, _IsStock, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		IF(_Count=0) THEN
			_ItemsCost:=0;

			_StockANo:=fTransNoGet('STOCKA');
			_StockANote:='Penjualan #'|| _SaleNo;
			IF (_StockANo IS NULL) THEN
				_retval.ErrNo:=50009;
				_retval.ErrMsg:='Error getting StockANo';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spStockA_New(
				_StockANo, _CustID, 'SALE', _SaleDate, 'O', 'SALE', _SaleNo, _StockANote, 
				TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID
			); 

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	
		_StockANote:='Penjualan #'||_SaleNo;
		SELECT INTO _CostSubTotal * FROM fUnitCost(_ItemID, _WhID, _PID, _Qty, _Unit);
		IF (_Qty IS NOT NULL) THEN
			_Cost:=_CostSubTotal/_Qty;
		ELSE
			_Cost:=0;
		END IF;

		IF (_Cost IS NULL) THEN
			_retval.ErrNo:='50003';
			_retval.ErrMsg:='Error menghitung HPP! Item:'||COALESCE(_ItemID,'<kosong>')||', Wh:'||COALESCE(_WhID,'<kosong>')||', PID:'||COALESCE(_PID,'<kosong>')||', Unit:'||COALESCE(_Unit,'<kosong>')||' (fUnitCost)(spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		/*################ Generate Stock-Details ##################*/
		IF (_ItemType='PACK') THEN
			SELECT INTO _BoMID BoMID FROM BoM WHERE ItemID=_ItemID;
			/**_StockADNo:=_StockADNo+1; dicomment karena sudah ada modifikasi di spstockad_new_bom*/
			
			IF(_BomID IS NULL) THEN
				_retval.ErrNo:='50014';
				_retval.ErrMsg='Error tidak dapat menemukan B.o.M. untuk Item:'||_ItemID||'(spSale_New_POS)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
	
			_SaleQty:=_Qty*(-1);
			_SaleQtyX:=_QtyX*(-1);

			SELECT INTO _spret * FROM spStockAD_New_BoM(
				_SaleDate, _StockAType, _StockANo, _StockADNo, _WhID, _ItemID, _PID, 
				_SaleQty, _Unit, _SaleQtyX, _Cost, _DeptID, _PrjID, _CrcID
			);

			_StockADNo:=_spret.ErrNo;

			IF _spret.ErrNo<>0 AND _spret.ErrMsg<>'OK' THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSE
			IF (_PID IS NULL) THEN
				_PID:='';
			END IF;

			_StockADNo:=_StockADNo+1;
			
			_SaleQty:=_Qty*(-1);
			_SaleQtyX:=_QtyX*(-1);

			/*--IF (_ConsNo IS NOT NULL) THEN*/
				/*--UPDATE consd SET QtyXSellOut=QtyXSellOut+_QtyX,QtySellOut=QtySellOut+_Qty WHERE consno=_ConsNo AND itemid=_ItemID;*/
			/*--ELSIF (_NumOfSaleDeli<=0) THEN*/
			/*--ELSE*/
			
			SELECT INTO _spret * FROM spStockAD_New(_SaleDate, 'SALE', _StockANo, _StockADNo, _WhID, _ItemID, _PID, _SaleQty, _Unit, _SaleQtyX, _Cost, _DeptID, _PrjID, true, _SaleNo, _CostSubTotal*-1);


			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		/*--END IF;*/
		END IF;

		SELECT INTO _Conv * FROM fItemUnit_Conv(_ItemID, _Unit);

		IF (_Conv IS NULL) THEN
			_retval.ErrNo:='50055';
			_retval.ErrMsg:='Error, Konversi ItemID:'||COALESCE(_ItemID,'<kosong>')||', Unit:'||COALESCE(_Unit,'<kosong>')||'!  (fItemUnit_Conv)(spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		UPDATE SaleD SET Cost=_Cost, Conv=_Conv WHERE SaleNo=_SaleNo AND SaleDNo=_SaleDNo;

		_ItemsCost:=_ItemsCost+(_Qty*_Cost);
		_Count:=_Count+1;
	END LOOP;
	CLOSE cSaleD;

	/************************ Proses BONUS ******************************/

	_StockADNo:=0;
	_Count:=0;
	OPEN cSaleD FOR
		SELECT 
			d.salebnsno, d.itemid, d.whid, d.pid, d.qty, d.unit, d.qtyx, i.itemtype, i.isstock,
			NULL, NULL
		FROM salebns d JOIN Item i ON d.itemid=i.itemid
		WHERE d.saleno=_SaleNo AND ((i.IsStock=true) OR (i.ItemType='PACK'));
		/*Item biasa:GOOD, Paket:PACK*/
	LOOP
		FETCH cSaleD INTO _SaleDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ItemType, _IsStock, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		IF(_Count=0) THEN
			_ItemsCost:=0;

			_StockANo:=fTransNoGet('STOCKA');
			_StockANote:='Bonus Penjualan #'|| _SaleNo;
			IF (_StockANo IS NULL) THEN
				_retval.ErrNo:=50009;
				_retval.ErrMsg:='Error getting StockANo';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spStockA_New(
				_StockANo, _CustID, 'SALE', _SaleDate, 'O', 'SALE', _SaleNo, _StockANote, 
				TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID
			); 

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	
		_StockANote:='Bonus Penjualan #'||_SaleNo;
		SELECT INTO _Cost * FROM fUnitCost(_ItemID, _WhID, _PID, _Unit);

		IF (_Cost IS NULL) THEN
			_retval.ErrNo:='50003';
			_retval.ErrMsg:='Error menghitung HPP! Item:'||COALESCE(_ItemID,'<kosong>')||', Wh:'||COALESCE(_WhID,'<kosong>')||', PID:'||COALESCE(_PID,'<kosong>')||', Unit:'||COALESCE(_Unit,'<kosong>')||' (fUnitCost)(spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		/*################ Generate Stock-Details ##################*/
		IF (_ItemType='PACK') THEN
			SELECT INTO _BoMID BoMID FROM BoM WHERE ItemID=_ItemID;
			/**_StockADNo:=_StockADNo+1; dicomment karena sudah ada modifikasi di spstockad_new_bom*/

			IF(_BomID IS NULL) THEN
				_retval.ErrNo:='50014';
				_retval.ErrMsg='Error tidak dapat menemukan B.o.M. untuk Item:'||_ItemID||'(spSale_New_POS)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
	
			_SaleQty:=_Qty*(-1);
			_SaleQtyX:=_QtyX*(-1);

			SELECT INTO _spret * FROM spStockAD_New_BoM(
				_SaleDate, _StockAType, _StockANo, _StockADNo, _WhID, _ItemID, _PID, 
				_SaleQty, _Unit, _SaleQtyX, _Cost, _DeptID, _PrjID, _CrcID
			);

			_StockADNo:=_spret.ErrNo;

			IF _spret.ErrNo<>0 AND _spret.ErrMsg<>'OK' THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSE
			IF (_PID IS NULL) THEN
				_PID:='';
			END IF;

			_StockADNo:=_StockADNo+1;
			
			_SaleQty:=_Qty*(-1);
			_SaleQtyX:=_QtyX*(-1);

			/*--IF (_ConsNo IS NOT NULL) THEN*/
				/*--UPDATE consd SET QtyXSellOut=QtyXSellOut+_QtyX,QtySellOut=QtySellOut+_Qty WHERE consno=_ConsNo AND itemid=_ItemID;*/
			/*--ELSIF (_NumOfSaleDeli<=0) THEN*/
			/*--ELSE*/
			
			SELECT INTO _spret * FROM spStockAD_New(_SaleDate, 'SALE', _StockANo, _StockADNo, _WhID, _ItemID, _PID, _SaleQty, _Unit, _SaleQtyX, _Cost, _DeptID, _PrjID, true);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		/*--END IF;*/
		END IF;

		SELECT INTO _Conv * FROM fItemUnit_Conv(_ItemID, _Unit);

		IF (_Conv IS NULL) THEN
			_retval.ErrNo:='50055';
			_retval.ErrMsg:='Error, Konversi ItemID:'||COALESCE(_ItemID,'<kosong>')||', Unit:'||COALESCE(_Unit,'<kosong>')||'!  (fItemUnit_Conv)(spSale_New_POS)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		UPDATE salebns SET cost=_Cost, conv=_Conv WHERE saleno=_SaleNo AND salebnsno=_SaleDNo;

		_ItemsCost:=_ItemsCost+(_Qty*_Cost);
		_Count:=_Count+1;
	END LOOP;
	CLOSE cSaleD;

	/** proses item non-stock **/
	OPEN cSaleD2 FOR
		SELECT d.SaleDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, BasePrice, ListPrice, d.DiscExp, d.DiscAmt, d.TaxID, d.TaxAmt, i.ItemType
		FROM SaleD d JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.SaleNo=_SaleNo AND ((i.ItemType='SERV') OR (i.IsStock=false AND i.IsAsm=false) OR (_ConsNo IS NOT NULL));
	LOOP
		FETCH cSaleD2 INTO _SaleDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _BasePrice, _ListPrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _ItemType;
		EXIT WHEN NOT FOUND;

		IF (_ItemType='CDIS') THEN
			_Cost=_BasePrice-_DiscAmt;
		ELSE
			SELECT INTO _Cost * FROM fUnitCost(_ItemID, _WhID, _PID, _Unit);
		END IF;

		IF (_Cost IS NULL) THEN
			_retval.ErrNo:='50003';
			_retval.ErrMsg:='Error menghitung HPP! Item:'||COALESCE(_ItemID,'<kosong>')||', Wh:'||COALESCE(_WhID,'<kosong>')||', Batch:'||COALESCE(_PID,'<kosong>')||', Unit:'||COALESCE(_Unit,'<kosong>')||' (fUnitCost)(spSale_New_POS)';
		END IF;

		UPDATE SaleD SET Cost=_Cost WHERE SaleNo=_SaleNo AND SaleDNo=_SaleDNo;

		_ItemsCost:=_ItemsCost+(_Qty*_Cost);
		_TotDisc:=_TotDisc+_DiscAmt;

	END LOOP;
	CLOSE cSaleD2;

		/** reserved for double processing 
		SELECT into _spret * from fTransNoGet('SALE2');

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF; 

		UPDATE Sale SET SaleNo2=_SaleNo2, CostTot=_ItemsCost WHERE SaleNo=_SaleNo;
		*/

	IF (_TermType='C') THEN
		_CashANote:='Penjualan '||COALESCE(fBPName(_CustID),'Cust : <kosong>')||':'||_SaleNo;
		_first:=TRUE;

		OPEN cSaleCRcv FOR
			SELECT s.RcvType, s.CashID, s.EDCID, s.RcvAmt, s.CardNo, s.TRefNo, c.isAutoReconcileIn
			FROM SaleCRcv s JOIN Cash c ON s.CashID=c.CashID 
			WHERE SaleNo=_SaleNo;
		LOOP
			FETCH cSaleCRcv INTO __RcvType, __CashID, __EDCID, __RcvAmt, __CardNo, __TRefNo, _isAutoReconcileIn;
			EXIT WHEN NOT FOUND;

			IF (_first=TRUE) THEN
				SELECT INTO _RcvNo * from fTransNoGet('RCV');
				IF (_RcvNo IS NULL) THEN
					_retval.ErrNo:=50041;
					_retval.ErrMsg:='Error getting RcvNo (spSale_New_POS)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				INSERT INTO rcv(rcvno,rcvtype,rcvdate,bpid,empid,totspaid,totrcvdisc,totrcvamt,reftype,refno,crtby,crtdate,updby,upddate,isautogen, crcid, fisrate, excrate,totspaidbc,totrcvdiscbc,totrcvamtbc) 
				VALUES(_RcvNo,'SALE', _SaleDate, _Custid, _EmpId, _Total, 0, _Total, 'SALE', _SaleNo, _CrtBy, _CrtDate, _UpdBy, _UpdDate,true, _CrcID, _Fisrate, _Excrate,_Total*_ExcRate,0,_Total*_ExcRate);

				_first:=FALSE;
				_Counter:=1;
			END IF; /** end _first */

			IF (__RcvType='CASH') THEN
				INSERT INTO rcvd(rcvno, rcvdno, rcvdmtd, cashid, rcvdcash, rcvdamt, refno, rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'CASH', __CashID, __RcvAmt, __RcvAmt, _SaleNo, 'Tunai', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
				IF (_isAutoReconcileIn=FALSE) THEN
					_Unreconciled:=_Unreconciled+__RcvAmt;
				ELSE
					_Reconciled:=_Reconciled+__RcvAmt;
				END IF;
			ELSIF (__RcvType='DC') THEN
				INSERT INTO rcvd(rcvno, rcvdno, rcvdmtd, cashid, rcvdcash, rcvdamt, refno, rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'DBC', __CashID, __RcvAmt, __RcvAmt, _SaleNo, 'Kartu Debit', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
				IF (_isAutoReconcileIn=FALSE) THEN
					_Unreconciled:=_Unreconciled+__RcvAmt;
				ELSE
					_Reconciled:=_Reconciled+__RcvAmt;
				END IF;
			ELSIF (__RcvType='CC') THEN
				INSERT INTO rcvd(rcvno, rcvdno, rcvdmtd, cashid, rcvdcash, rcvdamt, refno, rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'CRC', __CashID, __RcvAmt, __RcvAmt, _SaleNo, 'Kartu Kredit', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
				IF (_isAutoReconcileIn=FALSE) THEN
					_Unreconciled:=_Unreconciled+__RcvAmt;
				ELSE
					_Reconciled:=_Reconciled+__RcvAmt;
				END IF;
			ELSIF (__RcvType='VOUC') THEN
				INSERT INTO rcvd(rcvno, rcvdno, rcvdmtd, cashid, rcvdcash, rcvdamt, refno, rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'VOUC', __CashID, __RcvAmt, __RcvAmt, _SaleNo, 'Voucher', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
				IF (_isAutoReconcileIn=FALSE) THEN
					_Unreconciled:=_Unreconciled+__RcvAmt;
				ELSE
					_Reconciled:=_Reconciled+__RcvAmt;
				END IF;
			ELSIF (__RcvType='CHG') THEN
				INSERT INTO rcvd(rcvno, rcvdno, rcvdmtd, cashid, rcvdcash, rcvdamt, refno, rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'CASH', __CashID, __RcvAmt*(-1), __RcvAmt*(-1), _SaleNo, 'Kembalian', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
				IF (_isAutoReconcileIn=FALSE) THEN
					_Unreconciled:=_Unreconciled+(__RcvAmt*(-1));
				ELSE
					_Reconciled:=_Reconciled+(__RcvAmt*(-1));
				END IF;
			END IF;
		END LOOP;
		CLOSE cSaleCRcv;

			/** set dulu biar mau di generate penerimaan */
		_tmp_va1:='N';
		UPDATE Sale SET PayStatus=_tmp_va1, PaidAmt=0 WHERE SaleNo=_SaleNo;
		
			/** SELECT INTO _RcvNo * from fTransNoGet('RCV');
			IF (_RcvNo IS NULL) THEN
				_retval.ErrNo:=50041;
				_retval.ErrMsg:='Error getting RcvNo (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			INSERT INTO rcv(rcvno,rcvtype,rcvdate,bpid,empid,totspaid,totrcvdisc,totrcvamt,reftype,refno,crtby,crtdate,updby,upddate,isautogen, crcid, fisrate, excrate) 
				VALUES(_RcvNo,'SALE', _SaleDate, _Custid, _EmpId, _Total, 0, _Total, 'SALE', _SaleNo, _CrtBy, _CrtDate, _UpdBy, _UpdDate,true, _CrcID, _Fisrate, _Excrate);

			INSERT INTO rcvd(rcvno,rcvdno, rcvdmtd, cashid,rcvdcash,rcvdamt,refno,rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, 1, 'CASH', _CashID, _TotRcvAmt, _TotRcvAmt, _SaleNo, 'Tunai', _CrcID, _ExcRate, _FisRate);

			_Counter:=2;

			IF (_TotDebitAmt>0) THEN
				INSERT INTO rcvd(rcvno,rcvdno, rcvdmtd, cashid,rcvdcash,rcvdamt,refno,rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'DBC', _CashID, _TotDebitAmt, _TotDebitAmt, _SaleNo, 'Debit Card', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
			END IF;

			IF (_TotCreditAmt>0) THEN
				INSERT INTO rcvd(rcvno,rcvdno, rcvdmtd, cashid,rcvdcash,rcvdamt,refno,rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'CRC', _CashID, _TotCreditAmt, _TotCreditAmt, _SaleNo, 'Credit Card', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
			END IF;

			IF (_TotVcrAmt>0) THEN
				INSERT INTO rcvd(rcvno,rcvdno, rcvdmtd, cashid,rcvdcash,rcvdamt,refno,rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'VOUC', _CashID, _TotVcrAmt, _TotVcrAmt, _SaleNo, 'Voucher', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
			END IF;

			IF (_TotChgAmt>0) THEN
				INSERT INTO rcvd(rcvno,rcvdno, rcvdmtd, cashid,rcvdcash,rcvdamt,refno,rcvdnote, crcid, excrate, fisrate) VALUES(_RcvNo, _Counter, 'CASH', _CashID, _TotChgAmt*(-1), _TotChgAmt*(-1), _SaleNo, 'Kembalian', _CrcID, _ExcRate, _FisRate);
				IF FOUND THEN
					_Counter:=_Counter+1;
				END IF;
			END IF;
			*/
		INSERT INTO spaid(rcvno,spaidno, reftype, refno,spaiddiscamt,spaidamt) VALUES(_RcvNo, 1, 'SALE', _SaleNo, 0, _Total);

		SELECT INTO _spret * from spRcv_New(_RcvNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	ELSIF (_TermType='R') THEN
		_DebtNote:='Penjualan '||fBPName(_CustID)||':'||_SaleNo;
		SELECT INTO _spret * FROM spBPA_New(
			_CustID, 'C', _CrcID, _SaleDate, 'SALE', _SaleNo, 
			_Total, _DebtNote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	/*------------------ Set PaidAmt & PayStatus ---------------------------*/
	/** TODO: optimasi set di BL supaya tidak memberati backend **/
	IF (_TermType='C') THEN	/* including POS, CANVAS, and STD-CASH */
		_tmp_va1:='F';
		UPDATE Sale SET PaidAmt=_Total, PayStatus=_tmp_va1 WHERE SaleNo=_SaleNo;
	ELSEIF (_TermType='R') THEN
		_tmp_va1:='N';
		UPDATE Sale SET PaidAmt=0, PayStatus=_tmp_va1 WHERE SaleNo=_SaleNo;
	END IF;

	OPEN cSaleD2 FOR
		SELECT SONo, SODNo, ItemID, PID, fQtyZToQty1(ItemID, Qty, Unit)
		FROM SaleD 
		WHERE 
			SaleNo=_SaleNo 
			AND (SONo IS NOT NULL AND SONo<>'') 
			AND (DELINo IS NULL OR DeliNo='');
	LOOP
		FETCH cSaleD2 INTO _SONo, _SODNo, _ItemID, _PID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Rcv * FROM spSO_Item_Delivered(_SONo, _ItemID, _PID, _Qty1, _SODNo);
		IF (_Qty1Rcv<_Qty1) THEN
			_retval.ErrNo:=50018;
			_retval.ErrMsg:='Item:'||COALESCE(_ItemID,'<kosong>')||', Qty Delivery > QtySO (spDeli_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSaleD2;

	OPEN cSO FOR
		SELECT DISTINCT SONo FROM SaleD 
		WHERE SaleNo=_SaleNo 
		AND (SONo IS NOT NULL AND SONo<>'')
		AND (DELINo IS NULL OR DeliNo='')
		ORDER BY SONo;
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _CustID2 CustID FROM SO WHERE SONo=_SONo;
		IF (_CustID<>_CustID2) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Customer SO dan Penjualan tidak sama (spSale_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		

		SELECT INTO _spret * FROM spSO_ChNUp_Status(_SONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSO;

	/** IF _ConsNo IS NOT NULL THEN
		OPEN cSaleD FOR
			SELECT itemid, fQtyZToQty1(itemid, qty, unit), QtyX from saled where saleno=_SaleNo;
			LOOP
				FETCH cSaleD INTO _ItemID, _Qty, _QtyX;
				EXIT WHEN NOT FOUND;
				UPDATE consd SET QtyXSellOut=QtyXSellOut+_QtyX,qtysellout=qtysellout+_Qty WHERE consno=_ConsNo AND itemid=_ItemID;
			END LOOP;
		CLOSE cSaleD;
	END IF; **/

	/** Update Cons **/
	OPEN cSaleD FOR
		SELECT ItemID, fQtyZToQty1(ItemID, Qty, Unit), QtyX, ConsNo, PID 
		FROM SaleD  
		WHERE SaleNo=_SaleNo AND ConsNo IS NOT NULL;
		LOOP
			FETCH cSaleD INTO _ItemID, _Qty, _QtyX, _ConsNo, _PID;
			EXIT WHEN NOT FOUND;
			
			IF (_PID IS NOT NULL AND _PID!='') THEN
				SELECT INTO _CountPID COUNT(pid) FROM saled WHERE saleno=_SaleNo AND pid=_PID;
				IF (_CountPID>1) THEN
					_retval.ErrNo:=50555;
					_retval.ErrMsg:='Duplicate PID#'||_PID||' (spSale_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			IF (_PID IS NULL) THEN
				UPDATE ConsD SET QtyXSellOut=QtyXSellOut+_QtyX, QtySellOut=QtySellOut+_Qty
				WHERE ConsNo=_ConsNo AND ItemID=_ItemID;
			ELSE
				UPDATE ConsD SET QtyXSellOut=QtyXSellOut+_QtyX, QtySellOut=QtySellOut+_Qty
				WHERE ConsNo=_ConsNo AND ItemID=_ItemID AND PID=_PID;			
			END IF;
		END LOOP;
	CLOSE cSaleD;

	OPEN cSaleCons FOR
		SELECT DISTINCT ConsNo FROM SaleD WHERE SaleNo=_SaleNo;
		LOOP
			FETCH cSaleCons INTO _ConsNo;
			EXIT WHEN NOT FOUND;
			IF _ConsNo IS NOT NULL THEN
				SELECT INTO _spret * FROM spSetConsStatus(_ConsNo);
				IF _spret.ErrNO<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END LOOP;
	CLOSE cSaleCons;
	END IF; /** END IF ISDRAFT */

/** /////////////// AKUNTANSI ////////////////////// */
	IF (isAccInstalled() AND _isDraft=FALSE) THEN
		_JnlNo1:=fTransNoGet('JURNAL');
		_JnlNote:='Penjualan '||fBPName(_CustID)||':'||_SaleNo;

		IF(_SaleType='B')THEN
			_JnlNote:='Beggining Balance Piutang '||fBPName(_CustID)||':'||_SaleNo;
			_JnlType:='BEG';
		ELSE
			_JnlType:='SALE';
		END IF;

		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo1, _JnlType, _SaleDate,'SALE', _SAleNo,
			_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF(_SaleType<>'B') THEN
			_JnlNo2:=fTransNoGet('JURNAL');
			_JnlNote:= 'HPP '||fBPName(_CustID)||':'||_SaleNo;
			SELECT INTO _spret * FROM spJnl_Insert(
				_JnlNo2, 'SALE', _SaleDate,'SALE', _SAleNo,
				_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
				_CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		IF _TermType='C' THEN
			_AccAsset:=fAccCash(_CashID);
			IF (_Unreconciled>0) THEN
				_AccUnreconciled:=fAccDefault('UNRECBANK');
				IF (_AccUnreconciled IS NULL OR _AccUnreconciled='') THEN
					_retval.ErrNo:=50555;
					_retval.ErrMsg:='Akun Default Untuk unreconciled bank belum diset (spSale_New_POS)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		/*
		ELSIF (_ConsNo IS NOT NULL) THEN
			_AccAsset:=fAccCash(_CashID);
		*/
		ELSE
			_AccAsset:=fBPAcc(_CustID, _CrcID, 'AR');
		END IF;

		IF(_SaleType='B') THEN
			_AccAsset:=_AccAR;
		END IF;

		_Count1:=0;
		_Count2:=0;

		/******************* JURNAL SALE: DEBET ***************************/
		IF (_TermType='C') THEN
			IF (_Reconciled>0) THEN
				_Count1:=_Count1+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccAsset, 'D', ROUND(_Reconciled*_ExcRate, _Round), _DeptID, _PrjID, _AccAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			IF (_Unreconciled>0) THEN
				_Count1:=_Count1+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccUnreconciled, 'D', ROUND(_Unreconciled*_ExcRate, _Round), _DeptID, _PrjID, _AccAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		ELSIF (_TermType='R') THEN
			_Count1:=_Count1+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccAsset, 'D', ROUND(_Total*_ExcRate, _Round), _DeptID, _PrjID, _Total);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		SELECT INTO _TotalBasePrice Sum(ROUND(qty*(BasePrice-COALESCE(DiscAmt,0))*_ExcRate, _Round)) 
		FROM saled WHERE SaleNo=_SaleNo AND (fItemAcc(ItemID, 'SALE') IS NOT NULL AND fItemAcc(ItemID, 'SALE')<>'');

		_SelisihBasePrice:=(_Subtotal*_ExcRate)-_TotalBasePrice;
		_iterLoop:=0;

		/******************* JURNAL SALE: KREDIT **************************/
		OPEN cSaleAcc FOR
			SELECT fItemAcc(ItemID, 'SALE'), DeptID, PrjID, Sum(ROUND(qty*(BasePrice-COALESCE(DiscAmt,0))*_ExcRate, _Round)) 
			FROM SaleD 
			WHERE SaleNo=_SaleNo 
			GROUP BY fItemAcc(ItemID, 'SALE'), DeptID, PrjID;
		LOOP
			FETCH cSaleAcc INTO _AccSale, _DeptID, _PrjID, _AccAmt;
			EXIT WHEN NOT FOUND;
		
			_Count1:=_Count1+1;

			IF(_iterLoop=0) THEN
				_AccAmt:=_AccAmt+_SelisihBasePrice;
			END IF;
			_iterLoop:=_iterLoop+1;
		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccSale, 'K', _AccAmt, _DeptID, _PrjID, _AccAmt);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSaleAcc;

		_Disc:=COALESCE(_Disc, 0);
		IF (_Disc<>0) THEN
			IF (_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('SALEDISC');
			END IF;

			_Count1:=_Count1+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccDisc, 'K', ROUND(_Disc*_ExcRate*-1, _Round), NULL, NULL, ROUND(_Disc*_ExcRate*-1, _Round));
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		SELECT INTO _TotalTax SUM(ROUND(d.taxamt*Qty*_ExcRate, _RoundTax)) 
		FROM SaleD d JOIN tax ON d.taxid=tax.taxid
		WHERE d.SaleNo=_SaleNo and d.taxid is not null;

		_SelisihTax:=(_MTaxAmt*_ExcRate)-_TotalTax;
		_iterLoop:=0;

		/*POSTING TAX DI GRUP IN SESUAI PAJAK TIAP2 ITEM*/
		OPEN cTax FOR
			SELECT Round(sum(d.taxamt*Qty*_ExcRate),_Round),accsaletax,deptid,prjid
			FROM SaleD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.SaleNo=_SaleNo and d.taxid is not null
			GROUP BY accsaletax,deptid,prjid;
			LOOP
				FETCH cTax INTO _SaleDTaxAmt,_AccSaleTax,_DeptID,_PrjID;
				EXIT WHEN NOT FOUND;

				IF(_iterLoop=0) THEN
					_SaleDTaxAmt:=_SaleDTaxAmt+_SelisihTax;	
				END IF;
				_iterLoop:=_iterLoop+1;
				_Count1:=_Count1+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1,_Count1,_AccSaleTax,'K',_SaleDTaxAmt,_DeptID,_PrjID, _SaleDTaxAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
		CLOSE cTax;

		/*IF (_MTaxAmt<>0) THEN
			_Count1:=_Count1+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, fAccDefault('SALETAX'), 'K', _MTaxAmt*_ExcRate, NULL, NULL);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;*/

		IF (_Freight<>0) THEN
			_Count1:=_Count1+1;

			IF (_AccFreight IS NULL) THEN
			_AccFreight:=fAccDefault('SALEFREIGHT');
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccFreight, 'K', _Freight*_ExcRate, NULL, NULL, _Freight*_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		IF (_Others<>0) THEN
			_Count1:=_Count1+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, fAccDefault('PKROUND'), 'K', _Others*_ExcRate, NULL, NULL, _Others*_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		IF(_SaleType<>'B') THEN

		/*Ngitung Selisih antara D/K...Klo ada selisih dimasukin ke akun HPP*/
		SELECT INTO _SumD coalesce(Round(sum(Qty*Cost),_Round),0) 
		FROM SaleD LEFT JOIN Item ON SaleD.ItemID=Item.ItemID
		WHERE SaleNo=_SaleNo AND isstock=true AND consno IS NULL AND (fItemAcc(SaleD.ItemID, 'COST') IS NOT NULL AND fItemAcc(SaleD.ItemID, 'COST')<>'');

		SELECT INTO _SumKDeli coalesce(Round(sum(Cost*Qty),_Round),0) FROM SaleD WHERE DeliNo IS NOT NULL AND DeliNo<>'' and saleno=_SaleNo; 

		SELECT INTO _SumK coalesce(Round(sum(Qty*Cost),_Round),0) 
		FROM SaleD LEFT JOIN Item ON SaleD.ItemID=Item.ItemID
		WHERE SaleNo=_SaleNo AND isstock=true AND consno IS NULL AND delino IS NULL AND (fItemAcc(SaleD.ItemID, 'INV') IS NOT NULL AND fItemAcc(SaleD.ItemID, 'INV')<>'');

		_SelisihHPP:=_SumD-(_SumKDeli+_SumK);
		
		_CountSelisihHPP:=0;

		/*--------------- proses jurnal untuk HPP: DEBET (buat Non Konsinyasi)----------------------------*/
		OPEN cSaleAcc FOR
			SELECT fItemAcc(d.itemid, 'COST'), d.deptid, d.prjid, SUM(ROUND(fUnitCost(d.itemid, d.whid, d.pid, d.qty, d.unit), _Round))
			FROM saled d 
			JOIN item i ON d.itemid=i.itemid
			WHERE saleno=_SaleNo AND (i.isstock=TRUE OR i.itemtype='PACK') 
			AND consno IS NULL
			GROUP BY fItemAcc(d.itemid, 'COST'), d.deptid, d.prjid;
		LOOP
			FETCH cSaleAcc INTO _AccCOGS, _DeptID, _PrjID, _AccAmt;
			EXIT WHEN NOT FOUND;
			
			IF(_AccCOGS IS NOT NULL) THEN
				_Count2:=_Count2+1;

				IF(_CountSelisihHPP=0) THEN
					_AccAmt:=_AccAmt-_SelisihHPP;
					_CountSelisihHPP:=_CountSelisihHPP+1;
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, _AccCOGS, 'D', _AccAmt, _DeptID, _PrjID, _AccAmt);

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END LOOP;
		CLOSE cSaleAcc;

		/*--------------- proses jurnal untuk HPP: KREDIT (buat NON Konsinyasi)----------------------------*/
		IF (_useDeli) THEN	
			_Count2:=_Count2+1;
			SELECT INTO _UBDeliAmt SUM(ROUND(Cost*Qty, _Round)) 
			FROM SaleD 
			WHERE DeliNo IS NOT NULL AND DeliNo<>'' and saleno=_SaleNo;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, fAccDefault('UBDELI'), 'K', _UBDeliAmt, _DeptID, _PrjID, _UBDeliAmt);

		END IF;

		OPEN cSaleAcc FOR
			SELECT accno, deptid, prjid, SUM(accamt) 
			FROM (
				SELECT fItemAcc(d.ItemID, 'INV') AS accno, SUM(ROUND(fUnitCost(d.itemid, d.whid, d.pid, d.qty, d.unit), _Round)) AS accamt, 
				d.DeptID, d.PrjID 
				FROM SaleD d JOIN Item i ON d.ItemID=i.ItemID
				WHERE d.SaleNo=_SaleNo AND i.isStock=true 
				AND (d.ConsNo IS NULL OR d.ConsNo='') 
				AND (d.DeliNo IS NULL OR d.DeliNo='')
				GROUP BY fItemAcc(d.ItemID, 'INV'), d.DeptID, d.PrjID
				UNION ALL

				SELECT f.accno, ROUND(f.accamt, _Round), f.deptid, f.prjid 
				FROM spJnlPacketInv(_SaleNo) f
			) a 
			GROUP BY accno, deptid, prjid;
		LOOP
			FETCH cSaleAcc INTO _AccInv, _DeptID, _PrjID, _AccAmt;
			EXIT WHEN NOT FOUND;
			
			IF(_AccInv IS NOT NULL) THEN
				_Count2:=_Count2+1;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, _AccInv, 'K', _AccAmt, _DeptID, _PrjID, _AccAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END LOOP;
		CLOSE cSaleAcc;

		/*------Proses jurnal buat HPP yang laku (khusus konsinyasi)-----*/
		IF(_CountCons>0) THEN
		_JnlNo3:=fTransNoGet('JURNAL');
		_JnlNote:='HPP '||fBPName(_CustID)||':'||_SaleNo;

		_Count3:=0;
		_Count4:=0;
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo3, _JnlType, _SaleDate,'SALE', _SAleNo,
			_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		OPEN cCons FOR
			SELECT fItemAcc(SaleD.ItemID, 'COST'), SaleD.DeptID, SaleD.PrjID, 
			Round(Sum(Qty*SaleD.Cost),_Round) 
			FROM SaleD LEFT JOIN CONSD ON SaleD.consno=consd.consno AND SaleD.ItemID=consd.ItemID
			WHERE SaleNo=_SaleNo AND SaleD.consno IS NOT NULL
			GROUP BY fItemAcc(SaleD.ItemID, 'COST'), SaleD.DeptID, SaleD.PrjID;
			LOOP
				FETCH cCons INTO _AccCOGS,_DeptID,_PrjID,_AccAmt;
				EXIT WHEN NOT FOUND;

				IF(_AccCOGS IS NOT NULL) THEN
				_Count3:=_Count3+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo3, _Count3, _AccCOGS, 'D', _AccAmt, _DeptID, _PrjID, _AccAmt);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

				_Count3:=_Count3+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo3, _Count3, fAccDefault('ARCONS'), 'K', _AccAmt, _DeptID, _PrjID, _AccAmt);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;
			END LOOP;
		CLOSE cCons;

		/*------Proses jurnal buat Inventory yang balik (khusus konsinyasi)-----*/
		_JnlNo4:=fTransNoGet('JURNAL');
		_JnlNote:='Inventory '||fBPName(_CustID)||':'||_SaleNo;

		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo4, _JnlType, _SaleDate,'SALE', _SAleNo,
			_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		OPEN cCons FOR
			SELECT fItemAcc(SaleD.ItemID, 'INV'), SaleD.DeptID, SaleD.PrjID, 
			Round(Sum((QtySellIn-Qty)*SaleD.Cost),_Round) 
			FROM SaleD LEFT JOIN CONSD ON SaleD.consno=consd.consno AND SaleD.ItemID=consd.ItemID
			WHERE SaleNo=_SaleNo AND SaleD.consno IS NOT NULL
			GROUP BY fItemAcc(SaleD.ItemID, 'INV'), SaleD.DeptID, SaleD.PrjID;
			LOOP
				FETCH cCons INTO _AccInv,_DeptID,_PrjID,_AccAmt;
				EXIT WHEN NOT FOUND;

				IF(_AccInv IS NOT NULL) THEN
				_Count4:=_Count4+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo4, _Count4, _AccInv, 'D', _AccAmt, _DeptID, _PrjID, _AccAmt);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

				_Count4:=_Count4+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo4, _Count4, fAccDefault('ARCONS'), 'K', _AccAmt, _DeptID, _PrjID, _AccAmt);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;
			END LOOP;
		CLOSE cCons;
		END IF;
		END IF;

		/*---------- execute spJnlNew --------------*/
		SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF(_SaleType<>'B') THEN
			IF(_Count2>0) THEN
				SELECT INTO _spret * FROM spJnl_New(_JnlNo2);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSE
				DELETE FROM jnl WHERE jnlno=_JnlNo2;
			END IF;
			IF(_CountCons>0) THEN
				SELECT INTO _spret * FROM spJnl_New(_JnlNo3);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				SELECT INTO _spret * FROM spJnl_New(_JnlNo4);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSale_Void (
	_SaleNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval				SPStatus%ROWTYPE;
	spretspStockA_Void_byRef	SPStatus%ROWTYPE;
	spretspCashA_Void_byRef		SPStatus%ROWTYPE;
	spretspBPA_Void_byRef		SPStatus%ROWTYPE;
	spretSO_Item_Cancel		SPStatus%ROWTYPE;
	spretspSO_ChNUp_Status		SPStatus%ROWTYPE;
	spret				SPStatus%ROWTYPE;
	spretSisaCons 			SPStatus%ROWTYPE;
	_spret				SPStatus%ROWTYPE;

	_SaleType			VARCHAR;
	_TermType			VARCHAR(1);
	_Term				INT;
	_PoSPayMtd			VARCHAR(4);
	_PayStatus			VARCHAR(1);
	_SONo				VARCHAR(15);
	_DeliNo				VARCHAR(15);
	_RcvNo				VARCHAR;
	_JnlNo				VARCHAR(15);
	/** SO stuff **/
	_ItemID				VARCHAR(20);
	_Qty1				DECIMAL(15,4);
	_Qty1Cancel			DECIMAL(15,4);
	_Qty1Tot			DECIMAL(15,4);
	_ConsNo				VARCHAR(15);
	_ConsDNo			SMALLINT;
	_Qty				DECIMAL(15,4);
	_QtyX				DECIMAL(15,4);

	cDeliSale			REFCURSOR;
	cSO 				REFCURSOR;
	cSaleD				REFCURSOR;
	cSaleD2				REFCURSOR;
	cAcc				REFCURSOR;
	cSaleCons			REFCURSOR;
	_regval				VARCHAR(30);
	_PID				VARCHAR;

	_PoSSesNo			VARCHAR;
	_Total				NUMERIC(19,4);

	_isDraft			BOOLEAN;
	_SaleDate			DATE;
	_LockDate			DATE;
	_SODNo				INT2;

	__ChangeAmt			NUMERIC(19,4); /** kembalian */

	_isCollect			BOOLEAN; /** status pos sessionnya sudah diclose atau belum */

	_SRetCount			INT2;
	_SRetNos			VARCHAR;
	_RcvNos				VARCHAR;

BEGIN
/**
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
	RETURN;

**/
	SELECT INTO 
		_isDraft, _SaleType,_TermType,_Term,_PayStatus,_SONo,_ConsNo, _PoSSesNo, _Total, _SaleDate
		isDraft, SaleType,TermType,duedays,PayStatus,SONo,ConsNo, PoSSesNo, Total, SaleDate
	FROM Sale WHERE SaleNo=_SaleNo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Penjualan % tidak ditemukan (spSale_Void)', _SaleNo;
	END IF;

	SELECT INTO _spret * FROM spCheckDP('SALE', _SaleNo);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM SOSale WHERE SaleNo=_SaleNo;
		DELETE FROM SaleBns WHERE SaleNo=_SaleNo;
		DELETE FROM SaleCRcv WHERE SaleNo=_SaleNo;
		DELETE FROM SaleD WHERE SaleNo=_SaleNo;
		DELETE FROM Sale WHERE SaleNo=_SaleNo;


		_retval:=(0,'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SALE';
	IF(_LockDate is not null AND _LockDate>_SaleDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penjualan Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _SRetCount, _SRetNos COUNT(*), SUM(DISTINCT sretno) FROM sretd WHERE saleno=_SaleNo;
	IF (FOUND AND _SRetCount>0) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:=E'Penjualan ini sudah dipakai dalam transaksi retur penjualan:\n'||COALESCE(_SRetNos, 'BATCH')||E'\nVoid terlebih dahulu retur penjualan tersebut ! (spSale_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF((_TermType='R') AND (_PayStatus<>'N')) THEN
		SELECT INTO _RcvNos SUM(DISTINCT rcvno) FROM spaid WHERE refno=_SaleNo;
		_retval.ErrNo='50002';
		_retval.ErrMsg=E'Sudah ada transaksi penerimaan pembayaran:\n'||COALESCE(_RcvNos, 'BATCH')||E'\nVoid terlebih dahulu penerimaan pembayaran tersebut ! (spSale_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	 END IF;

	 -- Penjualan lewat Deli ? VOID status Deli
	 OPEN cDeliSale FOR
	 	SELECT DeliNo FROM SaleD 
		WHERE SaleNo=_SaleNo 
		AND (DeliNo IS NOT NULL AND DeliNo<>'');
	 LOOP
		FETCH cDeliSale INTO _DeliNo;
		EXIT WHEN NOT FOUND;

		UPDATE deli SET IsInvoiced=false WHERE DeliNo=_DeliNo;

		DELETE FROM DeliSale WHERE DeliNo=_DeliNo AND SaleNo=_SaleNo;
	 END LOOP;
	 CLOSE cDeliSale;
	 
	-- VOID STOCK transaction untuk yang PoS atau non-Deli 
	SELECT INTO spretspStockA_Void_byRef * FROM spStockA_Void_byRef('SALE', _SaleNo);
	IF spretspStockA_Void_byRef.ErrNo<>0 THEN
		_retval.ErrNo:=spretspStockA_Void_byRef.ErrNo;
		_retval.ErrMsg:=spretspStockA_Void_byRef.ErrMsg;			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 

	IF (_PoSSesNo IS NOT NULL AND _TermType='C') THEN
		SELECT INTO _isCollect iscollect FROM posses WHERE possesno=_PoSSesNo;
		IF (_isCollect=TRUE) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='POS Session#'||_PoSSesNo||' sudah disetorkan, tidak diperkenankan untuk edit (sp_Sale_Void)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		SELECT INTO __ChangeAmt rcvamt FROM salecrcv WHERE rcvtype='CHG' AND saleno=_SaleNo;
		IF NOT FOUND THEN
			__ChangeAmt:=0;
		END IF;
		UPDATE POSSES SET TotIn=(TotIn-_Total-__ChangeAmt), EndBal=(EndBal-_Total), TotOut=(TotOut-__ChangeAmt) WHERE PoSSesNo=_PoSSesNo;
	END IF;


	IF (_TermType='C') THEN
		/*SELECT into _spret * FROM spCashA_Void_ByRef('SALE', _SaleNo, NULL, NULL);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;*/
		/** A. Void pembayaran auto-nya */ 	
		SELECT INTO _RcvNo RcvNo FROM Rcv WHERE RefType='SALE' AND RefNo=_SaleNo;

		IF _RcvNo IS NOT NULL THEN
			SELECT INTO spret * from spRcv_Void(_RcvNo);
			
			IF spret.ErrNo<>0 THEN
					_retval.ErrNo:=spret.ErrNo;
					_retval.ErrMsg:=spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrNo;
			END IF;
		END IF;
	
	ELSIF (_TermType='R') THEN
			SELECT INTO spretspBPA_Void_byRef * FROM spBPA_Void_byRef('SALE',_SaleNo);
			IF spretspBPA_Void_byRef.ErrNo<>0 THEN
				_retval.ErrNo:=spretspBPA_Void_byRef.ErrNo;
				_retval.ErrMsg:=spretspBPA_Void_byRef.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
	 END IF;
	
	/**
	IF _ConsNo IS NOT NULL THEN
		OPEN cSaleD FOR
			SELECT itemid, fQtyZToQty1(itemid, qty, unit), QtyX from saled where saleno=_SaleNo;
			LOOP
				FETCH cSaleD INTO _ItemID, _Qty, _QtyX;
				EXIT WHEN NOT FOUND;
				UPDATE consd SET QtyXSellOut=QtyXSellOut-_QtyX,qtysellout=qtysellout-_Qty WHERE consno=_ConsNo AND itemid=_ItemID;
			END LOOP;
		CLOSE cSaleD;
	END IF; 

	
	IF _ConsNo IS NOT NULL THEN
		SELECT INTO spretSisaCons * FROM spSetConsStatus(_ConsNo);
		IF spretSisaCons.ErrNO<>0 THEN
			_retval.ErrNo:=spretSisaCons.ErrNo;
			_retval.ErrMsg:=spretSisaCons.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF; **/
	
	/** Update Cons **/
	OPEN cSaleD FOR
		SELECT ItemID, fQtyZToQty1(ItemID, Qty, Unit), QtyX, ConsNo, ConsDNo, PID 
		FROM SaleD  
		WHERE SaleNo=_SaleNo AND ConsNo IS NOT NULL;
	LOOP
		FETCH cSaleD INTO _ItemID, _Qty, _QtyX, _ConsNo, _ConsDNo, _PID;
		EXIT WHEN NOT FOUND;

		IF (_PID IS NULL) THEN
			UPDATE ConsD SET QtyXSellOut=QtyXSellOut-_QtyX, QtySellOut=QtySellOut-_Qty
			WHERE ConsNo=_ConsNo AND ItemID=_ItemID AND ConsDNo=_ConsDNo;
		ELSE
			UPDATE ConsD SET QtyXSellOut=QtyXSellOut-_QtyX, QtySellOut=QtySellOut-_Qty
			WHERE ConsNo=_ConsNo AND ItemID=_ItemID AND PID=_PID AND ConsDNo=_ConsDNo;			
		END IF;
	END LOOP;
	CLOSE cSaleD;

	OPEN cSaleCons FOR
		SELECT DISTINCT ConsNo FROM SaleD WHERE SaleNo=_SaleNo;
	LOOP
		FETCH cSaleCons INTO _ConsNo;
		EXIT WHEN NOT FOUND;
		IF _ConsNo IS NOT NULL THEN
			SELECT INTO _spret * FROM spSetConsStatus(_ConsNo);
			IF _spret.ErrNO<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			
			DELETE FROM ConsSale WHERE ConsNo=_ConsNo AND saleno=_SaleNo;
			/*IF NOT FOUND THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Error Delete ConsSale ConsNo#'||COALESCE(_ConsNo,'ConsNo')||' (spSale_Void)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;*/
		END IF;
	END LOOP;
	CLOSE cSaleCons;

	OPEN cSaleD2 FOR
		SELECT SONo, SODNo, ItemID, PID, fQtyZToQty1(ItemID, Qty, Unit)
		FROM SaleD WHERE 
			SaleNo=_SaleNo 
			AND (SONo IS NOT NULL AND SONo<>'')
			AND (DELINo IS NULL OR DeliNo='');
	LOOP
		FETCH cSaleD2 INTO _SONo, _SODNo, _ItemID, _PID, _Qty1;
		EXIT WHEN NOT FOUND;

		_Qty1Cancel:=spSO_Item_Cancel(_SONo, _ItemID, _PID, _Qty1, _SODNo);
	END LOOP;
	CLOSE cSaleD2;

 	OPEN cSO FOR
		SELECT DISTINCT SONo FROM SaleD 
		WHERE 
		SaleNo=_SaleNo 
		AND (SONo IS NOT NULL AND SONo<>'')
		AND (DELINo IS NULL OR DeliNo='')
		ORDER BY SONo;
 	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;
	
		SELECT INTO spretspSO_ChNUp_Status * FROM spSO_ChNUp_Status(_SONo);
		IF spretspSO_ChNUp_Status.ErrNo<>0 THEN
			_retval.ErrNo:=spretspSO_ChNUp_Status.ErrNo;
			_retval.ErrMsg:=spretspSO_ChNUp_Status.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

 	END LOOP;
 	CLOSE cSO;

	IF (isAccInstalled()) THEN
		SELECT into spretSisaCons * from spJnl_Void_byRef('SALE',_SaleNo);
		IF spretSisaCons.ErrNo<>0 THEN
			_retval.ErrNo:=spretSisaCons.ErrNo;
			_retval.ErrMsg:=spretSisaCons.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM conssale WHERE saleno=_SaleNo;
	DELETE FROM SOSale WHERE SaleNo=_SaleNo;
	DELETE FROM SaleBns WHERE SaleNo=_SaleNo;
	DELETE FROM SaleCRcv WHERE SaleNo=_SaleNo;
	DELETE FROM SaleD WHERE SaleNo=_SaleNo;
	DELETE FROM Sale WHERE SaleNo=_SaleNo;

	 _retval:=(0, 'OK');
	 RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSRet_New (
	_SRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CustID			VARCHAR(15);
	_ConsNo			VARCHAR(15);
	_SRetDate		DATE;
	_TermType		VARCHAR(1);
	_Total			NUMERIC(19,4);
	_CashID			VARCHAR(15);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;

	_SRetDNo		SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_Conv			NUMERIC(15,4);
	_QtyX			NUMERIC(15,4);
	_Cost			NUMERIC(19,4);
	_ItemType		VARCHAR(5);

	_StockANo		VARCHAR(15);
	_StockANote		VARCHAR(255);
	_StockADNo		SMALLINT;

	_SRetQty		NUMERIC(15,4);
	_SRetQtyX		NUMERIC(15,4);
	_ItemsCost		NUMERIC(19,4);

	_CashAmt		NUMERIC(19,4);
	_CashANote		VARCHAR(255);

	_IsContra		BOOLEAN;
	_regval			VARCHAR(30);

	_RcvNo			VARCHAR(15);
	_CrcID			VARCHAR(5);

	_SpaidNo		INT2;
	_SpaidAmt		NUMERIC(19,4);
	_Bayar			NUMERIC(19,4);

	_Freight		DECIMAL(19,4);
	_Others			DECIMAL(19,4);
	_Disc			DECIMAL(19,4);
	_Count1			INT;
	_Count2			INT;
	_JnlNo1			VARCHAR(15);
	_JnlNo2			VARCHAR(15);
	_JnlNote		VARCHAR;
	_ListPrice		DECIMAL(19,4);
	_DebtAmt		DECIMAL(19,4);
	_ExcRate		DECIMAL(19,4);
	_DebtNote		VARCHAR(50);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
   	_DeptID     		VARCHAR(5);
   	_PrjID      		VARCHAR(5);

	cSRetD			REFCURSOR;
	cSRetD2			REFCURSOR;
	cSRet			REFCURSOR;
	cSRetAcc		REFCURSOR;
	
    	_AccAsset		VARCHAR(10);
    	_AccDisc		VARCHAR(10);
    	_AccInv			VARCHAR(10);
    	_AccCOGS		VARCHAR(10);
    	_AccFreight		VARCHAR(10);
	_AccSRet		VARCHAR(10);
	_AccAmt			DECIMAL(19,4);    
	_MTaxAmt		DECIMAL(19,4);  
  	_countDeli		INT;
	_UBDeliAmt		DECIMAL(19,4);
	_Round			INT;
	_RoundTax		INT;
	

/** buat SRetdeli **/
	cSRetDeli		REFCURSOR;
	_DeliNo			VARCHAR(15);
	_SaleNo			VARCHAR;
	_SaleDNo		SMALLINT;

	_unit1			VARCHAR;
	_unit2			VARCHAR;
	_unit3			VARCHAR;
	_conv2			DECIMAL;
	_conv3			DECIMAL;
	_listpriceAsli		DECIMAL;
	_QtyAsli		DECIMAL;
	_UnitAsli		VARCHAR;

	_qty_saled		DECIMAL;
	_qty_delid		DECIMAL;
	_qty_sretd		DECIMAL;

	cSale			REFCURSOR;
	cTax			REFCURSOR;

	_isDraft		BOOLEAN;
	_LockDate		DATE;

	_ItemIDCheck		VARCHAR;
	_SRetDTaxAmt		NUMERIC(19,4);
	_AccSRetTax		VARCHAR;
	_TotalTax		NUMERIC(19,4);
	_SelisihTax		NUMERIC(19,4);
	_i			INT;
	_Subtotal		NUMERIC(19,4);
	_SelisihBasePrice	NUMERIC(19,4);
	_TotalBasePrice		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_SRetStatusYes		VARCHAR(1);

	_BTotal			NUMERIC(19,4); -- base currency total
	_BSubtotal		NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt		NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt		NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt		NUMERIC(19,4);
	_TotalTaxAmt		NUMERIC(19,4);
	_BTotalTaxAmt		NUMERIC(19,4);
	_TotalDiscAmt		NUMERIC(19,4);
	_TotalDisc2Amt		NUMERIC(19,4);
	_DSubtotal		NUMERIC(19,4);
	_DBSubtotal		NUMERIC(19,4);
	
	_AccRateLR		VARCHAR; -- akun laba/rugi selisih kurs
	_RateLR			NUMERIC(19,4); -- nilai laba/rugi selisih kurs

	_CashCrcID		VARCHAR;
	_BoMID			bomd.bomid%TYPE;
	_StockAType		VARCHAR(5):='SRET';
	_InOut			CHAR(1);
	_DeliDNo		INT2;
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_SRetStatusYes:='Y';
	_InOut:='O';

	SELECT INTO
		_isDraft, _PerID, _BranchID, _CustID, _SRetDate, _TermType, _BTotal, _Total,_CrcID, _CashID, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _ConsNo, _Freight, _MBDiscAmt, _Disc, _AccDisc, _AccFreight, _ExcRate, _MBFTaxAmt, _MTaxAmt,_Others,_IsContra, _BSubtotal, _SubTotal, _FisRate
		isDraft, PerID, BranchID, CustID, SRetDate, TermType, basetotal, Total, crcid, CashID, CrtBy, CrtDate, UpdBy, UpdDate, ConsNo, freight, basediscamt, discamt, accsdisc, accfreight, excrate, basftaxamt, TaxAmt,others,iscontra, basesubtotal, Subtotal, FisRate
	FROM SRet
	WHERE SRetNo=_SRetNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='ReturJual #'||_SRetNo||' tidak ditemukan (spSRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');
	_RoundTax:=COALESCE(fReg('ROUND_TAX'), '0');	

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SRET';
	IF (_LockDate IS NOT NULL AND _LockDate>_SRetDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='ReturJual Melebihi Batas Tanggal Kunci Transaksi ! (spSRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _countDeli COUNT(*) FROM SRetD WHERE SRetNo=_SRetNo AND DeliNo IS NOT NULL AND DeliNo<>'';


	IF (_TermType='R' AND _IsContra=TRUE) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='ReturJual sudah ada Penerimaan! (spSRet_New)';	
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;
	
	OPEN cSRetDeli FOR
		SELECT delino, delidno, itemid, whid, pid, qty, unit 
		FROM sretd 
		WHERE sretno=_SRetNo 
		AND (delino IS NOT NULL AND delino<>'');
	LOOP	
		FETCH cSRetDeli INTO _DeliNo, _DeliDNo, _ItemID, _WhID, _PID, _Qty, _Unit;	
		EXIT WHEN NOT FOUND;

		SELECT INTO _qty_delid COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM delid WHERE itemid=_ItemID AND pid=_PID AND delino=_DeliNo AND delidno=_DeliDNo;
		SELECT INTO _qty_sretd COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM sretd WHERE itemid=_ItemID AND pid=_PID AND delino=_DeliNo AND delidno=_DeliDNo;

		IF _qty_sretd>_qty_delid THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg:='Error : Qty Retur Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' tidak boleh lebih dari Qty pada Pengiriman (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- Update isInvoice=TRUE, saat ini gak bisa diInvoice pecah/parsial, jadi ALL or None
		UPDATE Deli SET isInvoiced=TRUE WHERE DeliNo=_DeliNo;

		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Error Update Pengiriman#'||_DeliNo|| !' (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSRetDeli;

	_ItemsCost:=0;
/** 
	Generate StockA
	jika countDeli = 0, artinya ndak import dari deli sama sekali, jadi generate stocka 
	jika countDeli > 0, stockanya udah didelinya
**/
	IF (_countDeli=0) THEN
		OPEN cSRetD FOR
			SELECT 
				d.SRetDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, i.ItemType, d.SaleNo, d.SaleDNo
			FROM SRetD d JOIN Item i ON d.ItemID=i.ItemID
			WHERE d.SRetNo=_SRetNo;

			--semua HPP baca dari saled, gak peduli PACK, SERV dll
			--AND (i.IsStock=TRUE OR i.ItemType='PACK');	

		_StockADNo=0;
		_i:=0;
		LOOP
			FETCH cSRetD INTO _SRetDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ItemType, _SaleNo, _SaleDNo;
			EXIT WHEN NOT FOUND;
			_i:=_i+1;

			SELECT INTO _qty_saled COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM saled WHERE itemid=_ItemID AND pid=_PID AND saleno=_SaleNo AND saledno=_SaleDNo;
			SELECT INTO _qty_sretd COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)),0) FROM sretd WHERE itemid=_ItemID AND pid=_PID AND saleno=_SaleNo AND saledno=_SaleDNo;
--RAISE EXCEPTION '% vs %', _qty_sretd, _qty_saled;
			IF _qty_sretd>_qty_saled THEN
				_retval.ErrNo:=50003;
				_retval.ErrMsg:='Error : Qty Retur Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' tidak boleh lebih dari Qty pada Penjualan (spSRet_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_i=1) THEN		-- generate StockA header
				_StockANo:=fTransNoGet('STOCKA');
				_StockANote='Retur Jual '||fBPName(_CustID)||':'||_SRetNo;
				SELECT INTO _spret * FROM spStockA_New(
					_StockANo,_CustID, 'SRET', _SRetDate, 'O', 'SRET', _SRetNo, _StockANote, TRUE, 
					_CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID								
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;		
			END IF;			

/** mbuh gak jelas kok moro2 ngupdate ConsD , mestine dikumpulin jadi 1 blok dgn yg lain

			IF (_PID IS NULL) THEN
				UPDATE ConsD SET QtyCRet=QtyCRet+_Qty, QtyXCRet=QtyXCRet+_QtyX 
				WHERE ConsNo=_ConsNo AND ItemID=_ItemID;
			ELSE
				UPDATE ConsD SET QtyCRet=QtyCRet+_Qty, QtyXCRet=QtyXCRet+_QtyX 
				WHERE ConsNo=_ConsNo AND ItemID=_ItemID 
				AND PID=_PID; 
			END IF;
**/

			_StockADNo=_StockADNo+1;

			IF (_ItemType='PACK') THEN

				SELECT INTO _BoMID BoMID FROM BoM WHERE ItemID=_ItemID;
				/**_StockADNo:=_StockADNo+1; dicomment karena sudah ada modifikasi di spstockad_new_bom_sret*/
				
				IF(_BomID IS NULL) THEN
					_retval.ErrNo:='50014';
					_retval.ErrMsg='Error tidak dapat menemukan B.o.M. untuk Item:'||_ItemID||'(spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
		
				SELECT INTO _spret * FROM spStockAD_New_BoM_SRet(
					_SRetDate, _StockAType, _StockANo, _StockADNo, _WhID, _ItemID, _PID, 
					_Qty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, _CrcID, _SaleNo, _SaleDNo, _SRetNo
				);

				_StockADNo:=_spret.ErrNo;

				IF _spret.ErrNo<>0 AND _spret.ErrMsg<>'OK' THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;			
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				_Cost:=fSale_ItemCost(_ItemID, _SaleNo, _SaleDNo);
				_ItemsCost=_ItemsCost+(_Qty*_Cost);

				IF (_Cost IS NULL) THEN
					_retval.ErrNo:=50003;
					_retval.ErrMsg:='Error menghitung HPP Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
		
				_Conv:=fItemUnit_Conv(_ItemID, _Unit);

				UPDATE SRetD SET Cost=_Cost, Conv=_Conv WHERE SRetNo=_SRetNo AND SRetDNo=_SRetDNo;
				IF NOT FOUND THEN
					_retval.ErrNo:=50010;
					_retval.ErrMsg:='Error UPDATE SRetD:Cost (spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

			ELSE

				_Cost:=fSale_ItemCost(_ItemID, _SaleNo, _SaleDNo);
				_ItemsCost=_ItemsCost+(_Qty*_Cost);

				IF (_Cost IS NULL) THEN
					_retval.ErrNo:=50003;
					_retval.ErrMsg:='Error menghitung HPP Item:'||_ItemID||', Wh:'||_WhID||', PID:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spStockAD_New(
					_SRetDate, _StockAType, _StockANo, _StockADNo, _WhID, 
					_ItemID, _PID, _Qty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, TRUE);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
		
				_Conv:=fItemUnit_Conv(_ItemID, _Unit);

				UPDATE SRetD SET Cost=_Cost, Conv=_Conv WHERE SRetNo=_SRetNo AND SRetDNo=_SRetDNo;
				IF NOT FOUND THEN
					_retval.ErrNo:=50010;
					_retval.ErrMsg:='Error UPDATE SRetD:Cost (spSRet_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

			END IF;


		END LOOP;
		CLOSE cSRetD;
	END IF;
	
/**
	OPEN cSRetD2 FOR
	SELECT d.SRetDNo, d.ItemID, d.WhID, d.PID, d.Qty, d.Unit, d.QtyX, d.SaleNo, d.SaleDNo
	FROM SRetD d JOIN Item i ON d.ItemID=i.ItemID
	WHERE 
		d.SRetNo=_SRetNo 
		AND ((i.ItemType='SERV') 
		OR (i.IsStock=FALSE AND i.IsAsm=FALSE));
	LOOP
		FETCH cSRetD2 INTO _SRetDNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _SaleNo, _SaleDNo;
		EXIT WHEN NOT FOUND;
		
		_Cost:=fUnitCost(_ItemID, _WhID, _PID, _Unit);
		IF (_Cost IS NULL) THEN
			_retval.ErrNo:=50005;
			_retval.ErrMsg:='Error menghitung HPP! Item:'||_ItemID||', Wh:-, PID:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error Update SRetD:Cost [non-stock-item](spSRet_New)';

		UPDATE SRetD SET Cost=_Cost WHERE SRetNo=_SRetNo AND SRetDNo=_SRetDNo;

		IF NOT FOUND THEN
			_retval.ErrNo:=50010;
			_retval.ErrMsg:='Error UPDATE SRetD:Cost (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_ItemsCost=_ItemsCost+(_Qty*_Cost);
	END LOOP;
	CLOSE cSRetD2;

**/

	IF (_TermType='C') THEN
		_IsContra=TRUE;
	ELSE
		_IsContra=FALSE;
	END IF;

	UPDATE SRet SET CostTot=_ItemsCost, IsContra=_IsContra WHERE SRetNo=_SRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50010;
		_retval.ErrMsg:='Error UPDATE SRet:CostTot (spSRet_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	

	IF (_TermType='C' AND _ConsNo IS NULL) THEN
		IF (_CashID IS NULL) THEN
			_retval.ErrNo:=50001;
			_retval.ErrMsg:='Error, Kas Retur Penjualan belum diset ! (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _CashCrcID crcid FROM cash WHERE cashid=_CashID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' belum diset (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
 					
		IF (_CrcID!=_CashCrcID) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' tidak sama dengan mata uang penjualan (spSRet_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;		
		END IF;
		_CashANote='Retur #'||_SRetNo;
		_CashAmt=_Total*(-1);

		SELECT INTO _spret * FROM spCashA_New(
				_CashID, _CustID, 'SRET', _SRetDate, 'O', 'SRET', _SRetNo, 
			 _CashAmt, _CashANote, TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Update SRet:SRetStatus, IsContra (spSRet_New)';
		UPDATE SRet SET SRetStatus=_SRetStatusYes, IsContra=_IsContra WHERE SRetNo=_SRetNo;

	ELSE
      	IF (_ConsNo IS NOT NULL) THEN	
			SELECT INTO _spret * FROM spSetConsStatus(_ConsNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSE
			_DebtAmt:=_Total*(-1);		--- penerimaan : potong piutang (-)
			_DebtNote:='Retur #'||_SRetNo;

			SELECT INTO _spret * FROM spBPA_New(
				_CustID, 'P',_CrcID, _SRetDate, 'SRET', _SRetNo, _DebtAmt, _DebtNote, TRUE, 
				_CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate								
			);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;	-- IF (ConsNo IS NOT NULL)

	END IF;	-- Term=C && ConsNo IS NULL

	OPEN cSale FOR
		SELECT 
			Qty, Unit, SaleNo, ItemID
		FROM SRetd 
		WHERE (SaleNo IS NOT NULL and SaleNo<>'') AND SRetNo=_SRetNo;

		LOOP
			FETCH cSale INTO _Qty,_unit,_SaleNo,_ItemID;
			EXIT WHEN NOT FOUND;

				SELECT INTO 
					_QtyAsli, _UnitAsli, _unit1, _unit2, _unit3, _conv2, _conv3
					Qty, unit, unit1, unit2, unit3, conv2, conv3
				FROM SaleD LEFT JOIN item on saled.ItemID=item.ItemID
				WHERE SaleNo=_SaleNo 
				AND saled.ItemID=_ItemID;

				IF (_UnitAsli<>_unit) THEN
					IF (_UnitAsli=_unit1) THEN
						IF (_Unit=_unit2) THEN	
							_Qty:=_Qty*_conv2;
						ELSIF (_Unit=_unit3) THEN
							_Qty:=_Qty*_conv3;
						END IF;
					ELSIF (_UnitAsli=_unit2) THEN
						IF (_Unit=_unit1) THEN
							_Qty:=_Qty/_conv2;
						ELSIF (_Unit=_unit3) THEN
							_Qty:=(_Qty*_conv3)/_conv2;
						END IF;
					ELSIF (_UnitAsli=_unit3) THEN
						IF (_Unit=_unit1) THEN
							_Qty:=_Qty/_conv3;
						ELSIF (_Unit=_unit2) THEN
							_Qty:=(_Qty*_conv2)/_conv3;
						END IF;
					END IF;
				END IF;

				IF (_Qty>_QtyAsli) THEN
					RAISE EXCEPTION '%', 'Qty ITEM:'||_ItemID||' melebihi pembelian no:'||_SaleNo;
				END IF;
		END LOOP;
	CLOSE cSale;


/* ====================================== AKUNTANSI ==================================== */
	IF isAccInstalled() THEN
		_JnlNo1:=fTransNoGet('JURNAL');
		_JnlNote:= 'Retur Jual '||fBPName(_CustID)||':'||_SRetNo;
		
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo1, 'SRET', _SRetDate, 'SRET', _SRetNo,
			_JnlNote,TRUE,TRUE,cast(0 as NUMERIC(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _Excrate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		------------------------ Ayat HPP ----------------		
		_JnlNo2:=fTransNoGet('JURNAL');
		_JnlNote:= 'HPP Retur Jual '||fBPName(_CustID)||':'||_SRetNo;
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo2, 'SRET', _SRetDate, 'SRET',_SRetNo,
			_JnlNote,TRUE,TRUE,cast(0 as NUMERIC(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _Excrate, _FisRate			
			);			
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		
		IF _TermType='C' THEN
			_AccAsset:=fAccCash(_CashID, _InOut);
/*
		ELSIF (_ConsNo IS NOT NULL) THEN
			_AccAsset:=fAccCash(_CashID, _InOut);		
*/
		ELSE
			_AccAsset:=fBpAcc(_CustID, _CrcID, 'AR');
		END IF;

		_Count1:=0;
		_Count2:=0;

		SELECT INTO 
			_TotalBasePrice 
			Round(Sum(Qty*(BasePrice-COALESCE(DiscAmt,0))*_ExcRate),_Round) 
		FROM SRetD WHERE SRetNo=_SRetNo AND (fItemAcc(ItemID, 'SRET') IS NOT NULL AND fItemAcc(ItemID, 'SRET')<>'');

		_SelisihBasePrice:=(_Subtotal*_ExcRate)-_TotalBasePrice;
		_i:=0;
		--******************* JURNAL SRet: DEBET *************************--
		OPEN cSRetAcc FOR
			SELECT fItemAcc(ItemID, 'SRET'), DeptID, PrjID, SUM(basesubtotal), SUM(subtotal) 
			FROM SRetD 
			WHERE SRetNo=_SRetNo 
			GROUP BY fItemAcc(ItemID, 'SRET'), DeptID, PrjID;
		LOOP
			FETCH cSRetAcc INTO _AccSRet, _DeptID, _PrjID, _DBSubtotal, _DSubtotal;
			EXIT WHEN NOT FOUND;

			IF (_i=0) THEN
				_AccAmt:=_AccAmt+_SelisihBasePrice;
			END IF;
			_i:=_i+1;
		
			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccSRet, 'D', _DBSubtotal, _DeptID, _PrjID, _DSubtotal);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSRetAcc;


		_Disc:=COALESCE(_Disc, 0);
		IF (_Disc<>0) THEN
			IF (_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('SALEDISC');
			END IF;

			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccDisc, 'D', ROUND(_Disc*_ExcRate*-1, _Round), NULL, NULL,_Disc*-1);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		SELECT INTO _TotalTax SUM(ROUND(d.taxamt*Qty*_ExcRate, _RoundTax)) 
		FROM SRetD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.SRetNo=_SRetNo and d.taxid IS NOT NULL;

		_SelisihTax:=(_MTaxAmt*_ExcRate)-_TotalTax;
		_i:=0;

		/*POSTING TAX DI GRUP IN SESUAI PAJAK TIAP2 ITEM*/
		OPEN cTax FOR
			SELECT SUM(d.basftotaltaxamt), SUM(d.totaltaxamt), accsaletax, deptid, prjid
			FROM SRetD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.SRetNo=_SRetNo and d.taxid IS NOT NULL
			GROUP BY accsaletax, deptid, prjid;
		LOOP
			FETCH cTax INTO _BTotalTaxAmt, _TotalTaxAmt, _AccSRetTax, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			IF (_i=0) then
				_SRetDTaxAmt:=_SRetDTaxAmt+_SelisihTax;
			end if;
			_i:=_i+1;

			_Count1:=_Count1+1;	
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccSRetTax, 'D', _BTotalTaxAmt, _DeptID, _PrjID, _TotalTaxAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;
		/*IF (_MTaxAmt<>0) THEN
			_Count1:=_Count1+1;	
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, fAccDefault('SALETAX'), 'D', _MTaxAmt*_ExcRate, NULL, NULL);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;*/

/** ==================== LABA RUGI SELISIH KURS ================================ */
		_RateLR:=(_BTotal+_MBDiscAmt)-(_BSubtotal+_MBFTaxAmt);
		IF (_RateLR!=0) THEN
			_Count1:=_Count1+1;
			_AccRateLR:=fAccDefault('PKKURS');
			IF (_AccRateLR IS NULL OR _AccRateLR='') THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Akun PK Selisih Kurs belum diset.'||E'\n'||'Silahkan disetting pada form Setting Akun';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccRateLR, 'D', _RateLR, _DeptID, _PrjID, NULL, _RateLR/_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		
		IF (_Freight<>0) THEN
			IF (_AccFreight IS NULL) THEN
				_AccFreight:=fAccDefault('SALEFREIGHT');
			END IF;
			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccFreight, 'D', _Freight*_ExcRate, NULL, NULL,_Freight);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		IF (_Others<>0) THEN
			_Count1:=_Count1+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, fAccDefault('PKROUND'), 'D', _Others*_ExcRate, NULL, NULL,_Others);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		/******************* JURNAL SRet: KREDIT ***************************/
		_Count1:=_Count1+1;		
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, _Count1, _AccAsset, 'K', _BTotal, _DeptID, _PrjID, _Total);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		/*--------------- proses jurnal untuk HPP: DEBET ----------------------------*/
		IF (_countDeli>0) THEN	
			
			_Count2:=_Count2+1;
			SELECT INTO _UBDeliAmt SUM(ROUND(Cost*Qty, _Round)) FROM SRetD WHERE SRetNo=_SRetNo AND DeliNo IS NOT NULL AND DeliNo<>'';

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, fAccDefault('UBSRet'), 'D', _UBDeliAmt, _DeptID, _PrjID, _UBDeliAmt);
			
		ELSE
			
			OPEN cSRetAcc FOR
				SELECT fItemAcc(d.ItemID, 'INV'), d.DeptID, d.PrjID, SUM(ROUND(d.Qty*d.Cost, _Round))
				FROM StockAD d
				JOIN StockA s ON s.StockANo=d.StockANo
				LEFT JOIN item i ON i.ItemID=d.ItemID
				WHERE s.RefNo=_SRetNo AND s.RefType='SRET'
				GROUP BY fItemAcc(d.ItemID, 'INV'), d.DeptID, d.PrjID;
			LOOP
				FETCH cSRetAcc INTO _AccInv, _DeptID, _PrjID, _AccAmt;
				EXIT WHEN NOT FOUND;
				
				_Count2:=_Count2+1;		
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, _AccInv, 'D', _AccAmt, _DeptID, _PrjID, _AccAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
			CLOSE cSRetAcc;		
		END IF;
		
		/*--------------- proses jurnal untuk HPP: KREDIT ----------------------------*/
		OPEN cSRetAcc FOR
			SELECT fItemAcc(d.ItemID, 'COST'), d.DeptID, d.PrjID, SUM(ROUND(d.Qty*d.Cost, _Round))
			FROM StockAD d
			JOIN StockA s ON s.StockANo=d.StockANo
			LEFT JOIN item i on i.ItemID=d.ItemID
			WHERE s.RefNo=_SRetNo AND s.RefType='SRET'
			GROUP BY fItemAcc(d.ItemID, 'COST'), d.DeptID, d.PrjID;			
		LOOP
			FETCH cSRetAcc INTO _AccCOGS, _DeptID, _PrjID, _AccAmt;
			EXIT WHEN NOT FOUND;
			
			_Count2:=_Count2+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo2, _Count2, _AccCOGS, 'K', _AccAmt, _DeptID, _PrjID,_AccAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cSRetAcc;

		/*---------- execute spJnlNew --------------*/
		SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_Count2>1) THEN
		SELECT INTO _spret * FROM spJnl_New(_JnlNo2);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		ELSE
			DELETE FROM jnl WHERE jnlno=_JnlNo2;
		END IF;

	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSRet_Void (
	_SRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_CustID		VARCHAR(15);
	_ConsNo		VARCHAR(15);
	_TermType		VARCHAR(1);
	_IsContra	BOOLEAN;
	_JnlNo		VARCHAR(15);
	_regval		VARCHAR(30);

	_itemid		VARCHAR(20);
	_qty		NUMERIC(15,4);
	_qtyx		NUMERIC(15,4);
	_unit		VARCHAR(5);
	_qtysret	NUMERIC(15,4);
	cSretD		refcursor;

	_DeliNo		VARCHAR(15);
	cSRetDeli	REFCURSOR;

	_SRetStatus	VARCHAR(1);
	_PID		VARCHAR;
	
	_isDraft	BOOLEAN;
	_SRetDate	DATE;
	_LockDate	DATE;
BEGIN
	SELECT into _CustID, _TermType, _IsContra, _ConsNo, _SRetStatus, _SRetDate
		    CustID, TermType, IsContra, ConsNo, SRetStatus, SRetDate
	FROM SRet
	WHERE SRetNo=_SRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Retur/Memo-Piutang #'||_SRetNo||' tidak ditemukan (spSRet_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SRET';
	IF(_LockDate is not null AND _LockDate>_SRetDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Retur Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF ((_TermType='R') AND (_IsContra=true)) THEN /*--(_SRetStatus<>'N')) THEN*/
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Retur/Memo-Piutang sudah dipotongkan pembayaran Hutang, VOID dulu pembayarannya ! (spSRet_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** CEK ISDRAFT */
	SELECT INTO _isDraft IsDraft FROM SRet WHERE SRetNo=_SRetNo;
	IF NOT FOUND THEN
		_isDraft:=FALSE;
	END IF;

	IF (_isDraft IS NULL) THEN
		_isDraft:=FALSE;
	END IF;

	IF (_isDraft=FALSE) THEN /** MULAI ISDRAFT */

	OPEN cSRetDeli FOR
	SELECT DeliNo FROM SRetDeli WHERE SRetNo=_SRetNo;
	LOOP
		FETCH cSRetDeli INTO _DeliNo;
		EXIT WHEN NOT FOUND;
		UPDATE Deli SET isinvoiced=false WHERE DeliNo=_DeliNo;
		DELETE FROM SRetDeli WHERE DeliNo=_DeliNo AND SRetNo=_SRetNo;
	END LOOP;
	CLOSE cSRetDeli;

	SELECT into _spret * FROM spStockA_Void_byRef('SRET', _SRetNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_TermType='C') THEN
		SELECT into _spret * FROM spCashA_Void_byRef('SRET', _SRetNo,null,null);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;			
		END IF;
	ELSE
		IF (_ConsNo IS NOT NULL) THEN	
			/** do nothing, khan saat new nggak bikin bpa */
		ELSE		
			SELECT into _spret * FROM spBPA_Void_byRef('SRET', _SretNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END IF;

	IF (_ConsNo IS NOT NULL) THEN
		OPEN cSretD FOR
			SELECT itemid, qty, qtyx, PID, unit from sretd where sretno=_SRetNo;
			LOOP
				FETCH cSretD INTO _itemid, _qty, _qtyx, _PID, _unit;
				EXIT WHEN NOT FOUND;
				_qtysret:=fQtyZToQty1(_itemid,_qty,_unit);
				IF (_PID IS NULL) THEN
				UPDATE consd set qtycret=qtycret-_qtysret, qtyxcret=qtyxcret-_qtyx WHERE consno=_consno AND itemid=_itemid;
				ELSE
				UPDATE consd set qtycret=qtycret-_qtysret, qtyxcret=qtyxcret-_qtyx WHERE consno=_consno AND itemid=_itemid AND PID=_PID;
				END IF;
			END LOOP;
		CLOSE cSretD;

		SELECT INTO _spret * FROM spSetConsStatus(_ConsNo);
		IF _spret.ErrNO<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;	

	END IF; /** END IF ISDRAFT */

	IF (isAccInstalled() AND _isDraft=FALSE) THEN
		SELECT into _spret * from spJnl_Void_byRef('SRET',_SRetNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM SRetD WHERE SRetNo=_SRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Delete SRetD (spSRet_Void)';
	END IF;
	
	DELETE FROM SRet WHERE SRetNo=_SRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error Delete SRet (spSRet_Void)';
	END IF;	

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockAD_New(
	__StockADate		DATE, 
	__StockAType		VARCHAR(5),
	__StockANo		VARCHAR(15), 
	__StockADNo		INT2, 
	__whid			VARCHAR(3), 
	__itemid		VARCHAR(20), 
	__pid			VARCHAR(25), 
	__qty			DECIMAL(15,4), 
	__unit			VARCHAR(5), 
	__qtyx			DECIMAL(15,4), 
	__cost			DECIMAL(19,4), 
	__DeptID		VARCHAR(5),
	__PrjID			VARCHAR(5),
	__insert		BOOLEAN,
	__RefNo			VARCHAR,
	__SubTotal		DECIMAL(15,4)
) RETURNS SETOF SPStatus AS
$BODY$ 
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
	_debug		VARCHAR(255);
	_Conv		DECIMAL(15,4);
	_PID		VARCHAR(25);

	_WhID		VARCHAR(3);
	_ItemID		VARCHAR(20);
	_Qty		DECIMAL(15,4);
	_Unit		VARCHAR(5);
	_QtyX		DECIMAL(15,4);
	_Cost		DECIMAL(19,4);
	_SubTotal	DECIMAL(19,4);

	_DeptID		VARCHAR(5);
	_PrjID 		VARCHAR(5);
	_StockAType	VARCHAR(5);
	_isStock	BOOLEAN;
BEGIN
	SELECT INTO _isStock isStock FROM Item WHERE ItemID=__ItemID;
	IF (_isStock=FALSE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Conv:=fItemUnit_Conv(__ItemID, __Unit);
	IF (_Conv IS NULL) THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error, Konversi ItemID: '||COALESCE(__ItemID, '<kosong>')||', Unit:'||COALESCE(__Unit, '<kosong>')||'!  (fItemUnit_Conv)(spStockAD_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (__Insert=true) THEN
		_PID:=__PID;
		_WhID:=__WhID;
		_ItemID:=__ItemID;
		_Qty:=__Qty;
		_Unit:=__Unit;
		_QtyX:=__QtyX;
		_DeptID:=__DeptID;
		_PrjID:=__PrjID;
		_StockAType:=__StockAType;
		_SubTotal:=__SubTotal;

		IF (_SubTotal IS NOT NULL) THEN

			IF (_Qty IS NOT NULL AND _Qty<>0) THEN
				_Cost:=__SubTotal/_Qty;
			ELSE
				_Cost:=0;
			END IF;

		ELSE
			_Cost:=__Cost;
			_SubTotal:=_Cost*_Qty;
		END IF;
			
		INSERT INTO StockAD (StockANo, StockADNo, WhID, ItemID, PID, Qty, Unit, Conv, QtyX, Cost, SubTotal, DeptID, PrjID)
		VALUES (__StockANo, __StockADNo, _WhID, _ItemID, _PID, _Qty, _Unit, _Conv, _QtyX, _Cost, _SubTotal, _DeptID, _PrjID);

		IF NOT FOUND THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Error INSERT StockAD, untuk Item: '||COALESCE(_ItemID, '<kosong>')||' (spStockAD_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	ELSE
		SELECT INTO 
				_WhID, _ItemID, _PID, _Qty, _Unit, _Conv, _QtyX, _Cost, _DeptID, _PrjID
			WhID, ItemID, PID, Qty, Unit, Conv, QtyX, Cost, DeptID, PrjID
		FROM StockAD WHERE StockANo=__StockANo AND StockADNo=__StockADNo;

		IF NOT FOUND THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='StockAD tidak ditemukan! StockANo:'||COALESCE(__StockANo, '<kosong>')||', StockADNo:'||COALESCE(CAST(__StockADNo AS VARCHAR), '<kosong>')||' (spStockAD_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		UPDATE StockAD SET Conv=_Conv WHERE StockANo=__StockANo AND StockADNo=__StockADNo;
	END IF;

	IF (_Qty IS NULL) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Error, Qty Item:'||COALESCE(_ItemID, '<kosong>')||' Gudang:'||COALESCE(_WhID, '<kosong>')||' PID:'||COALESCE(_PID, '<kosong>')||' tidak boleh KOSONG (spStockAD_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_Cost IS NULL) THEN
		_retval.ErrNo:=50005;
		_retval.ErrMsg:='Error, Cost Item:'||COALESCE(_ItemID, '<kosong>')||' Gudang:'||COALESCE(_WhID, '<kosong>')||' PID:'||COALESCE(_PID, '<kosong>')||' tidak boleh KOSONG (spStockAD_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (__SubTotal IS NOT NULL) THEN
		SELECT INTO _spret * FROM spStockBal_Update(_ItemID, _WhID, _pid, _Qty, _Unit, _Conv, _QtyX, _Cost, __StockADate, __StockAType,__RefNo, __SubTotal);
	ELSE
		SELECT INTO _spret * FROM spStockBal_Update(_ItemID, _WhID, _pid, _Qty, _Unit, _Conv, _QtyX, _Cost, __StockADate, __StockAType,__RefNo);
	END IF;

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD-1 ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spStockAD_New(
	__StockADate		DATE, 
	__StockAType		VARCHAR(5),
	__StockANo		VARCHAR(15), 
	__StockADNo		INT2, 
	__whid			VARCHAR(3), 
	__itemid		VARCHAR(20), 
	__pid			VARCHAR(25), 
	__qty			DECIMAL(15,4), 
	__unit			VARCHAR(5), 
	__qtyx			DECIMAL(15,4), 
	__cost			DECIMAL(19,4), 
	__DeptID		VARCHAR(5),
	__PrjID			VARCHAR(5),
	__insert		BOOLEAN,
	__RefNo			VARCHAR
) RETURNS SETOF spstatus AS 
$BODY$
DECLARE
	_spret			SPStatus%ROWTYPE;
	_retval			SPStatus%ROWTYPE;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _spret * FROM spStockAD_New(
		__StockADate, __StockAType, __StockANo, __StockADNo, __WhID, __ItemID,
		__PID, __Qty, __Unit, __QtyX, __Cost, __DeptID, __PrjID, __insert, 
		__RefNo, CAST(NULL AS DECIMAL(15,4))
	);
		
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;		
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD-2 ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spStockAD_New(
	__StockADate		DATE, 
	__StockAType		VARCHAR(5),
	__StockANo		VARCHAR(15), 
	__StockADNo		INT2, 
	__whid			VARCHAR(3), 
	__itemid		VARCHAR(20), 
	__pid			VARCHAR(25), 
	__qty			DECIMAL(15,4), 
	__unit			VARCHAR(5), 
	__qtyx			DECIMAL(15,4), 
	__cost			DECIMAL(19,4), 
	__DeptID		VARCHAR(5),
	__PrjID			VARCHAR(5),
	__insert		BOOLEAN
) RETURNS SETOF spstatus AS 
$BODY$
DECLARE
	_spret	SPStatus%ROWTYPE;
	_retval	SPStatus%ROWTYPE;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _spret * FROM spStockAD_New(
		__StockADate, __StockAType, __StockANo, __StockADNo, __WhID, __ItemID,
		__PID, __Qty, __Unit, __QtyX, __Cost, __DeptID, __PrjID, __insert, 
		CAST(NULL AS VARCHAR), CAST(NULL AS DECIMAL(15,4))
	);

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;		
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockAD_New_BoM(
	_StockADate 		DATE, 
	_StockAType		VARCHAR(5),
	_StockANo 		VARCHAR(15), 
	_StockADNo 		SMALLINT,
	_WhID 			VARCHAR(3), 
	_ItemID 		VARCHAR(20), 
	_PID 			VARCHAR(25), 
	_Qty 			DECIMAL(15,4),
	_Unit 			VARCHAR, 
	_QtyX 			DECIMAL(15,4), 
	_Cost 			DECIMAL(19,4),
	_DeptID			VARCHAR(5),
	_PrjID			VARCHAR(5),
	_CrcID			VARCHAR,
	_Subtotal		NUMERIC(19,4)
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval 		SPStatus%ROWTYPE;
	_spretStockAD_New_BoM 	SPStatus%ROWTYPE;
	_spretStockAD_New 	SPStatus%ROWTYPE;

	_ItemType		VARCHAR(5);
	_IsAsm			BOOLEAN;
	_BoMID			VARCHAR(20);
	_IsStock		BOOLEAN;

	_BoMD_ItemID		VARCHAR(20);
	_BoMD_WhID		VARCHAR(3);
	_BoMD_Qty		DECIMAL(15,4);
	_BoMD_Unit		VARCHAR(5);
	_BoMD_PID		VARCHAR(25);

	_BoM_Mul		DECIMAL(15,4);
	_tmpQty			DECIMAL(15,4);
	_tmpQtyX		DECIMAL(15,4);
	_tmpCost		DECIMAL(19,4);
	cBoMD			REFCURSOR;
	__StockADNo 		SMALLINT;
	__Subtotal		NUMERIC(19,4);
BEGIN
	__StockADNo:=_StockADNo;
	_retval:=(0, 'OK');

	SELECT INTO 
		_ItemType, _IsAsm, _IsStock 
		itemtype, isasm, isstock 
	FROM item WHERE itemid=_ItemID;

	IF NOT FOUND THEN 
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error ItemID:'||COALESCE(_ItemID, '<kosong>')||' tidak ditemukan ! (spStockAD_New_BoM)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF ((_ItemType='PACK') OR (_IsAsm=TRUE)) AND (_IsStock=FALSE) THEN
		SELECT INTO _BoMID bomid FROM bom WHERE itemid=_ItemID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50001;
			_retval.ErrMsg:='Error B.o.M. untuk ItemID:'||COALESCE(_ItemID, '<kosong>')||' tidak ditemukan ! (spStockAD_New_BoM)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _BoM_Mul * FROM fQtyZToQty1(_ItemID, 1, _Unit);
		IF (_BoM_Mul IS NULL) THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Error konversi Item:'||COALESCE(_ItemID, '<kosong>')||' Unit:'||COALESCE(_Unit, '<kosong>')||' ke Unit1 (spStockAD_New_BoM)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		OPEN cBoMD FOR
			SELECT d.itemid, d.whid, d.qty, d.unit, d.pid, i.itemtype, i.isstock 
			FROM bomd d JOIN item i ON d.itemid=i.itemid 
			WHERE bomid=_BoMID
			ORDER BY d.itemid;
		LOOP
			FETCH cBoMD INTO _BoMD_ItemID, _BoMD_WhID, _BoMD_Qty, _BoMD_Unit, _BoMD_PID, _ItemType, _IsStock;
			EXIT WHEN NOT FOUND;

			_tmpQty:=_Qty*_BoM_Mul*_BomD_Qty;
			_tmpQtyX:=0;
		
			IF (_ItemType='SERV' OR _IsStock=FALSE) THEN
				_tmpCost:=fPriceList(_BoMD_ItemID, _BoMD_Unit, '0', _CrcID);
				__Subtotal:=_tmpCost*_tmpQty;
			ELSE
				_tmpCost:=fUnitCost(_BoMD_ItemID, _WhID, _BoMD_PID, _BoMD_Unit);
				__Subtotal:=fUnitCost(_BoMD_ItemID, _WhID, _BoMD_PID, _tmpQty, _BoMD_Unit);
			END IF;

			SELECT INTO _spretStockAD_New_BoM * FROM spStockAD_New_BoM (
				_StockADate, _StockAType, _StockANo, __StockADNo, _WhID, _BoMD_ItemID,
				_BoMD_PID, _tmpQty, _BoMD_Unit, _tmpQtyX, _tmpCost, _DeptID, _PrjID, _CrcID, __Subtotal
			); 

			__StockADNo:=_spretStockAD_New_BoM.ErrNo;

			IF _spretStockAD_New_BoM.ErrNo<>0 AND _spretStockAD_New_BoM.ErrMsg<>'OK' THEN
				_retval.ErrNo:=_spretStockAD_New_BoM.ErrNo;
				_retval.ErrMsg:=_spretStockAD_New_BoM.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_retval.ErrNo:=__StockADNo;
		END LOOP;
		CLOSE cBoMD;
	ELSIF (_IsStock=true) THEN
		_tmpQty=_Qty;
		_tmpQtyX=_QtyX;
		__StockADNo:=__StockADNo+1;

		SELECT INTO _spretStockAD_New * FROM spStockAD_New(
			_StockADate, _StockAType, _StockANo, __StockADNo, 
			_WhID, _ItemID, _PID, _tmpQty, _Unit, _tmpQtyX, 
			_Cost, _DeptID, _PrjID, true, NULL, _Subtotal
		);

		IF _spretStockAD_New.ErrNo<>0 THEN 
			_retval.ErrNo:=_spretStockAD_New.ErrNo;
			_retval.ErrMsg:=_spretStockAD_New.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval.ErrNo:=__StockADNo;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION spStockAD_New_BoM(
	_StockADate 		DATE, 
	_StockAType		VARCHAR(5),
	_StockANo 		VARCHAR(15), 
	_StockADNo 		SMALLINT,
	_WhID 			VARCHAR(3), 
	_ItemID 		VARCHAR(20), 
	_PID 			VARCHAR(25), 
	_Qty 			DECIMAL(15,4),
	_Unit 			VARCHAR, 
	_QtyX 			DECIMAL(15,4), 
	_Cost 			DECIMAL(19,4),
	_DeptID			VARCHAR(5),
	_PrjID			VARCHAR(5),
	_CrcID			VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 		SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _spret * FROM spStockAD_New_BoM(_StockADate, _StockAType, _StockANo, _StockADNo, _WhID, _ItemID, _PID, _Qty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, _CrcID, NULL);
	IF (_spret.ErrNo<>0 AND _spret.ErrMsg<>'OK') THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=_spret.ErrNo;
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION spStockAD_New_BoM(
	_StockADate 		DATE, 
	_StockAType		VARCHAR(5),
	_StockANo 		VARCHAR(15), 
	_StockADNo 		SMALLINT,
	_WhID 			VARCHAR(3), 
	_ItemID 		VARCHAR(20), 
	_PID 			VARCHAR(25), 
	_Qty 			DECIMAL(15,4),
	_Unit 			VARCHAR, 
	_QtyX 			DECIMAL(15,4), 
	_Cost 			DECIMAL(19,4),
	_DeptID			VARCHAR(5),
	_PrjID			VARCHAR(5)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 		SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CrcID			VARCHAR;
BEGIN
	_retval:=(0, 'OK');

	_CrcID:=fCrcDefault();

	SELECT INTO _spret * FROM spStockAD_New_BoM(_StockADate, _StockAType, _StockANo, _StockADNo, _WhID, _ItemID, _PID, _Qty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, _CrcID, NULL);
	IF (_spret.ErrNo<>0 AND _spret.ErrMsg<>'OK') THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=_spret.ErrNo;
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockAD_Void(_StockANo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval				SPStatus%ROWTYPE;
	_spret				SPStatus%ROWTYPE;
	
	_StockADate			DATE;
	_StockAType			VARCHAR(5);
	_StockADNo			SMALLINT;
	_WhID				VARCHAR(3);
	_ItemID				VARCHAR(20);
	_PID				VARCHAR(25);
	_Qty				DECIMAL(15,4);
	_Unit				VARCHAR(5);
	_Conv				DECIMAL(15,4);
	_QtyX				DECIMAL(15,4);
	_Cost				DECIMAL(19,4);
	_SubTotal			DECIMAL(19,4);
	_SubTotal_Void		DECIMAL(19,4);

	_tmpQty				DECIMAL(15,4);
	_tmpQtyX			DECIMAL(15,4);

	_RefNo				VARCHAR;
	
	cStockAD			REFCURSOR;
BEGIN

	SELECT INTO _StockADate, _StockAType, _RefNo 
				stockadate, stockatype, refno  
	FROM stocka WHERE stockano=_StockANo;

	OPEN cStockAD FOR
		SELECT StockADNo, WhID, ItemID, PID, Qty, Unit, Conv, QtyX, Cost, SubTotal
		FROM StockAD WHERE StockANo=_StockANo;
	LOOP
		FETCH cStockAD INTO _StockADNo, _WhID, _ItemID, _PID, _Qty, _Unit, _Conv, _QtyX, _Cost, _SubTotal;
		EXIT WHEN NOT FOUND;

		IF (_PID IS NULL) THEN
			_PID:='';
		END IF;

		IF NOT EXISTS( SELECT 1	FROM Stock WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID) THEN
			_retval.ErrNo:=50005;
			_retval.ErrMsg:='Kartu Stock tidak ditemukan!(spStockAD_Void)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		_tmpQty:=_Qty*(-1);
		_tmpQtyX:=_QtyX*(-1);
		_SubTotal_Void:=_SubTotal*(-1);


		IF (_SubTotal IS NOT NULL) THEN

			SELECT INTO _spret * FROM spStockBal_Update(_ItemID, _WhID, _PID, _tmpQty, _Unit, _Conv, _tmpQtyX, _Cost, _StockADate, _StockAType, COALESCE(_StockANo, _RefNo), _SubTotal_Void);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		ELSE
			SELECT INTO _spret * FROM spStockBal_Update(_ItemID, _WhID, _PID, _tmpQty, _Unit, _Conv, _tmpQtyX, _Cost, _StockADate, _StockAType, _RefNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		END IF;

	
	END LOOP;
	CLOSE cStockAD;

	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error DELETE StockAD! (spStockAD_Void)';
	DELETE FROM StockAD WHERE StockANo=_StockANo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockCard_ChNCr (
	_ItemID		VARCHAR,
	_WhID		VARCHAR,
	_PID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_foundItem	VARCHAR;
	_foundWh	VARCHAR;
	_foundPid	VARCHAR;

	_found		BOOLEAN;
	_isStock	BOOLEAN;
BEGIN

	SELECT INTO 
		_isStock
		isStock
	FROM Item WHERE ItemID=_ItemID;

	IF (_isStock IS NULL OR _isStock=FALSE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;


	IF _PID IS NULL THEN
		SELECT INTO _foundItem,_foundWh itemid,whid FROM stock WHERE itemid=_ItemID and whid=_WhID;
		_found:=FOUND;
	ELSE
		SELECT INTO _foundItem,_foundWh,_foundPid itemid,whid,pid FROM stock WHERE itemid=_ItemID AND whid=_WhID AND pid=_PID;
		_found:=FOUND;
	END IF;

	_retval.ErrNo:='1052';
	_retval.ErrMsg:='Error (INSERT Stock)!';
	
	IF (NOT _found AND _PID IS NULL)THEN	
		INSERT INTO Stock (ItemID, WhID, PID) VALUES (_foundItem,_foundWh, '');
	ELSIF (NOT _found AND _PID IS NOT NULL)THEN
		INSERT INTO Stock (ItemID, WhID, PID) VALUES (_ItemID, _WhID, _PID);
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockA_User_New (
	_StockANo	VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spretstockad_new 	SPStatus%ROWTYPE;
	cStockad 		REFCURSOR;
	
	_StockAType		VARCHAR(5);
	_itemidad		VARCHAR(20);
	_qtyad			DECIMAL(15,4);
	_whidad			VARCHAR(3);
	_pidad			VARCHAR(25);
	_stockadnoad		SMALLINT;
	_unitad			VARCHAR(5);
	_qtyxad			DECIMAL(15,4);
	_stockadate 		DATE;
	_qtys			DECIMAL(15,4);
	_conv			DECIMAL(15,4);
	_cost 			DECIMAL(19,4);

	_Count			INT;
	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;

   	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
   	_DeptID     		VARCHAR(5);
   	_PrjID      		VARCHAR(5);
	_crtby 			VARCHAR(10);
	_crtdate 		TIMESTAMP;
	_updby 			VARCHAR(10);
	_upddate 		TIMESTAMP;
	_Round 			INT;
	_AccInv			VARCHAR;
	_DK			VARCHAR;
	_total			NUMERIC;
	_Acc			VARCHAR;
	_JnlType		VARCHAR;
	_LockDate		DATE;
	_RefNo			VARCHAR;
	_ItemBegBal		VARCHAR;

	_StockANote		VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate		numeric(19,4);
	_FisRate		numeric(19,4);

BEGIN
	
	SELECT INTO 
		_StockADate, _StockAType, _DeptID, _PrjID, _CrtBy, _CrtDate,_UpdBy,_UpdDate, _RefNo, _PerID, _BranchID, _StockANote
		StockADate, StockAType, DeptID, PrjID, CrtBy, CrtDate, UpdBy, UpdDate, RefNo, PerID, BranchID, StockANote
	FROM stocka WHERE stockano=_stockano;
	
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Transaksi-Stock #'||_stockano|| 'tidak ditemukan (spStockA_User_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _CrcID, _ExcRate, _FisRate crcid, excrate, fisrate FROM crc WHERE crcid=fcrcdefault();

	_StockANote:=REPLACE(_StockANote, '%', _StockANo);
	UPDATE stocka SET stockanote=_StockANote WHERE stockano=_StockANo;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SADJ';
	IF(_LockDate is not null AND _LockDate>_StockADate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penyesuaian Stock Melebihi Batas LockDate!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/*periksa klo2 item begbal dipake penyesuaian stock!!*/
	SELECT INTO _ItemBegBal ItemID FROM stockad JOIN stocka on stockad.stockano=stocka.stockano 
	WHERE stockad.stockano=_stockaNo AND itemid='0' AND stockatype<>'BEG';
	IF(_ItemBegBal IS NOT NULL) THEN
		_retval.ErrNo:=50009;
		_retval.ErrMsg:='Item Beginning Balance Tidak Bisa Digunakan Penyesuaian Stock !!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	END IF;

	IF(_ItemBegBal IS NOT NULL) THEN
		_retval.ErrNo:=50009;
		_retval.ErrMsg:='Item Beginning Balance Tidak Bisa Digunakan Pembelian !!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	END IF;

	/*SET REFNO DI STOCKA*/
	IF(_RefNo is null) THEN
		UPDATE stocka SET refno=_StockANo WHERE stockano=_StockANo;
	END IF;

	OPEN cStockad FOR
	SELECT stockadno,whid,itemid,pid,qty,unit,qtyx,cost from stockad where stockano=_stockano;
	LOOP
		FETCH cStockad INTO _stockadnoad,_whidad,_itemidad,_pidad,_qtyad,_unitad,_qtyxad,_cost;

		IF NOT FOUND THEN
			EXIT;
		END IF;
		
		SELECT INTO _conv * from fitemunit_conv(_itemidad, _unitad);
		
		IF _conv IS NULL THEN
			_retval.ErrNo:=50055;
			_retval.ErrMsg:='Error, Konversi ItemID:'||_itemidad||', Unit:'||_unitad||'!  (fItemUnit_Conv)(spStockA_User_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF _cost IS NULL THEN
			SELECT INTO _cost * from funitcost(_itemidad, _whidad, _pidad, _unitad);
		END IF;

		IF _cost IS NULL THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg:='Error menghitung HPP! Item:'||_itemidad||', Wh:'||_whidad||', Batch:'||_pidad||', Unit:'||_unitad||' (fUnitCost)(spStockA_User_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF _pidad IS NULL THEN
			_pidad:='';
		END IF;
		
		UPDATE stockad SET pid=_pidad, conv=_conv, cost=_cost WHERE stockano=_stockano AND stockadno=_stockadnoad;
		
		IF NOT FOUND THEN		
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Error UPDATE StockADPID,Cost (spStockA_User_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	
		SELECT INTO _spretstockad_new * FROM spStockAD_New(_stockadate, _StockAType, _stockano,_stockadnoad,_whidad,_itemidad,_pidad,_qtyad,_unitad,_qtyxad,_cost, _DeptID, _PrjID, false);
			
		IF _spretstockad_new.ErrNo<>0 THEN
			_retval.ErrNo:=50005;
			_retval.ErrMsg:=_spretstockad_new.ErrNo || 'Error in spStockAD_New (spStockA_User_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	
	END LOOP;


	CLOSE cStockad;
	
	UPDATE stocka SET isautogen=false WHERE stockano=_stockano;
	
	IF NOT FOUND THEN
		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error UPDATE StockA:IsAutoGen (spStockA_User_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	/** /////////////// AKUNTANSI ////////////////////// */
	IF isAccInstalled() THEN
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:= 'Penyesuaian Stock '||':'||_stockano;

		IF(_StockAType='BEG') THEN
			_JnlType:='BEG';
			_JnlNote:='Beggining Balance Stock '||':'||_stockano;
		ELSE
			_JnlType:='SADJ';
		END IF;

		SELECT INTO _spretstockad_new * FROM spJnl_Insert(
			_JnlNo, _JnlType, _stockadate, 'SADJ', _stockano,
			_JnlNote,true,true, cast(0 as numeric(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spretstockad_new.ErrNo<>0 THEN
			_retval.ErrNo:=_spretstockad_new.ErrNo;
			_retval.ErrMsg:=_spretstockad_new.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		SELECT INTO _Round * FROM fReg('ROUND');

		IF(_Round is null) THEN
			_Round:=0;
		END IF;
		_Count:=0;
		_total:=0;
		OPEN cStockad FOR
		SELECT fItemAcc(itemid,'INV'),Round(sum(qty*cost),_Round),deptid,prjid 
		from stockad where stockano=_stockano
		GROUP BY fItemAcc(itemid,'INV'),deptid,prjid;
		LOOP
			FETCH cStockad INTO _AccInv,_cost, _DeptID,_PrjID;
			EXIT WHEN NOT FOUND;
			
			
			_Count:=_Count+1;
		IF(_cost<0)THEN
			SELECT INTO _spretstockad_new * 
			FROM spJnlD_Insert(_JnlNo,_Count,_AccInv,'K', _cost*(-1),_DeptID,_PrjID,_cost*(-1));
				IF _spretstockad_new.ErrNo<>0 THEN
					_retval.ErrNo:=_spretstockad_new.ErrNo;
					_retval.ErrMsg:=_spretstockad_new.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;	
			_total:=_total+(_cost*(-1));
			_DK:='D';
		ELSE
			SELECT INTO _spretstockad_new * 
			FROM spJnlD_Insert(_JnlNo,_Count,_AccInv,'D', _cost,_DeptID,_PrjID, _cost);
				IF _spretstockad_new.ErrNo<>0 THEN
					_retval.ErrNo:=_spretstockad_new.ErrNo;
					_retval.ErrMsg:=_spretstockad_new.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;	
			_total:=_total+_cost;
			_DK:='K';
		END IF;	
		END LOOP;
		CLOSE cStockad;

		_Count:=_Count+1;
		_Acc:=fAccDefault('PKSTOCK');
		IF(_StockAtype='BEG') THEN
			_Acc:=fAccDefault('BEGBAL');
		END IF;
		SELECT INTO _spretstockad_new * 
		FROM spJnlD_Insert(_JnlNo,_Count,_Acc,_DK,_total,_DeptID,_PrjID, _total);
		IF _spretstockad_new.ErrNo<>0 THEN
			_retval.ErrNo:=_spretstockad_new.ErrNo;
			_retval.ErrMsg:=_spretstockad_new.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;	


		SELECT INTO _spretstockad_new * FROM spJnl_New(_JnlNo);
		IF _spretstockad_new.ErrNo<>0 THEN
			_retval.ErrNo:=_spretstockad_new.ErrNo;
			_retval.ErrMsg:=_spretstockad_new.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockA_Void (
	_stockano	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	cAcc			refcursor;
	_JnlNo			VARCHAR(15);
	_spretstockad_void 	SPStatus%ROWTYPE;
	_found 			VARCHAR;
	_StockADate		DATE;
	_LockDate		DATE;
BEGIN
	SELECT INTO _found,_StockADate stockano,stockadate FROM StockA WHERE StockANo=_StockANo;
	IF NOT FOUND THEN
		_retval.ErrNo=50001;
		_retval.ErrMsg='Transaksi-Stock #'||_stockano||' tidak ditemukan (spStockA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SADJ';
	IF(_LockDate is not null AND _LockDate>_StockADate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penyesuaian Stock Melebihi Batas LockDate!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spretstockad_void * FROM spstockad_void(_stockano);
	IF _spretstockad_void.ErrNo<>0 THEN
		_retval.ErrNo=_spretstockad_void.ErrNo;
		_retval.ErrMsg=_spretstockad_void.ErrMsg||'Error in spStockAD_Void (spStockA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	IF isAccInstalled() THEN
		SELECT into _spretstockad_void * from spJnl_Void_byRef('SADJ',_stockano);
		IF _spretstockad_void.ErrNo<>0 THEN
			_retval.ErrNo:=_spretstockad_void.ErrNo;
			_retval.ErrMsg:=_spretstockad_void.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
	/*--		EXIT;*/
		END IF;
	END IF;
	
	DELETE FROM stocka WHERE stockano=_stockano;	
	_retval.ErrNo:=0;
	_retval.ErrMsg:=':p wong hehehe';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStock_Check (
	_ItemID		VARCHAR,
	_WhID		VARCHAR,
	_PID		VARCHAR,
	_LOpDate	DATE,
	_LOpEmpID	VARCHAR,
	_LOpUsrID	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error UPDATE Stock! for Item:'||_ItemID||' (spStock_Check)';
	UPDATE Stock SET LOpEmpID=_LOpEmpID, LOpUsrID=_LOpUsrID, LOpDate=_LOpDate
	WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;
	_retval.ErrNo:=0;
	_retval.ErrMsg:='UPDATE SUCCESS!!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStockMinMax_Update(
	_ItemID		VARCHAR,
	_WhID		VARCHAR,
	_PID		VARCHAR,
	_LeadTime	SMALLINT,
	_QtyMin		NUMERIC,
	_QtyMax		NUMERIC,
	_QtyReOrder	NUMERIC
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	IF (_LeadTime IS NOT NULL) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error UPDATE Item:LeadTime (spStockMinMax_Update)';
		UPDATE Item SET LeadTime=_LeadTime WHERE ItemID=_ItemID;
		_retval.ErrNo:=0;
		_retval.ErrMsg:='UPDATE SUCCESS!';
	END IF;
	
	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error UPDATE Stock:Min,Max,ReOrder (spStockMinMax_Update)';
	UPDATE Stock SET QtyMin=_QtyMin, QtyMax=_QtyMax, QtyReOrder=_QtyReOrder
	WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;
	_retval.ErrNo:=0;
	_retval.ErrMsg:='UPDATE SUCCESS!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStock_RemoveForced (
	_ItemID		VARCHAR,
	_WhID		VARCHAR,
	_PID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error DELETE Stock (spStock_RemoveForced)';
	DELETE FROM Stock WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;
	_retval.ErrNo:=0;
	_retval.ErrMsg:='DELETE SUCCESS!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




/******************************************************/
/************** PO Check and Update Status ************/
/******************************************************/
CREATE OR REPLACE FUNCTION spPO_ChNUp_Status (
	_PONo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_POExpDate	DATE;

	_PODNo		SMALLINT;
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_Qty1_O		NUMERIC(15,4);		/** Ordered Qty **/
	_Qty1_D		NUMERIC(15,4);		/** Delivered Qty **/
	_Qty1_B		NUMERIC(15,4);		/** Back-Order Qty **/

	_Qty1_O_SUM	NUMERIC(15,4);		/** SUM Ordered Qty **/
	_Qty1_D_SUM	NUMERIC(15,4);		/** SUM Delivered Qty **/
	_Qty1_B_SUM	NUMERIC(15,4);		/** SUM Back-Order Qty **/

	_Qty1_R1	NUMERIC(15,4);		/** R1 = Order - Delivered ==> untuk menentukan POstatus **/
	_Qty1_R2	NUMERIC(15,4);		/** R2 = Order - Delivered - B.O. ==> untuk menentukan "Active" **/

	_Expired	BOOLEAN;
	_IsFull		BOOLEAN;
	_IsPartial	BOOLEAN;
	_IsClear	BOOLEAN;
	_Active_orig	BOOLEAN;
	_Active		BOOLEAN;
	_POStatus	VARCHAR(1);
	cPOD 		REFCURSOR;
BEGIN	
	IF (_PONo IS NULL OR _PONo='') THEN	-- utk melayani SP lain yg memang gak nge-check valuenya, kita ngalahin ngecheck disini
		_retval:=(0,'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;


	SELECT INTO _Active_orig active FROM po WHERE pono=_PONo;
	
	IF NOT FOUND THEN
		_retval.ErrNo:=12020002;
		_retval.ErrMsg:='PO#'||_PONo||' tidak ditemukan ! (spPO_ChNUp_Status)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_IsFull=true;			/** default POstatus=FULL ***/
	_IsPartial=false;		/** default POstatus=FULL ***/
	_IsClear=true;			/** default CLOSED **/
	_Active=true;			/** default OPEN **/
	_Qty1_O_SUM:=0;
	_Qty1_D_SUM:=0;
	_Qty1_B_SUM:=0;
	
	OPEN cPOD FOR
		SELECT podno, fQtyZToQty1(itemid, qty, unit), qtydelivered, qtybo
		FROM pod WHERE pono=_PONo;
	
	LOOP
		FETCH cPOD INTO _PODNo, _Qty1_O, _Qty1_D, _Qty1_B;
		EXIT WHEN NOT FOUND;

		IF(_Qty1_O<0) THEN
			_Qty1_O:=_Qty1_O*(-1);
		END IF;
			_Qty1_O_SUM:=_Qty1_O_SUM+_Qty1_O;

		IF(_Qty1_D<0) THEN
			_Qty1_D:=_Qty1_D*(-1);
		END IF;
			_Qty1_D_SUM:=_Qty1_D_SUM+_Qty1_D;

		IF(_Qty1_B<0) THEN
			_Qty1_B:=_Qty1_B*(-1);
		END IF;
			_Qty1_B_SUM:=_Qty1_B_SUM+_Qty1_B;
				
	END LOOP;
	CLOSE cPOD;

	_Qty1_R1 = _Qty1_O_SUM - _Qty1_D_SUM;
	_Qty1_R2 = _Qty1_O_SUM - _Qty1_D_SUM - _Qty1_B_SUM;

	IF (_Qty1_R1>0) THEN 			/** not full **/
		_IsFull=false;			
	END IF;
	
	IF (_Qty1_D>0) THEN 			/** sudah ada sebagian yg terkirim **/
		_IsPartial=true;
	END IF;
	
	IF (_Qty1_R2>0) THEN 			/** ada sisa & belum di BO**/
		_IsClear=false;			/** jika ada sisa item yg BELUM CLEAR (belum dikirim & belum di-B.O.) **/
	END IF;

	IF(_Qty1_R1=0) THEN
		_POStatus='F';
	ELSIF(_Qty1_R1<>0 AND _Qty1_O_SUM<>_Qty1_R1) THEN
		_POStatus='P';
	ELSE
		_POStatus='N';
	END IF;

/*	_retval.ErrNo=50001;
	_retval.ErrMsg='Error UPDATE PO:POStatus,Active (spPO_ChNUp_Status)';	*/

	/** set flags (unused) **/
/*	IF (_IsFull=true) THEN _POStatus='F';									
	ELSIF (_IsFull=false) AND (_IsPartial=true) THEN _POStatus='P';
	ELSIF (_IsFull=false) AND (_IsPartial=false) THEN _POStatus='N';
	END IF;*/

	/*** Jika sudah Clear semua, tutup PO ***/
/*	IF (_IsClear=true) THEN 
		_Active=false;
	END IF;*/

	IF(_Qty1_R2>0 AND _POStatus='F') THEN
		_Active=FALSE;
	ELSIF(_Qty1_R2>0) THEN
		_Active=TRUE;
	ELSE
		_Active=FALSE;
	END IF;
--RAISE EXCEPTION '%',_Active;
	UPDATE po SET postatus=_POStatus, active=_Active WHERE pono=_PONo;

	_retval=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




DROP FUNCTION spPO_Item_Cancel(VARCHAR,VARCHAR,VARCHAR,NUMERIC,INT);
CREATE OR REPLACE FUNCTION spPO_Item_Cancel(
	__PONo 		VARCHAR, 
	__ItemID 	VARCHAR, 
	__PID		VARCHAR,
	__Qty1 		NUMERIC(15,4), 
	__PODNo 	INT
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE
	_retval 	NUMERIC(15,4);
	_Active		BOOLEAN;
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_WhID		VARCHAR(3);
	_PODPID		VARCHAR(25);
	_qty1_Ordered	NUMERIC(15,4);		/** Ordered Qty **/
	_qty1_Delivered	NUMERIC(15,4);		/** Delivered Qty **/
	_qty1_BackOrder	NUMERIC(15,4);		/** Back-Order Qty **/
	_qty1_Remains	NUMERIC(15,4);		/** Remaining Open Qty **/
	_qty1_cancel	NUMERIC(15,4);	
BEGIN
	_qty1_cancel:=0;

	SELECT INTO _Active active FROM po WHERE pono=__PONo;
	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Order Pembelian #%  tidak ditemukan! (spPO_Item_Cancel)', __PONo;
	END IF;
		
	
	SELECT	INTO  
		_WhID, _PODPID, _qty1_Delivered, _qty1_BackOrder 
		whid, pid, qtydelivered, qtybo 
	FROM pod WHERE pono=__PONo AND podno=__PODNo AND itemid=__ItemID;	-- sengaja di cek PONo, PODNo, ItemID biar pasti
	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Order-Pembelian #%, detail ke-% tidak ditemukan! (spPO_Item_Cancel)', __PONo, __PODNo;
	END IF;


	IF (__Qty1>_qty1_Delivered) THEN
		_qty1_cancel=_qty1_Delivered;
	ELSE
		_qty1_cancel=__Qty1;
	END IF;

	UPDATE pod SET qtydelivered=qtydelivered-_qty1_cancel 
	WHERE pono=__PONo AND podno=__PODNo;
	
	_PODPID:=COALESCE(_PODPID, '');

	UPDATE stock SET qtypo=qtypo+_qty1_cancel 
	WHERE itemid=__ItemID AND whid=_WhID AND pid=_PODPID;	

	RETURN _qty1_cancel;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



-- drop varian yg obsolete !
DROP FUNCTION spPO_Item_Receive(VARCHAR,VARCHAR,NUMERIC,INT);
DROP FUNCTION spPO_Item_Receive(VARCHAR,VARCHAR,VARCHAR,NUMERIC,INT);

CREATE OR REPLACE FUNCTION spPO_Item_Receive(
	__PONo 		VARCHAR, 
	__ItemID	VARCHAR, 
	__PID		VARCHAR,
	__Qty1		NUMERIC, 
	__PODNo 	INT
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE
	_spretStockCard_ChNCr 	SPStatus%ROWTYPE;
	_retval			SPStatus%ROWTYPE;

	_Active			BOOLEAN;
	_isDraft		BOOLEAN;
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_WhID			VARCHAR(3);
	_PODPID			VARCHAR(25);
	_qty1_order		NUMERIC(15,4);		-- qty order
	_qty1_delivered		NUMERIC(15,4);		-- yg sdh terkirim
	_qty1_backorder		NUMERIC(15,4);		-- qty ditolak (nyerah)
	_qty1_outstanding	NUMERIC(15,4);	-- sisa yg masih harus dikirim
	_qty1_receive_now	NUMERIC(15,4);
	_qty1_tolerance		NUMERIC(15,4);

	_isstock		BOOLEAN;

	_PODNo			INT;
/*	_POVarUp	NUMERIC(15,4);
	_POVarDown	NUMERIC(15,4);*/

BEGIN
	_qty1_receive_now:=0;

	SELECT INTO _Active, _isDraft active, isdraft
	FROM po WHERE pono=__PONo;

	IF NOT FOUND THEN
		RAISE EXCEPTION '%', 'Order-Pembelian #'||__PONo||' tidak ditemukan (spPO_Item_Receive)';
	END IF;

	IF (_Active=FALSE) THEN
		RAISE EXCEPTION '%', 'Status Order-Pembelian #'||__PONo||' : Tidak Aktiv (spPO_Item_Receive)';
	END IF;
	
	IF (_isDraft=TRUE) THEN
		RAISE EXCEPTION '%', 'Order-Pembelian masih Draft tidak bisa digunakan! (spPO_Item_Receive)', __SONo;
	END IF;

	

	SELECT INTO 
		_Qty, _Unit, _WhID, _PODPID, _qty1_delivered, _qty1_backorder, _PODNo 
		qty, unit, whid, pid, qtydelivered, qtybo, podno
	FROM pod
	WHERE pono=__PONo AND itemid=__ItemID AND podno=__PODNo;
	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Order-Pembelian #%, detail ke-% tidak ditemukan! (spPO_Item_Cancel)', __PONo, __PODNo;
	END IF;


	/** toleransi yg dicek cuma POVarUp, asumsi apabila dikirim kurang dari order dan memang mau diterima, ya bisa diclose manual dari program 
	SELECT INTO 
		_POVarUp, _POVarDown
		COALESCE(POVarUp,0), COALESCE(POVarDOwn,0)
	FROM pod
	WHERE PONo=__PONo AND PODNo=__PODNo;

**/


	_qty1_order:=fQtyZToQty1(__ItemID, _Qty, _Unit);
	_qty1_outstanding:=_qty1_order-_qty1_delivered;
	_qty1_tolerance:=0;
--	_qty1_tolerance:=_qty1_order*(_POVarUp/100);


	IF (__Qty1>(_qty1_outstanding+_qty1_tolerance)) THEN
		RAISE EXCEPTION 'Item:%, PID:%. Qty yang diterima melebihi Qty Outstanding+toleransi! (spPRcv_Item_Receive)', __ItemID, __PID;
		_qty1_receive_now:=0;
		RETURN 0;	
	ELSE
		_qty1_receive_now:=__Qty1;
	END IF;

	
	UPDATE pod SET qtydelivered=qtydelivered+_qty1_receive_now
	WHERE pono=__PONo AND podno=_PODNo;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Error Update PO:QtyDelivered # % (spPO_Item_Receive)', __PONo;
	END IF;
	
	SELECT INTO _isStock isstock 
	FROM item WHERE itemid=__ItemID;	
	

	_PODPID:=COALESCE(__PID, '');
	IF(_isstock=true) THEN
		SELECT INTO _spretStockCard_ChNCr * FROM spStockCard_ChNCr(__ItemID, _WhID, _PODPID);

		UPDATE stock SET qtypo=qtypo-_qty1_receive_now
		WHERE itemid=__ItemID AND whid=_WhID AND pid=_PODPID;
			
	END IF;

	RETURN _qty1_receive_now;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/***************************************** OVERLOAD *************************************************/
CREATE OR REPLACE FUNCTION spPO_Item_Receive(
	__PONo 		VARCHAR, 
	__ItemID	VARCHAR, 
	__Qty1		NUMERIC, 
	__PODNo 	INT
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE
	_retval	NUMERIC(15,4);
BEGIN
	SELECT INTO _retval * FROM spPO_Item_Receive(__PONo, __ItemID, NULL, __Qty1, __PODNo);
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPO_ReOpen (
	_PONo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spretPO_ChNUp_Status	SPStatus%ROWTYPE;
	
	_PODNo			SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_POStatus		VARCHAR;
	_Qty1Order		NUMERIC(15,4);
	_Qty1Delivered		NUMERIC(15,4);
	_Qty1BO			NUMERIC(15,4);
	_Qty1BO_new		NUMERIC(15,4);
	cPOD 			REFCURSOR;
BEGIN
	SELECT INTO _POStatus postatus FROM PO WHERE pono=_PONo;
	IF (_POStatus='F') THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Sudah Terkirim Penuh! Order Pembelian tidak dapat dibuka kembali (spPO_ReOpen)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	OPEN cPOD FOR
	SELECT PODNo, WhID, ItemID, fQtyZToQty1(ItemID, Qty, Unit), QtyDelivered, QtyBO 
	FROM POD 
	WHERE (PONo=_PONo) AND (QtyBO>0);
	LOOP
		FETCH cPOD INTO _PODNo, _WhID, _ItemID, _Qty1Order, _Qty1Delivered, _Qty1BO;
		EXIT WHEN NOT FOUND;

		_Qty1BO_new=_Qty1BO;

		_retval.ErrMsg:='Error UPDATE POD:QtyBO (spPO_Close)';
		UPDATE POD SET QtyBO=QtyBO - _Qty1BO_new
		WHERE PONo=_PONo AND PODNo=_PODNo;

		_PID:='';

		_retval.ErrMsg:='Error UPDATE Stock:QtyPO (spPO_Close)';
		UPDATE Stock SET QtyPO=QtyPO + _Qty1BO_new								/** kembalikan BO -> Order **/
		WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;
	
	END LOOP;
	CLOSE cPOD;

	SELECT into _spretPO_ChNUp_Status * FROM spPO_ChNUp_Status (_PONo);
	IF _spretPO_ChNUp_Status.ErrNo<>0 THEN
		_retval.ErrNo:=_spretPO_ChNUp_Status.ErrNo;
		_retval.ErrMsg:=_spretPO_ChNUp_Status.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPO_Close (
	_PONo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_PODNo		SMALLINT;
	_ItemID		VARCHAR(20);
	_WhID		VARCHAR(3);
	_PID		VARCHAR(25);
	_Qty1Order	NUMERIC(15,4);
	_Qty1Delivered	NUMERIC(15,4);
	_Qty1BO		NUMERIC(15,4);
	_Qty1BO_new	NUMERIC(15,4);

	cPOD		REFCURSOR;

BEGIN
	OPEN cPOD FOR
		SELECT podno, whid, itemid, fQtyZToQty1(itemid, qty, unit), qtydelivered, qtybo 
		FROM pod 
		WHERE pono=_PONo AND ((fQtyZToQty1(itemid, qty, unit)-qtydelivered-qtybo)>0);
	LOOP
		FETCH cPOD INTO _PODNo, _WhID, _ItemID, _Qty1Order, _Qty1Delivered, _Qty1BO;
		EXIT WHEN NOT FOUND;		
	
		_Qty1BO_new=_Qty1Order-_Qty1Delivered-_Qty1BO;
	
		_retval.ErrNo:=12020003;
		_retval.ErrMsg:='Error UPDATE POD:QtyBO (spPO_Close)';
	
		UPDATE pod SET qtybo=qtybo+_Qty1BO_new
		WHERE pono=_PONo AND podno=_PODNo;
		_retval.ErrNo:=0;
		_retval.ErrMsg:='UPDATE SUKSES!';

		_PID='';

		_retval.ErrNo:=12020004;
		_retval.ErrMsg:='Error UPDATE Stock:QtyPO (spPO_Close)';
		UPDATE Stock SET QtyPO=QtyPO - _Qty1BO_new
		WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;
		
	END LOOP;
	CLOSE cPOD;

	SELECT INTO _spret * FROM spPO_ChNUp_Status(_PONo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'Tutup PO Berhasil');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPurc_Pay (
	_PurcNo		VARCHAR,
	_PPaidAmt	DECIMAL(19,4),
	_PayDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;

	_Total		DECIMAL(19,4);
	_PaidAmt	DECIMAL(19,4);
	isNegatif	BOOLEAN;
	_isDraft	BOOLEAN;
	_tmp_va1	VARCHAR(1);
BEGIN

	SELECT INTO
		_Total, _PaidAmt, _isDraft
		Total, PaidAmt, isDraft
	FROM Purc WHERE PurcNo=_PurcNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Pembelian #'||_PurcNo||' tidak ditemukan (spPurc_Pay)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Penjualan #'||_SaleNo||' masih DRAFT (spPurc_Pay)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	
	IF ((_PaidAmt+_PPaidAmt)>_Total) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Jumlah pembayaran Pembelian #'||_PurcNo||' melebihi dari sisa Hutang:'||CAST((_Total-_PaidAmt) AS VARCHAR)||' (spPurc_Pay)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	ELSIF ((_PaidAmt+_PPaidAmt)=_Total) THEN

		_tmp_va1:='F';
		UPDATE Purc SET PaidAmt=PaidAmt+_PPaidAmt, PayStatus=_tmp_va1, SettleDate=_PayDate WHERE PurcNo=_PurcNo;
		IF NOT FOUND THEN
			RAISE EXCEPTION 'Error Update PayStatus=F (spPurc_Pay)';
		END IF;
	ELSIF ((_PaidAmt+_PPaidAmt)<>0) AND ((_PaidAmt+_PPaidAmt)<_Total) THEN

		_tmp_va1:='P';
		UPDATE Purc SET PaidAmt=PaidAmt+_PPaidAmt, PayStatus=_tmp_va1, SettleDate=NULL WHERE PurcNo=_PurcNo;
		IF NOT FOUND THEN
			RAISE EXCEPTION 'Error Update PayStatus=P (spPurc_Pay)';
		END IF;
	ELSIF ((_PaidAmt+_PPaidAmt)=0) AND ((_PaidAmt+_PPaidAmt)<_Total) THEN
		_tmp_va1:='N';
		UPDATE Purc SET PaidAmt=PaidAmt+_PPaidAmt, PayStatus=_tmp_va1, SettleDate=NULL WHERE PurcNo=_PurcNo;
		IF NOT FOUND THEN
			RAISE EXCEPTION 'Error Update PayStatus=N (spPurc_Pay)';
		END IF;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSO_ReOpen (
	_SONo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 		SPStatus%ROWTYPE;
	_spretSO_ChNUp_Status	SPStatus%ROWTYPE;

	_SODNo			SMALLINT;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_SOStatus		VARCHAR;
	_Qty1Order		NUMERIC(15,4);
	_Qty1Delivered		NUMERIC(15,4);
	_Qty1BO			NUMERIC(15,4);
	_Qty1BO_new		NUMERIC(15,4);

	cSOD			REFCURSOR;

BEGIN
	_retval:=(50001, 'General Error spSO_ReOpen');

	SELECT INTO _SOStatus sostatus FROM SO WHERE sono=_SONo;
	IF (_SOStatus='F') THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Sudah Terkirim Penuh! Order Penjualan tidak dapat dibuka kembali (spSO_ReOpen)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	OPEN cSOD FOR
	SELECT SODNo, WhID, ItemID, fQtyZToQty1(ItemID, Qty, Unit), QtyDelivered, QtyBO 
	FROM SOD 
	WHERE (SONo=_SONo) AND (QtyBO>0);
	LOOP
		FETCH cSOD INTO _SODNo, _WhID, _ItemID, _Qty1Order, _Qty1Delivered, _Qty1BO;
		EXIT WHEN NOT FOUND;
		
		_Qty1BO_new=_Qty1BO;
	
		_retval.ErrMsg:='Error UPDATE SOD:QtyBO (spSO_Close)';
		UPDATE SOD SET QtyBO=QtyBO - _Qty1BO_new
		WHERE SONo=_SONo AND SODNo=_SODNo;

		_retval.ErrMsg:='Error UPDATE Stock:QtySO (spSO_Close)';	
		
		_PID:='';
		
		UPDATE Stock SET QtySO=QtySO + _Qty1BO_new WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;
		
	END LOOP;
	CLOSE cSOD;

	SELECT into _spretSO_ChNUp_Status * FROM spSO_ChNUp_Status (_SONo);
	IF _spretSO_ChNUp_Status.ErrNo<>0 THEN
		_retval.ErrNo:=_spretSO_ChNUp_Status.ErrNo;
		_retval.ErrMsg:=_spretSO_ChNUp_Status.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSO_Close (
	_SONo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	
	_SODNo		SMALLINT;
	_ItemID		VARCHAR(20);
	_WhID		VARCHAR(3);
	_PID		VARCHAR(25);
	_Qty1Order	NUMERIC(15,4);
	_Qty1Delivered	NUMERIC(15,4);
	_Qty1BO		NUMERIC(15,4);
	_Qty1BO_new	NUMERIC(15,4);

	cSOD		REFCURSOR;

BEGIN
	OPEN cSOD FOR
		SELECT SODNo, WhID, ItemID, fQtyZToQty1(ItemID, Qty, Unit), QtyDelivered, QtyBO 
		FROM SOD 
		WHERE (SONo=_SONo) AND ((fQtyZToQty1(ItemID, Qty, Unit)-QtyDelivered-QtyBO) > 0);
	LOOP
		FETCH cSOD INTO _SODNo, _WhID, _ItemID, _Qty1Order, _Qty1Delivered, _Qty1BO;
		
		IF NOT FOUND THEN
			EXIT;
		END IF;

		_Qty1BO_new=_Qty1Order-_Qty1Delivered-_Qty1BO;

		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error UPDATE SOD:QtyBO (spSO_Close)';
		UPDATE SOD SET QtyBO=QtyBO+_Qty1BO_new
		WHERE SONo=_SONo AND SODNo=_SODNo;
		

		_PID='';

		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error UPDATE Stock:QtySO (spSO_Close)';
		UPDATE Stock SET QtySO=QtySO - _Qty1BO_new
		WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;
		
	END LOOP;
	CLOSE cSOD;

	SELECT INTO _spret * FROM spSO_ChNUp_Status(_SONo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

		_retval.ErrNo:=0;
		_retval.ErrMsg:='Tutup SO Berhasil!!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSale_Item(
	_StartDate	DATE,
	_EndDate	DATE
) RETURNS SETOF tSale_Item AS 
$BODY$
DECLARE
	tmp	tSale_Item%ROWTYPE;
BEGIN
	
	FOR tmp IN
		SELECT 
			ItemID, 
			SUM(Qty) ,
			SUM(ListPrice*Qty) 
		FROM PurcD 
		GROUP BY ItemID LOOP

		RETURN NEXT tmp;
	END LOOP;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSRet_Contra (
	_SRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	
	_SRetType	VARCHAR(1);
	_TermType	VARCHAR(1);
	_IsContra	BOOLEAN;

BEGIN
	SELECT INTO
		_SRetType, _TermType, _IsContra 
		SRetType, TermType, IsContra
	FROM SRet 
	WHERE SRetNo=_SRetNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error Retur/Memo-Piutang #'||_SRetNo||' tidak ditemukan (spSRet_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_TermType='C') THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Retur/Memo-Piutang #'||_SRetNo||'=TUNAI, tidak bisa dipotong-bayarkan (spSRet_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF ((_TermType='R') AND (_IsContra=true)) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Retur/Memo-Piutang #'||_SRetNo||' sudah pernah dipotong-bayarkan sebelumnya (spSRet_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error Update SRet:IsContra (spSRet_Contra)';
	UPDATE SRet SET IsContra=true, sretstatus='Y' WHERE SRetNo=_SRetNo;
	_retval.ErrNo:=0;
	_retval.ErrMsg:='UPDATE SUKSES!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItemDesc_Unique (
	_ItemDesc	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	SELECT * FROM Item WHERE UPPER(ItemDesc)=UPPER(_ItemDesc);
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='ItemDesc not unique (spItemDesc_Unique)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSO_Item_Cancel(
	__SONo 		VARCHAR, 
	__ItemID 	VARCHAR, 
	__PID		VARCHAR,
	__Qty1 		NUMERIC(15,4), 
	__SODNo		INT
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE	
	_retval		NUMERIC(15,4);
	_Active		BOOLEAN;
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_WhID		VARCHAR(3);
	_SODPID		VARCHAR(25);
	_qty1_Ordered	NUMERIC(15,4);
	_qty1_Delivered	NUMERIC(15,4);
	_qty1_BackOrder	NUMERIC(15,4);
	_qty1_Remains	NUMERIC(15,4);
	_qty1_cancel	NUMERIC(15,4);
	_SODNo		INT;
BEGIN
	_qty1_cancel:=0;
	_SODNo:=__SODNo;

	SELECT INTO _Active Active 
	FROM so WHERE SONo=__SONo;

	IF NOT FOUND  THEN
		RAISE EXCEPTION '%', 'Order-Penjualan #'||__SONo||' tidak ditemukan (spSO_Item_Cancel)';
	END IF;

	SELECT INTO
		_WhID, _SODPID, _qty1_Delivered, _qty1_BackOrder 
		WhID, PID, QtyDelivered, QtyBO 
	FROM SOD
	WHERE 
		SONo=__SONo 
		AND ItemID=__ItemID 
		AND SODNo=_SODNo;

	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Order-Penjualan #%, detail ke-% tidak ditemukan! (spSO_Item_Cancel)', __SONo, __SODNo;
	END IF;


	IF (__Qty1>_qty1_Delivered) THEN
		_qty1_cancel:=_qty1_Delivered;
	ELSE
		_qty1_cancel:=__Qty1;
	END IF;

	UPDATE SOD SET QtyDelivered=QtyDelivered-_qty1_cancel
	WHERE SONo=__SONo AND SODNo=_SODNo;

	_SODPID:=COALESCE(_SODPID, '');	
	UPDATE Stock SET QtySO=QtySO + _qty1_cancel 
	WHERE ItemID=__ItemID AND WhID=_WhID AND PID=_SODPID;

	RETURN _qty1_cancel;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItemID_Exist(
	_ItemID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	SELECT * FROM Item WHERE ItemID=_ItemID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='ItemID tidak ditemukan (spItemID_Exist)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




DROP FUNCTION spSO_Item_Delivered(VARCHAR, VARCHAR, VARCHAR, NUMERIC, INT);

CREATE OR REPLACE Function spSO_Item_Delivered(
	__SONo		VARCHAR, 
	__ItemID	VARCHAR, 
	__PID		VARCHAR,
	__Qty1		NUMERIC, 
	__SODNo		INT
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE	
	_spretStockCard_ChNCr 	SPStatus%ROWTYPE;
	
	_Active			BOOLEAN;
	_isDraft		BOOLEAN;
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_WhID			VARCHAR(3);
	_SODPID			VARCHAR(25);
	_qty1_order		NUMERIC(15,4);
	_qty1_delivered		NUMERIC(15,4);
	_qty1_backorder		NUMERIC(15,4);
	_qty1_outstanding	NUMERIC(15,4);
	_qty1_deliveringnow	NUMERIC(15,4);

	_isstock		BOOLEAN;
BEGIN
	_qty1_deliveringnow:=0;

	SELECT INTO
		_Active, _isDraft
		Active, isDraft
	FROM SO WHERE SONo=__SONo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Order-Penjualan #% tidak ditemukan (spSO_Item_Delivered)', __SONo;
	END IF;

	IF (_Active=FALSE) THEN
		RAISE EXCEPTION 'Status Order-Penjualan #% : Tidak Aktiv (spSO_Item_Delivered)', __SONo;
	END IF;

	IF (_isDraft=TRUE) THEN
		RAISE EXCEPTION 'Order-Penjualan $% masih Draft tidak bisa digunakan! (spSO_Item_Delivered)', __SONo;
	END IF;


	SELECT INTO
		_Qty, _SODPID, _Unit, _WhID, _qty1_delivered, _qty1_backorder
		Qty, PID, Unit, WhID, QtyDelivered, QtyBO
	FROM SOD WHERE SONo=__SONo AND ItemID=__ItemID AND SODNo=__SODNo;	-- sengaja di select 3 kriteria biar mantap

	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Order-Penjualan #%, detail ke-% tidak ditemukan! (spSO_Item_Delivered)', __SONo, __SODNo;
	END IF;
	
	_qty1_order:=fQtyZToQty1(__ItemID, _Qty, _Unit);
	_qty1_outstanding=_qty1_order-_qty1_delivered;

	IF (__Qty1>_qty1_outstanding) THEN
		_qty1_deliveringnow=_qty1_outstanding;
	ELSE
		_qty1_deliveringnow=__Qty1;
	END IF;
	
	UPDATE SOD SET QtyDelivered=QtyDelivered + _qty1_deliveringnow
	WHERE SONo=__SONo AND SODNo=__SODNo;
			
	_SODPID:=COALESCE(_SODPID, '');
	
	SELECT INTO _isstock isstock FROM item where itemid=__ItemID;
	IF(_isstock=TRUE) THEN
		SELECT INTO _spretStockCard_ChNCr * FROM spStockCard_ChNCr (__ItemID, _WhID, _SODPID);

		UPDATE Stock SET QtySO=QtySO - _qty1_deliveringnow 
		WHERE ItemID=__ItemID AND WhID=_WhID AND PID=_SODPID;
	END IF;	

	RETURN _qty1_deliveringnow;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSO_Item_Cancel_DO(
	__SONo 		VARCHAR, 
	__ItemID 	VARCHAR, 
	__PID		VARCHAR,
	__Qty1 		NUMERIC(15,4), 
	__SODNo		INT
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE	
	_retval		NUMERIC(15,4);
	_Active		BOOLEAN;
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_WhID		VARCHAR(3);
	_SODPID		VARCHAR(25);
	_qty1_Ordered	NUMERIC(15,4);
	_qty1_DO	NUMERIC(15,4);
	_qty1_cancel	NUMERIC(15,4);
	_SODNo		INT;
BEGIN
	_qty1_cancel:=0;
	_SODNo:=__SODNo;

	SELECT INTO _Active Active 
	FROM so WHERE SONo=__SONo;

	IF NOT FOUND  THEN
		RAISE EXCEPTION '%', 'Order-Penjualan #'||__SONo||' tidak ditemukan (spSO_Item_Cancel_DO)';
	END IF;


	SELECT INTO
		_WhID, _SODPID, _qty1_DO
		WhID, PID, QtyDO
	FROM SOD
	WHERE 
		SONo=__SONo 
		AND ItemID=__ItemID 
		AND SODNo=_SODNo;

	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Order-Penjualan #%, detail ke-% tidak ditemukan! (spSO_Item_Cancel_DO)', __SONo, __SODNo;
	END IF;


	IF (__Qty1>_qty1_DO) THEN
		_qty1_cancel:=_qty1_DO;
	ELSE
		_qty1_cancel:=__Qty1;
	END IF;

	UPDATE SOD SET QtyDO=QtyDO-_qty1_cancel
	WHERE SONo=__SONo AND SODNo=_SODNo;
	UPDATE SO SET _xt=_xt WHERE SONO=__SONo;

	RETURN _qty1_cancel;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




DROP FUNCTION spSO_Item_DO(VARCHAR, VARCHAR, VARCHAR, NUMERIC, INT);

CREATE OR REPLACE Function spSO_Item_DO(
	__SONo		VARCHAR, 
	__ItemID	VARCHAR, 
	__PID		VARCHAR,
	__Qty1		NUMERIC, 
	__SODNo		INT
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE		
	_Active			BOOLEAN;
	_isDraft		BOOLEAN;
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_WhID			VARCHAR(3);
	_SODPID			VARCHAR(25);
	_qty1_order		NUMERIC(15,4);
	_qty1_DO		NUMERIC(15,4);
	_qty1_outstanding	NUMERIC(15,4);
	_qty1_deonow		NUMERIC(15,4);

	_isstock		BOOLEAN;
BEGIN
	_qty1_deonow:=0;

	SELECT INTO
		_Active, _isDraft
		Active, isDraft
	FROM SO WHERE SONo=__SONo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Order-Penjualan #% tidak ditemukan (spSO_Item_DO)', __SONo;
	END IF;

	IF (_Active=FALSE) THEN
		RAISE EXCEPTION 'Status Order-Penjualan #% : Tidak Aktiv (spSO_Item_DO)', __SONo;
	END IF;

	IF (_isDraft=TRUE) THEN
		RAISE EXCEPTION 'Order-Penjualan $% masih Draft tidak bisa digunakan! (spSO_Item_DO)', __SONo;
	END IF;


	SELECT INTO
		_Qty, _SODPID, _Unit, _WhID, _qty1_DO
		Qty, PID, Unit, WhID, QtyDO
	FROM SOD WHERE SONo=__SONo AND ItemID=__ItemID AND SODNo=__SODNo;	-- sengaja di select 3 kriteria biar mantap

	IF NOT FOUND  THEN
		RAISE EXCEPTION 'Order-Penjualan #%, detail ke-% tidak ditemukan! (spSO_Item_DO)', __SONo, __SODNo;
	END IF;
	
	_qty1_order:=fQtyZToQty1(__ItemID, _Qty, _Unit);
	_qty1_outstanding=_qty1_order-_qty1_DO;

	IF (__Qty1>_qty1_outstanding) THEN
		_qty1_deonow=_qty1_outstanding;
	ELSE
		_qty1_deonow=__Qty1;
	END IF;
	
	UPDATE SOD SET QtyDO=QtyDO + _qty1_deonow
	WHERE SONo=__SONo AND SODNo=__SODNo;
	UPDATE SO SET _xt=_xt WHERE SONO=__SONo;

	RETURN _qty1_deonow;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItemID_Last (
	_prefix			VARCHAR,
	__LastItemID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_LastItemID 	VARCHAR(20);
	_temp		VARCHAR(16);
BEGIN
	_temp=_prefix+'%';
	SELECT into _LastItemID MAX(ItemID)
	FROM Item WHERE ItemID LIKE _temp;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error ItemID dengan Prefix:'||_prefix||' tidak ditemukan! (spItemID_Last)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItemUnit_Conv (
	_ItemID		VARCHAR(20),
	_Unit		VARCHAR(5)
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE
	_Unit1		VARCHAR(5);
	_Unit2		VARCHAR(5);
	_Conv2		NUMERIC(15,4);
	_Unit3		VARCHAR(5);
	_Conv3		NUMERIC(15,4);
	_retval		NUMERIC(15,4);
BEGIN
	_retval:=NULL;

	SELECT INTO 
		_Unit1,_Unit2,_Unit3,_Conv2,_Conv3 
		Unit1,Unit2,Unit3,Conv2,Conv3
	FROM Item WHERE ItemID=_ItemID;

	IF NOT FOUND THEN
		EXIT;
	END IF;

	IF _Unit=_Unit1 THEN
		_retval=1;
	ELSIF _Unit=_Unit2 THEN
		_retval=_Conv2;
	ELSIF _Unit=_Unit3 THEN
		_retval=_Conv3;
	END IF;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItem_Unit_Valid (
	_ItemID		VARCHAR,
	_Unit		VARCHAR
) RETURNS BOOLEAN AS 
$BODY$
DECLARE	
	_TmpItemID	VARCHAR(20);
	_tmp		BOOLEAN;
BEGIN
	_tmp:=NULL;

	SELECT INTO _TmpItemID ItemID FROM Item
	WHERE ItemID=_ItemID AND ((_Unit=Unit1) OR ((Unit2 IS NOT NULL)
	AND (_Unit=Unit2)) OR ((Unit3 IS NOT NULL) AND (_Unit=Unit3)));
	IF FOUND THEN
		_tmp:=true;
	END IF;

	IF (_tmp IS NULL) THEN
		_tmp=false;
	END IF;

	RETURN _tmp;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItemID_Update (
	_oldItemID		VARCHAR,
	_newItemID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	
	_tempItemID	VARCHAR(20);
	__oldItemID	VARCHAR(20);
	_ItemID1	VARCHAR(20);
	_ItemID2	VARCHAR(20);
	_stage		SMALLINT;
	_state		BOOLEAN;
BEGIN
	IF (_oldItemID=_newItemID) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Kode Item lama tidak boleh sama dengan kode baru (spItemID_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO __oldItemID ItemID FROM Item WHERE ItemID=_newItemID;
	IF FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Kode Item baru sudah ada ! (spItemID_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_tempItemID:='TEMP_ITEMID';
	_ItemID1:=_oldItemID;
	_ItemID2:=_tempItemID;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error Insert Temporary Item! (spItemID_Update)';
	INSERT INTO Item(ItemID, ItemDesc) VALUES (_tempItemID, 'ItemID sementara');
	_retval.ErrNo:=333;
	_retval.ErrMsg:='INSERT SUKSES!';

	_stage:=1;
	_state:=true;

	WHILE ((_stage=1) OR (_state=true)) LOOP
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Error UPDATE SAdjD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE SAdjD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Error UPDATE PPRC, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE pprc SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Error UPDATE JOBCARDM, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE jobcardm SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Error UPDATE JOBCARDP, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE jobcardp SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Error UPDATE QPRC, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE qprc SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Error UPDATE SBS, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE sbs SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Error UPDATE COST, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE cost SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error UPDATE StockAD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE StockAD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error UPDATE STOCKLOG, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE stocklog SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error UPDATE CONSD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE consd SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
		
		_retval.ErrNo:=50005;
		_retval.ErrMsg:='Error UPDATE StockBal, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE StockBal SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error UPDATE Stock, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE Stock SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50007;
		_retval.ErrMsg:='Error UPDATE StTrD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE StTrD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50008;
		_retval.ErrMsg:='Error UPDATE CND, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE PRETD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50008;
		_retval.ErrMsg:='Error UPDATE PRETBNS, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE pretbns SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50009;
		_retval.ErrMsg:='Error UPDATE PurcD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE PurcD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50010;
		_retval.ErrMsg:='Error UPDATE PRcvD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE PRcvD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50011;
		_retval.ErrMsg:='Error UPDATE PRC, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE PRC SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50012;
		_retval.ErrMsg:='Error UPDATE POD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE POD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50012;
		_retval.ErrMsg:='Error UPDATE PREQD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE preqd SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50013;
		_retval.ErrMsg:='Error UPDATE DND, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE SRETD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50013;
		_retval.ErrMsg:='Error UPDATE WOD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE wod SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50014;
		_retval.ErrMsg:='Error UPDATE SaleD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE SaleD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50014;
		_retval.ErrMsg:='Error UPDATE DELID, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE delid SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50015;
		_retval.ErrMsg:='Error UPDATE DOD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE DEOD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50016;
		_retval.ErrMsg:='Error UPDATE SOD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE SOD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50017;
		_retval.ErrMsg:='Error UPDATE Prc, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE Prc SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
		
		_retval.ErrNo:=50019;
		_retval.ErrMsg:='Error UPDATE BoMD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE BoMD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50019;
		_retval.ErrMsg:='Error UPDATE BoM, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE bom SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50020;
		_retval.ErrMsg:='Error UPDATE Mfg, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE Mfg SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50021;
		_retval.ErrMsg:='Error UPDATE MfgM, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE MfgMD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;
	
		_retval.ErrNo:=50021;
		_retval.ErrMsg:='Error UPDATE MfgM, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE MfgMD SET itemidsub=_ItemID2 WHERE itemidsub=_ItemID1;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error UPDATE MfgP, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE MfgPD SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error UPDATE BNS, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE bns SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error UPDATE BNSD, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE bnsd SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error UPDATE BNST, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE bnst SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error UPDATE SALEBNS, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE salebns SET ItemID=_ItemID2 WHERE ItemID=_ItemID1;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error UPDATE Pic, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE Pic SET linkno=_ItemID2 where linkno=_ItemID1;
		update pic set picid=REPLACE(picid,_ItemID1,_ItemID2) where linkno=_ItemID1;
	
		_retval.ErrNo:=111;
		_retval.ErrMsg:='ALL UPDATE SUKSES!';
	
		IF (_stage=2) THEN
			EXIT;
		END IF;	
	
		_retval.ErrNo:=50025;
		_retval.ErrMsg:='Error UPDATE Item:(new)ItemID, stage:'||CAST(_stage AS VARCHAR)||' (spItemID_Update)';
		UPDATE Item SET ItemID=_newItemID WHERE ItemID=_oldItemID;
		_retval.ErrNo:=222;
		_retval.ErrMsg:='Update SEKSES!';
	
		_ItemID1:=_tempItemID;
		_ItemID2:=_newItemID;
		_stage:=2;
	END LOOP;

	_retval.ErrNo:=50026;
	_retval.ErrMsg:='Error DELETE (old)Item (spItemID_Update)';
	DELETE FROM Item WHERE ItemID=_tempItemID;

	_retval:=(0, 'DELETE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItem_Merge (
	_srcItemID	VARCHAR,
	_destItemID	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_WhID		VARCHAR(3);
	_PID		VARCHAR(25);
	_PYear		INTEGER;

	_StockBal01	NUMERIC(15,4);
	_StockBal02	NUMERIC(15,4);
	_StockBal03	NUMERIC(15,4);
	_StockBal04	NUMERIC(15,4);
	_StockBal05	NUMERIC(15,4);
	_StockBal06	NUMERIC(15,4);
	_StockBal07	NUMERIC(15,4);
	_StockBal08	NUMERIC(15,4);
	_StockBal09	NUMERIC(15,4);
	_StockBal10	NUMERIC(15,4);
	_StockBal11	NUMERIC(15,4);
	_StockBal12	NUMERIC(15,4);

	_Unit1		VARCHAR(5);
	_AvgCost	NUMERIC(19,4);
	_tmpDate	DATE;

	_srcLCDate	DATE;
	_srcLCUnit	VARCHAR(5);
	_srcLCPrice	NUMERIC(19,4);
	_srcLCDiscExp	VARCHAR(15);
	_srcLCDiscAmt	NUMERIC(19,4);
	_srcLCTaxID	VARCHAR(5);
	_srcLCTaxAmt	NUMERIC(19,4);
	_srcVendorID	VARCHAR(15);
	_StockAType	VARCHAR(5);

	_destLCDate	DATE;
	cStockBal	REFCURSOR;
	_ItemID		VARCHAR;
BEGIN	
	_StockAType=NULL;


	SELECT INTO _ItemID ItemID FROM Item WHERE ItemID=_srcItemID OR ItemID=_destItemID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Kode Barang Asal atau Tujuan tidak ditemukan ! (spItem_Merge)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cStockBal FOR
	SELECT WhID, PID, PYear, StockBal01, StockBal02, StockBal03, StockBal04, StockBal05,
	StockBal06, StockBal07, StockBal08, StockBal09, StockBal10, StockBal11, StockBal12 
	FROM StockBal 
	WHERE ItemID=_srcItemID ORDER BY WhID, PYear;
	LOOP
		FETCH cStockBal INTO _WhID, _PID, _PYear,
		  	_StockBal01, _StockBal02, _StockBal03, _StockBal04, _StockBal05, _StockBal06, 
			_StockBal07, _StockBal08, _StockBal09, _StockBal10, _StockBal11, _StockBal12;
		EXIT WHEN NOT FOUND;
		
		SELECT into _Unit1 Unit1 FROM Item WHERE ItemID=_srcItemID;
		_AvgCost:=fUnitCost(_srcItemID, _WhID, _PID, _Unit1);
		_tmpDate:=fDateTime_Compose(_PYear, 1, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal01, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
							    
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 2, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal02, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 3, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal03, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		_tmpDate:=fDateTime_Compose(_PYear, 4, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal04, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 5, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal05, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 6, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal06, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 7, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal07, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 8, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal08, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 9, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal09, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 10, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal10, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 11, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal11, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpDate:=fDateTime_Compose(_PYear, 12, 1);
		SELECT into _spret * FROM spStockBal_Update(_destItemID, _WhID, _PID, _StockBal12, 
							    _Unit1, 1, 0, _AvgCost, _tmpDate, _StockAType);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cStockBal;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error UPDATE StockAD (spItem_Merge)';
	UPDATE StockAD SET ItemID=_destItemID WHERE ItemID=_srcItemID;
	
	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error UPDATE StTrD (spItem_Merge)';
	UPDATE StTrD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error UPDATE PRETD (spItem_Merge)';
	UPDATE PRETD SET ItemID=_destItemID WHERE ItemID=_srcItemID;
	
	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error UPDATE PurcD (spItem_Merge)';
	UPDATE PurcD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error UPDATE PRcvD (spItem_Merge)';
	UPDATE PRcvD SET ItemID=_destItemID WHERE ItemID=_srcItemID;
		
	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error UPDATE POD (spItem_Merge)';
	UPDATE POD SET ItemID=_destItemID WHERE ItemID=_srcItemID;
	
	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error UPDATE PRvD (spItem_Merge)';
	UPDATE PReqD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50009;
	_retval.ErrMsg:='Error UPDATE SRETD (spItem_Merge)';
	UPDATE SRETD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50010;
	_retval.ErrMsg:='Error UPDATE SaleD (spItem_Merge)';
	UPDATE SaleD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50011;
	_retval.ErrMsg:='Error UPDATE ConsD (spItem_Merge)';
	UPDATE ConsD SET ItemID=_destItemID WHERE ItemID=_srcItemID;


	_retval.ErrNo:=50012;
	_retval.ErrMsg:='Error UPDATE DeliD (spItem_Merge)';
	UPDATE DeliD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50012;
	_retval.ErrMsg:='Error UPDATE DeOD (spItem_Merge)';
	UPDATE DeOD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50013;
	_retval.ErrMsg:='Error UPDATE SOD (spItem_Merge)';
	UPDATE SOD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50014;
	_retval.ErrMsg:='Error UPDATE BoMD (spItem_Merge)';
	UPDATE BoMD SET ItemID=_destItemID WHERE ItemID=_srcItemID;

	_retval.ErrNo=0;
	_retval.ErrMsg:='UPDATE SUKSES!';

/*
	SELECT into _srcLCPrice, _srcLCUnit, _srcLCDiscExp, _srcLCDiscAmt, _srcLCTaxID, _srcLCTaxAmt, _srcLCDate, _srcVendorID
		    LCPrice, LCUnit, LCDiscExp, LCDiscAmt, LCTaxID, LCTaxAmt, LCDate, VendorID
	FROM Item
	WHERE ItemID=_srcItemID;
	
	SELECT into _destLCDate LCDate 
	FROM Item WHERE ItemID=_destItemID;

	IF (_srcLCDate>_destLCDate) THEN
		_retval.ErrNo:=50015;
		_retval.ErrMsg:='Error Update LastCost! (spItem_Merge)';
		UPDATE Item SET 
			LCPrice=_srcLCPrice,
			LCDiscExp=_srcLCDiscExp,
			LCDiscAmt=_srcLCDiscAmt,
			LCTaxID=_srcLCTaxID,
			LCTaxAmt=_srcLCTaxAmt,
			LCUnit=_srcLCUnit,
			LCDate=_srcLCDate,
			VendorID=_srcVendorID
		WHERE ItemID=_destItemID;
	END IF;
*/

	_retval.ErrNo:=50016;
	_retval.ErrMsg:='Error DELETE source StockBal! (spItem_Merge)';
	DELETE FROM StockBal WHERE ItemID=_srcItemID;
	
	_retval.ErrNo:=50017;
	_retval.ErrMsg:='Error DELETE source Stock! (spItem_Merge)';
	DELETE FROM Stock WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50018;
	_retval.ErrMsg:='Error DELETE source Prc! (spItem_Merge)';
	DELETE FROM Prc WHERE ItemID=_srcItemID;

	_retval.ErrNo:=50019;
	_retval.ErrMsg:='Error DELETE source Item! (spItem_Merge)';
	DELETE FROM Item WHERE ItemID=_srcItemID;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItem_RemoveForced (
	_ItemID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_BoMID		VARCHAR(20);

BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error DELETE PurcD (spItem_RemoveForced)';
	DELETE FROM PurcD WHERE ItemID=_ItemID;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error DELETE PRETD (spItem_RemoveForced)';
	DELETE FROM PRETD WHERE ItemID=_ItemID;
	
	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error DELETE SaleD (spItem_RemoveForced)';
	DELETE FROM SaleD WHERE ItemID=_ItemID;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE SRETD (spItem_RemoveForced)';
	DELETE FROM SRETD WHERE ItemID=_ItemID;

	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error DELETE StockAD (spItem_RemoveForced)';
	DELETE FROM StockAD WHERE ItemID=_ItemID;

	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error DELETE Stock (spItem_RemoveForced)';
	DELETE FROM Stock WHERE ItemID=_ItemID;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error DELETE Stock (spItem_RemoveForced)';
	DELETE FROM StockBal WHERE ItemID=_ItemID;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE Prc (spItem_RemoveForced)';
	DELETE FROM Prc WHERE ItemID=_ItemID;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE StockLog (spItem_RemoveForced)';
	DELETE FROM stocklog WHERE itemid=_ItemID;

	_retval.ErrNo:=50009;
	_retval.ErrMsg:='Error DELETE Item (spItem_RemoveForced)';
	DELETE FROM Item WHERE ItemID=_ItemID;

	_retval:=(0, 'DELETE SUKSES!');
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItem_Remove (
	_ItemID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_RefNo		VARCHAR;
BEGIN
	SELECT INTO _RefNo stockano FROM stockad WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Item ini sudah digunakan transaksi (spItem_Remove)'||E'\n'||'Silahkan cek kartu stock untuk melihat histori transaksi yang menggunakan item ini !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	
	SELECT INTO _RefNo bomid FROM bom WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item ini sudah dipakai dalam B.o.M. #'||_BomID||' (spItem_Remove)'||E'\n'||'Silahkan cek B.o.M. tersebut !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo sretno FROM sretd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Retur Penjualan ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo sretno FROM sretd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Retur Penjualan ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo saleno FROM saled WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Penjualan ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo purcno FROM purcbns WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Bonus Pembelian ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo sretno FROM sretbns WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Bonus Retur Penjualan ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo bomid FROM bomp WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Output B.O.M. ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo delino FROM delid WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Pengiriman ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	SELECT INTO _RefNo pretno FROM pretd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Retur Pembelian ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo delino FROM delibns WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Bonus Pengiriman ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo mfgpno FROM mfgpd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Output Produksi ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo saleno FROM salebns WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Bonus Penjualan ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo purcno FROM purcd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Pembelian ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo mfgmno FROM mfgmd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Input Produksi ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo rouid FROM rou WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Alur Produksi ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo wono FROM wod WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Perintah Kerja ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo jobcardno FROM jobcardm WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Job Card ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo jobcardno FROM jobcardp WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Job Card ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	SELECT INTO _RefNo purcno FROM purcbns WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Bonus Pembelian ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo prcvno FROM prcvd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Penerimaan Pembelian ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo deono FROM deod WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Order Pengiriman ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo mfgno FROM mfg WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Produksi ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo consno FROM consd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Sell In Konsinyasi ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo consno FROM consbns WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Bonus Sell In Konsinyasi ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo deono FROM deobns WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada Bonus Order Pengiriman ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	SELECT INTO _RefNo pono FROM pod WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Order Pembelian ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo bnsno FROM bnst WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Bonus ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	SELECT INTO _RefNo pretno FROM pretd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Retur Pembelian ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo bnsno FROM bnsd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Bonus ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo stockano FROM stockad WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Stock ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo sttrno FROM sttrd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Mutasi Stock ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo bnsno FROM bns WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Bonus ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo preqno FROM preqd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Permintaan Pembelian ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo sono FROM sod WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Order Penjualan ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo sadjno FROM sadjd WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Penyesuaian Stock ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	SELECT INTO _RefNo pprcno FROM pprc WHERE itemid=_ItemID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item#'||_ItemID||' sudah direferensikan pada transaksi Promo ! (spItem_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error DELETE prc (spItem_Remove)';
	DELETE FROM prc WHERE itemid=_ItemID;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error DELETE Stock (spItem_Remove)';
	DELETE FROM stock WHERE itemid=_ItemID;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE StockBal (spItem_Remove)';
	DELETE FROM stockbal WHERE itemid=_ItemID;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE StockLog (spItem_Remove)';
	DELETE FROM stocklog WHERE itemid=_ItemID;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE PriceLog (spItem_Remove)';
	DELETE FROM pricelog WHERE itemid=_ItemID;

	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error DELETE Item (spItem_Remove)';
	DELETE FROM item WHERE itemid=_ItemID;
	
	_retval:=(0, 'DELETE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItem_Reset()
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
	_ItemID		VARCHAR;
	cItem		REFCURSOR;
BEGIN
	OPEN cItem FOR
		SELECT itemid 
		FROM Item
		WHERE (_xt!=true OR (itemid<>'0' AND itemid<>'-1'));
	LOOP 
		FETCH cItem INTO _ItemID;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spItem_RemoveForced(_ItemID);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMSg;
		END IF;

	END LOOP; 
	CLOSE cItem;

	_retval:=(0, 'DELETE SUKSES!');
	RETURN NEXT _retval;

END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;

/******************************************************************************************/
/*************************************** OVERLOAD ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spItem_Reset(
	_bujukan	VARCHAR(20)
) RETURNS SETOF spstatus AS 
$BODY$
DECLARE
	_spret	SPStatus%ROWTYPE;
	_retval	SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spItem_Reset();
		
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=_spret;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItemUnit_Change (
	_ItemID		VARCHAR,
	_oldUnit	VARCHAR,
	_newUnit	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	
	_oldUnit1		VARCHAR;
	_oldUnit2		VARCHAR;
	_oldUnit3		VARCHAR;
	_oldPurcUnit		VARCHAR;
	_oldStockUnit		VARCHAR;
	_oldSaleUnit		VARCHAR;
	_newUnit1		VARCHAR;
	_newUnit2		VARCHAR;
	_newUnit3		VARCHAR;
	_newPurcUnit		VARCHAR;
	_newStockUnit		VARCHAR;
	_newSaleUnit		VARCHAR;
	_newUnitDesc		VARCHAR;
	_Conv2			NUMERIC(15,4);
	_Conv3			NUMERIC(15,4);

BEGIN
	IF (_oldUnit=_newUnit) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Satuan tujuan tidak boleh sama dengan satuan asal (spItemUnit_Change)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT into _oldUnit1, _oldUnit2, _Conv2, _oldUnit3, _Conv3, _oldPurcUnit, _oldStockUnit, _oldSaleUnit
		    Unit1, Unit2, Conv2, Unit3, Conv3, PurcUnit, StockUnit, SaleUnit
	FROM Item WHERE ItemID=_ItemID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='ItemID:'||_ItemID||' tidak ditemukan (spItemUnit_Change)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF ((_newUnit=_oldUnit1) OR (_newUnit=_oldUnit2) OR (_newUnit=_oldUnit3)) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Satuan baru tidak boleh sama dengan salah satu satuan lama! (spItemUnit_Change)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_newUnit1=_oldUnit1;
	_newUnit2=_oldUnit2;
	_newUnit3=_oldUnit3;

	_newPurcUnit=_oldPurcUnit;
	_newStockUnit=_oldStockUnit;
	_newSaleUnit=_oldSaleUnit;
	_newUnitDesc='';

	IF (_oldUnit1=_oldUnit) THEN
		_newUnit1=_newUnit;
	ELSIF (_oldUnit2=_oldUnit) THEN
		_newUnit2=_newUnit;
	ELSIF (_oldUnit3=_oldUnit) THEN
		_newUnit3=_newUnit;
	END IF;

	IF (_oldPurcUnit=_oldUnit) THEN
		_newPurcUnit=_newUnit;
	END IF;
	IF (_oldStockUnit=_oldUnit) THEN
		_newStockUnit=_newUnit;
	END IF;
	IF (_oldSaleUnit=_oldUnit) THEN
		_newSaleUnit=_newUnit;
	END IF;

	_newUnitDesc=_newUnit1;
	IF ((_newUnit2 IS NOT NULL) AND (_Conv2>0)) THEN
		_newUnitDesc=_newUnitDesc||', '||_newUnit2||'/'||CAST(CAST(_Conv2 AS FLOAT) AS VARCHAR);
	END IF;
	IF ((_newUnit3 IS NOT NULL) AND (_Conv3>0)) THEN
		_newUnitDesc=_newUnitDesc||', '||_newUnit3||'/'||CAST(CAST(_Conv3 AS FLOAT) AS VARCHAR);
	END IF;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error Update Item:Unit (spItemUnit_Change)';
	UPDATE Item SET 
		Unit1=_newUnit1, Unit2=_newUnit2, Unit3=_newUnit3, 
		PurcUnit=_newPurcUnit, StockUnit=_newStockUnit, SaleUnit=_newSaleUnit,
		UnitDesc=_newUnitDesc
	WHERE ItemID=_ItemID;
	
	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error Update SOD (spItemUnit_Change)';
	UPDATE SOD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error Update BoMD (spItemUnit_Change)';
	UPDATE BoMD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error Update StTrD (spItemUnit_Change)';
	UPDATE StTrD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error Update StockAD (spItemUnit_Change)';
	UPDATE StockAD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50018;
	_retval.ErrMsg:='Error Update SAdjD (spItemUnit_Change)';
	UPDATE sadjd SET unit=_newUnit WHERE itemid=_ItemID AND unit=_oldUnit;

	_retval.ErrNo:=50019;
	_retval.ErrMsg:='Error Update PReq (spItemUnit_Change)';
	UPDATE preqd SET unit=_newUnit WHERE itemid=_ItemID AND unit=_oldUnit;

	_retval.ErrNo:=50009;
	_retval.ErrMsg:='Error Update POD (spItemUnit_Change)';
	UPDATE POD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50010;
	_retval.ErrMsg:='Error Update SRETD (spItemUnit_Change)';
	UPDATE SRetD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	/*_retval.ErrNo:=50011;
	_retval.ErrMsg:='Error Update PRC (spItemUnit_Change)';
	UPDATE PRC SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;*/

	_retval.ErrNo:=50012;
	_retval.ErrMsg:='Error Update PRETD (spItemUnit_Change)';
	UPDATE PRetD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50013;
	_retval.ErrMsg:='Error Update PurcD (spItemUnit_Change)';
	UPDATE PurcD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50014;
	_retval.ErrMsg:='Error Update PRcvD (spItemUnit_Change)';
	UPDATE PRcvD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50016;
	_retval.ErrMsg:='Error Update SaleD (spItemUnit_Change)';
	UPDATE SaleD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50017;
	_retval.ErrMsg:='Error Update cONSD (spItemUnit_Change)';
	UPDATE ConsD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50018;
	_retval.ErrMsg:='Error Update BoM (spItemUnit_Change)';
	UPDATE BoM SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50019;
	_retval.ErrMsg:='Error Update WOD (spItemUnit_Change)';
	UPDATE WOD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50020;
	_retval.ErrMsg:='Error Update JobcardM (spItemUnit_Change)';
	UPDATE JobcardM SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50021;
	_retval.ErrMsg:='Error Update JobcardP (spItemUnit_Change)';
	UPDATE JobcardP SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50022;
	_retval.ErrMsg:='Error Update MFGMD (spItemUnit_Change)';
	UPDATE MFGMD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval.ErrNo:=50023;
	_retval.ErrMsg:='Error Update MFGMP (spItemUnit_Change)';
	UPDATE MFGPD SET Unit=_newUnit WHERE ItemID=_ItemID AND Unit=_oldUnit;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spItGrp_Reset()
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error DELETE Item Grup (spItgrp_Reset)';
	DELETE FROM itgrp WHERE itgrpid<>'*';

	_retval:=(0, 'DELETE SUKSES!');
	RETURN NEXT _retval;

END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;

/******************************************************************************************/
/*************************************** OVERLOAD ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spItGrp_Reset(
	_bujukan	VARCHAR(20)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_spret	SPStatus%ROWTYPE;
	_retval	SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spItGrp_Reset();
		
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=_spret;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spModel_Reset()
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error DELETE Model (spModel_Reset)';
	DELETE FROM model;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='ALL DELETE SUCCESS!!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;

/******************************************************************************************/
/*************************************** OVERLOAD ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spModel_Reset(
	_bujukan	VARCHAR(20)
) RETURNS SETOF spstatus AS 
$BODY$
DECLARE
	_spret	SPStatus%ROWTYPE;
	_retval	SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spModel_Reset();
		
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=_spret;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spBrand_Reset()
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error DELETE Merk (spBrand_Reset)';
	DELETE FROM brand;

	_retval:=(0,'ALL DELETE SUCCESS');
	RETURN NEXT _retval;

END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spBrand_Reset(
	_bujukan	VARCHAR(20)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_spret	SPStatus%ROWTYPE;
	_retval	SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spBrand_Reset();
		
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=_spret;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCostInfo (
	__ItemID	VARCHAR,
	__LCPrice	NUMERIC,
	__LCUnit	VARCHAR,
	__LCDiscExp	VARCHAR,
	__LCDiscAmt	NUMERIC,
	__LCTaxID	VARCHAR,
	__LCTaxAmt	NUMERIC,
	__LCVendorID	VARCHAR,
	__LCDate	DATE,
	__AvgCost	NUMERIC
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_ItemID 	VARCHAR(20);
	_LCPrice 	NUMERIC(19,4);
	_LCUnit 	VARCHAR(5);
	_LCDiscExp 	VARCHAR(15);
	_LCDiscAmt 	NUMERIC(19,4);
	_LCTaxID 	VARCHAR(5);
	_LCTaxAmt 	NUMERIC(19,4);
	_LCVendorID 	VARCHAR(15);
	_LCDate 	DATE;
	_AvgCost 	NUMERIC(19,4);
BEGIN
	SELECT  into _LCPrice, _LCUnit, _LCDiscExp, _LCDiscAmt, _LCTaxID, _LCTaxAmt, _LCVendorID, _LCDate
		     LCPrice, LCUnit, LCDiscExp, LCDiscAmt, LCTaxID, LCTaxAmt, VendorID, LCDate
	FROM Item WHERE ItemID=_ItemID;

	SELECT into _AvgCost AVG(AvgCost)
	FROM STOCK WHERE ItemID=_ItemID;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fNoMiliSec (
	__Date	DATE
) RETURNS DATE AS 
$BODY$
DECLARE
	_tmp		VARCHAR(19);
	_retval		DATE;
BEGIN
	SELECT into _tmp * FROM CONVERT(VARCHAR,__Date,120);
	SELECT into _retval * FROM CAST(_tmp AS DATE);
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function spBoM_New (
	_BoMID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE	
	_retval		SPStatus%ROWTYPE;


	_ItemID		VARCHAR;
	_ItemID2	VARCHAR;
	_sumCost	NUMERIC(19,4);
	_BomDNo		INT2;
	_WHID		VARCHAR;
	_PID		VARCHAR;
	_Unit		VARCHAR;
	_Qty		NUMERIC(15,4);
	_Cost		NUMERIC(19,4);
	_bomdpercent	NUMERIC(19,4);

	_ItemUnit1	VARCHAR;
	_ItemUnit2	VARCHAR;
	_ItemUnit3	VARCHAR;
	cBomD		REFCURSOR;
BEGIN

	SELECT INTO _ItemID, _PID, _Unit itemid, pid, unit FROM bom WHERE bomid=_BoMID;
	IF (NOT FOUND OR _ItemID IS NULL) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BoM# '||_BoMID||' tidak ditemukan (spBoM_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 

	SELECT INTO _ItemUnit1, _ItemUnit2, _ItemUnit3 unit1,unit2,unit3 FROM item WHERE itemid=_ItemID;
	IF (_Unit<>_ItemUnit1 OR _Unit<>_ItemUnit2 OR _Unit<>_ItemUnit3) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Unit# '||_Unit||' pada Item# '||_ItemID||' tidak ditemukan  (spBoM_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _ItemID2 itemid FROM bomd WHERE bomid=_BoMID AND itemid=_ItemID; 
	IF (_ItemID2=_ItemID) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Item Output tidak boleh sama dengan Item Input (spBoM_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _ItemID2 itemid FROM bom 
	WHERE bomid<>_BoMID 
	AND itemid=_ItemID
	AND pid=_PID
	AND Active=TRUE;

	IF (_ItemID2=_ItemID) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Ada B.o.M lain yang masih Aktif. In-Aktifkan B.o.M yang lama terlebih dahulu !!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	

	/*UPDATE BoM SET TotStdCost=(SELECT SUM(Qty*StdCost) FROM BoMD WHERE BoMID=_BoMID) WHERE BoMID=_BoMID;	*/

	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function spBoM_Remove(_BoMID VARCHAR)
RETURNS SETOF SPSTATUS AS 
$BODY$
DECLARE	
	_retval		SPStatus%ROWTYPE;

	_count 		INT;
	_ID		VARCHAR;
BEGIN
	SELECT INTO _ID MfgNo FROM Mfg WHERE BomID=_BomID;
	IF (FOUND AND _ID IS NOT NULL) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='B.o.M. ini sudah digunakan dalam transaksi Produksi #'||_ID||' (spBoM_Remove)'||E'\n'||'Silahkan cek transaksi produksi yang menggunakan B.o.M. ini !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	DELETE FROM BoMD WHERE BoMID=_BoMID;
	DELETE FROM BoM WHERE BoMID=_BoMID;

	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCsTr_New (
	_CsTrNo			VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret 			SPStatus%ROWTYPE;

	_CsTrDate		DATE;
	_CashID1		VARCHAR(15);
	_CashID2		VARCHAR(15);
	_Amount			DECIMAL(19,4);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_CstrNote		VARCHAR;
	_CashANote		VARCHAR;
	_tmpAmount		DECIMAL(19,4);
	
	_AccNo1			VARCHAR(10);
	_AccNo2			VARCHAR(10);
	_Count			INT;
	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);
	_RefType		VARCHAR;
	_CashDesc1		VARCHAR;
	_CashDesc2		VARCHAR;
	_ForeignAmt		NUMERIC(19,4);
	_ExcRate1		NUMERIC(19,4);
	_ExcRate2		NUMERIC(19,4);
	_ExcRate3		NUMERIC(19,4);
	_CrcID1			VARCHAR;
	_CrcID2			VARCHAR;

	_JnlDAmt		NUMERIC(19,4);
	_JnlDAmtForex		NUMERIC(19,4);
	_isAutoReconcileIn	BOOLEAN;
	_isAutoReconcileOut	BOOLEAN;
	_LockDate		DATE;
	_isDraft		BOOL;
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;

	SELECT INTO
		_PerID, _BranchID, _CashID1, _CashID2, _CsTrDate, _Amount, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _CstrNote, 
		_RefType, _DeptID, _PrjID, _ForeignAmt, _ExcRate1, _ExcRate2, _ExcRate3, _CrcID1, _CrcID2, _isDraft
		perid, branchid, cashid1, cashid2, cstrdate, cstramt, crtby, crtdate, updby, upddate, cstrnote, 
		reftype, deptid, prjid, foreignamt, excrate1, excrate2, excrate3, crcid1, crcid2, isdraft
	FROM cstr WHERE cstrno=_CsTrNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Mutasi kas#'||COALESCE(_CsTrNo, '<kosong>')||' tidak ditemukan! (spCsTr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM tLock WHERE transtype='CSTR';
	IF(_LockDate IS NOT NULL AND _LockDate>_CsTrDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Mutasi kas melebihi batas tanggal Kunci transaksi ! (spCsTr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_CashID1=_CashID2) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Kas asal tidak boleh sama dengan kas tujuan! (spCsTr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_tmpAmount=(_Amount*-1);
	IF(_ForeignAmt IS NULL) THEN
		_ForeignAmt=_Amount;
	END IF;

	SELECT INTO _CashDesc1 cashdesc FROM cash WHERE cashid=_CashID1;
	SELECT INTO _CashDesc2 cashdesc FROM cash WHERE cashid=_CashID2;

	_CashANote:=COALESCE(_CstrNote, '')||', Mutasi dari '||_CashDesc1||' ke '||_CashDesc2;

	-- posting ke kartu kas
	SELECT INTO _spret * FROM spCashA_New(
		_CashID1, NULL, 'CSTR', _CsTrDate, 'O', 'CSTR', _CsTrNo,
		_tmpAmount, _CashANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
		_PerID, _BranchID, _DeptID, _PrjID, _CrcID1, _ExcRate1, 1
	);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spCashA_New(
		_CashID2, NULL, 'CSTR', _CsTrDate, 'I', 'CSTR', _CsTrNo,
		_ForeignAmt, _CashANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
		_PerID, _BranchID, _DeptID, _PrjID, _CrcID2, _ExcRate2, 1
	);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	------------------ /////////////// AKUNTANSI ////////////////////// ---------------------
	-- yang cbg nggak masuk sini soalnya udah dihandle sendiri ama sp nya CBG , _RefType null buat njagani klo bikin mutasi sendiri gak lewat transaksi
	IF(_RefType<>'CBG' OR _RefType IS NULL) THEN
		IF isAccInstalled() THEN
			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:='Mutasi Kas : '||fCashDesc(_CashID1)||' : '||_CsTrNo;

			SELECT INTO _spret * FROM spJnl_Insert(
				_JnlNo,'CADJ',_CsTrDate,'CADJ',_CsTrNo,
				_JnlNote,true,true,cast(0 as NUMERIC(19,4)),
				_CrtBy,_CrtDate,_UpdBy,_UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID1, _ExcRate1, 1
			);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			-- klo excrate3 ada isinya brarti beli mata uang yang bukan base currency. 
			-- jadi diambil dari nilainya sama kayak sisi kredit biar sama jumlahnya..
			-- klo g gitu kehalang di pembulatan soalnya!selisih dikit di digit belakang koma se
			-- ...tapi tetep aja g sama..g mau masuk jurnal!mangkannya diakalin gitu...*/
			IF(_ExcRate3<>0) THEN
				_JnlDAmt:=_Amount*_ExcRate1;
				_JnlDAmtForex:=_Amount;
			ELSE
				_JnlDAmt:=_ForeignAmt*_ExcRate2;
				_JnlDAmtForex:=_ForeignAmt;
			END IF;

			--- baca setting akun & reconcile cash/bank
			SELECT INTO 
				_AccNo1, _isAutoReconcileIn, _isAutoReconcileOut
				AccNo, isAutoReconcileIn, isAutoReconcileOut
			FROM cash 
			WHERE cashid=_CashID1;
			IF (_isAutoReconcileOut=FALSE) THEN
				_AccNo1:=fAccDefault('UNRECBANK');
			END IF;

			SELECT INTO 
				_AccNo2, _isAutoReconcileIn, _isAutoReconcileOut
				AccNo, isAutoReconcileIn, isAutoReconcileOut
			FROM cash 
			WHERE cashid=_CashID2;
			IF (_isAutoReconcileIn=FALSE) THEN
				_AccNo2:=fAccDefault('UNRECBANK');
			END IF;

			-- ayat debet
			_Count:=1;
			SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo2, 'D', _Amount*_ExcRate1, _DeptID, _PrjID, _JnlDAmtForex);

			-- ayat kredit, kenapa dibikin hitung ulang, useless karena untuk kasus mutasi kas, nilai basecurrency selalu sama antara debet dan kredit 
			_Count:=_Count+1;
			SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo1, 'K', _Amount*_ExcRate1, _DeptID, _PrjID, _Amount);

			-- trigger jurnal
			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCsTr_Void (
	_CsTrNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	cAcc		refcursor;
	_JnlNo		VARCHAR(15);
	_Tmp		VARCHAR;
	_LockDate	DATE;
	_CsTrDate	DATE;
	_isDraft		BOOL;
BEGIN

	SELECT INTO _Tmp,_CsTrDate,_isDraft CsTrNo,CsTrDate,isDraft FROM CsTr WHERE CsTrNo=_CsTrNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Mutasi-Kas #'||_CsTrNo||' tidak ditemukan! (spCsTr_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		DELETE FROM CsTr WHERE CsTrNo=_CsTrNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CSTR';
	IF(_LockDate is not null AND _LockDate>_CsTrDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Mutasi Kas Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT into _spret * FROM spCashA_Void_byRef('CSTR', _CsTrNo, null, null);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF isAccInstalled() THEN
		/*--SELECT into _spret * from spJnl_Void('CADJ',_CsTrNo);*/
		/*--SELECT INTO _JnlNo jnlno FROM jnl WHERE refno=_CsTrNo;*/
		/*--SELECT into _spret * from spJnl_Void(_JnlNo);*/
		SELECT into _spret * from spJnl_Void_byRef('CADJ',_CsTrNo);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
	END IF;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error Delete CsTr (spCsTr_Void)';
	DELETE FROM CsTr WHERE CsTrNo=_CsTrNo;

	_retval:=(0, 'DELETE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSRet_VoidContra (
	_SRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_SRetType	VARCHAR(1);
	_TermType	VARCHAR(1);
	_IsContra	BOOLEAN;
BEGIN
	SELECT into 
		_SRetType, _TermType, _IsContra
		SRetType, TermType, IsContra
	FROM SRet WHERE SRetNo=_SRetNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error Memo-Piutang #'||_SRetNo||' tidak ditemukan! (spSRet_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_TermType='C') THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Memo-Piutang #'||_SRetNo||'=TUNAI! (spSRet_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_IsContra<>true) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Error Memo-Piutang #'||_SRetNo||' belum pernah dipotongkan sebelumnya (spSRet_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;


	UPDATE SRet SET IsContra=FALSE WHERE SRetNo=_SRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error Update SRet:IsContra (spSRet_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/************************************

fUnitCost overloaded jadi 2 param: 
	a. 5 params (pakai Qty)
	b. 4 params (tanpa Qty, dianggap Qty1)

*************************************/


CREATE OR REPLACE FUNCTION fUnitCost(
	_ItemID 	VARCHAR(20),
	_WhID 		VARCHAR(3),
	_PID 		VARCHAR(25),
	_Qty		DECIMAL(15,4),
	_Unit 		VARCHAR(5)
) RETURNS NUMERIC(19,4) AS 
$BODY$
DECLARE
	_retval			NUMERIC(19,4);

	_ItemType		VARCHAR(5);
	_IsStock		BOOLEAN;
	_IsAsm			BOOLEAN;
	_IsMfg			BOOLEAN;
	_BoMID			VARCHAR(20);
	_Unit1			VARCHAR(5);
	_Conv			NUMERIC(15,4);		/* Multiply Factor from UnitX to Unit1 */
	_BoMDItemID		VARCHAR(15);
	_BoMDWhID		VARCHAR(3);
	_BoMDQty		NUMERIC(15,4);
	_BoMDUnit		VARCHAR(5);
	_BoMD_ItemCost		NUMERIC(40,20);		/*-- Cost 1PCS item detail BoM*/
	_BoM_ItemsCost		NUMERIC(40,20);		/*-- Total cost untuk 1 set(UNIT1) Item BoM*/

	_CostUnit1		NUMERIC(40,20);
	_stock_SubTotal		DECIMAL(40,20);
	_stock_Qty		DECIMAL(15,4);

	cBoMD			REFCURSOR;

	_AvgCost		NUMERIC(19,4);

	_Multiplier		NUMERIC(19,4);
	_BomItemID		VARCHAR;
	_BomQty			NUMERIC(19,4);
	_BomUnit		VARCHAR;
BEGIN

	SELECT INTO 
		_ItemType,_IsMfg,_Unit1,_IsStock 
		ItemType,IsMfg,Unit1,IsStock
	FROM Item WHERE ItemID=_ItemID;

	-- read qty conversion
	_Conv:=fItemUnit_Conv(_ItemID, _Unit);

	IF (_IsStock=TRUE) THEN	-- Item Normal
		IF _PID IS NULL THEN
			SELECT INTO 
				_stock_SubTotal, _stock_Qty, _AvgCost 
				SubTotal, Qty, avgcost
			FROM Stock 
			WHERE ItemID=_ItemID AND WhID=_WhID;
		ELSE
			SELECT INTO 
				_stock_SubTotal, _stock_Qty, _AvgCost 
				SubTotal, Qty, avgcost 
			FROM Stock 
			WHERE ItemID=_ItemID AND WhID=_WhID AND PID=_PID;
		END IF;

		IF NOT FOUND THEN
			RETURN NULL;
		END IF;

		IF (_stock_Qty<>0) THEN
			_CostUnit1:=_Qty*_Conv*_stock_SubTotal/_stock_Qty;
		ELSE
			_CostUnit1:=_Qty*_Conv*_AvgCost;
		END IF;

		IF (_CostUnit1 IS NULL) THEN
			RETURN NULL;
		END IF;
	ELSIF (_ItemType='PACK' OR _IsAsm=TRUE) THEN /** item produksi atau paket */
		_BoM_ItemsCost:=0;
		SELECT INTO _BoMID, _BomQty, _BomUnit bomid, qty, unit FROM bom WHERE itemid=_ItemID AND pid=COALESCE(_PID, '') AND active=TRUE;
		IF FOUND THEN
			_BoM_ItemsCost:=0;

			_BomQty:=fQtyZToQty1(_ItemID, _BomQty, _BomUnit);
			_Multiplier:=fQtyZToQty1(_ItemID, _Qty, _Unit)/_BomQty;
			OPEN cBoMD FOR
				SELECT ItemID, WhID, Qty, Unit FROM BoMD WHERE BoMID=_BoMID;
			LOOP
				FETCH cBoMD INTO _BoMDItemID, _BoMDWhID, _BoMDQty, _BoMDUnit;
				EXIT WHEN NOT FOUND;

				SELECT INTO _BoMD_ItemCost * FROM fUnitCost(_BoMDItemID, _WhID, null, _BomDQty, _BoMDUnit);
				_BoM_ItemsCost=_BoM_ItemsCost+(_Conv*_BoMD_ItemCost*_Multiplier);

			END LOOP;
			CLOSE cBoMD;
		END IF;

		_CostUnit1:=_BoM_ItemsCost;

	ELSIF (_ItemType='SERV') THEN 
		_CostUnit1:=0;
	ELSE
		_CostUnit1:=0;
	END IF;

	_retval:=_CostUnit1;
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD-1 ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION fUnitCost(
	_ItemID		VARCHAR(20),
	_WhID		VARCHAR(3),
	_PID		VARCHAR(25),
	_Unit		VARCHAR(5)
) RETURNS DECIMAL(19,4) AS 
$BODY$
DECLARE
	_retval		DECIMAL(19,4);
BEGIN
	_retval:=fUnitCost(_ItemID, _WhID, _PID, 1, _Unit);

	RETURN _retval;
END
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fUnitDesc (
	_ItemID		VARCHAR(20)
) RETURNS VARCHAR(255) AS 
$BODY$
DECLARE
	_Unit1		VARCHAR(5);
	_Unit2		VARCHAR(5);
	_Conv2		DECIMAL(15,4);
	_Unit3		VARCHAR(5);
	_Conv3		DECIMAL(15,4);
	_retval		VARCHAR(255);
BEGIN
	_retval:=null;

	BEGIN
	SELECT INTO _Unit1,_Unit2,_Unit3,_Conv2,_Conv3 Unit1,Unit2,Unit3,Conv2,Conv3
	FROM Item WHERE ItemID=_ItemID;

	IF NOT FOUND THEN
		EXIT;
	END IF;
	_retval = _Unit1;

	IF(_Unit2 IS NOT NULL AND LENGTH(_Unit2)>0) THEN
		_retval=_retval||',';
		_retval=_retval||_Unit2;
		_retval=_retval||'/';
		_retval=_retval||CAST(CAST(_Conv2 AS DECIMAL(15)) AS VARCHAR);
	END IF;

	IF(_Unit3 IS NOT NULL AND LENGTH(_Unit3)>0) THEN
		_retval=_retval||',';
		_retval=_retval||_Unit3;
		_retval=_retval||'/';
		_retval=_retval||CAST(CAST(_Conv3 AS DECIMAL(15)) AS VARCHAR);
	END IF;

		/*--_retval = _Unit1||','||_Unit2||'/'||CAST(CAST(_Conv2 AS DECIMAL(15)) AS VARCHAR)||','||_Unit3||'/'||CAST(CAST(_Conv3 AS DECIMAL(15)) AS VARCHAR);*/
	END;
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fUnit1 (
	_ItemID		VARCHAR(20)
) RETURNS VARCHAR(5) AS 
$BODY$
DECLARE
	_retval		VARCHAR(5);
BEGIN
	_retval:=NULL;

	SELECT INTO _retval Unit1
	FROM Item WHERE ItemID=_ItemID;

	IF NOT FOUND THEN
		RETURN NULL;
	END IF;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function spSale_xt_Test (
	_SaleNo		VARCHAR
) RETURNS BOOLEAN AS 
$BODY$
DECLARE	
	_Total		NUMERIC(19,4);
	_xt		BOOL;
BEGIN
	/*_xt=true;*/

	_xt:=true;

	SELECT INTO _Total Total FROM Sale WHERE SaleNo=_SaleNo;
	IF (_Total>200000) THEN
		_xt:=FALSE;
	END IF;

	RETURN _xt;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCash_Check (
	_CashID		VARCHAR,
	_LOpEmpID	VARCHAR,
	_LOpUsrID	VARCHAR,
	_LOpDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval SPStatus%ROWTYPE;
	
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error Update Cash:LastChkDate (spCash_Check)';
	UPDATE Cash SET LOpEmpID=_LOpEmpID, LOpUsrID=_LOpUsrID, LOpDate=_LOpDate 
	WHERE CashID=_CashID;

	_retval:=(0, 'UPDATE SUKSES (spCash_Check)!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spGetDateTime (
	__CurDateTime	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_CurDateTime	DATE;
BEGIN
	SELECT into _CurDateTime * FROM fNoMiliSec(GETDATE());
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spChPass(
	_UsrID		VARCHAR,
	_OldPass	VARCHAR,
	_NewPass	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	SELECT UsrID FROM Usr
	WHERE UsrID=_UsrID AND (passwd=_OldPass OR passwd IS NULL);
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Login/Password salah ! (spChPass)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	ELSE
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Update Usr:Password ! (spChPass)';
		UPDATE Usr SET Pwd=_NewPass WHERE UsrID=_UsrID;
		_retval:=(0, 'UPDATE SUKSES');
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spgeneratepriv(_grpid varchar)
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval spStatus%ROWTYPE;
	_ada int;
	_objid varchar;
	_acsenable boolean;
	_acstype varchar;
	cGrpPrv refcursor;
BEGIN
	_retval := (1, 'fail to Generate Privilage ' || _grpid);
	_ada:=0;
	SELECT INTO _ada count(*) FROM grpprv WHERE grpid=_grpid;
	IF _ada>0 THEN
		_retval := (2, 'Group ' || _grpid || ' Sudah ada Privilage');
	ELSE
		OPEN cGrpPrv FOR
			select objid,acstype,acsenable from objacl;
		LOOP
			FETCH cGrpPrv INTO _objid,_acstype,_acsenable;
			IF NOT FOUND THEN
				EXIT;
			END IF;

			INSERT INTO grpprv(grpid,objid,acstype,acsenable) values(_grpid,_objid,_acstype,_acsenable);
		END LOOP;
		CLOSE cGrpPrv;
		_retval := (0, 'Succes to Generate Privilage ' || _grpid);
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spGrp_Remove(
	_GrpID	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error DELETE Users yang termasuk dalam Grup:'||_GrpID||' (spGrp_Remove)';
	DELETE FROM UsrGrp WHERE GrpID=_GrpID;
	
	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error DELETE GrpPrv (spGrp_Remove)';
	DELETE FROM GrpPrv WHERE GrpID=_GrpID;
	
	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error DELETE WhPrv (spGrp_Remove)';
	DELETE FROM WhPrv WHERE GrpID=_GrpID;


	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE Grup (spGrp_Remove)';
	DELETE FROM Grp WHERe GrpID=_GrpID;

	_retval:=(0, 'DELETE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spsetpassword(_userid varchar, _pass varchar, _oldpass varchar, _reset boolean)
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval spStatus%ROWTYPE;
	_found	VARCHAR;
BEGIN
	BEGIN
		IF _reset THEN
			_retval.ErrNo:=1;
			_retval.ErrMsg:='Error ganti password';

			UPDATE usr SET passwd = md5(_pass) WHERE usrid=_userid;
			_retval := (0, 'OK');
		ELSE
			_retval.ErrNo:=1;
			_retval.ErrMsg:='Password tidak sesuai dengan yang ada di database';

			SELECT INTO _found usrid FROM usr WHERE (usrid=_userid AND (passwd IS NULL OR passwd='' OR passwd=_oldpass));
			IF found THEN
				UPDATE usr SET passwd = md5(_pass) WHERE usrid=_userid;
				_retval := (0, 'OK');
			END IF;
		END IF;
	EXCEPTION
		WHEN OTHERS THEN NULL;
	END;

	RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPRet_Contra (
	_PRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_PRetType	VARCHAR(1);
	_TermType	VARCHAR(1);
	_IsContra	BOOLEAN;

BEGIN
	SELECT into _PRetType, _IsContra PRetType, IsContra
	FROM PRet WHERE PRetNo=_PRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Retur/Memo-Hutang #'||_PRetNo||' tidak ditemukan (spPRet_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_TermType='C') THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Retur/Memo-Hutang #'||_PRetNo||'=TUNAI, tidak bisa dipotong-bayarkan (spPRet_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_IsContra=true) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Retur/Memo-Hutang #'||_PRetNo||' sudah pernah dipotong-bayarkan sebelumnya (spPRet_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;
	
	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error Update PRet:IsContra (spPRet_Contra)';
	UPDATE PRet SET IsContra=true,pretstatus='Y' WHERE PRetNo=_PRetNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPRet_VoidContra (
	_PRetNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;

	_IsContra	BOOLEAN;
BEGIN
	SELECT INTO _IsContra IsContra
	FROM PRet WHERE PRetNo=_PRetNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Retur/Memo-Hutang #'||_PRetNo||' tidak ditemukan! (spPRet_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_IsContra<>true) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Retur/Memo-Hutang #'||_PRetNo||' belum pernah dipotongkan sebelumnya! (spPRet_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error Update PRet:IsContra! (spPRet_VoidContra)';
	UPDATE PRet SET IsContra=false,pretstatus='N' WHERE PRetNo=_PRetNo;

	_retval:=(0, 'UPDATE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function spSO_ChNUp_Status (
	_SONo		VARCHAR
) RETURNS SETOF SPSTATUS AS 
$BODY$
DECLARE	
	_retval		SPStatus%ROWTYPE;
	
	_SOExpDate	DATE;

	_SODNo		SMALLINT;
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_Qty1_O		NUMERIC(15,4);		/** Ordered Qty **/
	_Qty1_D		NUMERIC(15,4);		/** Delivered Qty **/
	_Qty1_B		NUMERIC(15,4);		/** Back-Order Qty **/

	_Qty1_O_sum	NUMERIC(15,4);		/** Sum Ordered Qty **/
	_Qty1_D_sum	NUMERIC(15,4);		/** Sum Delivered Qty **/
	_Qty1_B_sum	NUMERIC(15,4);		/** Sum Back-Order Qty **/

	_Qty1_R1	NUMERIC(15,4);		/** R1 = Order - Delivered ==> untuk menentukan SOStatus **/
	_Qty1_R2	NUMERIC(15,4);		/** R2 = Order - Delivered - B.O. ==> untuk menentukan "Active" **/

	_Expired	BOOLEAN;
	_IsFull		BOOLEAN;
	_IsPartial	BOOLEAN;
	_IsClear	BOOLEAN;
	_Active_orig	BOOLEAN;
	_Active		BOOLEAN;
	_SOStatus	VARCHAR(1);
	cSOD		refcursor;

BEGIN
	IF (_SONo IS NULL OR _SONo='') THEN	-- utk melayani SP lain yg memang gak nge-check valuenya, kita ngalahin ngecheck disini
		_retval:=(0,'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;


	SELECT into _Active_orig Active from SO WHERE SONo=_SONo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error SO not found (spSO_ChNUp_Status)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_IsFull=true;			/** default SOStatus=FULL ***/
	_IsPartial=false;		/** default SOStatus=FULL ***/
	_IsClear=true;			/** default CLOSED **/
	_Active=true;			/** default OPEN **/
	_Qty1_O_sum:=0;
	_Qty1_D_sum:=0;
	_Qty1_B_sum:=0;

	OPEN cSOD FOR
	SELECT SODNo, fQtyZToQty1(ItemID, Qty, Unit), QtyDelivered, QtyBO from soD WHERE SONo=_SONo;
	LOOP
		FETCH cSOD INTO _SODNo, _Qty1_O, _Qty1_D, _Qty1_B;
		EXIT WHEN NOT FOUND;

		IF(_Qty1_O<0) THEN
			_Qty1_O:=_Qty1_O*(-1);
		END IF;
			_Qty1_O_sum:=_Qty1_O_sum+_Qty1_O;

		IF(_Qty1_D<0) THEN
			_Qty1_D:=_Qty1_D*(-1);
		END IF;
			_Qty1_D_sum:=_Qty1_D_sum+_Qty1_D;

		IF(_Qty1_B<0) THEN
			_Qty1_B:=_Qty1_B*(-1);
		END IF;
			_Qty1_B_sum:=_Qty1_B_sum+_Qty1_B;

		_Qty1_R1 = _Qty1_O_sum - _Qty1_D_sum;
		_Qty1_R2 = _Qty1_O_sum - _Qty1_D_sum - _Qty1_B_sum;

		IF (_Qty1_R1>0) THEN
			_IsFull:=false;
		END IF;

		IF (_Qty1_D>0) THEN
			_IsPartial:=true;
		END IF;

		IF (_Qty1_R2>0) THEN
			_IsClear:=false;
		END IF;
	END LOOP;
	CLOSE cSOD;

/*	IF (_IsFull=true) THEN _SOStatus:='F';
	ELSIF ((_IsFull=false) AND (_IsPartial=true)) THEN _SOStatus:='P';
	ELSIF ((_IsFull=false) AND (_IsPartial=false)) THEN _SOStatus:='N';
	END IF;

	IF (_IsClear=true) THEN 
		_Active:=false;
	END IF;*/

	IF(_Qty1_R1=0) THEN
		_SOStatus='F';
	ELSIF(_Qty1_R1<>0 AND _Qty1_O_SUM<>_Qty1_R1) THEN
		_SOStatus='P';
	ELSE
		_SOStatus='N';
	END IF;

	IF(_Qty1_R2>0 AND _SOStatus='F') THEN
		_Active=FALSE;
	ELSIF(_Qty1_R2>0) THEN
		_Active=TRUE;
	ELSE
		_Active=FALSE;
	END IF;


	_retval.ErrNo=50001;
	_retval.ErrMsg:='Error UPDATE SO:SOStatus,Active (spSO_ChNUp_Status)';
	UPDATE SO SET SOStatus=_SOStatus, Active=_Active WHERE SONo=_SONo;

	_retval=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSale_DP (
	_SaleNo		VARCHAR,
	_DPAmt		NUMERIC
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	
	_CustID		VARCHAR(15);
	_SaleDate	DATE;
	_TermType	VARCHAR(1);
	_Total		NUMERIC(19,4);
	_PayStatus	VARCHAR(1);
	_PaidAmt	NUMERIC(19,4);
	_PoSID		VARCHAR(5);

	_RcvNo		VARCHAR(15);
	_RcvNote	VARCHAR(50);

	_CashID		VARCHAR(15);

BEGIN
	SELECT INTO _CustID, _SaleDate, _TermType, _Total, _PayStatus, _PaidAmt, _PoSID
		    CustID, SaleDate, TermType, Total, PayStatus, PaidAmt, PoSID
	FROM Sale
	WHERE SaleNo=_SaleNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Penjualan #'||_SaleNo||' tidak ditemukan (spSale_DP)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_TermType<>'R') THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Penjualan #'||_SaleNo||' bukan Credit (spSale_DP)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF ((_PayStatus<>'N') OR (_PaidAmt>0)) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Penjualan #'||_SaleNo||' sudah ada pembayaran (spSale_DP)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF ((_PoSID IS NULL) OR (_PoSID='')) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Error, Kode P.o.S. tidak ditemukan (spSale_DP)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO _CashID CashID FROM PoS WHERE PoSID=_PoSID;
	IF ((_CashID IS NULL) OR (_CashID='')) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Error, Kas untuk D.P. tidak ditemukan (spSale_DP)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO _spret * FROM spTransNoGet('RECV', _RcvNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.Errmsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_RcvNote='DP -> '||_SaleNo;
	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error INSERT Rcv! (spSale_DP)';
	INSERT INTO Rcv(RcvNo, RcvDate, RcvType, CustID, TotSPaid, TotRcvAmt, RcvNote)
	VALUES (_RcvNo, _SaleDate, 'P', _CustID, _DPAmt, _DPAmt, _RcvNote);
	
	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error INSERT SPaid! (spSale_DP)';
	INSERT INTO SPaid(RcvNo, SPaidNo, SaleNo, SPaidAmt, SPaidNote)
	VALUES (_RcvNo, true, _SaleNo, _DPAmt, NULL);

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error INSERT RcvD! (spSale_DP)';
	INSERT INTO RcvD(RcvNo, RcvDNo, RcvDMtd, CashID, RcvDAmt, RcvDNote)
	VALUES (_RcvNo, true, 'CASH', _CashID, _DPAmt, 'D.P. Tunai');

	_retval.ErrNo:=0;
	_retval.ErrMsg:='ALL INSERT SUCCESS!!';
	
	SELECT INTO _spret * FROM spRcv_New(_RcvNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSaleNo2_Sync (
	_newNo		INT
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval	SPStatus%ROWTYPE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='ERROR UPDATE NCount (spSaleNo2_Sync)';
	UPDATE NCount SET CurNo=_newNo WHERE NoID='SALE2';
	_retval.ErrNo:=0;
	_retval.ErrMsg:='UPDATE SUCCESS!!';
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSale_Rcv (
	_SaleNo		VARCHAR,
	_SPaidAmt	DECIMAL(19,4),
	_RcvDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_Total		DECIMAL(19,4);
	_PaidAmt	DECIMAL(19,4);
	isNegatif	BOOLEAN;
	_isDraft	BOOLEAN;
	_tmp_va1	VARCHAR(1);
BEGIN

	SELECT INTO 
		_Total, _PaidAmt, _isDraft
		Total, PaidAmt, isDraft
	FROM Sale WHERE SaleNo=_SaleNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Penjualan #'||_SaleNo||' tidak ditemukan (spSale_Rcv)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	IF (_isDraft=TRUE) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Penjualan #'||_SaleNo||' masih DRAFT (spSale_Rcv)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF ((_PaidAmt+_SPaidAmt)>_Total) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Jumlah pembayaran Penjualan #'||_SaleNo||' lebih besar dari sisa Piutang:'||CAST((_Total-_PaidAmt+_SPaidAmt) AS VARCHAR)||' (spSale_Rcv)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	ELSIF ((_PaidAmt+_SPaidAmt)=_Total) THEN
		_tmp_va1:='F';
		UPDATE Sale SET PaidAmt=PaidAmt+_SPaidAmt, PayStatus=_tmp_va1, SettleDate=_RcvDate WHERE SaleNo=_SaleNo;
		IF NOT FOUND THEN
			RAISE EXCEPTION 'Error Update PayStatus=F (spSale_Rcv)';
		END IF;

	ELSIF (((_PaidAmt+_SPaidAmt)<>0) AND ((_PaidAmt+_SPaidAmt)<_Total)) THEN
		_tmp_va1:='P';
		UPDATE Sale SET PaidAmt=PaidAmt+_SPaidAmt, PayStatus=_tmp_va1, SettleDate=NULL WHERE SaleNo=_SaleNo;
		IF NOT FOUND THEN
			RAISE EXCEPTION 'Error Update PayStatus=P (spSale_Rcv)';
		END IF;

	ELSIF (((_PaidAmt+_SPaidAmt)=0) AND ((_PaidAmt+_SPaidAmt)<_Total)) THEN
		_tmp_va1:='N';
		UPDATE Sale SET PaidAmt=PaidAmt+_SPaidAmt, PayStatus=_tmp_va1, SettleDate=NULL WHERE SaleNo=_SaleNo;
		IF NOT FOUND THEN
			RAISE EXCEPTION 'Error Update PayStatus=N (spSale_Rcv)';
		END IF;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPoS_CollectAuto (
	_PoSSesNo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_CsTrNo		VARCHAR(15);

	_PoSID		VARCHAR(5);
	_Shift		VARCHAR(2);
	_UsrID		VARCHAR(10);
	_CashID1	VARCHAR(15);
	_CashID2	VARCHAR(15);
	_Amount		NUMERIC(19,4);
	_CsTrNote	VARCHAR(255);
	_CrcID1		VARCHAR; /*-- asal*/
	_CrcID2		VARCHAR; /*-- tujuan*/
	_ExcRate1	NUMERIC(19,4);
	_ExcRate2	NUMERIC(19,4);

	_PerID		VARCHAR;
	_CrtDate	TIMESTAMP;
BEGIN
	SELECT INTO _PoSID, _Shift, _UsrID PoSID, Shift, UsrID
	FROM PoSSes WHERE PoSSesNo=_PoSSesNo AND IsCollect=false;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Sesi Kasir:'||_PoSSesNo||' tidak ditemukan atau sudah disetorkan sebelumnya (spPoS_Collect)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO _CashID1 CashID FROM PoS WHERE PoSID=_PoSID;
	SELECT INTO _CashID2 * FROM fReg('POSCOLLECT');

	SELECT INTO _CrcID1 crcid FROM cash WHERE cashid=_CashID1;
	SELECT INTO _CrcID2 crcid FROM cash WHERE cashid=_CashID2;

	SELECT INTO _ExcRate1 excrate FROM crc WHERE crcid=_CrcID1;
	SELECT INTO _ExcRate2 excrate FROM crc WHERE crcid=_CrcID2;

	_PerID:=fGetPer(CURRENT_DATE);
	_CrtDate:=CAST(current_timestamp AS TIMESTAMP);

	SELECT INTO _Amount CashBal FROM Cash WHERE CashID=_CashID1;
	_CsTrNote='Setoran Kasir:'||_PoSID||' Shift:'||_Shift;

	IF (_Amount>0) THEN
		SELECT INTO _CsTrNo * FROM spTransNoGet('CSTR');
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error INSERT CsTr! (spPoS_CollectAuto)';
		INSERT INTO CsTr(CsTrDate, CsTrNo, PerID, CashID1, crcid1, excrate1, CashID2, crcid2, excrate2,CsTrAmt, CsTrNote, CrtBy, CrtDate, foreignamt)
		VALUES (CURRENT_DATE, _CsTrNo, _PerID, _CashID1, _CrcID1, _ExcRate1, _CashID2, _CrcID2, _ExcRate2, _Amount, _CsTrNote, _UsrID, _CrtDate, _Amount);
		_retval.ErrNo:=0;
		_retval.ErrMsg:='UPDATE SUKSES!!';

		SELECT INTO _spret * FROM spCsTr_New(_CsTrNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;
	
	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error UPDATE PoSSes:IsCollect (spPoS_Collect)';
	UPDATE PoSSes SET IsCollect=true WHERE PoSSesNo=_PoSSesNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPoS_Collect (
	_PoSSesNo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	IF NOT EXISTS(SELECT 1 FROM PoSSes WHERE PoSSesNo=_PoSSesNo AND IsCollect=false)
	THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Sesi Kasir:'||_PoSSesNo||' tidak ditemukan atau sudah disetorkan sebelumnya (spPoS_Collect)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error UPDATE PoSSes:IsCollect (spPoS_Collect)';
	UPDATE PoSSes SET IsCollect=true WHERE PoSSesNo=_PoSSesNo;
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPoS_Start (
	_PoSID		VARCHAR,
	_PoSShift	VARCHAR,
	_UsrID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_PoSSesNo	VARCHAR;
	_PoSDate	DATE;
	__UsrID		VARCHAR;
	_CashBal	NUMERIC(19,4);
	_EmpID		VARCHAR;

	_Status		VARCHAR(8);
BEGIN
	/* Cek apakah automodal atau tidak */
	SELECT INTO _Status RegVal
	FROM Reg
	WHERE RegID='POS_AUTOMODAL';
	IF _Status='1' THEN
		SELECT INTO _spret * FROM spPoS_AutoModal(_PoSID, _UsrID);
		IF _spret.ErrNo <> 0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	/* cek apakah kasir bersangkutan masih aktif */
	SELECT INTO __UsrID UsrID 
	FROM PoSSes
	WHERE PoSID=_PoSID AND StartTime IS NOT NULL AND EndTime IS NULL;
	IF FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Kasir '||_PoSID||' masih Aktif dengan user:'||COALESCE(__UsrID,'<kosong>')||' (spPoS_Start)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/* cek apakah kasir bersangkutan masih belum setor pada penjualan sebelumnya*/
	SELECT INTO _PoSSesNo PoSSesNo 
	FROM PoSSes 
	WHERE PoSID=_PoSID AND IsCollect=FALSE;
	IF FOUND THEN
		_retval.ErrNo:=50002;
		_retval.Errmsg:='Penjualan Kasir '||_PoSID||' sebelumnya masih belum disetorkan ! (spPoS_Start)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	/* cek apakah shift untuk kasir bersangkutan sudah ada */
	SELECT INTO _PoSSesNo PoSSesNo 
	FROM PoSSes 
	WHERE PoSID=_PoSID AND PoSSesDate=CURRENT_DATE AND Shift=_PoSShift;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Shift #'||_PoSShift||' untuk POS#'||_PoSID||' sudah ada ! (spPoS_Start)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/* buat PoSSes baru */	
	SELECT INTO _CashBal c.CashBal 
	FROM Cash c JOIN PoS p ON p.CashID=c.CashID WHERE p.PoSID=_PoSID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50100;
		_retval.ErrMsg:='Kas tidak ditemukan ! (spPoS_Start)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF _CashBal=0 THEN
		_retval.ErrNo:=50101;
		_retval.ErrMsg:='Kas awal kasir '||_PoSID||' masih kosong (spPoS_Start)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _PoSSesNo * FROM spTransNoGet('POSSES');
	IF (_PoSSeSNo IS NULL)THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='NCount gagal (spPoS_Start)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _EmpID EmpID
	FROM Usr WHERE UsrID = _UsrID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Emp dengan userid '||_UsrID||' tidak ditemukan (spPoS_Start)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	INSERT INTO PoSSes (PoSSesNo, PoSID, PoSSesDate, Shift, EmpID, UsrID, StartTime, StartBal, IsCollect, EndBal)
	VALUES (_PoSSesNo, _PoSID, CURRENT_DATE, _PoSShift, _EmpID, _UsrID, LOCALTIMESTAMP, _CashBal, CAST(0 AS BOOLEAN), _CashBal);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPoS_AutoModal (
	_PosID		VARCHAR,
	_UsrID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_Amount		NUMERIC(19,4);
	_CsTrNo		VARCHAR(15);
	_CsTrNote	VARCHAR;
	_CashBal	NUMERIC(19,4);

	_CashID1	VARCHAR; /*-- asal*/
	_CashID2	VARCHAR; /*-- tujuan*/

	_CrcID1		VARCHAR; /*-- asal*/
	_CrcID2		VARCHAR; /*-- tujuan*/
	_ExcRate1	NUMERIC(19,4);
	_ExcRate2	NUMERIC(19,4);

	_PerID		VARCHAR;
	_CrtDate	TIMESTAMP;

BEGIN
	_CsTrNo:=fTransNoGet('CSTR');
	_Amount:=fReg('POS_MODAL');
	_CashID1:=fReg('POSCOLLECT');
	_CsTrNote:='MODAL UNTUK KASIR'||_PosID;

	SELECT INTO _CashID2 CashID
	FROM Pos
	WHERE PosID=_PosID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50101;
		_retval.ErrMsg:='Cash untuk POS '||||' tidak ditemukan (spPoS_AutoModal)';
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;

	SELECT INTO _CrcID1 crcid FROM cash WHERE cashid=_CashID1;
	SELECT INTO _CrcID2 crcid FROM cash WHERE cashid=_CashID2;

	SELECT INTO _ExcRate1 excrate FROM crc WHERE crcid=_CrcID1;
	SELECT INTO _ExcRate2 excrate FROM crc WHERE crcid=_CrcID2;

	_PerID:=fGetPer(CURRENT_DATE);
	_CrtDate:=CAST(current_timestamp AS TIMESTAMP);

	INSERT INTO CsTr (CsTrNo, PerID, CashID1, crcid1, excrate1, CashID2, crcid2, excrate2, CsTrAmt,CsTrDate, CrtBy, CrtDate, CsTrNote, foreignamt)
	VALUES (_CsTrNo, _PerID, _CashID1, _CrcID1, _ExcRate1, _CashID2, _CrcID2, _ExcRate2, _Amount, CURRENT_DATE, _UsrID, _CrtDate, _CsTrNote, _Amount);
	
	SELECT INTO _spret * FROM spCsTr_New(_CsTrNo);
	IF _spret.ErrNo <> 0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPoS_End (
	_PoSID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	
	_PoSSesNo	VARCHAR(8);
	_PoSDate	DATE;
	_PoSShift	SMALLINT;
	_UsrID_		VARCHAR(10);
	_StartTime	DATE;
	_EndBal		NUMERIC(19,4);
	_TotIn		NUMERIC(19,4);
	_TotOut		NUMERIC(19,4);

	_CashBal	NUMERIC(19,4);
	_PoSCashID	VARCHAR(15);

	_Status		VARCHAR;
BEGIN
	SELECT INTO _PoSSesNo, _UsrID_, _StartTime, _TotOut PoSSesNo, UsrID, StartTime, TotOut
	FROM PoSSes WHERE (PoSID=_PoSID) AND (StartTime IS NOT NULL) AND (EndTime IS NULL);
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Kasir '||_PoSID||' tidak aktiv! (spPoS_End)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	SELECT INTO _PoSCashID CashID FROM PoS WHERE PoSID=_PoSID;
	
	UPDATE PoSSes SET
		EndTime=LOCALTIMESTAMP
	WHERE PoSSesNo=_PoSSesNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Update PoSSes Gagal (spPoS_End)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;	
	
	SELECT INTO _Status *
	FROM fReg('POS_AUTOSETOR');
	IF _Status='1' THEN
		SELECT INTO _spret * FROM spPoS_CollectAuto(_PoSSesNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPoS_Status (
	__PoSID		VARCHAR,
	__PoSSesNo	VARCHAR,
	__Shift		VARCHAR,
	__CashID	VARCHAR,
	__Active	BOOLEAN
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_PoSID		VARCHAR(5);
	_PoSSesNo	VARCHAR(8);
	_Shift 		VARCHAR(2);
	_CashID		VARCHAR(15);
	_Active		BOOLEAN;
BEGIN
	SELECT INTO _CashID CashID FROM PoS WHERE PoSID=__PoSID;

	_Active=true;
	SELECT INTO  _PoSSesNo, _Shift PoSSesNo, Shift
	FROM PoSSes 
	WHERE (PoSID=__PoSID) AND (StartTime IS NOT NULL) AND (EndTime IS NULL);
	IF NOT FOUND THEN
		_Active=false;
		_PoSSesNo=NULL;
	END IF;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPoSSes_Update (
	_PoSSesNo	VARCHAR,
	_RcvAmt		NUMERIC,
	_ChgAmt		NUMERIC
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_TotIn		NUMERIC(19,4);
	_TotOut		NUMERIC(19,4);
	_StartBal	NUMERIC(19,4);
BEGIN
	SELECT INTO _TotIn, _TotOut, _StartBal TotIn, TotOut, StartBal
	FROM PoSSes
	WHERE PoSSesNo=_PoSSesNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50010;
		_retval.ErrMsg:='POS Session '||_PoSSesNo||' tidak ditemukan (spPoSSes_Update)';
	END IF;

	UPDATE PoSSes
	SET TotIn=_TotIn+_RcvAmt, TotOut=_TotOut+_ChgAmt 
	WHERE PoSSesNo=_PoSSesNo;
	
	_TotIn:=_TotIn+_RcvAmt;
	_TotOut:=_TotOut+_ChgAmt;
	
	UPDATE PoSSes
	SET EndBal=(_StartBal+_TotIn-_TotOut)
	WHERE PoSSesNo=_PoSSesNo;	

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spBPID_Update (
	_oldBPID	VARCHAR,
	_newBPID	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_tempBPID	VARCHAR(15);
	_BPID1		VARCHAR(15);
	_BPID2		VARCHAR(15);
	_BPID3		VARCHAR(15);
	_stage		SMALLINT;
	_state		BOOLEAN;
BEGIN
	IF (_oldBPID=_newBPID) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='KODE BPact lama tidak boleh sama dengan kode baru!';
		RAISE EXCEPTION '%', _retval.ErrMSg;
		EXIT;
	END IF;

	IF (_newBPID IS NULL OR _newBPID='') THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='KODE BPact baru tidak boleh null!';
		RAISE EXCEPTION '%', _retval.ErrMSg;
		EXIT;
	END IF;

	SELECT INTO _BPID3 BPID FROM BP WHERE BPID=_newBPID;
	IF FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='KODE BPact yang baru sudah ada/dipakai, Pilih yang lain !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_tempBPID:='TEMP_BPID';
	_BPID1:=_oldBPID;
	_BPID2:=_tempBPID;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error Insert temporary-BPact (spBPID_Update)';
	INSERT into BP(BPID, BPName) VALUES (_tempBPID, 'BPID sementara');
	
	_stage:=1;
	_state:=true;

	WHILE ((_stage=1) OR (_state=true)) LOOP
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error Update Item:VendorID (spBPID_Update)';
		UPDATE Addr SET BPID=_BPID2 WHERE BPID=_BPID1;

		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error Update Item:VendorID (spBPID_Update)';
		UPDATE Item SET VendorID=_BPID2 WHERE VendorID=_BPID1;
	
		_retval.ErrNo:=50005;
		_retval.ErrMSg:='Error Update PO:VendorID (spBPID_Update)';
		UPDATE PO SET VendorID=_BPID2 WHERE VendorID=_BPID1;

		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error Update PRcv:VendorID (spBPID_Update)';
		UPDATE PRcv SET VendorID=_BPID2 WHERE VendorID=_BPID1;
	
		_retval.ErrNo:=50007;
		_retval.ErrMsg:='Error Update Purc:VendorID (spBPID_Update)';
		UPDATE Purc SET VendorID=_BPID2 WHERE VendorID=_BPID1;
	
		_retval.ErrNo:=50008;
		_retval.ErrMsg:='Error Update CN:VendorID (spBPID_Update)';
		UPDATE PRET SET VendorID=_BPID2 WHERE VendorID=_BPID1;
	
		_retval.ErrNo:=50009;
		_retval.ErrMsg:='Error Update BPA:VendorID (spBPID_Update)';
		UPDATE BPA SET BPID=_BPID2 WHERE BPID=_BPID1;
		
		_retval.ErrNo:=50010;
		_retval.ErrMsg:='Error Update Pay:VendorID (spBPID_Update)';
		UPDATE Pay SET BPID=_BPID2 WHERE BPID=_BPID1;
	
		_retval.ErrNo:=50011;
		_retval.ErrMsg:='Error Update SO:CustID (spBPID_Update)';
		UPDATE SO SET CustID=_BPID2 WHERE CustID=_BPID1;
	
		_retval.ErrNo:=50012;
		_retval.ErrMsg:='Error Update SO:RefrID (spBPID_Update)';
		UPDATE SO SET RefrID=_BPID2 WHERE RefrID=_BPID1;

		_retval.ErrNo:=50013;
		_retval.ErrMsg:='Error Update DO:CustID (spBPID_Update)';
		UPDATE DEO SET CustID=_BPID2 WHERE CustID=_BPID1;

		_retval.ErrNo:=50014;
		_retval.ErrMsg:='Error Update Sale:CustID (spBPID_Update)';
		UPDATE Sale SET CustID=_BPID2 WHERE CustID=_BPID1;

		_retval.ErrNo:=50015;
		_retval.ErrMsg:='Error Update Sale:RefrID (spBPID_Update)';
		UPDATE Sale SET RefrID=_BPID2 WHERE RefrID=_BPID1;

		_retval.ErrNo:=50016;
		_retval.ErrMsg:='Error Update DN:CustID (spBPID_Update)';
		UPDATE SRET SET CustID=_BPID2 WHERE CustID=_BPID1;
	
		_retval.ErrNo:=50017;
		_retval.ErrMsg:='Error Update Rcv:CustID (spBPID_Update)';
		UPDATE Rcv SET BPID=_BPID2 WHERE BPID=_BPID1;	

		_retval.ErrNo:=50018;
		_retval.ErrMsg:='Error Update BPA:CustID (spBPID_Update)';
		UPDATE BPA SET BPID=_BPID2 WHERE BPID=_BPID1;

		_retval.ErrNo:=50019;
		_retval.ErrMsg:='Error Update StockA:BPID (spBPID_Update)';
		UPDATE StockA SET BPID=_BPID2 WHERE BPID=_BPID1;
	
		_retval.ErrNo:=50020;
		_retval.ErrMsg:='Error Update CashA:BPID (spBPID_Update)';
		UPDATE CashA SET BPID=_BPID2 WHERE BPID=_BPID1;
	
		_retval.ErrNo:=50021;
		_retval.ErrMsg:='Error Update BPBal:BPID (spBPID_Update)';
		UPDATE BPCBal SET BPID=_BPID2 WHERE BPID=_BPID1;

		_retval.ErrNo:=50021;
		_retval.ErrMsg:='Error Update BPBal:BPID (spBPID_Update)';
		UPDATE BPDBal SET BPID=_BPID2 WHERE BPID=_BPID1;

		_retval.ErrNo:=50021;
		_retval.ErrMsg:='Error Update BPBal:BPID (spBPID_Update)';
		UPDATE BPLog SET BPID=_BPID2 WHERE BPID=_BPID1;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error Update BPSched:BPID (spBPID_Update)';
		UPDATE BPSched SET BPID=_BPID2 WHERE BPID=_BPID1;
		
		_retval.ErrNo:=50023;
		_retval.ErrMsg:='Error Update BPBPTYPE:BPID (spBPID_Update)';
		UPDATE BPBPType SET BPID=_BPID2 WHERE BPID=_BPID1;

		_retval.ErrNo:=50023;
		_retval.ErrMsg:='Error Update Addr:BPID (spBPID_Update)';
		UPDATE Addr SET BPID=_BPID2 WHERE BPID=_BPID1;

		_retval.ErrNo:=50023;
		_retval.ErrMsg:='Error Update Cont:BPID (spBPID_Update)';
		UPDATE Cont SET BPID=_BPID2 WHERE BPID=_BPID1;

		_retval.ErrNo:=50024;
		_retval.ErrMsg:='Error Update BPAcc:BPID (spBPID_Update)';
		UPDATE BPAcc SET BPID=_BPID2 WHERE BPID=_BPID1;

		IF (_stage=2) THEN 
			EXIT;
		END IF;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error Update old-BPID to new BPID. ! (spBPID_Update)';
		UPDATE BP SET BPID=_newBPID WHERE BPID=_oldBPID;

		_retval.ErrNo:=0;
		_retval.ErrMsg:='ALL UPDATE SUKSES!';
		
		_BPID1:=_tempBPID;
		_BPID2:=_newBPID;
		_stage:=2;
	END LOOP;

	_retval.ErrNo:=50023;
	_retval.ErrMsg:='Error Delete temporary-BPact ! (spBPID_Update)';
	DELETE FROM BP WHERE BPID=_tempBPID;
	_retval:=(0,'OK');

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spBP_Remove(_BPID VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_RefType	VARCHAR(5);
	_RefNo		VARCHAR(15);
	
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _RefType, _RefNo reftype, refno 
	FROM stocka WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Sudah ada Transaksi Stock (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo sadjno FROM sadj WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Sudah ada Transaksi Stock (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefType, _RefNo reftype, refno
	FROM casha WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Sudah ada Transaksi Cash (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefType, _RefNo bpatype, refno 
	FROM bpa WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Sudah ada Aktivitas BP (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo pono FROM po WHERE vendorid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Order Pembelian ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo prcvno FROM prcv WHERE vendorid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Penerimaan Pembelian ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo purcno FROM purc WHERE vendorid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Invoice Pembelian ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo pretno FROM pret WHERE vendorid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Retur Pembelian ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo itemid FROM item WHERE vendorid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan sebagai Supplier Item ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo sretno FROM sret WHERE custid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Retur Penjualan ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;
	
	SELECT INTO _RefNo mfgno FROM mfg WHERE custid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Produksi ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo deono FROM deo WHERE custid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Order Pengiriman ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo consno FROM cons WHERE custid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Konsinyasi ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo saleno FROM sale WHERE custid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Invoice Penjualan ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo sono FROM so WHERE custid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Order Penjualan ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo delino FROM deli WHERE custid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Pengiriman ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo rcvno FROM rcv WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Penerimaan Pembayaran Piutang ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo cadjno FROM cadj WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Penyesuaian Kas ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;
	
	SELECT INTO _RefNo bplogno FROM bplog WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Riwayat Kontak ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;
	
	SELECT INTO _RefNo payno FROM pay WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Pembayaran Hutang ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo fcnno FROM fcn WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Nota Potong ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo prjid FROM prj WHERE bpid=_BPID;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Proyek ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo bnsno FROM bns WHERE (custid=_BPID OR vendorid=_BPID);
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Bonus ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo pprcno FROM pprc WHERE (custid=_BPID OR vendorid=_BPID);
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi Promo ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	SELECT INTO _RefNo cbgid FROM cbg WHERE (tobpid=_BPID OR frombpid=_BPID);
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='BP#'||_BPID||' masih direferensikan pada transaksi C.B.G. ! (spBP_Remove)';
		RAISE EXCEPTION '%', _retval.ErrMsg;				
	END IF;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE BPCBal (spBP_Remove)';
	DELETE FROM BPCBal WHERE BPID=_BPID;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE BPCBal (spBP_Remove)';
	DELETE FROM BPDBal WHERE BPID=_BPID;

	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error DELETE BPbptype (spBP_Remove)';
	DELETE FROM BPBPType WHERE BPID=_BPID;


	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error DELETE Addr (spBP_Remove)';
	DELETE FROM addr WHERE BPID=_BPID;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error DELETE cont (spBP_Remove)';
	DELETE FROM cont WHERE BPID=_BPID;

	_retval.ErrNo:=50009;
	_retval.ErrMsg:='Error DELETE BPAcc (spBP_Remove)';
	DELETE FROM BPAcc WHERE BPID=_BPID;

	_retval.ErrNo:=50010;
	_retval.ErrMsg:='Error DELETE BPLog (spBP_Remove)';
	DELETE FROM BPLog WHERE BPID=_BPID;	

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE BPact (spBP_Remove)';
	DELETE FROM BP WHERE BPID=_BPID;

	_retval:=(0,'DELETE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION spBP_RemoveForced(_BPID VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_RefType	VARCHAR(5);
	_RefNo		VARCHAR(15);
	
BEGIN
	_retval:=(0, 'OK');

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE BPCBal (spBP_RemoveForced)';
	DELETE FROM BPCBal WHERE BPID=_BPID;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE BPCBal (spBP_RemoveForced)';
	DELETE FROM BPDBal WHERE BPID=_BPID;

	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error DELETE BPbptype (spBP_RemoveForced)';
	DELETE FROM BPBPType WHERE BPID=_BPID;

	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error DELETE Addr (spBP_RemoveForced)';
	DELETE FROM addr WHERE BPID=_BPID;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error DELETE cont (spBP_RemoveForced)';
	DELETE FROM cont WHERE BPID=_BPID;

	_retval.ErrNo:=50009;
	_retval.ErrMsg:='Error DELETE BPAcc (spBP_RemoveForced)';
	DELETE FROM BPAcc WHERE BPID=_BPID;

	_retval.ErrNo:=50010;
	_retval.ErrMsg:='Error DELETE BPLog (spBP_RemoveForced)';
	DELETE FROM BPLog WHERE BPID=_BPID;	

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE BPact (spBP_RemoveForced)';
	DELETE FROM BP WHERE BPID=_BPID;

	_retval:=(0,'DELETE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION spBP_Reset()
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
	_BPID		VARCHAR;
	cBP		REFCURSOR;
BEGIN
	OPEN cBP FOR
		SELECT bpid 
		FROM BP
		WHERE bpid<>'CASH';
	LOOP 
		FETCH cBP INTO _BPID;	
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spBP_RemoveForced(_BPID);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMSg;
		END IF;

	END LOOP; 
	CLOSE cBP;

	_retval:=(0,'ALL Deleted Success');
	RETURN NEXT _retval;

END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spBP_Reset(
	_bujukan	VARCHAR(20)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_spret	SPStatus%ROWTYPE;
	_retval	SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spBP_Reset();
		
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=_spret;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPrice_Update (
	_ItemID		VARCHAR,
	_PrcLvlID	SMALLINT,
	_Price1		NUMERIC,
	_Price2		NUMERIC,
	_Price3		NUMERIC,
	_PriceDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	SELECT * FROM Item WHERE ItemID=_ItemID;
	IF FOUND THEN
		SELECT * FROM Prc WHERE ItemID=_ItemID AND PrcLvlID=_PrcLvlID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50001;
			_retval.ErrMsg:='ERROR INSERT Prc (spPrice_Update)';
			INSERT INTO Prc(ItemID, PrcLvlID) VALUES (_ItemID, _PrcLvlID);
			_retval.ErrNo:=0;
			_retval.ErrMsg:='INSERT SUKSES!!';
		END IF;
		
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Update Prc! (spPrice_Update)';
		UPDATE Prc SET 
			Price1=_Price1,
			Price2=_Price2,
			Price3=_Price3,
			PriceDate=_PriceDate
		WHERE ItemID=_ItemID AND PrcLvlID=_PrcLvlID;
			_retval:=(0, 'UPDATE SUKSES!');
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spLCInfo (
	_ItemID		VARCHAR,
	__LCPrice	NUMERIC,
	__LCUnit	VARCHAR,
	__LCDiscExp	VARCHAR,
	__LCTaxID	VARCHAR,
	__LCDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	__LCPrice 	NUMERIC;
	__LCUnit 	VARCHAR;
	__LCDiscExp	VARCHAR;
	__LCTaxID	VARCHAR;
	__LCDate 	DATE;
BEGIN
	__LCPrice:=0;
	__LCUnit:=NULL;
	__LCDiscExp:=NULL;
	__LCTaxID:=NULL;
	__LCDate:=NULL;
	
	SELECT INTO __LCPrice, __LCUnit, __LCDiscExp, __LCTaxID, __LCDate
		LCPrice, LCUnit, LCDiscExp, LCTaxID, LCDate
	FROM Item WHERE ItemID=_ItemID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Error membaca info pembelian terakhir untuk Item:'||_ItemID||' (spLCInfo)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spModel_Update (
	_oldModel	VARCHAR,
	_newModel	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	
	_tempModel	VARCHAR(5);
	_Model1		VARCHAR(5);
	_Model2		VARCHAR(5);
	_tmp		VARCHAR(5);
	_stage		SMALLINT;
	_state		BOOLEAN;

BEGIN
	IF (_oldModel=_newModel) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='KODE Model lama tidak boleh sama dengan kode baru! (spModel_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO _tmp Model FROM Model WHERE Model=_newModel;
	IF _tmp<>null THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='KODE Model yang baru sudah ada/dipakai, Pilih yang lain ! (spModel_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_tempModel:='TEMP';
	_Model1:=_oldModel;
	_Model2:=_tempModel;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error Insert temporary-Contact (spModel_Update)';
	INSERT INTO Model(Model, ModelDesc) VALUES (_tempModel, 'Model sementara');
	_retval.ErrNo:=0;
	_retval.ErrMsg:='INSERT SUKSES!';

	_stage:=1;
	_state:=true;

	WHILE ((_stage=1) OR (_state=true)) LOOP
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error Update Item:Model (spModel_Update)';
		UPDATE Item SET Model=_Model2 WHERE Model=_Model1;

		_retval.ErrNo:=50005;
		_retval.ErrMsg:='Error Update OEM:Model (spModel_Update)';
		UPDATE OEM SET Model=_Model2 WHERE Model=_Model1;

		IF (_stage=2) THEN
			EXIT;
		END IF;

		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error Update old-Model to new-Model. ! (spModel_Update)';
		UPDATE Model SET Model=_newModel WHERE Model=_oldModel;

		_retval.ErrNo:=0;
		_retval.ErrMsg:='ALL UPDATE SUKSES!';

		_Model1:=_tempModel;
		_Model2:=_newModel;
		_stage:=2;
	END LOOP;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error Delete temporary-Model ! (spModel_Update)';
	DELETE FROM Model WHERE Model=_tempModel;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spMfg_New (
	__MfgNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_ProcsID	VARCHAR;
	_RouID		VARCHAR;
	_BoMID		VARCHAR;
	_found		VARCHAR;
	_MfgType	VARCHAR;

	_JobCardNo	VARCHAR;
	_Used		BOOLEAN;
	
	cJobCardCheck	REFCURSOR;
	_JobCardCheck	VARCHAR;
	_JCMfgNo	VARCHAR;
BEGIN

	SELECT INTO 
		_RouID, _ProcsID, _BoMID, _JobCardNo, _Used, _JCMfgNo, _MfgType
		m.rouid, m.procsid, m.bomid, jc.jobcardno, COALESCE(jc.used,FALSE), jc.mfgno, m.mfgtype  
	FROM mfg m LEFT JOIN jobcard jc ON m.jobcardno=jc.jobcardno 
	WHERE m.mfgno=__MfgNo;

	IF (_BoMID IS NULL OR _BoMID='') THEN
		RAISE EXCEPTION 'B.o.M. belum di-isi! (spMfg_New)';
	END IF;
	
	-- check apakah mfg nya simple atau advance
	IF _MfgType<>'S' THEN
			IF (_RouID IS NULL OR _RouID='') THEN
				RAISE EXCEPTION 'Routing (alur produksi) belum di-isi! (spMfg_New)';
			END IF;

			IF (_ProcsID IS NULL OR _ProcsID='') THEN
				RAISE EXCEPTION 'Proses belum di-isi! (spMfg_New)';
			END IF;

			IF (_JobCardNo IS NOT NULL AND _Used=TRUE AND __MfgNo!=_JCMfgNo) THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Job Card#'||_JobCardNo||' sudah dipakai oleh produksi lain ! (spMfg_New)';		
				RAISE EXCEPTION '%', _retval.ErrMsg;
			ELSIF (_JobCardNo IS NOT NULL AND _Used=FALSE) THEN
				UPDATE jobcard SET used=TRUE, mfgno=__MfgNo WHERE jobcardno=_JobCardNo;	
			END IF;

			/** DOUBLE CHECKING karena edit MFG tidak panggil void */
			OPEN cJobCardCheck FOR
				SELECT jobcardno FROM jobcard WHERE mfgno=__MfgNo;
			LOOP
				FETCH cJobCardCheck INTO _JobCardCheck;
				EXIT WHEN NOT FOUND;

				IF (COALESCE(_JobCardCheck, '')!=COALESCE(_JobCardNo, '')) THEN
					UPDATE jobcard SET used=FALSE, mfgno=NULL WHERE jobcardno=_JobCardCheck;
				END IF;
			END LOOP;
			CLOSE cJobCardCheck;

			-- untuk awal set status produksi Draft, nanti pas mfgM masuk pertama akan diupdate jadi <Progress>
			/** sudah di set di trans
				UPDATE mfg SET MfgStatus='D' WHERE MfgNo=__MfgNo;
				IF NOT FOUND THEN
					RAISE EXCEPTION 'Gagal update status Produksi % (spMfg_New)', __MfgNo;
				END IF;
			**/

			-- cek apakah process kompatibel dgn routing (sudah gak perlu, sdh dicek di form 

			-- cek bomid kompatibel dgn process yg dipilih
			SELECT INTO 
				_found 
				bomid 
			FROM procsbom WHERE procsid=_ProcsID AND BoMID=_BoMID;

			IF NOT FOUND THEN
				RAISE EXCEPTION 'BoM #% tidak kompatibel dengan Proses #% (spMfg_New)', _BoMID, _ProcsID;
			END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spMfg_Void (
	__MfgNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	
	_Tmp		VARCHAR(15);
	_TmpM		VARCHAR(15);
	_TmpP		VARCHAR(15);
	_mfgmno		VARCHAR(15);
	_mfgpno		VARCHAR(15);
	_mfgno		VARCHAR(15);
	_WONo		VARCHAR(15);
	_count		INT;

	cmfgm		REFCURSOR;
	cmfgp		REFCURSOR;
	cWO		REFCURSOR;
	cmfgvoid	REFCURSOR;
	
	_JobCardNo	VARCHAR;
	_MfgType	VARCHAR;
BEGIN
	SELECT INTO _Tmp, _JobCardNo, _MfgType mfgno, jobcardno, MfgType FROM mfg WHERE mfgno=__MfgNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Produksi #'||__MfgNo||' tidak ditemukan! (spMfg_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	UPDATE Mfg SET MfgStatus='P' WHERE MfgNo=__MfgNo;	-- di set 'P' dulu agar tidak di block oleh spMfgP_Void

	IF _MfgType<>'S' THEN

		OPEN cWO FOR
		SELECT WONO FROM Mfg WHERE MfgNo=__MfgNo;
		LOOP
			FETCH cWO INTO _WONo;
			EXIT WHEN NOT FOUND;

			SELECT INTO _spret * FROM spWO_Open(_WONo,__MfgNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=spWO_Close.ErrNo;
				_retval.ErrMsg:=spWO_Close.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF; 

		END LOOP;
		CLOSE cWO;
		
	END IF;

	OPEN cmfgm FOR
		SELECT MfgMNo FROM MFGM WHERE MfgNo=__MfgNo;
	LOOP
		FETCH cmfgm INTO _mfgmno;
		EXIT WHEN NOT FOUND;
		
		SELECT INTO _spret * FROM spMfgM_Void(_mfgmno);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cmfgm;	

	/**Di Update F dulu soalnya biar gak di blok di ReOpen**/
	UPDATE Mfg SET mfgstatus='F' WHERE MfgNo=__MfgNo;

/**	OPEN cmfgvoid FOR
		SELECT MfgNo FROM MfgP WHERE MfgNo=__MfgNo;
	LOOP
		FETCH cmfgvoid INTO _mfgno;
		EXIT WHEN NOT FOUND;
	**/
		SELECT INTO _spret * FROM spMfg_ReOpen(_Tmp);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	/**END LOOP;
	CLOSE cmfgvoid;		
	**/
	OPEN cmfgp FOR
		SELECT MfgPNo FROM MfgP WHERE MfgNo=__MfgNo;
	LOOP
		FETCH cmfgp INTO _mfgpno;
		EXIT WHEN NOT FOUND;
	
		SELECT INTO _spret * FROM spMfgP_Void(_mfgpno);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cmfgp;

	IF (_JobCardNo IS NOT NULL) THEN
		UPDATE jobcard SET used=FALSE, mfgno=NULL WHERE jobcardno=_JobCardNo;
	END IF;
	
	DELETE FROM Mfg WHERE MfgNo=__MfgNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spRcv_New(_RcvNo VARCHAR) 
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_CustID			VARCHAR(15);
	_RcvType		VARCHAR(5);
	_RcvDate		DATE;
	_TotSPaid		DECIMAL(19,4);
	_TotRcvAmt		DECIMAL(19,4);
	_OverRcv		DECIMAL(19,4);
	_IsContra		BOOL;
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_RefType		VARCHAR;
	_SaleNo			VARCHAR(15);
	_SPaidAmt		DECIMAL(19,4);
	_SPaidAmtSRet		DECIMAL(19,4);
	_IsPosted		BOOLEAN;

	_RcvDMtd		VARCHAR(4);
	_CashID			VARCHAR(15);
	_RcvDAmt		DECIMAL(19,4);
	_GDate			DATE;
	_GDueDate		DATE;

	_RcvNote		TEXT;
	_RcvDNote		TEXT;

	_CashAmt		DECIMAL(19,4);
	_CashANote		VARCHAR;
	_ExtNote		VARCHAR;
	_LastRefType		VARCHAR(5);


	_DebtAmt		DECIMAL(19,4);
	_totBpAmt		DECIMAL(19,4);
	_DebtNote		VARCHAR(50);
	_TempStr		TEXT;

	_RefNo			VARCHAR;
	_RcvD_RefNo		VARCHAR;
	_CrcID			VARCHAR(5);
	_Inout			VARCHAR(3);
	_CbgID			VARCHAR;
	_SrcCash		VARCHAR;
	_CbgAmt			NUMERIC;
	_CbgDate		DATE;
	cSPaid			REFCURSOR;
	cRcvD			REFCURSOR;

	_retval			SPStatus%ROWTYPE;
	spretSale_Rcv		SPStatus%ROWTYPE;
	spretOverRcv_Contra	SPStatus%ROWTYPE;
	spretDN_Contra		SPStatus%ROWTYPE;
	spretCashA_New		SPStatus%ROWTYPE;
	spretDebt_New		SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_AccCash		VARCHAR(10);
	_AccNo			VARCHAR(10);
	_Count			INT;
	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;
	_isAutoGen		BOOLEAN;

	_CustID_Check		VARCHAR(15);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_ACTIVE			BOOLEAN;
	_BPID			VARCHAR;
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);

	_AccFcn			VARCHAR(10);
	_AccOver		VARCHAR(10);
	_NomerCbg		VARCHAR;
	_IsRetur		BOOLEAN;
	_ClrDate		DATE;
	_LockDate		DATE;
	_SRetAmt		NUMERIC(19,4);
	_AccName		VARCHAR;
	_TotRcvDisc		NUMERIC(19,4);
	_AccDisc		VARCHAR;
	_ExcRateSale		NUMERIC(19,4);
	_ExcRate		NUMERIC(19,4);
	_SelisihRate		NUMERIC(19,4);
	_SelisihNote		VARCHAR;
	_JnlNo1			VARCHAR;
	_JnlNote1		VARCHAR;
	_FCNAmt			NUMERIC(19,4);
	_OverRcvBC		NUMERIC(19,4);
	_TotRcvDiscBC		NUMERIC(19,4);
	_ExcRateRcvD		NUMERIC(19,4);	
	_ExcRateSPaid		NUMERIC(19,4);
	_SPaidAmtSO		NUMERIC(19,4);
	_CrcIDSPaid		VARCHAR;
	_CrcIDSale		VARCHAR;
	_CrcIDSret		VARCHAR;
	_saldoAkhir		NUMERIC(19,4);
	_saldo			NUMERIC(19,4);
	_paidAmt		NUMERIC(19,4);
	_CrcIDDefa		VARCHAR;
	_payStatus		VARCHAR;
	__xt			BOOLEAN;
	_CrcIDFcn		VARCHAR;
	_FisRate		NUMERIC(19,4);
	_SPaidAmtAsli		NUMERIC(19,4);
	_SPaidAmtSretAsli	NUMERIC(19,4);
	_SPaidAmtSOAsli		NUMERIC(19,4);
	_CustIDSO		VARCHAR;
	_FisRateSPaid		NUMERIC(19,4);
	_CrcIDRcvD		VARCHAR;
	_FisRateRcvD		NUMERIC(19,4);
	_countSale		INT;

	_isAutoReconcileIn	BOOLEAN;
	_isDraft		BOOLEAN;

	_MRefNo			VARCHAR;
	_MRefType		VARCHAR;
	_AccDP			VARCHAR;

	_SPaidAmtFCN		NUMERIC(19,4);
	_SPaidAmtFCNAsli	NUMERIC(19,4);
	_CashInOut		CHAR(1);
	_CashCBG		VARCHAR;
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_IsRetur:=false;
	_CashInOut:='I';
	_DebtAmt:=0;

	SELECT INTO 
		_PerID, _BranchID, _DeptID, _PrjID, _CustID, _RcvType, _RcvDate, _RcvNote, _TotSPaid, _TotRcvAmt, _OverRcv, _IsContra, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _MRefType, _MRefNo, _CrcID, _perid, _isAutoGen, _TotRcvDisc, _OverRcvBC, _TotRcvDiscBC, __xt, _ExcRate, _FisRate, _isDraft 
		PerID, BranchID, DeptID, PrjID, BPID, RcvType, RcvDate, RcvNote, TotSPaid, TotRcvAmt, OverRcv, IsContra, CrtBy, CrtDate, UpdBy, UpdDate, RefType, RefNo, CrcID, perid, isautogen, TotRcvDisc, OverRcvBC, TotRcvDiscBC, _xt, ExcRate, FisRate, isDraft
	FROM Rcv WHERE RcvNo=_RcvNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Penerimaan-pembayaran #'||_RcvNo||' tidak ditemukan (spRcv_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='RCV';
	IF(_LockDate is not NULL AND _LockDate>_RcvDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penerimaan Pembayaran Melebihi Tanggal Batas Kunci Transaksi ! (spRcv_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_OverRcv<CAST(0 AS Decimal(19,4))) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Total cara bayar < (Total piutang yang dibayar-Diskon) (spRcv_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

/*klo pembayaran penjualan tunai harus autogen nya true!*/
	IF(_RcvType='SALE' AND _MRefNo IS NULL) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Tipe Penerimaan Cash Harus Lewat Penjualan Tunai!! (spRcv_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _countSale COUNT(*) FROM spaid WHERE rcvno=_RcvNo and reftype='SALE';

	_ExtNote:='';
	_LastRefType:='';
	_totBpAmt:=0;
	/*** Update Sales Paid (SPaid) ***/
	OPEN cSPaid FOR
		SELECT RefType,Refno, SPaidAmt, CrcID, excrate
		FROM SPaid 
		WHERE RcvNo=_RcvNo
		ORDER BY RefType, SPaidNo;
	LOOP
		FETCH cSPaid INTO _RefType, _RefNo, _SPaidAmt, _CrcIDSPaid, _ExcRateSPaid;
		EXIT WHEN NOT FOUND;

		IF _RefType='SALE' THEN
			IF (_LastRefType<>'SALE') THEN
				_ExtNote:=_ExtNote||'Jual ';
				_LastRefType:='SALE';
			END IF;

			SELECT INTO _CustID_Check, _CrcIDSale custid,crcid FROM sale WHERE saleno=_RefNo;
			IF _CustID_Check<>_CustID THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Customer yang akan dipotong tidak sesuai dengan customer yang bayar (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF _CrcIDSPaid<>_CrcIDSale THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Mata Uang '||_RefNo||' harus sama dengan mata uang asli nya!! (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_SPaidAmt<0) THEN
				SELECT INTO  spretSale_Rcv * FROM spSale_Rcv(_RefNo, _SPaidAmt*(-1), _RcvDate);
			ELSE
				SELECT INTO  spretSale_Rcv * FROM spSale_Rcv(_RefNo, _SPaidAmt, _RcvDate);
			END IF;

			_totBpAmt:=_totBpAmt+_SPaidAmt;
		ELSIF _RefType='SRET' THEN
			IF (_LastRefType<>'SRET') THEN
				_ExtNote:=_ExtNote||'R.Jual ';
				_LastRefType:='SRET';
			END IF;

			SELECT INTO _CustID_Check,_SRetAmt,_CrcIDSret custid,total,crcid FROM sret WHERE sretno=_RefNo;
			IF _CustID_Check<>_CustID THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Customer yang akan dipotong tidak sesuai dengan customer yang bayar (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF _CrcIDSPaid<>_CrcIDSret THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Mata Uang '||_RefNo||' harus sama dengan mata uang asli nya!! (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (ABS(_SPaidAmt)<>_SRetAmt) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Jumlah Penerimaan Retur Harus Sama Retur!! (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_countSale=0) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Belum Ada Piutang yang Dipilih? (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_totBpAmt:=_totBpAmt+_SPaidAmt;
			SELECT INTO spretSale_Rcv * FROM spSret_Contra(_RefNo);
			IF (spretSale_Rcv.ErrNo<>0) THEN
				_retval.ErrNo:=spretSale_Rcv.ErrNo;
				_retval.ErrMsg:=spretSale_Rcv.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			_IsRetur:=true;
		ELSIF _RefType='DP' THEN
			_AccDP:=fAccDefault('DEPOCUST');
			IF (_RcvType='DP') THEN
				SELECT INTO _spret * FROM spDP_New(_CustID, _RcvDate, _MRefNo, COALESCE(_MRefType, 'RCV'), _CrcID, _ExcRate, _FisRate, _SPaidAmt, _AccDP, _RcvNo, 'RCV');
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSE
				IF (_CrcIDSPaid!=_CrcID) THEN
					_retval.ErrNo:=50555;
					_retval.ErrMsg:='Mata Uang D.P. tidak sama dengan mata uang Pembayaran (spPay_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				SELECT INTO _spret * FROM spDPBal_Update(CAST(_RefNo AS BIGINT), _SPaidAmt, _SPaidAmt*_ExcRateSPaid);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			--_totBpAmt:=_totBpAmt+_SPaidAmt;
		ELSIF _RefType='FCN' THEN
			SELECT INTO _CustID_Check,_FCNAmt,_CrcIDFcn bpid,fcnamt,CrcID FROM FCN WHERE FcnNo=_RefNo;
			IF _CustID_Check<>_CustID THEN
				_retval.ErrNo:=55555;
				_retval.ErrMsg:='Mitra Bisnis FCN tidak sama dengan Customer yang akan diterima';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF _CrcIDSPaid<>_CrcIDFcn THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Mata Uang '||_RefNo||' harus sama dengan mata uang asli nya!!';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;


			IF(ABS(_SPaidAmt)<>_FcnAmt) THEN
				_retval.ErrNo:=55555;
				_retval.ErrMsg:='Jumlah Tidak Sama Dengan FCN !!';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_countSale=0) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Belum Ada Piutang yang Dipilih?';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		
			UPDATE FCN SET isContra=true WHERE FcnNo=_RefNo;
			IF NOT FOUND THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='FCN tidak ditemukan ! (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			SELECT INTO _AccFcn AccFcn FROM FCN WHERE FcnNo=_RefNo;
			
			_totBpAmt:=_totBpAmt+_SPaidAmt;
		ELSIF _RefType='ACC' THEN
			IF (_LastRefType<>'ACC') THEN
				_ExtNote:=_ExtNote||'Acc ';
				_LastRefType:='ACC';
			END IF;
			SELECT INTO _AccName accname FROM acc where accno=_RefNo;
			_RefNo:=_AccName;
		
			--_totBpAmt:=_totBpAmt+_SPaidAmt;
		END IF;

		IF _CrcIDSPaid<>_CrcID THEN
			_retval.ErrNo:=101;
			_retval.ErrMsg:='Mata Uang '||_RefNo||' harus sama dengan mata uang Master Transaksi!!';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF spretSale_Rcv.ErrNo<>0 THEN
				_retval.ErrNo:=spretSale_Rcv.ErrNo;
				_retval.ErrMsg:=spretSale_Rcv.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_ExtNote:=_ExtNote||_RefNo||' ';
	END LOOP;
	CLOSE cSPaid;

/****************************** Generate CashA from RcvD ************************************/

	OPEN cRcvD FOR
		SELECT RcvDMtd, CashID, RcvDAmt, RefNo, 
			RcvDNote,CbgID,CrcID, ExcRate, FisRate  FROM RcvD WHERE RcvNo=_RcvNo;

	LOOP
		FETCH cRcvD INTO _RcvDMtd, _CashID, _RcvDAmt, _RcvD_RefNo, _RcvDNote,_CbgID, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD;
		EXIT WHEN NOT FOUND;

		IF (_RcvDMtd='OVER') THEN
			SELECT INTO _CustID_Check bpid FROM rcv WHERE rcvno=_RcvD_RefNo;
			IF _CustID_Check<>_CustID THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Customer yang akan dipotong tidak sesuai dengan customer yang bayar';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;												/** potong lebih bayar */
			SELECT INTO spretOverRcv_Contra * from spOverRcv_Contra(_RcvD_RefNo);
			IF spretOverRcv_Contra.ErrNo<>0 THEN
				_retval.ErrNo:=spretOverRcv_Contra.ErrNo;
				_retval.ErrMsg:=spretOverRcv_Contra .ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		ELSIF ((_RcvDMtd='CASH') OR (_RcvDMtd='BTRF') OR (_RcvDMtd='CRC') OR (_RcvDMtd='DBC')) THEN

/**
		ELSIF ((_RcvDMtd='CRC') OR (_RcvDMtd='DBC')) THEN
			_CashAmt:=_RcvDAmt;
			
			IF (_RcvDMtd='CRC') THEN
				_CashANote:='Penerimaan #'||_RcvNo || ',Credit Card:' || COALESCE(_RcvDNote, '');
			ELSE  	-- RcvDMtd=DBC 
				_CashANote:='Penerimaan #'||_RcvNo || ',Debit Card:' || COALESCE(_RcvDNote, '');
			END IF;	
			
			SELECT INTO spretCashA_New * FROM spCashA_New(
				_CashID, _CustID, 'RCV', _RcvDate, 'I', 'RCV', _RcvNo, _CashAmt,
			   _ExtNote , TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD
			);
			IF spretCashA_New.ErrNo<>0 THEN
				_retval.ErrNo:=spretCashA_New.ErrNo;
				_retval.ErrMsg:=spretCashA_New.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
**/
		
			_CashAmt:=_RcvDAmt;
			
			IF (_RcvDMtd='BTRF') THEN
				_CashANote:='B.Tranfer:'||COALESCE(_RcvDNote, '')||COALESCE(':'||_ExtNote,'');
			ELSIF (_RcvDMtd='CASH') THEN
				_CashANote:='Tunai'||COALESCE(' '||_RcvDNote, '')||COALESCE(':'||_ExtNote,'');
			ELSIF (_RcvDMtd='CRC') THEN
				_CashANote:='CCard:'||COALESCE(_RcvDNote, '')||COALESCE(':'||_ExtNote,'');
			ELSIF (_RcvDMtd='DBC') THEN
				_CashANote:='DCard:'||COALESCE(_RcvDNote, '')||COALESCE(':'||_ExtNote,'');
			END IF;

			_CashANote:=_CashANote||E' :\nKeterangan : '||COALESCE(_RcvNote,'');
			
			SELECT INTO _isAutoReconcileIn isAutoReconcileIn FROM Cash WHERE CashID=_CashID;

			IF ((_RcvDMtd='CASH' OR _RcvDMtd='BTRF') AND (_isAutoReconcileIn=FALSE)) THEN
				SELECT INTO spretCashA_New * FROM spCashA_New(
					_CashID, _CustID, 'RCV', _RcvDate, 'I', 'RCV', _RcvNo, _CashAmt,
				   _CashANote , TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD
				);
				IF spretCashA_New.ErrNo<>0 THEN
					_retval.ErrNo:=spretCashA_New.ErrNo;
					_retval.ErrMsg:=spretCashA_New.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				--- saat ini block diatas sama dengan block dibawah, karena d spCashA_New nanti isReconcile akan dicheck & kas tidak akan bertambah/kurang apabila autoreconcilenya FALSE
			ELSE
				SELECT INTO spretCashA_New * FROM spCashA_New(
					_CashID, _CustID, 'RCV', _RcvDate, 'I', 'RCV', _RcvNo, _CashAmt,
				   _CashANote , TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD
				);
				IF spretCashA_New.ErrNo<>0 THEN
					_retval.ErrNo:=spretCashA_New.ErrNo;
					_retval.ErrMsg:=spretCashA_New.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;


		ELSIF((_RcvDMtd='CHQ') OR (_RcvDMtd='GIRO')) THEN

			SELECT INTO 
				_inout,_ACTIVE,_BPID,_srccash,_CbgDate,_CbgAmt,_NomerCbg,_ClrDate 
				inout,active,frombpid,srccashid,cbgdate,cbgamt,cbgno,clrdate 
			FROM cbg WHERE cbgid=_CbgID;

			IF(_inout<>'I') THEN
				_retval.ErrNo:=5001;
				_retval.ErrMsg:='Tipe Cek/BG harus IN! (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			IF(_ClrDate is NOT NULL) THEN
				_retval.ErrNo:=50002;
				_retval.ErrMsg:='Cek/BG #'||_CbgID||' sudah di-Clearingkan (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			IF (_CbgID IS NULL) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Cek/BG belum diset  (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			SELECT INTO _ACTIVE active FROM cbg WHERE CBGID=_CbgID;
			IF (_ACTIVE=false) then
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Cek/BG tidak aktif (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			IF (_inout='I') AND (_bpid<>_CustID) then
				_retval.ErrNo:=101;
				_retval.ErrMsg:='MitraBisnis tidak sama dengan di Cek/BG!! (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_CbgAmt<>_RcvDAmt) then
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Jumlah Pembayaran dengan Cek/BG Harus memiliki jumlah yang sama Cek/BG-nya! (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			_ExtNote:='C/BG '||_NomerCbg||' dari '||_CustID;

			SELECT INTO spretCashA_New * FROM spCashA_New(
				_srccash, _BPID, 'CBG', _RcvDate, 'I', 'CBG', _CbgID, _CbgAmt,
				_ExtNote , TRUE, NULL, NULL, NULL, NULL,NULL, NULL, NULL, NULl,_CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD);
			IF spretCashA_New.ErrNo<>0 THEN
				_retval.ErrNo:=spretCashA_New.ErrNo;
				_retval.ErrMsg:=spretCashA_New.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			UPDATE cbg SET active=false, useddate=_RcvDate WHERE cbgid=_CbgID;
		
		END IF;
	END LOOP;
	CLOSE cRcvD;

	SELECT INTO _ExcRateSPaid excrate FROM spaid WHERE rcvno=_RcvNo;

	IF _RcvType='RCV' THEN	
		/*--_DebtAmt:=_TotSPaid*(-1);		--penerimaan : potong piutang (-) */
		_DebtAmt:=_totBpAmt*(-1);
		_DebtNote:='Penerimaan #'||_RcvNo;

		SELECT INTO spretDebt_New * from spBPA_New(
			_CustID, 'P',_CrcID, _RcvDate, 'RCV', _RcvNo, _DebtAmt, _DebtNote, true, 
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID,_ExcRateSPaid, _FisRateSPaid
		);

		IF spretDebt_New.ErrNo<>0 THEN
			_retval.ErrNo:=spretDebt_New.ErrNo;
			_retval.ErrMsg:=spretDebt_New.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;
	SELECT INTO _CrcIDDefa CrcID FROM Crc where isdefault=true;


/************************************ AKUNTANSI ******************************************/

	IF isAccInstalled() AND _isAutoGen=false THEN
		IF(_IsRetur=TRUE AND _TotSPaid=0) THEN
			_retval:=(0, 'OK');
		ELSE
			/** cek akun dp */
			IF (_RcvType='DP') THEN
				_AccDP:=fAccDefault('DEPOCUST');
				IF (_AccDP IS NULL OR _AccDP='') THEN
					_retval.ErrNo:=50555;
					_retval.ErrMsg:=E'Default akun deposit Customer belum disetting\nSilahkan disetting melalui form default akun (spRcv_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:= 'Penerimaan '||fBPName(_CustID)||':'||_RcvNo;
			SELECT INTO _spret * FROM spJnl_Insert(
				_JnlNo, 'RCV', _RcvDate, 'RCV', _RcvNo,
				_JnlNote,true,true,cast(0 as NUMERIC(19,4)),
				_CrtBy,_CrtDate,_UpdBy,_UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_Count:=0;
			OPEN cRcvD FOR
				SELECT RcvDMtd, CashID, RcvdAmt, RefNo, RcvDNote, ExcRate, cbgid
				FROM RcvD WHERE RcvNo=_RcvNo;
			LOOP
				FETCH cRcvD INTO _RcvDMtd, _CashID, _RcvDAmt, _RcvD_RefNo, _RcvDNote, _ExcRateRcvD, _CbgID;
				EXIT WHEN NOT FOUND;
			
				_Count:=_Count+1;
				IF (_RcvDMtd='OVER') THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('DEPOCUST'), 'D', _RcvDAmt*_ExcRateRcvD, NULL, NULL, _RcvDAmt);

				ELSIF ((_RcvDMtd='CASH') OR (_RcvDMtd='BTRF') OR (_RcvDMtd='DBC') OR (_RcvDMtd='CRC')) THEN
					_AccCash:=fAccCash(_CashID, _CashInOut);

					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccCash, 'D', _RcvDAmt*_ExcRateRcvD, NULL, NULL, _RcvDAmt);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

				ELSIF((_RcvDMtd='CHQ') OR (_RcvDMtd='GIRO')) THEN
					SELECT INTO _CashCBG srccashid FROM cbg WHERE cbgid=_CbgID;
					_AccCash:=fAccCash(_CashCBG, _CashInOut);
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccCash, 'D', _RcvDAmt*_ExcRateRcvD, NULL, NULL, _RcvDAmt);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;
			END LOOP;
			CLOSE cRcvD;

			SELECT INTO _SPaidAmtSret, _SPaidAmtSretAsli SUM(spaidamt*excrate), SUM(spaidamt) 
			FROM SPaid WHERE RcvNo=_RcvNo AND reftype='SRET';

			_SPaidAmtSret:=COALESCE(_SPaidAmtSret, 0);
			_SPaidAmtSretAsli:=COALESCE(_SPaidAmtSretAsli,0);
		
			SELECT INTO _SPaidAmtSO, _SPaidAmtSOAsli SUM(spaidamt*excrate), SUM(spaidamt) 
			FROM SPaid WHERE RcvNo=_RcvNo AND reftype='SO';

			_SPaidAmtSO:=COALESCE(_SPaidAmtSO, 0);
			_SPaidAmtSOAsli:=COALESCE(_SPaidAmtSOAsli,0);
			
			SELECT INTO 
				_SPaidAmtFCN, _SPaidAmtFCNAsli 
				SUM(spaidamt*excrate), SUM(spaidamt) 
			FROM spaid WHERE rcvno=_RcvNo and reftype='FCN';

			_SPaidAmtFCN:=COALESCE(_SPaidAmtFCN, 0);
			_SPaidAmtFCNAsli:=COALESCE(_SPaidAmtFCNAsli, 0);

			_IsPosted:=false;

			OPEN cSPaid FOR
				SELECT reftype, (CASE WHEN reftype='SALE' THEN fAccTrans('SALE', refno, 'AR') WHEN reftype='DP' THEN _AccDP ELSE refno END), 
				SUM(spaidamt*excrate), SUM(spaidamt), deptid, prjid, SUM(spaidnote), crcid 
				FROM spaid 
				WHERE rcvno=_RcvNo AND reftype NOT IN ('SRET', 'FCN')
				GROUP BY reftype, (CASE WHEN reftype='SALE' THEN fAccTrans('SALE', refno, 'AR') WHEN reftype='DP' THEN _AccDP ELSE refno END), deptid, prjid, crcid;
			LOOP
				FETCH cSPaid INTO _RefType, _RefNo, _SPaidAmt, _SPaidAmtAsli, _DeptID, _PrjID, _CrcIDSPaid;
				EXIT WHEN NOT FOUND;
				_Count:=_Count+1;
				IF _RefType='SALE' THEN
					IF((_SPaidAmtSret!=0 OR _SPaidAmtFCN!=0) AND (_IsPosted=false)) THEN
					--SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,_Count,fBpAcc(_CustID,_CrcIDSPaid,'AR'),'K',(_SPaidAmt*_ExcRateSPaid)+_SPaidAmtSret,_DeptID,_PrjID,_SPaidAmt+_SPaidAmtSretAsli);
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _RefNo, 'K', _SPaidAmt+_SPaidAmtSret+_SPaidAmtFCN, _DeptID, _PrjID, _SPaidAmtAsli+_SPaidAmtSretAsli+_SPaidAmtFCNAsli);
						_IsPosted:=true;
					ELSE
					--SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,_Count,fBpAcc(_CustID,_CrcIDSPaid,'AR'),'K',_SPaidAmt*_ExcRateSPaid,_DeptID,_PrjID,_SPaidAmt);
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _RefNo, 'K', _SPaidAmt, _DeptID, _PrjID, _SPaidAmtAsli);
					END IF;
				ELSIF _RefType='ACC' THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _RefNo, 'K', _SPaidAmt, _DeptID, _PrjID, _SPaidAmt);
				ELSIF _RefType='DP' THEN
					IF (_RcvType='DP') THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _RefNo, 'K', _SPaidAmt, _DeptID, _PrjID, _SPaidAmtAsli);
					ELSE
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _RefNo, 'K', _SPaidAmt, _DeptID, _PrjID, _SPaidAmtAsli);
					END IF;
				END IF;
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
			CLOSE cSPaid;

			_OverRcvBC:=COALESCE(_OverRcvBC, 0);
			IF (_OverRcvBC<>0) THEN
				_AccOver:=fAccDefault('DEPOCUST');

				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccOver, 'K', _OverRcvBC, NULL, NULL, _OverRcv);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			_TotRcvDiscBC:=COALESCE(_TotRcvDiscBC,0);
			IF(_TotRcvDiscBC<>0) THEN
				_AccDisc:=fAccDefault('SALEDISC');
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccDisc, 'D', _TotRcvDiscBC, NULL, NULL, _TotRcvDisc);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			------------  Trigger post Jurnal -------------
			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			--- Jurnal klo buat ada selisih kurs saat ini ama pembelian waktu itu 
			-- cek saldo buat ngitung selisih kurs

			_saldoAkhir:=0;
			OPEN cSPaid FOR
				SELECT reftype,refno,spaidamt FROM spaid WHERE rcvno=_RcvNo AND crcid<>_CrcIDDefa;
			LOOP
				FETCH cSPaid INTO _RefType, _RefNo, _PaidAmt;
				EXIT WHEN NOT FOUND;

				IF(_RefType='SALE') THEN
					SELECT INTO _PayStatus paystatus FROM sale WHERE saleno=_RefNo;
					
					IF(_PayStatus='F') THEN
						SELECT INTO _saldo * FROM fCheckSaldo(_RefType, _RefNo);
					END IF;
				ELSIF(_RefType='SRET') THEN
					SELECT INTO _saldo * FROM fCheckSaldo(_RefType, _RefNo);
				END IF;

				_saldoAkhir:=_saldoAkhir+_saldo;
			END LOOP;
			CLOSE cSPaid;
		
			IF(_saldoAkhir<>0) THEN
				_JnlNo1:=fTransNoGet('JURNAL');
				_JnlNote1:= 'Revaluasi Kurs Penerimaan'||':'||_RcvNo;

				SELECT INTO _spret * FROM spJnl_Insert(_JnlNo1,'RCV',_RcvDate,'RCV',_RcvNo,
					_JnlNote1,true,true,CAST(0 AS NUMERIC(19,4)),_CrtBy,_CrtDate,_UpdBy,_UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, __xt, _CrcID, _ExcRate, _FisRate
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				IF(_saldoAkhir>0) THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, 1, fAccDefault('ACCPROFIT'), 'D', _saldoAkhir, NULL, NULL, _saldoAkhir);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1,2, fBpAcc(_CustID,_CrcID,'AR'), 'K', _saldoAkhir, NULL, NULL, _saldoAkhir);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				ELSIF(_saldoAkhir<0) THEN
					_saldoAkhir:=abs(_saldoAkhir);
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1,1, fBpAcc(_CustID,_CrcID,'AR'), 'D', _saldoAkhir, NULL, NULL, _saldoAkhir);

					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
					
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, 2, fAccDefault('ACCPROFIT'), 'K', _saldoAkhir, NULL, NULL, _saldoAkhir);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;

				SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spRcv_Void (
	_RcvNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	
	_CustID		VARCHAR(15);
	_RcvType	VARCHAR(5);
	_RcvDate	DATE;
	_TotSPaid	DECIMAL(19,4);
	_OverRcv	DECIMAL(19,4);
	_IsContra	BOOLEAN;
	_CrtBy		VARCHAR(10);
	_CrtDate	DATE;
	_UpdBy		VARCHAR(10);
	_UpdDate	DATE;

	_SaleNo		VARCHAR(15);
	_SPaidAmt	DECIMAL(19,4);
	_SPaidAmtBC	DECIMAL(19,4);

	_RcvDMtd	VARCHAR(4);
	_CashID		VARCHAR(15);
	_RcvDAmt	DECIMAL(19,4);
	_GDate		DATE;
	_GDueDate	DATE;
	_GClrDate	DATE;
	
	_RefNo		VARCHAR(15);
	_RcvD_RefNo 	VARCHAR(15);

	_CashAmt	DECIMAL(19,4);
	_CashANote	VARCHAR(50);

	_DebtAmt	DECIMAL(19,4);
	_DebtNote	VARCHAR(50);
		
	_tmpAmount	DECIMAL(19,4);
	_RefType	VARCHAR(5);
	_RefType1	VARCHAR(5);
	
	_JnlNo		VARCHAR(15);
	_CBGID		varchar;
	_ClearDate	DATE;
	_SrcCash	VARCHAR;

	cSPaid		REFCURSOR;
	cRcvD 		REFCURSOR;
	_LockDate	DATE;
	_isDraft	BOOLEAN;
	_ExcRate	NUMERIC(19,4);

	_DPID		BIGINT;
BEGIN
	_retval:=(50001,'GENERAL ERROR spRcv_Void');	/*-- praduga (tak) bersalah*/
	
	SELECT INTO _CustID, _RcvType, _RcvDate, _TotSPaid, _OverRcv, _IsContra, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _RefType,_RefNo, _isDraft
		    BPID, RcvType, RcvDate, TotSPaid, OverRcv, IsContra, CrtBy, CrtDate, UpdBy, UpdDate, RefType, RefNo, isDraft
	FROM Rcv WHERE RcvNo=_RcvNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Penerimaan #'||_RcvNo||' tidak ditemukan (spRcv_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM SPaid WHERE RcvNo=_RcvNo;
		DELETE FROM RcvD WHERE RcvNo=_RcvNo;
		DELETE FROM Rcv WHERE RcvNo=_RcvNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='RCV';
	IF(_LockDate is not null AND _LockDate>_RcvDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penerimaan Pembayaran Melebihi Tanggal Batas Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_OverRcv>0 AND EXISTS(SELECT RcvNo FROM RcvD WHERE RcvDMtd='OVER' AND RefNo=_RcvNo)) THEN
		_retval.ErrNo:=50002;
		_retval.Errmsg:='Deposit/Penerimaan lebih bayar sudah potong tidak boleh di-Void (spRcv_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	OPEN cSPaid FOR
		SELECT reftype, refno, spaidamt, excrate 
		FROM SPaid WHERE RcvNo=_RcvNo;
	LOOP
		FETCH cSPaid INTO _RefType1, _SaleNo, _SPaidAmt, _ExcRate;
		EXIT WHEN NOT FOUND;

		_SPaidAmtBC:=_SPaidAmt*_ExcRate;
		IF (_RefType1='SALE') THEN
			_tmpAmount:=_SPaidAmt*(-1);	/** contra */
			IF(_tmpAmount>0) THEN
				SELECT INTO _spret * FROM spSale_Rcv(_SaleNo, _tmpAmount*(-1), _RcvDate);
			ELSE
				SELECT INTO _spret * FROM spSale_Rcv(_SaleNo, _tmpAmount, _RcvDate);
			END IF;
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF (_RefType1='SRET') THEN
			SELECT INTO _spret * FROM spSRet_VoidContra(_SaleNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF (_RefType1='FCN') THEN
			SELECT INTO _spret * FROM spFCN_VoidContra(_SaleNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF (_RefType1='DP') THEN
			IF (_RcvType='DP') THEN
				SELECT INTO _DPID dpid FROM dp WHERE paidno=_RcvNo;
				SELECT INTO _spret * FROM spDP_Void(_DPID);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSE
				SELECT INTO _spret * FROM spDPBal_Update(CAST(_SaleNo AS BIGINT), _SPaidAmt*(-1), _SPaidAmtBC*(-1));
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END IF;
	END LOOP;
	CLOSE cSPaid;

	OPEN cRcvD FOR
		SELECT RcvDMtd, CashID, RcvDAmt, Refno,cbgid
		FROM RcvD WHERE RcvNo=_RcvNo;
	LOOP
		FETCH cRcvD INTO _RcvDMtd, _CashID, _RcvDAmt, _RcvD_RefNo,_CbgID;
		IF NOT FOUND THEN
			EXIT;
		END IF;
		IF (_RcvDMtd='OVER') THEN
			SELECT INTO _spret * FROM spOverRcv_VoidContra(_RcvD_RefNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF (_RcvDMtd='CTRA') THEN
			SELECT INTO _spret * FROM spSRet_VoidContra(_RcvD_RefNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF ((_RcvDMtd='CASH') OR (_RcvDMtd='BTRF') OR (_RcvDMtd='DBC') OR (_RcvDMtd='CRC') OR (_RcvDMtd='VOUC')) THEN
			SELECT INTO _spret * FROM spCashA_Void_byRef('RCV', _RcvNo, _CashID, null);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF((_RcvDMtd='CHQ') OR (_RcvDMtd='GIRO')) THEN
			IF (_CBGID IS NULL) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Tidak ada CBG';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _ClearDate,_SrcCash clrdate,srccashid FROM cbg WHERE cbgid=_CBGID;
			IF (_ClearDate IS NOT NULL) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='CBG Sudah di cairkan!!Unclear kan dulu CBG nya!!(spRcv_Void)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			UPDATE cbg SET active=true, useddate=NULL WHERE cbgid=_CBGID;
			SELECT INTO _spret * FROM spCashA_Void_byRef('CBG', _CBGID, _SrcCash, null);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cRcvD;

	IF _RcvType='RCV' THEN
		SELECT INTO _spret * FROM spBPA_Void_byRef('RCV', _RcvNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrNo;
		END IF;
	END IF;

	IF isAccInstalled() THEN
		SELECT into _spret * from spJnl_Void_byRef('RCV',_RcvNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
			EXIT;
		END IF;
	END IF;

	DELETE FROM SPaid WHERE RcvNo=_RcvNo;
	DELETE FROM RcvD WHERE RcvNo=_RcvNo;
	DELETE FROM Rcv WHERE RcvNo=_RcvNo;

	_retval:=(0, 'OK');	/*--  HAPPY ENDING :)*/
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spOverPay_Contra (
	_PayNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_OverPay	NUMERIC(19,4);
	_IsContra	BOOLEAN;
BEGIN
	SELECT INTO _OverPay, _IsContra OverPay, IsContra
	FROM Pay WHERE PayNo=_PayNo;
	
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Pembayaran #'||_PayNo||' tidak ditemukan (spOverPay_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMSg;
	END IF;

	IF (_OverPay<=0) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Pembayaran #'||_PayNo||' tidak lebih bayar (spOverPay_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_IsContra=true) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Lebih-bayar sudah dipotongkan Pembayaran lain sebelumnya (spOverPay_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error UPDATE Pay:IsContra! (spOverPay_Contra)';
	UPDATE Pay SET IsContra=true WHERE PayNo=_PayNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spOverPay_VoidContra (
	_PayNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_IsContra	BOOLEAN;
BEGIN
	SELECT INTO _IsContra IsContra FROM Pay WHERE PayNo=_PayNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Pembayaran #'||_PayNo||' tidak ditemukan (spOverPay_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_IsContra<>true) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Pembayaran #'||_PayNo||' belum pernah dipotong-bayarkan sebelumnya (spOverPay_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error UPDATE Pay:IsContra (spOverPay_VoidContra)';
	UPDATE Pay SET IsContra=false WHERE PayNo=_PayNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spOverRcv_Contra (
	_RcvNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_OverRcv	NUMERIC(19,4);
	_IsContra	BOOLEAN;
BEGIN
	SELECT INTO _OverRcv, _IsContra OverRcv, IsContra
	FROM Rcv WHERE RcvNo=_RcvNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Pembayaran #'||_RcvNo||' tidak ditemukan (spOverRcv_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_OverRcv<=0) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Pembayaran #'||_RcvNo||' tidak lebih bayar (spOverRcv_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_IsContra=true) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Lebih bayar sudah dipotongkan Pembayaran sebelumnya (spOverRcv_Contra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error UPDATE Rcv:IsContra (spOverRcv_Contra)';
	UPDATE Rcv SET IsContra=true WHERE RcvNo=_RcvNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spOverRcv_VoidContra (
	_RcvNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_IsContra	BOOLEAN;
BEGIN
	SELECT INTO _IsContra IsContra FROM Rcv WHERE RcvNo=_RcvNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Pembayaran #'||_RcvNo||' tidak ditemukan (spOverRcv_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_IsContra<>true) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Pembayaran #'||_RcvNo||' belum pernah dipotong sebelumnya (spOverRcv_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error UPDATE Rcv:IsContra (spOverRcv_VoidContra)';
	UPDATE Rcv SET IsContra=false WHERE RcvNo=_RcvNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spRpt_Remove (
	_RptID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error DELETE RBand (spRpt_Remove)';
	DELETE FROM RBand WHERE RptID=_RptID;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error DELETE RVar (spRpt_Remove)';
	DELETE FROM RVar WHERE RptID=_RptID;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error DELETE Rpt (spRpt_Remove)';
	DELETE FROM Rpt WHERE RptID=_RptID;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='ALL DELETE SUCCESS!!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPay_New(_PayNo VARCHAR) 
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	spretCashA_New		SPStatus%ROWTYPE;

	_VendorID		VARCHAR(15);
	_PayType		VARCHAR(5);
	_PayDate		DATE;
	_TotPPaid		DECIMAL(19,4);
	_TotPayAmt		DECIMAL(19,4);
	_OverPay		DECIMAL(19,4);
	_IsContra		BOOLEAN;
	_CrtBy			VARCHAR(10);
	_CrtDate		TIMESTAMP;
	_UpdBy			VARCHAR(10);
	_UpdDate		TIMESTAMP;
	_CbgDate		DATE;
	_PurcNo			VARCHAR(15);
	_PPaidAmt		DECIMAL(19,4);
	_PPaidAmtPret		DECIMAL(19,4);
	_PayDMtd		VARCHAR(4);
	_CashID			VARCHAR(15);
	_CashCBG		VARCHAR(15);
	_PayDAmt		DECIMAL(19,4);
	_GDate			DATE;
	_GDueDate		DATE;
	_RefType		VARCHAR(5);
	_RefNo			VARCHAR;
	_PayD_RefNo		VARCHAR(15);
	_PayDNote		TEXT;

	_CashAmt		DECIMAL(19,4);
	_CashANote		VARCHAR;
	_ExtNote		VARCHAR;
	_LastRefType		VARCHAR(5);

	_CrcID			VARCHAR(5);
	_ExcRate		DECIMAL(19,4);
	_BPAAmt			DECIMAL(19,4);
	_BPANote 		TEXT;

	_TempStr		TEXT;
	
	cPPaid			REFCURSOR;
	cPayD			REFCURSOR;

	_AccCash		VARCHAR(10);
	_AccAsset		VARCHAR(10);
	_AccNo			VARCHAR(10);
	_Count			INT;
	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;
	_isAutoGen		BOOLEAN;

	_VendorID_Check		VARCHAR(15);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_CBGID			VARCHAR;
	_ACTIVE			BOOLEAN;
	_BPID			VARCHAR;
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);
	_inout			VARCHAR;
	_NomerCbg		VARCHAR;
	_CbgAmt			DECIMAL(19,4);
	_outbal			DECIMAL(19,4);
	_total			DECIMAL(19,4);
	_ToCustID		VARCHAR;
	_CustIDCBG		VARCHAR;
	_totBpAmt		DECIMAL(19,4);
	_isPosted		BOOLEAN;
	_ClrDate		DATE;
	_AccFcn			VARCHAR(10);
	_AccOver 		VARCHAR(10);
	_IsRetur		BOOLEAN;
	__xt			BOOLEAN;
	_ExcRatePurc		DECIMAL(19,4);
	_SelisihRate		DECIMAL(19,4);
	_JnlNo1			VARCHAR;
	_JnlNote1		VARCHAR;
	_PayNote		VARCHAR;
	_TotalPurc		NUMERIC(19,4);
	_LockDate		DATE;
	_Cnt			INT;
	_ReftypeDesc		VARCHAR;
	_PPaidNote		TEXT;
	_PRetAmt		NUMERIC(19,4);
	_AccName		VARCHAR;
	_totPayDisc		NUMERIC(19,4);
	_AccDisc		VARCHAR;
	_SelisihNote		VARCHAR;
	_FcnAmt			NUMERIC(19,4);
	_OverPayBC		NUMERIC(19,4);
	_TotPayDiscBC		NUMERIC(19,4);
	_CrcIDPPaid		VARCHAR;
	_CrcIDPurc		VARCHAR;
	_CrcIDPret		VARCHAR;
	_ExcRatePPaid		NUMERIC(19,4);
	_ExcRatePayD		NUMERIC(19,4);
	_saldoAkhir		NUMERIC(19,4);
	_saldo			NUMERIC(19,4);
	_PaidAmt		NUMERIC(19,4);
	_TotalPaid		NUMERIC(19,4);
	_CrcIDDefa		VARCHAR;
	_PayStatus		VARCHAR;
	_CrcIDFcn		VARCHAR;
	_FisRate		NUMERIC(19,4);
	_PPaidAmtAsli		NUMERIC(19,4);
	_PPaidAmtPretAsli	NUMERIC(19,4);
	_VendorIDPo		VARCHAR;
	_FisRatePPaid		NUMERIC(19,4);
	_CrcIDRcvD		VARCHAR;
	_ExcRateRcvD		NUMERIC(19,4);
	_FisRateRcvD		NUMERIC(19,4);
	_countPurc		INT;
	_isAutoReconcileOut	BOOLEAN;
	_isDraft		BOOLEAN;

	_MRefNo			VARCHAR;
	_MRefType		VARCHAR;
	_AccDP			VARCHAR;

	_PPaidAmtFCN		NUMERIC(19,4);
	_PPaidAmtFCNAsli	NUMERIC(19,4);
	_CashInOut		CHAR(1);
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_CashInOut:='O';

	SELECT INTO 
		_PerID, _BranchID, _DeptID, _PrjID, _VendorID, _PayType, _PayDate, _PayNote, _TotPPaid, _TotPayAmt, _OverPay, _IsContra, _CrtBy, _CrtDate, _UpdBy, _UpdDate, 
		_CrcID, _isAutoGen, __xt, _totPayDisc, _ExcRate, _OverPayBC, _totPayDiscBC, _FisRate, _isDraft, _MRefNo, _MRefType  
		perid, branchid, deptid, prjid, bpid, paytype, paydate, paynote, totppaid, totpayamt, overpay, iscontra, crtby, crtdate, updby, upddate, 
		crcid, isautogen, _xt, totpaydisc, excrate, overpaybc, totpaydiscbc, fisrate, isdraft, refno, reftype 
	FROM pay WHERE payno=_PayNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Pembayaran #% tidak ditemukan (spPay_New)', _PayNo;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');

		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM tlock WHERE transtype='PAY';
	IF(_LockDate IS NOT NULL AND _LockDate>_PayDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Pembayaran Melebihi Batas Tanggal Kunci Transaksi ! (spPay_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_OverPay<0) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Total Cara-Bayar < (Total hutang yang dibayar-Diskon) (spPay_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF(_PayType='PURC' AND _MRefNo IS NULL) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Tipe Pembayaran Cash Harus Lewat Pembelian Tunai!!(spPay_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	SELECT INTO _countPurc COUNT(*) FROM ppaid WHERE payno=_PayNo and reftype='PURC';

	_totBpAmt:=0;
	_ExtNote:='';
	_LastRefType:='';
	OPEN cPPaid FOR
		SELECT reftype, refno, ppaidamt, crcid, excrate FROM ppaid WHERE payno=_PayNo;
	LOOP
		FETCH cPPaid INTO _RefType, _RefNo, _PPaidAmt, _CrcIDPPaid, _ExcRatePPaid;
		EXIT WHEN NOT FOUND;

		IF _RefType='PURC' THEN
			IF (_LastRefType<>'PURC') THEN
				_ExtNote:=_ExtNote||'Beli ';
				_LastRefType:='PURC';
			END IF;

			SELECT INTO _VendorID_Check, _CrcIDPurc vendorid, crcid FROM purc WHERE purcno=_RefNo;
			IF _VendorID_Check<>_VendorID THEN
				_retval.ErrNo:=50001;
				_retval.ErrMsg:='Supplier yang akan dipotong tidak sesuai dengan supplier yang bayar (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;	

			IF _CrcIDPurc<>_CrcIDPPaid THEN
				_retval.ErrNo:=50002;
				_retval.ErrMsg:='Mata uang '||_Refno||' tidak sama dengan aslinya! (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;	


			IF(_PPaidAmt<0) THEN
				--SELECT INTO _spret * FROM spPurc_Pay(_RefNo, _PPaidAmt*(-1));
				_retval.ErrNo:=50003;
				_retval.ErrMsg:='Jumlah Untuk Bayar tidak boleh minus (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spPurc_Pay(_RefNo, _PPaidAmt, _PayDate);

			_totBpAmt:=_totBpAmt+_PPaidAmt;
		ELSIF _RefType='PRET' THEN

			IF (_LastRefType<>'PRET') THEN
				_ExtNote:=_ExtNote||'R.Beli ';
				_LastRefType:='PRET';
			END IF;

			SELECT INTO _VendorID_Check, _PRetAmt, _CrcIDPRet vendorid, total, crcid FROM pret WHERE pretno=_RefNo;
			IF _VendorID_Check<>_VendorID THEN
				_retval.ErrNo:=50001;
				_retval.ErrMsg:='Supplier yang akan dipotong tidak sesuai dengan supplier yang bayar(spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF _CrcIDPRet<>_CrcIDPPaid THEN
				_retval.ErrNo:=50002;
				_retval.ErrMsg:='Mata uang '||_Refno||' tidak sama dengan aslinya! (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (ABS(_PPaidAmt)<>_PRetAmt) THEN
				_retval.ErrNo:=50003;
				_retval.ErrMsg:='Jumlah Melebihi Retur!! (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_countPurc=0) THEN
				_retval.ErrNo:=50004;
				_retval.ErrMsg:='Belum Ada Hutang yang Dipilih? (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_totBpAmt:=_totBpAmt+_PPaidAmt;

			SELECT INTO _spret * FROM spPret_Contra(_RefNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			_IsRetur:=true;

		ELSIF _RefType='DP' THEN
			_AccDP:=fAccDefault('DEPOVEND');
			IF (_PayType='DP') THEN
				SELECT INTO _spret * FROM spDP_New(_VendorID, _PayDate, _MRefNo, COALESCE(_MRefType, 'PAY'), _CrcID, _ExcRate, _FisRate, _PPaidAmt, _AccDP, _PayNo, 'PAY');
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSE
				IF (_CrcIDPPaid!=_CrcID) THEN
					_retval.ErrNo:=50555;
					_retval.ErrMsg:='Mata Uang D.P. tidak sama dengan mata uang Pembayaran (spPay_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				
				SELECT INTO _spret * FROM spDPBal_Update(CAST(_RefNo AS BIGINT), _PPaidAmt, _PPaidAmt*_ExcRatePPaid);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			--_totBpAmt:=_totBpAmt+_PPaidAmt;
		ELSIF _RefType='FCN' THEN
			SELECT INTO _VendorID_Check,_FCNAmt,_CrcIDFcn bpid,fcnamt,crcid FROM FCN WHERE FcnNo=_RefNo;
			IF _VendorID_Check<>_VendorID THEN
				_retval.ErrNo:=55555;
				_retval.ErrMsg:='Supplier FCN tidak sama! (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			IF _CrcIDFcn<>_CrcIDPPaid THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Mata uang '||_Refno||' tidak sama! (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(abs(_PPaidAmt)<>_FcnAmt) THEN
				_retval.ErrNo:=55555;
				_retval.ErrMsg:='Jumlah Tidak Sama Dengan FCN!(spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_countPurc=0) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Belum Ada Hutang yang Dipilih?(spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			UPDATE FCN SET isContra=true WHERE FcnNo=_RefNo;
			IF NOT FOUND THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Error Update FCN ! (spRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _AccFcn AccFcn FROM FCN WHERE FcnNo=_RefNo;

			_totBpAmt:=_totBpAmt+_PPaidAmt;
		ELSIF _RefType='ACC' THEN
			IF (_LastRefType<>'ACC') THEN
				_ExtNote:=_ExtNote||'Acc ';
				_LastRefType:='ACC';
			END IF;
			SELECT INTO _AccName accname FROM acc where accno=_RefNo;
			_RefNo:=_AccName;

			--_totBpAmt:=_totBpAmt+_PPaidAmt;
		END IF;
		

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF _CrcID<>_CrcIDPPaid THEN
			_retval.ErrNo:=50011;
			_retval.ErrMsg:='Mata uang '||_Refno||' tidak sama! (spPay_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_ExtNote:=_ExtNote||_RefNo||' ';

	END LOOP;
	CLOSE cPPaid;
	
	OPEN cPayD FOR
		SELECT paydmtd, cashid, paydamt, refno, paydnote, cbgid, crcid, excrate, fisrate
		FROM payd WHERE payno=_PayNo;
	LOOP
		FETCH cPayD INTO _PayDMtd, _CashID, _PayDAmt, _PayD_RefNo, _PayDNote,_CBGID, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD;
		EXIT WHEN NOT FOUND;
	
		IF (_PayDMtd='OVER') THEN
			SELECT INTO _VendorID_Check bpid FROM pay WHERE payno=_PayD_RefNo;
			IF _VendorID_Check<>_VendorID THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Supplier yang akan dipotong tidak sesuai dengan supplier yang bayar';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			--- potong lebih bayar -----
			SELECT INTO _spret * FROM spOverPay_Contra(_PayD_RefNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		ELSEIF ((_PayDMtd='CASH') OR (_PayDMtd='BTRF') OR (_PayDMtd='CRC') OR (_PayDMtd='DBC')) THEN
/***
		ELSIF ((_PayDMtd='CRC') OR (_PayDMtd='DBC')) THEN
			_CashAmt:=_PayDAmt*(-1);

			IF ((_PayDMtd='CRC')) THEN
				_CashANote:='Pembayaran #'||_PayNo||', Credit Card:'||COALESCE(_PayDNote, '');
			ELSE
				_CashANote:='Pembayaran #'||_PayNo||', Debit Card'||COALESCE(_PayDNote, '');
			END IF;
			
			SELECT INTO _spret * FROM spCashA_New(
				_CashID, _VendorID, 'PAY', _PayDate, 'O', 'PAY', _PayNo, _CashAmt,
				_ExtNote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
***/

			_CashAmt:=_PayDAmt*(-1);

			IF (_PayDMtd='BTRF') THEN
				_CashANote:='B.Tranfer:'||COALESCE(_PayDNote, '')||COALESCE(':'||_ExtNote,'');
			ELSIF (_PayDMtd='CASH') THEN
				_CashANote:='Tunai'||COALESCE(' '||_PayDNote, '')||COALESCE(':'||_ExtNote,'');
			ELSIF (_PayDMtd='CRC') THEN
				_CashANote:='CCard:'||COALESCE(_PayDNote, '')||COALESCE(':'||_ExtNote,'');
			ELSIF (_PayDMtd='DBC') THEN
				_CashANote:='DCard:'||COALESCE(_PayDNote, '')||COALESCE(':'||_ExtNote,'');
			END IF;

			_CashANote:=_CashANote||E' :\nKeterangan : '||COALESCE(_PayNote,'');

			SELECT INTO _isAutoReconcileOut isAutoReconcileOut FROM Cash WHERE CashID=_CashID;

			IF ((_PayDMtd='CASH' OR _PayDMtd='BTRF') AND (_isAutoReconcileOut=FALSE)) THEN

				SELECT INTO _spret * FROM spCashA_New(
					_CashID, _VendorID, 'PAY', _PayDate, 'O', 'PAY', _PayNo, _CashAmt,
					_CashANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				--- saat ini block diatas sama dengan block dibawah, karena d spCashA_New nanti isReconcile akan dicheck & kas tidak akan bertambah/kurang apabila autoreconcilenya FALSE
			ELSE

				SELECT INTO _spret * FROM spCashA_New(
					_CashID, _VendorID, 'PAY', _PayDate, 'O', 'PAY', _PayNo, _CashAmt,
					_CashANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

			END IF;


		ELSIF((_PayDMtd='CHQ') OR (_PayDMtd='GIRO')) THEN
			IF (_CBGID IS NULL) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Tidak ada Cek/BG (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _ACTIVE, _bpid, _inout, _ClrDate active, tobpid, inout, clrdate FROM cbg WHERE cbgid=_CBGID;
			IF (_ACTIVE=false) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Cek/BG tidak aktif (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_ClrDate is NOT NULL) THEN
				_retval.ErrNo:=50002;
				_retval.ErrMsg:='CBG #'||_CBGID||' sudah di clear kan (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_bpid<>_VendorID) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Supplier tidak sama dengan Cek/BG!! (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_bpid IS NULL) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Cek/BG tidak memiliki customer tujuan!! (spPay_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;


			SELECT INTO _CashCBG, _CustIDCBG, _CbgDate, _CbgAmt, _NomerCbg, _crcid, _ExcRate, _ToCustID 
				srccashid, frombpid, cbgdate, cbgamt, cbgno, crcid, excrate, tobpid 
			FROM cbg WHERE cbgid=_CBGID;

			IF(_CbgAmt<>_PayDAmt) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Pembayaran dengan Giro Harus memiliki jumlah yang sama dengan giro asli nya!';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_inout='I') THEN
			
				_ExtNote:='CBG '||_NomerCbg||' dari '||_CustIDCBG;

				SELECT INTO spretCashA_New * FROM spCashA_New(
					_CashCBG, _CustIDCBG, 'CBG', _PayDate, 'I', 'CBG', _CBGID, _CbgAmt,
					_ExtNote , TRUE, NULL, NULL, NULL, NULL,NULL, NULL, NULL, NULl, _CrcIDRcvD, _ExcRateRcvD, _FisRateRcvD);

				IF spretCashA_New.ErrNo<>0 THEN
					_retval.ErrNo:=spretCashA_New.ErrNo;
					_retval.ErrMsg:=spretCashA_New.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				UPDATE cbg SET active=false, useddate=_PayDate WHERE cbgid=_CBGID;
			ELSIF(_inout='O') THEN
				/*SELECT INTO _outbal outbal FROM cash WHERE cashid=_CashCBG;
				_total:=_outbal+_CbgAmt;

				UPDATE cash set outbal=total where cashid=_CashCBG;*/
				UPDATE cbg SET active=false, useddate=_PayDate WHERE cbgid=_CBGID;
			END IF;

		END IF;

	END LOOP;
	CLOSE cPayD;

	--_BPAAmt=_TotPPaid;	-- pembayaran : potong hutang (+) 
	SELECT INTO _ExcRatePPaid, _FisRatePPaid excrate, fisrate FROM ppaid WHERE payno=_PayNo;
	_BPAAMT:=_totBpAmt;
	_BPANote='Pembayaran #'||_PayNo;
	IF _PayType='PAY' THEN
		SELECT INTO _spret * FROM spBPA_New(
			_VendorID, 'P',_CrcID, _PayDate, 'PAY', _PayNo, _BPAAmt, 
			_BPANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _ExcRatePPaid, _FisRatePPaid
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	SELECT INTO _CrcIDDefa crcid FROM crc where isdefault=true;


/* /////////////// AKUNTANSI ////////////////////// */

	IF isAccInstalled() AND _isAutoGen=false THEN

		IF(_IsRetur=TRUE AND _TotPPAID=0) THEN
			_retval:=(0, 'OK');
		ELSE
			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:= 'Pembayaran '||fBPName(_VendorID)||':'||_PayNo;
			SELECT INTO _spret * FROM spJnl_Insert(_JnlNo,'PAY',_PayDate,'PAY',_PayNo,
				_JnlNote,true,true,cast(0 as NUMERIC(19,4)),_CrtBy,_CrtDate,_UpdBy,_UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID,__xt, _CrcID, _ExcRate, _FisRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_Count:=0;

			/** cek akun dp */
			IF (_PayType='DP') THEN
				_AccDP:=fAccDefault('DEPOVEND');
				IF (_AccDP IS NULL OR _AccDP='') THEN
					_retval.ErrNo:=50555;
					_retval.ErrMsg:=E'Default akun deposit Supplier belum disetting\nSilahkan disetting melalui form default akun (spPay_New)';
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			SELECT INTO 
				_PPaidAmtPret, _PPaidAmtPretAsli 
				SUM(ppaidamt*excrate), SUM(ppaidamt) 
			FROM ppaid WHERE payno=_PayNo and reftype='PRET';

			_PPaidAmtPret:=COALESCE(_PPaidAmtPret, 0);
			_PPaidAmtPretAsli:=COALESCE(_PPaidAmtPretAsli, 0);

			SELECT INTO 
				_PPaidAmtFCN, _PPaidAmtFCNAsli 
				SUM(ppaidamt*excrate), SUM(ppaidamt) 
			FROM ppaid WHERE payno=_PayNo and reftype='FCN';

			_PPaidAmtFCN:=COALESCE(_PPaidAmtFCN, 0);
			_PPaidAmtFCNAsli:=COALESCE(_PPaidAmtFCNAsli, 0);

			_IsPosted:=false;

			OPEN cPPaid FOR
				SELECT reftype, (CASE WHEN reftype='PURC' THEN fAccTrans('PURC', refno, 'AP') WHEN reftype='DP' THEN _AccDP ELSE refno END), 
				SUM(ppaidamt*excrate), SUM(ppaidamt), deptid, prjid, SUM(ppaidnote), crcid 
				FROM ppaid 
				WHERE payno=_PayNo AND reftype NOT IN ('PRET', 'FCN')
				GROUP BY reftype, (CASE WHEN reftype='PURC' THEN fAccTrans('PURC', refno, 'AP') WHEN reftype='DP' THEN _AccDP ELSE refno END), deptid, prjid, crcid;
			LOOP
				FETCH cPPaid INTO _RefType, _RefNo, _PPaidAmt, _PPaidAmtAsli, _DeptID, _PrjID, _PPaidNote, _crcIDPPaid;
				EXIT WHEN NOT FOUND;

				_Count:=_Count+1;

				IF _RefType='PURC' THEN
					IF((_PPaidAmtPret!=0 OR _PPaidAmtFCN!=0) AND (_IsPosted=false)) THEN
					--SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,_Count,fBpAcc(_VendorID,_crcIDPPaid,'AP'),'D',(_PPaidAmt*_ExcRatePPaid)+_PPaidAmtPret,_DeptID,_PrjID, _PPaidNote,_PPaidAmt+_PPaidAmtPretAsli);
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _RefNo, 'D', _PPaidAmt+_PPaidAmtPret+_PPaidAmtFCN, _DeptID, _PrjID, _PPaidNote, _PPaidAmtAsli+_PPaidAmtPretAsli+_PPaidAmtFCNAsli);
						_IsPosted:=true;
					ELSE
					--SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,_Count,fBpAcc(_VendorID,_crcIDPPaid,'AP'),'D',_PPaidAmt*_ExcRatePPaid,_DeptID,_PrjID, _PPaidNote,_PPaidAmt);
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _RefNo, 'D', _PPaidAmt, _DeptID, _PrjID, _PPaidNote, _PPaidAmtAsli);
					END IF;
				ELSIF _RefType='ACC' THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _RefNo, 'D', _PPaidAmt, _DeptID, _PrjID, _PPaidNote, _PPaidAmtAsli);
				ELSIF _RefType='DP' THEN
					IF (_PayType='DP') THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccDP, 'D', _PPaidAmt, _DeptID, _PrjID, _PPaidNote, _PPaidAmtAsli);
					ELSE
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccDP, 'D', _PPaidAmt, _DeptID, _PrjID, _PPaidNote, _PPaidAmtAsli);
					END IF;
				END IF;

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
			CLOSE cPPaid;

			OPEN cPayD FOR
				SELECT paydmtd, cashid, paydamt, refno, paydnote, excrate, cbgid
				FROM payd WHERE payno=_PayNo;
			LOOP
				FETCH cPayD INTO _PayDMtd, _CashID, _PayDAmt, _PayD_RefNo, _PayDNote, _ExcRatePayD, _CBGID;
				EXIT WHEN NOT FOUND;
				_Count:=_Count+1;

				IF (_PayDMtd='OVER') THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('DEPOVEND'), 'K', _PayDAmt*_ExcRatePayD, NULL, NULL, _PayDAmt);
				ELSEIF ((_PayDMtd='CASH') OR (_PayDMtd='BTRF') OR (_PayDMtd='DBC') OR (_PayDMtd='CRC')) THEN
					_AccCash:=fAccCash(_CashID, _CashInOut);

					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccCash, 'K', _PayDAmt*_ExcRatePayD, NULL, NULL, _PayDAmt);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				ELSIF((_PayDMtd='CHQ') OR (_PayDMtd='GIRO')) THEN
					SELECT INTO _CashCBG srccashid FROM cbg WHERE cbgid=_CBGID;

					IF(_inout='I') THEN
						_AccAsset:=fAccCash(_CashCBG, _CashInOut);
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccAsset, 'K', _PayDAmt*_ExcRatePayD, NULL, NULL, _PayDAmt);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
					ELSIF(_inout='O') THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('OPENCBG'), 'K', _PayDAmt*_ExcRatePayD, NULL, NULL, _PayDAmt);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
					END IF;
				END IF;

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
			CLOSE cPayD;

			_OverPayBC:=COALESCE(_OverPayBC, 0);
			IF (_OverPayBC<>0) THEN
				_AccOver:=fAccDefault('DEPOVEND');

				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccOver, 'D', _OverPayBC, NULL, NULL,_OverPay);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			_totPayDiscBC:=COALESCE(_totPayDiscBC,0);
			IF (_totPayDiscBC<>0) THEN
				_AccDisc:=fAccDefault('PURCDISC');

				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccDisc, 'K', _totPayDiscBC, NULL, NULL, _totPayDisc);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;

			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		--- Jurnal klo buat ada selisih kurs saat ini ama pembelian waktu itu
			_Cnt:=0;

		--- cek saldo, buat klo ada selisih kurs.. ------
			_saldoAkhir:=0;
			OPEN cPPaid FOR
				SELECT reftype, refno, ppaidamt FROM ppaid WHERE payno=_PayNo AND crcid<>_CrcIDDefa;
			LOOP
				FETCH cPPaid INTO _RefType, _RefNo, _PaidAmt;
				EXIT WHEN NOT FOUND;
			
				IF(_RefType='PURC') THEN
					SELECT INTO _PayStatus paystatus FROM purc WHERE purcno=_RefNo;
					IF(_PayStatus='F') THEN
						SELECT INTO _saldo * FROM fCheckSaldo(_RefType, _RefNo);
					END IF;
				ELSIF(_RefType='PRET') THEN
					SELECT INTO _saldo * FROM fCheckSaldo(_RefType, _RefNo);
				END IF;


				_saldoAkhir:=_saldoAkhir+_saldo;
			END LOOP;
			CLOSE cPPaid;

			IF(_saldoAkhir<>0) THEN
				_JnlNo1:=fTransNoGet('JURNAL');
				_JnlNote1:= 'Revaluasi Kurs Pembayaran'||':'||_PayNo;

				SELECT INTO _spret * FROM spJnl_Insert(_JnlNo1, 'PAY', _PayDate, 'PAY', _PayNo, 
					_JnlNote1, true, true, CAST(0 AS NUMERIC(19,4)), _CrtBy, _CrtDate, _UpdBy, _UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, __xt, _CrcID, _ExcRate, _FisRate
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				IF(_saldoAkhir<0) THEN
					_saldoAkhir:=ABS(_saldoAkhir);
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, 1, fAccDefault('ACCPROFIT'), 'D', _saldoAkhir, NULL, NULL,_saldoAkhir);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1,2, fBpAcc(_VendorID,_CrcID,'AP'), 'K', _saldoAkhir, NULL, NULL, _saldoAkhir);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				ELSIF(_saldoAkhir>0) THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1,1, fBpAcc(_VendorID,_CrcID,'AP'), 'D', _saldoAkhir, NULL, NULL, _saldoAkhir);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, 2, fAccDefault('ACCPROFIT'), 'K', _saldoAkhir, NULL, NULL, _saldoAkhir);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END IF;

				SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;	-- if (saldoAkhir<0)
		END IF; -- _IF (saldoAkhir<>0)
	END IF; -- if (isAcc)


	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPay_Void (
	_PayNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_VendorID	VARCHAR(15);
	_PayType	VARCHAR(5);
	_PayDate	DATE;
	_TotPPaid	DECIMAL(19,4);
	_TotPayAmt	DECIMAL(19,4);
	_OverPay	DECIMAL(19,4);
	_IsContra	BOOLEAN;
	_CrtBy		VARCHAR(10);
	_CrtDate	DATE;
	_UpdBy		VARCHAR(10);
	_UpdDate	DATE;

	_PurcNo		VARCHAR(15);
	_PPaidAmt	DECIMAL(19,4);
	_PPaidAmtBC	DECIMAL(19,4);

	_PayDMtd	VARCHAR(4);
	_CashID		VARCHAR(15);
	_PayDAmt	DECIMAL(19,4);
	_GDate		DATE;
	_GDueDate	DATE;
	_GClrDate	DATE;
	_RefType	VARCHAR(5);
	_RefNo		VARCHAR(15);
	_PayD_RefNo 	VARCHAR(15);

	_CashAmt	DECIMAL(19,4);
	_CashANote	VARCHAR(50);

	_CredAmt	DECIMAL(19,4);
	_CredNote	VARCHAR(50);

	_tmpAmount	DECIMAL(19,4);
	_inout		VARCHAR;
	_CashCBG	varchar;
	_CbgAmt		DECIMAL(19,4);
	_CBGID		varchar;
	_outbal		DECIMAL(19,4);
	total		DECIMAL(19,4);
	_ClearDate	DATE;

	_JnlNo		VARCHAR(15);
	cPPaid		REFCURSOR;
	cPayD		REFCURSOR;
	_LockDate	DATE;
	_isDraft	BOOLEAN;
	_ExcRate	NUMERIC(19,4);

	_DPID		BIGINT;
BEGIN
	_retval:=(50001,'GENERAL ERROR spPay_Void');

	SELECT INTO _VendorID, _PayType, _PayDate, _TotPPaid, _OverPay, _IsContra, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _isDraft
		    BPID, PayType, PayDate, TotPPaid, OverPay, IsContra, CrtBy, CrtDate, UpdBy, UpdDate, isDraft
	FROM Pay WHERE PayNo=_PayNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.Errmsg:='Pembayaran #'||_PayNo||' tidak ditemukan (spPay_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM PayD WHERE PayNo=_PayNo;
		DELETE FROM PPaid WHERE PayNo=_PayNo;
		DELETE FROM Pay WHERE PayNo=_PayNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='PAY';
	IF(_LockDate is not null AND _LockDate>_PayDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Pembayaran Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_OverPay>0 AND EXISTS(SELECT PayNo FROM PayD WHERE PayDMtd='OVER' AND RefNo=_PayNo)) THEN
		_retval.ErrNo:=50002;
		_retval.Errmsg:='Deposit/Pembayaran lebih bayar sudah potong tidak boleh di-Void (spPay_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cPPaid FOR
		SELECT reftype, refno, ppaidamt, excrate
		FROM ppaid WHERE payno=_PayNo;
	LOOP
		FETCH cPPaid INTO _RefType, _RefNo, _PPaidAmt, _ExcRate;
		EXIT WHEN NOT FOUND;

		_PPaidAmtBC:=_PPaidAmt*_ExcRate;
		_tmpAmount=_PPaidAmt*(-1);
		IF (_RefType='PURC') THEN
			IF(_tmpAmount>0) THEN
				SELECT INTO _spret * FROM spPurc_Pay(_RefNo, _tmpAmount*(-1), _PayDate);
			ELSE
				SELECT INTO _spret * FROM spPurc_Pay(_RefNo, _tmpAmount, _PayDate);
			END IF;
		ELSIF (_RefType='PRET') THEN
			SELECT INTO _spret * FROM spPret_VoidContra(_RefNo);
		ELSIF (_RefType='FCN') THEN
			SELECT INTO _spret * FROM spFCN_VoidContra(_RefNo);

		ELSIF (_RefType='DP') THEN
			IF (_PayType='DP') THEN
				SELECT INTO _DPID dpid FROM dp WHERE paidno=_PayNo;
				SELECT INTO _spret * FROM spDP_Void(_DPID);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSE
				SELECT INTO _spret * FROM spDPBal_Update(CAST(_RefNo AS BIGINT), _PPaidAmt*(-1), _PPaidAmtBC*(-1));
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END IF;

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cPPaid;

	OPEN cPayD FOR
	SELECT PayDMtd, CashID, PayDAmt, RefNo,cbgid
	FROM PayD WHERE PayNo=_PayNo;
	LOOP
		FETCH cPayD INTO _PayDMtd, _CashID, _PayDAmt, _PayD_RefNo,_CBGID;
		EXIT WHEN NOT FOUND;
		
		IF (_PayDMtd='OVER') THEN
			SELECT INTO _spret * FROM spOverPay_VoidContra(_PayD_RefNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF ((_PayDMtd='CASH') OR (_PayDMtd='BTRF') OR (_PayDMtd='DBC') OR (_PayDMtd='CRC')) THEN
			SELECT INTO _spret * FROM spCashA_Void_byRef('PAY', _PayNo, _CashID, null);
			IF _spret.ErrNo	<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSIF((_PayDMtd='CHQ') OR (_PayDMtd='GIRO')) THEN
			
			IF (_CBGID IS NULL) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='Tidak ada CBG';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			SELECT INTO _inout,_CashCBG,_CbgAmt,_ClearDate inout,srccashid,cbgamt,clrdate from cbg where cbgid=_CBGID;

			IF (_ClearDate IS NOT NULL) THEN
				_retval.ErrNo:=101;
				_retval.ErrMsg:='CBG Sudah di cairkan!!Unclear kan dulu CBG nya!!(spPay_Void)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_inout='I') THEN
				SELECT INTO _spret * FROM spCashA_Void_byRef('CBG', _CBGID, _CashCBG, null);
				IF _spret.ErrNo	<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				UPDATE cbg SET active=true, useddate=NULL WHERE cbgid=_CBGID;
			ELSIF(_inout='O') THEN
				/*SELECT INTO _outbal outbal FROM cash WHERE cashid=_CashCBG;
				total:=_outbal-_CbgAmt;

				UPDATE cash set outbal=total where cashid=_CashCBG;*/
				UPDATE cbg SET active=true, useddate=NULL WHERE cbgid=_CBGID;
			END IF;
		END IF;
	END LOOP;
	CLOSE cPayD;
	IF _PayType='PAY' THEN
		SELECT INTO _spret * FROM spBPA_Void_byRef('PAY', _PayNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;
	IF isAccInstalled() THEN
		SELECT into _spret * from spJnl_Void_byRef('PAY',_PayNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE PayD (spPay_Void)';
	DELETE FROM PayD WHERE PayNo=_PayNo;

	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error DELETE PPaid (spPay_Void)';
	DELETE FROM PPaid WHERE PayNo=_PayNo;
	
	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error DELETE Pay (spPay_Void)';
	DELETE FROM Pay WHERE PayNo=_PayNo;


	_retval:=(0, 'OK');	/*--  HAPPY ENDING :)*/
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStTr_New (
	_StTrNo VARCHAR(15)
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_WhID1			VARCHAR(3);
	_WhID2			VARCHAR(3);
	_StTrDate		DATE;
	_StTrDNo		INT;
	_Cost			NUMERIC(19,4);

	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;

	_StockANo1		VARCHAR(15);
	_StockANo2		VARCHAR(15);
	_StockADNo		INT2;
	_StockANote		VARCHAR(50);
	_ItemID			VARCHAR(20);
	_PID			VARCHAR(25);
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_QtyX			NUMERIC(15,4);

	_tmpQty			NUMERIC(15,4);
	_tmpQtyX		NUMERIC(15,4);
	
	cStTrD			REFCURSOR;

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);
	_LockDate		DATE;

	_IsDraft		BOOLEAN;

	_StockQty		NUMERIC(19,4);
	_StockSubtotal		NUMERIC(19,4);
	_Subtotal		NUMERIC(19,4);
	_Conv			NUMERIC(19,4);
BEGIN
	_retval:=(0, 'OK');
	_DeptID:=NULL;
	_PrjID:=NULL;

	SELECT INTO 
		_PerID, _BranchID, _WhID1, _WhID2, _StTrDate, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _IsDraft
		perid, branchid, whid1, whid2, sttrdate, crtby, crtdate, updby, upddate, isdraft 
	FROM sttr WHERE sttrno=_StTrNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Mutasi-Stock #'||COALESCE(_StTrNo, '<kosong>')||' tidak ditemukan ! (spStTr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_IsDraft IS NOT NULL AND _IsDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='STTR';
	IF(_LockDate IS NOT NULL AND _LockDate>_StTrDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Tanggal mutasi stok melebihi batas tanggal kunci transaksi ! (spStTr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_WhID1=_WhID2) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Gudang asal dan gudang tujuan tidak boleh sama ! (spStTr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_StockANote='Mutasi-Stock #'||_StTrNo;

	_StockANo1:=fTransNoGet('STOCKA');
	IF (_StockANo1 IS NULL) THEN
		_retval.ErrNo:=2;
		_retval.ErrMsg:='ERROR getting StockANo (spStTr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spStockA_New (
		_StockANo1, NULL, 'STTR', _StTrDate, 'O', 'STTR', 
		_StTrNo, _StockANote,true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
		_PerID, _BranchID, _DeptID, _PrjID
	);

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_StockANo2:=fTransNoGet('STOCKA');
	IF (_StockANo2 IS NULL) THEN
		_retval.ErrNo:=2;
		_retval.ErrMsg:='ERROR getting StockANo (spStTr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;	
	SELECT INTO _spret * FROM spStockA_New(
		_StockANo2, NULL, 'STTR', _StTrDate, 'I', 'STTR',
		_StTrNo, _StockANote,true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
		_PerID, _BranchID, _DeptID, _PrjID
	);

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_StockADNo:=0;
	OPEN cStTrD FOR
		SELECT d.sttrdno, d.itemid, d.pid, d.qty, d.unit, d.qtyx
		FROM sttrd d 
		JOIN item i ON i.itemid=d.itemid
		WHERE d.sttrno=_StTrNo AND i.isstock=true;
	LOOP
		FETCH cStTrD INTO _StTrDNo, _ItemID, _PID, _Qty, _Unit, _QtyX;
		EXIT WHEN NOT FOUND;

		IF _PID is NULL THEN
			_PID:='';
		END IF;

		IF _Qty IS NULL THEN
			_Qty:=0;
		END IF;

		IF _QtyX IS NULL THEN
			_QtyX:=0;
		END IF;

		_Conv:=fItemUnit_Conv(_ItemID, _Unit);
		_StockADNo=_StockADNo+1;
		SELECT INTO _Cost * FROM fUnitCost(_ItemID, _WhID1, _PID, _Unit);
		IF (_Cost IS NULL) THEN
			_retval.ErrNo:=50003;
			_retval.ErrMsg:='Error menghitung HPP! Item:'||COALESCE(_ItemID, '<kosong>')||', Wh:'||COALESCE(_WhID1, '<kosong>')||', Batch:'||COALESCE(_PID, '<kosong>')||', Unit:'||COALESCE(_Unit, '<kosong>')||' (fUnitCost)(spStTr_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error UPDATE StTrD:Cost (spStTr_New)';
		UPDATE StTrD SET Cost=_Cost, conv=_Conv WHERE StTrNo=_StTrNo AND StTrDNo=_StTrDNo;

		_tmpQty:=_Qty*-1;
		_tmpQtyX:=_QtyX*-1;
		_StockADNo=_StTrDNo;

		SELECT INTO _StockQty, _StockSubtotal qty, subtotal FROM stock WHERE itemid=_ItemID AND whid=_WhID1 AND pid=_PID;
		IF (FOUND AND _StockQty>0) THEN
			_Subtotal:=((_tmpQty*_Conv)/_StockQty)*_StockSubtotal;
		ELSE
			_Subtotal:=_tmpQty*_Cost;
		END IF;

		SELECT INTO _spret * FROM spStockAD_New(
			_StTrDate, 'STTR', _StockANo1, _StockADNo, 
			_WhID1, _ItemID, _PID, _tmpQty, _Unit, _tmpQtyX, _Cost, 
			_DeptID, _PrjID, true, _StTrNo, _Subtotal
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmpQty:=_Qty;
		_tmpQtyX:=_QtyX;

		SELECT INTO _spret * FROM spStockAD_New(
			_StTrDate, 'STTR', _StockANo2, _StockADNo, 
			_WhID2, _ItemID, _PID, _tmpQty, _Unit, _tmpQtyX, _Cost, 
			_DeptID, _PrjID, true, _StTrNo, _Subtotal*(-1)
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cStTrD;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spStTr_Void(__StTrNo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_SttrDate	DATE;
	_LockDate	DATE;
	_StTrNo		VARCHAR(15);

	_IsDraft	BOOLEAN;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _StTrNo, _SttrDate, _IsDraft 
		sttrno, sttrdate, isdraft 
	FROM sttr WHERE sttrno=__StTrNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Mutasi-Barang #'||COALESCE(_StTrNo, '<kosong>')||' tidak ditemukan (spStTr_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_IsDraft IS NOT NULL AND _IsDraft=TRUE) THEN
		DELETE FROM sttrd WHERE sttrno=__StTrNo;
		DELETE FROM sttr WHERE sttrno=__StTrNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='STTR';
	IF(_LockDate is not null AND _LockDate>_StTrDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Mutasi Stock Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spStockA_Void_byRef('STTR', _StTrNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error DELETE StTrD (spStTr_Void)';
	DELETE FROM StTrD WHERE StTrNo=_StTrNo;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error DELETE StTr (spStTr_Void)';
	DELETE FROM StTr WHERE StTrNo=_StTrNo;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='DELETE SUCCESS!!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spUnitDelete (
	_ItemID		VARCHAR,
	_Unit		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_Unit1		VARCHAR(5);
	_Unit2		VARCHAR(5);
	_Unit3		VARCHAR(5);
	_Conv2		DECIMAL(19,4);
	_Conv3		DECIMAL(19,4);
	_StockADNo	VARCHAR;
	_newUnitDesc	VARCHAR;
BEGIN
	SELECT INTO _StockADNo StockADNo FROM StockAD WHERE ItemID=_ItemID AND Unit=_Unit;
	IF FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Masih ada transaksi yang menggunakan Unit tersebut (spUnitDelete)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT INTO _Unit1, _Unit2,_Unit3,_Conv2,_Conv3 Unit1, Unit2, Unit3,Conv2,Conv3
	FROM Item WHERE ItemID=_ItemID;
	IF (_Unit=_Unit3) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Update Item:Unit! (spUnitDelete)';
		UPDATE Item SET Unit3=NULL, Conv3=NULL WHERE ItemID=_ItemID;
		_Unit3:=null;
	ELSE
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Error Update Item:Unit! (spUnitDelete)';
		UPDATE Item SET Unit2=NULL, Conv2=NULL WHERE ItemID=_ItemID;
		_Unit2:=null;
	END IF;
	
	_newUnitDesc:=_Unit1;
	IF ((_Unit2 IS NOT NULL) AND (_Conv2>0)) THEN
		_newUnitDesc=_newUnitDesc||', '||_Unit2||'/'||CAST(CAST(_Conv2 AS FLOAT) AS VARCHAR);
	END IF;
	IF ((_Unit3 IS NOT NULL) AND (_Conv3>0)) THEN
		_newUnitDesc=_newUnitDesc||', '||_Unit3||'/'||CAST(CAST(_Conv3 AS FLOAT) AS VARCHAR);
	END IF;

	UPDATE Item SET Unitdesc=_newUnitDesc WHERE ItemID=_ItemID;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='UPDATE SUCCESS!!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spUsrAuth (
	__UsrID		VARCHAR,
	__ObjID		VARCHAR,
	__AcsType	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_AcsType	VARCHAR;
	_enabled 	BOOLEAN;
BEGIN
	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';

	SELECT INTO 
		_AcsType,_enabled AcsType,enabled
	FROM fUsrPrv(__UsrID) WHERE ObjID=__ObjID AND AcsType=__AcsType;


	IF (NOT FOUND OR _enabled IS NULL OR _enabled=false) then
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='User tidak berhak, Akses ditolak! (spUsrAuth)';
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spUsrAuthDate (
	__UsrID		VARCHAR,
	__ObjID		VARCHAR,
	__AcsType	VARCHAR,
	__date		DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_AcsType	VARCHAR;
	_AcsVal		VARCHAR;
	_periode	INT;
	_enabled	BOOLEAN;
	_found		BOOLEAN;
	_lastdate	DATE;
BEGIN
	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';

	SELECT INTO 
		_AcsType, _enabled, _AcsVal
		AcsType, enabled, AcsVal
	FROM fUsrPrv(__UsrID) 
	WHERE ObjID=__ObjID AND AcsType=__AcsType;

	_found:=FOUND;
	_periode:=-1;
	IF (__date IS NOT NULL AND _AcsVal IS NOT NULL AND _AcsVal<>'') THEN
		_periode:=CAST(_AcsVal AS INT);
		_lastdate:=CAST(now() AS DATE)-_periode;
	END IF;

	IF (
			NOT _found 
			OR _enabled IS NULL OR _enabled=false 
			OR (_periode<0 AND __date<=_lastdate) OR (_periode>0 AND __date>=_lastdate)
		) then
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='User tidak berhak, Akses ditolak! (spUsrAuth)';
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spUsr_Remove (
	_UsrID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error DELETE UsrGrp (spUsr_Remove)';
	DELETE FROM UsrGrp WHERE UsrID=_UsrID;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error DELETE Usr (spUsr_Remove)';
	DELETE FROM Usr WHERE UsrID=_UsrID;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='DELETE SUCCESS!!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spWhID_Update (
	_oldWhID	VARCHAR,
	_newWhID	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_tempWhID	VARCHAR(3);
	_WhID1		VARCHAR(3);
	_WhID2		VARCHAR(3);
	_stage		SMALLINT;
	_state		BOOLEAN;
BEGIN
	IF (_oldWhID=_newWhID) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Kode Gudang lama tidak boleh sama dengan kode baru (spWhID_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	SELECT WhID FROM Wh WHERE WhID=_newWhID;
	IF FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Kode Gudang baru sudah ada ! (spWhID_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_tempWhID:='X_X';
	_WhID1:=_oldWhID;
	_WhID2:=_tempWhID;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error Insert Temporary Gudang! (spWhID_Update)';
	INSERT INTO Wh(WhID, WhDesc) VALUES (_tempWhID, 'Temp WhID');
	_retval.ErrNo:=0;
	_retval.ErrMsg:='INSERT SUCCESS!';

	_stage:=1;
	_state:=true;

	WHILE ((_stage=1) OR (_state=true)) LOOP
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Error UPDATE Dept(1), stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Dept SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50005;
		_retval.ErrMsg:='Error UPDATE Dept(2), stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Dept SET WhIDP=_WhID2 WHERE WhIDP=_WhID1;
		
		_retval.ErrNo:=50006;
		_retval.ErrMsg:='Error UPDATE Dept(3), stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Dept SET WhIDS=_WhID2 WHERE WhIDS=_WhID1;

		_retval.ErrNo:=50007;
		_retval.ErrMsg:='Error UPDATE Canvas, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Canvas SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50008;
		_retval.ErrMsg:='Error UPDATE StockAD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE StockAD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50009;
		_retval.ErrMsg:='Error UPDATE StockA, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE StockA SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50010;
		_retval.ErrMsg:='Error UPDATE StockBal, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE StockBal SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50011;
		_retval.ErrMsg:='Error UPDATE Stock, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Stock SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50012;
		_retval.ErrMsg:='Error UPDATE StTr(1), stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE StTr SET WhID1=_WhID2 WHERE WhID1=_WhID1;

		_retval.ErrNo:=50013;
		_retval.ErrMsg:='Error UPDATE StTrD(2), stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE StTr SET WhID2=_WhID2 WHERE WhID2=_WhID1;

		_retval.ErrNo:=50014;
		_retval.ErrMsg:='Error UPDATE CN, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE PRET SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50015;
		_retval.ErrMsg:='Error UPDATE CND, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE PRETD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50016;
		_retval.ErrMsg:='Error UPDATE Purc, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Purc SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50017;
		_retval.ErrMsg:='Error UPDATE PurcD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE PurcD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50018;
		_retval.ErrMsg:='Error UPDATE PRcv, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE PRcv SET WhID=_WhID2 WHERE WhID=_WhID1;
	
		_retval.ErrNo:=50019;
		_retval.ErrMsg:='Error UPDATE PRcvD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE PRcvD SET WhID=_WhID2 WHERE WhID=_WhID1;
	
		_retval.ErrNo:=50020;
		_retval.ErrMsg:='Error UPDATE PReq, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE PReq SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50021;
		_retval.ErrMsg:='Error UPDATE PRD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE PRD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50022;
		_retval.ErrMsg:='Error UPDATE PO, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE PO SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50023;
		_retval.ErrMsg:='Error UPDATE POD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE POD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50024;
		_retval.ErrMsg:='Error UPDATE DN, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE SRET SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50025;
		_retval.ErrMsg:='Error UPDATE DND, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE SRETD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50026;
		_retval.ErrMsg:='Error UPDATE Sale, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Sale SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50027;
		_retval.ErrMsg:='Error UPDATE SaleD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE SaleD SET WhID=_WhID2 WHERE WhID=_WhID1;
		/**
		_retval.ErrNo:=50028;
		_retval.ErrMsg:='Error UPDATE Cons, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Cons SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50029;
		_retval.ErrMsg:='Error UPDATE ConsD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE ConsD SET WhID=_WhID2 WHERE WhID=_WhID1;
		**/
		_retval.ErrNo:=50030;
		_retval.ErrMsg:='Error UPDATE DO, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE DEO SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50031;
		_retval.ErrMsg:='Error UPDATE DOD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE DEOD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50032;
		_retval.ErrMsg:='Error UPDATE SO, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE SO SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50033;
		_retval.ErrMsg:='Error UPDATE SOD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE SOD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50034;
		_retval.ErrMsg:='Error UPDATE BoM, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE BoM SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50035;
		_retval.ErrMsg:='Error UPDATE BoMD, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE BoMD SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50036;
		_retval.ErrMsg:='Error UPDATE Mfg, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Mfg SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50037;
		_retval.ErrMsg:='Error UPDATE MfgM, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE MfgM SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=50038;
		_retval.ErrMsg:='Error UPDATE MfgP, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE MfgP SET WhID=_WhID2 WHERE WhID=_WhID1;

		_retval.ErrNo:=0;
		_retval.ErrMsg:='ALL UPDATE SUCCESS!';

		IF (_stage=2) THEN
			EXIT;
		END IF;

		_retval.ErrNo:=50039;
		_retval.ErrMsg:='Error UPDATE Wh:(new)WhID, stage:'||CAST(_stage AS VARCHAR)||' (spWhID_Update)';
		UPDATE Wh SET WhID=_newWhID WHERE WhID=_oldWhID;
		_retval.ErrNo:=0;
		_retval.ErrMsg:='UPDATE SUCCESS!';

		_WhID1:=_tempWhID;
		_WhID2:=_newWhID;
		_stage:=2;
	END LOOP;

	_retval.ErrNo:=50040;
	_retval.ErrMsg:='Error DELETE (old)Wh (spWhID_Update)';
	DELETE FROM Wh WHERE WhID=_tempWhID;
	_retval.ErrNo:=0;
	_retval.ErrMsg:='DELETE SUCCESS!';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCanvasAnalyze(
	_CanvasID	VARCHAR(5),
	_StartDate	DATE,
	_EndDate	DATE
) RETURNS SETOF tCanvas AS 
$BODY$
DECLARE
	_retval		tCanvas%ROWTYPE;
	cCanvasA	refcursor;
	_WhID		VARCHAR(3);
BEGIN
	SELECT INTO _WhID WhID FROM Canvas WHERE CanvasID=_CanvasID;
	OPEN cCanvasA FOR
		SELECT
		d.ItemID,
		i.itemdesc,
		fqty1desc(d.ItemID,fStockBal_StartDate(d.ItemID, _WhID, '', _StartDate)) AS Salwal,
		fqty1desc(d.ItemID,SUM(COALESCE(CASE WHEN h.StockAType='STTR' AND h.InOut='I' THEN fQtyZToQty1(d.ItemID, d.Qty, d.Unit) END, 0))) ,
		fqty1desc(d.ItemID,SUM(COALESCE(CASE WHEN h.StockAType='SALE' THEN fQtyZToQty1(d.ItemID, d.Qty, d.Unit) END, 0))) ,
		fqty1desc(d.ItemID,SUM(COALESCE(CASE WHEN h.StockAType='CONS' THEN fQtyZToQty1(d.ItemID, d.Qty, d.Unit) END, 0))) ,
		fqty1desc(d.ItemID,SUM(COALESCE(CASE WHEN h.StockAType='SRET' THEN fQtyZToQty1(d.ItemID, d.Qty, d.Unit) END, 0))) AS Retur,
		fqty1desc(d.ItemID,SUM(COALESCE(CASE WHEN h.StockAType='STTR' AND h.InOut='O' THEN fQtyZToQty1(d.ItemID, d.Qty, d.Unit) END, 0))) ,
		fqty1desc(d.ItemID,fStockBal_EndDate(d.ItemID, _WhID,'', _EndDate)) 
		FROM StockAD d 
		JOIN StockA h ON d.StockANo=h.StockANo 
		JOIN item i ON d.itemid=i.itemid
		WHERE d.WhID=_WhID 
		AND h.StockADate>=fNoTime(COALESCE(_StartDate,date('01-01-01'))) 
		AND h.StockADate<=fDate2359(COALESCE(_EndDate,date('12-30-9999')))
		GROUP BY d.ItemID,i.itemdesc;
	LOOP
		FETCH cCanvasA INTO _retval.ItemID, _retval.ItemDesc, _retval.QtyStart, _retval.QtyLoad, _retval.QtySold, _retval.QtySellIn, _retval.QtyMisc, _retval.QtyUnLoad,  _retval.QtyEnd;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cCanvasA;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION spItemMinMax_Update (
	_ItemID		VARCHAR,
	__LeadTime	SMALLINT,
	_QtyMin		NUMERIC,
	_QtyMax		NUMERIC,
	_QtyReOrder	NUMERIC
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_LeadTime 	SMALLINT;
BEGIN
	_LeadTime=__LeadTime;
	IF (_LeadTime IS NULL) THEN
		_LeadTime=0;
	END IF;

	_retval.ErrNo:=50001;
	_retval.ErrMsg:='Error UPDATE Item (spItemMinMax_Update)';
	UPDATE Item SET LeadTime=_LeadTime, QtyMin=_QtyMin, QtyMax=_QtyMax, QtyReOrder=_QtyReOrder
	WHERE ItemID=_ItemID;

	_retval:=(0, 'UPDATE SUKSES!');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBegOfMonth (
	_Date	DATE
) RETURNS DATE AS
$BODY$
DECLARE
	_day INT;
BEGIN
	_day:=date_part('day',_Date);
	IF (_day=1) THEN
		RETURN _Date;
	ELSE
		RETURN _Date - (_day-1);
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fbpacard(
	_type		VARCHAR,
	_branchid	VARCHAR,
	_bpid		VARCHAR,
	_startdate	DATE,
	_enddate	DATE
) RETURNS SETOF bparpt AS
$BODY$
DECLARE
	_retval		bparpt%ROWTYPE;

	_BPANo		VARCHAR(15);
	_BPADate	DATE;
	__branchid	VARCHAR(10);
	_BPAType	VARCHAR(1);
	_RefType	VARCHAR(5);
	_RefNo		VARCHAR(15);
	_BPANote	VARCHAR;
	_DC		VARCHAR(1);
	_Debit		DECIMAL(19,4);
	_Credit		DECIMAL(19,4);
	_APBalance	DECIMAL(19,4);

	_BPAAmt		DECIMAL(19,4);

	_Month		SMALLINT;
	_PYear		INTEGER;
	_LastMonthBal	DECIMAL(19,4);
	_YesterdayBal	DECIMAL(19,4);

	cBPA		REFCURSOR;
	_crcid		VARCHAR;
	cCRc		REFCURSOR;
	_CheckAmt	NUMERIC(19,4);

BEGIN
	_Month:=date_part('month', _StartDate);
	_PYear:=date_part('year', _StartDate);
	__branchid:=_branchid;

	OPEN cCrc FOR
		SELECT crcid FROM crc;
	LOOP
		FETCH cCrc INTO _crcid;
		EXIT WHEN NOT FOUND;

		IF (_Type='HUTANG') THEN
			IF (_Month=1) THEN
				SELECT INTO _LastMonthBal BPCBal00 
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=2) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01 
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=3) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02 
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=4) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03 
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=5) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04 
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=6) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04+BPCBal05 
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=7) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04+BPCBal05+BPCBal06
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=8) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04+BPCBal05+BPCBal06
				+BPCBal07
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=9) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04+BPCBal05+BPCBal06
				+BPCBal07+BPCBal08
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=10) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04+BPCBal05+BPCBal06
				+BPCBal07+BPCBal08+BPCBal09
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=11) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04+BPCBal05+BPCBal06
				+BPCBal07+BPCBal08+BPCBal09+BPCBal10
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=12) THEN
				SELECT INTO _LastMonthBal BPCBal00+BPCBal01+BPCBal02+BPCBal03+BPCBal04+BPCBal05+BPCBal06
				+BPCBal07+BPCBal08+BPCBal09+BPCBal10+BPCBal11
				FROM BPCBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			END IF;

			IF (_LastMonthBal IS NULL) THEN
				_LastMonthBal:=0;
			END IF;

			SELECT INTO _YesterdayBal (_LastMonthBal+COALESCE(SUM(BPAAmt), 0)) FROM BPA
			WHERE BPID=_BPID AND BPADate>=fBegOfMonth(_StartDate)
			AND BPADate<fNoTime(_StartDate) 
			AND (RefType='PURC' OR RefType='PAY' OR RefType='PRET' OR (RefType='FCN' AND BPAType='C'))
			AND (__branchid IS NULL OR BPA.branchid=__branchid)
			AND CRCID=_CrcID;

			_APBalance:=_YesterdayBal;

			_retval.bpano:=NULL;
			_retval.branchid:=__branchid;
			_retval.bpadate:=fNoTime(_StartDate);
			_retval.bpatype:=NULL;
			_retval.reftype:='Saldo';
			_retval.refno:=NULL;
			_retval.bpanote:='Saldo';
			_retval.dc:='B';
			_retval.debit:=NULL;
			_retval.credit:=NULL;
			_retval.bpbal:=_APBalance;
			_retval.crcid:=_CrcID;

			SELECT INTO _CheckAmt coalesce(sum(BPAAmt),0)
			FROM BPA
			WHERE BPID=_BPID AND BPADate>=fNoTime(_StartDate) AND BPADate<=fDate2359(_EndDate)
			AND (RefType='PURC' OR RefType='PAY' OR RefType='PRET' OR (RefType='FCN' AND BPAType='C'))
			AND CrcID=_CrcID
			AND (_branchid IS NULL OR BPA.branchid=_branchid);

			IF(_CheckAmt<>0) THEN
				RETURN NEXT _retval;
			END IF;

			OPEN cBPA FOR
				SELECT BPANo, BPADate, BPAType, RefType, RefNo, BPANote, BPAAmt
				FROM BPA
				WHERE BPID=_BPID AND BPADate>=fNoTime(_StartDate)
				AND BPADate<=fDate2359(_EndDate)
				AND (RefType='PURC' OR RefType='PAY' OR RefType='PRET' OR (RefType='FCN' AND BPAType='C'))
				AND CrcID=_CrcID
				AND (_branchid IS NULL OR BPA.branchid=_branchid) 
				ORDER BY BPADate,BPANo;
			LOOP
				FETCH cBPA INTO _BPANo, _BPADate, _BPAType, _RefType, _RefNo, _BPANote, _BPAAmt;
				EXIT WHEN NOT FOUND;
				
				IF (_BPAAmt>=0) THEN
					_Debit:=_BPAAmt;
					_Credit:=NULL;
					_DC:='P';
				ELSE
					_Debit:=NULL;
					_Credit:=ABS(_BPAAmt);
					_DC:='C';
				END IF;
				
				_APBalance:=_APBalance+_BPAAmt;

				_retval.bpano:=_BPANo;
				_retval.branchid:=__branchid;
				_retval.bpadate:=_BPADate;
				_retval.bpatype:=_BPAType;
				_retval.reftype:=_RefType;
				_retval.refno:=_RefNo;
				_retval.bpanote:=_BPANote;
				_retval.dc:=_DC;
				_retval.debit:=_Debit;
				_retval.credit:=_Credit;
				_retval.bpbal:=_APBalance;
				_retval.crcid:=_crcid;
				RETURN NEXT _retval;
			END LOOP;
			CLOSE cBPA;
		ELSIF (_Type='PIUTANG') THEN
			IF (_Month=1) THEN
				SELECT INTO _LastMonthBal BPDBal00 
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=2) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01 
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=3) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02 
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=4) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03 
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=5) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04 
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=6) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04+BPDBal05 
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=7) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04+BPDBal05+BPDBal06
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=8) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04+BPDBal05+BPDBal06
				+BPDBal07
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=9) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04+BPDBal05+BPDBal06
				+BPDBal07+BPDBal08
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=10) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04+BPDBal05+BPDBal06
				+BPDBal07+BPDBal08+BPDBal09
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=11) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04+BPDBal05+BPDBal06
				+BPDBal07+BPDBal08+BPDBal09+BPDBal10
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			ELSIF (_Month=12) THEN
				SELECT INTO _LastMonthBal BPDBal00+BPDBal01+BPDBal02+BPDBal03+BPDBal04+BPDBal05+BPDBal06
				+BPDBal07+BPDBal08+BPDBal09+BPDBal10+BPDBal11
				FROM BPDBal WHERE BPID=_BPID AND PYear=_PYear AND CrcID=_CrcID;
			END IF;

			IF (_LastMonthBal IS NULL) THEN
				_LastMonthBal:=0;
			END IF;

			SELECT INTO _YesterdayBal (_LastMonthBal+COALESCE(SUM(BPAAmt), 0)) FROM BPA
			WHERE BPID=_BPID AND BPADate>=fBegOfMonth(_StartDate)
			AND BPADate<fNoTime(_StartDate)
			AND (RefType='SALE' OR RefType='RCV' OR RefType='SRET' 
			OR (RefType='FCN' AND BPAType='P'))
			AND (_branchid IS NULL OR BPA.branchid=_branchid) AND CrcID=_CrcID;

			_APBalance:=_YesterdayBal;

			_retval.bpano:=NULL;
			_retval.branchid:=__branchid;
			_retval.bpadate:=fNoTime(_StartDate);
			_retval.bpatype:=NULL;
			_retval.reftype:='Saldo';
			_retval.refno:=NULL;
			_retval.bpanote:='Saldo';
			_retval.dc:='B';
			_retval.debit:=NULL;
			_retval.credit:=NULL;
			_retval.bpbal:=_APBalance;
			_retval.crcid=_crcid;

			SELECT INTO _CheckAmt coalesce(sum(BPAAmt),0)
			FROM BPA
			WHERE BPID=_BPID AND BPADate>=fNoTime(_StartDate) AND BPADate<=fDate2359(_EndDate)
			AND (RefType='SALE' OR RefType='RCV' OR RefType='SRET' OR (RefType='FCN' AND BPAType='P'))
			AND CrcID=_CrcID
			AND (_branchid IS NULL OR BPA.branchid=_branchid);

			IF(_CheckAmt<>0) then
				RETURN NEXT _retval;
			END IF;

			OPEN cBPA FOR
				SELECT BPANo, BPADate, BPAType, RefType, RefNo, BPANote, BPAAmt
				FROM BPA
				WHERE BPID=_BPID AND BPADate>=fNoTime(_StartDate)
				AND BPADate<=fDate2359(_EndDate)
				AND (RefType='SALE' OR RefType='RCV' OR RefType='SRET' OR (RefType='FCN' AND BPAType='P'))
				AND CrcID=_CrcID
				AND (_branchid IS NULL OR BPA.branchid=_branchid)
				ORDER BY BPADate,BPANo;
			LOOP
				FETCH cBPA INTO _BPANo, _BPADate, _BPAType, _RefType, _RefNo, _BPANote, _BPAAmt;
				EXIT WHEN NOT FOUND;

				IF (_BPAAmt>=0) THEN
					_Debit:=_BPAAmt;
					_Credit:=NULL;
					_DC:='P';
				ELSE
					_Debit:=NULL;
					_Credit:=ABS(_BPAAmt);
					_DC:='C';
				END IF;

				_APBalance:=_APBalance+_BPAAmt;

				_retval.bpano:=_BPANo;
				_retval.branchid:=__branchid;
				_retval.bpadate:=_BPADate;
				_retval.bpatype:=_BPAType;
				_retval.reftype:=_RefType;
				_retval.refno:=_RefNo;
				_retval.bpanote:=_BPANote;
				_retval.dc:=_DC;
				_retval.debit:=_Debit;
				_retval.credit:=_Credit;
				_retval.bpbal:=_APBalance;
				_retval.crcid:=_CrcID;
				RETURN NEXT _retval;
			END LOOP;
			CLOSE cBPA;
		END IF;
	END LOOP;
	CLOSE cCrc;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCopyRpt (
	_rptidAsal	VARCHAR,
	_rptidTujuan	VARCHAR,
	_rptKeterangan	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_found		VARCHAR;
	_retval		SPStatus%ROWTYPE;
	_paperwidth	INT2;
	_paperheight	INT2;
	_pagecols	INT2;
	_pagerows	INT2;
	_marginleft	INT2;
 	_marginright	INT2;
	_margintop	INT2;
	_marginbottom	INT2;
	_font		VARCHAR;
	_size		INT2;
	_sendff		BOOLEAN;
	_rvarid		VARCHAR;
	_rvardesc	VARCHAR;
	_rvarfield	VARCHAR;
	_rvaralign	VARCHAR;
	_rowno		INT;
	_bandtype	VARCHAR;
	_bandtext	VARCHAR;
	cRvar		REFCURSOR;
	cRband		REFCURSOR;
BEGIN
	SELECT INTO _found rptid FROM rpt WHERE rptid=_rptidAsal;

	IF NOT FOUND THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Report tidak ditemukan';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 

	SELECT INTO _found rptid FROM rpt WHERE rptid=_rptidTujuan;
	IF NOT FOUND THEN
		SELECT INTO _paperwidth, _paperheight, _pagecols, _pagerows, _marginleft, _marginright, _margintop, _marginbottom, _font, _size, _sendff 
			paperwidth, paperheight, pagecols, pagerows, marginleft, marginright, margintop, marginbottom, font, fontsize, sendff
		FROM rpt 
		WHERE rptid=_rptidAsal;

		INSERT INTO rpt (rptid, rptdesc, paperwidth, paperheight, pagecols, pagerows, marginleft, marginright, margintop, marginbottom, font, fontsize, sendff) 
		VALUES(_rptidTujuan, _rptketerangan, _paperwidth, _paperheight, _pagecols, _pagerows, _marginleft, _marginright, _margintop, _marginbottom, _font, _size, _sendff);

		OPEN cRvar FOR 
			SELECT rvarid, rvardesc, rvarfield, rvaralign 
			FROM rvar 
			WHERE rptid=_rptidAsal;
		LOOP
			FETCH cRvar INTO _rvarid,_rvardesc,_rvarfield,_rvaralign;
			EXIT WHEN NOT FOUND;

			INSERT INTO rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) VALUES(_rptidTujuan,_rvarid,_rvardesc,_rvarfield,_rvaralign);
		END LOOP;
		CLOSE cRvar;

		OPEN cRband FOR 
			SELECT rowno, bandtype, bandtext 
			FROM rband 
			WHERE rptid=_rptidAsal;
		LOOP
			FETCH cRband INTO _rowno,_bandtype,_bandtext;
			EXIT WHEN NOT FOUND;

			INSERT INTO rband(rptid,rowno,bandtype,bandtext) VALUES(_rptidTujuan,_rowno,_bandtype,_bandtext);
		END LOOP;
		CLOSE cRband;
	ELSE
		_retval.ErrNo:=2;
		_retval.ErrMsg:='Report ID sudah ada';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fDate2359 (
	_Date	DATE
) RETURNS DATE AS
$BODY$
BEGIN
	RETURN _Date + time ' 23:59:59';
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fDateTime_Compose(
	_Year	INT,
	_Month	INT,
	_Date	INT
) RETURNS DATE AS 
$BODY$
DECLARE
	_tmp		VARCHAR(10);
	_retval		DATE;
BEGIN
	_tmp=CAST(_Year AS VARCHAR)||'-'||CAST(_Month AS VARCHAR)||'-'||CAST(_Date AS VARCHAR);
	_retval=CAST(_tmp AS DATE);

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fgetday (
	_date	DATE
) RETURNS SETOF INT4 AS
$BODY$
DECLARE
	_retval INT;
BEGIN
	SELECT INTO _retval (_date-DATE '1872-01-01')%7;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItemDesc (
	_ItemID	VARCHAR(20)
) RETURNS VARCHAR(50) AS 
$BODY$
DECLARE
	_retval		VARCHAR(50);
BEGIN
	SELECT INTO _retval
		ItemDesc 
	FROM Item 
	WHERE ItemID=_ItemID;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



/** untuk posting Transaksi CashA yang di-ENTRY USER dari program 
	 i.e.: untuk penyesuian-kas
*/
CREATE OR REPLACE FUNCTION spCashA_New_NoIns (
	_CashANo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_CashADate	DATE;
	_Amount		NUMERIC(19,4);
	_cashid		VARCHAR;
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_CrtBy		VARCHAR(10);
	_CrtDate	TIMESTAMP;
	_UpdBy		VARCHAR(10);
	_UpdDate	TIMESTAMP;
	_Count		INT;
	_JnlNo		VARCHAR(15);
	_CashAType	VARCHAR(5);

	_CashANote	VARCHAR(255);
	_JnlNote	VARCHAR(255);
	_PerID		VARCHAR(5);
	_BranchID	VARCHAR(10);
	_DeptID		VARCHAR(5);
	_PrjID		VARCHAR(5);
	_ExcRate	NUMERIC(19,4);
	_JnlType	VARCHAR;
	_LockDate	DATE;
	_CrcID		VARCHAR;
	_FisRate	NUMERIC(19,4);

	_AccCash	VARCHAR;
	_AccDefa	VARCHAR;
	_InOut		CHAR(1);
BEGIN

	SELECT INTO 
		_CashID, _CashADate, _Amount, _CashANote, _CashAType, _CrtBy,_CrtDate,_UpdBy,_UpdDate,_CashANote,_ExcRate,_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _FisRate
		cashid, CashADate, Amount, CashANote, CashAType, CrtBy, CrtDate, UpdBy, UpdDate, CashANote,excrate,PerID, BranchID, DeptID, PrjID, CrcID, FisRate
	FROM CashA 
	WHERE CashANo=_CashANo;

		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error EXEC spCashBal_Update (spCashA_New_NoIns)';

	IF (_Amount>0) THEN
		_InOut:='I';
	ELSE
		_InOut:='O';
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CADJ';
	IF(_LockDate IS NOT NULL AND _LockDate>_CashADate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penyesuaian Kas Melebihi Batas LockDate!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spCashBal_Update(_CashID, _CashADate, _Amount);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** /////////////// AKUNTANSI ////////////////////// */
	IF isAccInstalled() THEN
		IF (_CashANote IS NULL) THEN
			_JnlNote:='Jurnal Penyesuaian Cash '||_CashANo;
		ELSE
			_JnlNote:=_CashANote;
		END IF;

		IF(_CashAType='BEG') THEN
			_JnlType:='BEG';
			_JnlNote:='Beggining Balance Cash '||':'||_CashANo;
		ELSE
			_JnlType:='CADJ';
		END IF;

		_JnlNo:=fTransNoGet('JURNAL');
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, _JnlType, _CashADate, 'CADJ', _CashANo,
			_JnlNote, true, true, _Amount*_ExcRate, 
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_CashAType='BEG') THEN
			_Count:=1;

			_AccCash:=fAccCash(_CashID, _InOut);
			SELECT INTO _spret * FROM checkAccNullify(_AccCash, 'Akun Kas#'||COALESCE(_CashID, '<kosong>')||' belum diset !', '(spCashA_New_NoIns)');
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_AccDefa:=fAccDefault('BEGBAL');
			SELECT INTO _spret * FROM checkAccNullify(_AccNo, 'Akun Default Saldo Awal belum diset, silahkan setting di form Setting Akun', '(spCashA_New_NoIns)');
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_Amount>0) THEN
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccCash, 'D', _Amount*_ExcRate, _DeptID, _PrjID);
				_Count:=_Count+1;
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccDefa, 'K', _Amount*_ExcRate, _DeptID, _PrjID);
			ELSE
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccDefa, 'D', ABS(_Amount*_ExcRate), _DeptID, _PrjID);
				_Count:=_Count+1;
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccCash, 'K', ABS(_Amount*_Excrate), _DeptID, _PrjID);
			END IF;
		ELSE
			_Count:=1;
			_AccCash:=fAccCash(_CashID, _InOut);
			SELECT INTO _spret * FROM checkAccNullify(_AccCash, 'Akun Kas#'||COALESCE(_CashID, '<kosong>')||' belum diset !', '(spCashA_New_NoIns)');
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_AccDefa:=fAccDefault('PKKAS');
			SELECT INTO _spret * FROM checkAccNullify(_AccNo, 'Akun Default P/K Kas belum diset, silahkan setting di form Setting Akun', '(spCashA_New_NoIns)');
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_Amount<0) THEN
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccDefa, 'D', ABS(_Amount*_ExcRate), _DeptID, _PrjID);
				_Count:=_Count+1;
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccCash, 'K', ABS(_Amount*_ExcRate), _DeptID, _PrjID);
			ELSE
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccCash, 'D', _Amount*_ExcRate, _DeptID, _PrjID);
				_Count:=_Count+1;
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccDefa, 'K', _Amount*_ExcRate, _DeptID, _PrjID);
			END IF;
		END IF;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPO_AutoCheck()
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_PONo		VARCHAR(15);
	cPOExp		REFCURSOR;
BEGIN
	OPEN cPOExp FOR
		SELECT pono 
		FROM PO
		WHERE active=true AND poexpdate<fNoTime(GETDATE());
	LOOP
		FETCH cPOExp INTO _PONo;
		IF NOT FOUND THEN
			_retval.ErrNo:=12020001;
			_retval.ErrMsg:='ERROR (spPO_AutoCheck)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
			EXIT;
		END IF;

		SELECT INTO _spret * FROM spPO_Close(_PONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cPOExp;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fNoTime (
	_Date	DATE
) RETURNS DATE AS 
$BODY$
BEGIN
	/*--RETURN CAST(substr(CAST(_Date AS VARCHAR), 10) AS DATE);*/
	RETURN _Date;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fNoTime (
	_Date	TIMESTAMP
) RETURNS DATE AS 
$BODY$
BEGIN
	/*--RETURN CAST(substr(CAST(_Date AS VARCHAR), 10) AS DATE);*/
	RETURN CAST(_Date AS DATE);
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spBPLog_New(_BPLogNo VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

BEGIN	


	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spBPLog_Void(_BPLogNo VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval	SPStatus%ROWTYPE;
BEGIN

	DELETE FROM BPLog WHERE BPLogNo=_BPLogNo;
	IF NOT FOUND THEN
		_retval:=(1, 'Error DELETE BPLogD (spBPLog_Void)');
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFCN_New(_FcnNo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
	
	_BPID		VARCHAR(15);
	_FcnAmt		NUMERIC(19,4);
	_FcnDate	DATE;
	_IsContra	BOOLEAN;
	_CrtBy		VARCHAR(10);
	_CrtDate	DATE;
	_UpdBy		VARCHAR(10);
	_UpdDate	DATE;
	_BPAType	VARCHAR(1);
	_DC		VARCHAR(1);
	_RefNo		VARCHAR(15);

	_BPAAmt		NUMERIC(19,4);
	_BPANote	VARCHAR;
	_CrcID		VARCHAR(5);
	
	_AccNo		VARCHAR(10);
	_AccHutang	VARCHAR(10);
	_AccPiutang	VARCHAR(10);
	_JnlNo		VARCHAR(15);
	_JnlNote	VARCHAR;

	_PerID		VARCHAR(5);
	_BranchID	VARCHAR(10);
	_DeptID		VARCHAR(5);
	_PrjID		VARCHAR(5);

	_Count 		INT;
	_LockDate	DATE;
	_ExcRate	NUMERIC(19,4);
	_FisRate	NUMERIC(19,4);
	_isDraft	BOOLEAN;

BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;
	_PerID:=NULL;

	_ExcRate:=1;

	SELECT INTO 
		_BranchID, _BPID, _IsContra, _DC, _FcnAmt, _FcnDate, _Crtby, _CrtDate, _UpdBy, _UpdDate, _RefNo, _AccNo, _CrcID, _ExcRate, _FisRate, _isDraft
		BranchID, BPID, IsContra, DC, FcnAmt, FcnDate, CrtBy, CrtDate, UpdBy, UpdDate, RefNo, AccFCN, CrcID, ExcRate,FisRate, isDraft
	FROM FCN 
	WHERE FcnNo=_FcnNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='FCN #'||COALESCE(_FcnNo,'<kosong>')||' tidak ditemukan! (spFCN_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;
	
	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='FCN';
	IF(_LockDate is not null AND _LockDate>_FcnDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='FCN Melebihi Batas Tanggal Kunci Transaksi ! (spFCN_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/*SELECT INTO _CrcID CrcID
	FROM BPAcc
	WHERE BPID=_BPID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Crc BPID : '||_BPID||' tidak ditemukan (spFCN_New)';
	END IF;*/

	_BPAAmt:=_FcnAmt;		/** penerimaan : potong hutang (-) */
	_BPANote:='FCN #'||_FcnNo;
	
	IF (_DC='C') THEN
		_BPAType:='C';
	ELSIF (_DC='D') THEN
		_BPAType:='P';
		_BPAAmt:=_BPAAmt*(-1);
	END IF;

	SELECT INTO _spret * from spBPA_New(
		_BPID, _BPAType, _CrcID, _FcnDate, 'FCN', _FcnNo, 
		_BPAAmt, _BPANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
		_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate);

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

/** /////////////// AKUNTANSI ////////////////////// */
	IF isAccInstalled() THEN
		_JnlNote:= 'FCN '||fBPName(_BPID)||':'||_FcnNo;
		_JnlNo:=fTransNoGet('JURNAL');
		_Count:=0;
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'FCN', _FcnDate,'FCN',_FcnNo,
			_JnlNote,true,true,cast(0 as NUMERIC(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_AccHutang:=fAccDefault('ACCAP');
		_AccPiutang:=fAccDefault('ACCAR');

		IF (_DC='C') THEN
			IF(_FcnAmt<0) THEN
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo, 'D', _FCNAmt*_ExcRate, _DeptID, _PrjID, _FCNAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccHutang, 'K', _FCNAmt*_ExcRate, _DeptID, _PrjID, _FCNAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSIF (_FcnAmt>=0) THEN
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccHutang, 'D', _FCNAmt*_ExcRate, _DeptID, _PrjID, _FCNAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo, 'K', _FCNAmt*_ExcRate, _DeptID, _PrjID, _FCNAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		ELSIF (_DC='D') THEN
			IF(_FcnAmt<0) THEN
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccPiutang, 'D', _FCNAmt*_ExcRate, _DeptID, _PrjID, _FCNAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo, 'K', _FCNAmt*_ExcRate, _DeptID, _PrjID, _FCNAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			ELSIF (_FcnAmt>=0) THEN
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo, 'D', _FCNAmt*_ExcRate, _DeptID, _PrjID, _FCNAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccPiutang, 'K', _FCNAmt*_ExcRate, _DeptID, _PrjID, _FCNAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END IF;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFCN_Void (
	_FcnNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	
	_BPID		VARCHAR(15);
	_IsContra	BOOLEAN;
	_FcnDate	DATE;
	_LockDate	DATE;
	_PayNo		VARCHAR(15);
	_RcvNo		VARCHAR(15);
	_isDraft	BOOLEAN;
BEGIN
	SELECT INTO _BPID, _IsContra, _FcnDate, _isDraft
			BPID, IsContra, FcnDate, isDraft
	FROM FCN WHERE FcnNo=_FcnNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='FCN-Hutang #'||_FcnNo||' tidak ditemukan (spFCN_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM FCN WHERE FcnNo=_FcnNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='FCN';
	IF(_LockDate is not null AND _LockDate>_FcnDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='FCN Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isContra=TRUE) THEN /*--(_IsContra=true) AND ()) THEN*/
		SELECT INTO _RcvNo RcvNo FROM spaid WHERE RefNo=_FcnNo;
		IF FOUND THEN
			_retval.ErrMsg:='FCN-Piutang sudah dipotongkan Penerimaan No '||_RcvNo||', VOID dulu Penerimaannya ! (spFCN_Void)';
		ELSE
			SELECT INTO _PayNo PayNo FROM ppaid WHERE RefNo=_FcnNo;
			_retval.ErrMsg:='FCN-Hutang sudah dipotongkan Pembayaran No '||_PayNo||', VOID dulu Pembayarannya ! (spFCN_Void)';
		END IF;
		_retval.ErrNo:=50002;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT into _spret * FROM spBPA_Void_byRef('FCN', _FcnNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	IF isAccInstalled() THEN
		SELECT into _spret * from spJnl_Void_byRef('FCN',_FcnNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM FCN WHERE FcnNo=_FcnNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Gagal Delete From Fcn';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFCN_VoidContra (
	_FcnNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;

	_IsContra	BOOLEAN;
BEGIN
	SELECT into _IsContra IsContra
	FROM FCN WHERE FcnNo=_FcnNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='FCN #'||_FcnNo||' tidak ditemukan! (spFCN_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_IsContra<>TRUE) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='FCN #'||_FcnNo||' belum pernah dipotongkan sebelumnya! (spFCN_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	UPDATE FCN SET IsContra=FALSE WHERE FcnNo=_FcnNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Gagal Update FCN #'||_FcnNo||' (spFCN_VoidContra)';
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;

	_retval:=(0, 'UPDATE SUKSES !');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPurcDetail (
	_VendorID	VARCHAR,
	_StartDate	DATE,
	_EndDate	DATE
) RETURNS SETOF Purcdrpt AS
$BODY$
DECLARE
	_retval		Purcdrpt%ROWTYPE;

	_PurcNo		VARCHAR(15);
	_PurcdNo	INT;
	_PurcDate	Date;
	_TermType1	VARCHAR(1);
	_TermType	VARCHAR(8);
	_ItemID		VARCHAR(20);
	_ItemDesc	VARCHAR(50);
	_WhID		VARCHAR(3);
	_PID		VARCHAR(25);
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_QtyX		NUMERIC(15,4);
	_ListPrice	NUMERIC(19,4);
	_DiscAmt	NUMERIC(19,4);
	_TaxAmt		NUMERIC(19,4);
	_SubTotal	NUMERIC(19,4);
	_Total		NUMERIC(19,4);

	cPurc		refcursor;
	cPurcD		refcursor;
BEGIN
	_SubTotal:=0;
	_Total:=0;

	OPEN cPurc FOR
		SELECT PurcNo, PurcDate, TermType FROM Purc WHERE VendorID=_VendorID
		AND PurcDate>=fNoTime(_StartDate)
		AND PurcDate<=fDate2359(_EndDate);
	LOOP
		FETCH cPurc INTO _PurcNo, _PurcDate, _TermType1;
		EXIT WHEN NOT FOUND;

		OPEN cPurcD FOR
			SELECT PurcdNo, ItemID, WhID, ItemDesc, PID, Qty, Unit,
				QtyX, ListPrice, DiscAmt, TaxAmt
			FROM PurcD
			WHERE PurcNo=_PurcNo;
		LOOP
			FETCH cPurcD INTO _PurcdNo, _ItemID, _WhID, _ItemDesc,
				_PID, _Qty, _Unit, _QtyX, _ListPrice, _DiscAmt, _TaxAmt;
			EXIT WHEN NOT FOUND;

			IF (_TermType1='C') THEN
				_TermType:='Cash';
			ELSIF (_TermType1='R') THEN
				_TermType:='Credit';
			END IF;

			_SubTotal:=_Qty*_ListPrice;
			_Total:=_Total+_SubTotal;

			_retval.Purcno:=_PurcNo;
			_retval.Purcdno:=_PurcdNo;
			_retval.Purcdate:=_PurcDate;
			_retval.termtype:=_TermType;
			_retval.itemid:=_ItemID;
			_retval.itemdesc:=_ItemDesc;
			_retval.pid:=_PID;
			_retval.qty:=_Qty;
			_retval.unit:=_Unit;
			_retval.qtyx:=_QtyX;
			_retval.listprice:=_ListPrice;
			_retval.discamt:=_DiscAmt;
			_retval.taxamt:=_TaxAmt;
			_retval.subtotal:=_SubTotal;
			_retval.total:=_Total;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cPurcd;
	END LOOP;
	CLOSE cPurc;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAccDefault (
	_ID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	_mycrc		VARCHAR;
BEGIN
/*--	_mycrc:=_CrcID;*/

/*--	IF (_mycrc IS NULL) THEN*/
/*		_mycrc:=fCrcDefault();  dihapus karena akun tidak bergantung pada mata uang */
/*--	END IF;*/
	

	SELECT INTO _retval	defaval
	FROM defa WHERE DefaID=_ID;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAccCash (
	_CashID		VARCHAR, 
	_InOut		CHAR(1),
	_IsBeg		BOOLEAN
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval			VARCHAR;
	_isAutoReconcileOut	BOOLEAN;
	_isAutoReconcileIn	BOOLEAN;
BEGIN
	SELECT INTO _retval, _isAutoReconcileIn, _isAutoReconcileOut 
		accno, isautoreconcilein, isautoreconcileout 
	FROM Cash WHERE CashID=_CashID;

	IF (_IsBeg=FALSE AND _InOut IS NOT NULL 
	AND ((_InOut ='I' AND _isAutoReconcileIn=FALSE) OR (_InOut='O' AND _isAutoReconcileOut=FALSE))) THEN
		_retval:=fAccDefault('UNRECBANK');
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fAccCash (
	_CashID		VARCHAR,
	_InOut		CHAR(1)
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval			VARCHAR;
BEGIN
	_retval:=fAccCash(_CashID, _InOut, FALSE);

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fAccCash (
	_CashID		VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval			VARCHAR;
BEGIN
	_retval:=fAccCash(_CashID, NULL);

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAccAP (
	_BPID	VARCHAR,
	_CrcID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval	VARCHAR;
	_mycrc	VARCHAR;
BEGIN	
	_mycrc:=_CrcID;
	IF (_mycrc IS NULL) THEN
		_mycrc:=fCrcDefault();
	END IF;

	SELECT INTO _retval AccAP
	FROM BPAcc WHERE BPID=_BPID AND CrcID=_mycrc;
	
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAccAR (
	_BPID	VARCHAR,
	_CrcID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval	VARCHAR;
	_mycrc	VARCHAR;
BEGIN	
	_mycrc:=_CrcID;
	IF (_mycrc IS NULL) THEN
		_mycrc:=fCrcDefault();
	END IF;

	SELECT INTO _retval AccAR
	FROM BPAcc WHERE BPID=_BPID AND CrcID=_mycrc;
	
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: frptjualharian(_StartDate date, _EndDate date, _BranchID varchar)*/

/*-- DROP FUNCTION frptjualharian(_StartDate date, _EndDate date, _BranchID varchar);*/

CREATE OR REPLACE FUNCTION fRptJualHarian(
	_StartDate	DATE, 
	_EndDate	DATE, 
	_BranchID	VARCHAR,
	_SaleType	VARCHAR,
	_PosID		VARCHAR, 
	_Shift		VARCHAR,
	_CrcID		VARCHAR
) RETURNS SETOF rptjualharian AS
$BODY$
DECLARE
	_retval		rptjualharian%ROWTYPE;
	
	cData		REFCURSOR;
BEGIN
	OPEN cData FOR 
		SELECT * FROM fRptJualHarian(_StartDate, _EndDate, _BranchID, _SaleType, _PosID, _Shift, _CrcID, NULL);
	LOOP
		FETCH cData INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION frptjualharian(
	_StartDate	DATE, 
	_EndDate	DATE, 
	_BranchID	VARCHAR,
	_SaleType	VARCHAR,
	_PosID		VARCHAR, 
	_Shift		VARCHAR,
	_CrcID		VARCHAR,
	_IsUsePPN	BOOLEAN
) RETURNS SETOF rptjualharian AS
$BODY$
DECLARE
	_retval		rptjualharian%ROWTYPE;
	_SaleDate	DATE;
	_HariAngka	INT;
	_Hari		VARCHAR(6);
	_Tunai		NUMERIC(19,4);
	_Credit		NUMERIC(19,4);
	_Total		NUMERIC(19,4);
	_Subtotal	NUMERIC(19,4);
	_DiscAmt	NUMERIC(19,4);
	_TaxAmt		NUMERIC(19,4);
	_Cost		NUMERIC(19,4);
	_PL		NUMERIC(19,4);
	_Profit		NUMERIC(19,4);
	_Qty		NUMERIC(15,4);
	__BranchID	VARCHAR(10);
	__BranchName	VARCHAR(30);
	cSale		REFCURSOR;
	cSale2		REFCURSOR;
	__CrcID		VARCHAR;
BEGIN
	_SaleDate:=null;
	_Hari:=null;
	_Tunai:=0;
	_Credit:=0;
	_Total:=0;
	
	OPEN cSale FOR
		SELECT sale.branchid, br.branchname, saledate, SUM(subtotal), SUM(sale.discamt), 
			SUM(sale.taxamt), SUM(costtot), SUM(fProfitSale(sale.saleno)), 
			SUM(fSaleQty(sale.saleno)), sale.crcid
		FROM sale 
		LEFT JOIN branch br ON sale.branchid=br.branchid 
		LEFT JOIN posses p ON p.possesno=sale.possesno  
		WHERE saledate>=fNoTime(cast(_StartDate as date))
		AND saledate<=fDate2359(_EndDate) AND sale.isdraft=false 
		AND (_BranchID IS NULL OR sale.branchid=_BranchID)
		AND (_SaleType IS NULL OR sale.saletype=_SaleType)
		AND (_PosID IS NULL OR sale.posid=_PosID)
		AND (_Shift IS NULL OR p.shift=_Shift)
		AND (_CrcID IS NULL OR sale.crcid=_CrcID)
		AND (_IsUsePPN IS NULL OR fIsUsePPn(sale.saleno)=_IsUsePPN) 
		GROUP BY sale.crcid, saledate, br.branchname, sale.branchid
		ORDER BY sale.crcid, br.branchname, saledate;
	LOOP
		FETCH cSale INTO __BranchID, __BranchName, _SaleDate, _Subtotal, _DiscAmt, _TaxAmt, _Cost, _PL, _Qty, __CrcID;
		EXIT WHEN NOT FOUND;
				
		SELECT INTO _Tunai SUM(total)
		FROM sale 
		LEFT JOIN posses p ON sale.possesno=p.possesno 
		WHERE termtype='C' 
		AND saledate=_SaleDate AND sale.isdraft=false 
		AND (__BranchID IS NULL OR sale.branchid=__BranchID)
		AND (_SaleType IS NULL OR sale.saletype=_SaleType)
		AND (_PosID IS NULL OR sale.posid=_PosID)
		AND (_Shift IS NULL OR p.shift=_Shift)
		AND crcid=__CrcID;
		
		IF(_Tunai IS NULL) THEN
			_Tunai:=0;
		END IF;

		SELECT INTO _Credit SUM(total)
		FROM sale 
		LEFT JOIN posses p ON sale.possesno=p.possesno 
		WHERE termtype='R' 
		AND saledate=_SaleDate AND sale.isdraft=false 
		AND (__BranchID IS NULL OR sale.branchid=__BranchID)
		AND (_SaleType IS NULL OR sale.saletype=_SaleType)
		AND (_PosID IS NULL OR sale.posid=_PosID)
		AND (_Shift IS NULL OR p.shift=_Shift)
		AND crcid=__CrcID;
		
		IF(_Credit IS NULL) THEN
			_Credit:=0;
		END IF;

		SELECT INTO _HariAngka EXTRACT(DOW FROM _SaleDate);
		IF (_HariAngka=1) THEN
			_Hari:='Senin';
		ELSIF (_HariAngka=2) THEN
			_Hari:='Selasa';
		ELSIF (_HariAngka=3) THEN
			_Hari:='Rabu';
		ELSIF (_HariAngka=4) THEN
			_Hari:='Kamis';
		ELSIF (_HariAngka=5) THEN
			_Hari:='Jumat';
		ELSIF (_HariAngka=6) THEN
			_Hari:='Sabtu';
		ELSIF (_HariAngka=0) THEN
			_Hari:='Minggu';
		END IF;

		_Total:=COALESCE(_Tunai,0)+COALESCE(_Credit,0);
		/** _Profit:=(_Subtotal-_DiscAmt+_TaxAmt)-_Cost; */
		_Profit:=_PL;
		_retval.saledate:=_SaleDate;
		_retval.posid:=_PosID;
		_retval.shift:=_Shift;
		_retval.hari:=_Hari;
		_retval.tunai:=_Tunai;
		_retval.credit:=_Credit;
		_retval.total:=_Total;
		_retval.profit:=_Profit;
		_retval.branchid:=__BranchID;
		_retval.branchname:=__BranchName;
		_retval.qty:=_Qty;
		_retval.crcid=__CrcID;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cSale;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSaleDetail (
	_CustID		VARCHAR,
	_StartDate	DATE,
	_EndDate	DATE
) RETURNS SETOF saledrpt AS
$BODY$
DECLARE
	_retval		saledrpt%ROWTYPE;

	_SaleNo		VARCHAR(15);
	_SaledNo	INT;
	_SaleDate	Date;
	_TermType1	VARCHAR(1);
	_TermType	VARCHAR(8);
	_ItemID		VARCHAR(20);
	_ItemDesc	VARCHAR(50);
	_WhID		VARCHAR(3);
	_PID		VARCHAR(25);
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_QtyX		NUMERIC(15,4);
	_ListPrice	NUMERIC(19,4);
	_DiscAmt	NUMERIC(19,4);
	_TaxAmt		NUMERIC(19,4);
	_SubTotal	NUMERIC(19,4);
	_Total		NUMERIC(19,4);
	
	cSale		REFCURSOR;
	cSaleD		REFCURSOR;

BEGIN
	_SubTotal:=0;
	_Total:=0;

	OPEN cSale FOR
		SELECT SaleNo, SaleDate, TermType FROM Sale WHERE custid=_CustID
		AND SaleDate>=fNoTime(_StartDate)
		AND SaleDate<=fDate2359(_EndDate);
	LOOP
		FETCH cSale INTO _SaleNo, _SaleDate, _TermType1;
		EXIT WHEN NOT FOUND;

		OPEN cSaleD FOR
			SELECT SaledNo, ItemID, WhID, ItemDesc, PID, Qty, Unit,
				QtyX, ListPrice, DiscAmt, TaxAmt
			FROM SaleD
			WHERE SaleNo=_SaleNo;
		LOOP
			FETCH cSaleD INTO _SaledNo, _ItemID, _WhID, _ItemDesc,
					_PID, _Qty, _Unit, _QtyX, _ListPrice, _DiscAmt, _TaxAmt;
			EXIT WHEN NOT FOUND;

			IF (_TermType1='C') THEN
				_TermType:='Cash';
			ELSIF (_TermType1='R') THEN
				_TermType:='Credit';
			END IF;
			
			_SubTotal:=_Qty*_ListPrice;
			_Total:=_Total+_SubTotal;

			_retval.saleno:=_SaleNo;
			_retval.saledno:=_SaledNo;
			_retval.saledate:=_SaleDate;
			_retval.termtype:=_TermType;
			_retval.itemid:=_ItemID;
			_retval.itemdesc:=_ItemDesc;
			_retval.pid:=_PID;
			_retval.qty:=_Qty;
			_retval.unit:=_Unit;
			_retval.qtyx:=_QtyX;
			_retval.listprice:=_ListPrice;
			_retval.discamt:=_DiscAmt;
			_retval.taxamt:=_TaxAmt;
			_retval.subtotal:=_SubTotal;
			_retval.total:=_Total;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cSaled;
	END LOOP;
	CLOSE cSale;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCons_New (
	_ConsNo VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret 			SPStatus%ROWTYPE;

	_CustID			VARCHAR(15);
	_SaleType		VARCHAR(5);
	_ConsDate		DATE;
	_TermType		VARCHAR(1);
	_Term			INT;
	_PoSID			VARCHAR(5);
	_PoSSesNo		VARCHAR(8);
	_Total			NUMERIC(19,4);
	_PaidAmt		NUMERIC(19,4);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_ConsdNo		INT;
	_ConsNo2		VARCHAR(15);
	_StockANo		VARCHAR(15);
	_StockANote		VARCHAR(50);
	_StockADNo		INT2;
	_ItemID			VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR(25);
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_Conv			NUMERIC(15,4);
	_QtyX			NUMERIC(15,4);
	_ListPrice		NUMERIC(19,4);
	_DiscExp		VARCHAR(15);
	_DiscAmt		NUMERIC(19,4);
	_TaxID			VARCHAR(5);
	_TaxAmt			NUMERIC(19,4);
	_Cost			NUMERIC(19,4);
	_ItemType		VARCHAR(5);
	_IsStock		BOOLEAN;
	_BoMID			VARCHAR(20);
	_Qty_Unit1		NUMERIC(15,4);
	_ConsQty		NUMERIC(15,4);
	_ConsQtyX		NUMERIC(15,4);
	_ItemsCost		NUMERIC(19,4);
	_CashID			VARCHAR(15);
	_CashANote		VARCHAR(50);
	_DebtNote		VARCHAR(50);
	_AllowCredit		NUMERIC(19,4);
	_CreditLimit		NUMERIC(19,4);
	_ContBal		NUMERIC(19,4);
	_RcvNo			VARCHAR;
	_EmpID			VARCHAR;
	_TotDisc		NUMERIC(19,4);
	_Round			INT;

	/* SO Stuff ****/
	_SONo			VARCHAR(15);
	_SODNo			SMALLINT;

	_Qty1 			NUMERIC(15,4);
	_Qty1Rcv 		NUMERIC(15,4);
	_xt1 			BOOLEAN;
	cConsD 			REFCURSOR;
	cConsD2 		REFCURSOR;
	cSO 			REFCURSOR;

	_JnlNo			VARCHAR(15);
	_Inv			VARCHAR(10);
	_Count			INT;

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);

	_isDraft		BOOLEAN;
	_Disc			DECIMAL(19,4);
	_MTaxAmt		DECIMAL(19,4);
	_Freight		DECIMAL(19,4);
	_Others			DECIMAL(19,4);
	_ExcRate		DECIMAL(19,4);
	_LockDate		DATE;

	_SelisihTax		NUMERIC(19,4);
	_iterLoop		INT;
	cTax			REFCURSOR;
	_ConsDTaxAmt		NUMERIC(19,4);
	_ConsDTaxAmtForex 	NUMERIC(19,4);
	_AccConsTax		VARCHAR;
	_TotalTax		NUMERIC(19,4);
	_CrcID			VARCHAR;
	_FisRate		NUMERIC(19,4);

	_BTotal			NUMERIC(19,4); -- base currency total
	_BSubtotal		NUMERIC(19,4); -- base currency subtotal
	_MSubtotal		NUMERIC(19,4);
	_MBFTaxAmt		NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt		NUMERIC(19,4); -- base currency discamt

	_TaxableAmt		NUMERIC(19,4);
	_TotalTaxAmt		NUMERIC(19,4);
	_BTotalTaxAmt		NUMERIC(19,4);
	_TotalDiscAmt		NUMERIC(19,4);
	_TotalDisc2Amt		NUMERIC(19,4);
	_DSubtotal		NUMERIC(19,4);
	_DBSubtotal		NUMERIC(19,4);

	_AccRateLR		VARCHAR; -- akun laba/rugi selisih kurs
	_RateLR			NUMERIC(19,4); -- nilai laba/rugi selisih kurs
BEGIN
	_TotDisc:=0;
	_DeptID:=NULL;
	_PrjID:=NULL;

	SELECT INTO _isDraft, _CustID, _SaleType, _ConsDate, _SONo, _PoSID, _CashID, _BTotal, _Total, _PaidAmt, 
		_CrtBy, _CrtDate, _UpdBy, _UpdDate, _EmpID, _PerID, _BranchID, _MBDiscAmt, _Disc, _MBFTaxAmt, _MTaxAmt, 
		_Freight, _Others, _ExcRate, _CrcID, _FisRate, _BSubtotal, _MSubtotal 
		isdraft, custid, saletype, consdate, sono, posid, cashid, basetotal, total, paidamt, 
		crtby, crtdate, updby, upddate, empid, perid, branchid, basediscamt, discamt, basftaxamt, taxamt, 
		freight, others, excrate, crcid, fisrate, basesubtotal, subtotal
	FROM Cons 
	WHERE ConsNo=_ConsNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg='Konsinyasi tidak ditemukan (spCons_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CONS';
	IF(_LockDate is not null AND _LockDate>_ConsDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Konsinyasi Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	IF (_isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_StockANote:='Konsinyasi #'||_ConsNo;

	SELECT INTO _StockANo * from fTransNoGet('STOCKA');

	IF (_StockANo IS NULL) THEN
		_retval.ErrNo:=1041;
		_retval.ErrMsg:='Error getting StockANo';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spStockA_New(_StockANo,_CustID,'CONS',_ConsDate, 'O', 'CONS', _ConsNo,_StockANote, TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _PerID, _BranchID, _DeptID, _PrjID); 


	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_StockADNo:=0;
	OPEN cConsD FOR
			SELECT d.ConsdNo, d.ItemID, d.WhID, d.PID, d.QtySellin, d.Unit, d.QtyXSellin, i.ItemType, i.IsStock, d.DeptID, d.PrjID
			FROM ConsD d JOIN Item i ON d.ItemID=i.ItemID
			WHERE d.ConsNo=_ConsNo AND ((i.IsStock=true) OR (i.ItemType='PACK'));
			/*Item biasa:GOOD, Paket:PACK*/	

	LOOP
		FETCH cConsD INTO _ConsdNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ItemType, _IsStock, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		/*################ Generate Stock-Details ##################*/
		_StockADNo:=_StockADNo+1;	

		IF (_ItemType='PACK') THEN
			SELECT INTO _BoMID BoMID FROM BoM WHERE ItemID=_ItemID;

			IF(_BomID IS NULL) THEN
				_retval.ErrNo:='50014';
				_retval.ErrMsg='Error tidak dapat menemukan B.o.M. untuk Item:'||_ItemID||'(spCons_New)';
			END IF;

			_ConsQty:=_Qty*(-1);
			_ConsQtyX:=_QtyX*(-1);

			SELECT INTO _spret * FROM spStockAD_New_BoM(_ConsDate, 'CONS', _StockANo,_StockADNo,_WhID, _ItemID,_PID,_Qty*(-1),_Unit,_QtyX*(-1), _Cost, _DeptID, _PrjID);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		ELSE
			IF (_PID IS NULL) THEN
				_PID:='';
			END IF;


			_ConsQty:=_Qty*(-1);
			_ConsQtyX:=_QtyX*(-1);

			SELECT INTO _Cost * FROM fUnitCost(_ItemID, _WhID, _PID, _Unit);

			IF (_Cost IS NULL) THEN
				_retval.ErrNo:='50003';
				_retval.ErrMsg:='Error menghitung HPP! Item:'||_ItemID||', Wh:'||_WhID||', Batch:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spCons_New)';
			END IF;

			SELECT INTO _spret * FROM spStockAD_New(_ConsDate, 'CONS', _StockANo,_StockADNo, _WhID,_ItemID, _PID,_Qty*(-1),_Unit,_QtyX*(-1), _Cost, _DeptID, _PrjID, true);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		SELECT INTO _Conv * FROM fItemUnit_Conv(_ItemID,_Unit);

		IF (_Conv IS NULL) THEN
			_retval.ErrNo:='50055';
			_retval.ErrMsg:='Error, Konversi ItemID:'||_ItemID||', Unit:'||_Unit||'!  (fItemUnit_Conv)(spSale_New)';
		END IF;

		UPDATE ConsD SET Cost=_Cost, Conv=_Conv WHERE ConsNo=_ConsNo AND ConsdNo=_ConsdNo;

		_ItemsCost:=_ItemsCost+(_Qty*_Cost);
	END LOOP;
	CLOSE cConsD;

	OPEN cConsD FOR
		SELECT d.ConsdNo, d.ItemID, d.WhID, d.PID, d.QtySellin, d.Unit, d.QtyXSellin, ListPrice, d.DiscExp, d.DiscAmt, d.TaxID, d.TaxAmt, i.ItemType
		FROM ConsD d JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.ConsNo=_ConsNo AND ((i.ItemType='SERV') OR (i.IsStock=false AND i.IsAsm=false));
	LOOP
		FETCH cConsD INTO _ConsdNo, _ItemID, _WhID, _PID, _Qty, _Unit, _QtyX, _ListPrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _ItemType;
		EXIT WHEN NOT FOUND;

		IF (_ItemType='CDIS') THEN
			_Cost=_ListPrice-_DiscAmt+_TaxAmt;
		ELSE
			SELECT INTO _Cost * FROM fUnitCost(_ItemID, _WhID, _PID, _Unit);
		END IF;

		IF (_Cost IS NULL) THEN
			_retval.ErrNo:='50003';
			_retval.ErrMsg:='Error menghitung HPP! Item:'||_ItemID||', Wh:'||_WhID||', Batch:'||_PID||', Unit:'||_Unit||' (fUnitCost)(spSale_New)';
		END IF;

		UPDATE SaleD SET Cost=_Cost WHERE ConsNo=_ConsNo AND ConsdNo=_ConsdNo;

		_ItemsCost:=_ItemsCost+(_Qty*_Cost);
		_TotDisc:=_TotDisc+_DiscAmt;

	END LOOP;
	CLOSE cConsD;


	UPDATE Cons SET ConsNo2=_ConsNo2, _xt=_xt1, CostTot=_ItemsCost WHERE ConsNo=_ConsNo;

	OPEN cConsD FOR
		SELECT SONo, SODNo, ItemID, PID, fQtyZToQty1(ItemID, QtySellIn, Unit) as Qty1Tot 
		FROM ConsD 
		WHERE ConsNo=_ConsNo AND SONo IS NOT NULL AND SONo<>'' 
		GROUP BY SONo, SODNo, ItemID, PID, QtySellIn, Unit;
	LOOP
		FETCH cConsD INTO _SONo, _SODNo, _ItemID, _PID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Rcv * FROM spSO_Item_Delivered(_SONo,_ItemID,_PID,_Qty1,_SODNo);

	END LOOP;
	CLOSE cConsD;

	OPEN cSO FOR
		SELECT SONo FROM Cons WHERE ConsNo=_ConsNo ORDER BY SONo;
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spSO_ChNUp_Status(_SONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSO;


/* ////////////////////// AKUNTANSI /////////////////////////////// */
	IF (isAccInstalled()) THEN
		_JnlNo:=fTransNoGet('JURNAL');
		SELECT INTO _spret * FROM spJnl_Insert(_JnlNo,'CONS', _ConsDate,'CONS',_ConsNo,
			'Jurnal Konsinyasi Keluar '||_ConsNo,true,true,cast(0 as numeric(19,4)), _CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
			);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_Round:=fReg('ROUND');
		IF(_Round is null) THEN
			_Round:=0;
		END IF;

		_Count:=0;
		_Count:=_Count+1;

		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('ARCONS'), 'D', _BTotal, _DeptID, _PrjID, _Total);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;


		OPEN cConsd FOR
			SELECT fAccDefault('INVCONS'), SUM(basesubtotal), SUM(subtotal), deptid, prjid
			FROM consd d
			WHERE consno=_ConsNo
			GROUP BY fAccDefault('INVCONS'), DeptID, PrjID;
		LOOP
			FETCH cConsd INTO _Inv, _DBSubtotal, _DSubtotal, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			_Count:=_Count+1;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _Inv, 'K', _DBSubtotal, _DeptID, _PrjID, _DSubtotal);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;

		_Disc:=COALESCE(_Disc, 0);
		IF (_Disc<>0) THEN

			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('SALEDISC'), 'K', _MBDiscAmt*-1, NULL, NULL, _Disc*-1);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		SELECT INTO _TotalTax Round(sum(d.taxamt*QtySellIn*_ExcRate),_Round) 
		FROM consD d JOIN tax ON d.taxid=tax.taxid
		WHERE d.consNo=_ConsNo and d.taxid is not null;

		/*POSTING TAX DI GRUP IN SESUAI PAJAK TIAP2 ITEM*/
		OPEN cTax FOR
			SELECT SUM(basftotaltaxamt), SUM(totaltaxamt), accsaletax, deptid, prjid
			FROM consD d JOIN tax ON d.taxid=tax.taxid
			WHERE d.ConsNo=_ConsNo and d.taxid is not null
			GROUP BY accsaletax,deptid,prjid;
		LOOP
			FETCH cTax INTO _ConsDTaxAmt, _ConsDTaxAmtForex, _AccConsTax,_DeptID,_PrjID;
			EXIT WHEN NOT FOUND;

			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccConsTax, 'K', _ConsDTaxAmt, _DeptID, _PrjID, _ConsDTaxAmtForex);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;

		/*IF (_MTaxAmt<>0) THEN
			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('SALETAX'), 'K', _MTaxAmt*_ExcRate, NULL, NULL);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;*/

		/** ==================== LABA RUGI SELISIH KURS ================================ */
		_RateLR:=(_BTotal+_MBDiscAmt)-(_BSubtotal+_MBFTaxAmt);
		IF (_RateLR!=0) THEN
			_Count:=_Count+1;
			_AccRateLR:=fAccDefault('PKKURS');
			IF (_AccRateLR IS NULL OR _AccRateLR='') THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Akun PK Selisih Kurs belum diset.'||E'\n'||'Silahkan disetting pada form Setting Akun';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccRateLR, 'K', _RateLR, _DeptID, _PrjID, NULL, _RateLR/_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		IF (_Freight<>0) THEN
			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('SALEFREIGHT'), 'K', _Freight*_ExcRate, NULL, NULL, _Freight);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		IF (_Others<>0) THEN
			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('PKROUND'), 'K', _Others*_ExcRate, NULL, NULL,_Others);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCashConvert (
	_CrcIDFrom	VARCHAR,
	_CrcIDTo	VARCHAR,
	_CashAmt	NUMERIC(19,4)
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);

	_FromExcRate	NUMERIC(19,4);
	_ToExcRate	NUMERIC(19,4);
BEGIN
	_retval:=0;
	
	SELECT INTO _ToExcRate excrate FROM crc WHERE crcid=_CrcIDTo;
	SELECT INTO _FromExcRate excrate FROM crc WHERE crcid=_CrcIDFrom;

	_retval:=(_CashAmt*_FromExcRate)/_ToExcRate;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tLastPurcCrc CASCADE;
CREATE TYPE tLastPurcCrc AS (
	purcdate	DATE,
	purccrcid	VARCHAR,
	purcprice	NUMERIC(19,4),
	purctaxid	VARCHAR(5),
	purctaxamt	NUMERIC(19,4),
	purcdiscexp	VARCHAR(15),
	purcdiscamt	NUMERIC(19,4),
	purcunit	VARCHAR
);


CREATE OR REPLACE FUNCTION fLastPurcCrc (
	__ItemID	VARCHAR,
	__CrcID		VARCHAR
) RETURNS SETOF tLastPurcCrc AS
$BODY$
DECLARE
	_retval		tLastPurcCrc%ROWTYPE;
	_UpdDate	TIMESTAMP;

	_CmpCrcID	VARCHAR;
BEGIN
	SELECT INTO _UpdDate, _retval.purcdate, _retval.purccrcid, _retval.purcprice, _retval.purctaxid, _retval.purctaxamt, _retval.purcdiscexp, _retval.purcdiscamt, _retval.purcunit 
		m.upddate, m.purcdate, m.crcid, d.listprice, d.taxid, d.taxamt, d.discexp, d.discamt, d.unit
	FROM purc m 
	JOIN purcd d ON d.purcno=m.purcno 
	WHERE m.crcid=__CrcID AND d.itemid=__ItemID 
	ORDER BY m.purcdate DESC, m.upddate DESC LIMIT 1;
	IF (FOUND) THEN
		RETURN NEXT _retval;
	ELSE
		SELECT INTO _CmpCrcID crcid FROM cmp; 
		SELECT INTO _UpdDate, _retval.purcdate, _retval.purccrcid, _retval.purcprice, _retval.purctaxid, _retval.purctaxamt, _retval.purcdiscexp, _retval.purcdiscamt, _retval.purcunit 
		m.upddate, m.purcdate, m.crcid, d.listprice, d.taxid, d.taxamt, d.discexp, d.discamt, d.unit
		FROM purc m 
		JOIN purcd d ON d.purcno=m.purcno 
		WHERE m.crcid=_CmpCrcID AND d.itemid=__ItemID 
		ORDER BY m.purcdate DESC, m.upddate DESC LIMIT 1;
		IF (FOUND) THEN
			_retval.purcprice:=fCashConvert(_CmpCrcID, __CrcID, _retval.purcprice);
			RETURN NEXT _retval;
		END IF;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fLastPurc (
	_ItemID		VARCHAR(20)
) RETURNS SETOF tLastPurc AS
$BODY$
DECLARE
	_retval			tLastPurc%ROWTYPE;
	_lastpurcdate		DATE;
	_lastpurcprice		NUMERIC(19,4);
	_lastpurctaxid		VARCHAR(5);
	_lastpurctaxamt		NUMERIC(19,4);
	_lastpurcdiscexp	VARCHAR(15);
	_lastpurcdiscamt	NUMERIC(19,4);
	_lastpurcunit		VARCHAR;

	_lastpurcno		VARCHAR;
BEGIN

/*
	-- before 090511
	SELECT INTO  _LastPurcDate, _LastPurcPrice, _LastPurcTaxID, _LastPurcTaxAmt,_LastPurcDiscExp, _LastPurcDiscAmt,_LastPurcUnit
	purc.PurcDate, ListPrice, purcd.TaxID, purcd.TaxAmt, purcd.DiscExp, purcd.DiscAmt,unit
	FROM purcd join purc on purcd.purcno=purc.purcno
	WHERE itemid=_ItemID ORDER BY _LastPurcDate desc, purcd.purcNo desc LIMIT 1 ;
	_retval.LastPurcDate=_LastPurcDate;
	_retval.LastPurcPrice=_LastPurcPrice;
	_retval.LastPurcTaxID=_LastPurcTaxID;
	_retval.LastPurcTaxAmt=_LastPurcTaxAmt;
        _retval.LastPurcDiscExp=_LastPurcDiscExp;
	_retval.LastPurcDiscAmt=_LastPurcDiscAmt;
	_retval.LastPurcUnit=_LastPurcUnit;
	RETURN NEXT _retval;
*/


	/*SELECT INTO _lastpurcno MAX(PurcNo) FROM Purc  
		WHERE PurcDate=(
		SELECT MAX(PurcDate) FROM PurcD d JOIN Purc h ON d.PurcNo=h.PurcNo WHERE ItemID=_ItemID
		);*/

	SELECT INTO _lastpurcno Purc.PurcNo FROM Purc JOIN PurcD ON Purc.PurcNo=PurcD.PurcNo WHERE ItemID=_ItemID ORDER BY PurcDate Desc Limit 1;

	IF (_lastpurcno IS NOT NULL) THEN
		SELECT INTO  
				_LastPurcDate, _LastPurcPrice, _LastPurcTaxID, _LastPurcTaxAmt, _LastPurcDiscExp, _LastPurcDiscAmt,_LastPurcUnit
				h.PurcDate, d.ListPrice, d.TaxID, d.TaxAmt, d.DiscExp, d.DiscAmt, d.unit
		FROM purcd d JOIN Purc h ON d.PurcNo=h.PurcNo and d.itemid=_ItemID
		WHERE d.PurcNo=_lastpurcno;


		_retval.LastPurcDate=_LastPurcDate;
		_retval.LastPurcPrice=_LastPurcPrice;
		_retval.LastPurcTaxID=_LastPurcTaxID;
		_retval.LastPurcTaxAmt=_LastPurcTaxAmt;
		_retval.LastPurcDiscExp=_LastPurcDiscExp;
		_retval.LastPurcDiscAmt=_LastPurcDiscAmt;
		_retval.LastPurcUnit=_LastPurcUnit;
	ELSE
		
		_retval.LastPurcDate=now();
		_retval.LastPurcPrice=0;
		_retval.LastPurcTaxID=NULL;
		_retval.LastPurcTaxAmt=0;
		_retval.LastPurcDiscExp='';
		_retval.LastPurcDiscAmt=0;
		_retval.LastPurcUnit=NULL;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/** bisa di set berdasarkan mode mau bersih atau basid */
CREATE OR REPLACE FUNCTION fLastPurc (
	_ItemID		VARCHAR(20),
	_Mode		INT
) RETURNS SETOF tLastPurc AS
$BODY$
DECLARE
	_retval			tLastPurc%ROWTYPE;
	_lastpurcdate		DATE;
	_lastpurcprice		NUMERIC(19,4);
	_lastpurctaxid		VARCHAR(5);
	_lastpurctaxamt		NUMERIC(19,4);
	_lastpurcdiscexp	VARCHAR(15);
	_lastpurcdiscamt	NUMERIC(19,4);
	_lastpurcdisc2amt	NUMERIC(19,4);
	_lastpurcunit		VARCHAR;

	_lastpurcno 		VARCHAR;
	_lastconv		NUMERIC;
	_lastbaseprice		NUMERIC(19,4);
BEGIN

	SELECT INTO _lastpurcno Purc.PurcNo FROM Purc JOIN PurcD ON Purc.PurcNo=PurcD.PurcNo WHERE ItemID=_ItemID ORDER BY PurcDate Desc Limit 1;

	IF (_lastpurcno IS NOT NULL) THEN
		SELECT INTO  _LastPurcDate, _LastPurcPrice, _LastPurcTaxID, _LastPurcTaxAmt, 
			_LastPurcDiscExp, _LastPurcDiscAmt,_LastPurcUnit, _LastPurcDisc2Amt, _lastconv, _lastbaseprice
			h.PurcDate, d.ListPrice, d.TaxID, d.TaxAmt, 
			d.DiscExp, d.DiscAmt, d.unit, d.disc2amt, d.conv, d.baseprice
		FROM purcd d JOIN Purc h ON d.PurcNo=h.PurcNo and d.itemid=_ItemID
		WHERE d.PurcNo=_lastpurcno;

		IF (_Mode=1) THEN
			_LastPurcPrice:=(_LastBasePrice-_LastPurcDiscAmt-_LastPurcDisc2Amt+_LastPurcTaxAmt)/_lastconv;
			SELECT INTO _LastPurcUnit unit1 FROM item WHERE itemid=_ItemID;
		END IF;

		_retval.LastPurcDate=_LastPurcDate;
		_retval.LastPurcPrice=_LastPurcPrice;
		_retval.LastPurcTaxID=_LastPurcTaxID;
		_retval.LastPurcTaxAmt=_LastPurcTaxAmt;
		_retval.LastPurcDiscExp=_LastPurcDiscExp;
		_retval.LastPurcDiscAmt=_LastPurcDiscAmt;
		_retval.LastPurcUnit=_LastPurcUnit;
	ELSE
		_retval.LastPurcDate=now();
		_retval.LastPurcPrice=0;
		_retval.LastPurcTaxID=NULL;
		_retval.LastPurcTaxAmt=0;
		_retval.LastPurcDiscExp='';
		_retval.LastPurcDiscAmt=0;
		_retval.LastPurcUnit=NULL;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPrcPromo(
	_ItemID		VARCHAR,
	_Qty		NUMERIC, 
	_Unit		VARCHAR, 
	_SaleDate	DATE, 
	_BPID		VARCHAR, 
	_Total		NUMERIC,
	_BPGrpID	VARCHAR
) RETURNS SETOF tPrcPromo AS
$BODY$
DECLARE
	_retval		tPrcPromo%ROWTYPE;

	_Price		NUMERIC(19,4);
	_DiscExp	VARCHAR;
	_Selisih	NUMERIC;
	_OldSelisih	NUMERIC;
	_MinAmt		NUMERIC;

	_ModeBP		INT;
	_ModeItem	INT;
	
	_TBPType	VARCHAR;
	_TItemType	VARCHAR;

	cRetval		REFCURSOR;
BEGIN

	_retval.Price:=0;
	_retval.DiscExp:='';

	IF (_BPGrpID IS NOT NULL AND _BPID IS NOT NULL) THEN
		_ModeBP:=0;
	ELSIF (_BPID IS NOT NULL) THEN
		_ModeBP:=1;
	ELSE
		_ModeBP:=2;
	END IF;

	IF (_ItemID IS NOT NULL) THEN
		_ModeItem:=0;
	ELSIF (_ItemID IS NULL AND _Total>0) THEN
		_ModeItem:=1;
	END IF;

	IF (_ModeBP=0 AND _ModeItem=0) THEN
		--RAISE NOTICE '%', _ModeBP||' -> '||_ModeItem;

		SELECT INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType 
				price1, disc1exp, (fqtyztoqty1(_ItemID,_Qty,_Unit)-fqtyztoqty1(itemid,qty,unit)), tbptype, titemtype
		FROM pprc WHERE active=TRUE 
		AND ((tbptype='BPGRP' AND fbpgrpin(_BPID, bpgrpid)) OR (tbptype='BPID' AND (custid=_BPID OR vendorid=_BPID)) OR (tbptype='ALL')) 
		AND titemtype='ITEM' AND itemid=_ItemID  
		AND unit=_Unit AND qty<=_Qty
		AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
		IF (FOUND) THEN
			_retval.price:=_Price;
			_retval.discexp:=_DiscExp;
		ELSE
			SELECT INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType 
				price1, disc1exp, (fqtyztoqty1(_ItemID,_Qty,_Unit)-fqtyztoqty1(itemid,qty,unit)), tbptype, titemtype
			FROM pprc WHERE active=TRUE 
			AND ((tbptype='BPGRP' AND fbpgrpin(_BPID, bpgrpid)) OR (tbptype='BPID' AND (custid=_BPID OR vendorid=_BPID)) OR (tbptype='ALL')) 
			AND titemtype='ITEM' AND itemid=_ItemID  
			AND unit!=_Unit AND fqtyztoqty1(itemid,qty,unit)<=fqtyztoqty1(_ItemID,_Qty,_Unit)
			AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
			IF (FOUND) THEN
				_retval.price:=_Price;
				_retval.discexp:=_DiscExp;
			END IF;
		END IF;

		RETURN NEXT _retval;
	ELSIF (_ModeBP=0 AND _ModeItem=1) THEN
		--RAISE NOTICE '%', _ModeBP||' -> '||_ModeItem;
		OPEN cRetval FOR 
			SELECT price1, disc1exp, (_total-minamt), tbptype, titemtype
			FROM pprc WHERE active=TRUE AND tbptype='BPGRP' AND fbpgrpin(_BPID, bpgrpid) AND titemtype='TOTAMT' AND minamt<=_Total
			AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
		LOOP
			FETCH cRetval INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType;
			EXIT WHEN NOT FOUND;
			--RAISE NOTICE '%', _TBPType||' -> '||_TItemType;
			--RAISE NOTICE '%', COALESCE(_Price, 0)||' -> '||COALESCE(_DiscExp, 'disc');
			IF (_OldSelisih IS NULL OR (_Selisih<=_OldSelisih)) THEN
				_OldSelisih:=_Selisih;
				_retval.price:=_Price;
				_retval.discexp:=_DiscExp;
			END IF;
		END LOOP;
		CLOSE cRetval;

		RETURN NEXT _retval;
	ELSIF (_ModeBP=1 AND _ModeItem=0) THEN
		--RAISE NOTICE '%', _ModeBP||' -> '||_ModeItem;
		SELECT INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType 
			price1, disc1exp, (fqtyztoqty1(_ItemID,_Qty,_Unit)-fqtyztoqty1(itemid,qty,unit)), tbptype, titemtype
		FROM pprc WHERE active=TRUE AND ((tbptype='BPID' AND (custid=_BPID OR vendorid=_BPID)) OR (tbptype='ALL')) 
		AND titemtype='ITEM' AND itemid=_ItemID 
		AND unit=_Unit AND qty<=_Qty 
		AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
		IF (FOUND) THEN
			_retval.price:=_Price;
			_retval.discexp:=_DiscExp;
		ELSE
			SELECT INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType 
				price1, disc1exp, (fqtyztoqty1(_ItemID,_Qty,_Unit)-fqtyztoqty1(itemid,qty,unit)), tbptype, titemtype
			FROM pprc WHERE active=TRUE AND ((tbptype='BPID' AND (custid=_BPID OR vendorid=_BPID)) OR (tbptype='ALL')) 
			AND titemtype='ITEM' AND itemid=_ItemID 
			AND unit!=_Unit AND fqtyztoqty1(itemid,qty,unit)<=fqtyztoqty1(_ItemID,_Qty,_Unit) 
			AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
			IF (FOUND) THEN
				_retval.price:=_Price;
				_retval.discexp:=_DiscExp;
			END IF;
		END IF;

		RETURN NEXT _retval;
	ELSIF (_ModeBP=1 AND _ModeItem=1) THEN
		--RAISE NOTICE '%', _ModeBP||' -> '||_ModeItem;
		OPEN cRetval FOR 
			SELECT price1, disc1exp, (_total-minamt), tbptype, titemtype
			FROM pprc WHERE active=TRUE AND ((tbptype='BPID' AND (custid=_BPID OR vendorid=_BPID)) OR (tbptype='ALL')) AND titemtype='TOTAMT' AND minamt<=_Total 
			AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
		LOOP
			FETCH cRetval INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType;
			EXIT WHEN NOT FOUND;
			--RAISE NOTICE '%', _TBPType||' -> '||_TItemType;
			--RAISE NOTICE '%', COALESCE(_Price, 0)||' -> '||COALESCE(_DiscExp, 'disc');
			IF (_OldSelisih IS NULL OR (_Selisih<=_OldSelisih)) THEN
				_OldSelisih:=_Selisih;
				_retval.price:=_Price;
				_retval.discexp:=_DiscExp;
			END IF;
		END LOOP;
		CLOSE cRetval;

		RETURN NEXT _retval;
	ELSIF (_ModeBP=2 AND _ModeItem=0) THEN
		--RAISE NOTICE '%', _ModeBP||' -> '||_ModeItem;
		SELECT INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType 
			price1, disc1exp, (fqtyztoqty1(_ItemID,_Qty,_Unit)-fqtyztoqty1(itemid,qty,unit)), tbptype, titemtype
		FROM pprc WHERE active=TRUE AND tbptype='ALL' AND titemtype='ITEM' AND itemid=_ItemID 
		AND unit=_Unit AND qty<=_Qty
		AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
		IF (FOUND) THEN
			_retval.price:=_Price;
			_retval.discexp:=_DiscExp;
		ELSE
			SELECT INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType 
				price1, disc1exp, (fqtyztoqty1(_ItemID,_Qty,_Unit)-fqtyztoqty1(itemid,qty,unit)), tbptype, titemtype
			FROM pprc WHERE active=TRUE AND tbptype='ALL' AND titemtype='ITEM' AND itemid=_ItemID 
			AND unit!=_Unit AND fqtyztoqty1(itemid,qty,unit)<=fqtyztoqty1(_ItemID,_Qty,_Unit)
			AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
			IF (FOUND) THEN
				_retval.price:=_Price;
				_retval.discexp:=_DiscExp;
			END IF;
		END IF;

		RETURN NEXT _retval;
	ELSIF (_ModeBP=2 AND _ModeItem=1) THEN
		--RAISE NOTICE '%', _ModeBP||' -> '||_ModeItem;
		OPEN cRetval FOR 
			SELECT price1, disc1exp, (_total-minamt), tbptype, titemtype
			FROM pprc WHERE active=TRUE AND tbptype='ALL' AND titemtype='TOTAMT' AND minamt<=_Total
			AND (startdate IS NULL OR startdate<=_SaleDate) AND (enddate IS NULL OR enddate>=_SaleDate);
		LOOP
			FETCH cRetval INTO _Price, _DiscExp, _Selisih, _TBPType, _TItemType;
			EXIT WHEN NOT FOUND;
			--RAISE NOTICE '%', _TBPType||' -> '||_TItemType;
			--RAISE NOTICE '%', COALESCE(_Price, 0)||' -> '||COALESCE(_DiscExp, 'disc');
			IF (_OldSelisih IS NULL OR (_Selisih<=_OldSelisih)) THEN
				_OldSelisih:=_Selisih;
				_retval.price:=_Price;
				_retval.discexp:=_DiscExp;
			END IF;
		END LOOP;
		CLOSE cRetval;

		RETURN NEXT _retval;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBns (
	_itemid		VARCHAR,
	_saledate	DATE,
	_bpid		VARCHAR,
	_qty		NUMERIC,
	_unit		VARCHAR,
	_Total		NUMERIC
) RETURNS SETOF tBns AS
$BODY$
DECLARE
	_retval		tBns%ROWTYPE;
BEGIN
	/*--BEGIN*/
		_retval.bnsno:='';
		_retval.multi:=0;
		select INTO _retval bnsno,CASE WHEN qtymin=0 THEN CASE WHEN titemtype='TOTAMT' AND minamt<>0 THEN _Total/minamt ELSE 0 END ELSE _qty/qtymin END from bns
		WHERE 
		(
		tbptype='ALL' OR 
		(tbptype='BPID' AND ((custid IS NULL OR custid=_bpid) AND (vendorid IS NULL OR vendorid=_bpid))) OR
		(tbptype='BPGRP' AND (custid IS NULL OR fbpgrpin(_bpid,bpgrpid)))
		)
		AND 
		(
		titemtype='ALL' OR 
		(titemtype='ITGRP' AND (itgrpid IS NULL OR fitgrpin(_itemid,itgrpid)) )  OR
			/*--AND fqtyztoqty1(itemid,qtymin,unit)<fqtyztoqty1(_itemid,_qty,_unit)) OR */
		(titemtype='TOTAMT' AND minamt<=_Total)
		) AND StartDate<=_saledate AND EndDate>=_saleDate;

		RETURN NEXT _retval;
	/*--EXCEPTION*/
		/*--WHEN OTHERS THEN RETURN NEXT _retval;*/
		
	/*--END;*/
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spBns_New(_bnsno VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		spstatus%ROWTYPE;
BEGIN
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spBns_Void(_bnsno VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		spstatus%ROWTYPE;
	_count		int2;
	
BEGIN
	_retval.ErrNo:=10;
	_retval.ErrMsg:='ERROR';
	SELECT INTO _count count(*) FROM salebns WHERE bnsno=_bnsno;

	IF (_count>0) THEN
		_retval.ErrNo:=11;
		_retval.ErrMsg:='Sudah Ada Tansaksi yang menggunakan nomer Bonus ini!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;
	DELETE FROM bnst WHERE bnsno=_bnsno;
	DELETE FROM bnsd WHERE bnsno=_bnsno;
	DELETE FROM bns WHERE bnsno=_bnsno;

	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION fStockBal_EndDate(
	_ItemID		VARCHAR, 
	_WhID		VARCHAR,
	_PID		VARCHAR,
	_Date		DATE
) RETURNS NUMERIC(15,4) AS
$BODY$
DECLARE
	_LastMonthBal	NUMERIC(15,4);
	_YesterdayBal	NUMERIC(15,4);
BEGIN
	_LastMonthBal:=COALESCE(fStockBal_StartMonth(_ItemID, _WhID, _PID, _Date), 0);
	_YesterdayBal:=(_LastMonthBal+ COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)), 0)) 
			FROM stockad d JOIN stocka h ON h.stockano=d.stockano 
			WHERE itemid=_ItemID AND (_PID IS NULL OR d.pid=_PID) AND (_WhID IS NULL OR d.whid=_WhID) 
			AND h.stockadate>=fBegOfMonth(_Date) AND h.stockadate<=fDate2359(_Date);
	IF (_YesterdayBal IS NULL) THEN
		_YesterdayBal:=0;
	END IF;

	RETURN _YesterdayBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCons_Void (
	_ConsNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
 	_retval		SPStatus%ROWTYPE;
 	spret		SPStatus%ROWTYPE;
 
 	_SaleType	VARCHAR(5);
 	_TermType	VARCHAR(1);
 	_Term		INT;
 	_PoSPayMtd	VARCHAR(4);
 	_PayStatus	VARCHAR(1);
 	_DONo		VARCHAR(15);
 	_RcvNo		VARCHAR;
 	_NumOfSaleDO	INT;
	-- SO Stuff
 	_ItemID		VARCHAR(20);
	_Qty1		DECIMAL(15,4);
 	_Qty1Cancel	DECIMAL(15,4);
 	_Qty1Tot	DECIMAL(15,4);
	_JnlNo		VARCHAR(15);
	_SONo		VARCHAR;
	_SODNo		SMALLINT;
	_SODPID		VARCHAR;

	_SaleNo		VARCHAR;

 	cDOSale 	REFCURSOR;
 	cSO 		REFCURSOR;
 	cConsD2 	REFCURSOR;

	_isDraft	BOOLEAN;
 	_LockDate	DATE;
	_ConsDate	DATE;
BEGIN
	_NumOfSaleDO:=0;

	 SELECT INTO 
		_isDraft, _SaleType, _PayStatus, _ConsDate 
		isDraft, SaleType, PayStatus, ConsDate 
	FROM Cons WHERE ConsNo=_ConsNo;

	IF NOT FOUND THEN
		_retval.ErrNo=50001;
		_retval.ErrMsg='Konsinyasi tidak ditemukan! (spCons_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CONS';
	IF(_LockDate is not null AND _LockDate>_ConsDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Tanggal Konsinyasi melebihi batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _SaleNo saleno FROM saled WHERE ConsNo=_ConsNo;
	IF _SaleNo IS NOT NULL THEN
		_retval.ErrNo=50002;
		_retval.ErrMsg='Sudah ada Sell Out (#'||_ConsNo||') untuk Konsinyasi ini ! (spCons_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	END IF;

	 IF (_PayStatus<>'N') THEN
		_retval.ErrNo=50002;
		_retval.ErrMsg='Sudah ada transaksi pembayaran untuk Konsinyasi ini.VOID dulu pembayarannya! (spCons_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	 END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM SOCons WHERE ConsNo=_ConsNo;
		DELETE FROM ConsD WHERE ConsNo=_ConsNo;
		DELETE FROM Cons WHERE ConsNo=_ConsNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

/**
	TODO: pengecekan kalau Cons via Deo & Deli
	
**/
				
	SELECT INTO spret * FROM spStockA_Void_byRef('CONS', _ConsNo);
	IF spret.ErrNo<>0 THEN
		_retval.ErrNo:=spret.ErrNo;
		_retval.ErrMsg:=spret.ErrMsg;			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 

	OPEN cConsD2 FOR
		SELECT ItemID, SONo, SODNo, PID, fQtyZToQty1(ItemID, QtySellin, Unit) AS Qty1Tot 
		FROM ConsD WHERE ConsNo=_ConsNo AND SONo IS NOT NULL AND SONo<>'';
	LOOP
		FETCH cConsD2 INTO _ItemID, _SONo, _SODNo, _SODPID, _Qty1;
		EXIT WHEN NOT FOUND;

		_Qty1Cancel:=spSO_Item_Cancel(_SONo, _ItemID, _SODPID, _Qty1, _SODNo);

	END LOOP;
	CLOSE cConsD2;
		

	OPEN cSO FOR
	SELECT SONo FROM ConsD WHERE ConsNo=_ConsNo AND SONo IS NOT NULL AND SONo<>'' ORDER BY SONo;
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;

		SELECT INTO spret * FROM spSO_ChNUp_Status(_SONo);
		IF spret.ErrNo<>0 THEN
			_retval.ErrNo:=spret.ErrNo;
			_retval.ErrMsg:=spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		END LOOP;
	CLOSE cSO;


	IF (isAccInstalled()) THEN
		SELECT INTO _JnlNo jnlno FROM jnl WHERE RefType='CONS' AND RefNo=_ConsNo;
		SELECT INTO spret * from spJnl_Void(_JnlNo);
		IF spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM SOCons WHERE ConsNo=_ConsNo;
	DELETE FROM ConsD WHERE ConsNo=_ConsNo;
	DELETE FROM Cons WHERE ConsNo=_ConsNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE plpgsql;




CREATE OR REPLACE FUNCTION fStockDate(
	_Tipe		INT,
	_Date		DATE,
	_WhID 		VARCHAR, 
	_ItGrpID 	VARCHAR, 
	_BrandID 	VARCHAR, 
	_BPID		VARCHAR, 
	_ModelID	VARCHAR,
	_NotZero	BOOLEAN,
	_BigThanZero	BOOLEAN,
	_active		BOOLEAN
) RETURNS SETOF tStockDate AS
$BODY$
DECLARE
	_retval		tStockDate%ROWTYPE;

	_PID		VARCHAR(25); 
	_Qty		NUMERIC(15,4); 
	_QtyStock	NUMERIC(15,4);
	_Stock		VARCHAR; 
	_ItGrpName	VARCHAR(50); 
	_WhName		VARCHAR(50); 
	_ItemDesc	VARCHAR(50); 
	_ItemID		VARCHAR(20); 
	_BrandName	VARCHAR(50); 
	_UnitDesc	VARCHAR(25); 
	_ModelName	VARCHAR(50);
	_Subtotal	NUMERIC(19,4);

	cStock		REFCURSOR;

	__Date		DATE;

	_PYear		INT2;
	_Day		INT;
	_Month		INT;
	_StartDate	DATE;

	__Qty		NUMERIC(15,4);
	__QtyBal	NUMERIC(15,4);
BEGIN
	IF (_Tipe=0 OR _Date IS NULL) THEN
		__Date:=CURRENT_DATE;
	ELSE
		__Date:=_Date;
	END IF;

	_PYear:=EXTRACT(year FROM __Date);
	_Month:=EXTRACT(month FROM __Date);
	_Day:=EXTRACT(day FROM __Date);
	_StartDate:=TO_DATE((_PYear||'-'||_Month||'-01'), 'YYYY-MM-DD');
	_Month:=_Month-1;

	OPEN cStock FOR
		SELECT vw.itemid, i.itemdesc, 0 AS qty, it.itgrpname, 
		b.brandname AS brandid, i.unitdesc, m.modelname 
		FROM (SELECT DISTINCT itemid 
			FROM stock 
			WHERE (itemid<>'0' AND itemid<>'-1') AND (_WhID IS NULL OR whid=_WhID)
		) vw
		JOIN item i ON vw.itemid=i.itemid
		LEFT JOIN brand b ON i.brandid=b.brandid
		LEFT JOIN model m ON i.modelid=m.modelid
		LEFT JOIN itgrp it ON i.itgrpid=it.itgrpid
		WHERE (_BPID IS NULL OR i.vendorid=_BPID)
		AND (_ItGrpID IS NULL OR i.itgrpid IN (SELECT * FROM fItGrpList(_ItGrpID))) 
		AND (_BrandID IS NULL OR i.brandid=_BrandID)
		AND (_ModelID IS NULL OR i.modelid=_ModelID)
		ORDER BY it.itgrpname, vw.itemid; 
	LOOP
		FETCH cStock INTO _ItemID, _ItemDesc, _Qty, _ItGrpName, _BrandName, _UnitDesc, _ModelName;
		EXIT WHEN NOT FOUND;

		SELECT INTO __QtyBal * FROM fStockDate_Qty(_ItemID, _Month, _PYear, _WhID);
		_Qty:=_Qty+COALESCE(__QtyBal, 0);

		SELECT INTO __Qty SUM(fQtyZToQty1(s.itemid, s.qty, s.unit)) AS qty 
		FROM stockad s 
		JOIN stocka m ON s.stockano=m.stockano
		WHERE s.itemid=_ItemID AND (_WhID IS NULL OR s.whid=_WhID) 
		AND m.stockadate>=_StartDate AND m.stockadate<=__Date;

		IF NOT FOUND THEN
			__Qty:=0;
		END IF;

		SELECT INTO _Subtotal SUM(s.subtotal) 
		FROM stockad s 
		JOIN stocka m ON s.stockano=m.stockano
		WHERE s.itemid=_ItemID AND (_WhID IS NULL OR s.whid=_WhID) 
		AND m.stockadate<=__Date;

		_Subtotal:=COALESCE(_Subtotal, 0);

		_Qty:=_Qty+COALESCE(__Qty,0);

		_Stock:=fQty1Desc(_ItemID, _Qty);

		IF (((_NotZero=TRUE AND _Qty!=0) OR (_NotZero=FALSE)) AND ((_BigThanZero=TRUE AND _Qty>0) OR (_BigThanZero=FALSE))) THEN 
			_retval.itemid:=_ItemID;
			_retval.itemdesc:=_ItemDesc;
			_retval.qty:=_Qty;
			_retval.stock:=_Stock;
			_retval.itgrpname:=_ItGrpName;
			_retval.brandname:=_BrandName;
			_retval.unitdesc:=_UnitDesc;
			_retval.modelname:=_ModelName;
			_retval.subtotal:=_Subtotal;

			RETURN NEXT _retval;
		END IF;
	END LOOP;
	CLOSE cStock;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fStockDate(
	_Tipe		INT,
	_Date		DATE,
	_WhID 		VARCHAR, 
	_ItGrpID 	VARCHAR, 
	_BrandID 	VARCHAR, 
	_BPID		VARCHAR, 
	_ModelID	VARCHAR,
	_NotZero	BOOLEAN,
	_BigThanZero	BOOLEAN
) RETURNS SETOF tStockDate AS
$BODY$
DECLARE
	_retval		tStockDate%ROWTYPE;
	cStock		REFCURSOR;
BEGIN
	OPEN cStock FOR 
		SELECT * FROM fStockDate(_Tipe, _Date, _WhID, _ItGrpID, _BrandID, _BPID, _ModelID, _NotZero, _BigThanZero, TRUE);
	LOOP
		FETCH cStock INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cStock;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStockDate_Qty (
	_ItemID		VARCHAR, 
	_Month		INT, 
	_PYear		INT, 
	_WhID		VARCHAR
) RETURNS NUMERIC(15,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(15,4);
	_MaxYear	INT;
	_PID		VARCHAR;

	_TotalBal	NUMERIC(15,4);
	cBal		REFCURSOR;
BEGIN
	/** inisialisasi */
	_retval:=0;

	OPEN cBal FOR
		SELECT pid, MAX(pyear) FROM stockbal WHERE itemid=_ItemID AND (_WhID IS NULL OR whid=_WhID) 
		GROUP BY pid;	
	LOOP
		FETCH cBal INTO _PID, _MaxYear;
		EXIT WHEN NOT FOUND;

		IF (_MaxYear<_PYear) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)+COALESCE(stockbal10, 0)+COALESCE(stockbal11, 0)+COALESCE(stockbal12, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_MaxYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
			CONTINUE;
		END IF;

		IF (_Month=0) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=1) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=2) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=3) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;
				
			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=4) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=5) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=6) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=7) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=8) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=9) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=10) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)+COALESCE(stockbal10, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=11) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)+COALESCE(stockbal10, 0)+COALESCE(stockbal11, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=12) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)+COALESCE(stockbal10, 0)+COALESCE(stockbal11, 0)+COALESCE(stockbal12, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;
	
			_retval:=_retval+COALESCE(_TotalBal, 0);
		END IF;
	END LOOP;
	CLOSE cBal;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStockDatePID(
	_Tipe		INT,
	_Date		DATE,
	_WhID 		VARCHAR, 
	_ItGrpID 	VARCHAR, 
	_BrandID 	VARCHAR, 
	_BPID		VARCHAR, 
	_ModelID	VARCHAR,
	_NotZero	BOOLEAN,
	_BigThanZero	BOOLEAN,
	_active		BOOLEAN
) RETURNS SETOF tStockDate AS
$BODY$
DECLARE
	_retval		tStockDate%ROWTYPE;

	_PID		VARCHAR(25); 
	_Qty		NUMERIC(15,4); 
	_Stock		VARCHAR; 
	_ItGrpName	VARCHAR(50); 
	_WhName		VARCHAR(50); 
	_ItemDesc	VARCHAR(50); 
	_ItemID		VARCHAR(20); 
	_BrandName	VARCHAR(50); 
	_UnitDesc	VARCHAR(25); 
	_ModelName	VARCHAR(50);

	cStock		REFCURSOR;

	__Date		DATE;

	_PYear		INT2;
	_Day		INT;
	_Month		INT;
	_StartDate	DATE;

	__Qty		NUMERIC(15,4);
	__QtyBal	NUMERIC(15,4);
BEGIN
	IF (_Tipe=0 OR _Date IS NULL) THEN
		__Date:=CURRENT_DATE;
	ELSE
		__Date:=_Date;
	END IF;

	_PYear:=EXTRACT(year FROM __Date);
	_Month:=EXTRACT(month FROM __Date);
	_Day:=EXTRACT(day FROM __Date);
	_StartDate:=TO_DATE((_PYear||'-'||_Month||'-01'), 'YYYY-MM-DD');
	_Month:=_Month-1;

	OPEN cStock FOR
		SELECT vw.itemid, vw.pid, i.itemdesc, 0 AS qty, it.itgrpname, 
		b.brandname AS brandid, i.unitdesc, m.modelname 
		FROM (SELECT DISTINCT itemid, pid 
			FROM stock 
			WHERE (itemid<>'0' AND itemid<>'-1') AND (_WhID IS NULL OR whid=_WhID)
		) vw
		JOIN item i ON vw.itemid=i.itemid
		LEFT JOIN brand b ON i.brandid=b.brandid
		LEFT JOIN model m ON i.modelid=m.modelid
		LEFT JOIN itgrp it ON i.itgrpid=it.itgrpid
		WHERE (_BPID IS NULL OR i.vendorid=_BPID)
		AND (_ItGrpID IS NULL OR i.itgrpid IN (SELECT * FROM fItGrpList(_ItGrpID))) 
		AND (_BrandID IS NULL OR i.brandid=_BrandID)
		AND (_ModelID IS NULL OR i.modelid=_ModelID)
		ORDER BY it.itgrpname, vw.itemid, vw.pid; 
	LOOP
		FETCH cStock INTO _ItemID, _PID, _ItemDesc, _Qty, _ItGrpName, _BrandName, _UnitDesc, _ModelName;
		EXIT WHEN NOT FOUND;
		
		SELECT INTO __QtyBal * FROM fStockDatePID_Qty(_ItemID, _Month, _PYear, _WhID, _PID);
		_Qty:=_Qty+COALESCE(__QtyBal, 0);

		SELECT INTO __Qty sum(fQtyZToQty1(s.itemid, s.qty, s.unit)) AS qty FROM stockad s 
		JOIN stocka m ON s.stockano=m.stockano
		WHERE s.itemid=_ItemID AND (_WhID IS NULL OR s.whid=_WhID) 
		AND (_PID IS NULL OR s.pid=_PID) AND m.stockadate>=_StartDate AND m.stockadate<=__Date;

		IF NOT FOUND THEN
			__Qty:=0;
		END IF;

		_Qty:=_Qty+COALESCE(__Qty,0);
		_Stock:=fQty1Desc(_ItemID, _Qty);

		IF (((_NotZero=TRUE AND _Qty!=0) OR (_NotZero=FALSE)) AND ((_BigThanZero=TRUE AND _Qty>0) OR (_BigThanZero=FALSE))) THEN 
			_retval.itemid:=_ItemID;
			_retval.itemdesc:=_ItemDesc;
			_retval.qty:=_Qty;
			_retval.stock:=_Stock;
			_retval.itgrpname:=_ItGrpName;
			_retval.brandname:=_BrandName;
			_retval.unitdesc:=_UnitDesc;
			_retval.modelname:=_ModelName;
			_retval.pid:=_PID;

			RETURN NEXT _retval;
		END IF;
	END LOOP;
	CLOSE cStock;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION fStockDatePID(
	_Tipe		INT,
	_Date		DATE,
	_WhID 		VARCHAR, 
	_ItGrpID 	VARCHAR, 
	_BrandID 	VARCHAR, 
	_BPID		VARCHAR, 
	_ModelID	VARCHAR,
	_NotZero	BOOLEAN,
	_BigThanZero	BOOLEAN
) RETURNS SETOF tStockDate AS
$BODY$
DECLARE
	_retval		tStockDate%ROWTYPE;
	cStock		REFCURSOR;
BEGIN
	OPEN cStock FOR 
		SELECT * FROM fStockDatePID(_Tipe, _Date, _WhID, _ItGrpID, _BrandID, _BPID, _ModelID, _NotZero, _BigThanZero, TRUE);
	LOOP
		FETCH cStock INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cStock;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION fStockDatePID_Qty(
	_ItemID		VARCHAR, 
	_Month		INT, 
	_PYear		INT, 
	_WhID		VARCHAR, 
	_PID		VARCHAR
) RETURNS NUMERIC(15,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(15,4);
	_MaxYear	INT;

	_TotalBal	NUMERIC(15,4);
	cBal		REFCURSOR;
BEGIN
	/** inisialisasi */
	_retval:=0;

	OPEN cBal FOR
		SELECT MAX(pyear) FROM stockbal WHERE itemid=_ItemID AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;
	LOOP
		FETCH cBal INTO _MaxYear;
		EXIT WHEN NOT FOUND;

		IF (_MaxYear<_PYear) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)+COALESCE(stockbal10, 0)+COALESCE(stockbal11, 0)+COALESCE(stockbal12, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_MaxYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
			CONTINUE;
		END IF;

		IF (_Month=0) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_pyear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=1) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=2) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=3) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;
				
			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=4) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=5) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=6) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=7) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=8) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=9) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=10) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)+COALESCE(stockbal10, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=11) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)+COALESCE(stockbal10, 0)+COALESCE(stockbal11, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;

			_retval:=_retval+COALESCE(_TotalBal, 0);
		ELSIF (_Month=12) THEN
			SELECT INTO _TotalBal SUM(COALESCE(stockbal00, 0)+COALESCE(stockbal01, 0)+COALESCE(stockbal02, 0)+COALESCE(stockbal03, 0)+COALESCE(stockbal04, 0)+COALESCE(stockbal05, 0)+COALESCE(stockbal06, 0)+COALESCE(stockbal07, 0)+COALESCE(stockbal08, 0)+COALESCE(stockbal09, 0)+COALESCE(stockbal10, 0)+COALESCE(stockbal11, 0)+COALESCE(stockbal12, 0)) AS qty 
			FROM stockbal 
			WHERE itemid=_ItemID AND pyear=_PYear AND (_WhID IS NULL OR whid=_WhID) AND pid=_PID;
	
			_retval:=_retval+COALESCE(_TotalBal, 0);
		END IF;
	END LOOP;
	CLOSE cBal;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStockBal_StartDate(
	_ItemID		VARCHAR, 
	_WhID		VARCHAR,
	_PID		VARCHAR,
	_Date		DATE
) RETURNS NUMERIC(15,4) AS
$BODY$
DECLARE
	_LastMonthBal	NUMERIC(15,4);
	_YesterdayBal	NUMERIC(15,4);
BEGIN
	_LastMonthBal:=COALESCE(fStockBal_StartMonth(_ItemID, _WhID, _PID, _Date), 0);
	_YesterdayBal:=(_LastMonthBal+ COALESCE(SUM(fQtyZToQty1(itemid, qty, unit)), 0)) 
			FROM stockad d JOIN stocka h ON h.stockano=d.stockano 
			WHERE itemid=_ItemID AND (_PID IS NULL OR d.pid=_PID) AND (_WhID IS NULL OR d.whid=_WhID) 
			AND h.stockadate>=fBegOfMonth(_Date) AND h.stockadate<_Date;
	IF (_YesterdayBal IS NULL) THEN
		_YesterdayBal:=0;
	END IF;

	RETURN _YesterdayBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStockBal_StartMonth(
	_ItemID		VARCHAR, 
	_WhID		VARCHAR, 
	_PID		VARCHAR, 
	_StartDate	DATE
) RETURNS NUMERIC(15,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(15,4);

	_Month		INT;
	_PYear		INT;
BEGIN
	SELECT INTO _Month * FROM date_part('month', CAST(_StartDate AS DATE));
	SELECT INTO _PYear * FROM date_part('year', CAST(_StartDate AS DATE));

	IF (_Month IS NULL) THEN
		_Month:=0;
	ELSE
		_Month:=_Month-1;
	END IF;
	
	SELECT INTO _retval SUM(fStockDatePID_Qty(itemid, _Month, _PYear, _WhID, s.pid)) AS qty 
	FROM (SELECT DISTINCT itemid, pid FROM stock s 
		WHERE (_WhID IS NULL OR s.whid=_WhID) AND (_PID IS NULL OR s.pid=_PID) AND itemid=_ItemID) s;

	RETURN COALESCE(_retval, 0);
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStockCard(
	_ItemID 	VARCHAR, 
	_WhID 		VARCHAR, 
	_PPID 		VARCHAR, 
	_StartDate 	DATE, 
	_EndDate 	DATE
) RETURNS SETOF stockrpt AS
$BODY$
DECLARE
	_retval			stockrpt%ROWTYPE;
	_StockANo		VARCHAR(15);
	_StockADate		DATE;
	_StockAType		VARCHAR(5);
	_BpName			VARCHAR(30);
	_RefType		VARCHAR(5);
	_RefNo			VARCHAR(15);
	_StockANote		TEXT;
	_InOut			VARCHAR(1);
	_StockIn		NUMERIC(15,4);
	_In			NUMERIC(15,4);
	_Out			NUMERIC(15,4);
	_Bal			NUMERIC(15,4);
	_StockOut		NUMERIC(15,4);
	_StockBal		NUMERIC(15,4);
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_QtyUnit1		NUMERIC(15,4);
	_YesterdayBal		NUMERIC(15,4);
	_PID			VARCHAR(25);
	cStockAD		REFCURSOR;
BEGIN
	_In :=0;
	_Out :=0;

	IF (_PPID='') THEN
		_YesterdayBal := fStockBal_StartDate(_ItemID, _WhID, NULL, _StartDate);
	ELSE	
		_YesterdayBal := fStockBal_StartDate(_ItemID, _WhID, _PPID, _StartDate);
	END IF;

	IF _PPID IS NULL OR _PPID='' THEN
		_PID='%%';
	ELSE
		_PID='%'||_PPID|| '%';
	END IF;

	_StockBal:=_YesterdayBal;
	_retval.stockano:=NULL;
	_retval.stockadate:=_StartDate;
	_retval.stockatype:='BAL';
	_retval.bpname:=NULL;
	_retval.reftype:=NULL;
	_retval.refno:=NULL;
	_retval.stockanote:='Saldo';
	_retval.inout:='B';
	_retval.stockin:=NULL;
	_retval.stockout:=NULL;
	_retval.stockbal:=fQty1Desc(_ItemID,_StockBal);

	/*--RETURN NEXT _retval; */
	OPEN cStockAD FOR
		SELECT h.StockANo, h.StockADate, h.StockAType, bp.bpname, d.pid,h.RefType, h.RefNo, h.StockANote,
			  	h.InOut, d.Qty, d.Unit 
		FROM StockAD d JOIN StockA h ON d.StockANo=h.StockANo LEFT JOIN bp on h.bpid=bp.bpid
--		WHERE d.ItemID=_ItemID AND d.WhID=_WhID AND d.pid like _PID AND
		WHERE d.ItemID=_ItemID AND d.WhID=_WhID AND ((_PID='%%') OR (d.pid LIKE _PID)) AND
				h.StockADate>=_StartDate AND h.StockADate<=fDate2359(_EndDate)
		ORDER BY h.StockADate, h.StockANo;
	LOOP
		FETCH cStockAD INTO _StockANo, _StockADate, _StockAType, _BpName, _PID,_RefType, _RefNo, _StockANote,	_InOut, _Qty, _Unit;
		EXIT WHEN NOT FOUND;

		_QtyUnit1 := fQtyZToQty1(_ItemID, _Qty, _Unit);
		IF (_QtyUnit1>=0) THEN
			_StockIn := _QtyUnit1;
			IF(_QtyUnit1=0)THEN
				_StockIn:=0;
			END IF;
			_StockOut := 0;
			_InOut := 'I';
		ELSE
			_StockIn := 0;
			_StockOut := ABS(_QtyUnit1);
			_InOut := 'O';
		END IF;

		RETURN NEXT _retval;

		_StockBal=_StockBal+_QtyUnit1;
		_retval.stockano:=_StockANo;
		_retval.stockadate:=_StockADate;
		_retval.stockatype:=_StockAType;
		_retval.bpname:=_BpName;
		_retval.pid:=_PID;
		_retval.reftype:=_RefType;
		_retval.refno:=_RefNo;
		_retval.stockanote:=_StockANote;
		_retval.inout:=_InOut;

		IF(_StockIn=0) THEN
			_retval.stockin:=NULL;
		ELSE
			_retval.stockin:=fQty1Desc(_ItemID, _StockIn);
		END IF;

		IF(_StockOut=0) THEN
			_retval.stockout:=NULL;
		ELSE
			_retval.stockout:=fQty1Desc(_ItemID, _StockOut);
		END IF;

		IF(_StockBal=0) THEN
			_retval.stockbal:=0;
		ELSE
			_retval.stockbal:=fQty1Desc(_ItemID, _StockBal);
		END IF;
	END LOOP;
	CLOSE cStockAD;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fRekapStockCard (
	_ItemID		VARCHAR, 
	_WhID		VARCHAR, 
	_StartDate	DATE, 
	_EndDate	DATE
) RETURNS SETOF stockrpt AS
$BODY$
DECLARE
	_retval			stockrpt%ROWTYPE;
	_StockANo		VARCHAR(15);
	_StockADate		DATE;
	_StockAType		VARCHAR(5);
	_BpName			VARCHAR;
	_RefType		VARCHAR(5);
	_RefNo			VARCHAR(15);
	_StockANote		TEXT;
	_InOut			VARCHAR(1);
	_StockIn		NUMERIC(15,4);
	_In			NUMERIC(15,4);
	_Out			NUMERIC(15,4);
	_Bal			NUMERIC(15,4);
	_StockOut		NUMERIC(15,4);
	_StockBal		NUMERIC(15,4);
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR(5);
	_QtyUnit1		NUMERIC(15,4);
	_YesterdayBal		NUMERIC(15,4);
	
	cStockAD		REFCURSOR;
BEGIN
	_In:=0;
	_Out:=0;

	_YesterdayBal:=fStockBal_StartDate(_ItemID, _WhID, null, _StartDate);
	_StockBal:=_YesterdayBal;
	
	_retval.stockano:=NULL;
	_retval.stockadate:=_StartDate;
	_retval.stockatype:='BAL';
	_retval.bpname:=NULL;
	_retval.reftype:=NULL;
	_retval.refno:=NULL;
	_retval.stockanote:='Saldo';
	_retval.inout:='B';
	_retval.stockin:=NULL;
	_retval.stockout:=NULL;
	_retval.stockbal:=fQty1Desc(_ItemID,_StockBal);

	/*--RETURN NEXT _retval; */
	OPEN cStockAD FOR
		SELECT h.StockANo, h.StockADate, h.StockAType, bp.bpname, h.RefType, h.RefNo, h.StockANote,
		  	h.InOut, sum(fQtyZToQty1(d.ItemID, d.Qty, d.Unit))
		FROM StockAD d LEFT JOIN StockA h ON d.StockANo=h.StockANo LEFT JOIN bp on h.bpid=bp.bpid
		WHERE d.ItemID=_ItemID AND d.WhID=_WhID AND 
				h.StockADate>=_StartDate AND h.StockADate<=fDate2359(_EndDate)
		GROUP BY h.StockANo, h.StockADate, h.StockAType, bp.bpname, h.RefType, h.RefNo, h.StockANote,
			  	h.InOut
		ORDER BY h.StockADate, h.StockANo;

	LOOP
		FETCH cStockAD INTO _StockANo, _StockADate, _StockAType, _BpName, _RefType, _RefNo, _StockANote,_InOut, _QtyUnit1;
		EXIT WHEN NOT FOUND;

		IF (_QtyUnit1>=0) THEN
			_StockIn:=_QtyUnit1;
			IF(_QtyUnit1=0)THEN
				_StockIn:=0;
			END IF;
				_StockOut:=0;
				_InOut:='I';
			ELSE
				_StockIn:=0;
				_StockOut:=ABS(_QtyUnit1);
				_InOut:='O';
			END IF;
		
			RETURN NEXT _retval;

			_StockBal=_StockBal+_QtyUnit1;
			_retval.stockano:=_StockANo;
			_retval.stockadate:=_StockADate;
			_retval.stockatype:=_StockAType;
			_retval.bpname:=_BpName;
			_retval.reftype:=_RefType;
			_retval.refno:=_RefNo;
			_retval.stockanote:=_StockANote;
			_retval.inout:=_InOut;
		IF(_StockIn=0) THEN
			_retval.stockin:=NULL;
		ELSE
			_retval.stockin:=fQty1Desc(_itemID,_StockIn);
		END IF;
		
		IF(_StockOut=0)THEN
			_retval.stockout:=NULL;
		ELSE
			_retval.stockout:=fQty1Desc(_itemID,_StockOut);
		END IF;

		IF(_StockBal=0)THEN
			_retval.stockbal:=0;
		ELSE
			_retval.stockbal:=fQty1Desc(_itemID,_StockBal);
		END IF;
	END LOOP;
	CLOSE cStockAD;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: fstock_mutasi_rekap(_StartDate date, _EndDate date, _WhId varchar, _ItGrpId varchar, _BPId varchar, _Brand varchar, _Model varchar)*/

/*-- DROP FUNCTION fstock_mutasi_rekap(_StartDate date, _EndDate date, _WhId varchar, _ItGrpId varchar, _BPId varchar, _Brand varchar, _Model varchar);*/


CREATE OR REPLACE FUNCTION fstock_mutasi_rekap(
	_StartDate 	DATE, 
	_EndDate 	DATE, 
	_WhId 		VARCHAR, 
	_ItGrpId 	VARCHAR, 
	_BPId 		VARCHAR, 
	_Brand 		VARCHAR, 
	_Model 		VARCHAR
) RETURNS SETOF tStTr AS
$BODY$
DECLARE
	_retval			tStTr%ROWTYPE;

	_Merek			VARCHAR(50);
	_ItGrpDesc		VARCHAR(50);
	_ItemDesc		VARCHAR(50);

	_StockAwal		VARCHAR(25);
	_StBeli			VARCHAR(25);
	_StrJual		VARCHAR(25);
	_TotalBrg		VARCHAR(25);

	_StJual			VARCHAR(25);
	_StrBeli		VARCHAR(25);
	_StockTmp		VARCHAR(25);
	_StockOpname		VARCHAR(25);
	_Selisih		VARCHAR(25);
	_StockAkhir		VARCHAR(25);
	_StCanvasKeluar 	VARCHAR(25);
	_StCanvasMasuk  	VARCHAR(25);

	_PId			VARCHAR(25);
	_StTrNo			VARCHAR(15);
	_StockANo		VARCHAR(15);
	_ItemId			VARCHAR(20);
	__WhId			VARCHAR(3);

	_StockAwal1		NUMERIC(15,4);
	_StBeli1		NUMERIC(15,4);
	_StrJual1		NUMERIC(15,4);
	_TotalBrg1		NUMERIC(15,4);

	_StJual1		NUMERIC(15,4);
	_StrBeli1		NUMERIC(15,4);
	_StockTmp1		NUMERIC(15,4);
	_StockOpname1		NUMERIC(15,4);
	_Selisih1		NUMERIC(15,4);
	_StCanvasKeluar1 	NUMERIC(15,4);
	_StCanvasMasuk1  	NUMERIC(15,4);

	_TotStockAwal1		NUMERIC(15,4);
	_TotStockTmp1		NUMERIC(15,4);
	_StockAkhir1  		NUMERIC(15,4);

	_Month			INT2;
	_PYear			INT4;
	cItem			REFCURSOR;
	cWh			REFCURSOR;
BEGIN
	_PId:=NULL;
	
	OPEN cItem FOR
		SELECT b.brandname, ig.itgrpname, i.itemdesc, i.itemid
		FROM item i
		LEFT JOIN itgrp ig ON i.itgrpid=ig.itgrpid
		LEFT JOIN brand b ON i.brandid=b.brandid
		LEFT JOIN model m ON i.modelid=m.modelid
		WHERE i.active=true AND (i._xt!=true OR (i.itemid<>'0' AND i.itemid<>'-1'))
		AND (_ItGrpId IS NULL OR i.itgrpid IN (SELECT * FROM fItGrpList(_ItGrpId)))
		AND (_Brand IS NULL OR i.brandid=_Brand)
		AND (_Model IS NULL OR i.modelid=_Model)
		AND (_BPID IS NULL OR i.vendorid=_BPID)
		ORDER BY b.brandname, ig.itgrpname;
	LOOP
		FETCH cItem INTO _Merek, _ItGrpDesc, _ItemDesc, _ItemId;
		EXIT WHEN NOT FOUND;
						
		_StockAwal1:=fStockBal_StartDate(_ItemID, _WhId, _PId, _StartDate);
		_StockAwal:=fqty1desc(_ItemId, _StockAwal1);

		/*--RAISE EXCEPTION '%', _ItemID||','||_WhId||','||_PId||','||_StartDate;*/
						
		SELECT INTO _StBeli1, _StBeli 
			SUM(fQtyZToQty1(purcd.itemid,purcd.qty,purcd.unit)), fqty1desc(purcd.itemid,SUM(fQtyZToQty1(purcd.itemid,purcd.qty,purcd.unit)))
		FROM purcd JOIN purc ON purc.purcno=purcd.purcno
		WHERE purcd.itemid=_ItemId
		AND purc.purcdate>=fNoTime(CAST(_StartDate AS DATE))
		AND purc.purcdate<=fDate2359(CAST(_EndDate AS DATE))
		AND (_WhId IS NULL OR purcd.whid=_WhId)
		GROUP BY purcd.itemid;
					
		SELECT INTO _StrJual1, _StrJual 
			SUM(fQtyZToQty1(sretd.itemid,sretd.qty,sretd.unit)), fqty1desc(sretd.itemid,SUM(fQtyZToQty1(sretd.itemid,sretd.qty,sretd.unit)))
		FROM sret JOIN sretd ON sret.sretno=sretd.sretno
		WHERE sretd.itemid=_ItemId
		AND sret.sretdate>=fNoTime(CAST(_StartDate AS DATE))
		AND sret.sretdate<=fDate2359(CAST(_EndDate AS DATE))
		AND (_WhId IS NULL OR sretd.whid=_WhId)
		GROUP BY sretd.itemid;

		SELECT INTO _StCanvasMasuk1, _StCanvasMasuk 
			SUM(fQtyZToQty1(sttrd.itemid,sttrd.qty,sttrd.unit)), fqty1desc(sttrd.itemid,SUM(fQtyZToQty1(sttrd.itemid,sttrd.qty,sttrd.unit)))
		FROM sttrd JOIN sttr ON sttr.sttrno=sttrd.sttrno
		WHERE sttrd.itemid=_ItemId
		AND sttr.sttrdate>=fNoTime(CAST(_StartDate AS DATE))
		AND sttr.sttrdate<=fDate2359(CAST(_EndDate AS DATE))
		AND (_WhId IS NULL OR whid2=_WhId)
		GROUP BY sttrd.itemid;

		_TotalBrg1:=COALESCE(_StockAwal1,0)+COALESCE(_StBeli1,0)+COALESCE(_StrJual1,0)+COALESCE(_StCanvasMasuk1,0);
		SELECT INTO _TotalBrg * FROM fqty1desc(_ItemId, _TotalBrg1);

		SELECT INTO _StJual1, _StJual 
			SUM(fQtyZToQty1(saled.itemid,saled.qty,saled.unit)), fqty1desc(saled.itemid,SUM(fQtyZToQty1(saled.itemid,saled.qty,saled.unit)))
		FROM saled JOIN sale ON sale.saleno=saled.saleno
		WHERE saled.itemid=_ItemId
		AND sale.saledate>=fNoTime(CAST(_StartDate AS DATE))
		AND sale.saledate<=fDate2359(CAST(_EndDate AS DATE))
		AND (_WhId IS NULL OR saled.whid=_WhId)
		GROUP BY saled.itemid;

		/*SELECT INTO _StrBeli1, _StrBeli 
			SUM(fQtyZToQty1(cnd.itemid,cnd.qty,cnd.unit)), fqty1desc(cnd.itemid,SUM(fQtyZToQty1(cnd.itemid,cnd.qty,cnd.unit)))
		FROM cnd JOIN cn ON cn.cnno=cnd.cnno
		WHERE cnd.itemid=_ItemId
		AND cn.cndate>=fNoTime(CAST(_StartDate AS DATE))
		AND cn.cndate<=fDate2359(CAST(_EndDate AS DATE))
		GROUP BY cnd.itemid;*/

		SELECT INTO _StrBeli1, _StrBeli 
			SUM(fQtyZToQty1(pretd.itemid,pretd.qty,pretd.unit)), fqty1desc(pretd.itemid,SUM(fQtyZToQty1(pretd.itemid,pretd.qty,pretd.unit)))
		FROM pretd JOIN pret ON pret.pretno=pretd.pretno
		WHERE pretd.itemid=_ItemId
		AND pret.pretdate>=fNoTime(CAST(_StartDate AS DATE))
		AND pret.pretdate<=fDate2359(CAST(_EndDate AS DATE))
		AND (_WhId IS NULL OR pretd.whid=_WhId)
		GROUP BY pretd.itemid;

		SELECT INTO _StCanvasKeluar1, _StCanvasKeluar 
			SUM(fQtyZToQty1(sttrd.itemid,sttrd.qty,sttrd.unit)), fqty1desc(sttrd.itemid,SUM(fQtyZToQty1(sttrd.itemid,sttrd.qty,sttrd.unit)))
		FROM sttrd JOIN sttr ON sttr.sttrno=sttrd.sttrno
		WHERE sttrd.itemid=_ItemId
		AND sttr.sttrdate>=fNoTime(CAST(_StartDate AS DATE))
		AND sttr.sttrdate<=fDate2359(CAST(_EndDate AS DATE))
		AND (_WhId IS NULL OR whid1=_WhId)
		AND (whid2<>'03')
		GROUP BY sttrd.itemid;

		_StockTmp1:=COALESCE(_TotalBrg1,0)-COALESCE(_StJual1,0)-COALESCE(_StrBeli1,0)-COALESCE(_StCanvasKeluar1,0);
		_TotStockTmp1:=COALESCE(_StockTmp1, 0);
		_StockTmp:=fqty1desc(_ItemId, _StockTmp1);

				
		SELECT INTO _StockAkhir1 * FROM fStockBal_EndDate(_ItemID, _WhId, _PId, _EndDate);
		_StockAkhir:=fqty1desc(_ItemId, _StockAkhir1);

		/**SELECT INTO _StockAkhir1 qty FROM stock WHERE itemid=_ItemID AND whid=_WhID AND (pid='' OR pid IS NULL);
		_StockAkhir:=fQty1Desc(_ItemID, _StockAkhir1);
		*/

		SELECT INTO _StockOpname1 SUM(fQtyZToQty1(d.itemid, d.qty, d.unit))
		FROM StockAD d JOIN StockA h ON h.StockANo=d.StockANo
		WHERE h.StockAType in ('SADJ','BEG','SIN','SOUT') AND d.ItemID=_ItemId
		AND h.StockADate>=fNoTime(CAST(_StartDate AS DATE))
		AND h.StockADate<=fDate2359(CAST(_EndDate AS DATE))
		AND (_WhId IS NULL OR d.whid=_WhId)
		GROUP BY d.itemid;
		_StockOpname:=fqty1desc(_ItemId, _StockOpname1);

		/*--_Selisih1:=abs(COALESCE(_StockTmp1,0)-COALESCE(_StockOpname1,0));*/
		/*--_Selisih1:=(_TotalBrg1+COALESCE(_StockOpname1,0))-(COALESCE(_StockTmp1,0)+_StockAkhir1);*/

		/**_Selisih1:=_TotalBrg1+(COALESCE(_StockOpname1,0));
		_Selisih1:=_Selisih1 - (_StockAkhir1 + _StJual1+COALESCE(_StrBeli1,0)+COALESCE(_StCanvasKeluar1,0));		*/

		/*--_Selisih1:=_StJual1;*/
		/*--_Selisih1:=_StrBeli1;*/
/** 		IF (_ItemId='BK264.B.HIAS.50X50') THEN
			RAISE EXCEPTION 'totstock %', COALESCE(_TotStockTmp1, 0)||' <> stock op '||_StockOpname1||' <> stock akhir '||_StockAkhir1;
		END IF;
*/
		_Selisih1:=COALESCE(_TotStockTmp1, 0)+_StockOpname1-_StockAkhir1;
		_Selisih:=fqty1desc(_ItemId, _Selisih1);

		_retval.merek:=_Merek;
		_retval.itgrpdesc:=_ItGrpDesc;
		_retval.itemid:=_ItemId;
		_retval.itemdesc:=_ItemDesc;
		_retval.stockawal:=_StockAwal;
		_retval.stpabrik:=_StBeli;
		_retval.strtoko:=_StrJual;
		_retval.totalbrg:=_TotalBrg;
		_retval.stsales:=_StJual;
		_retval.strpabrik:=_StrBeli;
		_retval.stockakhir:=_StockAkhir;/*--_StockTmp;*/
		_retval.stockopname:=_StockOpname;
		_retval.selisih:=_Selisih;
		_retval.canvasmasuk:=_StCanvasMasuk;
		_retval.canvaskeluar:=_StCanvasKeluar;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cItem;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION fstock_mutasi_rekap(_StartDate date, _EndDate date, _WhId varchar, _ItGrpId varchar, _BPId varchar, _Brand varchar, _Model varchar) OWNER TO bits;




CREATE OR REPLACE FUNCTION fstockvalue(
	_Mode		INT,
	_Date		DATE,
	_whid 		VARCHAR, 
	_itgrpid 	VARCHAR, 
	_brandid 	VARCHAR,
	_bpid 		VARCHAR, 
	_typeValue 	VARCHAR
) RETURNS SETOF tstockval AS
$BODY$
DECLARE
	_retval		tStockVal%ROWTYPE;

	cHPP		REFCURSOR;
	cSOLD		REFCURSOR;
BEGIN
	IF (_typeValue='HPP') THEN
		OPEN cHPP FOR
			SELECT f.itgrpname, f.itemid, f.itemdesc, f.brandname AS brandid, f.stock, 
			(CASE WHEN f.qty>0 THEN (f.subtotal/f.qty) ELSE 0 END) AS harga, f.subtotal
			FROM fStockDate(_Mode, _Date, _whid, _itgrpid, _brandid, _bpid, null, true, false) f 
			ORDER BY f.itgrpname, f.itemid;
		LOOP
			FETCH cHPP INTO _retval;
			EXIT WHEN NOT FOUND;
			
			RETURN NEXT _retval;
		END LOOP;
	ELSIF (_typeValue='Harga Jual') THEN
		OPEN cSold FOR
			SELECT f.itgrpname, f.itemid, f.itemdesc, f.brandname AS brandid, f.stock, 
			price1 AS harga, (f.qty*price1) AS subtotal
			FROM fStockDate(_Mode, _Date, _whid, _itgrpid, _brandid, _bpid, null, true, false) f
			LEFT JOIN prc ON prc.itemid=f.itemid
			WHERE prc.prclvlid='0' 
			ORDER BY f.itgrpname, f.itemid;
		LOOP
			FETCH cSold INTO _retval;
			EXIT WHEN NOT FOUND;
			
			RETURN NEXT _retval;
		END LOOP;
	END IF;	
END
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: fhutangjatuhtempo(_tgl DATE, _vendorid VARCHAR)*/

/*-- DROP FUNCTION fhutangjatuhtempo(_tgl DATE, _vendorid VARCHAR);*/

CREATE OR REPLACE FUNCTION fhutangjatuhtempo (
	_tgl		DATE,
	_vendorid	VARCHAR,
	_branchid	VARCHAR
) RETURNS SETOF hutangjt AS
$BODY$
DECLARE
	_retval		hutangjt%ROWTYPE;

	_bpname 	VARCHAR;
	_vsaleno	VARCHAR(15);
	_purcno 	VARCHAR(15);
	_purcdate 	DATE;
	_duedate 	DATE;
	_jt 		INTEGER;
	_term		INTEGER;
	_total 		NUMERIC(19,4);
	_paidamt 	NUMERIC(19,4);
	_sisa 		NUMERIC(19,4);
	_payno 		VARCHAR(15);
	_paydate 	DATE;
	_totpayamt 	NUMERIC(19,4);
	__branchid	VARCHAR(10);
	__branchname	VARCHAR(30);

	cPurc		REFCURSOR;
	cPPaid		REFCURSOR;
	cPPaidTemp	REFCURSOR;
	
	_refno		VARCHAR;
	_ppaidamttemp	NUMERIC(19,4);
	_crcid		VARCHAR;

BEGIN
	OPEN cPurc FOR
		select bp.bpname,p.purcno,p.purcdate, p.duedays,
		p.branchid,br.branchname,p.paidamt,p.total,p.crcid
		from purc p 
		left join bp on p.vendorid=bp.bpid
		left join branch br on p.branchid=br.branchid
		WHERE (p.total-p.paidamt)>0 AND p.isdraft=false 
		AND p.termtype='R'
		AND (_branchid IS null OR p.branchid=_branchid)
		AND (_vendorid IS null OR p.vendorid=_vendorid)
		AND (p.purcdate+p.duedays)<=_tgl
		ORDER  BY br.branchname;
	LOOP
		FETCH cPurc INTO _bpname, _purcno, _purcdate, _term, __branchid,
		__branchname,_paidamt, _total, _crcid;
		EXIT WHEN NOT FOUND;

		_duedate:=_purcdate+_term;
		_jt:=_duedate-CURRENT_DATE;
		_sisa:=_total-_paidamt;
		_retval.bpname:=_bpname;

		_retval.branchid:=__branchid;
		_retval.branchname:=__branchname;

		_retval.purcno:=_purcno;
		_retval.purcdate:=_purcdate;
		_retval.duedate:=_duedate;
		_retval.jt:=_jt;
		_retval.total:=_total;
		_retval.paidamt:=_paidamt;
		_retval.sisa:=_sisa;
		_retval.payno:=null;
		_retval.paydate:=null;
		_retval.totpayamt:=null;
		_retval.crcid:=_crcid;

		RETURN NEXT _retval;

		OPEN cPPaid FOR
			SELECT payno 
			FROM ppaid
			WHERE reftype='PURC' AND refno=_purcno;
		LOOP
			FETCH cPPaid INTO _payno;
			EXIT WHEN NOT FOUND;

			SELECT INTO _paydate, _totpayamt paydate, totpayamt
			FROM pay
			WHERE payno=_payno;

			_retval.bpname:=null;
			_retval.branchid:=null;
			_retval.branchname:=null;

			_retval.purcno:=null;
			_retval.purcdate:=null;
			_retval.duedate:=null;
			_retval.jt:=null;
			_retval.total:=null;
			_retval.paidamt:=null;
			_retval.sisa:=null;
			_retval.payno:=_payno;
			_retval.paydate:=_paydate;
			_retval.totpayamt:=_totpayamt;
			_retval.crcid:=_crcid;

			RETURN NEXT _retval;

			OPEN cPpaidTemp FOR
				SELECT refno,ppaidamt
				FROM ppaid 
				WHERE payno=_payno AND reftype<>'PURC';
			LOOP
				FETCH cPpaidTemp INTO _refno,_ppaidamttemp;
				EXIT WHEN NOT FOUND;

				IF (_ppaidamttemp<0) THEN
					_ppaidamttemp:=_ppaidamttemp*(-1);
				END IF;

				_retval.purcno:=null;
				_retval.branchid:=null;
				_retval.branchname:=null;
				_retval.purcdate:=null;
				_retval.duedate:=null;
				_retval.total:=null;
				_retval.paidamt:=null;
				_retval.sisa:=null;
				_retval.payno:=_refno;
				_retval.paydate:=_paydate;
				_retval.totpayamt:=_ppaidamttemp;
				_retval.crcid:=_crcid;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cPpaidTemp;
		END LOOP;
		CLOSE cPPaid;
	END LOOP;
	CLOSE cPurc;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: fpiutangjatuhtempo(_tgl date, _custid VARCHAR)*/

/*-- DROP FUNCTION fpiutangjatuhtempo(_tgl date, _custid VARCHAR);*/


CREATE OR REPLACE FUNCTION fpiutangjatuhtempo (
	_tgl		DATE,
	_custid		VARCHAR,
	_branchid	VARCHAR
) RETURNS SETOF piutangjt AS
$BODY$
DECLARE
	_retval		piutangjt%ROWTYPE;

	_bpname 	VARCHAR(250);
	_saleno 	VARCHAR(15);
	_saledate 	DATE;
	_duedate 	DATE;
	_jt 		INTEGER;
	_term		INTEGER;
	_total 		NUMERIC(19,4);
	_paidamt 	NUMERIC(19,4);
	_sisa 		NUMERIC(19,4);
	_rcvno 		VARCHAR(15);
	_rcvdate 	DATE;
	_totrcvamt 	NUMERIC(19,4);
	__branchid	VARCHAR(10);
	cSale		REFCURSOR;
	cSPaid		REFCURSOR;
	cSPaidTemp	REFCURSOR;
	
	_refno		VARCHAR;
	_spaidamttemp	NUMERIC(19,4);
	_crcid		VARCHAR;
BEGIN
	OPEN cSale FOR
		SELECT bp.bpname, s.saleno, s.branchid, s.saledate, s.duedays, s.paidamt, s.total,s.crcid
		FROM sale s 
		JOIN bp ON s.custid=bp.bpid 
		WHERE (s.total-s.paidamt)>0 AND s.termtype='R' AND s.isdraft=false  
		AND (_custid IS NULL OR s.custid=_custid)
		AND (_branchid IS NULL OR s.branchid=_branchid);
	LOOP
		FETCH cSale INTO _bpname, _saleno, __branchid,_saledate, _term, _paidamt, _total, _crcid;
		EXIT WHEN NOT FOUND;

		_duedate:=_saledate+_term;
		_jt:=_duedate-CURRENT_DATE;
		_sisa:=_total-_paidamt;

		_retval.bpname:=_bpname;
		_retval.saleno:=_saleno;
		_retval.branchid:=__branchid;
		_retval.saledate:=_saledate;
		_retval.duedate:=_duedate;
		_retval.jt:=_jt;
		_retval.total:=_total;
		_retval.paidamt:=_paidamt;
		_retval.sisa:=_sisa;
		_retval.rcvno:=null;
		_retval.rcvdate:=null;
		_retval.totrcvamt:=null;
		_retval.crcid=_crcid;

		RETURN NEXT _retval;

		OPEN cSPaid FOR
			SELECT rcvno
			FROM spaid
			WHERE refno=_saleno AND reftype='SALE';
		LOOP
			FETCH cSPaid INTO _rcvno;
			EXIT WHEN NOT FOUND;

			SELECT INTO _rcvdate, _totrcvamt rcvdate, totrcvamt
			FROM rcv
			WHERE rcvno=_rcvno;

			_retval.bpname:=null;
			_retval.saleno:=null;
			_retval.saledate:=null;
			_retval.duedate:=null;
			_retval.jt:=null;
			_retval.total:=null;
			_retval.paidamt:=null;
			_retval.sisa:=null;
			_retval.rcvno:=_rcvno;
			_retval.rcvdate:=_rcvdate;
			_retval.totrcvamt:=_totrcvamt;
			_retval.crcid:=_crcid;

			RETURN NEXT _retval;

			OPEN cSpaidTemp FOR
				SELECT refno,spaidamt
				FROM spaid 
				WHERE rcvno=_rcvno AND reftype<>'SALE';
			LOOP
				FETCH cSpaidTemp INTO _refno,_spaidamttemp;
				EXIT WHEN NOT FOUND;

				IF (_spaidamttemp<0) THEN
					_spaidamttemp:=_spaidamttemp*(-1);
				END IF;

				_retval.saleno:=null;
				_retval.saledate:=null;
				_retval.duedate:=null;
				_retval.total:=null;
				_retval.paidamt:=null;
				_retval.sisa:=null;
				_retval.rcvno:=_refno;
				_retval.rcvdate:=_rcvdate;
				_retval.totrcvamt:=_spaidamttemp;
				_retval.crcid:=_crcid;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cSpaidTemp;
		END LOOP;
		CLOSE cSPaid;
	END LOOP;
	CLOSE cSale;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: fstatementhutang(_vendorid VARCHAR)*/

/*-- DROP FUNCTION fstatementhutang(_vendorid VARCHAR);*/

CREATE OR REPLACE FUNCTION fstatementhutang (
	_vendorid	VARCHAR,
	_branchid	VARCHAR
) RETURNS SETOF sthutang AS
$BODY$
DECLARE
	_retval		sthutang%ROWTYPE;
	_vsaleno	VARCHAR(15);
	_purcno		VARCHAR(15);
	_purcDATE	DATE;
	_dueDATE	DATE;
	_term		INTEGER;
	_total		NUMERIC(19,4);
	_paidamt	NUMERIC(19,4);
	_sisa		NUMERIC(19,4);
	_payno		VARCHAR(15);
	_payDATE	DATE;
	_ppaidamt	NUMERIC(19,4);
	__branchid	VARCHAR(10);
	__branchname	VARCHAR(30);
	cPurc		REFCURSOR;
	cPpaid		REFCURSOR;
	cPpaidTemp	REFCURSOR;

	cPRet		REFCURSOR;
	_pretno		VARCHAR;
	_pretdate	DATE;
	_crcidRet	VARCHAR;

	cFCN		REFCURSOR;
	_fcnno		VARCHAR;
	_fcndate	DATE;
	_fcnamt		NUMERIC(19,4);

	_refno		VARCHAR;
	_ppaidamttemp	NUMERIC(19,4);
	_crcid		VARCHAR;
BEGIN
	OPEN cPurc FOR
		SELECT p.purcno, p.branchid, br.branchname, p.purcDATE, p.duedays, p.total, p.paidamt,p.crcid
		FROM purc p 
		LEFT JOIN branch br ON p.branchid=br.branchid
		WHERE p.paystatus<>'F' AND p.isdraft=false 
		AND p.termtype='R' 
		AND (_vendorid IS NULL OR p.vendorid=_vendorid)
		AND (_branchid IS NULL OR p.branchid=_branchid)
		ORDER BY br.branchname;
	LOOP
		FETCH cPurc INTO _purcno, __branchid,__branchname,_purcDATE, _term, _total, _paidamt,_crcid;
		EXIT WHEN NOT FOUND;

		_dueDATE:=_purcDATE+_term;
		_sisa:=_total-_paidamt;
		_retval.purcno:=_purcno;
		_retval.branchid:=__branchid;
		_retval.branchname:=__branchname;
		_retval.purcDATE:=_purcDATE;
		_retval.dueDATE:=_dueDATE;
		_retval.total:=_total;
		_retval.paidamt:=_paidamt;
		_retval.sisa:=_sisa;
		_retval.payno:=null;
		_retval.payDATE:=null;
		_retval.ppaidamt:=null;
		_retval.crcid:=_crcid;

		RETURN NEXT _retval;

/*		OPEN cPpaid FOR
			SELECT payno
			FROM ppaid
			WHERE refno=_purcno;
		LOOP
			FETCH cPpaid INTO _payno;
			EXIT WHEN NOT FOUND;
			
			SELECT INTO _payDATE, _ppaidamt payDATE,totppaid
			FROM pay
			WHERE payno=_payno;

			_retval.purcno:=null;
			_retval.branchid:=null;
			_retval.branchname:=null;
			_retval.purcDATE:=null;
			_retval.dueDATE:=null;
			_retval.total:=null;
			_retval.paidamt:=null;
			_retval.sisa:=null;
			_retval.payno:=_payno;
			_retval.payDATE:=_payDATE;
			_retval.ppaidamt:=_ppaidamt;
			_retval.crcid:=_crcid;
			RETURN NEXT _retval;
*/			

		SELECT INTO _payno,_paydate ppaid.payno,paydate 
		FROM ppaid 
		JOIN pay ON ppaid.payno=pay.payno
		WHERE ppaid.refno=_purcno;

		OPEN cPpaidTemp FOR
			SELECT refno,ppaidamt
			FROM ppaid 
			WHERE refno=_purcno;
		LOOP
			FETCH cPpaidTemp INTO _refno,_ppaidamttemp;
			EXIT WHEN NOT FOUND;

			IF (_ppaidamttemp<0) THEN
				_ppaidamttemp:=_ppaidamttemp*(-1);
			END IF;

			_retval.purcno:=null;
			_retval.branchid:=null;
			_retval.branchname:=null;
			_retval.purcDATE:=null;
			_retval.dueDATE:=null;
			_retval.total:=null;
			_retval.paidamt:=null;
			_retval.sisa:=null;
			_retval.payno:=_payno;
			_retval.payDATE:=_payDATE;
			_retval.ppaidamt:=_ppaidamttemp;
			_retval.crcid:=_crcid;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cPpaidTemp;
/*		END LOOP;
		CLOSE cPpaid;*/
	END LOOP;
	CLOSE cPurc;

	OPEN cPRet FOR
		SELECT pretno,pretdate,total,crcid FROM pret p WHERE isdraft=false AND iscontra=false
		AND (_vendorid IS NULL OR p.vendorid=_vendorid)
		AND (_branchid IS NULL OR p.branchid=_branchid);
	LOOP
		FETCH cPRet INTO _pretno,_pretdate,_total,_crcidRet;
		EXIT WHEN NOT FOUND;

		_retval.purcno:=_pretno;
		_retval.branchid:=__branchid;
		_retval.purcdate:=_pretdate;
		_retval.duedate:=null;
		_retval.total:=_total*(-1);
		_retval.paidamt:=null;
		_retval.sisa:=_total*(-1);
		_retval.payno:=null;
		_retval.paydate:=null;
		_retval.ppaidamt:=null;
		_retval.crcid:=_crcidRet;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cPRet;

	OPEN cFCN FOR
		SELECT fcnno,fcndate,fcnamt FROM fcn s WHERE iscontra=false and dc='C'
		AND (_vendorid IS NULL OR s.bpid=_vendorid)
		AND (_branchid IS NULL OR s.branchid=_branchid);
	LOOP
		FETCH cFCN INTO _fcnno,_fcndate,_fcnamt;
		EXIT WHEN NOT FOUND;

		_retval.purcno:=_fcnno;
		_retval.branchid:=__branchid;
		_retval.purcdate:=_fcndate;
		_retval.duedate:=null;
		_retval.total:=_fcnamt*(-1);
		_retval.paidamt:=null;
		_retval.sisa:=_fcnamt*(-1);
		_retval.payno:=null;
		_retval.paydate:=null;
		_retval.ppaidamt:=null;
		_retval.crcid:=_CrcID;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cFCN;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION fstatementhutang(_vendorid VARCHAR,_branchid VARCHAR) OWNER TO bits;


/*-- Function: fstatementpiutang(_custid VARCHAR)*/

/*-- DROP FUNCTION fstatementpiutang(_custid VARCHAR);*/


CREATE OR REPLACE FUNCTION fstatementpiutang (
	_custid		VARCHAR,
	_branchid	VARCHAR
) RETURNS SETOF stpiutang AS
$BODY$
DECLARE
	_retval		stpiutang%ROWTYPE;
	_saleno		VARCHAR(15);
	_saledate	DATE;
	_duedate	DATE;
	_term		INTEGER;
	_total		NUMERIC(19,4);
	_paidamt	NUMERIC(19,4);
	_sisa		NUMERIC(19,4);
	_rcvno		VARCHAR(15);
	_rcvdate	DATE;
	_spaidamt	NUMERIC(19,4);
	_saletype	VARCHAR;
	__branchid	VARCHAR(10);
	cSale		REFCURSOR;
	cSpaid		REFCURSOR;
	cSpaidTemp	REFCURSOR;
	_crcID		VARCHAR;
	_crcIDRet	VARCHAR;
	_refno		VARCHAR;
	_spaidamttemp	NUMERIC(19,4);
	cFCN		REFCURSOR;
	_fcnno		VARCHAR;
	_fcndate	DATE;
	_fcnamt		NUMERIC(19,4);
	_RefType	VARCHAR;
	cSRet		REFCURSOR;
	_sretno		VARCHAR;
	_sretdate	DATE;
	_srettot	NUMERIC(19,4);
BEGIN
	OPEN cSale FOR
		SELECT s.saleno,s.branchid, s.saledate, s.duedays, s.total, s.paidamt, s.saletype, s.crcid
		FROM sale s
		WHERE s.paystatus<>'F' AND s.termtype='R' AND s.isdraft=false 
		AND (_custid IS NULL OR s.custid=_custid)
		AND (_branchid IS NULL OR s.branchid=_branchid);
	LOOP
		FETCH cSale INTO _saleno,__branchid, _saledate, _term, _total, _paidamt, _saletype, _crcid;
		EXIT WHEN NOT FOUND;

		_duedate:=_saledate+_term;
		_sisa:=_total-_paidamt;

		_retval.saleno:=_saleno;
		_retval.branchid:=__branchid;
		_retval.saledate:=_saledate;
		_retval.duedate:=_duedate;
		_retval.total:=_total;
		_retval.paidamt:=_paidamt;
		_retval.sisa:=_sisa;
		_retval.rcvno:=null;
		_retval.rcvdate:=null;
		_retval.spaidamt:=null;
		_retval.saletype:=_saletype;
		_retval.crcid:=_crcid;
		RETURN NEXT _retval;

		SELECT INTO _RcvNo,_RcvDate spaid.rcvno,rcvdate FROM spaid JOIN rcv on spaid.rcvno=rcv.rcvno
		WHERE spaid.refno=_saleno;
		/*OPEN cSpaid FOR
			SELECT rcvno
			FROM spaid
			WHERE refno=_saleno;
		LOOP
			FETCH cSpaid INTO _rcvno;
			EXIT WHEN NOT FOUND;

			SELECT INTO _rcvdate, _spaidamt rcvdate,totspaid
			FROM rcv
			WHERE rcvno=_rcvno;

			_retval.saleno:=null;
			_retval.saledate:=null;
			_retval.duedate:=null;
			_retval.total:=null;
			_retval.paidamt:=null;
			_retval.sisa:=null;
			_retval.rcvno:=_rcvno;
			_retval.rcvdate:=_rcvdate;
			_retval.spaidamt:=_spaidamt;
			_retval.saletype:=null;
			RETURN NEXT _retval;

			OPEN cSpaidTemp FOR
				SELECT refno,spaidamt
				FROM spaid where rcvno=_rcvno AND reftype<>'SALE';
			LOOP
				FETCH cSpaidTemp INTO _refno,_spaidamttemp;
				EXIT WHEN NOT FOUND;

				IF (_spaidamttemp<0) THEN
					_spaidamttemp:=_spaidamttemp*(-1);
				END IF;

				_retval.saleno:=null;
				_retval.saledate:=null;
				_retval.duedate:=null;
				_retval.total:=null;
				_retval.paidamt:=null;
				_retval.sisa:=null;
				_retval.rcvno:=_refno;
				_retval.rcvdate:=_rcvdate;
				_retval.spaidamt:=_spaidamttemp;
				_retval.saletype:=null;
				RETURN NEXT _retval;
			END LOOP;
			CLOSE cSpaidTemp;
		END LOOP;
		CLOSE cSpaid;*/	

		OPEN cSpaidTemp FOR
			SELECT refno,spaidamt
			FROM spaid 
			WHERE refno=_SaleNo;
		LOOP
			FETCH cSpaidTemp INTO _refno,_spaidamttemp;
			EXIT WHEN NOT FOUND;

			_retval.saleno:=null;
			_retval.saledate:=null;
			_retval.duedate:=null;
			_retval.total:=null;
			_retval.paidamt:=null;
			_retval.sisa:=null;

			_retval.rcvno:=_rcvno;

			_retval.rcvdate:=_rcvdate;
			_retval.spaidamt:=_spaidamttemp;
			_retval.crcid:=_crcid;
			RETURN NEXT _retval;
		END LOOP;
		CLOSE cSpaidTemp;
	END LOOP;
	CLOSE cSale;

	OPEN cSRet FOR
		SELECT sretno,sretdate,total,crcid FROM sret s WHERE isdraft=false AND iscontra=false
		AND (_custid IS NULL OR s.custid=_custid)
		AND (_branchid IS NULL OR s.branchid=_branchid);
	LOOP
		FETCH cSRet INTO _sretno,_sretdate,_total,_crcidRet;
		EXIT WHEN NOT FOUND;

		_retval.saleno:=_sretno;
		_retval.branchid:=__branchid;
		_retval.saledate:=_sretdate;
		_retval.duedate:=null;
		_retval.total:=_total*(-1);
		_retval.paidamt:=null;
		_retval.sisa:=_total*(-1);
		_retval.rcvno:=null;
		_retval.rcvdate:=null;
		_retval.spaidamt:=null;
		_retval.crcid:=_crcidRet;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cSRet;

	OPEN cFCN FOR
		SELECT fcnno,fcndate,fcnamt FROM fcn s WHERE iscontra=false and dc='D'
		AND (_custid IS NULL OR s.bpid=_custid)
		AND (_branchid IS NULL OR s.branchid=_branchid);
	LOOP
		FETCH cFCN INTO _fcnno,_fcndate,_fcnamt;
		EXIT WHEN NOT FOUND;

		_retval.saleno:=_fcnno;
		_retval.branchid:=__branchid;
		_retval.saledate:=_fcndate;
		_retval.duedate:=null;
		_retval.total:=_fcnamt*(-1);
		_retval.paidamt:=null;
		_retval.sisa:=_fcnamt*(-1);
		_retval.rcvno:=null;
		_retval.rcvdate:=null;
		_retval.spaidamt:=null;
		_retval.crcid:=_CrcID;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cFCN;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBukuBesar (
	_accNo		VARCHAR,
	_startdate	DATE,
	_enddate	DATE,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	PrjID		VARCHAR
) RETURNS SETOF accrpt AS
$BODY$
DECLARE
	_retval		accrpt%ROWTYPE;

	_JnlNo		VARCHAR(15);
	_LgrDate	DATE;
	_accName	VARCHAR(30);
	_LgrNote	VARCHAR(255);
	_DK		VARCHAR(1);
	_accBal		NUMERIC(19,4);
	_Amount		NUMERIC(19,4);
	_Debet		NUMERIC(19,4);
	_Kredit		NUMERIC(19,4);
	_Month		SMALLINT;
	_PYear		INTEGER;
	_LastMonthBal	NUMERIC(19,4);
	_YesterdayBal	NUMERIC(19,4);
	_DefaultDK	VARCHAR(1);

	cLgr		REFCURSOR;
BEGIN
	_Month:=date_part('month', _StartDate);
	_PYear:=date_part('year', _StartDate);

	_accBal:=fSaldoAkhir(_AccNo,_StartDate, _BranchID, _DeptID, _PrjID);

	_retval.jnlno:=NULL;
	_retval.lgrdate:=_StartDate;
	_retval.accname:='saldo';
	_retval.lgrnote:='saldo';
	_retval.debet:=NULL;
	_retval.kredit:=NULL;
	_retval.accbal:=_accBal;
	RETURN NEXT _retval;
	
	SELECT INTO _DefaultDK accdk FROM Acc Where AccNo=_AccNo;

	OPEN cLgr FOR
		SELECT jnld.JnlNo, JnlDate, DK, jnlNote, jnldamt
		FROM jnld join jnl on jnl.jnlno=jnld.jnlno
		WHERE accNo=_accNo AND JnlDate>=_StartDate
		AND JnlDate<=fDate2359(_EndDate)
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID)
		ORDER BY JnlDate,JnlNo,Jnldno;
	LOOP
		FETCH cLgr INTO _JnlNo, _LgrDate, _DK, _LgrNote, _Amount;
		EXIT WHEN NOT FOUND;

		SELECT INTO _accName accName FROM Acc WHERE accNo=_accNo;
		IF (_DK='D') THEN
			_Debet:=_Amount;
			_Kredit:=NULL;
		ELSE
			_Debet:=NULL;
			_Kredit:=_Amount;
		END IF;

		IF (_DK=_DefaultDK) THEN
			_accBal=_accBal+_Amount;
		ELSE
			_accBal=_accBal-_Amount;
		END IF;

		_retval.jnlno:=_JnlNo;
		_retval.lgrdate:=_LgrDate;
		_retval.accname:=_accName;
		_retval.lgrnote:=_LgrNote;
		_retval.debet:=_Debet;
		_retval.kredit:=_Kredit;
		_retval.accbal:=_accBal;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cLgr;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAnalisaStock (
	_itemid		VARCHAR,
	_ItemDesc 	VARCHAR,
	_StartDate 	DATE,
	_EndDate 	DATE,
	_GrpID 		VARCHAR,
	_BrandID 	VARCHAR,
	_ModelID 	VARCHAR,
	_VendorID 	VARCHAR,
	_IsLow 		BOOLEAN,
	_InActive 	BOOLEAN, 
	_onSO 		BOOLEAN
) RETURNS SETOF tAnalisaStock AS
$BODY$
DECLARE
	_retval		tAnalisaStock%ROWTYPE;
	
	lama		INT4;
	cStock		REFCURSOR;
BEGIN
	lama=(_EndDate - _StartDate)/30;
	IF lama=0 THEN
		lama:=1;
	END IF;

	IF (_IsLow=TRUE) THEN 	
		IF (_onSO=TRUE) THEN
			OPEN cStock FOR
				SELECT v.ItemID, v.Qty, v.QtyPO, v.QtySO, i.QtyMin, i.QtyMax, i.QtyReOrder,i.leadtime,
				COALESCE(fSalesRekap(v.ItemID,_StartDate,_EndDate)/lama,0) AS QtySoldBln,
				COALESCE(fSalesRekap(v.ItemID,_StartDate,_EndDate),0) AS QtySold, fPOList(v.ItemID) AS PONo, fSOList(v.ItemID) AS SONo
				FROM vwStock_Sum v JOIN Item i ON v.ItemID=i.ItemID
				WHERE (_GrpID IS NULL OR fitgrpin(i.itemid,_GrpID)) AND
					(_BrandID IS NULL OR i.brandid=_BrandID) AND
					(_ModelID IS NULL OR i.modelid=_ModelID) AND
					(_VendorID IS NULL OR i.vendorid=_VendorID) AND 
					(_ItemID IS NULL OR UPPER(i.itemid) LIKE UPPER('%'||_ItemID||'%')) AND 
					(_ItemDesc IS NULL OR UPPER(i.itemdesc) LIKE UPPER('%'||_ItemDesc||'%'))
				AND v.Qty<=i.QtyMin AND i.active=_InActive AND v.QtySO>0;
		ELSE
			OPEN cStock FOR
				SELECT v.ItemID, v.Qty, v.QtyPO, v.QtySO, i.QtyMin, i.QtyMax, i.QtyReOrder,i.leadtime,
				COALESCE(fSalesRekap(v.ItemID,_StartDate,_EndDate)/lama,0) AS QtySoldBln,
				COALESCE(fSalesRekap(v.ItemID,_StartDate,_EndDate),0) AS QtySold, fPOList(v.ItemID) AS PONo, fSOList(v.ItemID) AS SONo
				FROM vwStock_Sum v JOIN Item i ON v.ItemID=i.ItemID
				WHERE (_GrpID IS NULL OR fitgrpin(i.itemid,_GrpID)) AND
					(_BrandID IS NULL OR i.brandid=_BrandID) AND
					(_ModelID IS NULL OR i.modelid=_ModelID) AND
					(_VendorID IS NULL OR i.vendorid=_VendorID) AND 
					(_ItemID IS NULL OR UPPER(i.itemid) LIKE UPPER('%'||_ItemID||'%')) AND 
					(_ItemDesc IS NULL OR UPPER(i.itemdesc) LIKE UPPER('%'||_ItemDesc||'%'))
				AND v.Qty<=i.QtyMin AND i.active=_InActive;
		END IF;
			LOOP
				FETCH cStock INTO _retval;
				EXIT WHEN NOT FOUND;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cStock;
	ELSE
		IF (_onSO=TRUE) THEN
			OPEN cStock FOR
				SELECT v.ItemID, v.Qty, v.QtyPO, v.QtySO, i.QtyMin, i.QtyMax, i.QtyReOrder,i.leadtime,
				COALESCE(fSalesRekap(v.ItemID,_StartDate,_EndDate)/lama,0) AS QtySoldBln,
				COALESCE(fSalesRekap(v.ItemID,_StartDate,_EndDate),0) AS QtySold, fPOList(v.ItemID) AS PONo, fSOList(v.ItemID) AS SONo
				FROM vwStock_Sum v JOIN Item i ON v.ItemID=i.ItemID
				WHERE (_GrpID IS NULL OR fitgrpin(i.itemid,_GrpID)) AND
					(_BrandID IS NULL OR i.brandid=_BrandID) AND
					(_ModelID IS NULL OR i.modelid=_ModelID) AND
					(_VendorID IS NULL OR i.vendorid=_VendorID) AND 
					(_ItemID IS NULL OR UPPER(i.itemid) LIKE UPPER('%'||_ItemID||'%')) AND 
					(_ItemDesc IS NULL OR UPPER(i.itemdesc) LIKE UPPER('%'||_ItemDesc||'%'))
				AND i.active=_InActive AND v.QtySO>0;
		ELSE
			OPEN cStock FOR
				SELECT v.ItemID, v.Qty, v.QtyPO, v.QtySO, i.QtyMin, i.QtyMax, i.QtyReOrder,i.leadtime,
				COALESCE(fSalesRekap(v.ItemID,_StartDate,_EndDate)/lama,0) AS QtySoldBln,
				COALESCE(fSalesRekap(v.ItemID,_StartDate,_EndDate),0) AS QtySold, fPOList(v.ItemID) AS PONo, fSOList(v.ItemID) AS SONo
				FROM vwStock_Sum v JOIN Item i ON v.ItemID=i.ItemID
				WHERE (_GrpID IS NULL OR fitgrpin(i.itemid,_GrpID)) AND
					(_BrandID IS NULL OR i.brandid=_BrandID) AND
					(_ModelID IS NULL OR i.modelid=_ModelID) AND
					(_VendorID IS NULL OR i.vendorid=_VendorID) AND 
					(_ItemID IS NULL OR UPPER(i.itemid) LIKE UPPER('%'||_ItemID||'%')) AND 
					(_ItemDesc IS NULL OR UPPER(i.itemdesc) LIKE UPPER('%'||_ItemDesc||'%'))
				AND i.active=_InActive;
		END IF;
			LOOP
				FETCH cStock INTO _retval;
				EXIT WHEN NOT FOUND;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cStock;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSalesRekap(
	_ItemID 	VARCHAR,
	_StartDate 	DATE,
	_EndDate 	DATE
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_retval		NUMERIC;
BEGIN
	SELECT INTO _retval SUM(fQtyZToQty1(itemid, qty, unit))
	FROM sale JOIN saled ON sale.saleno=saled.saleno  
	WHERE sale.saledate>=_StartDate AND sale.saledate<=_EndDate AND itemID=_ItemID;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDebt_Void_byRef (
	_RefType	VARCHAR(5),
	_RefNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	spretSpContBal_Update	SPStatus%ROWTYPE;
	_DebtNo			VARCHAR(15);
	_CustID			VARCHAR(15);
	_DebtDate		DATE;
	_Amount			DECIMAL(19,4);
	_tmpAmount		DECIMAL(19,4);
BEGIN
	SELECT INTO _DebtNo,_CustID,_DebtDate,_Amount DebtNo,CustID,DebtDate,Amount FROM Debt 
	WHERE RefType=_RefType AND RefNo=_RefNo;

	_tmpAmount:=_Amount*(-1);

	SELECT INTO spretSpContBal_Update * FROM spContBal_Update(_CustID,_DebtDate,_tmpAmount);

	IF spretSpContBal_Update.ErrNo<>0 THEN
			_retval.ErrNo:=spretSpContBal_Update.ErrNo;
			_retval.ErrMsg:=spretSpContBal_Update.ErrMsg;
	RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	DELETE FROM Debt WHERE DebtNo=_DebtNo;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fcredcard (
	_bpid		VARCHAR,
	_startdate	DATE,
	_enddate	DATE
) RETURNS SETOF bparpt AS
$BODY$
DECLARE
	_retval		bparpt%ROWTYPE;

	_BPANo		VARCHAR(15);
	_BPADate	DATE;
	_BPAType	VARCHAR(1);
	_RefType	VARCHAR(5);
	_RefNo		VARCHAR(15);
	_BPANote	VARCHAR(50);
	_DC		VARCHAR(1);
	_Debit		DECIMAL(19,4);
	_Credit		DECIMAL(19,4);
	_APBalance	DECIMAL(19,4);

	_BPAAmt		DECIMAL(19,4);

	_Month		SMALLINT;
	_PYear		INTEGER;
	_LastMonthBal	DECIMAL(19,4);
	_YesterdayBal	DECIMAL(19,4);

	cBPA		REFCURSOR;
BEGIN
	_Month:=DATE_part('month', _StartDate);
	_PYear:=DATE_part('year', _StartDate);

	IF (_Month=1) THEN
		SELECT INTO _LastMonthBal BPBal00 
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=2) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01 
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=3) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02 
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=4) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03 
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=5) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03+BPBal04 
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=6) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03+BPBal04+BPBal05 
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=7) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03+BPBal04+BPBal05+BPBal06
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=8) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03+BPBal04+BPBal05+BPBal06
		+BPBal07
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=9) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03+BPBal04+BPBal05+BPBal06
		+BPBal07+BPBal08
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=10) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03+BPBal04+BPBal05+BPBal06
		+BPBal07+BPBal08+BPBal09
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=11) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03+BPBal04+BPBal05+BPBal06
		+BPBal07+BPBal08+BPBal09+BPBal10
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	ELSIF (_Month=12) THEN
		SELECT INTO _LastMonthBal BPBal00+BPBal01+BPBal02+BPBal03+BPBal04+BPBal05+BPBal06
		+BPBal07+BPBal08+BPBal09+BPBal10+BPBal11
		FROM BPBal WHERE BPID=_BPID AND PYear=_PYear;
	END IF;

	IF (_LastMonthBal IS NULL) THEN
		_LastMonthBal:=0;
	END IF;

	SELECT INTO _YesterdayBal (_LastMonthBal+COALESCE(SUM(BPAAmt), 0)) 
	FROM BPA
	WHERE BPID=_BPID AND BPADate>=fBegOfMonth(_StartDate)
	AND BPADate<fNoTime(_StartDate);

	_APBalance:=_YesterdayBal;

	_retval.bpano:=NULL;
	_retval.branchid:=NULL;
	_retval.bpadate:=fNoTime(_StartDate);
	_retval.bpatype:=NULL;
	_retval.reftype:='Saldo';
	_retval.refno:=NULL;
	_retval.bpanote:='Saldo';
	_retval.dc:='B';
	_retval.debit:=NULL;
	_retval.credit:=NULL;
	_retval.bpbal:=_APBalance;

	RETURN NEXT _retval;

	OPEN cBPA FOR
		SELECT BPANo, BPADate, BPAType, RefType, RefNo, BPANote, BPAAmt
		FROM BPA
		WHERE BPID=_BPID AND BPADate>=fNoTime(_StartDate)
		AND BPADate<=fDate2359(_EndDate)
		AND RefType='PURC' OR RefType='PAY'
		ORDER BY BPADate, BPANo;
	LOOP
		FETCH cBPA INTO _BPANo, _BPADate, _BPAType, _RefType, _RefNo, _BPANote, _BPAAmt;
		EXIT WHEN NOT FOUND;

		IF (_BPAAmt>=0) THEN
			_Debit:=_BPAAmt;
			_Credit:=NULL;
			_DC:='P';
		ELSE
			_Debit:=NULL;
			_Credit:=ABS(_BPAAmt);
			_DC:='C';
		END IF;

		_APBalance:=_APBalance+_BPAAmt;

		_retval.bpano:=_BPANo;
		_retval.branchid:=NULL;
		_retval.bpadate:=_BPADate;
		_retval.bpatype:=_BPAType;
		_retval.reftype:=_RefType;
		_retval.refno:=_RefNo;
		_retval.bpanote:=_BPANote;
		_retval.dc:=_DC;
		_retval.debit:=_Debit;
		_retval.credit:=_Credit;
		_retval.bpbal:=_APBalance;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cBPA;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spClonepriv(_grpidSrc varchar,_grpidDst varchar)
  RETURNS spStatus AS
$BODY$
DECLARE
	_retval	spStatus;
BEGIN
	_retval.ErrNo:=1;
	_retval.ErrMsg:='Error Clone Privilege';

	DELETE FROM grpprv WHERE grpid=_grpidDst;

	INSERT INTO grpprv(grpid,objid,acstype,acsenable,acsval) SELECT _grpidDst,objid,acstype,acsenable,acsval FROM grpprv WHERE grpid=_grpidSrc;
	
	_retval:=(0, 'Sukses Clone Privilege');
	RETURN _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


/*
	REVISI : 070723   ada masalah dengan fNoTime jadi di-disable semua

*/

CREATE OR REPLACE FUNCTION fcashcard (
	_cashid 	VARCHAR, 
	_startdate 	DATE, 
	_enddate 	DATE
) RETURNS SETOF cashrpt AS
$BODY$
DECLARE
	_retval		cashrpt%ROWTYPE;

	_CashANo	VARCHAR(15);
	_CashADate	DATE;
	_CashAType	VARCHAR(5);
	_BPID		VARCHAR(15);
	_BPName		VARCHAR(50);
	_RefType	VARCHAR(5);
	_RefNo		VARCHAR(15);
	_CashANote	TEXT;
	_InOut		VARCHAR(1);
	_CashIn		NUMERIC(19,4);
	_CashOut	NUMERIC(19,4);
	_CashBal	NUMERIC(19,4);
	_Amount		NUMERIC(19,4);	
	_Month		SMALLINT;
	_PYear		INTEGER;
	_LastMonthBal	NUMERIC(19,4);
	_YesterdayBal	NUMERIC(19,4);
	
	/**Tambah Crc*/
	_CrcID		VARCHAR(10);
	_CrcName	VARCHAR(50);
	_CrcSymbol	VARCHAR(10);

	cCashA		REFCURSOR;
BEGIN
	_Month:=DATE_part('month', _StartDate);
	_PYear:=DATE_part('year', _StartDate);

	SELECT INTO _CrcID,_CrcName,_CrcSymbol cs.crcid,crc.crcname,crc.crcsymbol
	FROM cash cs 
	JOIN crc ON cs.crcid=crc.crcid 
	WHERE cs.cashid=_cashid;

	IF (_Month=1) THEN
		SELECT INTO _LastMonthBal CashBal00 FROM CashBal
		WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=2) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01 
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=3) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02 
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=4) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03 
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=5) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04 
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=6) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05 
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=7) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=8) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
		+CashBal07
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=9) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
		+CashBal07+CashBal08
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=10) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
		+CashBal07+CashBal08+CashBal09
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=11) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
		+CashBal07+CashBal08+CashBal09+CashBal10
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	ELSIF (_Month=12) THEN
		SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
		+CashBal07+CashBal08+CashBal09+CashBal10+CashBal11
		FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
	END IF;

	IF (_LastMonthBal IS NULL) THEN
		_LastMonthBal:=0;
	END IF;

	SELECT INTO _YesterdayBal (_LastMonthBal+COALESCE(SUM(Amount), 0)) 
	FROM CashA
	WHERE CashID=_CashID AND CashADate>=fBegOfMonth(_StartDate)
	AND CashADate<_StartDate;
	/*--AND CashADate<fNoTime(_StartDate);*/

	_CashBal:=_YesterdayBal;

	_retval.cashano:=NULL;
	/*--_retval.cashadate:=fNoTime(_StartDate);*/
	_retval.cashadate:=_StartDate;
	_retval.cashatype:='Saldo';
	_retval.bpname:=NULL;
	_retval.reftype:=NULL;
	_retval.refno:=NULL;
	_retval.cashanote:='saldo';
	_retval.inout:='B';
	_retval.cashin:=NULL;
	_retval.cashout:=NULL;
	_retval.cashbal:=_CashBal;

	_retval.crcid=_CrcID;
	_retval.crcname=_CrcName;
	_retval.crcsymbol=_CrcSymbol;

	RETURN NEXT _retval;

	OPEN cCashA FOR
		SELECT CashANo, CashADate, CashAType, BPID, RefType, RefNo, 
			CashANote, InOut, Amount
		FROM CashA
		/*--WHERE CashID=_CashID AND CashADate>=fNoTime(_StartDate)*/
		WHERE CashID=_CashID AND CashADate>=_StartDate
		AND CashADate<=fDate2359(_EndDate) 
		AND isreconciled=TRUE 
		ORDER BY CashADate, CashANo, RefNo;
	LOOP
		FETCH cCashA INTO _CashANo, _CashADate, _CashAType, _BPID,
			_RefType, _RefNo, _CashANote, _InOut, _Amount;
		EXIT WHEN NOT FOUND;

		SELECT INTO _BPName BPName FROM BP WHERE BPID=_BPID;

		IF (_Amount>=0) THEN
			_CashIn:=_Amount;
			_CashOut:=NULL;
			_InOut:='I';
		ELSE
			_CashIn:=NULL;
			_CashOut:=ABS(_Amount);
			_InOut:='O';
		END IF;

		_CashBal=_CashBal+_Amount;

		_retval.cashano:=_CashANo;
		_retval.cashadate:=_CashADate;
		_retval.cashatype:=_CashAType;
		_retval.bpname:=_BPName;
		_retval.reftype:=_RefType;
		_retval.refno:=_RefNo;
		_retval.cashanote:=_CashANote;
		_retval.inout:=_InOut;
		_retval.cashin:=_CashIn;
		_retval.cashout:=_CashOut;
		_retval.cashbal:=_CashBal;

		_retval.crcid=_CrcID;
		_retval.crcname=_CrcName;
		_retval.crcsymbol=_CrcSymbol;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cCashA;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spclosecons(_consno varchar, __consdno int, _isall bool)
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
	_consdno	int;
	_qtySellIn	numeric(15,4);
	_qtyxSellIn	numeric(15,4);
	_qtySellOut	numeric(15,4);
	_qtyXSellOut	numeric(15,4);
	cConsD	refcursor;
BEGIN
	_consdno:=__consdno;
	_retval.ErrNo:=1;
	_retval.ErrMsg:='Err :-(';
/*--_retval.ErrNo:=_consdno;*/
	IF _isAll THEN
		OPEN cConsD FOR
		SELECT consdno,qtySellin,qtyXSellin,qtySellout,qtyxSellOut FROM consd WHERE consno=_consno;
	ELSE
		OPEN cConsD FOR
		SELECT consdno,qtySellin,qtyXSellin,qtySellout,qtyxSellOut FROM consd WHERE consno=_consno AND consdno=_consdno;
	END IF;

	LOOP
		FETCH cConsD INTO _consdno,_qtySellin,_qtyXSellin,_qtySellout,_qtyxSellOut;
		IF NOT FOUND THEN
			EXIT;
		END IF;
		UPDATE consd SET qtycret=_qtySellIn-_qtySellOut, qtyxcret=_QtyXSellIn-_qtyXSellOut WHERE consno=_consno AND consdno=_consdno;
		_retval:=(0, 'OK');
	END LOOP;
	
	RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fPrice (
	__index		VARCHAR,
	__discexp	VARCHAR,
	__itemid	VARCHAR,
	__prclvlid	VARCHAR
) RETURNS SETOF TPrice AS 
$BODY$
DECLARE
	_retval		TPrice%ROWTYPE;
	_harga		NUMERIC;
	_discount	VARCHAR;
	_unit1		VARCHAR;
	_unit2		VARCHAR;
	_unit3		VARCHAR;
	_conv2		NUMERIC;
	_conv3		NUMERIC;
	_price1		NUMERIC;
	_disc1exp	VARCHAR;
	_price2		NUMERIC;
	_disc2exp	VARCHAR;
	_price3		NUMERIC;
	_disc3exp	VARCHAR;
	_hargaLain	NUMERIC;
	_discAkhir	VARCHAR;
	_hargaAkhir	NUMERIC;
	_prclvlid	VARCHAR;

	cPrice		REFCURSOR;
BEGIN
	SELECT INTO _unit1,_unit2,_conv2,_unit3,_conv3 unit1,unit2,conv2,unit3,conv3 
	FROM item 
	WHERE itemid=__itemid;

	OPEN cPrice FOR 
		SELECT price1,disc1exp,price2,disc2exp,price3,disc3exp,prclvlid 
		FROM prc 
		WHERE itemid=__itemid 
		ORDER BY prclvlid desc;
	LOOP
		FETCH cPrice INTO _price1,_disc1exp,_price2,_disc2exp,_price3,_disc3exp,_prclvlid;
		EXIT WHEN NOT FOUND;

		IF ((_prclvlid=__prclvlid) AND (_price1<>0)) THEN
			IF (__index='price1') THEN
				_hargaAkhir:=_price1;
				_discAkhir:=_disc1exp;
			ELSIF (__index='price2') THEN
				IF (_price2=0) THEN
					_hargaAkhir:=_price1*_conv2;
					_discAkhir:=_disc1exp;
				ELSE
					_hargaAkhir:=_price2;
					_discAkhir:=_disc2exp;
				END IF;
			ELSIF(__index='price3') THEN
				IF ((_price3=0) AND (_price2<>0)) THEN
					_hargaAkhir:=_price2*(_conv3/_conv2);
					_discAkhir:=_disc2exp;
				ELSIF((_price3=0) AND (_price2=0)) THEN
					_hargaAkhir:=_price1*_conv3;
					_discAkhir:=_disc1exp;
				ELSE
					_hargaAkhir:=_price3;
					_discAkhir:=_disc3exp;
				END IF;
			END IF;
			EXIT;
		ELSIF((_price1<>0) AND (_prclvlid<__prclvlid)) THEN
			IF (__index='price1') THEN
				_hargaAkhir:=_price1;
				_discAkhir:=_disc1exp;
			ELSIF(__index='price2') THEN
				IF(_price2=0) THEN
					_hargaAkhir:=_price1*_conv2;
					_discAkhir:=_disc1exp;
				ELSE
					_hargaAkhir:=_price2;
					_discAkhir:=_disc2exp;
				END IF;
			ELSIF(__index='price3') THEN
				IF ((_price3=0) AND (_price2<>0)) THEN
					_hargaAkhir:=_price2*(_conv3/_conv2);
					_discAkhir:=_disc2exp;
				ELSIF((_price3=0) AND (_price2=0)) THEN
					_hargaAkhir:=_price1*_conv3;
					_discAkhir:=_disc1exp;
				ELSE
					_hargaAkhir:=_price3;
					_discAkhir:=_disc3exp;
				END IF;
			END IF;
			EXIT;
		END IF;
	END LOOP;
	CLOSE cPrice;

	_retval.harga:=_hargaAkhir;
	_retval.disc:=_discAkhir;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fQtyStockDesc (
	_ItemID		VARCHAR
) RETURNS VARCHAR AS
$BODY$
/*-- _ItemID = item id*/
/*-- _Qty1   = banyak item pada unit 1*/
DECLARE
	_Tmp		NUMERIC(15,4);
	_retval		VARCHAR(255);
	_WHID 		VARCHAR(3);
BEGIN
	_WHID:=fReg('WHSALE');
	SELECT INTO _Tmp SUM(s.qty) FROM stock s WHERE s.itemid=_ItemID AND s.whid=_WHID;
	_retval:=fQty1Desc(_ItemID, _Tmp);

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fitgrpIn (
	_itemid		VARCHAR,
	_itgrpid	VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;
	cGrp		REFCURSOR;
	_itgrpid2	VARCHAR(10);
	_itgrpCari	VARCHAR(10);
BEGIN
	_retval:=false;

	SELECT INTO _itgrpCari itgrpid FROM item WHERE itemid=_itemid;
	IF (_itgrpCari=_itgrpid) THEN
		_retval:=true;
	END IF;

	OPEN cGrp FOR
		SELECT itgrpid FROM itgrp WHERE upitgrpid=_itgrpid;
	LOOP
		FETCH cGrp INTO _itgrpid2;
		EXIT WHEN NOT FOUND;
		if (_retval=false) THEN
			SELECT INTO _retval * FROM fitgrpIn(_itemid,_itgrpid2);
		END IF;
	END LOOP;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItGrp2In (
	_itemid		VARCHAR,
	_itgrp2id	VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;
	cGrp		REFCURSOR;
	_itgrpid2	VARCHAR(10);
	_itgrpCari	VARCHAR(10);
BEGIN
	_retval:=false;

	SELECT INTO _itgrpCari itgrp2id FROM item WHERE itemid=_itemid;
	IF (_itgrpCari=_itgrp2id) THEN
		_retval:=true;
	END IF;

	OPEN cGrp FOR
		SELECT itgrp2id FROM itgrp2 WHERE upitgrp2id=_itgrp2id;
	LOOP
		FETCH cGrp INTO _itgrpid2;
		EXIT WHEN NOT FOUND;

		IF (_retval=false) THEN
			SELECT INTO _retval * FROM fitgrpIn(_itemid,_itgrpid2);
		END IF;
	END LOOP;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItGrp3In(_itemid VARCHAR,_itgrp3id VARCHAR)
RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;
	cGrp		REFCURSOR;
	_itgrpid2	VARCHAR(10);
	_itgrpCari	VARCHAR(10);
BEGIN
	_retval:=false;

	SELECT INTO _itgrpCari itgrp3id FROM item WHERE itemid=_itemid;
	IF (_itgrpCari=_itgrp3id) THEN
		_retval:=true;
	END IF;

	OPEN cGrp FOR
		SELECT itgrp3id FROM itgrp3 WHERE upitgrpid=_itgrp3id;
	LOOP
		FETCH cGrp INTO _itgrpid2;
		EXIT WHEN NOT FOUND;

		IF (_retval=false) THEN
			SELECT INTO _retval * FROM fitgrpIn(_itemid,_itgrpid2);
		END IF;
	END LOOP;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fbpgrpin (
	_bpid		VARCHAR,
	_bpgrpid	VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;
	cGrp		REFCURSOR;
	_bpgrpid2	VARCHAR(10);
	_bpgrpCari	VARCHAR(10);
BEGIN
	_retval:=false;

	SELECT INTO _bpgrpCari bpgrpid FROM bp WHERE bpid=_bpid;
	IF (_bpgrpCari=_bpgrpid) THEN
		_retval:=true;
	END IF;

	OPEN cGrp FOR
	SELECT bpgrpid FROM bpgrp WHERE upbpgrpid=_bpgrpid;
	LOOP
		FETCH cGrp INTO _bpgrpid2;
		EXIT WHEN NOT FOUND;
		if (_retval=false) THEN
			SELECT INTO _retval * FROM fbpgrpIn(_itemid,_bpgrpid2);
		END IF;
	END LOOP;
	CLOSE cGrp;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSetConsStatus(__Consno VARCHAR)
RETURNS SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	sisa 		NUMERIC(19,4);
	_ConsDNo 	VARCHAR;
	_ConsNo	VARCHAR;
	_Qty1_O		NUMERIC(15,4);		/** Sell In **/
	_Qty1_D		NUMERIC(15,4);		/** Sell Out **/
	_Qty1_B		NUMERIC(15,4);		/** Retur **/

	_Qty1_R1	NUMERIC(15,4);		/** R1 = Sell In - Sell Out ==> untuk menentukan ConsStatus **/
	_Qty1_R2	NUMERIC(15,4);		/** R2 = Sell IN - Sell Out - Retur ==> untuk menentukan Active **/

	_Expired	BOOLEAN;
	_IsFull		BOOLEAN;
	_IsPartial	BOOLEAN;
	_IsClear	BOOLEAN;
	_Active_orig	BOOLEAN;
	_Active		BOOLEAN;
	_ConsStatus	VARCHAR(1);
	cConsD		REFCURSOR;
BEGIN
	

	IF (__Consno IS NULL OR __Consno='') THEN	-- utk melayani SP lain yg memang gak nge-check valuenya, kita ngalahin ngecheck disini
		_retval:=(0,'OK');
		RETURN _retval;
	END IF;


	SELECT into _Active_orig Active from CONS WHERE ConsNo=__Consno;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Konsinyasi tidak ditemukan % (spCons_ChNUp_Status)',__Consno;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_IsFull=true;			/** default ConsStatus=FULL ***/
	_IsPartial=false;		/** default ConsStatus=FULL ***/
	_IsClear=true;			/** default CLOSED **/
	_Active=true;			/** default OPEN **/

	OPEN cConsD FOR
	SELECT ConsDNo, fQtyZToQty1(ItemID, QtySellIn, Unit), QtySellOut, QtyCRet from consd WHERE ConsNo=__Consno;
	LOOP
		FETCH cConsD INTO _ConsDNo, _Qty1_O, _Qty1_D, _Qty1_B;
		EXIT WHEN NOT FOUND;

		IF(_Qty1_O<0) THEN
			_Qty1_O:=_Qty1_O*(-1);
		END IF;

		IF(_Qty1_D<0) THEN
			_Qty1_D:=_Qty1_D*(-1);
		END IF;

		IF(_Qty1_B<0) THEN
			_Qty1_B:=_Qty1_B*(-1);
		END IF;

		_Qty1_R1 = _Qty1_O - _Qty1_D;
		_Qty1_R2 = _Qty1_O - _Qty1_D - _Qty1_B;

		IF (_Qty1_R1>0) THEN
			_IsFull:=false;
		END IF;

		IF (_Qty1_D>0) THEN
			_IsPartial:=true;
		END IF;

		IF (_Qty1_R2>0) THEN
			_IsClear:=false;
		END IF;
	END LOOP;
	CLOSE cConsD;

	IF (_IsFull=true) THEN _ConsStatus:='F';
	ELSIF ((_IsFull=false) AND (_IsPartial=true)) THEN _ConsStatus:='P';
	ELSIF ((_IsFull=false) AND (_IsPartial=false)) THEN _ConsStatus:='N';
	END IF;

	IF (_IsClear=true) THEN 
		_Active:=false;
	END IF;

	UPDATE Cons SET ConsStatus=_ConsStatus, Active=_Active WHERE ConsNo=__Consno;


_retval=(0, 'OK');
RETURN _retval;

END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION fqtyDecode (
	_itemid		VARCHAR,
	_qty1		NUMERIC
) RETURNS tQtyDecode AS
$BODY$
/*-- _ItemID = item id*/
/*-- _Qty1   = banyak item pada unit 1*/
DECLARE
	_retval		tQtyDecode;
	_Unit1		VARCHAR(5);
	_Unit2		VARCHAR(5);
	_Conv2		NUMERIC(15,4);
	_Unit3		VARCHAR(5);
	_Conv3		NUMERIC(15,4);
	_tmp		NUMERIC(15,4);
	_tmp2		INT4;
	_tmpStr		VARCHAR(255);
BEGIN
	_retval.Qty1:=0;
	_retval.Qty2:=0;
	_retval.Qty3:=0;
	SELECT INTO _Unit1,_Unit2,_Unit3,_Conv2,_Conv3 unit1,unit2,unit3,conv2,conv3
	FROM item WHERE itemid=_ItemID;

	IF NOT FOUND OR _Qty1<=0 THEN
		_retval.Qty1:=_Qty1;
		RETURN _retval;
	END IF;
	_tmp := abs(_Qty1);
	IF _tmp <> 0 THEN
		IF _Unit3 IS NOT NULL AND _Unit3 <> '' THEN
			IF _tmp >= _Conv3 THEN
				_tmp2:= CAST(CAST(_tmp AS INT4)/CAST(_Conv3 AS INT4) AS INT4);
				IF _tmp2 > 0 THEN
					_retval.Qty3:=_tmp2;
					_tmp:= _tmp - (_tmp2 *_Conv3);
				END IF;
			END IF;
		END IF;
		IF _Unit2 IS NOT NULL AND _Unit2<>'' THEN
			IF _tmp>=_Conv2 THEN
				_tmp2= CAST(CAST(_tmp AS INT4)/CAST(_Conv2 AS INT4) AS INT4);
				IF _tmp2>0 THEN
					_retval.Qty2:=_tmp2;
					_tmp:= _tmp - (_tmp2 *_Conv2);	
				END IF;
			END IF;
		END IF;
		IF _tmp>0 THEN
			_retval.Qty1:=_tmp;
		END IF;
	END IF;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fqtySmartDecode (
	_itemid		VARCHAR, 
	_qty1		NUMERIC
) RETURNS tQtySmartDecode AS
$BODY$
DECLARE
	_retval		tQtySmartDecode;

	_Unit1		VARCHAR(5);
	_Unit2		VARCHAR(5);
	_Conv2		NUMERIC(15,4);
	_Unit3		VARCHAR(5);
	_Conv3		NUMERIC(15,4);
BEGIN
	SELECT INTO _Unit1,_Unit2,_Unit3,_Conv2,_Conv3 unit1,unit2,unit3,conv2,conv3
	FROM item WHERE itemid=_ItemID;
	_retval.Qty:=0;
	_retval.Unit:='';
	IF _Unit3 IS NOT NULL AND _Unit3 <> '' AND _retval.Unit='' THEN
		IF _qty1%_Conv3=0 THEN
			_retval.Qty:=_qty1/_Conv3;
			_retval.Unit:=_Unit3;
		END IF;
	END IF;
	IF _Unit2 IS NOT NULL AND _Unit2 <> '' AND _retval.Unit='' THEN
		IF _qty1%_Conv2=0 THEN
			_retval.Qty:=_qty1/_Conv2;
			_retval.Unit:=_Unit2;
		END IF;
	END IF;
	IF _retval.Unit='' THEN
		_retval.Qty:=_qty1;
		_retval.Unit:=_Unit1;
	END IF;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJnl_New(
	_JnlNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_AccNo			VARCHAR(10);
	_JnlDate		DATE;
	_amount			NUMERIC(19,4);
	_amountD		NUMERIC(19,4);
	_amountK		NUMERIC(19,4);
	_Count			INT;
	_DK			VARCHAR(1);
	_DefDK			VARCHAR(1);
	_AccType2 		VARCHAR(5);
	_CashID 		VARCHAR(15);
	_Auto			BOOLEAN;
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_JnlNote		VARCHAR(255);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);

	_tmpNote		VARCHAR(255);
	_RefType		VARCHAR;
	_RefNo			VARCHAR;
	_LockDate		DATE;
	_PerStart		DATE;

	tAccNo			VARCHAR;
	tJnlNo			VARCHAR;
	tJnlDNo			SMALLINT;

	cAcc			REFCURSOR;
	ctmp			REFCURSOR;
	cJnlD			REFCURSOR;

	_JnlDNo			SMALLINT;
	_selisih		DECIMAL(19,4);
	_JnlDAmt		DECIMAL(19,4);
	_JnlDAmt_Rounded	DECIMAL(19,4);
	_JnlDAmt_selisih	DECIMAL(19,4);

	_debet_found 		BOOLEAN;
	_kredit_found 		BOOLEAN;

	_jnldcount		INT;
	_ronde			INT;
BEGIN
	-- Dont process posting, for debugging purpose, to see what is the jurnal details amounts before gets actually posted

	--_retval:=(0, 'OK');
	--RETURN NEXT _retval;
	--RETURN;

	SELECT INTO 
		_CrtBy, _CrtDate,_UpdBy, _UpdDate, _JnlDate, _Auto, _JnlNote, _PerID, _BranchID, _RefType, _RefNo
		CrtBy, CrtDate,UpdBy, UpdDate, jnldate, isautogen, JnlNote, PerID, BranchID, reftype, refno 
	FROM jnl 
	WHERE JnlNo=_JnlNo;
	IF (NOT FOUND) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Jurnal Voucher#'||COALESCE(_JnlNo, '<kosong>')||' tidak ditemukan ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _jnldcount count(*) FROM jnld WHERE jnlNo=_JnlNo;

	IF (_jnldcount IS NULL OR _jnldcount=0) THEN
		DELETE FROM jnl WHERE JnlNo=_JnlNo;
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_ronde:=0;
	LOOP
		_ronde:=_ronde+1;

		SELECT INTO _amountD SUM(debit) FROM jnld WHERE JnlNo=_JnlNo;
		_debet_found:=FOUND;
		SELECT INTO _amountK SUM(credit) FROM jnld WHERE JnlNo=_JnlNo;
		_kredit_found:=FOUND;

		IF (_debet_found=FALSE AND _kredit_found=FALSE) THEN
			RAISE EXCEPTION 'Tidak ada transaksi di DEBET maupun KREDIT (spJnl_New)';
		END IF;

		_amountD:=COALESCE(_amountD, 0);
		_amountK:=COALESCE(_amountK, 0);

		_selisih:=_amountD-_amountK;

		EXIT WHEN (_selisih=0);

		IF (_ronde>1 AND _selisih<>0) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Referensi('||COALESCE(_RefType, '<kosong>')||')#'||COALESCE(_RefNo, '<kosong>')||E'\n'||COALESCE(_JnlNote, '<kosong>')||E'\n'||'Jumlah Debet Kredit Tidak Sama!(spJnl_New) '||_amountD||' -VS- '||_amountK;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_selisih<>0) THEN
			OPEN cJnlD FOR 
				SELECT DK, JnlDNo, JnlDAmt FROM JnlD WHERE JnlNo=_JnlNo;
			LOOP
				FETCH cJnlD INTO _DK, _JnlDNo, _JnlDAmt;
				EXIT WHEN NOT FOUND;

				_JnlDAmt_Rounded:=ROUND(_JnlDAmt, 4);
				_JnlDAmt_selisih:=_JnlDAmt-_JnlDAmt_Rounded;
			
				IF (_JnlDAmt_selisih<>0) THEN
					IF (_DK='D') THEN
						UPDATE JnlD SET Debit=_JnlDAmt_Rounded, JnlDAmt=_JnlDAmt_Rounded WHERE JnlNo=_JnlNo AND JnlDNo=_JnlDNo;
					ELSIF (_DK='K') THEN
						UPDATE JnlD SET Credit=_JnlDAmt_Rounded, JnlDAmt=_JnlDAmt_Rounded WHERE JnlNo=_JnlNo AND JnlDNo=_JnlDNo;
					END IF;
				END IF;
			END LOOP;
			CLOSE cJnlD;
		END IF;
	END LOOP;


	SELECT INTO 
		_CrtBy, _CrtDate,_UpdBy, _UpdDate, _JnlDate, _Auto, _JnlNote, _PerID, _BranchID
		CrtBy, CrtDate,UpdBy, UpdDate, jnldate, isautogen, JnlNote, PerID, BranchID
	FROM jnl 
	WHERE JnlNo=_JnlNo;

	SELECT INTO _PerStart perstart from per where active=true;
	IF(_PerStart>_JnlDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
	IF(_LockDate is not null AND _LockDate>_JnlDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _amount SUM(debit) FROM jnld WHERE JnlNo=_JnlNo;
	UPDATE jnl SET jnlamt=_amount WHERE JnlNo=_JnlNo;

	_tmpNote:=_JnlNote;

	INSERT INTO lgr(
		accno, jnlno, jnldno, lgrdate, lgramt, lgrnote, 
		PerID, BranchID, DeptID, PrjID, LgrAmtForex, CrcID, ExcRate, FisRate
		)
	SELECT 
		d.accno, d.jnlno, jnldno, _JnlDate,
		CASE WHEN a.accdk=d.dk THEN jnldamt ELSE jnldamt*-1 END,
		COALESCE(jnlnote,' ')||' '||COALESCE(jnldnote,' '),
		j.PerID,  j.BranchID, d.DeptID, d.PrjID, JnlDAmtForex, j.CrcID, j.ExcRate, j.FisRate
	FROM jnld d
	JOIN jnl j on j.jnlno=d.jnlno 
	JOIN acc a on d.accno=a.accno
	WHERE d.JnlNo=_JnlNo;

	
	OPEN cAcc FOR
		SELECT 
			acctype2.acctype2, jnld.accno, acc.AccDK, dk, jnldamt, branchid, deptid, prjid 
		FROM jnld
		JOIN jnl ON jnl.jnlno=jnld.jnlno 
		JOIN acc ON jnld.accno=acc.accno 
		LEFT JOIN acctype2 ON acc.acctype2=acctype2.acctype2
		WHERE jnld.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccType2, _AccNo, _DefDK, _DK, _amount, _BranchID, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		IF (_DefDK<>_DK) THEN	
			_amount:=_amount*-1;
		END IF;

		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_JnlDate,_amount, _BranchID, _DeptID, _PrjID);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cAcc;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJnl_Void (
	_JnlNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	cAcc		REFCURSOR;
	_AccNo		VARCHAR(10);
	_tgl		DATE;
	_amount		NUMERIC(19,4);
	_DK		VARCHAR(1);
	_DefDK		VARCHAR(1);
	_isAutoGen	BOOLEAN;
	_LockDate	DATE;
	_BranchID	VARCHAR(5);
	_DeptID		VARCHAR(5);
	_PrjID		VARCHAR(5);
	_PerStart	DATE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='General Error (spJnl_Void)';

	SELECT INTO 
		_tgl, _isAutoGen
		jnldate, isAutoGen
	FROM jnl WHERE JnlNo=_JnlNo;

	SELECT INTO _PerStart perstart from per where active=true;
	IF(_PerStart>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
	IF(_LockDate is not null AND _LockDate>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cAcc FOR SELECT d.accno,d.dk,d.jnldamt,j.branchid,d.deptid,d.prjid FROM jnl j, jnld d WHERE j.jnlno=d.jnlno
	AND d.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccNo,_DK,_amount,_BranchID,_DeptID,_PrjID;
		EXIT WHEN NOT FOUND;
		SELECT INTO _DefDK accdk FROM acc WHERE AccNo=_AccNo;
		IF _DefDK=_DK THEN	
			_amount:=_amount*-1;
		END IF;			
		
		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_tgl,_amount,_BranchID,_DeptID,_PrjID);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
	END LOOP;

	/** Void nested trans: BPA, CashA */

	IF (_isAutoGen=false) THEN
		SELECT INTO _spret * FROM spBPA_Void_ByRef('JNL', _JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spCashA_Void_ByRef('JNL', _JnlNo, NULL, NULL);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;		
	END IF;
	/***/

	_retval:=(50010, 'Error Delete Lgr (spJnl_Void)');
	DELETE FROM Lgr WHERE jnlno=_JnlNo;

	_retval:=(50015, 'Error Delete JnlD (spJnl_Void)');
	DELETE FROM Jnld WHERE jnlno=_JnlNo;

	_retval:=(50020, 'Error Delete Jnl (spJnl_Void)');
	DELETE FROM Jnl WHERE jnlno=_JnlNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spAccBal_Update(
	__AccNo		VARCHAR(10),
	__AccDate	DATE,
	__Amount	NUMERIC(19,4),
	__BranchID	VARCHAR(5),
	__DeptID	VARCHAR(5),
	__PrjID		VARCHAR(5)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	
	_Month			INT4;
	_Year			INT4;
	_FYear			INT4;
	_BegBal			NUMERIC(19,4);
	_UpAccNo		VARCHAR(15);
	_IsHeader		BOOL;
	_OpenDate		DATE;
	_Amount			NUMERIC(19,4);
	_BranchId		VARCHAR(5);
	_DeptId			VARCHAR(5);
	_PrjId			VARCHAR(5);

	_Found			VARCHAR;
	
	cFutureBal		CURSOR (_CYear INT4) IS 
					SELECT pyear FROM accbal 
					WHERE accno=__AccNo AND pyear>_CYear 
						AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
						AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
						AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
BEGIN

	_Month=DATE_PART('month', __AccDate);
	_Year=DATE_PART('year', __AccDate);

	UPDATE acc SET accbal=accbal+__Amount WHERE accno=__AccNo;		/* UPDATE saldo di master Cash */
	IF NOT FOUND THEN
		_retval.ErrNo:=4500;
		_retval.ErrMsg:='Gagal Update AccBal (spAccBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	-- cek apakah saldo utk tahun ini sudah ada 
	SELECT INTO _Found accno 
	FROM accbal 
	WHERE accno=__AccNo AND pyear=_Year 
	AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
	AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
	AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	IF NOT FOUND THEN
		SELECT INTO _BegBal (accbal00+accbal01+accbal02+accbal03+accbal04+accbal05+accbal06+accbal07+accbal08+accbal09+accbal10+accbal11+accbal12)
		FROM accbal WHERE accno=__AccNo 
			AND pyear=(
				SELECT MAX(pyear) FROM accbal WHERE accno=__AccNo AND pyear<_Year 
				AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
				AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
				AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END)
			)
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
		IF NOT FOUND THEN
			_BegBal:=0;
		END IF;

		INSERT INTO accbal(accno, pyear, accbal00, branchid, deptid, prjid) 
		VALUES (__AccNo, _Year, _BegBal, __BranchID, __DeptID, __PrjID);
		IF NOT FOUND THEN
			_retval.ErrNo:=1012;
			_retval.ErrMsg:='Error creating AccBal for ' || __AccNo || ' : Year ' || CAST(_Year AS VARCHAR) || ' (spAccBal_Update)' || CAST(_BegBal AS VARCHAR) ;
			RAISE EXCEPTION '%', _retval.ErrMsg;					
		END IF;
	END IF;
	
	_retval.ErrNo:=1013;
	_retval.ErrMsg:='Error UPDATING AccBal:month FOR ' || __AccNo || ' : Month ' || CAST(_Month AS VARCHAR) || ' (spAccBal_Update)';
	
	IF (_Month=1) THEN 
		UPDATE accbal SET accbal01=accbal01+__Amount WHERE accno=__AccNo AND pyear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=2) THEN 
		UPDATE accbal SET accbal02=accbal02+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=3) THEN 
		UPDATE accbal SET accbal03=accbal03+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=4) THEN 
		UPDATE accbal SET accbal04=accbal04+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=5) THEN 
		UPDATE accbal SET accbal05=accbal05+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=6) THEN 
		UPDATE accbal SET accbal06=accbal06+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=7) THEN 
		UPDATE accbal SET accbal07=accbal07+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=8) THEN 
		UPDATE accbal SET accbal08=accbal08+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=9) THEN 
		UPDATE accbal SET accbal09=accbal09+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=10) THEN 
		UPDATE accbal SET accbal10=accbal10+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=11) THEN 
		UPDATE accbal SET accbal11=accbal11+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	ELSIF (_Month=12) THEN 
		UPDATE accbal SET accbal12=accbal12+__Amount WHERE AccNo=__AccNo AND PYear=_Year 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
	END IF;

	IF NOT FOUND THEN /*-- CEK JIKA UPDATE GAGAL RAISE EXCEPTION*/
		RAISE EXCEPTION '%', _retval.ErrMsg;					
	END IF;


	OPEN cFutureBal(_Year);
	LOOP
		FETCH cFutureBal INTO _FYear;
		EXIT WHEN NOT FOUND;	

		UPDATE accbal SET accbal00=accbal00+__Amount WHERE accno=__AccNo AND pyear=_FYear 
			AND (CASE WHEN __BranchID IS NULL THEN branchid IS NULL OR branchid='' ELSE BranchID=__BranchID END) 
			AND (CASE WHEN __DeptID IS NULL THEN deptid IS NULL OR deptid='' ELSE DeptID=__DeptID END) 
			AND (CASE WHEN __PrjID IS NULL THEN prjid IS NULL OR prjid='' ELSE prjid=__PrjID END);
		IF NOT FOUND THEN
			_retval.ErrNo:=1014;
			_retval.ErrMsg:='Error UPDATE future AccBal (spAccBal_Update)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cFutureBal;

	SELECT INTO 
		_UpAccNo, _IsHeader, _OpenDate, _Amount, _BranchId, _DeptId, _PrjId
		a.upaccno, a.isheader, a.opendate, a.accbal, b.branchid, b.deptid, b.prjid
	FROM acc a JOIN accbal b ON b.accno=a.accno 
	WHERE a.accno=__AccNo;


	IF ( _IsHeader=false AND _upAccno IS NOT NULL AND _UpAccNo<>'*') THEN

		SELECT INTO _spret * FROM spAccBal_Update(_UpAccNo, __AccDate, __Amount, _BranchId, _DeptId, _PrjId);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spJnl_Insert(
	__JnlNo			VARCHAR(15),
	_JnlType		VARCHAR(5),
	_JnlDate		DATE,
	_RefType		VARCHAR(5),
	_RefNo			VARCHAR(15),
	_JnlNote		TEXT,
	_IsAutoGen		BOOLEAN,
	_IsPosted		BOOLEAN,
	_Amount			NUMERIC(19,4),
	_CrtBy			VARCHAR(10),
	_CrtDate		TIMESTAMP,
	_UpdBy			VARCHAR(10),
	_UpdDate		TIMESTAMP,
	_PerID			VARCHAR(5),
	_BranchID		VARCHAR(10),
	_DeptID			VARCHAR(5),
	_PrjID			VARCHAR(5),
	_CrcID			VARCHAR,
	_ExcRate		NUMERIC(19,4),
	_FisRate		NUMERIC(19,4)
)
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_JnlNo		varchar(15);
BEGIN

		_JnlNo:=__JnlNo;
		IF (_JnlNo IS NULL) THEN
			_JnlNo:=fTransNoGet('JURNAL');	
			IF (_JnlNo IS NULL) THEN
				_retval.ErrNo:=50002;
				_retval.ErrMsg:='ERROR generate Jurnal.No (spJurnal_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;		
		END IF;

		INSERT INTO Jnl (
			JnlNo, JnlType, JnlDate, 
			RefType, RefNo, JnlNote,IsAutoGen,jnlAmt,IsPosted,
			CrtBy, CrtDate, UpdBy, UpdDate,
			PerID, BranchID, CrcID, ExcRate, FisRate
			/*--, DeptID, PrjID*/			
			)
		VALUES (
			_JnlNo,  _JnlType, _JnlDate,
			_RefType, _RefNo, _JnlNote, _IsAutoGen,_Amount,_IsPosted,
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _CrcID, _ExcRate, _FisRate
			/*--, _DeptID, _PrjID*/			
			);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

/*Parameter nya ditambahi _xt*/
CREATE OR REPLACE FUNCTION spJnl_Insert(
	__JnlNo			VARCHAR(15),
	_JnlType		VARCHAR(5),
	_JnlDate		DATE,
	_RefType		VARCHAR(5),
	_RefNo			VARCHAR(15),
	_JnlNote		TEXT,
	_IsAutoGen		BOOLEAN,
	_IsPosted		BOOLEAN,
	_Amount			NUMERIC(19,4),
	_CrtBy			VARCHAR(10),
	_CrtDate		TIMESTAMP,
	_UpdBy			VARCHAR(10),
	_UpdDate		TIMESTAMP,
	_PerID			VARCHAR(5),
	_BranchID		VARCHAR(10),
	_DeptID			VARCHAR(5),
	_PrjID			VARCHAR(5),
	__xt			BOOLEAN,
	_CrcID			VARCHAR,
	_ExcRate		NUMERIC(19,4),
	_FisRate		NUMERIC(19,4)
)
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_JnlNo		varchar(15);
BEGIN

		_JnlNo:=__JnlNo;
		IF (_JnlNo IS NULL) THEN
			_JnlNo:=fTransNoGet('JURNAL');	
			IF (_JnlNo IS NULL) THEN
				_retval.ErrNo:=50002;
				_retval.ErrMsg:='ERROR generate Jurnal.No (spJurnal_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;		
		END IF;

		INSERT INTO Jnl (
			JnlNo, JnlType, JnlDate, 
			RefType, RefNo, JnlNote,IsAutoGen,jnlAmt,IsPosted,
			CrtBy, CrtDate, UpdBy, UpdDate,
			PerID, BranchID, _xt, CrcID, ExcRate, FisRate
			/*--, DeptID, PrjID*/			
			)
		VALUES (
			_JnlNo,  _JnlType, _JnlDate,
			_RefType, _RefNo, _JnlNote, _IsAutoGen,_Amount,_IsPosted,
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, __xt, _CrcID, _ExcRate, _FisRate
			/*--, _DeptID, _PrjID*/			
			);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;





/** complete version, 9 parameter **/
CREATE OR REPLACE FUNCTION spJnlD_Insert ( 
	__JnlNo VARCHAR, 
	__JnlDNo INT, 
	__AccNo VARCHAR, 
	__DK VARCHAR, 
	__Amount DECIMAL(19,4), 
	__DeptID VARCHAR, 
	__PrjID VARCHAR,
	__JnlDNote TEXT,
	__AmountForex DECIMAL(19,4)
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_DK		VARCHAR;
	_Amount		DECIMAL(19,4);
	_AmountForex	DECIMAL(19,4);
	_found		BOOLEAN;
BEGIN
	IF (__AccNo IS NULL) THEN
		_retval:=(50001, 'Error, Akun IS NULL (spJnlD_Insert)'||__Amount);
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _found TRUE FROM Acc WHERE AccNo=__AccNo;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Akun:% tidak ditemukan (spJnlD_Insert)', __AccNo;
	END IF;
	
	
	_DK:=__DK;
	_Amount:=__Amount;
	_AmountForex:=__AmountForex;
	IF (_Amount<0) THEN	-- jika negatif diputar aja D/K nya
		_Amount:=_Amount*-1;
		_AmountForex:=_AmountForex*-1;

		IF (_DK='D') THEN
			_DK:='K';
		ELSE
			_DK:='D';
		END IF;
	END IF;
	
	INSERT INTO jnld (JnlNo, JnlDNo, AccNo, debit, credit, dk, jnldamt, DeptID, PrjID, jnldamtForex, JnlDNote)
	VALUES (__JnlNo, __JnlDNo, __AccNo, 
		CASE WHEN _DK='D' THEN _Amount ELSE 0 END,
		CASE WHEN _DK='K' THEN _Amount ELSE 0 END,
		_DK, _Amount, __DeptID, __PrjID, _AmountForex, __JnlDNote
	);

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Error INSERT JnlD Acc:% = %  (spJnlD_Insert)', __AccNo, _Amount;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


---------------------------------- OVERLOAD tanpa jnldnote --------------------------------------------
/** overload version, 8 parameter **/

CREATE OR REPLACE FUNCTION spJnlD_Insert ( 
	__JnlNo VARCHAR, 
	__JnlDNo INT, 
	__AccNo VARCHAR, 
	__DK VARCHAR, 
	__Amount DECIMAL(19,4), 
	__DeptID VARCHAR, 
	__PrjID VARCHAR,
	__AmountForex DECIMAL(19,4)
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_spret		SPStatus%ROWTYPE;
	_retval		SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spJnlD_Insert(__JnlNo, __JnlDNo, __AccNo, __DK, __Amount, __DeptID, __PrjID, CAST(NULL AS VARCHAR), __AmountForex);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJnl_New(
	_JnlNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_AccNo			VARCHAR(10);
	_JnlDate		DATE;
	_amount			NUMERIC(19,4);
	_amountD		NUMERIC(19,4);
	_amountK		NUMERIC(19,4);
	_Count			INT;
	_DK			VARCHAR(1);
	_DefDK			VARCHAR(1);
	_AccType2 		VARCHAR(5);
	_CashID 		VARCHAR(15);
	_Auto			BOOLEAN;
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_JnlNote		VARCHAR(255);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);

	_tmpNote		VARCHAR(255);
	_RefType		VARCHAR;
	_RefNo			VARCHAR;
	_LockDate		DATE;
	_PerStart		DATE;

	tAccNo			VARCHAR;
	tJnlNo			VARCHAR;
	tJnlDNo			SMALLINT;

	cAcc			REFCURSOR;
	ctmp			REFCURSOR;
	cJnlD			REFCURSOR;

	_JnlDNo			SMALLINT;
	_selisih		DECIMAL(19,4);
	_JnlDAmt		DECIMAL(19,4);
	_JnlDAmt_Rounded	DECIMAL(19,4);
	_JnlDAmt_selisih	DECIMAL(19,4);

	_debet_found 		BOOLEAN;
	_kredit_found 		BOOLEAN;

	_jnldcount		INT;
	_ronde			INT;
BEGIN
	-- Dont process posting, for debugging purpose, to see what is the jurnal details amounts before gets actually posted

	--_retval:=(0, 'OK');
	--RETURN NEXT _retval;
	--RETURN;

	SELECT INTO 
		_CrtBy, _CrtDate,_UpdBy, _UpdDate, _JnlDate, _Auto, _JnlNote, _PerID, _BranchID, _RefType, _RefNo
		CrtBy, CrtDate,UpdBy, UpdDate, jnldate, isautogen, JnlNote, PerID, BranchID, reftype, refno 
	FROM jnl 
	WHERE JnlNo=_JnlNo;
	IF (NOT FOUND) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Jurnal Voucher#'||COALESCE(_JnlNo, '<kosong>')||' tidak ditemukan ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _jnldcount count(*) FROM jnld WHERE jnlNo=_JnlNo;

	IF (_jnldcount IS NULL OR _jnldcount=0) THEN
		DELETE FROM jnl WHERE JnlNo=_JnlNo;
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_ronde:=0;
	LOOP
		_ronde:=_ronde+1;

		SELECT INTO _amountD SUM(debit) FROM jnld WHERE JnlNo=_JnlNo;
		_debet_found:=FOUND;
		SELECT INTO _amountK SUM(credit) FROM jnld WHERE JnlNo=_JnlNo;
		_kredit_found:=FOUND;

		IF (_debet_found=FALSE AND _kredit_found=FALSE) THEN
			RAISE EXCEPTION 'Tidak ada transaksi di DEBET maupun KREDIT (spJnl_New)';
		END IF;

		_amountD:=COALESCE(_amountD, 0);
		_amountK:=COALESCE(_amountK, 0);

		_selisih:=_amountD-_amountK;

		EXIT WHEN (_selisih=0);

		IF (_ronde>1 AND _selisih<>0) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Referensi('||COALESCE(_RefType, '<kosong>')||')#'||COALESCE(_RefNo, '<kosong>')||E'\n'||COALESCE(_JnlNote, '<kosong>')||E'\n'||'Jumlah Debet Kredit Tidak Sama!(spJnl_New) '||_amountD||' -VS- '||_amountK;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_selisih<>0) THEN
			OPEN cJnlD FOR 
				SELECT DK, JnlDNo, JnlDAmt FROM JnlD WHERE JnlNo=_JnlNo;
			LOOP
				FETCH cJnlD INTO _DK, _JnlDNo, _JnlDAmt;
				EXIT WHEN NOT FOUND;

				_JnlDAmt_Rounded:=ROUND(_JnlDAmt, 4);
				_JnlDAmt_selisih:=_JnlDAmt-_JnlDAmt_Rounded;
			
				IF (_JnlDAmt_selisih<>0) THEN
					IF (_DK='D') THEN
						UPDATE JnlD SET Debit=_JnlDAmt_Rounded, JnlDAmt=_JnlDAmt_Rounded WHERE JnlNo=_JnlNo AND JnlDNo=_JnlDNo;
					ELSIF (_DK='K') THEN
						UPDATE JnlD SET Credit=_JnlDAmt_Rounded, JnlDAmt=_JnlDAmt_Rounded WHERE JnlNo=_JnlNo AND JnlDNo=_JnlDNo;
					END IF;
				END IF;
			END LOOP;
			CLOSE cJnlD;
		END IF;
	END LOOP;


	SELECT INTO 
		_CrtBy, _CrtDate,_UpdBy, _UpdDate, _JnlDate, _Auto, _JnlNote, _PerID, _BranchID
		CrtBy, CrtDate,UpdBy, UpdDate, jnldate, isautogen, JnlNote, PerID, BranchID
	FROM jnl 
	WHERE JnlNo=_JnlNo;

	SELECT INTO _PerStart perstart from per where active=true;
	IF(_PerStart>_JnlDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
	IF(_LockDate is not null AND _LockDate>_JnlDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _amount SUM(debit) FROM jnld WHERE JnlNo=_JnlNo;
	UPDATE jnl SET jnlamt=_amount WHERE JnlNo=_JnlNo;

	_tmpNote:=_JnlNote;

	INSERT INTO lgr(
		accno, jnlno, jnldno, lgrdate, lgramt, lgrnote, 
		PerID, BranchID, DeptID, PrjID, LgrAmtForex, CrcID, ExcRate, FisRate
		)
	SELECT 
		d.accno, d.jnlno, jnldno, _JnlDate,
		CASE WHEN a.accdk=d.dk THEN jnldamt ELSE jnldamt*-1 END,
		COALESCE(jnlnote,' ')||' '||COALESCE(jnldnote,' '),
		j.PerID,  j.BranchID, d.DeptID, d.PrjID, JnlDAmtForex, j.CrcID, j.ExcRate, j.FisRate
	FROM jnld d
	JOIN jnl j on j.jnlno=d.jnlno 
	JOIN acc a on d.accno=a.accno
	WHERE d.JnlNo=_JnlNo;

	
	OPEN cAcc FOR
		SELECT 
			acctype2.acctype2, jnld.accno, acc.AccDK, dk, jnldamt, branchid, deptid, prjid 
		FROM jnld
		JOIN jnl ON jnl.jnlno=jnld.jnlno 
		JOIN acc ON jnld.accno=acc.accno 
		LEFT JOIN acctype2 ON acc.acctype2=acctype2.acctype2
		WHERE jnld.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccType2, _AccNo, _DefDK, _DK, _amount, _BranchID, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		IF (_DefDK<>_DK) THEN	
			_amount:=_amount*-1;
		END IF;

		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_JnlDate,_amount, _BranchID, _DeptID, _PrjID);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cAcc;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJnl_Void (
	_JnlNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	cAcc		REFCURSOR;
	_AccNo		VARCHAR(10);
	_tgl		DATE;
	_amount		NUMERIC(19,4);
	_DK		VARCHAR(1);
	_DefDK		VARCHAR(1);
	_isAutoGen	BOOLEAN;
	_LockDate	DATE;
	_BranchID	VARCHAR(5);
	_DeptID		VARCHAR(5);
	_PrjID		VARCHAR(5);
	_PerStart	DATE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='General Error (spJnl_Void)';

	SELECT INTO 
		_tgl, _isAutoGen
		jnldate, isAutoGen
	FROM jnl WHERE JnlNo=_JnlNo;

	SELECT INTO _PerStart perstart from per where active=true;
	IF(_PerStart>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
	IF(_LockDate is not null AND _LockDate>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cAcc FOR SELECT d.accno,d.dk,d.jnldamt,j.branchid,d.deptid,d.prjid FROM jnl j, jnld d WHERE j.jnlno=d.jnlno
	AND d.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccNo,_DK,_amount,_BranchID,_DeptID,_PrjID;
		EXIT WHEN NOT FOUND;
		SELECT INTO _DefDK accdk FROM acc WHERE AccNo=_AccNo;
		IF _DefDK=_DK THEN	
			_amount:=_amount*-1;
		END IF;			
		
		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_tgl,_amount,_BranchID,_DeptID,_PrjID);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
	END LOOP;

	/** Void nested trans: BPA, CashA */

	IF (_isAutoGen=false) THEN
		SELECT INTO _spret * FROM spBPA_Void_ByRef('JNL', _JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spCashA_Void_ByRef('JNL', _JnlNo, NULL, NULL);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;		
	END IF;
	/***/

	_retval:=(50010, 'Error Delete Lgr (spJnl_Void)');
	DELETE FROM Lgr WHERE jnlno=_JnlNo;

	_retval:=(50015, 'Error Delete JnlD (spJnl_Void)');
	DELETE FROM Jnld WHERE jnlno=_JnlNo;

	_retval:=(50020, 'Error Delete Jnl (spJnl_Void)');
	DELETE FROM Jnl WHERE jnlno=_JnlNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJnl_Void_byRef (
	_RefType	VARCHAR(5),
	_RefNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_JnlNo		VARCHAR;
	cJnl		REFCURSOR;
BEGIN
	OPEN cJnl FOR
		SELECT JnlNo FROM Jnl WHERE RefType=_RefType AND RefNo=_RefNo;
	LOOP
		FETCH cJnl INTO _JnlNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spJnl_Void(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cJnl;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSetoranCanvas(
	_CanvasID	VARCHAR(5),
	_StartDate	DATE,
	_EndDate	DATE
) RETURNS SETOF tSetorCanvas AS 
$BODY$
DECLARE
	_retval		tSetorCanvas%ROWTYPE;
	cSetor		REFCURSOR;
	_WhID		VARCHAR(3);
	_tipe 		VARCHAR;
	_termtype 	VARCHAR;
	_id 		VARCHAR;
	_bpname 	VARCHAR;
	_total 		DECIMAL(15,4);
	_crcID		VARCHAR;
BEGIN
	OPEN cSetor FOR
		SELECT 'Sell In' AS tipe, c.consno, bp.bpname, c.total, c.crcid
		FROM cons c 
		JOIN bp ON c.custid=bp.bpid
		WHERE c.consdate>=_StartDate AND c.consdate<=_EndDate AND c.isdraft=false  
		AND (_CanvasID IS NULL OR c.canvasid=_CanvasID)
		AND c.canvasid IS NOT NULL
		ORDER BY tipe,c.consdate DESC;
	LOOP
		FETCH cSetor INTO _tipe,_id,_bpname,_total, _crcID;
		EXIT WHEN NOT FOUND;
	
		_retval.termtype:='Credit';
		_retval.saletype:=_tipe;
		_retval.id:=_id;
		_retval.custid:=_bpname;
		_retval.total:=_total;
		_retval.jenis:='NOTA ASLI';
		_retval.crcid:=_crcID;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cSetor;

	OPEN cSetor FOR
		SELECT s.termtype,saleno,bp.bpname,s.total,s.crcid 
		FROM sale s 
		JOIN bp ON s.custid=bp.bpid
		WHERE s.saledate>=_StartDate AND s.saledate<=_EndDate AND s.isdraft=false  
		AND (_CanvasID IS NULL OR s.canvasid=_CanvasID) AND s.termtype='R' AND s.canvasid IS NOT NULL
		ORDER BY saledate DESC;
	LOOP
		FETCH cSetor INTO _termtype,_id,_bpname,_total,_crcID;
		EXIT WHEN NOT FOUND;

		_retval.termtype:='Credit';
		_retval.saletype:='Penjualan Kredit';
		_retval.jenis:='NOTA ASLI';

		_retval.id:=_id;
		_retval.custid:=_bpname;
		_retval.total:=_total;
		_retval.crcid:=_crcID;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cSetor;

	OPEN cSetor FOR
		SELECT s.termtype,saleno,bp.bpname,s.total,s.crcid 
		FROM sale s 
		JOIN bp ON s.custid=bp.bpid
		WHERE s.saledate>=_StartDate AND s.saledate<=_EndDate AND (_CanvasID IS NULL OR s.canvasid=_CanvasID) 
		AND s.termtype='C' AND s.isdraft=false AND s.canvasid IS NOT NULL
		ORDER BY saledate DESC;
	LOOP
		FETCH cSetor INTO _termtype,_id,_bpname,_total,_crcid;
		EXIT WHEN NOT FOUND;

		_retval.termtype:='Cash';
		_retval.saletype:='Penjualan Tunai';
		_retval.jenis:='UANG';

		_retval.id:=_id;
		_retval.custid:=_bpname;
		_retval.total:=_total;
		_retval.crcid:=_crcID;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cSetor;

	OPEN cSetor FOR
		SELECT 'Sell Out' AS tipe,s.termtype,saleno,bp.bpname,s.total,s.crcid 
		FROM sale s 
		JOIN bp ON s.custid=bp.bpid 
		LEFT JOIN cons c ON s.consno=c.consno
		WHERE s.saledate>=_StartDate AND s.saledate<=_EndDate 
		AND s.consno IS not null AND s.isdraft=false 
		AND (_CanvasID IS NULL OR c.canvasid=_CanvasID)
		ORDER BY saledate DESC;
	LOOP
		FETCH cSetor INTO _tipe,_termtype,_id,_bpname,_total,_crcID;
		EXIT WHEN NOT FOUND;

		IF (_termtype='R') THEN
			_retval.termtype:='Credit';
		ELSE
			_retval.termtype:='Cash';
		END IF;
		_retval.saletype:=_tipe;
		_retval.id:=_id;
		_retval.custid:=_bpname;
		_retval.total:=_total;
		_retval.jenis:='UANG';
		_retval.crcid:=_crcID;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cSetor;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fQty1ToQtyZ (
	_ItemID		VARCHAR,
	_QtyUnit1	DECIMAL,
	_Unit		VARCHAR
) RETURNS NUMERIC AS 
$BODY$
DECLARE
	_Unit1		VARCHAR(5);
	_Unit2		VARCHAR(5);
	_Unit3		VARCHAR(5);
	_Conv2		NUMERIC(15,4);
	_Conv3		NUMERIC(15,4);
	_Qty		NUMERIC(15,4);
BEGIN
	SELECT INTO _Unit1, _Unit2, _Unit3, _Conv2, _Conv3
		Unit1, Unit2, Unit3, Conv2, Conv3
	FROM Item
	WHERE ItemID=_ItemID;

	IF NOT FOUND THEN
		EXIT;
	END IF;

	IF (_Unit=_Unit1) THEN
		_Qty=_QtyUnit1;
	ELSIF (_Unit2 IS NOT NULL) AND (_Unit2<>'') AND (_Unit=_Unit2) THEN
		_Qty=_QtyUnit1/_Conv2;
	ELSIF (_Unit3 IS NOT NULL) AND (_Unit3<>'') AND (_Unit=_Unit3) THEN
		_Qty=_QtyUnit1/_Conv3;
	END IF;

	RETURN _Qty;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fQtyZToQty1(
	_ItemID		VARCHAR(20),
	_Qty		NUMERIC(15,4),
	_Unit		VARCHAR(5)
) RETURNS NUMERIC(15,4) AS 
$BODY$
DECLARE
	_Unit1		VARCHAR(5);
	_Unit2		VARCHAR(5);
	_Unit3		VARCHAR(5);
	_Conv2		NUMERIC(15,4);
	_Conv3		NUMERIC(15,4);
	_Qty_Unit1	NUMERIC(15,4);

BEGIN
	SELECT INTO _Unit1,_Unit2,_Unit3,_Conv2,_Conv3 Unit1,Unit2,Unit3,Conv2,Conv3
	FROM Item WHERE ItemID=_ItemID;

	IF NOT FOUND THEN
		_Qty_Unit1:=0;
	END IF;

	/* konversi UNIT ke UNIT1 */
	IF _Unit1=_Unit THEN
		_Qty_Unit1:=_Qty;
	ELSIF _Unit2 IS NOT NULL AND _Unit2!='' AND _Unit2=_Unit THEN
		_Qty_Unit1:=_Qty*_Conv2;
	ELSIF _Unit3 IS NOT NULL AND _Unit3!='' AND _Unit3=_Unit THEN
		_Qty_Unit1:=_Qty*_Conv3;
	END IF;

	RETURN _Qty_Unit1;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




/** cari nilai konversi dari kesebandingan */
CREATE OR REPLACE FUNCTION fPortionItemValue(
	_ItemID		VARCHAR,
	_BaseQty	NUMERIC(15,4),
	_BaseUnit	VARCHAR,
	_BaseValue 	NUMERIC(19,4),
	_NewQty		NUMERIC(15,4),
	_NewUnit	VARCHAR
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	_retval:=0;

	_retval:=(fQtyZToQty1(_ItemID, _NewQty, _NewUnit)/fQtyZToQty1(_ItemID, _BaseQty, _BaseUnit))*_BaseValue;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPortionTotalValue( 
	_Total		NUMERIC(19,4),
	_BaseTotal	NUMERIC(19,4),
	_NewTotal	NUMERIC(19,4)
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	_retval:=0;

	_retval=(_NewTotal/_Total)*_BaseTotal;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: fqty1desc(_itemid VARCHAR, _qty1 NUMERIC)*/

/*-- DROP FUNCTION fqty1desc(_itemid VARCHAR, _qty1 NUMERIC);*/

CREATE OR REPLACE FUNCTION fqty1desc (
	_itemid		VARCHAR,
	_qty1		NUMERIC
) RETURNS VARCHAR AS
$BODY$
/*-- _ItemID = item id*/
/*-- _Qty1   = banyak item pada unit 1*/
DECLARE
	_Unit1		VARCHAR(5);
	_Unit2		VARCHAR(5);
	_Conv2		NUMERIC(15,4);
	_Unit3		VARCHAR(5);
	_Conv3		NUMERIC(15,4);
	_tmp		NUMERIC(15,4);
	_tmp2		INT4;
	_tmpStr		VARCHAR(255);
	_ret 		VARCHAR(255);
	_isMinus 	BOOLEAN;

	_subStr		VARCHAR;
BEGIN
	SELECT INTO _Unit1,_Unit2,_Unit3,_Conv2,_Conv3 unit1,unit2,unit3,conv2,conv3
	FROM item WHERE itemid=_ItemID;

	IF NOT FOUND OR _Qty1=0 THEN
		_ret:=cast(_Qty1 as integer)||' ' || _Unit1;
		RETURN _ret;
	END IF;

	_isMinus:=false;
	IF _Qty1<0 THEN
		_isMinus:=true;
	END IF;

	_ret='';
	_tmp := abs(_Qty1);
	IF _tmp <> 0 THEN
		IF _Unit3 IS NOT NULL AND _Unit3 <> '' THEN
			IF _tmp >= _Conv3 AND _Conv3!=0 THEN
				/*_tmp2:= CAST(CAST(_tmp AS INT4)/CAST(_Conv3 AS INT4) AS INT4);*/
				_tmp2:=FLOOR(_tmp/_Conv3);
				IF _tmp2 > 0 THEN
					_ret:= _ret || CAST(_tmp2 AS VARCHAR) ||' '|| _Unit3;
					_tmp:= _tmp - (_tmp2 *_Conv3);	
				END IF;
			END IF;
		END IF;

		IF _Unit2 IS NOT NULL AND _Unit2<>'' THEN
			IF _tmp>=_Conv2 AND _Conv2!=0 THEN
				/*_tmp2= CAST(CAST(_tmp AS INT4)/CAST(_Conv2 AS INT4) AS INT4);*/
				_tmp2:=FLOOR(_tmp/_Conv2);
				IF _tmp2>0 THEN
					IF _ret<>'' THEN
						_ret=_ret||', ';
					END IF;

					_ret:= _ret || CAST(_tmp2 AS VARCHAR) ||' ' || _Unit2;
					_tmp:= _tmp - (_tmp2 *_Conv2);	
				END IF;
			END IF;
		END IF;

		IF _tmp>0 THEN
			IF _ret<>'' THEN
				_ret=_ret||', ';
			END IF;
			/** _tmpStr=CAST( CAST(_tmp AS INT) AS VARCHAR);*/
			_tmpStr:=CAST(_tmp AS VARCHAR);
			LOOP
				_subStr:=SUBSTRING(_tmpStr FROM LENGTH(_tmpstr) FOR 1);
				IF (_subStr='0') THEN
					_tmpStr:=SUBSTRING(_tmpStr FROM 1 FOR LENGTH(_tmpStr)-1);
				ELSIF (_subStr='.') THEN /*BUAT YANG BELAKANG KOMA NYA NOL, KOMANYA DIHILANGIN*/
					_tmpStr:=SUBSTRING(_tmpStr FROM 1 FOR LENGTH(_tmpStr)-1);
					EXIT;
				ELSE
					EXIT;
				END IF;
			END LOOP;

			_ret:=_ret||_tmpStr||' '||_Unit1;
		END IF;
	ELSE
		_ret='0 ' || _Unit1;
		IF _Qty1<0 THEN
			_ret='- '||_ret;
		END IF;
	END IF;

	IF _isMinus=true THEN
		_ret :='-' || _ret;
	END IF;

	RETURN _ret;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
ALTER FUNCTION fqty1desc(_itemid VARCHAR, _qty1 NUMERIC) OWNER TO bits;




CREATE OR REPLACE FUNCTION fsaldoakhir (
	_accno		VARCHAR,
	_enddate	DATE,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_LastMonthBal	NUMERIC(19,4);
	_YesterdayBal	NUMERIC(19,4);
	_Month		SMALLINT;
	_PYear		INTEGER;
	_DefaultDK	VARCHAR(1);
BEGIN
	_Month:=DATE_part('month', _EndDate);
	_PYear:=DATE_part('year', _EndDate);

	IF (_Month=1) THEN
		SELECT INTO _LastMonthBal accBal00 
		FROM accBal
		WHERE accNo=_accNo AND PYear=_PYear 
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=2) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01 
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=3) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02 
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=4) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03 
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=5) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03+accBal04 
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=6) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03+accBal04+accBal05 
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=7) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=8) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=9) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=10) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08+accBal09
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=11) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08+accBal09+accBal10
		FROM accBal 
		WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	ELSIF (_Month=12) THEN
		SELECT INTO _LastMonthBal accBal00+accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08+accBal09+accBal10+accBal11
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID);
	END IF;

	IF (_LastMonthBal IS NULL) THEN
		_LastMonthBal:=0;
	END IF; 
	SELECT INTO _DefaultDK AccDK FROM ACC WHERE AccNo=_AccNo;
	SELECT INTO _YesterdayBal (_LastMonthBal+COALESCE(SUM(CASE WHEN dk=_DefaultDK THEN jnldamt ELSE -1*jnldamt END), 0)) 
	FROM jnld 
	JOIN jnl ON jnl.jnlno=jnld.jnlno
	WHERE accNo=_accNo AND JnlDate>=fBegOfMonth(_enddate)
	AND jnlDate<=_enddate
	AND (_BranchID IS NULL OR BranchID=_BranchID) 
	AND (_DeptID IS NULL OR DeptID=_DeptID) 
	AND (_PrjID IS NULL OR PrjID=_PrjID);

	RETURN _YesterdayBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spImportData (
	_itemid		VARCHAR,
	_itemdesc	VARCHAR,
	_model		VARCHAR,
	_harga		NUMERIC,
	_date		DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	__itemid	VARCHAR(20);
	__itemdesc	VARCHAR(50);
	__model		VARCHAR(50);
	__harga		NUMERIC(19,4);
	__tgl		DATE;
BEGIN
	SELECT INTO __model model FROM model WHERE model=_model;
	IF NOT FOUND THEN
		_retval.ErrNo:=1;
		INSERT INTO model(model,modeldesc) VALUES(_model, _model);
		_retval.ErrNo:=2;
	END IF;
	
	SELECT INTO __itemid itemid FROM item WHERE itemid=_itemid;
	IF FOUND THEN
		_retval.ErrNo:=3;
		UPDATE item SET model=_model, lcprice=_harga, lcdate=_date WHERE itemid=_itemid AND itemdesc=_itemdesc;
		_retval.ErrNo:=4;
	ELSE
		_retval.ErrNo:=5;
		INSERT INTO item(itemid, itemdesc, model, lcprice, lcdate)
			  VALUES(_itemid, _itemdesc, _model, _harga, _date);
		_retval.ErrNo:=6;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/*-- Function: fgrafikpenjualan(_StartDate DATE, _EndDate DATE, _CustID VARCHAR, _BPGrpID VARCHAR, _AreaID VARCHAR, _ItemID VARCHAR, _RL BOOLEAN)*/
/*-- DROP FUNCTION fgrafikpenjualan(_StartDate DATE, _EndDate DATE, _CustID VARCHAR, _BPGrpID VARCHAR, _AreaID VARCHAR, _ItemID VARCHAR, _RL BOOLEAN);*/


CREATE OR REPLACE FUNCTION fGrafikPenjualan(
	_StartDate	DATE, 
	_EndDate	DATE, 
	_BranchID	VARCHAR, 
	_CustID		VARCHAR, 
	_BPGrpID	VARCHAR, 
	_AreaID		VARCHAR, 
	_ItemID		VARCHAR,
	_SaleType	VARCHAR, 
	_RL		BOOLEAN, 
	_CrcID		VARCHAR, 
	_DeptID		VARCHAR, 
	_PrjID		VARCHAR
) RETURNS SETOF tgrafikpenjualan AS
$BODY$
DECLARE
	_retval		tGrafikPenjualan%ROWTYPE;

	cData		REFCURSOR;
BEGIN
	OPEN cData FOR
		SELECT * FROM fGrafikPenjualan(_StartDate, _EndDate, _BranchID, _CustID, _BPGrpID, _AreaID, _ItemID, _SaleType, _RL, _CrcID, _DeptID, _PrjID, NULL);
	LOOP
		FETCH cData INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fGrafikPenjualan(
	_StartDate	DATE, 
	_EndDate	DATE, 
	_BranchID	VARCHAR, 
	_CustID		VARCHAR, 
	_BPGrpID	VARCHAR, 
	_AreaID		VARCHAR, 
	_ItemID		VARCHAR,
	_SaleType	VARCHAR, 
	_RL		BOOLEAN, 
	_CrcID		VARCHAR, 
	_DeptID		VARCHAR, 
	_PrjID		VARCHAR,
	_IsUsePPN	BOOLEAN
) RETURNS SETOF tgrafikpenjualan AS
$BODY$
DECLARE
	_Omzet		NUMERIC(19,4);
	_PL		NUMERIC(19,4);
	_Tanggal	VARCHAR;
	_Bulan		VARCHAR;
	cGrafik		REFCURSOR;
	cLR		REFCURSOR;
	_retval		TGrafikPenjualan%ROWTYPE;
	_SaleDate	DATE;
BEGIN
	OPEN cGrafik FOR
		SELECT CAST(DATE_PART('day', saledate) AS VARCHAR), SUM(total) 
		FROM (
			SELECT s.saledate, s.saleno, s.total 
			FROM sale s JOIN saled sd ON s.saleno=sd.saleno 
			JOIN bp ON bp.bpid=s.custid
			WHERE s.saledate>=_StartDate AND s.isdraft=false AND s.saledate<=_EndDate AND (_CustID IS NULL OR s.custid=_CustID) 
			AND (_BPGrpID IS NULL OR bp.bpgrpid=_BPGrpID) AND (_AreaID IS NULL OR bp.areaid=_AreaID) 
			AND (_ItemID IS NULL OR sd.itemid=_ItemID) AND (_SaleType IS NULL OR s.saletype=_SaleType)
			AND (s.crcid=_CrcID) AND (_DeptID IS NULL OR sd.deptid=_DeptID) AND (_PrjID IS NULL OR sd.prjid=_PrjID)
			AND (_BranchID IS NULL OR s.branchid=_BranchID)
			AND (_IsUsePPN IS NULL OR fIsUsePPN(s.saleno)=_IsUsePPN)
			GROUP BY s.saledate, s.saleno, s.total
		) a
		GROUP BY a.saledate
		ORDER BY a.saledate;
	LOOP
		FETCH cGrafik INTO _Tanggal, _Omzet;
		EXIT WHEN NOT FOUND;

		_retval.tanggal:=_Tanggal;
		_retval.bulan:=_Bulan;
		_retval.tipe:='omzet';
		_retval.nilai:=_Omzet;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cGrafik;

	IF(_RL=TRUE) THEN
		OPEN cLR FOR
			SELECT CAST(DATE_PART('day', saledate) AS VARCHAR), SUM(pl) FROM (
				SELECT s.saledate, s.saleno, fProfitSale(s.saleno) AS pl 
				FROM sale s JOIN saled sd ON s.saleno=sd.saleno JOIN bp ON bp.bpid=s.custid
				WHERE s.saledate>=_StartDate AND s.saledate<=_EndDate AND (_CustID IS NULL OR s.custid=_CustID) 
				AND (_BPGrpID IS NULL OR bp.bpgrpid=_BPGrpID) AND (_AreaID IS NULL OR bp.areaid=_AreaID) 
				AND (_ItemID IS NULL OR sd.itemid=_ItemID) AND (_SaleType IS NULL OR s.saletype=_SaleType)
				AND (s.crcid=_CrcID) AND (_DeptID IS NULL OR sd.deptid=_DeptID) AND (_PrjID IS NULL OR sd.prjid=_PrjID)
				AND (_BranchID IS NULL OR s.branchid=_BranchID)
				AND (_IsUsePPN IS NULL OR fIsUsePPN(s.saleno)=_IsUsePPN)
				GROUP BY s.saledate, s.saleno, s.total, s.costtot
			) a
			GROUP BY a.saledate
			ORDER BY a.saledate;
		LOOP
			FETCH cLR INTO _Tanggal, _PL;
			EXIT WHEN NOT FOUND;

			_retval.tanggal:=_Tanggal;
			_retval.bulan:=_Bulan;
			_retval.tipe:='laba/rugi';
			_retval.nilai:=_PL;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cLR;
	END IF;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fGrafikBulanan(
	_StartDate	DATE,
	_EndDate	DATE,
	_BranchID	VARCHAR,
	_CustID		VARCHAR,
	_BPGrpID	VARCHAR,
	_AreaID		VARCHAR,
	_ItemID		VARCHAR,
	_SaleType	VARCHAR,
	_RL		BOOLEAN,
	_CrcID		VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS SETOF TGrafikBulanan AS
$BODY$
DECLARE
	_retval		TGrafikBulanan%ROWTYPE;

	cData		REFCURSOR;
BEGIN
	OPEN cData FOR
		SELECT * FROM fGrafikBulanan(_StartDate, _EndDate, _BranchID, _CustID, _BPGrpID, _AreaID, _ItemID, _SaleType, _RL, _CrcID, _DeptID, _PrjID, NULL);
	LOOP
		FETCH cData INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fGrafikBulanan(
	_StartDate	DATE,
	_EndDate	DATE,
	_BranchID	VARCHAR,
	_CustID		VARCHAR,
	_BPGrpID	VARCHAR,
	_AreaID		VARCHAR,
	_ItemID		VARCHAR,
	_SaleType	VARCHAR,
	_RL		BOOLEAN,
	_CrcID		VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR,
	_IsUsePPN	BOOLEAN
) RETURNS SETOF TGrafikBulanan AS
$BODY$
DECLARE
	_retval 	TGrafikBulanan%ROWTYPE;

	cGrafik 	REFCURSOR;
	cRL 		REFCURSOR;
	_Year 		FLOAT;
	_Month 		FLOAT;
	_Omzet 		NUMERIC(19,4);
	_PL 		NUMERIC(19,4);
BEGIN
	OPEN cGrafik FOR
		SELECT CAST(DATE_PART('year', saledate) AS INT), CAST(DATE_PART('month', saledate) AS INT), SUM(total) 
		FROM (
			SELECT s.saleDATE, s.saleno, s.total 
			FROM sale s JOIN saled sd ON s.saleno=sd.saleno JOIN bp ON bp.bpid=s.custid
			WHERE s.saledate>=_StartDate AND s.saledate<=_EndDate AND s.isdraft=false AND (_CustID IS NULL OR s.custid=_CustID) 
			AND (_BPGrpID IS NULL OR bp.bpgrpid=_BPGrpID) AND (_AreaID IS NULL OR bp.areaid=_AreaID) 
			AND (_ItemID IS NULL OR sd.itemid=_ItemID) AND (_SaleType IS NULL OR s.saletype=_SaleType)
			AND (s.crcid=_CrcID) AND (_deptID IS NULL OR sd.deptid=_DeptID) AND (_PrjID IS NULL OR sd.prjid=_PrjID)
			AND (_BranchID IS NULL OR s.branchid=_BranchID)
			AND (_IsUsePPN IS NULL OR fIsUsePPN(s.saleno)=_IsUsePPN)
			GROUP BY s.saledate, s.saleno, s.total
			ORDER BY s.saledate
		) a
		GROUP BY CAST(DATE_PART('year', saledate) AS INT), CAST(DATE_PART('month', saledate) AS INT)
		ORDER BY CAST(DATE_PART('year', saledate) AS INT), CAST(DATE_PART('month', saledate) AS INT);
	LOOP
		FETCH cGrafik INTO _Year,_Month,_Omzet;
		EXIT WHEN NOT FOUND;		

		_retval.tipe:='omzet';
		_retval.nilai:=_Omzet;

		IF (_Month=1) THEN
			_retval.month:='Januari';
		ELSIF (_Month=2) THEN
			_retval.month:='Februari';
		ELSIF (_Month=3) THEN
			_retval.month:='Maret';
		ELSIF (_Month=4) THEN
			_retval.month:='April';
		ELSIF (_Month=5) THEN
			_retval.month:='Mei';
		ELSIF (_Month=6) THEN
			_retval.month:='Juni';
		ELSIF (_Month=7) THEN
			_retval.month:='Juli';
		ELSIF (_Month=8) THEN
			_retval.month:='Agustus';
		ELSIF (_Month=9) THEN
			_retval.month:='September';
		ELSIF (_Month=10) THEN
			_retval.month:='Oktober';
		ELSIF (_Month=11) THEN
			_retval.month:='Nopember';
		ELSIF (_Month=12) THEN
			_retval.month:='Desember';
		END IF;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cGrafik;

	IF (_RL=TRUE) THEN
		OPEN cRL FOR
			SELECT CAST(DATE_PART('month', saledate) AS VARCHAR), SUM(pl) 
			FROM (
				SELECT s.saledate, s.saleno, fProfitSale(s.saleno) AS pl 
				FROM sale s JOIN saled sd ON s.saleno=sd.saleno JOIN bp ON bp.bpid=s.custid
				WHERE s.saledate>=_StartDate AND s.saledate<=_EndDate AND s.isdraft=false AND (_CustID IS NULL OR s.custid=_CustID) 
				AND (_BPGrpID IS NULL OR bp.bpgrpid=_BPGrpID) AND (_AreaID IS NULL OR bp.areaid=_AreaID) 
				AND (_ItemID IS NULL OR sd.itemid=_ItemID) AND (_SaleType IS NULL OR s.saletype=_SaleType)
				AND (s.crcid=_CrcID) AND (_DeptID IS NULL OR sd.deptid=_DeptID) 
				AND (_PrjID IS NULL OR sd.prjid=_PrjID)
				AND (_BranchID IS NULL OR s.branchid=_BranchID)
				AND (_IsUsePPN IS NULL OR fIsUsePPN(s.saleno)=_IsUsePPN)
				GROUP BY s.saleDATE,s.saleno,s.total
			) a
			GROUP BY CAST(DATE_part('month', saledate) AS VARCHAR)
			ORDER BY CAST(DATE_part('month', saledate) AS VARCHAR);
		LOOP
			FETCH cRL INTO _Month,_PL;
			EXIT WHEN NOT FOUND;

			_retval.tipe:='laba/rugi';
			_retval.nilai:=_PL;

			IF (_Month=1) THEN
				_retval.month:='Januari';
			ELSIF (_Month=2) THEN
				_retval.month:='Februari';
			ELSIF (_Month=3) THEN
				_retval.month:='Maret';
			ELSIF (_Month=4) THEN
				_retval.month:='April';
			ELSIF (_Month=5) THEN
				_retval.month:='Mei';
			ELSIF (_Month=6) THEN
				_retval.month:='Juni';
			ELSIF (_Month=7) THEN
				_retval.month:='Juli';
			ELSIF (_Month=8) THEN
				_retval.month:='Agustus';
			ELSIF (_Month=9) THEN
				_retval.month:='September';
			ELSIF (_Month=10) THEN
				_retval.month:='Oktober';
			ELSIF (_Month=11) THEN
				_retval.month:='Nopember';
			ELSIF (_Month=12) THEN
				_retval.month:='Desember';
			END IF;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cRL;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSOPaid (
	_SONo		VARCHAR
) RETURNS boolean AS
$BODY$
DECLARE
	_retval boolean;
	_RcvNo varchar;
BEGIN
	SELECT INTO _RcvNo rcvno FROM spaid WHERE reftype='SO' AND refno=_SONo;
	IF FOUND THEN
		_retval:=true;
	ELSE
		_retval:=false;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPOPaid (
	_PONo	VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval	BOOLEAN;
	_PayNo	VARCHAR;
BEGIN
	SELECT INTO _PayNo payno FROM ppaid WHERE reftype='PO' AND refno=_PONo;
	IF FOUND THEN
		_retval:=true;
	ELSE
		_retval:=false;
	END IF;

	RETURN(_retval);
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDeli_New (
	_DeliNo		VARCHAR
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_DeliDate	DATE;
	_CustID		VARCHAR(15);
	_CustID2	VARCHAR(15);
	_TermType	VARCHAR(1);
	_IsInvoice	BOOLEAN;
	_CrtBy		VARCHAR(10);
	_CrtDate	TIMESTAMP;
	_UpdBy		VARCHAR(10);
	_UpdDate	TIMESTAMP;
	_isSale 	BOOLEAN;
	_DeliDNo	INT2;
	_itemid		VARCHAR(20);
	_WhID		VARCHAR(3);
	_PID		VARCHAR(25);
	_qty		NUMERIC(15,4);
	_qtyx		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_ListPrice	NUMERIC(19,4);
	_DiscExp	VARCHAR(15);
	_DiscAmt	NUMERIC(19,4);
	_TaxID		VARCHAR(5);
	_TaxAmt		NUMERIC(19,4);
	_Qty1		NUMERIC(15,4);
	_Qty1Rcv	NUMERIC(15,4);

	_SONo		VARCHAR(15);
	_SODNo		INT;
	_DeONo		VARCHAR(15);
	_DeliDPID	VARCHAR;

	_StockANo	VARCHAR(15);
	_StockADNo	SMALLINT;
	_StockANote	VARCHAR(255);
	_StockADCost	DECIMAL(19,4);
	_UBDeliAmt	DECIMAL(19,4);
	_MTaxAmt	DECIMAL(19,4);
	_MDiscAmt	DECIMAL(19,4);
	_ExcRate	DECIMAL(19,4);
	_Freight	DECIMAL(19,4);
	_Total		DECIMAL(19,4);
	cDeliD		refcursor;
	cSO		refcursor;
	cDeO		refcursor;
	cDeliD2		refcursor;

	_Count		INT;
	_JnlNo		VARCHAR(15);
	_JnlNo2		VARCHAR(15);
	_JnlNote	VARCHAR;

	_Inv		VARCHAR(10);
	_AccSale	VARCHAR(10);
	_AccInv		VARCHAR(10);
	_AccUBDeliSvc	VARCHAR(10);

	_PerID		VARCHAR(5);
	_BranchID	VARCHAR(10);
   	_DeptID     	VARCHAR(5);
   	_PrjID      	VARCHAR(5);

	_DeliIType	VARCHAR(5);
	_InOut		VARCHAR(1);
	_Round		INT;

	_isDraft	BOOLEAN;
	_LockDate	DATE;

	_Cost		NUMERIC(19,4);
	_CostSubTotal	NUMERIC(19,4);

	_Conv		NUMERIC(19,4);
	_Subtotal	NUMERIC(19,4);
	_CrcID		VARCHAR;
	_FisRate	NUMERIC(19,4);
	_countSO	INT;
	_CntDeo		INT;
	_Unit1		VARCHAR;
	
	_RowCount	INT;

	_BTotal		NUMERIC(19,4); -- base currency total
	_BSubtotal	NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt	NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt	NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt	NUMERIC(19,4);
	_TotalTaxAmt	NUMERIC(19,4);
	_BTotalTaxAmt	NUMERIC(19,4);
	_TotalDiscAmt	NUMERIC(19,4);
	_TotalDisc2Amt	NUMERIC(19,4);
	_DSubtotal	NUMERIC(19,4);
	_DBSubtotal	NUMERIC(19,4);
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;	

	SELECT INTO 
		_isDraft, _PerID, _BranchID, _DeliDate, _CustID, _termtype, _IsInvoice, _CrtBy, _CrtDate, _UpdBy, _UpdDate,_MDiscAmt,_MTaxAmt,_ExcRate,_Freight,_Total,_DeliIType,_Subtotal, _CrcID, _ExcRate, _FisRate
		d.isDraft, d.PerID, d.BranchID, d.DeliDate, d.CustID, s.termtype, d.IsInvoiced, d.CrtBy, d.CrtDate, d.UpdBy, d.UpdDate, s.DiscAmt, s.TaxAmt, s.excrate, s.freight, s.total, d.deliitype, s.subtotal, s.crcid, s.excrate, s.fisrate
	FROM Deli d, DeliD dd, SO s
	WHERE d.DeliNo=dd.DeliNo AND dd.SONo=s.SONo AND d.DeliNo=_DeliNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=5000;
		_retval.ErrMsg:='Pengiriman '||_DeliNo||' tidak ditemukan! (spDeli_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='DELI';
	IF(_LockDate is not null AND _LockDate>_DeliDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Pengiriman Melebihi Batas Tanggal Kunci Transaksi ! (spDeli_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	--  cek itype untuk tentukan in/out ---
	IF (_DeliIType='DELI') THEN
		_InOut:='O';
	ELSIF (_DeliIType='SRET') THEN
		_InOut:='I';	
	END IF;

	IF (_DeliIType='DELI' AND _Subtotal<0) THEN
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Qty Pengiriman tidak boleh Negatif!! (spDeli_New)';			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_DeliIType='SRET' AND _Subtotal>0) THEN
		_retval.ErrNo:=50004;
		_retval.ErrMsg:='Penerimaan Barang Retur Qty harus Negatif! (spDeli_New)';			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

/** ////////////////////// AKUNTANSI /////////////////////////////// **/

	IF (isAccInstalled()) THEN
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:= 'Pengiriman ' || fBPName(_CustID) ||':'||_DeliNo;
		
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'DELI', _DeliDate, 'DELI', _DeliNo,
			_JnlNote, TRUE, TRUE, CAST(0 as NUMERIC(19,4)), _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_Count:=0;


		--- Ayat Unbilled Delivery HANYA untuk item yang stocked=TRUE ---------
		SELECT INTO _RowCount COUNT(*) 
		FROM DeliD d 
		JOIN Item i ON d.ItemID=i.ItemID 
		WHERE DeliNo=_DeliNo AND i.isStock=TRUE;

		IF (_RowCount>0) THEN
			SELECT INTO 
				_UBDeliAmt 
				SUM(CAST(ROUND(fUnitCost(d.ItemID, WhID, PID, (fQtyZtoQty1(d.ItemID, Qty, Unit) - QtyDRet), fUnit1(d.ItemID)), _Round) AS NUMERIC(19,4)))
			FROM DeliD d JOIN Item i ON d.ItemID=i.ItemID 
			WHERE DeliNo=_DeliNo AND i.isStock=TRUE;

			_Count:=_Count+1;
			IF (_DeliIType='SRET') THEN
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBSRET'), 'D', _UBDeliAmt, _DeptID, _PrjID, _UBDeliAmt);
			ELSIF (_DeliIType='DELI') THEN
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBDELI'), 'D', _UBDeliAmt, _DeptID, _PrjID, _UBDeliAmt);	
			END IF;

		END IF;

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		
		--------- Ayat INVENTORY untuk Item yang stocked -------------------
		OPEN cDeliD FOR
			SELECT 
			SUM(CAST(ROUND(fUnitCost(d.ItemID, WhID, PID, (fQtyZtoQty1(d.ItemID, Qty, Unit) - QtyDRet), fUnit1(d.ItemID)), _Round) AS NUMERIC(19,4))),
			fItemAcc(d.ItemID,'INV'), d.DeptID, d.PrjID
			FROM DeliD d 
			JOIN Item i ON d.ItemID=i.ItemID
			WHERE DeliNo=_DeliNo AND i.isStock=TRUE
			GROUP BY fItemAcc(d.ItemID,'INV'), d.DeptID, d.PrjID;
		LOOP
			FETCH cDeliD INTO _StockADCost, _AccInv, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;
			_Count:=_Count+1;

			SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccInv, 'K', _StockADCost, _DeptID, _PrjID, _StockADCost/_Excrate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cDeliD;

		----------- Ayat Unbilled-Service untuk Item yang Non-Stocked --------------

		SELECT INTO _RowCount COUNT(*) FROM DeliD d JOIN Item i ON d.ItemID=i.ItemID WHERE DeliNo=_DeliNo AND i.isStock=FALSE;

		IF (_RowCount>0) THEN

			--- Ayat Unbilled Deli/SRet -------

			SELECT INTO _UBDeliAmt, _BSubtotal 
				SUM(CAST(ROUND((sd.ListPrice-sd.DiscAmt)*(fQtyZtoQty1(d.ItemID, d.Qty, d.Unit) - QtyDRet)/d.Conv, _Round) AS NUMERIC(19,4))), 
				SUM(CAST(ROUND((sd.ListPrice-sd.DiscAmt)*_ExcRate*(fQtyZtoQty1(d.ItemID, d.Qty, d.Unit) - QtyDRet)/d.Conv, _Round) AS NUMERIC(19,4)))
			FROM DeliD d 
			JOIN SOD sd ON sd.SONo=d.SONo AND sd.SODNo=d.SODNo
			JOIN Item i ON d.ItemID=i.ItemID
			WHERE d.DeliNo=_DeliNo AND i.isStock=FALSE;

			IF FOUND THEN
				_Count:=_Count+1;
				IF (_DeliIType='SRET') THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBSRET'), 'D', _BSubtotal, _DeptID, _PrjID, _UBDeliAmt);
				ELSIF (_DeliIType='DELI') THEN
					SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBDELI'), 'D', _BSubtotal, _DeptID, _PrjID, _UBDeliAmt);	
				END IF;
			END IF;

			--- Ayat UB Deli Service -------
			_AccUBDeliSvc:=fAccDefault('UBDELISVC');

			OPEN cDeliD FOR
			SELECT 
				SUM(CAST(ROUND((sd.ListPrice-sd.DiscAmt)*(fQtyZtoQty1(d.ItemID, d.Qty, d.Unit) - QtyDRet)/d.Conv, _Round) AS NUMERIC(19,4))), 
				SUM(CAST(ROUND((sd.ListPrice-sd.DiscAmt)*_ExcRate*(fQtyZtoQty1(d.ItemID, d.Qty, d.Unit) - QtyDRet)/d.Conv, _Round) AS NUMERIC(19,4))), d.DeptID, d.PrjID
			FROM DeliD d 
			JOIN SOD sd ON sd.SONo=d.SONo AND sd.SODNo=d.SODNo
			JOIN Item i ON d.ItemID=i.ItemID
			WHERE d.DeliNo=_DeliNo AND i.isStock=FALSE
			GROUP BY d.DeptID, d.PrjID;
		
			LOOP
				FETCH cDeliD INTO _StockADCost, _BSubtotal, _DeptID, _PrjID;
				EXIT WHEN NOT FOUND;
				_Count:=_Count+1;

				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccUBDeliSvc, 'K', _BSubtotal, _DeptID, _PrjID, _StockADCost);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
			CLOSE cDeliD;
		END IF;


		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;	
		

	END IF;
	
	-- Update stocknya pake qtydeli ---
	OPEN cDeliD FOR
		SELECT 
			d.DeliDNo, d.ItemID, d.WhID, d.PID, fQty1ToQtyZ(d.ItemID,(fQtyZtoQty1(d.ItemID, d.qty, d.unit)-d.QtyDRet),d.unit), d.qtyx, d.Unit, 
			sd.ListPrice, sd.DiscExp, sd.DiscAmt, sd.TaxID, sd.TaxAmt, d.SONo, d.DeONo, i.isSale, d.DeptID, d.PrjID
		FROM DeliD d
		JOIN SOD sd ON sd.SONo=d.SONo AND sd.SODNo=d.SODNo
		LEFT JOIN Item i ON i.itemid=d.ItemID
		WHERE d.DeliNo=_DeliNo AND i.isStock=TRUE;

		_Count:=1;
		LOOP
			FETCH cDeliD INTO
				_DeliDNo, _itemid, _WhID, _PID, _Qty, _QtyX, _Unit,
				_ListPrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _SONo, _DeONo, _isSale, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;
			
			IF (_isSale=FALSE) THEN
				_retval.ErrNo:=50005;
				_retval.ErrMsg:='Item:' || fItemDesc(_ItemID) || '(' || _ItemID || ') tidak di-Set untuk Penjualan (spDeli_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_count=1) THEN
				_StockADNo:=0;
				_StockANote:='Pengiriman #'||_delino;

				_StockANo:=fTransNoGet('STOCKA');
				IF (_StockANo IS NULL) THEN
					_retval.ErrNo:=3;
					_retval.ErrMsg:='ERROR getting StockANo (spDeli_New)';			
					RAISE EXCEPTION '%', _retval.ErrMsg;				
				END IF;
				
				SELECT INTO _spret * FROM spStockA_New(
					_StockANo, _CustID, 'DELI', _DeliDate, 'O', 'DELI', _DeliNo, 
					_StockANote, TRUE, _CrtBy, CAST(_CrtDate AS DATE), _UpdBy, CAST(_UpdDate AS DATE),
					_PerID, _BranchID, _DeptID, _PrjID
				);

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;			
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
															
				_count:=_count+1;
			END IF;
			
			-- read current Cost SubTotal, calc unit cost
			_CostSubTotal:=fUnitCost(_ItemID,_WhID,_PID, _Qty, _Unit);
			_Conv:=fItemUnit_Conv(_ItemID, _Unit);

			IF (_Qty>0) THEN
				_Cost:=_CostSubTotal/_Qty;
			ELSE
				_Cost:=0;
			END IF;

			-- Update Cost & Conversi			
			UPDATE DeliD SET 
				Conv=_Conv, Cost=_Cost
			WHERE DeliNo=_DeliNo AND DeliDNo=_DeliDNo;

			IF NOT FOUND THEN
				_retval.ErrNo:=50056;
				_retval.ErrMsg:='Error UPDATE DeliD:Conv (spDeli_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (_PID IS NULL) THEN
				_PID:='';
			END IF;
			
			_StockADNo:=_StockADNo+1;

			SELECT INTO _spret * FROM spStockAD_New(
				_DeliDate, 'DELI', _StockANo, _StockADNo, _WhID, 
				_ItemID, _PID, _Qty*(-1), _Unit, _QtyX*(-1), _Cost, _DeptID, _PrjID, TRUE, _DeliNo, _CostSubTotal*-1
			);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;			
												
		END LOOP;
	CLOSE cDeliD;


/**** Process DeliD yang Non-Stocked ****/
	OPEN cDeliD FOR
		SELECT 
			d.DeliDNo, d.ItemID, d.WhID, d.PID, fQtyZtoQty1(d.ItemID, d.qty, d.unit)-d.QtyDRet, d.qtyx, d.Unit,
			sd.ListPrice, sd.DiscExp, sd.DiscAmt, sd.TaxID, sd.TaxAmt, d.SONo, d.DeONo, i.isSale, d.DeptID, d.PrjID
		FROM DeliD d
		JOIN SOD sd ON sd.SONo=d.SONo AND sd.SODNo=d.SODNo
		LEFT JOIN Item i ON i.itemid=d.ItemID
		WHERE d.DeliNo=_DeliNo AND i.isStock=FALSE;
		_Count:=1;
	LOOP
		FETCH cDeliD INTO 
			_DeliDNo, _itemid, _WhID, _PID, _qty, _qtyx, _Unit,
			_ListPrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _SONo, _DeONo, _isSale, _DeptID, _PrjID;

		EXIT WHEN NOT FOUND;
		
		_Cost:=fUnitCost(_ItemID,_WhID,_PID, _Unit);
		_Conv:=fItemUnit_Conv(_ItemID, _Unit);

	
		UPDATE DeliD SET 
			Cost=_Cost,
			Conv=_Conv
		WHERE DeliNo=_DeliNo AND DeliDNo=_DeliDNo;

	END LOOP;
	CLOSE cDeliD;	


	--------------- UPDATE SO KLO LANGSUNG ADA SO --------------
	OPEN cDeliD2 FOR
		SELECT 
			ItemID, SONo, SODNo, PID, fQtyZtoQty1(ItemID, Qty, unit) - QtyDRet
		FROM DeliD
		WHERE 
			DeliNo=_DeliNo 
			AND (SONo IS NOT NULL AND SONo<>'');
	LOOP
		FETCH cDeliD2 INTO _ItemID, _SONo, _SODNo, _DeliDPID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Rcv * FROM spSO_Item_Delivered(_SONo, _ItemID, _DeliDPID, _Qty1, _SODNo);
		
		IF (_Qty1Rcv<_Qty1) THEN
			_retval.ErrNo:=50018;
			_retval.ErrMsg:='Item:'||_ItemID||', Qty Pengiriman > QtySO '||_Qty1Rcv||' vs '||_Qty1||' (spDeli_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cDeliD2;			

	--------------- Check & Close SO ------------------
	OPEN cSO FOR
		SELECT SONo FROM DeliD
		WHERE DeliNo=_DeliNo 
		AND (SONo IS NOT NULL AND SONo<>'')
		ORDER BY SONo;
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;
		
		SELECT INTO _CustID2 CustID FROM SO WHERE SONo=_SONo;
		IF (_CustID<>_CustID2) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Customer di Order-Penjualan vs Pengiriman tidak sama (spDeli_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spSO_ChNUp_Status(_SONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cSO;

	---- Set Deo Inactive ----
	OPEN cDeO FOR
		SELECT DISTINCT(DeONo) FROM DeliD 
		WHERE DeliNo=_DeliNo 
		AND (DeliNo IS NOT NULL AND DeliNo<>'')
		ORDER BY DeONo;
	LOOP
		FETCH cDeO INTO _DeONo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _CustID2 CustID FROM DeO WHERE DeONo=_DeONo;
		IF (_CustID<>_CustID2) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Customer DeO dan Deli tidak sama (spDeli_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- change DeO status here 
		UPDATE Deo set Active=FALSE WHERE deono=_DeONo;
	END LOOP;
	CLOSE cDeO;


	------------- Update Deli status ----------
	SELECT INTO _spret * FROM spDeli_ChNUp_Status(_DeliNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPRcv_New(_PRcvNo VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_PRcvDNo 	INT2;
	_VendorID	VARCHAR(15);
	_VendorID2	VARCHAR(15);
	_PRcvDate	DATE;
	_TermType	VARCHAR(1);
	_Duedays	INT2;
	_Total		NUMERIC(19,4);
	_CrtBy		VARCHAR(10);
	_CrtDate	TIMESTAMP;
	_UpdBy		VARCHAR(10);
	_UpdDate	TIMESTAMP;
	_EmpID		VARCHAR(10);
	_IsPurc		BOOLEAN;
	_IsStock	BOOLEAN;
	_PID		VARCHAR(25);
	_Subtotal	NUMERIC(19,4);
	_ItemID		VARCHAR(20);
	_WhID		VARCHAR(3);
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_Qtyx		NUMERIC(15,4);
	_BasePrice	NUMERIC(19,4);
	_ListPrice	NUMERIC(19,4);
	_DiscExp	VARCHAR(15);
	_DiscAmt	NUMERIC(19,4);
	_TaxID		VARCHAR(5);
	_TaxAmt		NUMERIC(19,4);

	_StockANo	VARCHAR(15);
	_StockADNo	SMALLINT;
	_StockANote	VARCHAR(255);
	_StockADCost	DECIMAL(19,4);

	_tmpConv	DECIMAL(15,4);
	_PONo		VARCHAR(15);
	_count		INT;
	_AccNo		VARCHAR(15);
	cPRcvd		REFCURSOR;
	cPrcvD2		REFCURSOR;
	cPO		REFCURSOR;
	_Qty1		DECIMAL(15,4);
	_Qty1Rcv	DECIMAL(15,4);
	_JnlNo		VARCHAR(15);
	_JnlNote	VARCHAR;
	_InvAcc		VARCHAR(10);

	_PerID		VARCHAR(5);
	_BranchID	VARCHAR(10);
   	_DeptID     	VARCHAR(5);
   	_PrjID      	VARCHAR(5);
	_CrcID		VARCHAR(5);
	_ExcRate	DECIMAL(19,4);
	_MDiscAmt	DECIMAL(19,4);
	_MTaxAmt	DECIMAL(19,4);
	_Freight	DECIMAL(19,4);
	
	_Price		DECIMAL(19,4);
	_PrcvIType	VARCHAR(5);
	_InOut		VARCHAR(1);
	_Round		INT;

	_isDraft	BOOLEAN;
	_LockDate	DATE;
	_PODNo		INT;
	_FisRate	NUMERIC(19,4);

	_CustomNote	TEXT;

	_BTotal		NUMERIC(19,4); -- base currency total
	_BSubtotal	NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt	NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt	NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt	NUMERIC(19,4);
	_TotalTaxAmt	NUMERIC(19,4);
	_BTotalTaxAmt	NUMERIC(19,4);
	_TotalDiscAmt	NUMERIC(19,4);
	_TotalDisc2Amt	NUMERIC(19,4);
	_DSubtotal	NUMERIC(19,4);
	_DBSubtotal	NUMERIC(19,4);
BEGIN
	_DeptID:=NULL;
	_PrjID:=NULL;

	SELECT INTO 
		_isDraft, _VendorID, _PRcvDate, _TermType, _Duedays, _Total, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _EmpID, _PerID, _BranchID, _CrcID, _ExcRate, _MDiscAmt, _MTaxAmt, _Freight, _PrcvIType, _FisRate 

		p.isdraft, p.vendorid, p.prcvdate, po.termtype, po.duedays, po.total, p.crtby, p.crtdate, p.updby, p.upddate, p.empid, p.perid, p.branchid, po.crcid, po.excrate, po.discamt, po.taxamt, po.freight, p.prcvitype, po.fisrate
	FROM prcv p JOIN prcvd pd ON pd.prcvno=p.prcvno 
	LEFT JOIN po ON pd.pono=po.pono   
	WHERE p.prcvno=_PRcvNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=12030001;
		_retval.ErrMsg:='Penerimaan Pembelian #'||_PRcvNo||' tidak ditemukan (spPRcv_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');
	
	SELECT INTO _LockDate lockDATE FROM TLock WHERE TransType='PRCV';
	IF(_LockDate is not null AND _LockDate>_PRcvDate) THEN
		_retval.ErrNo:=12030002;
		_retval.ErrMsg='Penerimaan Pembelian Melebihi Batas Tanggal Kunci Transaksi! (spPRcv_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _Subtotal, _BSubtotal 
		SUM(CAST(ROUND(fPortionItemValue(pd.itemid, pd.qty, pd.unit, pd.subtotal, p.qty, p.unit), _Round) AS NUMERIC(19,4))), 
		SUM(CAST(ROUND(fPortionItemValue(pd.itemid, pd.qty, pd.unit, pd.basesubtotal, p.qty, p.unit), _Round) AS NUMERIC(19,4)))  
	FROM prcvd p JOIN pod pd ON p.pono=pd.pono AND p.podno=pd.podno
	WHERE p.prcvno=_PrcvNo;

	IF (_PrcvIType='PRCV' AND _Subtotal<0) THEN
		_retval.ErrNo:=12030003;
		_retval.ErrMsg:='Qty PRCV Tidak Boleh Negatif !!';			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_PrcvIType='PRET' AND _Subtotal>0) THEN
		_retval.ErrNo:=12030004;
		_retval.ErrMsg:='Qty PRET Harus Negatif !!';			
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	
	--- cek itype untuk tentukan in/out saat stocka 
	IF (_PrcvIType='PRCV') THEN
		_InOut:='I';
	ELSIF (_PrcvIType='PRET') THEN
		_InOut:='O';
	END IF;
	
	OPEN cPRcvD FOR
		SELECT d.prcvdno, d.itemid, d.whid, d.qty, d.unit, d.qtyx, pd.baseprice,
			pd.discexp, pd.discamt, pd.taxid, pd.taxamt, i.isstock, i.ispurc, d.pid, d.deptid, d.prjid
		FROM prcvd d JOIN pod pd ON d.pono=pd.pono AND d.podno=pd.podno
		JOIN item i ON d.itemid=i.itemid
		WHERE d.prcvno=_PRcvNo;
		
		_count:=1;
	LOOP
		FETCH cPRcvd INTO _PRcvDNo, _ItemID, _WhID, _Qty, _Unit, _Qtyx, _BasePrice, _DiscExp, _DiscAmt, _TaxID, _TaxAmt, _isStock, _IsPurc, _PID, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		IF (_IsPurc=false) THEN
			_retval.ErrNo:=12030005;
			_retval.ErrMsg:='Item:' || fItemDesc(_ItemID) || '(' || _ItemID || ') tidak di-Set untuk Pembelian (spPRcv_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	
		IF (_isStock=TRUE) THEN
			IF (_count=1) THEN	
				_StockADNo:=0;
				IF (_PrcvIType='PRCV') THEN
					_StockANote:='Penerimaan Pembelian #'|| _PRcvNo;
				ELSIF (_PrcvIType='PRET') THEN
					_StockANote:='Pengiriman Retur Pembelian #'|| _PRcvNo;
				END IF;
				_StockANo:=fTransNoGet('STOCKA');
				IF (_StockANo IS NULL) THEN
					_retval.ErrNo:=12030006;
					_retval.ErrMsg:='ERROR getting StockANo (spPRcv_New)';			
					RAISE EXCEPTION '%', _retval.ErrMsg;				
				END IF;

				SELECT INTO _spret * FROM spStockA_New(
					_StockANo, _VendorID, 'PRCV', _PRcvDate, _InOut, 'PRCV', _PRcvNo, 
					_StockANote, true, _CrtBy, CAST(_CrtDate AS DATE), _UpdBy, 
					CAST(_UpdDate AS DATE), _PerID, _BranchID, _DeptID, _PrjID);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;			
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			
				_count:=_count+1;
			END IF;

			_tmpConv:=fItemUnit_Conv(_ItemID, _Unit);
						
			IF (_tmpConv IS NULL) THEN
				_retval.ErrNo:=12030007;
				_retval.ErrMsg:='Error, Konversi ItemID:' || _ItemID || ', Unit:' || _Unit || '!  (fItemUnit_Conv)(spPRcv_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			UPDATE prcvd SET conv=_tmpConv 
			WHERE prcvno=_PRcvNo AND prcvdno=_PRcvDNo;
			IF NOT FOUND THEN
				_retval.ErrNo:=12030008;
				_retval.ErrMsg:='Error UPDATE PurcD:Conv (spPRrc_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		
			_StockADNo:=_StockADNo+1;
			_StockADCost:=(_BasePrice-COALESCE(_DiscAmt,0))*_ExcRate;
			_PID:=COALESCE(_PID,'');

			SELECT INTO _spret * FROM spStockAD_New(
				_PRcvDate, 'PRCV', _StockANo, _StockADNo, _WhID, 
				_ItemID, _pid, _Qty, _Unit, _QtyX, _StockADCost, _DeptID, _PrjID, true, _PrcvNo
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cPRcvd;

	OPEN cPrcvD2 FOR
		SELECT p.pono, p.podno, p.itemid, p.pid, fQtyZToQty1(p.itemid, p.qty, p.unit), pd.listPrice
		FROM prcvd p JOIN pod pd ON p.pono=pd.pono AND p.podno=pd.podno 
		WHERE p.prcvno=_PRcvNo AND (p.pono IS NOT NULL AND p.pono<>'');
	LOOP
		FETCH cPrcvD2 INTO _PONo, _PODNo, _ItemID, _PID, _Qty1, _Price;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Rcv * FROM spPO_Item_Receive(_PONo, _ItemID, _PID, _Qty1, _PODNo);
		IF (_Qty1Rcv<_Qty1) THEN
			_retval.ErrNo:=12030009;
			_retval.ErrMsg:='Item:'||_ItemID||', Qty Penerimaan > Qty PO (spPRcv_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cPrcvD2;

	OPEN cPO FOR
		SELECT DISTINCT pono FROM prcvd 
		WHERE prcvno=_PrcvNo AND (pono IS NOT NULL AND pono<>'')
		ORDER BY pono;
	LOOP
		FETCH cPO INTO _PONo;
		EXIT WHEN NOT FOUND;
		
		SELECT INTO _VendorID2 vendorid FROM po WHERE pono=_PONo;
		IF (_VendorID<>_VendorID2) THEN
			_retval.ErrNo:=12030010;
			_retval.ErrMsg:='Supplier PO dan Penerimaan tidak sama (spPRcv_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		SELECT INTO _spret * FROM spPO_ChNUp_Status(_PONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cPO;
	

/** ////////////////////// AKUNTANSI /////////////////////////////// */
	IF (isAccInstalled()) THEN
		_JnlNo:=fTransNoGet('JURNAL');
		IF (_PrcvIType='PRCV') THEN
			_JnlNote:= 'Purc.Receive '||fBPName(_VendorID)||':'||_PrcvNo;
		ELSIF (_PrcvIType='PRET') THEN
			_JnlNote:= 'Pengiriman Retur Pembelian '||fBPName(_VendorID)||':'||_PrcvNo;
		END IF;
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'PRCV', _PrcvDate, 'PRCV', _PrcvNo,
			_JnlNote, true, true, CAST(0 AS NUMERIC(19,4)), _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate			
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		_Count:=0;

		_Count:=_Count+1;
		_AccNo:=fAccDefault('UBPRCV');
		_MDiscAmt:=COALESCE(_MDiscAmt, 0);
		_Freight:=COALESCE(_Freight, 0);
		_MTaxAmt:=COALESCE(_MTaxAmt, 0);
		
		SELECT INTO _CustomNote SUM(prcvdnote) FROM prcvd WHERE prcvno=_PRcvNo;
		IF(_Total<0) THEN
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('UBPRET'), 'K', _BSubtotal, _DeptID, _PrjID, _CustomNote, _Subtotal);
		ELSE
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo, 'K', _BSubtotal, _DeptID, _PrjID, _CustomNote, _Subtotal);
		END IF;
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		OPEN cPRcvd FOR
			SELECT fItemAcc(p.itemid,'INV'), p.deptid, p.prjid, 
				SUM(CAST(ROUND(fPortionItemValue(pd.itemid, pd.qty, pd.unit, pd.subtotal, p.qty, p.unit), _Round) AS NUMERIC(19,4))), 
				SUM(CAST(ROUND(fPortionItemValue(pd.itemid, pd.qty, pd.unit, pd.basesubtotal, p.qty, p.unit), _Round) AS NUMERIC(19,4))) 
			FROM prcvd p JOIN pod pd ON p.pono=pd.pono AND p.podno=pd.podno
			WHERE p.prcvno=_PrcvNo
			GROUP BY fItemAcc(p.itemid,'INV'), p.deptid, p.prjid;
		LOOP
			FETCH cPRcvd INTO _InvAcc, _DeptID, _PrjID, _DSubtotal, _DBSubtotal;
			EXIT WHEN NOT FOUND;

			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _InvAcc, 'D', _DBSubtotal, _DeptID, _PrjID, _CustomNote, _DSubtotal);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;

		/*** posting jurnal ***/	
		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;	

	END IF;	

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPRcv_Void(_PrcvNo VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_VendorID	varchar(15);
	_PrcvDate	date;
	_CrtBy		varchar(10);
	_CrtDate	timestamp;
	_UpdBy		varchar(10);
	_UpdDate	timestamp;
	_isInvoiced		Boolean;
	_ItemID		VARCHAR(20);
	_PID		VARCHAR;
	_Qty1		NUMERIC(15,4);
	_Qty1Cancel	NUMERIC(15,4);
	_PONo		VARCHAR(15);
	_JnlNo		VARCHAR(15);

	cPO		refcursor;
	cPrcvD2		refcursor;

	_isDraft	BOOLEAN;
	_LockDate	DATE;
	_PODNo		int;

BEGIN
	SELECT INTO _VendorID, _PrcvDate, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _isInvoiced, _isDraft
		    VendorID, PrcvDate, CrtBy, CrtDate, UpdBy, UpdDate, isInvoiced, isDraft
	FROM PRCV
	WHERE PRcvNo=_PrcvNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Penerimaan Pembelian #'||_PrcvNo||' tidak ditemukan (spPRcv_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM POPrcv WHERE PrcvNo=_PrcvNo;
		DELETE FROM PrcvD WHERE PrcvNo=_PrcvNo;
		DELETE FROM Prcv WHERE PrcvNo=_PrcvNo;
	
		_retval:=(0,'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='PRCV';
	IF(_LockDate is not null AND _LockDate>_PRcvDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penerimaan Pembelian Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	

	IF (_isInvoiced=true) THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Sudah ada Transaksi Pembelian untuk Penerimaan Pembelian #'||_PrcvNo||', Void dulu Pembeliannya (spPRcv_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spStockA_Void_byRef('PRCV', _PrcvNo);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cPrcvD2 FOR
		SELECT PONo, PODNo, ItemID, PID, fQtyZToQty1(ItemID, Qty, Unit)
		FROM PrcvD
		WHERE PrcvNo=_PrcvNo 
		AND (PONo IS NOT NULL AND PONo<>'');
	LOOP
		FETCH cPrcvD2 INTO _PONo, _PODNo, _ItemID, _PID, _Qty1;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Qty1Cancel * FROM spPO_Item_Cancel(_PONo, _ItemID, _PID, _Qty1, _PODNo);
	
	END LOOP;
	CLOSE cPrcvD2;

	OPEN cPO FOR 
		SELECT DISTINCT PONo FROM PRcvD 
		WHERE PRcvNo=_PRcvNo 
		AND (PONo IS NOT NULL AND PONo<>'')
		ORDER BY PONo;
	LOOP
		FETCH cPO INTO _PONo;
		EXIT WHEN NOT FOUND;
	
		SELECT into _spret * from spPO_ChNUp_Status (_PONo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cPO;
	
	IF (isAccInstalled()) THEN
		SELECT into _spret * from spJnl_Void_byRef('PRCV',_PrcvNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM POPrcv WHERE PrcvNo=_PrcvNo;
	DELETE FROM PrcvD WHERE PrcvNo=_PrcvNo;
	DELETE FROM Prcv WHERE PrcvNo=_PrcvNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCashDesc (
	_CashID		VARCHAR
) RETURNS VARCHAR(50) AS 
$BODY$
DECLARE
	_retval		VARCHAR(50);
BEGIN
	SELECT INTO _retval CashDesc FROM Cash WHERE CashID=_CashID;
	RETURN(_retval);
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCrcDesc (
	_CrcID	VARCHAR
) RETURNS VARCHAR(50) AS 
$BODY$
DECLARE
	_retval		VARCHAR(50);
BEGIN
	SELECT INTO _retval Crcname FROM Crc WHERE CrcID=_CrcID;
	RETURN(_retval);
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE VIEW vwStock_Sum AS
	SELECT s.itemid, 
		CAST(SUM(s.qty) AS DECIMAL(19,4))  AS qty, CAST(SUM(s.qtyx) AS DECIMAL(19,4))  AS qtyx, 
		CAST(SUM(s.qtypo) AS DECIMAL(19,4))  AS qtypo, CAST(SUM(s.qtyso) AS DECIMAL(19,4))  AS qtyso, 
		CAST(SUM((CASE WHEN s.qtymin!=0 THEN s.qtymin ELSE i.qtymin END)) AS DECIMAL(19,4)) AS qtymin, 
		CAST(SUM((CASE WHEN s.qtymax!=0 THEN s.qtymax ELSE i.qtymax END)) AS DECIMAL(19,4)) AS qtymax, 
		CAST(SUM((CASE WHEN s.qtyreorder!=0 THEN s.qtyreorder ELSE i.qtyreorder END)) AS DECIMAL(19,4)) AS qtyreorder,
		CAST(SUM(s.qtydefect) AS DECIMAL(19,4))  AS qtydefect, CAST(SUM(s.qty-s.qtyso-s.qtydefect) AS DECIMAL(19,4)) AS qtyavail, 
		CAST((CASE WHEN SUM(s.qty)>0 THEN SUM(s.qty*s.avgcost)/SUM(s.qty) ELSE AVG(s.avgcost) END) AS DECIMAL(19,4)) AS avgcost 
	FROM stock s 
	JOIN item i ON s.itemid=i.itemid 
	WHERE s.itemid<>'0' AND s.itemid<>'-1'
	GROUP BY s.itemid;




CREATE OR REPLACE VIEW vwStockPID_Sum As
	SELECT s.itemid, s.pid, 
		CAST(SUM(s.qty) AS DECIMAL(19,4))  AS qty, CAST(SUM(s.qtyx) AS DECIMAL(19,4))  AS qtyx,
		CAST(SUM(s.qtypo) AS DECIMAL(19,4))  AS qtypo, CAST(SUM(s.qtyso) AS DECIMAL(19,4))  AS qtyso,	
		CAST(SUM(s.qtydefect) AS DECIMAL(19,4))  AS qtydefect, CAST(SUM(s.qty-s.qtyso-s.qtydefect) AS DECIMAL(19,4)) AS qtyavail, 
		CAST((CASE WHEN SUM(s.qty)>0 THEN SUM(s.qty*s.avgcost)/SUM(s.qty) ELSE AVG(s.avgcost) END) AS DECIMAL(19,4)) AS avgcost 
	FROM stock s
	GROUP BY s.itemid, s.pid;




CREATE OR REPLACE VIEW vwStockWH_Sum AS
	SELECT s.itemid, s.whid,
		CAST(SUM(s.qty) AS DECIMAL(19,4))  AS qty, CAST(SUM(s.qtyx) AS DECIMAL(19,4))  AS qtyx, 
		CAST(SUM(s.qtypo) AS DECIMAL(19,4))  AS qtypo, CAST(SUM(s.qtyso) AS DECIMAL(19,4))  AS qtyso, 
		CAST(SUM((CASE WHEN s.qtymin!=0 THEN s.qtymin ELSE i.qtymin END)) AS DECIMAL(19,4)) AS qtymin, 
		CAST(SUM((CASE WHEN s.qtymax!=0 THEN s.qtymax ELSE i.qtymax END)) AS DECIMAL(19,4)) AS qtymax, 
		CAST(SUM((CASE WHEN s.qtyreorder!=0 THEN s.qtyreorder ELSE i.qtyreorder END)) AS DECIMAL(19,4)) AS qtyreorder, 
		CAST(SUM(s.qtydefect) AS DECIMAL(19,4))  AS qtydefect, CAST(SUM(s.qty-s.qtyso-s.qtydefect) AS DECIMAL(19,4))  AS qtyavail, 
		CAST((CASE WHEN SUM(s.qty)>0 THEN SUM(s.qty*s.avgcost)/SUM(s.qty) ELSE AVG(s.avgcost) END) AS DECIMAL(19,4)) AS avgcost
	FROM stock s 
	JOIN item i ON s.itemid=i.itemid 
	GROUP BY s.itemid, s.whid;




CREATE OR REPLACE VIEW vwStockBrief AS
	SELECT s.itemid, s.whid,
		SUM(s.qty) AS qty, SUM(s.qtyx) AS qtyx,
		(SELECT * FROM fQty1Desc(s.itemid, SUM(s.qty))) AS qtystr,
		SUM(s.qtypo) AS qtypo, SUM(s.qtyxpo) AS qtyxpo,
		SUM(s.qtyso) AS qtyso, SUM(s.qtyxso) As qtyxso, 
		SUM(s.qtydefect) AS qtydefect, SUM(s.qtyxdefect) AS qtyxdefect, 
		SUM(s.qty-s.qtyso-s.qtydefect) AS qtyavail, 
		SUM(s.qtyx-s.qtyxso-s.qtyxdefect) AS qtyxavail, 
		CASE WHEN SUM(s.qty)<>0 THEN SUM(s.qty*s.avgcost)/SUM(s.qty) ELSE 0 END AS avgcost,
		SUM(s.qty*s.avgcost) AS totcost
	FROM stock s 
	JOIN item i ON s.itemid=i.itemid
	GROUP BY s.itemid, s.whid;




CREATE OR REPLACE VIEW vwStock AS 
	SELECT 
		s.itemid, s.pid, s.whid, s.qty, s.qtyx, s.qtypo, s.qtyso, s.qtydefect, 
		s.qty - s.qtyso - s.qtydefect AS qtyavail, s.avgcost, (CASE WHEN s.qtymin!=0 THEN s.qtymin ELSE i.qtymin END) AS qtymin, 
		(CASE WHEN s.qtymax!=0 THEN s.qtymax ELSE i.qtymax END) AS qtyMax, 
		(CASE WHEN s.qtyreorder!=0 THEN s.qtyreorder ELSE i.qtyreorder END) AS qtyreorder,
		s.lopdate, s.lopusrid, now() - s.lopdate AS lopdays, lopempid, s.location
	FROM stock s 
	JOIN item i ON s.itemid=i.itemid;




CREATE OR REPLACE VIEW vwStock_Rpt AS
	SELECT 
		s.itemid, i.itemdesc, i.unitdesc, i.itgrpid,
		bp.bpName AS vendorname, 
		i.brandid, i.modelid, i.grade, i.vendorid,
		s.qty, s.qtyX, fqty1desc(s.itemid, s.qty) AS qtydesc,
		s.qtypo, s.qtyxpo, fQty1Desc(s.itemid, s.qtypo) AS qtypodesc,
		s.qtyso, s.qtyxso, fQty1Desc(s.itemid, s.qtyso) AS qtysodesc,
		(s.qty-s.qtyso-s.qtydefect) AS qtyAvail, (s.qtyx-s.qtyxso-s.qtyxdefect) AS qtyxavail, 
		i.active
	FROM stock s 
	JOIN item i ON s.itemid=i.itemid 
	LEFT JOIN bp ON i.vendorid=bp.bpid;




CREATE OR REPLACE VIEW vwBoMList AS
	SELECT b.bomid, b.bomtype, b.itemid, i.itemdesc, b.qty, b.unit, b.bomname
	FROM bom b 
	JOIN item i ON b.itemid=i.itemid;




CREATE OR REPLACE VIEW vwCanvasList AS
	SELECT canvasid, canvasname, whid, cashid 
	FROM canvas;




CREATE OR REPLACE VIEW vwStock_SumAll AS
	SELECT s.itemid, 
		SUM(s.qty) AS qty, SUM(s.qtyx) AS qtyx, 
		SUM(s.qtypo) AS qtypo, SUM(s.qtyxpo) AS qtyxpo,
		SUM(s.qtyso) AS qtyso, SUM(s.qtyxso) AS qtyxso,
		SUM(s.qtydefect) AS qtydefect, SUM(s.qtyxdefect) AS qtyxdefect,
		SUM(s.qty-s.qtyso-s.qtydefect) AS qtyavail, 
		AVG(s.avgcost) AS avgcost
	FROM stock s
	GROUP BY s.itemid;




CREATE OR REPLACE VIEW vwStock_Sum AS
	SELECT s.itemid, 
		CAST(SUM(s.qty) AS DECIMAL(19,4))  AS qty, CAST(SUM(s.qtyx) AS DECIMAL(19,4))  AS qtyx, 
		CAST(SUM(s.qtypo) AS DECIMAL(19,4))  AS qtypo, CAST(SUM(s.qtyso) AS DECIMAL(19,4))  AS qtyso, 
		CAST(SUM((CASE WHEN s.qtymin!=0 THEN s.qtymin ELSE i.qtymin END)) AS DECIMAL(19,4)) AS qtymin, 
		CAST(SUM((CASE WHEN s.qtymax!=0 THEN s.qtymax ELSE i.qtymax END)) AS DECIMAL(19,4)) AS qtymax, 
		CAST(SUM((CASE WHEN s.qtyreorder!=0 THEN s.qtyreorder ELSE i.qtyreorder END)) AS DECIMAL(19,4)) AS qtyreorder,
		CAST(SUM(s.qtydefect) AS DECIMAL(19,4))  AS qtydefect, CAST(SUM(s.qty-s.qtyso-s.qtydefect) AS DECIMAL(19,4)) AS qtyavail, 
		CAST((CASE WHEN SUM(s.qty)>0 THEN SUM(s.qty*s.avgcost)/SUM(s.qty) ELSE AVG(s.avgcost) END) AS DECIMAL(19,4)) AS avgcost 
	FROM stock s 
	JOIN item i ON s.itemid=i.itemid 
	WHERE s.itemid<>'0' AND s.itemid<>'-1'
	GROUP BY s.itemid;




CREATE OR REPLACE VIEW vwCashAList AS
	SELECT s.cashano, s.cashid, h.cashdesc, s.bpid, c.bpname, s.cashadate,
		s.cashatype, s.inout, s.reftype, s.refno, s.amount, s.cashanote
	FROM cashA s
	JOIN cash h ON s.cashid=h.cashid
	LEFT JOIN bp c ON s.bpid=c.bpid;




CREATE OR REPLACE VIEW vwStock_Sum_Rpt As
	SELECT 
		s.*, i.itemdesc, i.unitdesc,
		bp.bpname AS vendorname, 
		i.brandid, i.modelid, i.grade, i.vendorid,
		i.qtymin, i.qtymax, i.qtyreorder,	/** Stock-Control (Gabungan) **/
		i.active
	FROM vwStockPID_Sum s
	JOIN item i ON s.itemid=i.itemid 
	LEFT JOIN bp ON i.vendorid=bp.bpid;




CREATE OR REPLACE VIEW vwCashList AS
	SELECT cashid, cashbank, cashtype, cashdesc 
	FROM cash;




CREATE OR REPLACE VIEW vwCash AS
	SELECT cashid, cashbank, cashtype, cashdesc, cashbal, outbal,
		empid, lopusrid, lopdate,
		(CURRENT_DATE-lopdate) AS lopdays
	FROM cash;




CREATE OR REPLACE VIEW vwStockTotalItem As
	SELECT itemid, 
		SUM(qty) AS qty, SUM(qtyx) AS qtyx, 
		SUM(qtyso) AS qtyso, SUM(qtyxso) AS qtyxso, 
		SUM(qtypo) AS qtypo, SUM(qtyxpo) AS qtyxpo, 
		SUM(qtydefect) AS qtydefect, SUM(qtyxdefect) AS qtyxdefect,
		SUM(qty-qtyso-qtydefect) AS qtyavail,
		SUM(qtyx-qtyxso-qtyxdefect) AS qtyxavail,
		CASE WHEN (SUM(qty)>0) THEN SUM(avgcost*qty)/SUM(qty) ELSE 0 END AS avgcost
	FROM stock s
	GROUP BY s.itemid;




CREATE OR REPLACE VIEW vwChk_Item_LastPurc AS
	SELECT itemid, MAX(h.purcno) AS purcno
	FROM purcd d 
	JOIN purc h ON d.purcno=h.purcno
	GROUP BY d.itemid;




CREATE OR REPLACE VIEW vwChk_Item_LastPurcD AS
	SELECT i.itemid,
		d.listprice,
		d.discexp,
		d.discamt,
		d.taxid,
		d.taxamt,
		d.baseprice-COALESCE(d.discamt, 0)+COALESCE(d.taxamt, 0) AS netto,  
		d.unit,
		p.purcdate,
		p.vendorid
	FROM item i 
	JOIN vwChk_Item_LastPurc lp ON i.itemid=lp.itemid
	JOIN purc p ON p.purcno=lp.purcno
	JOIN purcd d ON d.purcno=lp.purcno AND i.itemid=d.itemid;




CREATE OR REPLACE VIEW vwPRetList AS
	SELECT m.pretno, m.vendorid, bp.bpname AS vendorname, m.pretdate, m.prettype,
	m.total, m.iscontra, m.pretstatus 
	FROM pret m 
	JOIN bp ON m.vendorid=bp.bpid;




CREATE OR REPLACE VIEW vwStTrList AS
	SELECT s.sttrno, s.sttrdate, 
		s.whid1, w1.whname AS whname1,
		s.whid2, w2.whname AS whname2,
		s.canvasid, s.sttrnote
	FROM sttr s
	JOIN wh w1 ON w1.whid=s.whid1
	JOIN wh w2 ON w2.whid=s.whid2
	LEFT JOIN canvas c ON s.canvasid=c.canvasid;


/*

CREATE OR REPLACE VIEW vwStTrList2 AS
	SELECT 	s.StTrNo, s.StTrDate, s.StTrType, s.CanvasID, 
		s.WhID1, w1.WhName AS WhDesc1, 
		s.WhID2, w2.WhName AS WhDesc2,
		s.StTrNote
	FROM StTr s
	JOIN Wh w1 ON w1.WhID=S.WhID1
	JOIN Wh w2 ON w2.WhID=S.WhID2;

*/





CREATE OR REPLACE VIEW vwCredBalance AS
	SELECT bp.bpid, bp.bpname, ac.crcid, ac.apbalance
	FROM bp 
	LEFT JOIN bpacc ac ON bp.bpid=ac.bpid;




CREATE OR REPLACE VIEW vwCredStatement AS
	SELECT vendorid, bp.bpname AS vendorname, purcno, purcdate, purctype,
		docno, duedays, (purcdate+duedays) AS duedate, 
		discdays, earlydisc, latecharge,
		total, paidamt,
		(total-paidamt) AS dueamt, paystatus
	FROM purc p
	JOIN bp ON p.vendorid=bp.bpid
	WHERE paystatus<>'F';




CREATE OR REPLACE VIEW vwCredOverDue AS
	SELECT vendorid, vendorname, purcno, docno,
		purctype, purcdate, duedays, duedate,
		CAST(CURRENT_DATE-duedate AS INTEGER)-1 AS overdue,
		total, paidamt, dueamt, paystatus
	FROM vwCredStatement
	WHERE duedate<=CURRENT_DATE;




CREATE OR REPLACE VIEW vwqtydesc AS 
	SELECT s.whid, s.pid, i.itemdesc, i.itemid, ( 
		SELECT fqty1desc.fqty1desc
		FROM fqty1desc(i.itemid, s.qty) fqty1desc(fqty1desc)) AS stock
	FROM item i
	LEFT JOIN stock s ON i.itemid = s.itemid;




CREATE OR REPLACE VIEW vwQtyAll AS 
	SELECT ( SELECT * FROM fqty1desc(i.itemid, SUM(qty))) AS stock, 
		i.itemid,
		SUM(s.qtypo) AS po, SUM(s.qtyso) AS so, SUM(s.qty) AS hiddenqty 
	FROM item i 
	LEFT JOIN stock s ON i.itemid=s.itemid 
	GROUP BY i.itemid;




CREATE OR REPLACE VIEW vwSRepList AS
	SELECT srepid, srepname, sreptype
	FROM srep;




CREATE OR REPLACE VIEW vwStockAList AS
	SELECT s.stockano, s.bpid, bp.bpname,
		s.stockadate, s.stockatype, s.inout,
		s.reftype, s.refno, s.stockanote
	FROM stocka s
	LEFT JOIN bp ON s.bpid=bp.bpid;




CREATE OR REPLACE VIEW vwSOList AS
	SELECT sono, sodate, soexpdate, custid, bp.bpName AS custname,
		total, s.active, sostatus
	FROM so s
	JOIN bp ON s.custid=bp.bpid;




CREATE OR REPLACE VIEW vwSales AS
	SELECT (SELECT * FROM fNoTime(saledate)) AS saledateonly, saledate, saleno, saletype, 
		s.possesno, s.posid, p.shift, s.canvasid, s.srepid, s.empid, 
		custid, refrid, termtype, duedays, (saledate+duedays) AS duedate, discdays, earlydisc, latecharge, 
		costtot, subtotal, discexp, discamt, taxamt, freight, others, total,
		(subtotal-discamt+taxamt-costtot) AS profit,
		paidamt, (total-paidamt) AS amtdue, payStatus, s._xt
	FROM sale s
	JOIN bp ON s.custid=bp.bpid 
	LEFT JOIN bp bp2 ON s.refrid=bp2.bpid 
	LEFT JOIN posses p ON s.possesno=p.possesno;




CREATE OR REPLACE VIEW vwSales_Note AS
	SELECT (SELECT * FROM fNoTime(saledate)) AS saledateonly, 
		saledate, saleno, saletype, s.possesno, s.posid, 
		p.shift, s.canvasid, s.srepid, s.empid, custid, refrid, 
		termtype, duedays, (saledate+duedays) AS duedate, discdays, earlydisc, latecharge, 
		costtot, 
		subtotal, discexp, discamt, taxamt, freight, others, total, 
		(subtotal-discamt+taxamt-costtot) AS profit, 
		paidamt, (total-paidamt) AS amtdue, paystatus, s._xt, 
		s.SaleNote /** added SALENOTE, difference with vwSales **/
	FROM sale s
	JOIN bp ON s.custid=bp.bpid 
	LEFT JOIN bp bp2 ON s.refrid=bp2.bpid 
	LEFT JOIN posses p ON s.possesno=p.possesno;




CREATE OR REPLACE VIEW vwSales_Hour_Avg AS
	SELECT DATE_PART('hh', saledate) AS sale_hour,
		/*--AVG(NettAmt) AS Sale_Avg, */
		AVG(total-discamt+taxamt-costtot) AS profit_avg
	FROM sale 
	GROUP BY DATE_PART('hh', saledate);




CREATE OR REPLACE VIEW vwPurcDetails AS
	SELECT purcno, purcdno, d.itemid, 
		COALESCE(d.itemdesc, i.itemdesc) AS itemdesc,
		d.pid, d.qty, d.unit, d.qtyx, d.listprice, 
		d.discexp, d.discamt, d.taxid, d.taxamt,
		(d.baseprice-d.discamt) AS unitprice,
		(d.baseprice-d.discamt+taxamt) AS unitpricetax,
		(qty*(d.baseprice-d.discamt)) AS subtotal,
		(qty*(d.baseprice-d.discamt+taxamt)) AS subtotaltax
	FROM purcd d
	JOIN item i ON d.itemid=i.itemid;




CREATE OR REPLACE VIEW vwPurcList AS
	SELECT p.purcno, p.purcdate, p.docno, p.vendorid, bp.bpname AS vendorname, 
		p.termtype, p.duedays, p.purctype, p.total,
		p.paidamt, (p.total-p.paidamt) AS amtdue, p.paystatus
	FROM purc p
	JOIN bp ON p.vendorid=bp.bpid;




CREATE OR REPLACE VIEW vwPurc AS
	SELECT purcdate, purcno, purctype, docno, vendorid,
		bp.bpname AS vendorname, termtype, duedays,
		(purcdate+duedays) AS duedate, subtotal, discexp,
		discamt, taxamt, freight, others, total,
		paidamt, (total-paidamt) AS dueamt,
		purcnote, paystatus
	FROM purc p
	JOIN bp ON p.vendorid=bp.bpid;




CREATE OR REPLACE VIEW vwRcvList AS
	SELECT r.rcvno, r.bpid, bp.bpname AS custname, 
		r.rcvtype, r.rcvdate, r.totspaid, r.totrcvdisc, 
		r.totrcvamt, r.overrcv, r.iscontra, r.rcvstatus
	FROM rcv r 
	JOIN bp ON r.bpid=bp.bpid;




CREATE OR REPLACE VIEW vwRcvSPaid AS
	SELECT r.rcvno, r.rcvdate, r.bpid, bp.areaid, s.saleno,
		s.saledate, s.duedays, (s.saledate+s.duedays) AS duedate,
		s.saletype, s.refrid, s.canvasid, s.srepid, s.total,
		(s.total-(s.paidamt-p.spaidamt)) AS amtdue1,
		p.spaidamt, (s.total-s.paidamt) AS amtdue2
	FROM spaid p
	JOIN rcv r ON p.rcvno=r.rcvno
	JOIN sale s ON p.reftype='SALE' AND p.refno=s.saleno
	JOIN bp ON r.bpid=bp.bpid;




CREATE OR REPLACE VIEW vwSales_Daily AS
	SELECT (SELECT * FROM fNoTime(saledate)) AS saledate, 
/*
		SUM(CASE WHEN DueDays=0 THEN Total ELSE 0 END) AS Sale_Cash,
		SUM(CASE WHEN DueDays>0 THEN Total ELSE 0 END) AS Sale_Credit,
*/
		SUM(CASE WHEN termtype='C' THEN total ELSE 0 END) AS sale_cash,
		SUM(CASE WHEN termtype='R' THEN total ELSE 0 END) AS sale_credit,
		SUM(total) AS sale_total,
		SUM(subtotal-discamt+taxamt-costtot) AS profit
	FROM sale
	GROUP BY (SELECT * FROM fNoTime(saledate));




CREATE OR REPLACE VIEW vwSaleList AS
	SELECT s.saledate, s.saleno, s.posid, s.possesno, p.usrid,
		s.custid, bp.bpname AS custname, s.saletype, s.srepid, 
		s.termtype, s.duedays, s.total, s.paidamt,
		(s.total-s.paidamt) AS amtdue, s.paystatus
	FROM sale s 
	JOIN bp ON s.custid=bp.bpid
	LEFT JOIN posses p ON s.possesno=p.possesno;




CREATE OR REPLACE VIEW vwSaleDetails AS
	SELECT d.saleno, d.saledno, d.itemid, d.whid, 
		COALESCE(d.itemdesc, i.itemdesc) AS itemdesc,
		d.pid, d.qty, d.unit, d.qtyx, d.listprice, d.discexp, d.taxid,
		(d.baseprice-d.discamt+d.taxamt) AS nettprice, d.cost,
		(d.qty*(d.baseprice-d.discamt+d.taxamt)) AS subtotal,
		(d.qty*(d.baseprice-d.discamt+d.taxamt-d.cost)) AS profit,
		d.isbonus, d.ischprice
	FROM saled d
	JOIN item i ON d.itemid=i.itemid;




CREATE OR REPLACE VIEW vwunitdesc AS 
	SELECT item.itemid, (
		SELECT funitdesc.funitdesc
		FROM funitdesc(item.itemid) funitdesc(funitdesc)) AS unit
	FROM item;




CREATE OR REPLACE VIEW vwCsTrList AS
	SELECT t.cstrno, t.cashid1, c1.cashdesc AS cashdesc1, t.cashid2, 
		c2.cashdesc AS cashdesc2, t.cstrdate, t.cstramt
	FROM cstr t
	JOIN cash c1 ON c1.cashid=t.cashid1
	JOIN cash c2 ON c2.cashid=t.cashid2;




CREATE OR REPLACE VIEW vwDebtStatement AS
	SELECT custid, bp.bpname AS custname, saleno, saletype,
		saledate, duedays, (saledate+duedays) AS duedate,
		discdays, earlydisc, latecharge,
		total, paidamt, (total-paidamt) AS dueamt, paystatus
	FROM sale p
	JOIN bp ON p.custid=bp.bpid
	WHERE paystatus<>'F';




CREATE OR REPLACE VIEW vwDebtBalance AS
	SELECT b.bpid, b.bpname, a.crcid, a.arbalance
	FROM bp b 
	LEFT JOIN bpacc a ON b.bpid=a.bpid;




CREATE OR REPLACE VIEW vwPriceListAll AS
	SELECT i.itemid, i.itemdesc, p.prclvlid, l.prclvldesc,
		p.price1, p.price2, p.price3, p.pricedate
	FROM item i 
	JOIN prc p ON i.itemid=p.itemid
	JOIN prclvl l ON p.prclvlid=l.prclvlid;




CREATE OR REPLACE VIEW vwPoSLog AS
	SELECT possesno, l.posid, shift, usrid, 
		p.cashid, starttime, startbal,
		totin, totout, endtime,
		endbal, iscollect
	FROM posses l
	JOIN pos p ON l.posid=p.posid
	WHERE l.starttime IS NOT NULL AND l.endtime IS NOT NULL;




CREATE OR REPLACE VIEW vwPrc_Tab AS
	SELECT itemid, 
		CAST(SUM(CASE prclvlid WHEN '0' THEN price1 ELSE 0 END) AS NUMERIC(19,4)) AS harga1,
		CAST(SUM(CASE prclvlid WHEN '1' THEN price1 ELSE 0 END) AS NUMERIC(19,4)) AS harga2,
		CAST(SUM(CASE prclvlid WHEN '2' THEN price1 ELSE 0 END) AS NUMERIC(19,4)) AS harga3
	FROM prc
	GROUP BY itemid;




CREATE OR REPLACE VIEW vwPoSActive AS
	SELECT possesno, l.posid, shift, usrid, p.cashid,
		starttime, startbal,
		(SELECT COALESCE(SUM(total), 0)
			FROM sale
			WHERE possesno=l.possesno AND termtype='C' /*--AND PoSCash>0*/
			) AS totin,
		CAST(0 AS DECIMAL(19,4)) AS totout, endtime,
		(SELECT cashbal FROM cash WHERE cashid=p.cashid) AS endbal,
		iscollect
	FROM posses l
	JOIN pos p ON l.posid=p.posid
	WHERE l.starttime IS NOT NULL AND l.endtime IS NULL;




CREATE OR REPLACE VIEW vwPoSInActive AS
	SELECT CAST(NULL AS VARCHAR(8)) AS possesno, p.posid, 
		CAST(NULL AS VARCHAR(2)) AS shift, 
		CAST(NULL AS VARCHAR(10)) AS usrid, p.cashid,
		CAST(NULL AS DATE) AS starttime, c.cashbal AS startbal,
		CAST(0 AS DECIMAL(19,4)) AS totin,
		CAST(0 AS DECIMAL(19,4)) AS totout,
		CAST(NULL AS DATE) AS endtime, c.cashbal AS endbal,
		CAST(0 AS BIT) AS iscollect
	FROM pos p
	JOIN cash c ON p.cashid=c.cashid
	WHERE posid NOT IN 
		(SELECT posid FROM posses WHERE starttime IS NOT NULL AND endtime IS NULL);




CREATE OR REPLACE VIEW vwDebtCollect AS
	SELECT saleno, saledate, saletype, duedays,
		(saledate+duedays) AS duedate, custID, srepid,
		canvasid, total, paidamt, (total-paidamt) AS dueamt
	FROM sale
	WHERE termtype='R' AND paystatus<>'F';




CREATE OR REPLACE VIEW vwDebtOverDue AS
	SELECT custid, custname, saleno, 
	saletype, saledate, duedays, duedate, CAST(CURRENT_DATE-duedate AS INTEGER)-1 AS overdue,
	total, paidamt, dueamt, paystatus
	FROM vwDebtStatement
	WHERE duedate<=CURRENT_DATE;




CREATE OR REPLACE VIEW vwSRetList AS
	SELECT d.sretno, d.sretdate, d.custid, bp.bpname AS custname, 
		d.srettype, d.total, d.iscontra, d.sretstatus 
	FROM sret d 
	JOIN bp ON d.custid=bp.bpid;




CREATE OR REPLACE VIEW vwPayList AS
	SELECT payno, paydate, p.bpid, bp.bpname AS bpname, paytype,
		overpay, (overpay>0) AS isover, iscontra, totppaid,
		totpaydisc, totpayamt, paystatus
	FROM pay p 
	JOIN bp ON p.bpid=bp.bpid;




CREATE OR REPLACE VIEW vwPOList AS
	SELECT pono, podate, poexpdate, vendorid, bp.bpname AS vendorname,
		total, p.active, postatus
	FROM po p
	JOIN bp ON p.vendorid=bp.bpid;




CREATE OR REPLACE VIEW vwModelList AS
	SELECT modelid, modelname 
	FROM model;




CREATE OR REPLACE VIEW vwMfgList AS
	SELECT m.mfgno, m.mfgdate, m.mfgitype, m.itemid, i.itemdesc, m.mfgnote
	FROM mfg m 
	JOIN item i ON m.itemid=i.itemid;




CREATE OR REPLACE VIEW vwDNSumm AS
	SELECT sretno, sretdate, srettype, custid, canvasid, 
		cashid, total, iscontra, sretstatus 
	FROM sret;




CREATE OR REPLACE VIEW vwEmpList AS
	SELECT empid, empname, emptype, emptitle 
	FROM emp;




CREATE OR REPLACE VIEW vwItemList AS
	SELECT i.itemid, barcode, itemdesc, itemtype, itgrpid, 
		brandid, modelid, grade, partno, 
		unit1, unit2, conv2, unit3, conv3, 
		unitx, convx, unitdesc,
		purcunit, stockunit, saleunit,
		ispurc, isstock, ismfg, issale, usepid, useqtyx, useed, active,
		i.purctaxiD, i.saletaxid
	FROM item i;



CREATE OR REPLACE FUNCTION isAccInstalled()
  RETURNS boolean AS
$BODY$
DECLARE
	_retval		boolean;

	
BEGIN
	_retval=false;
	SELECT INTO _retval isinstalled FROM modul WHERE modulid='ACC';
	RETURN _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
CREATE OR REPLACE FUNCTION spJnl_Insert(
	__JnlNo			VARCHAR(15),
	_JnlType		VARCHAR(5),
	_JnlDate		DATE,
	_RefType		VARCHAR(5),
	_RefNo			VARCHAR(15),
	_JnlNote		TEXT,
	_IsAutoGen		BOOLEAN,
	_IsPosted		BOOLEAN,
	_Amount			NUMERIC(19,4),
	_CrtBy			VARCHAR(10),
	_CrtDate		TIMESTAMP,
	_UpdBy			VARCHAR(10),
	_UpdDate		TIMESTAMP,
	_PerID			VARCHAR(5),
	_BranchID		VARCHAR(10),
	_DeptID			VARCHAR(5),
	_PrjID			VARCHAR(5),
	_CrcID			VARCHAR,
	_ExcRate		NUMERIC(19,4),
	_FisRate		NUMERIC(19,4)
)
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_JnlNo		varchar(15);
BEGIN

		_JnlNo:=__JnlNo;
		IF (_JnlNo IS NULL) THEN
			_JnlNo:=fTransNoGet('JURNAL');	
			IF (_JnlNo IS NULL) THEN
				_retval.ErrNo:=50002;
				_retval.ErrMsg:='ERROR generate Jurnal.No (spJurnal_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;		
		END IF;

		INSERT INTO Jnl (
			JnlNo, JnlType, JnlDate, 
			RefType, RefNo, JnlNote,IsAutoGen,jnlAmt,IsPosted,
			CrtBy, CrtDate, UpdBy, UpdDate,
			PerID, BranchID, CrcID, ExcRate, FisRate
			/*--, DeptID, PrjID*/			
			)
		VALUES (
			_JnlNo,  _JnlType, _JnlDate,
			_RefType, _RefNo, _JnlNote, _IsAutoGen,_Amount,_IsPosted,
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _CrcID, _ExcRate, _FisRate
			/*--, _DeptID, _PrjID*/			
			);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

/*Parameter nya ditambahi _xt*/
CREATE OR REPLACE FUNCTION spJnl_Insert(
	__JnlNo			VARCHAR(15),
	_JnlType		VARCHAR(5),
	_JnlDate		DATE,
	_RefType		VARCHAR(5),
	_RefNo			VARCHAR(15),
	_JnlNote		TEXT,
	_IsAutoGen		BOOLEAN,
	_IsPosted		BOOLEAN,
	_Amount			NUMERIC(19,4),
	_CrtBy			VARCHAR(10),
	_CrtDate		TIMESTAMP,
	_UpdBy			VARCHAR(10),
	_UpdDate		TIMESTAMP,
	_PerID			VARCHAR(5),
	_BranchID		VARCHAR(10),
	_DeptID			VARCHAR(5),
	_PrjID			VARCHAR(5),
	__xt			BOOLEAN,
	_CrcID			VARCHAR,
	_ExcRate		NUMERIC(19,4),
	_FisRate		NUMERIC(19,4)
)
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_JnlNo		varchar(15);
BEGIN

		_JnlNo:=__JnlNo;
		IF (_JnlNo IS NULL) THEN
			_JnlNo:=fTransNoGet('JURNAL');	
			IF (_JnlNo IS NULL) THEN
				_retval.ErrNo:=50002;
				_retval.ErrMsg:='ERROR generate Jurnal.No (spJurnal_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;		
		END IF;

		INSERT INTO Jnl (
			JnlNo, JnlType, JnlDate, 
			RefType, RefNo, JnlNote,IsAutoGen,jnlAmt,IsPosted,
			CrtBy, CrtDate, UpdBy, UpdDate,
			PerID, BranchID, _xt, CrcID, ExcRate, FisRate
			/*--, DeptID, PrjID*/			
			)
		VALUES (
			_JnlNo,  _JnlType, _JnlDate,
			_RefType, _RefNo, _JnlNote, _IsAutoGen,_Amount,_IsPosted,
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, __xt, _CrcID, _ExcRate, _FisRate
			/*--, _DeptID, _PrjID*/			
			);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;





/** complete version, 9 parameter **/
CREATE OR REPLACE FUNCTION spJnlD_Insert ( 
	__JnlNo VARCHAR, 
	__JnlDNo INT, 
	__AccNo VARCHAR, 
	__DK VARCHAR, 
	__Amount DECIMAL(19,4), 
	__DeptID VARCHAR, 
	__PrjID VARCHAR,
	__JnlDNote TEXT,
	__AmountForex DECIMAL(19,4)
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_DK		VARCHAR;
	_Amount		DECIMAL(19,4);
	_AmountForex	DECIMAL(19,4);
	_found		BOOLEAN;
BEGIN
	IF (__AccNo IS NULL) THEN
		_retval:=(50001, 'Error, Akun IS NULL (spJnlD_Insert)'||__Amount);
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _found TRUE FROM Acc WHERE AccNo=__AccNo;
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Akun:% tidak ditemukan (spJnlD_Insert)', __AccNo;
	END IF;
	
	
	_DK:=__DK;
	_Amount:=__Amount;
	_AmountForex:=__AmountForex;
	IF (_Amount<0) THEN	-- jika negatif diputar aja D/K nya
		_Amount:=_Amount*-1;
		_AmountForex:=_AmountForex*-1;

		IF (_DK='D') THEN
			_DK:='K';
		ELSE
			_DK:='D';
		END IF;
	END IF;
	
	INSERT INTO jnld (JnlNo, JnlDNo, AccNo, debit, credit, dk, jnldamt, DeptID, PrjID, jnldamtForex, JnlDNote)
	VALUES (__JnlNo, __JnlDNo, __AccNo, 
		CASE WHEN _DK='D' THEN _Amount ELSE 0 END,
		CASE WHEN _DK='K' THEN _Amount ELSE 0 END,
		_DK, _Amount, __DeptID, __PrjID, _AmountForex, __JnlDNote
	);

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Error INSERT JnlD Acc:% = %  (spJnlD_Insert)', __AccNo, _Amount;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


---------------------------------- OVERLOAD tanpa jnldnote --------------------------------------------
/** overload version, 8 parameter **/

CREATE OR REPLACE FUNCTION spJnlD_Insert ( 
	__JnlNo VARCHAR, 
	__JnlDNo INT, 
	__AccNo VARCHAR, 
	__DK VARCHAR, 
	__Amount DECIMAL(19,4), 
	__DeptID VARCHAR, 
	__PrjID VARCHAR,
	__AmountForex DECIMAL(19,4)
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_spret		SPStatus%ROWTYPE;
	_retval		SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spJnlD_Insert(__JnlNo, __JnlDNo, __AccNo, __DK, __Amount, __DeptID, __PrjID, CAST(NULL AS VARCHAR), __AmountForex);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJnl_New(
	_JnlNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_AccNo			VARCHAR(10);
	_JnlDate		DATE;
	_amount			NUMERIC(19,4);
	_amountD		NUMERIC(19,4);
	_amountK		NUMERIC(19,4);
	_Count			INT;
	_DK			VARCHAR(1);
	_DefDK			VARCHAR(1);
	_AccType2 		VARCHAR(5);
	_CashID 		VARCHAR(15);
	_Auto			BOOLEAN;
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_JnlNote		VARCHAR(255);

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);

	_tmpNote		VARCHAR(255);
	_RefType		VARCHAR;
	_RefNo			VARCHAR;
	_LockDate		DATE;
	_PerStart		DATE;

	tAccNo			VARCHAR;
	tJnlNo			VARCHAR;
	tJnlDNo			SMALLINT;

	cAcc			REFCURSOR;
	ctmp			REFCURSOR;
	cJnlD			REFCURSOR;

	_JnlDNo			SMALLINT;
	_selisih		DECIMAL(19,4);
	_JnlDAmt		DECIMAL(19,4);
	_JnlDAmt_Rounded	DECIMAL(19,4);
	_JnlDAmt_selisih	DECIMAL(19,4);

	_debet_found 		BOOLEAN;
	_kredit_found 		BOOLEAN;

	_jnldcount		INT;
	_ronde			INT;
BEGIN
	-- Dont process posting, for debugging purpose, to see what is the jurnal details amounts before gets actually posted

	--_retval:=(0, 'OK');
	--RETURN NEXT _retval;
	--RETURN;

	SELECT INTO 
		_CrtBy, _CrtDate,_UpdBy, _UpdDate, _JnlDate, _Auto, _JnlNote, _PerID, _BranchID, _RefType, _RefNo
		CrtBy, CrtDate,UpdBy, UpdDate, jnldate, isautogen, JnlNote, PerID, BranchID, reftype, refno 
	FROM jnl 
	WHERE JnlNo=_JnlNo;
	IF (NOT FOUND) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Jurnal Voucher#'||COALESCE(_JnlNo, '<kosong>')||' tidak ditemukan ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _jnldcount count(*) FROM jnld WHERE jnlNo=_JnlNo;

	IF (_jnldcount IS NULL OR _jnldcount=0) THEN
		DELETE FROM jnl WHERE JnlNo=_JnlNo;
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_ronde:=0;
	LOOP
		_ronde:=_ronde+1;

		SELECT INTO _amountD SUM(debit) FROM jnld WHERE JnlNo=_JnlNo;
		_debet_found:=FOUND;
		SELECT INTO _amountK SUM(credit) FROM jnld WHERE JnlNo=_JnlNo;
		_kredit_found:=FOUND;

		IF (_debet_found=FALSE AND _kredit_found=FALSE) THEN
			RAISE EXCEPTION 'Tidak ada transaksi di DEBET maupun KREDIT (spJnl_New)';
		END IF;

		_amountD:=COALESCE(_amountD, 0);
		_amountK:=COALESCE(_amountK, 0);

		_selisih:=_amountD-_amountK;

		EXIT WHEN (_selisih=0);

		IF (_ronde>1 AND _selisih<>0) THEN
			_retval.ErrNo:=50019;
			_retval.ErrMsg:='Referensi('||COALESCE(_RefType, '<kosong>')||')#'||COALESCE(_RefNo, '<kosong>')||E'\n'||COALESCE(_JnlNote, '<kosong>')||E'\n'||'Jumlah Debet Kredit Tidak Sama!(spJnl_New) '||_amountD||' -VS- '||_amountK;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_selisih<>0) THEN
			OPEN cJnlD FOR 
				SELECT DK, JnlDNo, JnlDAmt FROM JnlD WHERE JnlNo=_JnlNo;
			LOOP
				FETCH cJnlD INTO _DK, _JnlDNo, _JnlDAmt;
				EXIT WHEN NOT FOUND;

				_JnlDAmt_Rounded:=ROUND(_JnlDAmt, 4);
				_JnlDAmt_selisih:=_JnlDAmt-_JnlDAmt_Rounded;
			
				IF (_JnlDAmt_selisih<>0) THEN
					IF (_DK='D') THEN
						UPDATE JnlD SET Debit=_JnlDAmt_Rounded, JnlDAmt=_JnlDAmt_Rounded WHERE JnlNo=_JnlNo AND JnlDNo=_JnlDNo;
					ELSIF (_DK='K') THEN
						UPDATE JnlD SET Credit=_JnlDAmt_Rounded, JnlDAmt=_JnlDAmt_Rounded WHERE JnlNo=_JnlNo AND JnlDNo=_JnlDNo;
					END IF;
				END IF;
			END LOOP;
			CLOSE cJnlD;
		END IF;
	END LOOP;


	SELECT INTO 
		_CrtBy, _CrtDate,_UpdBy, _UpdDate, _JnlDate, _Auto, _JnlNote, _PerID, _BranchID
		CrtBy, CrtDate,UpdBy, UpdDate, jnldate, isautogen, JnlNote, PerID, BranchID
	FROM jnl 
	WHERE JnlNo=_JnlNo;

	SELECT INTO _PerStart perstart from per where active=true;
	IF(_PerStart>_JnlDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
	IF(_LockDate is not null AND _LockDate>_JnlDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _amount SUM(debit) FROM jnld WHERE JnlNo=_JnlNo;
	UPDATE jnl SET jnlamt=_amount WHERE JnlNo=_JnlNo;

	_tmpNote:=_JnlNote;

	INSERT INTO lgr(
		accno, jnlno, jnldno, lgrdate, lgramt, lgrnote, 
		PerID, BranchID, DeptID, PrjID, LgrAmtForex, CrcID, ExcRate, FisRate
		)
	SELECT 
		d.accno, d.jnlno, jnldno, _JnlDate,
		CASE WHEN a.accdk=d.dk THEN jnldamt ELSE jnldamt*-1 END,
		COALESCE(jnlnote,' ')||' '||COALESCE(jnldnote,' '),
		j.PerID,  j.BranchID, d.DeptID, d.PrjID, JnlDAmtForex, j.CrcID, j.ExcRate, j.FisRate
	FROM jnld d
	JOIN jnl j on j.jnlno=d.jnlno 
	JOIN acc a on d.accno=a.accno
	WHERE d.JnlNo=_JnlNo;

	
	OPEN cAcc FOR
		SELECT 
			acctype2.acctype2, jnld.accno, acc.AccDK, dk, jnldamt, branchid, deptid, prjid 
		FROM jnld
		JOIN jnl ON jnl.jnlno=jnld.jnlno 
		JOIN acc ON jnld.accno=acc.accno 
		LEFT JOIN acctype2 ON acc.acctype2=acctype2.acctype2
		WHERE jnld.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccType2, _AccNo, _DefDK, _DK, _amount, _BranchID, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		IF (_DefDK<>_DK) THEN	
			_amount:=_amount*-1;
		END IF;

		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_JnlDate,_amount, _BranchID, _DeptID, _PrjID);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cAcc;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJnl_Void (
	_JnlNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	cAcc		REFCURSOR;
	_AccNo		VARCHAR(10);
	_tgl		DATE;
	_amount		NUMERIC(19,4);
	_DK		VARCHAR(1);
	_DefDK		VARCHAR(1);
	_isAutoGen	BOOLEAN;
	_LockDate	DATE;
	_BranchID	VARCHAR(5);
	_DeptID		VARCHAR(5);
	_PrjID		VARCHAR(5);
	_PerStart	DATE;
BEGIN
	_retval.ErrNo:=50001;
	_retval.ErrMsg:='General Error (spJnl_Void)';

	SELECT INTO 
		_tgl, _isAutoGen
		jnldate, isAutoGen
	FROM jnl WHERE JnlNo=_JnlNo;

	SELECT INTO _PerStart perstart from per where active=true;
	IF(_PerStart>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Melebihi Batas Periode Akuntansi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='JNL';
	IF(_LockDate is not null AND _LockDate>_tgl) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Jurnal Melebihi Batas Tanggal Kunci Transaksi ! (spJnl_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cAcc FOR SELECT d.accno,d.dk,d.jnldamt,j.branchid,d.deptid,d.prjid FROM jnl j, jnld d WHERE j.jnlno=d.jnlno
	AND d.JnlNo=_JnlNo;
	LOOP
		FETCH cAcc INTO _AccNo,_DK,_amount,_BranchID,_DeptID,_PrjID;
		EXIT WHEN NOT FOUND;
		SELECT INTO _DefDK accdk FROM acc WHERE AccNo=_AccNo;
		IF _DefDK=_DK THEN	
			_amount:=_amount*-1;
		END IF;			
		
		SELECT INTO _spret * FROM spAccBal_Update(_AccNo,_tgl,_amount,_BranchID,_DeptID,_PrjID);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
	END LOOP;

	/** Void nested trans: BPA, CashA */

	IF (_isAutoGen=false) THEN
		SELECT INTO _spret * FROM spBPA_Void_ByRef('JNL', _JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spCashA_Void_ByRef('JNL', _JnlNo, NULL, NULL);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;		
	END IF;
	/***/

	_retval:=(50010, 'Error Delete Lgr (spJnl_Void)');
	DELETE FROM Lgr WHERE jnlno=_JnlNo;

	_retval:=(50015, 'Error Delete JnlD (spJnl_Void)');
	DELETE FROM Jnld WHERE jnlno=_JnlNo;

	_retval:=(50020, 'Error Delete Jnl (spJnl_Void)');
	DELETE FROM Jnl WHERE jnlno=_JnlNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBpAcc (
	_BpID		VARCHAR,
	_CrcID		VARCHAR,
	_AccType	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval
	CASE _AccType
		WHEN 'AR' THEN accar
		WHEN 'AP' THEN accap
	END
	FROM bpacc WHERE BpID=_BpID AND crcid=_CrcID;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItemAcc (
	_ItemID		VARCHAR,
	_AccType	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval
	CASE _AccType
		WHEN 'INV' THEN accinv
		WHEN 'COST' THEN acccost
		WHEN 'SALE' THEN accsale
		WHEN 'PRET' THEN accpret
		WHEN 'SRET' THEN accsret
		WHEN 'DISC' THEN accitemdisc
		WHEN 'PRCV' THEN accprcv
		WHEN 'EXP' THEN accexp
--		WHEN 'CRET' THEN acccret
	END
	FROM item WHERE itemid=_ItemID;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE VIEW vw_CashBal_vs_CashA_SUM AS
	SELECT c.cashid, c.cashbal, v.casha_sum 
	FROM cash c, 
		(SELECT cashid, SUM(amount) AS casha_sum FROM casha GROUP BY cashid) v
	WHERE c.cashid=v.cashid;




CREATE OR REPLACE VIEW vw_ARBalance_vs_SaleDue AS
	SELECT b.bpid, a.arbalance, a.crcid, v.sale_due_total 
	FROM bp b
	LEFT JOIN (SELECT custid, crcid, SUM(total-paidamt) AS sale_due_total FROM sale WHERE paystatus<>'F' GROUP BY custid, crcid) v ON b.bpid=v.custid
	LEFT JOIN bpacc a ON b.bpid=a.bpid;




CREATE OR REPLACE FUNCTION fToDo() 
RETURNS SETOF tToDo AS
$BODY$
DECLARE
	_retval		tToDo%ROWTYPE;
	cTabel		REFCURSOR;
BEGIN

	_retval.todoid='REMINDER';
	/*--_retval.uptodoid='*';*/
	_retval.todoname='Reminder';
	_retval.tooltip='Pengingat Data Hari ini';
	_retval.todolvl=1;
	RETURN NEXT _retval;

	_retval.todoid='HUTANG';
	_retval.uptodoid='REMINDER';
	_retval.todoname='Hutang';
	_retval.tooltip='Data Hutang Jatuh Tempo';
	_retval.todolvl=2;

	RETURN NEXT _retval;

	OPEN cTabel FOR
		SELECT purcno,'HUTANG',purcno||'('||bpname||')',CAST(purcdate AS VARCHAR)||':: sisa='||CAST(sisa AS VARCHAR),3 FROM fhutangjatuhtempo(CURRENT_DATE,NULL);
	LOOP
		FETCH cTabel INTO _retval;
		IF NOT FOUND THEN
			EXIT;
		END IF;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cTabel;

	_retval.todoid='PIUTANG';
	_retval.uptodoid='REMINDER';
	_retval.todoname='Piutang';
	_retval.tooltip='Data Piutang Jatuh Tempo';
	_retval.todolvl=2;

	RETURN NEXT _retval;

	OPEN cTabel FOR
		SELECT saleno,'PIUTANG',saleno||'('||bpname||')',CAST(saledate AS VARCHAR)||':: sisa='||CAST(sisa AS VARCHAR),3 FROM fpiutangjatuhtempo(CURRENT_DATE,NULL);
	LOOP
		FETCH cTabel INTO _retval;
		IF NOT FOUND THEN
			EXIT;
		END IF;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cTabel;

	_retval.todoid='CEKBG';
	_retval.uptodoid='REMINDER';
	_retval.todoname='Cek Giro';
	_retval.tooltip='Cek Giro';
	_retval.todolvl=2;

	RETURN NEXT _retval;

	_retval.todoid='STOCKLOW';
	_retval.uptodoid='REMINDER';
	_retval.todoname='Stock Habis';
	_retval.tooltip='Data Stock Yang hampir / sudah habis';
	_retval.todolvl=2;

	RETURN NEXT _retval;

	OPEN cTabel FOR
	SELECT v.itemid,'STOCKLOW',i.itemdesc,'Qty:'||CAST(fQty1Desc(v.ItemID, v.Qty) AS VARCHAR)||' in '|| v.whid ||' pid :'||v.pid,3
	FROM vwStock v JOIN Item i ON v.ItemID=i.ItemID
	WHERE qty<v.qtymin;
		
	LOOP
		FETCH cTabel INTO _retval;
		IF NOT FOUND THEN
			EXIT;
		END IF;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cTabel;
	_retval.todoid='DO';
	_retval.uptodoid='REMINDER';
	_retval.todoname='Delivery Order';
	_retval.tooltip='Data Pengiriman Hari ini';
	_retval.todolvl=2;

	RETURN NEXT _retval;

	OPEN cTabel FOR
	SELECT deono,'DO',deono||'('||custid||')','kirim :'||CAST(reqdtime AS VARCHAR),3 FROM deo where reqdtime>=current_timestamp;
	LOOP
		FETCH cTabel INTO _retval;
		IF NOT FOUND THEN
			EXIT;
		END IF;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cTabel;

	_retval.todoid='INBOX';
	/*--_retval.uptodoid='*';*/
	_retval.todoname='Inbox';
	_retval.tooltip='Messages';
	_retval.todolvl=1;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION isAccInstalled()
  RETURNS boolean AS
$BODY$
DECLARE
	_retval		boolean;

	
BEGIN
	_retval=false;
	SELECT INTO _retval isinstalled FROM modul WHERE modulid='ACC';
	RETURN _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION fsaldoPeriode (
	_accno		VARCHAR,
	_PerID		VARCHAR,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
	_StartDate	DATE;
	_EndDate	DATE;
	_DefaultDK	VARCHAR(1);
BEGIN

/*--select * from lgr*/

	SELECT INTO _StartDate,_EndDate perstart,perend FROM per WHERE perid=_PerID;
	SELECT INTO _DefaultDK AccDK FROM ACC WHERE AccNo=_AccNo;
	SELECT INTO _retval (COALESCE(SUM(CASE WHEN dk=_DefaultDK THEN jnldamt ELSE -1*jnldamt END), 0)) 
	FROM jnld 
	JOIN jnl ON jnl.jnlno=jnld.jnlno
	WHERE accNo=_accNo AND JnlDate>=_StartDate AND JnlDate<=_EndDate
	AND (_BranchID IS NULL OR BranchID=_BranchID) 
	AND (_DeptID IS NULL OR DeptID=_DeptID) 
	AND (_PrjID IS NULL OR PrjID=_PrjID);

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fLabaBersih (
	_Periode	VARCHAR,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	SELECT INTO 
	_retval 
	flabaOperasi(_Periode, _BranchID, _DeptID, _PrjID)
	+
	(SELECT SUM(fsaldoPeriode(accno,_Periode, _BranchID, _DeptID, _PrjID)) FROM acc WHERE acctype='700')
	-
	(SELECT SUM(fsaldoPeriode(accno,_Periode, _BranchID, _DeptID, _PrjID)) FROM acc WHERE acctype='800');

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fLabaKotor (
	_Periode	VARCHAR,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
	
BEGIN
	SELECT INTO 
	_retval 
	((SELECT sum(fsaldoPeriode(accno, _Periode, _BranchID, _DeptID, _PrjID)) FROM acc WHERE acctype='400') - 
	(SELECT sum(fsaldoPeriode(accno, _Periode, _BranchID, _DeptID, _PrjID)) FROM acc WHERE acctype='500'));

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fLabaOperasi (
	_Periode	VARCHAR,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	SELECT INTO 
	_retval 
	flabakotor(_Periode, _BranchID, _DeptID, _PrjID)
	-
	(select sum(fsaldoPeriode(accno, _Periode, _BranchID, _DeptID, _PrjID)) from acc where acctype='600');

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fNeraca (
	_PerID		VARCHAR,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS SETOF tNeraca AS
$BODY$
DECLARE
	_retval tNeraca%ROWTYPE;
	_PrevPerID	VARCHAR;
	cNeraca	refcursor;
BEGIN
	SELECT INTO _PrevPerID perid FROM per WHERE perno=(SELECT perno-1 FROM per WHERE perid=_PerID);

	OPEN cNeraca FOR
		SELECT acctypedesc, acctype1bdesc, 
		acctype2desc, accno, accname, 
		fsaldoPeriode(accno, _PrevPerID, _BranchID, _DeptID, _PrjID) as accbalold, 
		fsaldoPeriode(accno,_PerID, _BranchID, _DeptID, _PrjID) as accbal 
		FROM acc a 
		JOIN acctype1 d ON a.acctype=d.acctype 
		join acctype2 c ON a.acctype2=c.acctype2 
		LEFT JOIN acctype1b b ON c.acctype1B=b.acctype1b
		WHERE d.nr='N'
		ORDER BY d.acctype, b.acctype1b, c.acctype2;
	LOOP
		FETCH cNeraca INTO _retval;
		EXIT WHEN NOT FOUND;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cNeraca;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPOPRcv (
	_PRcvNo		VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_cpo	REFCURSOR;
	_PONo	VARCHAR;
	_retval	VARCHAR;
BEGIN
	_retval:='';

	OPEN _cpo FOR
		/*--SELECT PONo FROM POPRcv WHERE PRcvNo=_PRcvNo;*/
		SELECT DISTINCT(PONo) FROM PRcvD WHERE PRcvNo=_PRcvNo;
	LOOP
		FETCH _cpo INTO _PONo;
		EXIT WHEN NOT FOUND;

		_retval:=_retval||_PONo||',';
	END LOOP;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPOPurc (
	_PurcNo		VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_cpo	REFCURSOR;
	_PONo	VARCHAR;
	_retval	VARCHAR;
BEGIN
	_retval:='';
	OPEN _cpo FOR
		/*--SELECT PONo FROM POPurc WHERE PurcNo=_PurcNo;*/
		SELECT DISTINCT(PONo) FROM PurcD WHERE PurcNo=_PurcNo;
	LOOP
		FETCH _cpo INTO _PONo;
		EXIT WHEN NOT FOUND;

		_retval:=_retval||_PONo||',';
	END LOOP;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSODeli (
	_DeliNo		VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_cso	REFCURSOR;
	_SONo	VARCHAR;
	_retval	VARCHAR;
BEGIN
	_retval:='';
	OPEN _cso FOR
		/*--SELECT SONo FROM SODeli WHERE DeliNo=_DeliNo;*/
		SELECT DISTINCT(SONo) FROM DeliD WHERE DeliNo=_DeliNo;
	LOOP
		FETCH _cso INTO _SONo;
		EXIT WHEN NOT FOUND;

		_retval:=_retval||_SONo||',';
	END LOOP;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSOSale (
	_SaleNo		VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_cso	REFCURSOR;
	_SONo	VARCHAR;
	_retval	VARCHAR;
BEGIN
	_retval:='';
	OPEN _cso FOR
		/*--SELECT SONo FROM SOSale WHERE SaleNo=_SaleNo;*/
		SELECT DISTINCT(SONo) FROM SaleD WHERE SaleNo=_SaleNo;
	LOOP
		FETCH _cso INTO _SONo;
		EXIT WHEN NOT FOUND;

		_retval:=_retval||_SONo||',';
	END LOOP;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fWhList(
	_UsrID	VARCHAR(20)
) RETURNS SETOF tWhList AS 
$BODY$
DECLARe
	_WhID		VARCHAR(50);
	_WhName		VARCHAR(50);
	_GrpID		VARCHAR(20);
	cGrp		REFCURSOR;
	cWh		REFCURSOR;
	cWhAll		REFCURSOR;
	_retval		tWhList%ROWTYPE;
	jum		INT;
BEGIN
	OPEN cGrp FOR
		SELECT grpid FROM usrgrp WHERE usrid=_UsrID;
	LOOP
		FETCH cGrp INTO _GrpID;
		EXIT WHEN NOT FOUND;

		jum=0;
		OPEN cWh FOR
			SELECT wp.whid,whname 
			FROM whprv wp 
			JOIN wh on wp.whid=wh.whid 
			WHERE grpid=_GrpID AND acsenable=true;
		LOOP
			FETCH cWh INTO _WhID,_WhName;
			EXIT WHEN NOT FOUND;

			_retval.whid:=_WhID;
			_retval.whname:=_WhName;
			jum:=jum+1;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cWh;

		IF (jum=0) THEN
			OPEN cWhAll FOR
				SELECT whid,whname FROM wh;
			LOOP
				FETCH cWhAll INTO _WhID,_WhName;
				EXIT WHEN NOT FOUND;

				_retval.whid:=_WhID;
				_retval.whname:=_WhName;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cWhAll;
		END IF;
	END LOOP;
	CLOSE cGrp;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBranchList (
	_UsrID	VARCHAR(20)
) RETURNS SETOF tBranchList AS 
$BODY$
DECLARE
	_BranchID	VARCHAR(50);
	_BranchName	VARCHAR(50);
	_GrpID		VARCHAR(20);
	cGrp		REFCURSOR;
	cBranch		REFCURSOR;
	cBranchAll	REFCURSOR;
	_retval		tBranchList%ROWTYPE;
	jum		INT;
BEGIN
	OPEN cGrp FOR
		SELECT grpid FROM usrgrp WHERE usrid=_UsrID;
	LOOP
		FETCH cGrp INTO _GrpID;
		EXIT WHEN NOT FOUND;

		jum=0;
		OPEN cBranch FOR
			SELECT bcp.branchid, branchname 
			FROM branchprv bcp 
			JOIN branch ON bcp.branchid=branch.branchid 
			WHERE grpid=_GrpID AND acsenable=true;
		LOOP
			FETCH cBranch INTO _BranchID,_BranchName;
			EXIT WHEN NOT FOUND;

			_retval.branchid:=_BranchID;
			_retval.branchname:=_BranchName;
			jum:=jum+1;

			RETURN NEXT _retval;
		END LOOP; 
		CLOSE cBranch;

		IF (jum=0) THEN
			OPEN cBranchAll FOR
				SELECT branchid, branchname FROM branch;
			LOOP
				FETCH cBranchAll INTO _BranchID,_BranchName;
				EXIT WHEN NOT FOUND;

				_retval.branchid:=_BranchID;
				_retval.branchname:=_BranchName;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cBranchAll;
		END IF;
	END LOOP;
	CLOSE cGrp;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCashList (
	_UsrID	VARCHAR(20)
) RETURNS SETOF tCashList AS 
$BODY$
DECLARE
	_CashID		VARCHAR(50);
	_CashDesc	VARCHAR(50);
	_GrpID		VARCHAR(20);
	jum		INT;
	cGrp		REFCURSOR;
	cCash		REFCURSOR;
	cCashAll	REFCURSOR;
	_retval		tCashList%ROWTYPE;
BEGIN
	OPEN cGrp FOR
		SELECT grpid FROM usrgrp WHERE usrid=_UsrID;
	LOOP
		FETCH cGrp INTO _GrpID;
		EXIT WHEN NOT FOUND;

		jum:=0;
		OPEN cCash FOR
			SELECT cp.cashid, cashdesc 
			FROM cashprv cp 
			JOIN cash ON cp.cashid=cash.cashid 
			WHERE grpid=_GrpID AND acsenable=true;
		LOOP
			FETCH cCash INTO _CashID,_CashDesc;
			EXIT WHEN NOT FOUND;

			_retval.cashid:=_CashID;
			_retval.cashdesc:=_CashDesc;

			RETURN NEXT _retval;
			jum:=jum+1;
		END LOOP; 
		CLOSE cCash;

		IF (jum=0) THEN
			OPEN cCashAll FOR
				select cashid,cashdesc from cash;
			LOOP
				FETCH cCashAll INTO _CashID,_CashDesc;
				EXIT WHEN NOT FOUND;

				_retval.cashid:=_CashID;
				_retval.cashdesc:=_CashDesc;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cCashAll;
		END IF;
	END LOOP;
	CLOSE cGrp;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fExcRate(
	__CrcID	VARCHAR
) RETURNS DECIMAL(19,4) AS
$BODY$
DECLARE
	_retval	DECIMAL(19,4);
BEGIN

	SELECT INTO 
		_retval
		ExcRate
	FROM crc
	WHERE crcid=__CrcID;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCbg_New(_CBgNo VARCHAR(15)) 
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval  		SPStatus%ROWTYPE;
	spretCashA_New 		SPStatus%ROWTYPE;
	spret 	 		SPStatus%ROWTYPE;
	_InOut 	 		VARCHAR(3);
	_CashID  		VARCHAR(20);
	_CustID  		VARCHAR(20);
	_toBPID  		VARCHAR(20);
	_fromBPID  		VARCHAR(20);
	_CbgDate  		DATE;
	_NomerCbg  		VARCHAR(20);
	_ExtNote  		VARCHAR(50);
	_outbal  		DECIMAL(19,4);
	_crcid 	 		VARCHAR;
	_ExcRate  		DECIMAL(19,4);
	total 	 		DECIMAL(19,4);
	_SrcCashID 		VARCHAR;
	_ClrCashID 		VARCHAR;
	_CBgNomer 		VARCHAR;
	_CbgID	 		VARCHAR;
	_CBGDueDate 		DATE;
	_AllowCredit 		DECIMAL(19,4);
	_CreditLimit 		DECIMAL(19,4);
	_CBGAmt	 		DECIMAL(19,4);
BEGIN

	SELECT INTO 
		_InOut,_SrcCashID,_ClrCashID,_CbgDate,_CBGDueDate,_fromBPID,_toBPID,_CBgNomer, _CBGAmt
		inout, srccashid, clrcashid, cbgdate, cbgduedate, frombpid, tobpid, cbgno, cbgamt
	FROM cbg WHERE cbgid=_CBgNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Cheque #'||_CBgNo||' tidak ditemukan (spCbg_New) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF(_InOut='O') THEN
		IF (_SrcCashID IS NULL OR _SrcCashID='') THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Kas asal belum diisi (spCbg_New) !';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;


		SELECT INTO _CustID toBPID FROM cbg WHERE cbgid=_CBgNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Penerima Cek/Giro Tidak Boleh Kosong (spCbg_New) !';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	IF(_SrcCashID=_ClrCashID) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Kas Asal dengan Kas Tujuan Pencairan tidak boleh sama (spCbg_New) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	END IF;

	IF(_CBGDueDate<_CbgDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Tanggal Efektif tidak boleh kurang dari Tanggal Terbit (spCbg_New) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	END IF;

	IF(_fromBPID=_toBPID) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Penerbit Cek/Giro tidak boleh sama dengan Penerima Cek/Giro (spCbg_New) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _CbgID CbgID FROM cbg WHERE CBgNo=_CBgNomer AND cbgid<>_CBgNo;
	IF FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Nomor Cek/Giro sudah pernah dipakai (spCbg_New) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	IF(_toBPID IS NOT NULL) THEN
		SELECT INTO _AllowCredit, _CreditLimit 
			arlimitfreq, arlimitamt 
		FROM bpacc WHERE bpid=_toBPID;

		IF (_CreditLimit=0) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Penerima Cek/Giro #'||_toBPID||' tidak diperkenankan KREDIT (spCbg_New) !';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSIF (_AllowCredit=0) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Penerima Cek/Giro #'||_toBPID||' tidak diperkenankan KREDIT (spCbg_New) !';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;
	
	IF (_fromBPID IS NOT NULL) THEN		
		SELECT INTO _AllowCredit, _CreditLimit 
			arlimitfreq, arlimitamt 
		FROM bpacc WHERE bpid=_fromBPID;

		IF (_CreditLimit=0) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Penerbit Cek/Giro #'||_fromBPID||' tidak diperkenankan KREDIT (spCbg_New) !';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSIF (_AllowCredit=0) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Penerbit Cek/Giro #'||_fromBPID||' tidak diperkenankan KREDIT (spCbg_New) !';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
	END IF;

	IF (_InOut='O') THEN
		UPDATE cash SET outbal=outbal+_CBGAmt WHERE cashid=_srccashid;
	END IF;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;		
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCbg_Void(_CbgNo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	
	_CashID		VARCHAR(15);
	_SrcCashID	VARCHAR(15);
	_InOut 		VARCHAR(3);
	_CbgAmt		NUMERIC(19,4);
	_active		BOOLEAN;
	_ClearDate	DATE;
BEGIN

	IF EXISTS (SELECT 1 FROM payd WHERE cbgid=_CbgNo) THEN 
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Cheque #'||_CbgNo||' sudah dipakai untuk pembayaran';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF EXISTS (SELECT 1 FROM rcvd WHERE cbgid=_CbgNo) THEN 
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Cheque #'||_CbgNo||' sudah dipakai untuk penerimaan';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO 
		_InOut, _ClearDate, _SrcCashID, _CBGAmt
		inout, ClrDate, SrcCashID, CBGAmt
	FROM cbg WHERE CBGID=_CbgNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Cheque #'||_CbgNo||' tidak ditemukan (spCbg_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_ClearDate IS NOT NULL) THEN
		SELECT INTO _spret spCbg_Unclear(_CbgNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;	
	END IF;

	--- Reset back Cash:OutBal
	IF (_InOut='O' AND _SrcCashID IS NOT NULL) THEN
		UPDATE Cash SET OutBal=OutBal-_CBGAmt WHERE cashid=_SrcCashID;
	END IF;

	DELETE FROM cbg WHERE CbgID=_CbgNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;
/**
	sp untuk pencarian Cheque / Giro

**/


CREATE OR REPLACE FUNCTION spCbg_Clear(
	__CbgID 	VARCHAR,
	__ClearDate	DATE,
	__ClearCashID	VARCHAR
)
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval 		SPStatus%ROWTYPE;
	spretCashA_New 		SPStatus%ROWTYPE;
	_spret 			SPStatus%ROWTYPE;
	_SrcCashid 		VARCHAR(20);
	_InOut 			VARCHAR(3);
	_CbgDate 		DATE;
	_CbgAmt 		DECIMAL(19,4);
	_ForeignAmt 		DECIMAL(19,4);
	_CbgTypeDesc		VARCHAR;
	_CbgDueDate		DATE;

	_ExtNote 		VARCHAR;
	_CsTrNo 		VARCHAR(20);
	_CsTrNote 		VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate 		DECIMAL(19,4);
	_FisRate		NUMERIC(19,4);

	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;	
	_Count			INT;
	_AccAssetCbg		VARCHAR(10);
	_AccAssetClr		VARCHAR(10);
	_CbgClrDate		DATE;
	_isExpired		BOOLEAN;
	_AccNo1			VARCHAR(10);
	_AccNo2			VARCHAR(10);
	_LockDate		DATE;
	_Active			BOOLEAN;

	_isAutoReconcileIn	BOOLEAN;
	_isAutoReconcileOut	BOOLEAN;
	_JnlDAmt		NUMERIC(19,4);
	_JnlDAmtForex		NUMERIC(19,4);

	_ClearExcRate		NUMERIC(19,4);
	_ClearFisRate		NUMERIC(19,4);


	_FromBPID		VARCHAR;
	_ToBPID			VARCHAR;	
	_CbgNo			VARCHAR;
BEGIN

	SELECT INTO 
		_CbgNo, _CbgDate, _CbgDueDate, _SrcCashID, _FromBPID, _ToBPID, _CbgAmt, _InOut, 
		_CrcID, _ExcRate, _FisRate, _CbgClrDate, _Active, _CbgTypeDesc
		cbgno, cbgdate, cbgduedate, srccashid, frombpid, tobpid, cbgamt, inout, 
		crcid, excrate, fisrate, clrdate, active, ct.cbgtypedesc
	FROM cbg c JOIN cbgtype ct ON c.cbgtype=ct.cbgtype 
	WHERE cbgid=__CbgID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:=_CbgTypeDesc||'#'||__CbgID||' tidak ditemukan (spCbg_Clear) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CBG';
	IF(_LockDate is not NULL AND _LockDate>__ClearDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Tanggal Kliring melebihi batas Tanggal Kunci Transaksi (spCbg_Clear) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	IF(_CbgClrDate is NOT NULL) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:=_CbgTypeDesc||'#'||__CbgID||' sudah dikliringkan (spCbg_Clear) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF(_Active=TRUE) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:=_CbgTypeDesc||'#'||__CbgID||' masih aktif (spCbg_Clear) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (__ClearDate<_CbgDueDate) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Kliring tidak boleh dilakukan sebelum tanggal efektif (spCbg_Clear) !';		
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF(_InOut='I') THEN		
		IF(__ClearCashID IS NOT NULL) THEN
			_CsTrNo:=fTransNoGet('CSTR');

			-- konversi kurs antar mata uang berbeda disini 
			-- untuk saat ini belum support beda, kita block dulu kalau beda
			IF (fCashCrc(_SrcCashID)<>fCashCrc(__ClearCashID)) THEN
				RAISE EXCEPTION 'Kliring dengan Mata Uang berbeda belum disupport. %(%) -vs- %(%) (spCbg_Clear) !', _SrcCashID, fCashCrc(_SrcCashID), __ClearCashID, fCashCrc(__ClearCashID);
			END IF;
			
			_CsTrNote:='Pencairan '||_CbgTypeDesc||'# '||_CbgNo||' dari '||_FromBPID;
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Error INSERT CsTr! (spCbg_Clear)';
			

			_ClearExcRate:=fExcRate(fCashCrc(__ClearCashID));

			_ForeignAmt:=_CbgAmt;
			INSERT INTO CsTr(CsTrDate, CsTrNo, CashID1, CrcID1, ExcRate1, CashID2, CrcID2, ExcRate2, cstramt, foreignamt, CsTrNote, reftype, refno)
			VALUES (__ClearDate, _CsTrNo, _SrcCashid, fCashCrc(_SrcCashID), _ExcRate, __ClearCashID, fCashCrc(__ClearCashID), _ClearExcRate, _CbgAmt, _ForeignAmt, _CsTrNote, 'CBG', __CbgID);

			SELECT INTO _spret * FROM spCsTr_New(_CsTrNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
			END IF;
		END IF;
			
	ELSIF(_InOut='O') THEN
		_ExtNote:='Pencairan '||_CbgTypeDesc||'#'||_CbgNo||' untuk '|| _ToBPID;
		
		SELECT INTO spretCashA_New * FROM spCashA_New(
			   _SrcCashID, _ToBPID, 'CBG', __ClearDate, 'O', 'CBG', __CbgID, _CbgAmt*(-1),
			   _ExtNote , TRUE, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, _CrcID, _ExcRate, _FisRate);
	   	IF spretCashA_New.ErrNo<>0 THEN
			_retval.ErrNo:=spretCashA_New.ErrNo;
			_retval.ErrMsg:=spretCashA_New.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		UPDATE cash SET 
			OutBal=OutBal-_CBGAmt 
		WHERE cashid=_SrcCashID;

	END IF;

	-- update status active
	UPDATE cbg SET active=false, clrdate=__ClearDate, clrcashid=__ClearCashID 
	WHERE cbgid=__CbgID;

	--------------- ACCOUNTING --------------------
	IF isAccInstalled() THEN

		_Count:=0;
		IF(_InOut='O') THEN	
			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:= 'Pencairan '||_CbgTypeDesc||'#'|| _CbgNo ||' untuk '||fBPName(_ToBPID);
			SELECT INTO _spret * FROM spJnl_Insert(_JnlNo, 'CBG', __ClearDate, 'CBG', __CbgID,
					_JnlNote, TRUE, TRUE, CAST(0 as NUMERIC(19,4)), NULL, NULL, NULL, NULL,
					NULL, NULL, NULL, NULL, _CrcID, _ExcRate, _FisRate
				);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
	
			_AccAssetCbg:=fAccCash(_srcCashID, _InOut);

			--- baca setting akun & reconcile cash/bank
			SELECT INTO 
				_AccNo1, _isAutoReconcileIn, _isAutoReconcileOut
				AccNo, isAutoReconcileIn, isAutoReconcileOut
			FROM cash 
			WHERE cashid=_srcCashID;

			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('OPENCBG'), 'D', _CbgAmt*_ExcRate, NULL, NULL, _CbgAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccAssetCbg, 'K', _CbgAmt*_ExcRate, NULL, NULL, _CbgAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
	
			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
	
		ELSIF(_InOut='I') THEN
				_JnlNo:=fTransNoGet('JURNAL');
				_JnlNote:='Pencairan '||_CbgTypeDesc||'#'|| _CbgNo ||' dari '||fBPName(_FromBPID);
		
				SELECT INTO _spret * FROM spJnl_Insert(
					_JnlNo, 'CBG', __ClearDate, 'CBG', __CbgID,
					_JnlNote, TRUE, TRUE,CAST(0 as NUMERIC(19,4)),
					NULL, NULL, NULL, NULL,
					NULL, NULL, NULL, NULL,_CrcID, _ExcRate, _FisRate
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				_JnlDAmt:=_CbgAmt*_ExcRate;
				_JnlDAmtForex:=_CbgAmt;

				--- baca setting akun & reconcile cash/bank
				SELECT INTO 
					_AccNo1, _isAutoReconcileIn, _isAutoReconcileOut
					AccNo, isAutoReconcileIn, isAutoReconcileOut
				FROM cash 
				WHERE cashid=_srcCashID;
				IF (_isAutoReconcileOut=FALSE) THEN
					_AccNo1:=fAccDefault('UNRECBANK');
				END IF;

				SELECT INTO 
					_AccNo2, _isAutoReconcileIn, _isAutoReconcileOut
					AccNo, isAutoReconcileIn, isAutoReconcileOut
				FROM cash 
				WHERE cashid=__ClearCashID;
				IF (_isAutoReconcileIn=FALSE) THEN
					_AccNo2:=fAccDefault('UNRECBANK');
				END IF;


				-- ayat debet						
				_Count:=1;
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count,_AccNo1, 'K', _JnlDAmt, NULL, NULL, _JnlDAmtForex);

				-- ayat kredit
				_Count:=_Count+1;
				SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count,_AccNo2,'D', _JnlDAmt, NULL, NULL, _JnlDAmtForex);


				SELECT INTO _spret * FROM spJnl_New(_JnlNo);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;		
	END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION spCbg_Unclear(
	_CbgNo VARCHAR(15)
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval 	SPStatus%ROWTYPE;
	spretCashA_New 	SPStatus%ROWTYPE;
	_spret 		SPStatus%ROWTYPE;
	_ClrCashid 	VARCHAR(20);
	_SrcCashid 	VARCHAR(20);
	_InOut 		VARCHAR(3);
	_CashID 	VARCHAR(20);
	_CustID		VARCHAR(20);
	_CbgDate 	DATE;
	_CbgAmt 	DECIMAL(19,4);
	_NomerCbg 	VARCHAR(20);
	_ExtNote 	VARCHAR(50);
	_CsTrNo 	VARCHAR(20);
	_CsTrNote 	VARCHAR(50);
	_outbal 	DECIMAL(19,4);
	total 		DECIMAL(19,4);
	_CashaNo 	VARCHAR(20);
	_JnlNo		VARCHAR(15);
	_RefNo		VARCHAR;
	_ClrDate 	DATE;
	_LockDate	DATE;

BEGIN

	SELECT INTO _InOut, _ClrDate inout, clrdate FROM cbg WHERE cbgid=_CbgNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='CBG #'||_CbgNo||' tidak ditemukan (spCbg_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CBG';
	IF(_LockDate is not NULL AND _LockDate>_ClrDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Clear Date Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF(_ClrDate is NULL) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='CBG #'||_CbgNo||' belum di clear kan';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF(_InOut='I') THEN
		SELECT INTO _ClrCashid clrcashid FROM cbg WHERE cbgid=_CbgNo;
		IF(_ClrCashid is NOT NULL) THEN
			/*SELECT INTO _CashaNo cashano FROM casha WHERE reftype='CBG' and refno=_CbgNo and inout='O';
			SELECT INTO _spret * FROM spCashA_Void(_CashaNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
				
		ELSE*/

			SELECT INTO _CsTrNo cstrno FROM cstr WHERE reftype='CBG' AND refno=_CbgNo;
			SELECT INTO _spret * FROM spCsTr_Void(_CsTrNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		END IF;
			
	ELSIF(_InOut='O') THEN
		SELECT INTO _CashID, _CustID, _CbgDate, _CbgAmt, _NomerCbg 
			srccashid, tobpid, cbgdate, cbgamt, cbgno 
		FROM cbg 
		WHERE cbgid=_CbgNo;

		SELECT INTO _spret * FROM spCashA_Void_byRef('CBG', _CbgNo, _CashID, null);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _outbal outbal FROM cash WHERE cashid=_CashID;
		total:=_outbal+_CbgAmt;

		UPDATE cash SET outbal=total WHERE cashid=_CashID;
	END IF;

	UPDATE cbg SET clrdate=NULL,clrcashid=NULL WHERE cbgid=_CbgNo;

	------- AKUNTANSI ----------
	IF isAccInstalled() THEN
		SELECT INTO _spret * FROM spJnl_Void_byRef('CBG',_CbgNo);
		SELECT INTO _spret * FROM spJnl_Void(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;		
	END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION spTransLock(
	_StartDate 	DATE, 
	_EndDate 	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_Trans		REFCURSOR;
	_TransNo	VARCHAR(15);
BEGIN 
	/** update Purchase Request **/
	OPEN _Trans FOR
		SELECT PReqNo FROM PReq WHERE PReqDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE PReq SET isLocked=TRUE WHERE PReqNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update PReq : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Purchase Order **/
	OPEN _Trans FOR
		SELECT PONo FROM PO WHERE PODate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE PO SET isLocked=TRUE WHERE PONo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update PO : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Purchase Receive **/
	OPEN _Trans FOR
		SELECT PRcvNo FROM PRcv WHERE PRcvDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE PRcv SET isLocked=TRUE WHERE PRcvNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update PRcv : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Purchase Invoice **/
	OPEN _Trans FOR
		SELECT PurcNo FROM Purc WHERE PurcDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE Purc SET isLocked=TRUE WHERE PurcNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update Purc : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Purchase Return **/
	OPEN _Trans FOR
		SELECT PRetNo FROM PRet WHERE PRetDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE PRet SET isLocked=TRUE WHERE PRetNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update PRet : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Sale Order **/
	OPEN _Trans FOR
		SELECT SONo FROM SO WHERE SODate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE SO SET isLocked=TRUE WHERE SONo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update SO : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Delivery Order **/
	OPEN _Trans FOR
		SELECT DeONo FROM DeO WHERE DeODate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE DeO SET isLocked=TRUE WHERE DeONo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update DeO : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Delivery **/
	OPEN _Trans FOR
		SELECT DeliNo FROM Deli WHERE DeliDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE Deli SET isLocked=TRUE WHERE DeliNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update Deli : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Sale **/
	OPEN _Trans FOR
		SELECT SaleNo FROM Sale WHERE SaleDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE Sale SET isLocked=TRUE WHERE SaleNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update Sale : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Sale Return **/
	OPEN _Trans FOR
		SELECT SRetNo FROM SRet WHERE SRetDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE SRet SET isLocked=TRUE WHERE SRetNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update SRet : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Pay **/
	OPEN _Trans FOR
		SELECT PayNo FROM Pay WHERE PayDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE Pay SET isLocked=TRUE WHERE PayNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update Pay : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Rcv **/
	OPEN _Trans FOR
		SELECT RcvNo FROM Rcv WHERE RcvDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE Rcv SET isLocked=TRUE WHERE RcvNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update Rcv : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Cash Transfer **/
	OPEN _Trans FOR
		SELECT CsTrNo FROM CsTr WHERE CsTrDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE CsTr SET isLocked=TRUE WHERE CsTrNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update CsTr : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Cash Adjustment **/
	OPEN _Trans FOR
		SELECT CashANo FROM CashA WHERE CashADate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE CashA SET isLocked=TRUE WHERE CashANo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update CashA : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Stock Transfer **/
	OPEN _Trans FOR
		SELECT StTrNo FROM StTr WHERE StTrDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE StTr SET isLocked=TRUE WHERE StTrNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update StTr : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Stock Adjustment **/
	OPEN _Trans FOR
		SELECT StockANo FROM StockA WHERE StockADate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE StockA SET isLocked=TRUE WHERE StockANo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update StockA : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Cons **/
	OPEN _Trans FOR
		SELECT ConsNo FROM Cons WHERE ConsDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE Cons SET isLocked=TRUE WHERE ConsNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update Cons : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Jnl **/
	OPEN _Trans FOR
		SELECT JnlNo FROM Jnl WHERE JnlDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE Jnl SET isLocked=TRUE WHERE JnlNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update Jnl : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update Mfg **/
	OPEN _Trans FOR
		SELECT MfgNo FROM Mfg WHERE MfgDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE Mfg SET isLocked=TRUE WHERE MfgNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update Cons : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	/** update FCN **/
	OPEN _Trans FOR
		SELECT FcnNo FROM FCN WHERE FcnDate BETWEEN _StartDate AND _EndDate;
	LOOP
		FETCH _Trans INTO _TransNo;
		EXIT WHEN NOT FOUND;
		UPDATE FCN SET isLocked=TRUE WHERE FcnNo=_TransNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Gagal Update FCN : '||_TransNo;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE _Trans;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fYTD (
	_accno		VARCHAR,
	_enddate	DATE
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_LastMonthBal	NUMERIC(19,4);
	_YesterdayBal	NUMERIC(19,4);
	_Month		SMALLINT;
	_PYear		INTEGER;
	_DefaultDK	VARCHAR(1);
BEGIN
	_Month:=DATE_part('month', _EndDate);
	_PYear:=DATE_part('year', _EndDate);

	IF (_Month=1) THEN
		SELECT INTO _LastMonthBal accBal01 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=2) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=3) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=4) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=5) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04+accBal05 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=6) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=7) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=8) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=9) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08+accBal09
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=10) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08+accBal09+accBal10
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=11) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08+accBal09+accBal10+accBal11
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=12) THEN
		SELECT INTO _LastMonthBal accBal01+accBal02+accBal03+accBal04+accBal05+accBal06
		+accBal07+accBal08+accBal09+accBal10+accBal11+accBal12
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	END IF;

	IF (_LastMonthBal IS NULL) THEN
		_LastMonthBal:=0;
	END IF; 

	RETURN _LastMonthBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fMTD (
	_accno		VARCHAR,
	_enddate	DATE
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_LastMonthBal	NUMERIC(19,4);
	_YesterdayBal	NUMERIC(19,4);
	_Month		SMALLINT;
	_PYear		INTEGER;
	_DefaultDK	VARCHAR(1);
BEGIN
	_Month:=date_part('month', _EndDate);
	_PYear:=date_part('year', _EndDate);

	IF (_Month=1) THEN
		SELECT INTO _LastMonthBal accBal01 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=2) THEN
		SELECT INTO _LastMonthBal accBal02 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=3) THEN
		SELECT INTO _LastMonthBal accBal03 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=4) THEN
		SELECT INTO _LastMonthBal accBal04 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=5) THEN
		SELECT INTO _LastMonthBal accBal05 
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=6) THEN
		SELECT INTO _LastMonthBal accBal06
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=7) THEN
		SELECT INTO _LastMonthBal accBal07
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=8) THEN
		SELECT INTO _LastMonthBal accBal08
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=9) THEN
		SELECT INTO _LastMonthBal accBal09
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=10) THEN
		SELECT INTO _LastMonthBal accBal10
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=11) THEN
		SELECT INTO _LastMonthBal accBal11
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	ELSIF (_Month=12) THEN
		SELECT INTO _LastMonthBal accBal12
		FROM accBal WHERE accNo=_accNo AND PYear=_PYear;
	END IF;

	IF (_LastMonthBal IS NULL) THEN
		_LastMonthBal:=0;
	END IF; 
	RETURN _LastMonthBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function fPendingPO()
RETURNS SETOF TPendingPO AS
$BODY$
DECLARE	
	_retval			TPendingPO%ROWTYPE;
	_ItemID			VARCHAR;
	_ItemDesc		VARCHAR;
	_ItGrpID		VARCHAR;
	_PReqQty		NUMERIC(19,4);
	_PReqNo			VARCHAR;
	_PReqDate		DATE;
	_PONo			VARCHAR;
	_PODate			DATE;
	_PODQty			NUMERIC(19,4);
	_ReqDate		DATE;
	_PRcvNo			VARCHAR;
	_PrcvDate		DATE;
	_PrcvQty		NUMERIC(19,4);
	_PrcvNoConcate		VARCHAR;
	_PrcvDateConcate	VARCHAR;
	_QtyPrcvTotal		NUMERIC(19,4);
	_PONoTamp		VARCHAR;
	_PODateTamp		DATE;
	_PODQtyTamp		NUMERIC(19,4);
	_ReqDateTamp		DATE;
	cPReq			REFCURSOR;
	cPO			REFCURSOR;
	cPRcv			REFCURSOR;
BEGIN
	OPEN cPReq FOR
		SELECT preqd.itemid,preqd.itemdesc,qty,preqd.preqno,preqdate,itgrp.itgrpid 
		FROM preqd 
		JOIN preq ON preq.preqno=preqd.preqno 
		JOIN item ON preqd.itemid=item.itemid 
		LEFT JOIN itgrp ON item.itgrpid=itgrp.itgrpid;
	LOOP
		FETCH cPReq INTO _ItemID,_ItemDesc,_PReqQty,_PReqNo,_PReqDate,_ItGrpID;
		EXIT WHEN NOT FOUND;

		_PrcvNoConcate:=NULL;
		_PrcvDateConcate:=NULL;
		_QtyPrcvTotal:=0;

		_PONoTamp :=NULL;
		_PODateTamp:=NULL;
		_PODQtyTamp:=0;	
		_ReqDateTamp:=NULL;

		OPEN cPO FOR
			SELECT po.pono,podate,pod.qty,pod.reqdate
			FROM pod 
			JOIN po ON po.pono=pod.pono 
			LEFT JOIN preqpo ON preqpo.pono=po.pono
			LEFT JOIN preq ON preq.preqno=preqpo.preqno
			LEFT JOIN preqd ON preq.preqno=preqd.preqno AND preqd.itemid=pod.itemid
			WHERE preqpo.preqno=_PReqNo
			AND pod.itemid=_ItemID;
		LOOP
			FETCH cPO INTO _PONo,_PODate,_PODQty,_ReqDate;
			EXIT WHEN NOT FOUND;

			_PONoTamp :=_PONo;
			_PODateTamp:=_PODate;
			_PODQtyTamp:=_PODQty;	
			_ReqDateTamp:=_ReqDate;

			OPEN cPRcv FOR
				SELECT prcv.prcvno,prcvdate,prcvd.qty
				FROM prcvd 
				JOIN prcv ON prcv.prcvno=prcvd.prcvno
				LEFT JOIN poprcv ON poprcv.prcvno=prcv.prcvno
				LEFT JOIN po ON po.pono=poprcv.pono
				LEFT JOIN pod ON pod.pono=po.pono AND pod.itemid=prcvd.itemid
				WHERE poprcv.pono=_PONo AND prcvd.itemid=_ItemID;
			LOOP
				FETCH cPRcv INTO _PRcvNo,_PrcvDate,_PrcvQty;
				EXIT WHEN NOT FOUND;

				IF(_PrcvNoConcate IS NOT NULL) THEN
					_PrcvNoConcate:=_PrcvNoConcate||','||_PRcvNo;
				ELSE
					_PrcvNoConcate:=_PRcvNo;
				END IF;

				IF(_PrcvDateConcate IS NOT NULL) THEN
					_PrcvDateConcate:=_PrcvDateConcate||','||CAST(_PRcvDate as VARCHAR);
				ELSE
					_PrcvDateConcate:=CAST(_PRcvDate as varchar);
				END IF;

				_QtyPrcvTotal:=_QtyPrcvTotal+_PrcvQty;
			END LOOP;
			CLOSE cPRcv;
		END LOOP;
		CLOSE cPO;

		_retval.itemid:=_ItemID;
		_retval.itemdesc:=_ItemDesc;
		_retval.itgrpid:=_ItGrpID;
		_retval.preqno:=_PreqNo;
		_retval.preqdate:=_PReqDate;
		_retval.preqqty:=_PreqQty;
		_retval.pono:=_PONoTamp;
		_retval.podate:=_PODateTamp;
		_retval.podqty:=_PODQtyTamp;
		_retval.prcvno:=_PrcvNoConcate;
		_retval.prcvdatestring:=_PrcvDateConcate;
		_retval.prcvdqty:=_QtyPrcvTotal;
		_retval.reqdate:=_ReqDateTamp;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cPReq;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function fRekapDeoPerItem()
RETURNS SETOF TRekapDeoPerItem AS
$BODY$
DECLARE
	cCust		REFCURSOR;
	cDeoD		REFCURSOR;
	_branchid	VARCHAR;
	_branchname	VARCHAR;
	_custid		VARCHAR;
	_bpname		VARCHAR;
	_itgrpid	VARCHAR;
	_itgrpname	VARCHAR;
	_itemid		VARCHAR;
	_itemdesc	VARCHAR;
	_qty		VARCHAR;
	_retval		TRekapDeoPerItem%ROWTYPE;
	_crcid		VARCHAR;
	_deptid		VARCHAR;
	_prjid		VARCHAR;
	_deodnote	VARCHAR;
BEGIN
	OPEN cCust FOR
		SELECT DISTINCT deo.branchid,branchname,deo.custid,bpname
		FROM deo 
		JOIN bp ON bp.bpid=deo.custid
		LEFT JOIN branch ON branch.branchid=deo.branchid;
	LOOP
		FETCH cCust INTO _branchid,_branchname,_custid,_bpname;
		EXIT WHEN NOT FOUND;

		OPEN cDeoD FOR
			SELECT d.crcid,i.itgrpid, it.itgrpname, dd.itemid, dd.itemdesc, 
			fQty1Desc(dd.itemid, SUM(fQtyZToQty1(dd.itemid,dd.qty,unit))) AS qty,deptid,prjid,deodnote 
			FROM deod dd 
			JOIN deo d ON d.deono=dd.deono
			JOIN item i ON i.itemid=dd.itemid 
			LEFT JOIN itgrp it ON it.itgrpid=i.itgrpid
			WHERE d.custid=_custid
			GROUP BY d.crcid,i.itgrpid,it.itgrpname,dd.itemid,dd.itemdesc,deptid,prjid,deodnote;
		LOOP
			FETCH cDeoD INTO _crcid,_itgrpid,_itgrpname,_itemid,_itemdesc,_qty,_deptid,_prjid,_deodnote;
			EXIT WHEN NOT FOUND;

			_retval.itgrpid:=_itgrpid;
			_retval.itgrpname:=_itgrpname;
			_retval.itemid:=_itemid;
			_retval.itemdesc:=_itemdesc;
			_retval.qty:=_qty;
			_retval.custid:=_custid;
			_retval.bpname:=_bpname;
			_retval.branchid:=_branchid;
			_retval.branchname:=_branchname;
			_retval.crcid:=_crcid;
			_retval.deptid:=_deptid;
			_retval.prjid:=_prjid;
			_retval.deodnote:=_deodnote;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cDeoD;
	END LOOP;
	CLOSE cCust;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBukuBesar1 (
	_startdate	DATE,
	_enddate	DATE,
	_branchid	VARCHAR,
	_deptid		VARCHAR,
	_prjid		VARCHAR
) RETURNS SETOF taccrpt1 AS
$BODY$
DECLARE
	_retval		taccrpt1%ROWTYPE;
	_JnlNo		VARCHAR(15);
	_LgrDate	DATE;
	_accName	VARCHAR;
	_LgrNote	TEXT;
	_DK		VARCHAR(1);
	_accBal		NUMERIC(19,4);
	_Amount		NUMERIC(19,4);
	_Debet		NUMERIC(19,4);
	_Kredit		NUMERIC(19,4);
	_Month		SMALLINT;
	_PYear		INTEGER;
	_LastMonthBal	NUMERIC(19,4);
	_YesterdayBal	NUMERIC(19,4);
	cLgr		REFCURSOR;
	cAcc		REFCURSOR;
	_accno		VARCHAR;
	_accdk		VARCHAR;
	_saldo		NUMERIC;
BEGIN
	_Month:=DATE_PART('month', _StartDate);
	_PYear:=DATE_PART('year', _StartDate);

	OPEN cAcc FOR
		SELECT accno, accname, accdk FROM acc WHERE accno<>'*' ORDER BY accno;
	LOOP
		FETCH cAcc INTO _accno,_accname,_accdk;
		EXIT WHEN NOT FOUND;

		IF (_StartDate=fBegOfMonth(_StartDate)) THEN
			SELECT INTO _Saldo * FROM COALESCE(fAccBal_StartMonth(_AccNo, _StartDate, _BranchID, _DeptID, _PrjID), 0);
		ELSE
			SELECT INTO _Saldo * FROM COALESCE(fAccBal_StartDate(_accno, _startdate-1,_branchid,_deptid,_prjid),0);
		END IF;

		_retval.jnlno:=NULL;
		_retval.lgrdate:=NULL;
		_retval.accname:=_accname;
		_retval.accno:=_accno;
		_retval.lgrnote:='Saldo Awal';
		_retval.debet:=NULL;
		_retval.kredit:=NULL;
		_retval.accbal:=_saldo;

		RETURN NEXT _retval;

		OPEN cLgr FOR
			SELECT jnld.JnlNo, JnlDate, DK, lgrNote, jnldamt
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN lgr ON lgr.jnlno=jnl.jnlno AND lgr.jnldno=jnld.jnldno
			WHERE jnld.accNo=_accNo AND JnlDate>=_StartDate
			AND JnlDate<=fDate2359(_EndDate) 
			AND (_branchid IS NULL OR jnl.branchid=_branchid) 
			AND (_deptid IS NULL OR jnld.deptid=_deptid) 
			AND (_prjid IS NULL OR jnld.prjid=_prjid)
			ORDER BY JnlDate,JnlNo,jnld.Jnldno;
		LOOP
			FETCH cLgr INTO _JnlNo, _LgrDate, _DK, _LgrNote, _Amount;
			EXIT WHEN NOT FOUND;

			SELECT INTO _accName accName FROM Acc WHERE accNo=_accNo;
			IF (_DK='D') THEN
				_Debet:=_Amount;
				_Kredit:=NULL;
			ELSE
				_Debet:=NULL;
				_Kredit:=_Amount;
			END IF;

			IF (_DK=_accdk) THEN
				_saldo=_saldo+_Amount;
			ELSE
				_saldo=_saldo-_Amount;
			END IF;

			_retval.jnlno:=_JnlNo;
			_retval.lgrdate:=_LgrDate;
			_retval.accname:=_accName;
			_retval.accno:=_accno;
			_retval.lgrnote:=_LgrNote;
			_retval.debet:=_Debet;
			_retval.kredit:=_Kredit;
			_retval.accbal:=_saldo;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cLgr;
	END LOOP;
	CLOSE cAcc;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fTrialBalanced (
	_StartDate	DATE,
	_EndDate	DATE,
	_BranchId	VARCHAR,
	_DeptId		VARCHAR,
	_PrjId		VARCHAR
) RETURNS SETOF tTrialBalanced AS
$BODY$
DECLARE
	cAcc			REFCURSOR;
	_retval			tTrialBalanced%ROWTYPE;

	_AccNo			VARCHAR;
	_AccName		VARCHAR;
	_AccDK			VARCHAR;
	_SaldoAwal		NUMERIC;
	_SaldoAkhir		NUMERIC;
	_SaldoTengah		NUMERIC;
	_SaldoAwalD		NUMERIC;
	_SaldoAwalK		NUMERIC;
	_SaldoAkhirD		NUMERIC;
	_SaldoAkhirk		NUMERIC;
	_SaldoTengahD		NUMERIC;
	_SaldoTengahK		NUMERIC;
	_JumD			NUMERIC;
	_JumK			NUMERIC;
	_SaldoAwalVD		NUMERIC;
	_SaldoAwalVK		NUMERIC;
	_PenambahAwalD		NUMERIC;
	_PenambahAwalK		NUMERIC;
	_PenambahTengahD	NUMERIC;
	_PenambahTengahK	NUMERIC;
	_TotalAwalD		NUMERIC;
	_TotalAwalK		NUMERIC;
	_TotalTengahD		NUMERIC;
	_TotalTengahK		NUMERIC;
	_TotalAkhirD		NUMERIC;
	_TotalAkhirK		NUMERIC;
	_acchist		VARCHAR;
	_JumMutasiD		NUMERIC;
	_JumMutasiK		NUMERIC;
	_JumPenyesuaianD	NUMERIC;
	_JumPenyesuaianK	NUMERIC;
	_SaldoMutasiD		NUMERIC;
	_SaldoMutasiK		NUMERIC;
	_SaldoPenyesuaianD	NUMERIC;
	_SaldoPenyesuaianK	NUMERIC;
	_NR			VARCHAR;
	_LabaRugiD		NUMERIC;
	_LabaRugiK		NUMERIC;
	_NeracaD		NUMERIC;
	_NeracaK		NUMERIC;
	_SaldoAfterD		NUMERIC;
	_SaldoAfterK		NUMERIC;
	_JumBegBal		NUMERIC;
	_AccLabaThnJln		VARCHAR;
	_NamaLabaThnJln		VARCHAR;
	_LabaThnJln		NUMERIC;

	_JurnalTutupTahunDate	DATE;
	_AccLabaDiTahan		VARCHAR;
BEGIN
	SELECT INTO _JurnalTutupTahunDate MAX(jnldate) 
	FROM jnl 
	WHERE reftype='CLSY' AND jnldate>_StartDate;

	_AccLabaDiTahan:=fAccDefault('LABATAHAN');
	_AccLabaThnJln:=fAccDefault('LABATHNJLN');

	OPEN cAcc FOR
		SELECT accno, accname, accdk, nr 
		FROM acc 
		JOIN acctype1 on acc.acctype=acctype1.acctype 
		WHERE accno<>'*'
		ORDER BY accno;
	LOOP
		FETCH cAcc INTO _AccNo, _AccName, _AccDK, _NR;
		EXIT WHEN NOT FOUND;

		SELECT INTO _SaldoAwal * FROM fAccBal_StartDate(_AccNo,_StartDate-1,_BranchId,_DeptId,_PrjId);
		SELECT INTO _SaldoAkhir * FROM fAccBal_StartDate(_AccNo,_EndDate,_BranchId,_DeptId,_PrjId);
		_SaldoAwal:=COALESCE(_SaldoAwal,0);
		_SaldoAkhir:=COALESCE(_SaldoAkhir,0);

		IF (_AccNo=_AccLabaDiTahan AND _SaldoAkhir<>0) THEN
			IF (_JurnalTutupTahunDate IS NOT NULL) THEN
				IF (_StartDate<_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
					_SaldoAkhir:=0;
				END IF;
			END IF;
		ELSIF (_AccNo=_AccLabaThnJln) THEN
			_SaldoAwal:=0;
			IF (_JurnalTutupTahunDate IS NOT NULL AND _StartDate>=_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
				SELECT INTO _SaldoAkhir lababersih 
				FROM fProfitLoss(_JurnalTutupTahunDate, _EndDate, _BranchID, _DeptID, _PrjID);
			ELSE
				SELECT INTO _SaldoAkhir lababersih 
				FROM fProfitLoss(_StartDate, _EndDate, _BranchID, _DeptID, _PrjID);
			END IF;
		END IF;

		/**Nyari yang tipenya begbal*/
		SELECT INTO _JumBegBal COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0) 
		FROM jnld 
		JOIN jnl ON jnld.jnlno=jnl.jnlno 
		JOIN acc ON acc.accno=jnld.accno
		WHERE jnldate>=_StartDate AND jnldate<=_EndDate AND acc.accno=_AccNo 
		AND jnltype='BEG' AND (_BranchId IS NULL OR jnl.branchid=_BranchId) 
		AND (_DeptId IS NULL OR jnld.deptid=_DeptId) 
		AND (_PrjId IS NULL OR jnld.prjid=_PrjId);
		IF (_AccNo=_AccLabaDiTahan AND _JumBegBal<>0) THEN
			IF (_JurnalTutupTahunDate IS NOT NULL) THEN
				IF (_StartDate<_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
					_JumBegBal:=0;
				END IF;
			END IF;
		ELSIF (_AccNo=_AccLabaThnJln) THEN
			--_JumBegBal:=0;
		END IF;

		/*Tipe Begbal tampilnya di saldo awal...*/
		SELECT INTO _JumMutasiD COALESCE(SUM(jnldamt), 0) 
		FROM jnld 
		JOIN jnl ON jnld.jnlno=jnl.jnlno 
		WHERE jnldate>=_StartDate AND jnldate<=_EndDate AND accno=_AccNo AND dk='D' 
		and (jnltype<>'CADJ' AND jnltype<>'SADJ' AND jnltype<>'BEG') AND (_BranchId IS NULL OR jnl.branchid=_BranchId) 
		AND (_DeptId IS NULL OR jnld.deptid=_DeptId) 
		AND (_PrjId IS NULL OR jnld.prjid=_PrjId);
		IF (_AccNo=_AccLabaDiTahan AND _JumMutasiD<>0) THEN
			IF (_JurnalTutupTahunDate IS NOT NULL) THEN
				IF (_StartDate<_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
					_JumMutasiD:=0;
				END IF;
			END IF;
		ELSIF (_AccNo=_AccLabaThnJln) THEN
			--_JumMutasiD:=0;
		END IF;

		SELECT INTO _JumMutasiK COALESCE(SUM(jnldamt), 0) 
		FROM jnld 
		JOIN jnl ON jnld.jnlno=jnl.jnlno 
		WHERE jnldate>=_StartDate AND jnldate<=_EndDate AND accno=_AccNo AND dk='K'
		and (jnltype<>'CADJ' AND jnltype<>'SADJ' AND jnltype<>'BEG') AND (_BranchId IS NULL OR jnl.branchid=_BranchId) 
		AND (_DeptId IS NULL OR jnld.deptid=_DeptId) 
		AND (_PrjId IS NULL OR jnld.prjid=_PrjId);
		IF (_AccNo=_AccLabaDiTahan AND _JumMutasiK<>0) THEN
			IF (_JurnalTutupTahunDate IS NOT NULL) THEN
				IF (_StartDate<_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
					_JumMutasiK:=0;
				END IF;
			END IF;
		ELSIF (_AccNo=_AccLabaThnJln) THEN
			--_JumMutasiK:=0;
		END IF;

		SELECT INTO _JumPenyesuaianD COALESCE(SUM(jnldamt), 0) 
		FROM jnld 
		JOIN jnl ON jnld.jnlno=jnl.jnlno 
		WHERE jnldate>=_StartDate AND jnldate<=_EndDate AND accno=_AccNo AND dk='D' 
		and (jnltype='CADJ' OR jnltype='SADJ') AND (_BranchId IS NULL OR jnl.branchid=_BranchId) 
		AND (_DeptId IS NULL OR jnld.deptid=_DeptId) 
		AND (_PrjId IS NULL OR jnld.prjid=_PrjId);
		IF (_AccNo=_AccLabaDiTahan AND _JumPenyesuaianD<>0) THEN
			IF (_JurnalTutupTahunDate IS NOT NULL) THEN
				IF (_StartDate<_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
					_JumPenyesuaianD:=0;
				END IF;
			END IF;
		ELSIF (_AccNo=_AccLabaThnJln) THEN
			--_JumPenyesuaianD:=0;
		END IF;

		SELECT INTO _JumPenyesuaianK COALESCE(SUM(jnldamt), 0) 
		FROM jnld 
		JOIN jnl ON jnld.jnlno=jnl.jnlno 
		WHERE jnldate>=_StartDate AND jnldate<=_EndDate AND accno=_AccNo AND dk='K'
		and (jnltype='CADJ' OR jnltype='SADJ') AND (_BranchId IS NULL OR jnl.branchid=_BranchId) 
		AND (_DeptId IS NULL OR jnld.deptid=_DeptId) 
		AND (_PrjId IS NULL OR jnld.prjid=_PrjId);
		IF (_AccNo=_AccLabaDiTahan AND _JumPenyesuaianK<>0) THEN
			IF (_JurnalTutupTahunDate IS NOT NULL) THEN
				IF (_StartDate<_JurnalTutupTahunDate AND _EndDate>_JurnalTutupTahunDate) THEN
					_JumPenyesuaianK:=0;
				END IF;
			END IF;
		ELSIF (_AccNo=_AccLabaThnJln) THEN
			--_JumPenyesuaianK:=0;
		END IF;

		/*IF(_SaldoAwal=0) THEN
			_SaldoAwal:=NULL;
		END IF;*/

		IF(_SaldoAkhir=0) THEN
			_SaldoAkhir:=NULL;
		END IF;

		IF(_JumMutasiD=0) THEN
			_SaldoMutasiD:=NULL;
		ELSE
			_SaldoMutasiD:=_JumMutasiD;
		END IF;

		IF(_JumMutasiK=0) THEN
			_SaldoMutasiK:=NULL;
		ELSE
			_SaldoMutasiK:=_JumMutasiK;
		END IF;

		IF(_JumPenyesuaianD=0) THEN
			_SaldoPenyesuaianD:=NULL;
		ELSE
			_SaldoPenyesuaianD:=_JumPenyesuaianD;
		END IF;

		IF(_JumPenyesuaianK=0) THEN
			_SaldoPenyesuaianK:=NULL;
		ELSE
			_SaldoPenyesuaianK:=_JumPenyesuaianK;
		END IF;

		IF(_AccDK='D') THEN
			_SaldoAwalD:=_SaldoAwal+_JumBegBal;
			RAISE NOTICE '%', 'D '||_AccNo||' '||_SaldoAwal;
			IF(_SaldoAwalD=0) THEN
				_SaldoAwalD:=NULL;
			END IF;

			_SaldoAwalK:=NULL;
		ELSE
			_SaldoAwalK:=_SaldoAwal+_JumBegBal;
			RAISE NOTICE '%', 'K '||_AccNo||' '||_SaldoAwal;
			IF(_SaldoAwalK=0) THEN
				_SaldoAwalK:=NULL;
			END IF;

			_SaldoAwalD:=NULL;

		END IF;

		_SaldoAfterD:=COALESCE(_SaldoAwalD,0)+COALESCE(_SaldoMutasiD,0)+COALESCE(_SaldoPenyesuaianD,0);
		_SaldoAfterK:=COALESCE(_SaldoAwalK,0)+COALESCE(_SaldoMutasiK,0)+COALESCE(_SaldoPenyesuaianK,0);

		IF(_SaldoAfterD=0) THEN
			_SaldoAfterD=NULL;
		END IF;

		IF(_SaldoAfterK=0) THEN
			_SaldoAfterK=NULL;
		END IF;

		_LabaRugiD=NULL;
		_LabaRugiK=NULL;
		_NeracaD=NULL;
		_NeracaK=NULL;

		IF(_SaldoAkhir<>0) THEN
			IF(_NR='R') THEN
				IF(_AccDK='D') THEN
					_LabaRugiD:=_SaldoAkhir;
					_LabaRugiK:=NULL;
				ELSIF(_AccDK='K') THEN
					_LabaRugiK:=_SaldoAkhir;
					_LabaRugiD:=NULL;
				END IF;

				_NeracaD:=NULL;
				_NeracaK:=NULL;

			ELSIF(_NR='N') THEN
				IF(_AccDK='D') THEN
					_NeracaD:=_SaldoAkhir;
					_NeracaK:=NULL;
				ELSIF(_AccDK='K') THEN
					_NeracaK:=_SaldoAkhir;
					_NeracaD:=NULL;
				END IF;

				_LabaRugiD:=NULL;
				_LabaRugiK:=NULL;
			END IF;

			IF (_AccNo=_AccLabaThnJln) THEN
				IF(_AccDK='D') THEN
					_LabaRugiK:=_SaldoAkhir+_SaldoAfterK;
					_LabaRugiD:=NULL;
				ELSIF(_AccDK='K') THEN
					_LabaRugiD:=_SaldoAkhir+_SaldoAfterD;
					_LabaRugiK:=NULL;
				END IF;

				IF(_AccDK='D') THEN
					_NeracaD:=_SaldoAkhir;
					_NeracaK:=NULL;
				ELSIF(_AccDK='K') THEN
					_NeracaK:=_SaldoAkhir;
					_NeracaD:=NULL;
				END IF;
			END IF;
		END IF;

		_retval.accno:=_AccNo;
		_retval.accname:=_AccName;
		_retval.SaldoAwalD:=_SaldoAwalD;
		_retval.SaldoAwalK:=_SaldoAwalK;
		_retval.SaldoAkhirD:=_SaldoAfterD;
		_retval.SaldoAkhirK:=_SaldoAfterK;
		_retval.SaldoMutasiD:=_SaldoMutasiD;
		_retval.SaldoMutasiK:=_SaldoMutasiK;
		_retval.SaldoTransferD:=_SaldoPenyesuaianD;
		_retval.SaldoTransferK:=_SaldoPenyesuaianK;
		_retval.LabaRugiD:=_LabaRugiD;
		_retval.LabaRugiK:=_LabaRugiK;
		_retval.NeracaD:=_NeracaD;
		_retval.NeracaK:=_NeracaK;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cAcc;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE VIEW vwChk_PO_QtyDeli_Conflict AS
	SELECT pono, itemid, 
		SUM(qty) AS qtyorder, 
		SUM(qtydelivered) AS qtysodelivered, 
		(SELECT SUM(qty) FROM prcvd WHERE pono=pod.pono AND prcvd.itemid=pod.itemid) AS qtyactualdelivered
	FROM pod 
	GROUP BY pono, itemid
	HAVING SUM(qtydelivered)<>(SELECT SUM(qty) FROM prcvd WHERE pono=pod.pono AND prcvd.itemid=pod.itemid);




CREATE OR REPLACE VIEW vwChk_SO_QtyDeli_Conflict AS
	SELECT sono, itemid, 
		SUM(qty) AS qtyorder, 
		SUM(qtydelivered) AS qtysodelivered, 
		(SELECT SUM(qty) FROM delid WHERE sono=sod.sono AND delid.itemid=sod.itemid) AS qtyactualdelivered
	FROM sod 
	GROUP BY sono, itemid
	HAVING SUM(qtydelivered)<>(SELECT SUM(qty) FROM delid WHERE sono=sod.sono AND delid.itemid=sod.itemid);




CREATE OR REPLACE VIEW vwChk_StockAD_QtyTotal AS
	SELECT itemid, whid, fValidatePID(pid) AS pid, 
	SUM(fQtyZToQty1(itemid, qty, unit)) AS qtytotal 
	FROM stockad
	GROUP BY itemid, whid, fValidatePID(pid);




CREATE OR REPLACE VIEW vwChk_Stock_Conflict AS
	SELECT s.itemid, s.whid, s.pid, s.qty, v.qtytotal
	FROM stock s 
	JOIN vwChk_StockAD_QtyTotal v ON s.itemid=v.itemid AND s.whid=v.whid AND s.pid=v.pid
	WHERE s.qty<>v.qtytotal;




CREATE OR REPLACE FUNCTION spSale_Void_ByDate (
	_SaleDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_count		INT;

	csale		REFCURSOR;
	_saleno		VARCHAR(15);
BEGIN
	_count:=0;
	OPEN cSale FOR
		SELECT SaleNo FROM Sale WHERE SaleDate=_SaleDate;
	LOOP
		FETCH cSale INTO _SaleNo;
		EXIT WHEN NOT FOUND;


		RAISE NOTICE 'Voiding %', _SaleNo;
		SELECT INTO _spret * FROM spSale_Void(_SaleNo);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_count:=_count+1;
		RAISE NOTICE ' Status : OK';
	END LOOP;
	CLOSE cSale;

	RAISE NOTICE 'Total Voided: %', _count;
	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE VIEW vwChk_Item_tanpa_StockCard AS
	SELECT v.itemid, v.whid, s.whid AS stock_whid 
	FROM (SELECT i.itemid, w.whid FROM item i, wh w) v
	LEFT JOIN stock s ON s.itemid=v.itemid AND s.whid=v.whid
	WHERE s.whid IS NULL;




CREATE OR REPLACE FUNCTION fNeracaFix (
	_StartDate		DATE,
	_EndDate		DATE,
	_StartDateOld		DATE,
	_EndDateOld		DATE,
	_branchid		VARCHAR,
	_deptid			VARCHAR,
	_prjid			VARCHAR
) RETURNS SETOF tNeraca1 AS
$BODY$
DECLARE
	_retval			tNeraca1%ROWTYPE;
	cAcc			REFCURSOR;
	cAccType1		REFCURSOR;
	_acctype1		VARCHAR;
	_acctypedesc1		VARCHAR;
	_accno			VARCHAR;
	_accname		VARCHAR;
	_acctype2desc		VARCHAR;
	_acctype2		VARCHAR;
	_acctype1bdesc		VARCHAR;
	_accbal			NUMERIC(19,4);
	_accbalOld		NUMERIC(19,4);
	_StartDatePer		DATE;
	inc			INTEGER;
	_bulanPer		INTEGER;
	_tahunPer		INTEGER;
	_bulan			INTEGER;
	_tahun			INTEGER;
	_StartDatePrev		DATE;
	_EndDatePrev		DATE;
	_PerNo			INT;
	_accdk			VARCHAR;
	_StartDatePerNow	DATE;
	_StartDatePerOld	DATE;
	_condition		BOOLEAN;
BEGIN
	SELECT INTO _StartDatePer, _PerNo perstart, perno 
	FROM per 
	WHERE active IS true;

	SELECT INTO _bulanPer DATE_PART('month',_StartDatePer);
	SELECT INTO _tahunPer DATE_PART('year',_StartDatePer);
	SELECT INTO _bulan DATE_PART('month',_StartDate);
	SELECT INTO _tahun DATE_PART('year',_StartDate);

	SELECT INTO _StartDatePrev,_EndDatePrev perstart,perend 
	FROM per 
	WHERE perno=_PerNo-1 AND perend IS NOT NULL;

	SELECT INTO _StartDatePerNow perstart 
	FROM per 
	WHERE perid=fGetPer(_StartDate);
	IF(_StartDateOld IS NOT NULL) THEN
		SELECT INTO _StartDatePerOld perstart 
		FROM per 
		WHERE perid=fGetPer(_StartDateOld);
	END IF;

	OPEN cAccType1 FOR
		SELECT acctype,acctypedesc FROM acctype1 WHERE nr='N' ORDER BY acctype;
	LOOP
		FETCH cAccType1 INTO _acctype1,_acctypedesc1;
		EXIT WHEN NOT FOUND;

			OPEN cAcc FOR
				SELECT acc.accno,accname,acc.acctype2,acctype2desc,acctype1bdesc,accdk
				FROM acc
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype 
				LEFT JOIN acctype2 ON acctype2.acctype2=acc.acctype2
				LEFT JOIN acctype1b ON acctype1b.acctype1b=acc.acctype1b
				WHERE acctype1.acctype=_acctype1
				GROUP BY acc.acctype2,acctype2desc,acctype1bdesc,acc.accno,accname,accdk
				ORDER BY acc.accno;
			LOOP
				FETCH cAcc INTO _accno,_accname,_acctype2,_acctype2desc,_acctype1bdesc,_accdk;
				EXIT WHEN NOT FOUND;

				/*Laba thn jalan posting sendiri!!*/
				IF(_accno<>fAccDefault('LABATHNJLN')) THEN
					_accBal := fAccBal_StartDate(_Accno, _EndDate,_branchid,_deptid,_prjid);

					_condition:=false;

					IF(_StartDate IS NOT NULL AND _EndDate IS NOT NULL) THEN
						_accBalOld := fAccBal_StartDate(_Accno, _EndDateOld,_branchid,_deptid,_prjid); 
						_retval.accbalOld:=coalesce(_accbalOld,0);	
						IF(_acctype1<>'100') THEN
							_retval.accbalnonhartaold:=coalesce(_accbalOld,0);
						ELSE
							_retval.accbalnonhartaold:=0;
						END IF;
					END IF;

					IF(_accno<>fAccDefault('LABATAHAN')) THEN
						_condition:=true;
					END IF;

					IF(_accno=fAccDefault('LABATAHAN') AND _accBal<>0) THEN
						_condition:=true;
					END IF;

					IF(_StartDate IS NOT NULL AND _EndDate IS NOT NULL AND _accno=fAccDefault('LABATAHAN') AND _accBalOld<>0) THEN
						_condition:=true;
					END IF;

					IF(_condition=true) THEN 
						_retval.accno:=_accno;
						_retval.accname:=_accname;
						_retval.acctype2:=_acctype2;
						_retval.acctype2desc:=_acctype2desc;
						_retval.acctype1:=_acctype1;
						_retval.acctypedesc1:=_acctypedesc1;
						_retval.acctype1bdesc:=_acctype1bdesc;
						_retval.accbal:=coalesce(_accbal,0);
						_retval.accdk:=_accdk;
					ELSE
						_retval.accno:=NULL;
						_retval.accname:=NULL;
						_retval.acctype2desc:=NULL;
						_retval.acctype2:=NULL;
						_retval.acctype1:=NULL;
						_retval.acctypedesc1:=NULL;
						_retval.acctype1bdesc:=NULL;
						_retval.accbal:=0;
						_retval.accdk:=NULL;
					END IF;

					IF(_acctype1<>'100') THEN
						_retval.accbalnonharta:=coalesce(_accbal,0);
					ELSE
						_retval.accbalnonharta:=0;
					END IF;

					RETURN NEXT _retval;
				END IF;

				/*Posting Laba Tahun Berjalan!!*/
				IF(_accno=fAccDefault('LABATHNJLN')) THEN
					SELECT INTO _accBal lababersih FROM 
					fProfitLoss(_StartDatePerNow,_EndDate,_branchid,_deptid,_prjid);
					IF(_accBal<>0) THEN
						SELECT INTO _accname,_acctype2,_acctype2desc 
							accname,acc.acctype2,acctype2desc 
						FROM acc 
						LEFT JOIN acctype2 ON acc.acctype2=acctype2.acctype2 
						WHERE accno=fAccDefault('LABATHNJLN');

						_retval.accno:=fAccDefault('LABATHNJLN');
						_retval.accname:=_accname;
						_retval.acctype2desc:=_acctype2desc;
						_retval.acctype2:=_acctype2;
						_retval.acctype1:=_acctype1;
						_retval.acctypedesc1:=_acctypedesc1;
						_retval.acctype1bdesc:=_acctype1bdesc;
						_retval.accbal:=coalesce(_accbal,0);
						_retval.accbalnonharta:=coalesce(_accbal,0);
						SELECT INTO _retval.accdk accdk FROM acc where accno=_retval.accno;
					ELSE
						_retval.accno:=NULL;
						_retval.accname:=NULL;
						_retval.acctype2desc:=NULL;
						_retval.acctype2:=NULL;
						_retval.acctype1:=NULL;
						_retval.acctypedesc1:=NULL;
						_retval.acctype1bdesc:=NULL;
						_retval.accbal:=0;
						_retval.accbalnonharta:=0;
						_retval.accdk:=NULL;
					END IF;

					IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
						SELECT INTO _accbalOld lababersih FROM 
						fProfitLoss(_StartDatePerOld,_EndDateOld,_branchid,_deptid,_prjid);
						_retval.accbalOld:=coalesce(_accbalOld,0);
						_retval.accbalnonhartaold:=coalesce(_accbalOld,0);
					END IF;

					RETURN NEXT _retval;
				END IF;
			END LOOP;
			CLOSE cAcc; 
	END LOOP;
	CLOSE cAccType1;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAccBal_StartDate(
	_AccNo		VARCHAR, 
	_Date		DATE, 
	_BranchID	VARCHAR, 
	_DeptID		VARCHAR, 
	_PrjID		VARCHAR, 
	_CheckUp	BOOLEAN
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_LastMonthBal	NUMERIC(19,4);
	_YesterdayBal	NUMERIC(19,4);
BEGIN
	_LastMonthBal:=fAccBal_StartMonth(_AccNo, _Date, _BranchID, _DeptID, _PrjID);

	IF (_CheckUp=TRUE) THEN
		_YesterdayBal:=_LastMonthBal+COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0) 
			FROM jnld JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			WHERE jnldate>=fBegOfMonth(_Date) AND jnldate<=_Date AND (acc.accno=_AccNo OR acc.upaccno=_AccNo)
			AND (_BranchID IS NULL OR jnl.branchid=_BranchID) 
			AND (_DeptID IS NULL OR jnld.deptid=_DeptID) 
			AND (_PrjID IS NULL OR jnld.prjid=_PrjID);
	ELSE
		_YesterdayBal:=_LastMonthBal+COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0) 
			FROM jnld JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			WHERE jnldate>=fBegOfMonth(_Date) AND jnldate<=_Date AND acc.accno=_AccNo
			AND (_BranchID IS NULL OR jnl.branchid=_BranchID) 
			AND (_DeptID IS NULL OR jnld.deptid=_DeptID) 
			AND (_PrjID IS NULL OR jnld.prjid=_PrjID);
	END IF;

	RETURN _YesterdayBal;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fAccBal_StartDate(
	_AccNo		VARCHAR, 
	_Date		DATE, 
	_BranchID	VARCHAR, 
	_DeptID		VARCHAR, 
	_PrjID		VARCHAR
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	_retval:=fAccBal_StartDate(_AccNo, _Date, _BranchID, _DeptID, _PrjID, FALSE);

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION faccbal_StartMonth (
	_AccNo		VARCHAR,
	_StartDate	DATE, 
	_BranchID	VARCHAR(5),
	_DeptID		VARCHAR(5),
	_PrjID		VARCHAR(5)
) RETURNS NUMERIC AS
$BODY$
DECLARE
	_Month		INT;
	_PYear		INT;
	_LastMonthBal	NUMERIC(19,4);
	_LastYearBal	NUMERIC(19,4);
	_PID		VARCHAR;
	
	cLastYear	REFCURSOR;
	_LY_BranchId	VARCHAR;
	_LY_DeptId	VARCHAR;
	_LY_PrjId	VARCHAR;
BEGIN

	SELECT INTO _Month * FROM DATE_PART('month', CAST(_StartDate AS DATE));
	SELECT INTO _PYear * FROM DATE_PART('year', CAST(_StartDate AS DATE));

	IF (_Month=1) THEN
		SELECT INTO _LastMonthBal SUM(accbal00) 
		FROM accbal 
		WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=2) THEN
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01) 
		FROM accbal 
		WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=3) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02)
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=4) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03) 
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=5) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03+accbal04) 
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=6) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03+accbal04+accbal05) 
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=7) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03+accbal04+accbal05+accbal06)
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=8) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03+accbal04+accbal05+accbal06
		+accbal07)
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=9) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03+accbal04+accbal05+accbal06
		+accbal07+accbal08)
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=10) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03+accbal04+accbal05+accbal06
		+accbal07+accbal08+accbal09)
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=11) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03+accbal04+accbal05+accbal06
		+accbal07+accbal08+accbal09+accbal10)
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	ELSIF (_Month=12) THEN 
		SELECT INTO _LastMonthBal SUM(accbal00+accbal01+accbal02+accbal03+accbal04+accbal05+accbal06
		+accbal07+accbal08+accbal09+accbal10+accbal11)
		FROM accbal WHERE pyear=_PYear AND accno=_AccNo
		AND (_BranchID IS NULL OR branchid=_BranchID) 
		AND (_DeptID IS NULL OR deptid=_DeptID) 
		AND (_PrjID IS NULL OR prjid=_PrjID);
	END IF;

	_LastMonthBal:=COALESCE(_LastMonthBal, 0);
	OPEN cLastYear FOR
		SELECT COALESCE(accbal00, 0)+COALESCE(accbal01, 0)+COALESCE(accbal02, 0)+COALESCE(accbal03, 0)+COALESCE(accbal04, 0)+COALESCE(accbal05, 0)+COALESCE(accbal06, 0)+
			COALESCE(accbal07, 0)+COALESCE(accbal08, 0)+COALESCE(accbal09, 0)+COALESCE(accbal10, 0)+COALESCE(accbal11, 0)+COALESCE(accbal12, 0)
		FROM accbal b JOIN (
			SELECT accno, COALESCE(branchid, '') AS branchid, COALESCE(deptid, '') AS deptid, COALESCE(prjid, '') AS prjid, MAX(pyear) AS pyear
			FROM accbal WHERE accno=_AccNo
			GROUP BY accno, branchid, deptid, prjid HAVING MAX(pyear)<_PYear
		) a ON a.accno=b.accno AND a.pyear=b.pyear AND a.branchid=COALESCE(b.branchid, '') AND a.deptid=COALESCE(b.deptid, '') AND a.prjid=COALESCE(b.prjid, '');
	LOOP
		FETCH cLastYear INTO _LastYearBal;
		EXIT WHEN NOT FOUND;
		
		_LastMonthBal:=_LastMonthBal+_LastYearBal;
	END LOOP;
	CLOSE cLastYear; 

	RETURN _LastMonthBal;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fProfitLoss (
	_StartDate	DATE,
	_EndDate	DATE,
	_branchid	VARCHAR,
	_deptid		VARCHAR,
	_prjid		VARCHAR
) RETURNS SETOF tProfitLoss AS
$BODY$
DECLARE
	_retval		tProfitLoss%ROWTYPE;
	_acctype1	VARCHAR;
	_acctypedesc1	VARCHAR;
	_accno		VARCHAR;
	_accname	VARCHAR;
	_acctype2desc	VARCHAR;
	_jum400		NUMERIC(19,4);
	_jum500		NUMERIC(19,4);
	_jum600		NUMERIC(19,4);
	_jum700		NUMERIC(19,4);
	_jum800		NUMERIC(19,4);
	cAccType1	REFCURSOR;
	_LabaKotor	NUMERIC(19,4);
	_LabaOperasi	NUMERIC(19,4);
	_LabaBersih	NUMERIC(19,4);
BEGIN
	
	OPEN cAccType1 FOR
		SELECT acctype,acctypedesc 
		FROM acctype1 
		WHERE nr='R' 
		ORDER BY acctype;
	LOOP
		FETCH cAccType1 INTO _acctype1,_acctypedesc1;
		EXIT WHEN NOT FOUND;

		/*Yang CLSM(tutup bulan) gak ikut dihitung, soalnya klo dihitung ntar hasilnya 0*/
		IF(_acctype1='400') THEN
			SELECT INTO _jum400 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='400' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) 
				AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid) AND (jnltype<>'CLSM' OR jnltype IS NULL);
		END IF;

		IF(_acctype1='500') THEN
			SELECT INTO _jum500 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='500' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) 
				AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid) AND (jnltype<>'CLSM' OR jnltype IS NULL);
		END IF;

		IF(_acctype1='600') THEN
			SELECT INTO _jum600 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='600' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) 
				AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid) AND (jnltype<>'CLSM' OR jnltype IS NULL);
		END IF;

		IF(_acctype1='700') THEN
			SELECT INTO _jum700 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='700' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) 
				AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid) AND (jnltype<>'CLSM' OR jnltype IS NULL);
		END IF;

		IF(_acctype1='800') THEN
			SELECT INTO _jum800 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='800' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) 
				AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid) AND (jnltype<>'CLSM' OR jnltype IS NULL);
		END IF;
	END LOOP;
	CLOSE cAccType1;

	_LabaKotor:=_jum400-_jum500;
	_LabaOperasi:=_LabaKotor-_jum600;
	_LabaBersih:=_LabaOperasi+_jum700-_jum800;

	_retval.labakotor:=_LabaKotor;
	_retval.labaoperasi:=_LabaOperasi;
	_retval.lababersih:=_LabaBersih;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fLabaRugiFix1 (
	_StartDate	DATE,
	_EndDate	DATE,
	_StartDateOld	DATE,
	_EndDateOld	DATE,
	_branchid	VARCHAR,
	_deptid		VARCHAR,
	_prjid		VARCHAR
) RETURNS SETOF tLabaRugi1 AS
$BODY$
DECLARE
	_retval		tLabaRugi1%ROWTYPE;
	_acctype1	VARCHAR;
	_acctypedesc1	VARCHAR;
	_accno		VARCHAR;
	_accname	VARCHAR;
	_acctype2desc	VARCHAR;
	_accbal		NUMERIC(19,4);
	_jum400		NUMERIC(19,4);
	_jum500		NUMERIC(19,4);
	_jum600		NUMERIC(19,4);
	_jum700		NUMERIC(19,4);
	_jum800		NUMERIC(19,4);
	_accbalOld	NUMERIC(19,4);
	_jum400Old	NUMERIC(19,4);
	_jum500Old	NUMERIC(19,4);
	_jum600Old	NUMERIC(19,4);
	_jum700Old	NUMERIC(19,4);
	_jum800Old	NUMERIC(19,4);
	_accdk		VARCHAR; 
	cAccType1	REFCURSOR;
	cAcc		REFCURSOR;
BEGIN
	OPEN cAccType1 FOR
		SELECT acctype,acctypedesc FROM acctype1 WHERE nr='R' ORDER BY acctype;
	LOOP
		FETCH cAccType1 INTO _acctype1,_acctypedesc1;
		EXIT WHEN NOT FOUND;

		IF(_acctype1='400') THEN
			SELECT INTO _jum400 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0))
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='400' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) 
			AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);

			IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
				SELECT INTO _jum400Old 
				(COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
				FROM jnld 
				JOIN jnl ON jnl.jnlno=jnld.jnlno 
				JOIN acc ON jnld.accno=acc.accno 
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
				WHERE acctype1.acctype='400' AND 
				JnlDate>=_StartDateOld AND JnlDate<=_EndDateOld AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);
			ELSE
				_jum400Old:=0;
			END IF;
		END IF;

		IF(_acctype1='500') THEN
			SELECT INTO _jum500 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0))
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
		WHERE acctype1.acctype='500' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);

			IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
				SELECT INTO _jum500Old 
				(COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
				FROM jnld 
				JOIN jnl ON jnl.jnlno=jnld.jnlno 
				JOIN acc ON jnld.accno=acc.accno 
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
				WHERE acctype1.acctype='500' AND 
				JnlDate>=_StartDateOld AND JnlDate<=_EndDateOld AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);
			ELSE
				_jum500Old:=0;
			END IF;
		END IF;

		IF(_acctype1='600') THEN
			SELECT INTO _jum600 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='600' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);

			IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
				SELECT INTO _jum600Old 
				(COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
				FROM jnld JOIN jnl ON jnl.jnlno=jnld.jnlno 
				JOIN acc ON jnld.accno=acc.accno 
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
				WHERE acctype1.acctype='600' AND 
				JnlDate>=_StartDateOld AND JnlDate<=_EndDateOld AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);
			ELSE
				_jum600Old:=0;
			END IF;
		END IF;

		IF(_acctype1='700') THEN
			SELECT INTO _jum700 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='700' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);

			IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
				SELECT INTO _jum700Old 
				(COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
				FROM jnld JOIN jnl ON jnl.jnlno=jnld.jnlno 
				LEFT JOIN acc ON jnld.accno=acc.accno 
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
				WHERE acctype1.acctype='700' AND 
				JnlDate>=_StartDateOld AND JnlDate<=_EndDateOld AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);
			ELSE
				_jum700Old:=0;
			END IF;
		END IF;

		IF(_acctype1='800') THEN
			SELECT INTO _jum800 (COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
			FROM jnld 
			JOIN jnl ON jnl.jnlno=jnld.jnlno 
			JOIN acc ON jnld.accno=acc.accno 
			LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
			WHERE acctype1.acctype='800' AND JnlDate>=_StartDate AND JnlDate<=_EndDate AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);

			IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
				SELECT INTO _jum800Old 
				(COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
				FROM jnld 
				JOIN jnl ON jnl.jnlno=jnld.jnlno 
				JOIN acc ON jnld.accno=acc.accno 
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype
				WHERE acctype1.acctype='800' AND 
				JnlDate>=_StartDateOld AND JnlDate<=_EndDateOld AND (_branchid IS NULL OR jnl.branchid=_branchid) AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid);
			ELSE
				_jum800Old:=0;
			END IF;
		END IF;

		IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL)THEN
			OPEN cAcc FOR
				SELECT acc.accno,accname,acctype2desc,accdk
				FROM acc 
				LEFT JOIN jnld ON jnld.accno=acc.accno AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid)
				JOIN jnl ON jnl.jnlno=jnld.jnlno AND ((JnlDate>=_StartDate AND JnlDate<=_EndDate) or (JnlDate>=_StartDateOld AND JnlDate<=_EndDateOld)) AND (_branchid IS NULL OR jnl.branchid=_branchid) 
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype 
				LEFT JOIN acctype2 ON acctype2.acctype2=acc.acctype2
				WHERE acctype1.acctype=_acctype1
				GROUP BY acctype2desc,acc.accno,accname,accdk
				ORDER BY acc.accno;
		ELSE
			OPEN cAcc FOR
				SELECT acc.accno,accname,acctype2desc,accdk
				FROM acc 
				LEFT JOIN jnld ON jnld.accno=acc.accno AND (_deptid IS NULL OR jnld.deptid=_deptid) AND (_prjid IS NULL OR jnld.prjid=_prjid)
				JOIN jnl ON jnl.jnlno=jnld.jnlno AND (JnlDate>=_StartDate AND JnlDate<=_EndDate) AND (_branchid IS NULL OR jnl.branchid=_branchid)
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype 
				LEFT JOIN acctype2 ON acctype2.acctype2=acc.acctype2
				WHERE acctype1.acctype=_acctype1
				GROUP BY acctype2desc,acc.accno,accname,accdk
				ORDER BY acc.accno;
		END IF;
			LOOP
				FETCH cAcc INTO _accno,_accname,_acctype2desc,_accdk;
				EXIT WHEN NOT FOUND;

				SELECT INTO _AccBal 
				(COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
				FROM jnld 
				JOIN jnl ON jnl.jnlno=jnld.jnlno 
				JOIN acc ON jnld.accno=acc.accno 
				LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype 
				LEFT JOIN acctype2 ON acctype2.acctype2=acc.acctype2
				WHERE acctype1.acctype=_acctype1 AND acc.accno=_accno
				AND JnlDate>=_StartDate AND JnlDate<=_EndDate
				AND (_branchid IS NULL OR jnl.branchid=_branchid) 
				AND (_deptid IS NULL OR jnld.deptid=_deptid) 
				AND (_prjid IS NULL OR jnld.prjid=_prjid)
				GROUP BY acctype2desc,acc.accno,accname;

				IF(_StartDateOld IS NOT NULL AND _EndDateOld IS NOT NULL) THEN
					SELECT INTO _AccBalOld 
					(COALESCE(SUM(CASE WHEN dk=accdk THEN jnldamt ELSE -1*jnldamt END), 0)) 
					FROM jnld 
					JOIN jnl ON jnl.jnlno=jnld.jnlno 
					JOIN acc ON jnld.accno=acc.accno 
					LEFT JOIN acctype1 ON acctype1.acctype=acc.acctype 
					LEFT JOIN acctype2 ON acctype2.acctype2=acc.acctype2
					WHERE acctype1.acctype=_acctype1 AND acc.accno=_accno
					AND JnlDate>=_StartDateOld AND JnlDate<=_EndDateOld
					AND (_branchid IS NULL OR jnl.branchid=_branchid) 
					AND (_deptid IS NULL OR jnld.deptid=_deptid) 
					AND (_prjid IS NULL OR jnld.prjid=_prjid)
					GROUP BY acctype2desc,acc.accno,accname;
				END IF;

				_retval.accno:=_accno;
				_retval.accname:=_accname;
				_retval.acctype2desc:=_acctype2desc;
				_retval.acctype1:=_acctype1;
				_retval.acctypedesc1:=_acctypedesc1;
				_retval.jum400:=COALESCE(_jum400,0);
				_retval.jum500:=COALESCE(_jum500,0);
				_retval.jum600:=COALESCE(_jum600,0);
				_retval.jum700:=COALESCE(_jum700,0);
				_retval.jum800:=COALESCE(_jum800,0);
				_retval.jum400Old:=COALESCE(_jum400Old,0);
				_retval.jum500Old:=COALESCE(_jum500Old,0);
				_retval.jum600Old:=COALESCE(_jum600Old,0);
				_retval.jum700Old:=COALESCE(_jum700Old,0);
				_retval.jum800Old:=COALESCE(_jum800Old,0);
				_retval.accbal:=COALESCE(_accbal,0);
				_retval.accbalold:=COALESCE(_accbalold,0);
				RETURN NEXT _retval;
			END LOOP;
			CLOSE cAcc; 
	END LOOP;
	CLOSE cAccType1;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCashFlow (
	_StartDate	DATE,
	_EndDate	DATE,
	_BranchID	VARCHAR(5),
	_DeptID		VARCHAR(5),
	_PrjID		VARCHAR(5)
) RETURNS SETOF tCashFlow AS
$BODY$
DECLARE
	cAcc		REFCURSOR;
	cLgr		REFCURSOR;
	_retval		tCashFlow%ROWTYPE;

	_accno		VARCHAR;
	_accname	VARCHAR;
	_dk		VARCHAR;
	_awalTotal	NUMERIC;
	_akhirTotal	NUMERIC;
	_selisihTotal	NUMERIC;
	_awal		NUMERIC;
	_akhir		NUMERIC;
	_selisih	NUMERIC;
	_jnlno		VARCHAR;
	_reftype	VARCHAR;
	_jnldamt	NUMERIC;
	_jnldate	DATE;
	_jnlnote	VARCHAR;
	_OprPos		NUMERIC(19,4);
	_OprNeg		NUMERIC(19,4);
	_InvPos		NUMERIC(19,4);
	_InvNeg		NUMERIC(19,4);
	_DanPos		NUMERIC(19,4);
	_DanNeg		NUMERIC(19,4);
	_AccType	VARCHAR;
	_AccType2	VARCHAR;
BEGIN
	_OprPos:=0;
	_OprNeg:=0;
	_InvPos:=0;
	_InvNeg:=0;
	_DanPos:=0;
	_DanNeg:=0;

	OPEN cLgr FOR
		SELECT lgr.jnlno 
		FROM lgr 
		JOIN acc ON lgr.accno=acc.accno 
		WHERE lgrdate>=_StartDate AND lgrdate<=_EndDate
		AND acctype2='110' 
		AND (_BranchID IS NULL OR BranchID=_BranchID) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID)
		GROUP BY lgr.jnlno;
	LOOP
		FETCH cLgr INTO _JnlNo;
		EXIT WHEN NOT FOUND;

		OPEN cAcc FOR
			SELECT d.accno,jnldamt,dk 
			FROM jnld d 
			JOIN acc ON d.accno=acc.accno 
			JOIN jnl ON d.jnlno=jnl.jnlno
			WHERE d.jnlno=_JnlNo and acctype2<>'110' 
			AND (_BranchID IS NULL OR BranchID=_BranchID) 
			AND (_DeptID IS NULL OR DeptID=_DeptID) 
			AND (_PrjID IS NULL OR PrjID=_PrjID);
		LOOP
			FETCH cAcc INTO _accno, _jnldamt, _dk;
			EXIT WHEN NOT FOUND;

			SELECT INTO _AccType,_AccType2 acctype,acctype2 FROM acc WHERE AccNo=_AccNo;

			IF (_AccType='300') THEN
				IF(_dk='K') THEN
					_DanPos:=_DanPos+_jnldamt;
				ELSIF(_dk='D') THEN
					_DanNeg=_DanNeg+_jnldamt;
				END IF;
			ELSIF (_AccType2='107') THEN
				IF(_dk='K') THEN
					_InvPos:=_InvPos+_jnldamt;
				ELSIF(_dk='D') THEN
					_InvNeg=_InvNeg+_jnldamt;
				END IF;
			ELSE
				IF(_dk='K') THEN
					_OprPos:=_OprPos+_jnldamt;
				ELSIF(_dk='D') THEN
					_OprNeg=_OprNeg+_jnldamt;
				END IF;
			END IF;
		END LOOP;
		CLOSE cAcc;
	END LOOP;
	CLOSE cLgr;


	/*Buat aktifitas Operasi*/
	_retval.keterangan := 'Arus kas dari aktivitas operasi :';
	_retval.amtKiri:=NULL;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='          Kas yang diterima dari pelanggan';
	_retval.amtKiri:=_OprPos;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='          Dikurangi pembayaran kas untuk beban dan pembayaran untuk kreditor';
	_retval.amtKiri:=_OprNeg;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='Arus kas bersih dari aktivitas operasi :';
	_retval.amtKiri:=NULL;
	_retval.amtKanan:=_OprPos-_OprNeg;
	RETURN NEXT _retval;


	/*Buat aktifitas investasi*/
	_retval.keterangan := 'Arus kas dari aktivitas investasi :';
	_retval.amtKiri:=NULL;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='          Kas yang diterima dari investasi';
	_retval.amtKiri:=_InvPos;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='          Pembayaran kas untuk investasi';
	_retval.amtKiri:=_InvNeg;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='Arus kas bersih dari aktivitas investasi :';
	_retval.amtKiri:=NULL;
	_retval.amtKanan:=_InvPos-_InvNeg;
	RETURN NEXT _retval;

	/*Buat aktifitas pendanaan*/
	_retval.keterangan := 'Arus kas dari aktivitas pendanaan :';
	_retval.amtKiri:=NULL;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='          Kas yang diterima sebagai investasi pemilik';
	_retval.amtKiri:=_DanPos;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='          Dikurangi penarikan kas oleh pemilik';
	_retval.amtKiri:=_DanNeg;
	_retval.amtKanan:=NULL;
	RETURN NEXT _retval;

	_retval.keterangan:='Arus kas bersih dari aktivitas pendanaan :';
	_retval.amtKiri:=NULL;
	_retval.amtKanan:=_DanPos-_DanNeg;
	RETURN NEXT _retval;

	/*_retval.keterangan:='Arus kas bersih dan saldo kas '||_EndDate||' :';
	_retval.amtKiri:=NULL;
	_retval.amtKanan:=(_OprPos-_OprNeg)+(_InvPos-_InvNeg)+(_DanPos-_DanNeg);
	RETURN NEXT _retval;*/
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spEndMonth(_StartDate date,_EndDate date)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_selisih	Numeric(19,4);
	_ExcRateAsli	Numeric(19,4);	
	cSale		refcursor;
	cPurc		refcursor;
	cCrc		refcursor;
	_totalPembelian Numeric(19,4);
	_totalPenjualan Numeric(19,4);
	_AccAsset	varchar;
	_JnlNote	varchar;
	_JnlNo		varchar;
	_totalSelisih	numeric;
	_saledk 	varchar;
	_purcdk		varchar;
	_selisihSale	varchar;
	_selisihPurc	varchar;
	_count		int;
	_CrcID		varchar;
	_ExcRate	decimal;
	_FisRate	numeric(19,4);

BEGIN

	OPEN cCrc FOR
		SELECT crcid,excrate,fisrate FROM crc where isdefault=false;
	LOOP
		FETCH cCrc INTO _CrcID,_ExcRate,_FisRate;
		EXIT WHEN NOT FOUND;
	/*SISI PENJUALAN*/
	
	SELECT INTO _count count(saleno) FROM sale WHERE crcid=_CrcID and saledate>=_StartDate and saledate<=_EndDate
	and termtype='R' and paystatus<>'F';

	IF(_count>0) THEN
		_totalPenjualan:=0;
		_JnlNote:= 'Currency Revaluation Piutang Usaha:';
		_JnlNo:=fTransNoGet('JURNAL');
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'GJ', _EndDate,'GJ',null,
			_JnlNote,true,true,cast(0 as numeric(19,4)),
			null,null,null,null,
			null,null,null,null							
			);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		OPEN cSale FOR
			SELECT sum((total-paidamt)*(excrate-_ExcRate)),fBpAcc(CustID, _CrcID, 'AR')
			FROM sale
			WHERE crcid=_CrcID AND saledate>=_StartDate AND saledate<=_EndDate 
			and termtype='R' and paystatus<>'F'
			GROUP BY fBpAcc(CustID, _CrcID, 'AR');
			LOOP
				FETCH cSale INTO _totalPenjualan,_AccAsset;
				EXIT WHEN NOT FOUND;
				
				IF(_totalPenjualan>0) THEN
					_saledk:='K';
					_selisihSale:='D';
				ELSE
					_totalPenjualan:=_totalPenjualan*(-1);
					_saledk='D';
					_selisihSale:='K';
				END IF;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, _AccAsset, _saledk, _totalPenjualan, null, null);

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fAccDefault('ACCPROFIT'), _selisihSale, _totalPenjualan, null, null);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
		CLOSE cSale;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;
	
	/*SISI PEMBELIAN*/
	SELECT INTO _count count(purcno) FROM purc WHERE crcid=_CrcID and purcdate>=_StartDate and purcdate<=_EndDate
	and termtype='R' and paystatus<>'F';

	IF(_count>0) THEN
		_totalPembelian:=0;
		_JnlNote:= 'Currency Revaluation Hutang Usaha:';
		_JnlNo:=fTransNoGet('JURNAL');
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'GJ', _EndDate,'GJ',null,
			_JnlNote,true,true,cast(0 as numeric(19,4)),
			null,null,null,null,
			null,null,null,null, _CrcID, _ExcRate, _FisRate							
			);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		OPEN cPurc FOR
			SELECT sum((total-paidamt)*(excrate-_ExcRate)),fBpAcc(VendorID, _CrcID, 'AP')
			FROM purc
			WHERE crcid=_CrcID AND purcdate>=_StartDate AND purcdate<=_EndDate 
			and termtype='R' and paystatus<>'F'
			GROUP BY fBpAcc(VendorID, _CrcID, 'AP');
			LOOP
				FETCH cPurc INTO _totalPembelian,_AccAsset;
				EXIT WHEN NOT FOUND;
				
				IF(_totalPembelian>0) THEN
					_purcdk:='D';
					_selisihPurc:='K';
				ELSE
					_totalPembelian:=_totalPembelian*(-1);
					_purcdk='K';
					_selisihPurc:='D';
				END IF;

				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, _AccAsset, _purcdk, _totalPembelian, null, null);

				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fAccDefault('ACCPROFIT'), _selisihPurc, _totalPembelian, null, null);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
		CLOSE cPurc;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;
	UPDATE crc SET excrate=_ExcRate WHERE crcid=_CrcID;

	END LOOP;
	CLOSE cCrc;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
	
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;

CREATE OR REPLACE FUNCTION spEndYear(_PerID varchar,_StartDate date,_EndDate date)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_PerIDNext	varchar;
	_accLabaTahan	varchar;
	_accLabaJalan	varchar;
	_StartDatePer	date;
	_JumLaba	numeric(19,4);
	_JnlNo1		VARCHAR(15);
	_JnlNote	VARCHAR;
	_PerNo		int;
	_CrcID		varchar;
	_ExcRate	decimal;
	_FisRate	numeric(19,4);
BEGIN
	SELECT INTO _CrcID,_Excrate,_FisRate crcid,excrate,fisrate FROM crc WHERE crcid=(select * from fcrcdefault());

	SELECT INTO _StartDatePer,_PerNo perstart,perno FROM per WHERE perid=_PerID;
	UPDATE per SET perend=_StartDate,active=false WHERE perid=_PerID;
	_PerIDNext:=fTransNoGet('PER');
	INSERT INTO PER(perid,perstart,active,perno) VALUES(_PerIDNext,_EndDate,true,_PerNo+1);	

	_accLabaTahan:=fAccDefault('LABATAHAN');
	_accLabaJalan:=fAccDefault('LABATHNJLN');

	_JnlNo1:=fTransNoGet('JURNAL');
	_JnlNote:='Year-End '||_EndDate;
	SELECT INTO _JumLaba lababersih FROM fProfitLoss(_StartDatePer,_StartDate,null,null,null);

	_JumLaba:=COALESCE(_JumLaba,0);
	SELECT INTO _spret * FROM spJnl_Insert(
		_JnlNo1, 'GJ', _EndDate,'GJ', null,
		_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
		null, null, null, null,
		null, null, null, null, _CrcID, _ExcRate, _FisRate				
	);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, 1, _AccLabaTahan, 'K', _JumLaba, null, null);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, 2, _AccLabaJalan, 'D', _JumLaba, null, null);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE VIEW vwChk_StockCard AS
	SELECT v.itemid, v.whid, s.whid AS s_whid 
	FROM (SELECT itemid, whid FROM item,wh) v
	LEFT JOIN stock s ON v.itemid=s.itemid AND v.whid=s.whid;




CREATE OR REPLACE FUNCTION spStockCard_CreateAll() 
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval	SPStatus%ROWTYPE;
	_count	INT;
BEGIN
	_count:=0;

	INSERT INTO Stock(ItemID, WhID, PID)
	SELECT ItemID, WhID, '' FROM vwChk_StockCard WHERE s_WhID IS NULL;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spACL_Gen (
	_ObjID	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval	SPStatus%ROWTYPE;
BEGIN
	_retval:=(1, 'ERROR generate ACL');

	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'ENB',true);
	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'NEW',true);
	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'OPN',true);
	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'UPD',true);
	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'DEL',true);
	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'VOI',true);
	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'PRN',true);
	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'CHK',true);
	INSERT INTO OBJACL (objid, acstype, acsenable) VALUES (_ObjID,'APR',true);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


DROP TABLE SRetDeli;

CREATE TABLE SRetDeli (
	SRetNo                VARCHAR(15)          not null,
	DeliNo                VARCHAR(15)          not null,
constraint PK_SRetDeli primary key (SRetNo, DeliNo),
constraint FK_SRetDeli_RELATIONS_SRet foreign key (SRetNo)
   references SRet (SRetNo)
   on delete restrict ,
constraint FK_SRetDeli_RELATIONS_Deli foreign key (DeliNo)
   references Deli (DeliNo)
   on delete restrict 	
);

DROP TABLE PRetPRcv;

CREATE TABLE PRetPRcv (
	PRetNo                VARCHAR(15)          not null,
	PRcvNo                VARCHAR(15)          not null,
constraint PK_PRetPRcv primary key (PRetNo, PRcvNo),
constraint FK_PRetPRcv_RELATIONS_PRet foreign key (PRetNo)
   references PRet (PRetNo)
   on delete restrict ,
constraint FK_PRetPRcv_RELATIONS_PRcv foreign key (PRcvNo)
   references PRcv (PRcvNo)
   on delete restrict 
);



CREATE OR REPLACE FUNCTION flog_action()
RETURNS SETOF tlog AS
$BODY$
DECLARE
	retval	tlog%ROWTYPE;
	cLog	refcursor;
BEGIN
	OPEN cLog for
		SELECT 'SALE','Aktifitas Penjualan',saleno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM sale t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'PURC','Aktifitas Pembelian',purcno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM purc t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'RCV','Aktifitas Pembayaran Piutang',rcvno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM rcv t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'PAY','Aktifitas Pembayaran Hutang',payno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM pay t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

/*
	OPEN cLog for
	SELECT 'STOCKA','Aktifitas Stock',stockano,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
	FROM stocka t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid where t.isautogen=false;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;
		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;
*/

	OPEN cLog for
		SELECT 'STTR','Aktifitas Mutasi Stock',sttrno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM sttr t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'PRET','Aktifitas Retur Beli',pretno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM PRET t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'SRET','Aktifitas Retur Jual',sretno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM SRET t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'BPA','Aktifitas Bisnis Partner',bpano,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM bpa t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

/*	
	OPEN cLog for
	SELECT 'CASHA','Aktifitas Kas',cashano,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
	FROM casha t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;
		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;
*/

	OPEN cLog for
		SELECT 'MFG','Aktifitas Manufaktur',mfgno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM mfg t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'PO','Aktifitas Order Pembelian',pono,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM po t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'SO','Aktifitas Order Penjualan',sono,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM so t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'PREQ','Aktifitas Permintaan Pembelian',preqno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM preq t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'PRCV','Aktifitas Penerimaan Pembelian',prcvno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM prcv t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'DO','Aktifitas Order Pengiriman',deono,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM deo t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'DELI','Aktifitas Pengiriman',delino,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM deli t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'SADJ','Aktifitas Penyesuaian Stock',sadjno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM sadj t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'CSTR','Aktifitas Mutasi Kas',cstrno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM cstr t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;

	OPEN cLog for
		SELECT 'SADJ','Aktifitas Penyesuaian Stock',sadjno,crtby,c.usrname,crtdate,updby,u.usrname,upddate 
		FROM sadj t LEFT JOIN usr c on t.crtby=c.usrid LEFT JOIN usr u ON t.updby=u.usrid;
	LOOP
		FETCH cLog INTO retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT retval;
	END LOOP;
	CLOSE cLog;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fRekapArusKas (
	_startdate	DATE,
	_enddate	DATE
) RETURNS SETOF tRekapArusKas AS
$BODY$
DECLARE
	_retval		tRekapArusKas%ROWTYPE;
	_Month		SMALLINT;
	_PYear		INTEGER;
	_LastMonthBal	NUMERIC(19,4);
	_YesterdayBal	NUMERIC(19,4);
	_jumPositif	NUMERIC(19,4);
	_jumNegatif	NUMERIC(19,4);
	_Reftype	VARCHAR;
	_ReftypeDesc	VARCHAR;
	_cashid		VARCHAR;
	_cashdesc	VARCHAR;

	/**Tambah Crc*/
	_CrcID		VARCHAR;
	_CrcName	VARCHAR;
	_CrcSymbol	VARCHAR;

	cRef		REFCURSOR;
	cCash		REFCURSOR;
	masuk		BOOLEAN;
BEGIN
	OPEN cCash FOR
		SELECT cashid,cashdesc FROM cash;
	LOOP
		FETCH cCash INTO _cashid,_cashdesc;
		EXIT WHEN NOT FOUND;

		SELECT INTO _CrcID,_CrcName,_CrcSymbol cs.crcid,crc.crcname,crc.crcsymbol
		FROM cash cs JOIN crc ON cs.crcid=crc.crcid WHERE cs.cashid=_cashid;

		_Month:=date_part('month', _StartDate);
		_PYear:=date_part('year', _StartDate);

		IF (_Month=1) THEN
			SELECT INTO _LastMonthBal CashBal00 FROM CashBal
			WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=2) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01 
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=3) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02 
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=4) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03 
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=5) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04 
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=6) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05 
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=7) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=8) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
			+CashBal07
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=9) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
			+CashBal07+CashBal08
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=10) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
			+CashBal07+CashBal08+CashBal09
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=11) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
			+CashBal07+CashBal08+CashBal09+CashBal10
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		ELSIF (_Month=12) THEN
			SELECT INTO _LastMonthBal CashBal00+CashBal01+CashBal02+CashBal03+CashBal04+CashBal05+CashBal06
			+CashBal07+CashBal08+CashBal09+CashBal10+CashBal11
			FROM CashBal WHERE CashID=_CashID AND PYear=_PYear;
		END IF;

		IF (_LastMonthBal IS NULL) THEN
			_LastMonthBal:=0;
		END IF;

		SELECT INTO _YesterdayBal (_LastMonthBal+COALESCE(SUM(Amount), 0)) 
		FROM CashA
		WHERE CashID=_CashID AND CashADate>=fBegOfMonth(_StartDate)
		AND CashADate<_StartDate;

		_retval.cashid:=_cashid;
		_retval.cashdesc:=_cashdesc;
		_retval.reftype:='BegBal';
		_retval.saldoawal:=_YesterdayBal;
		_retval.jumpositif:=null;
		_retval.jumnegatif:=null;

		_retval.crcid=_CrcID;
		_retval.crcname=_CrcName;
		_retval.crcsymbol=_CrcSymbol;

		RETURN NEXT _retval;

		OPEN cRef FOR
			SELECT reftype,reftypedesc FROM reftype;
		LOOP
			FETCH cRef INTO _Reftype,_ReftypeDesc;
			EXIT WHEN NOT FOUND;


			SELECT INTO _jumPositif coalesce(SUM(amount),0) FROM casha WHERE cashadate>=_StartDate 
			AND cashadate<=_EndDate AND amount>0 AND reftype=_reftype and cashid=_cashid;

			SELECT INTO _jumNegatif coalesce(SUM(amount),0) FROM casha WHERE cashadate>=_StartDate 
			AND cashadate<=_EndDate AND amount<0 AND reftype=_reftype and cashid=_cashid;

			IF(_jumPositif>0 OR _jumNegatif<0) THEN
				_retval.cashid:=_cashid;
				_retval.cashdesc:=_cashdesc;
				_retval.reftype:=_ReftypeDesc;
				_retval.jumPositif:=_jumPositif;
				_retval.jumNegatif:=(_jumNegatif)*(-1);
				_retval.saldoawal:=null;

				_retval.crcid=_CrcID;
				_retval.crcname=_CrcName;
				_retval.crcsymbol=_CrcSymbol;

				RETURN NEXT _retval;
			END IF;
		END LOOP;
		CLOSE cRef;
	END LOOP;
	CLOSE cCash;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fWajib_Vs_Modal()
RETURNS SETOF tWajib_Vs_Modal AS
$BODY$
DECLARE
	_retval 	tWajib_Vs_Modal%ROWTYPE;
	_wajib		NUMERIC(19,4);
	_modal		NUMERIC(19,4);
	_rasio		NUMERIC(19,4);
BEGIN
	SELECT INTO _wajib abs(sum(accbal)) FROM acc WHERE acctype='200';
	SELECT INTO _modal abs(sum(accbal)) FROM acc WHERE acctype='300';
	_rasio:=0;

	IF(_modal<>0) THEN
		_rasio:=_wajib/_modal;
	END IF;

	_retval.wajib:=_wajib;
	_retval.modal:=_modal;
	_retval.rasio:=_rasio;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPerubahanModal(
	_StartDate	DATE,
	_EndDate	DATE,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS SETOF tPerubahanModal AS
$BODY$
DECLARE
	_retval		tPerubahanModal%ROWTYPE;
	_accnoAwal	VARCHAR;
	_Acctype2Awal	VARCHAR;
	_Jum		NUMERIC(19,4);
	_Jum31		NUMERIC(19,4);
	_Jum32		NUMERIC(19,4);
	_Jum33		NUMERIC(19,4);
	_SaldoAwal	NUMERIC(19,4);
	_AccNo		VARCHAR;
	_AccType2	VARCHAR;
	_AccDk		VARCHAR;
	_AccName	VARCHAR;
	_Jum31Now	NUMERIC(19,4);
	_Jum32Now	NUMERIC(19,4);
	_Jum33Now	NUMERIC(19,4);
	_SaldoAkhir	NUMERIC(19,4);
	_LabaBerjalan	NUMERIC(19,4);
	_LabaDitahan	NUMERIC(19,4);
	_bulanPer 	INTEGER;
	_tahunPer 	INTEGER;
	_bulan 		INTEGER;
	_tahun 		INTEGER;
	_StartDatePrev 	DATE;
	_EndDatePrev 	DATE;
	_PerNo		INT;
	_StartDatePer 	DATE;
	cAccAwal	REFCURSOR;
	cNow		REFCURSOR;
BEGIN
	_Jum31:=0;
	_Jum32:=0;
	_Jum33:=0;
	_Jum31Now:=0;
	_Jum32Now:=0;
	_Jum33Now:=0;

	OPEN cAccAwal FOR
		SELECT accno,acctype2 
		FROM acc 
		WHERE acctype2 LIKE '3%' AND accno<>fAccDefault('LABATHNJLN') 
		AND accno<>fAccDefault('LABATAHAN');
	LOOP
		FETCH cAccAwal INTO _accnoAwal,_acctype2Awal;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Jum * FROM fAccBal_StartDate(_accnoAwal,_StartDate-1,_BranchID,_DeptID,_PrjID);
		_Jum:=COALESCE(_Jum,0);

		IF(_acctype2Awal='310') THEN
			_Jum31:=_Jum31+_Jum;
		ELSIF(_acctype2Awal='320') THEN
			_Jum32:=_Jum32+_Jum;
		ELSIF(_acctype2Awal='330') THEN
			_Jum33:=_Jum33+_Jum;
		END IF;
	END LOOP;
	CLOSE cAccAwal;

	_SaldoAwal:=_Jum31-_Jum32+_Jum33;
	_retval.Accname:='Saldo Awal';
	_retval.Jum:=_SaldoAwal;

	RETURN NEXT _retval;

	OPEN cNow FOR
		SELECT accno,acctype2,accdk,accname 
		FROM acc 
		WHERE acctype2 LIKE '3%' AND accno<>fAccDefault('LABATHNJLN') 
		AND accno<>fAccDefault('LABATAHAN') ORDER BY accno;
	LOOP
		FETCH cNow INTO _AccNo,_acctype2,_AccDk,_AccName;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Jum COALESCE(SUM(CASE WHEN dk=_AccDk THEN jnldamt ELSE -1*jnldamt END),0)
		FROM jnld 
		JOIN jnl ON jnl.jnlno=jnld.jnlno 
		WHERE accNo=_accNo 
		AND (_branchid IS NULL OR jnl.branchid=_branchid) 
		AND (_DeptID IS NULL OR DeptID=_DeptID) 
		AND (_PrjID IS NULL OR PrjID=_PrjID)
		AND JnlDate>=_StartDate AND JnlDate<=_EndDate; 

		_Jum:=COALESCE(_Jum,0);

		IF(_acctype2='310') THEN
			_Jum31Now:=_Jum31Now+_Jum;
		ELSIF(_acctype2='320') THEN
			_Jum32Now:=_Jum32Now+_Jum;
		ELSIF(_acctype2='330') THEN
			_Jum33Now:=_Jum33Now+_Jum;
		END IF;

		IF(_Jum<>0) THEN
			_retval.accno:=_AccNo;
			_retval.accname:=_AccName;
			_retval.Jum:=_Jum;

			RETURN NEXT _retval;
		END IF;
	END LOOP; 
	CLOSE cNow;

	_LabaBerjalan:=0;
	_LabaDitahan:=0;

	RAISE NOTICE '%', 'Change ACCBAL in ACC';

	SELECT INTO _StartDatePer,_PerNo perstart,perno 
	FROM per 
	WHERE active IS true;

	SELECT INTO _bulanPer DATE_part('month',_StartDatePer);
	SELECT INTO _tahunPer DATE_part('year',_StartDatePer);
	SELECT INTO _bulan DATE_part('month',_StartDate);
	SELECT INTO _tahun DATE_part('year',_StartDate);

	SELECT INTO _StartDatePrev,_EndDatePrev perstart,perend FROM per WHERE perno=_PerNo-1;

	SELECT INTO _LabaBerjalan lababersih FROM fProfitLoss(_StartDatePer,_EndDate,_branchid,_DeptID,_PrjID);
	
	SELECT INTO _retval.accname accname FROM acc WHERE accno=fAccDefault('LABATHNJLN');
	_retval.accno:=fAccDefault('LABATHNJLN');
	_retval.Jum:=_LabaBerjalan;

	RETURN NEXT _retval;

	IF((_bulan=_bulanPer) AND (_tahun=_tahunPer)) THEN
		SELECT INTO _LabaDitahan lababersih FROM fProfitLoss(_StartDatePrev,_EndDatePrev,_branchid,_DeptID,_PrjID);
		IF(_LabaDitahan<>0) THEN
			SELECT INTO _retval.accname accname FROM acc WHERE accno=fAccDefault('LABATAHAN');
			_retval.accno:=fAccDefault('LABATAHAN');
			_retval.Jum:=_LabaDitahan;
			RETURN NEXT _retval;
		END IF;
	END IF;

	_SaldoAkhir:=_SaldoAwal+_Jum31Now-_Jum32Now+_Jum33Now+_LabaBerjalan+_LabaDitahan;

	_retval.AccNo:=NULL;
	_retval.Accname:='Saldo Akhir';
	_retval.Jum:=_SaldoAkhir;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fGetPer (
	_StartDate	DATE
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval		VARCHAR;
	_perid		VARCHAR;
	_perstart	DATE;
	_perend		DATE;

	cPer		REFCURSOR;
BEGIN
	/*Default nilai retval, klo tanggal nya di luar periode manapun dianggep ngambil perstart nya periode aktif*/
	SELECT INTO _retval perid FROM per WHERE active=true;

	OPEN cPer FOR
		SELECT perid, perstart FROM per ORDER BY perno desc;
	LOOP
		FETCH cPer INTO _perid,_perstart;
		EXIT WHEN NOT FOUND;

		IF(_perstart<=_StartDate) THEN
			_retval:=_perid;
			EXIT;
		END IF;
	END LOOP;
	CLOSE cPer;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spTutupBukuBulan(
	_StartDate	DATE,
	_EndDate	DATE,
	_isDepresiasi	BOOLEAN,
	_isRevaluated	BOOLEAN,
	_isLocked	BOOLEAN,
	_isBackup	BOOLEAN
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_JnlNo1			VARCHAR;
	_JnlNo2			VARCHAR;
	_JnlNo3			VARCHAR;
	_JnlNo4			VARCHAR;
	_JnlNote1		VARCHAR;
	_JnlNote2		VARCHAR;
	_JnlNote3		VARCHAR;
	_JnlNote4		VARCHAR;
	_spret			SPStatus%ROWTYPE;
	_retval			SPStatus%ROWTYPE;
	_CountPendapatan	INT;
	_tampPendapatan		NUMERIC(19,4);
	_accnoPendapatan	VARCHAR;
	_accbalPendapatan	NUMERIC(19,4);
	_CountBeban		INT;
	_tampBeban		NUMERIC(19,4);
	_accnoBeban		VARCHAR;
	_selisih		NUMERIC(19,4);
	_accLabaJalan		VARCHAR;
	_tampPrive		NUMERIC(19,4);
	_CountPrive		INT;
	_accbalPrive		NUMERIC(19,4);
	cAccPendapatan		REFCURSOR;
	cAccBeban		REFCURSOR;
	cPrive			REFCURSOR;
	_accbalBeban		NUMERIC(19,4);
	_accnoPrive		VARCHAR;
	_month			INT;
	_year			INT;
	_found			VARCHAR;
	_PerID			VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);
	
BEGIN
	_month:=DATE_PART('month',_EndDate);
	_year:=DATE_PART('year',_EndDate);

	SELECT INTO  _PerID PerID FROM per WHERE active=true;

	SELECT INTO  _found jnlno FROM jnl WHERE jnltype='CLSM' and date_part('month',jnldate)=_month and date_part('year',jnldate)=_year;
	IF FOUND THEN
		_retval.ErrNo:=5005;
		_retval.ErrMsg:='Bulan '|| CAST(_month AS VARCHAR)||'/'|| CAST(_year AS VARCHAR)|| ' sudah Tutup Buku. Void Dulu Jurnal lalu ulangi (spTutupBukuBulan)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_CrcID:=fCrcDefault();
	_ExcRate:=1;
	_FisRate:=1;

	--- Revaluasi ---
	IF (_isRevaluated=TRUE) THEN
		SELECT INTO  _spret * FROM spRevaluasi(_EndDate);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	IF(_isDepresiasi=TRUE) THEN
		SELECT INTO  _spret * FROM spFixedAsset_Depr(_EndDate);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	-- update tgl lock transaksi->di set tgl endDate+1
	IF(_isLocked=TRUE) THEN
		UPDATE tlock SET lockdate=_EndDate+1;	-- sengaja gak pake WHERE karena memang utk Lock semua transaksi
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spTutupBukuTahun (
	_PerID 		VARCHAR,
	_StartDate	DATE,
	_EndDate	DATE,
	_isDepresiasi	BOOLEAN,
	_isRevaluated	BOOLEAN,
	_isLocked	BOOLEAN,
	_isBackup	BOOLEAN,
	_isPosted	BOOLEAN
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_StartDatePer	DATE;
	_EndDatePer	DATE;
	_Active		BOOLEAN;
	_PerNo		INT;
	_MonthStartPer	INT;
	_YearStartPer	INT;
	_MonthEndPer	INT;
	_YearEndPer	INT;
	_iterMonth	INT;
	_iterYear	INT;
	cJnl		REFCURSOR;
	_jnlno		VARCHAR;
	_dayEndMonth	INT;
	_jnlStartDate	VARCHAR;
	_jnlEndDate	VARCHAR;
	_accLabaTahan	VARCHAR;
	_accLabaJalan	VARCHAR;
	_JnlNo1		VARCHAR;
	_JnlNote	VARCHAR;
	_JumLaba	NUMERIC(19,4);
	_PerIDNext	VARCHAR;
	_JumLabaBulan	NUMERIC(19,4);
	_CrcID		VARCHAR;
	_ExcRate	numeric(19,4);
	_FisRate	numeric(19,4);

	_CurrPerId	VARCHAR;
	_CurrActive	BOOLEAN;
BEGIN
	_retval:=(0, 'OK');
	SELECT INTO _StartDatePer, _EndDatePer, _PerNo, _Active 
		perstart, perend, perno, active FROM per WHERE perid=_PerID;

	IF (_EndDatePer IS NOT NULL AND _Active=FALSE) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Proses Tutup Buku Tahun untuk periode ini sudah dilakukan sebelumnya (spTutupBukuTahun)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	ELSE
		_CurrPerId:=_PerID;
	END IF;

	SELECT INTO _CurrPerId, _CurrActive perid, active 
	FROM per WHERE _StartDate>=perstart AND (perend IS NULL OR (perend IS NOT NULL AND _StartDate<=perend)) LIMIT 1;
	IF (_CurrPerId IS NOT NULL AND _CurrActive=FALSE) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Proses Tutup Buku Tahun untuk periode ini sudah dilakukan sebelumnya (spTutupBukuTahun)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _MonthStartPer date_part('month',_StartDatePer);
	SELECT INTO _YearStartPer date_part('year',_StartDatePer);

	SELECT INTO _MonthEndPer date_part('month',_EndDate);
	SELECT INTO _YearEndPer date_part('year',_EndDate);

	_iterMonth:=_MonthStartPer;
	_iterYear:=_YearStartPer;

	_jumLaba:=0;

	SELECT INTO _CrcID CrcID from crc where isdefault=true;
	_ExcRate:=1;
	_FisRate:=1;

	UPDATE TLOCK SET LOCKDATE=_StartDatePer;

	-- iterasi tiap bulan sampek ketemu end year --
	WHILE (NOT(_iterMonth=_MonthEndPer AND _iterYear=_YearEndPer)) LOOP
		
		-- klo pernah tutup bulan sebelumnya dihapus dulu ntar dibuatin lagi .. --
		/*OPEN cJnl FOR
			SELECT jnlno FROM jnl WHERE date_part('month', jnldate)=_iterMonth AND 
			date_part('year',jnldate)=_iterYear AND jnltype='CLSM';
			LOOP
				FETCH cJnl INTO _jnlno;
				EXIT WHEN NOT FOUND;

				SELECT INTO _spret * FROM spJnl_Void(_jnlno);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
		CLOSE cJnl;*/

		/*proses create jurnalnya..*/
		IF(_iterMonth=1) THEN
			_dayEndMonth:=31;
		ELSIF(_iterMonth=2) THEN
			IF ((_iterYear%4=0 AND _iterYear%100<>0) OR _iterYear%400=0) THEN 
				_dayEndMonth:=29;
			ELSE
				_dayEndMonth:=28;
			END IF;
		ELSIF(_iterMonth=3) THEN
			_dayEndMonth:=31;
		ELSIF(_iterMonth=4) THEN
			_dayEndMonth:=30;
		ELSIF(_iterMonth=5) THEN
			_dayEndMonth:=31;
		ELSIF(_iterMonth=6) THEN
			_dayEndMonth:=30;
		ELSIF(_iterMonth=7) THEN
			_dayEndMonth:=31;
		ELSIF(_iterMonth=8) THEN
			_dayEndMonth:=31;
		ELSIF(_iterMonth=9) THEN
			_dayEndMonth:=30;
		ELSIF(_iterMonth=10) THEN
			_dayEndMonth:=31;
		ELSIF(_iterMonth=11) THEN
			_dayEndMonth:=30;
		ELSIF(_iterMonth=12) THEN
			_dayEndMonth:=31;
		END IF;

		_jnlStartDate:=_iterYear||'-'||_iterMonth||'-'||'01';
		_jnlEndDate:=_iterYear||'-'||_iterMonth||'-'||_dayEndMonth;

		SELECT INTO _jumLabaBulan coalesce(lababersih,0) FROM fProfitLoss(CAST(_jnlStartDate AS DATE),CAST(_jnlEndDate AS DATE),null,null,null);

		_jumLaba:=_jumLaba+_jumLabaBulan;
		RAISE NOTICE '%', _jnlStartDate||' '||_jumLaba||' '||_jumLabaBulan;
		SELECT INTO _spret * FROM spTutupBukuBulan(CAST(_jnlStartDate AS DATE),CAST(_jnlEndDate AS DATE), _isDepresiasi, _isRevaluated, _isLocked, _isBackup);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF(_iterMonth=12) THEN
			_iterMonth:=1;
			_iterYear:=_iterYear+1;
		ELSE
			_iterMonth:=_iterMonth+1;
		END IF;
	END LOOP;

	UPDATE per SET perend=_StartDate,active=false WHERE perid=_CurrPerId;
	_PerIDNext:=fTransNoGet('PER');
	INSERT INTO PER(perid,perdesc,perstart,active,perno) VALUES(_PerIDNext,date_part('year', 		_EndDate),_EndDate,true,_PerNo+1);

	/*Posting Laba thn jalan dipindah ke laba tahan*/

	IF(_IsPosted=true)  THEN
	_accLabaTahan:=fAccDefault('LABATAHAN');
	_accLabaJalan:=fAccDefault('LABATHNJLN');

	_JnlNo1:=fTransNoGet('JURNAL');
	_JnlNote:='Year-End '||_EndDate;

	SELECT INTO _spret * FROM spJnl_Insert(
		_JnlNo1, 'CLSY', _EndDate,'CLSY', null,
		_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
		null, null, null, null,
		_PerIDNext, null, null, null, _CrcID, _ExcRate, _FisRate				
	);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, 1, _AccLabaTahan, 'K', _JumLaba, null, null, _JumLaba);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo1, 2, _AccLabaJalan, 'D', _JumLaba, null, null, _JumLaba);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	SELECT INTO _spret * FROM spJnl_New(_JnlNo1);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	END IF; --end if nya isPosted	

	-- update tgl lock transaksi->di set tgl endDate+1 --
	UPDATE tlock SET lockdate=_EndDate;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPPaid(
	__PayNo 	VARCHAR,
	__DeptId	VARCHAR,
	__PrjId		VARCHAR
) RETURNS SETOF tPPaid AS
$BODY$
DECLARE
	_retval		tPPaid%ROWTYPE;	

	_PayNo		VARCHAR;
	_RefNo		VARCHAR;
	_RefDesc	VARCHAR;
	_RefType	VARCHAR;
	_PPaidAmt	NUMERIC(19,4);
	_transDate	DATE;
	_Total		NUMERIC(19,4);
	_JT		DATE;
	_DeptId		VARCHAR;
	_PrjId		VARCHAR;

	cPPaid		REFCURSOR;
BEGIN
	_PayNo:=__PayNo;
	OPEN cPPaid FOR
		SELECT refno, reftype, ppaidamt, deptid, prjid, ppaidnote 
		FROM ppaid 
		WHERE payno=_PayNo AND (__DeptId IS NULL OR deptid=__DeptId) AND (__PrjId IS NULL OR prjid=__PrjId);
	LOOP
		FETCH cPPaid INTO _RefNo, _RefType, _PPaidAmt, _DeptId, _PrjId, _RefDesc;
		EXIT WHEN NOT FOUND;
	
		_transDate:=NULL;
		_Total:=NULL;
		_JT:=NULL;

		IF(_RefType='PURC') THEN
			SELECT INTO _transDate, _Total, _JT purcdate, total, purcdate+duedays 
			FROM purc 
			WHERE purcno=_RefNo;
			IF (_RefDesc IS NULL OR _RefDesc='') THEN
				_RefDesc:='Pembelian#'||_RefNo;
			END IF;
		ELSIF(_RefType='PRET') THEN
			SELECT INTO _transDate, _Total pretdate, total 
			FROM pret 
			WHERE pretno=_RefNo;
			IF (_RefDesc IS NULL OR _RefDesc='') THEN
				_RefDesc:='Retur Pembelian#'||_RefNo;
			END IF;
		ELSIF(_RefType='FCN') THEN
			SELECT INTO _transDate, _Total fcndate, fcnamt 
			FROM fcn
			WHERE fcnno=_RefNo;
			IF (_RefDesc IS NULL OR _RefDesc='') THEN
				_RefDesc:='Nota Potong Pembelian#'||_RefNo;
			END IF;
		END IF;
		
		_retval.refno:=_RefNo;
		_retval.refdesc:=_RefDesc;
		_retval.ppaidamt:=_PPaidAmt;
		_retval.transDate:=_transDate;
		_retval.total:=_Total;
		_retval.jt:=_JT;
		_retval.deptid:=_DeptId;
		_retval.prjid:=_PrjId;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cPPaid;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPayD (
	_PayNo		VARCHAR,
	_Excrate 		NUMERIC(19,4)
) RETURNS SETOF tPayD AS
$BODY$
DECLARE
	_retval		tPayD%ROWTYPE;

	cPayD		REFCURSOR;

	_CashDesc	VARCHAR;
	_PaydAmt	NUMERIC(19,4);
	_CbgID		VARCHAR;
	_CashID		VARCHAR;
	_PaydMtd	VARCHAR;
	_CbgDueDate	DATE;
	_CbgNo		VARCHAR;
	_BankName	VARCHAR;
	_BankID		VARCHAR;
	_PayDNote	VARCHAR;
BEGIN

	OPEN cPayD FOR
		SELECT cashid, paydmtd, (paydamt*excrate)/_Excrate, cbgid, paydnote FROM payd WHERE payno=_PayNo;
	LOOP
		FETCH cPayD INTO _CashID, _PaydMtd, _PaydAmt, _CbgID, _PayDNote;
		EXIT WHEN NOT FOUND;

		SELECT INTO _CashDesc cashdesc FROM cash WHERE cashid=_CashID;
		SELECT INTO _CbgDueDate,_CbgNo,_BankName cbgduedate,cbgno,bankname 
		FROM cbg LEFT JOIN bank ON cbg.bankid=bank.bankid WHERE cbgid=_CbgID;

		IF(_CbgID IS NOT NULL) THEN
			_retval.cashdesc:=_BankName;
		ELSE
			_retval.cashdesc:=_CashDesc;
		END IF;
			
		_retval.paydamt:=_PaydAmt;
		_retval.cbgid:=_CbgNo;
		_retval.cbgduedate:=_CbgDueDate;
		_retval.paydnote:=_PayDNote;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cPayD;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSPaid(
	__RcvNo 	VARCHAR,
	__DeptId	VARCHAR,
	__PrjId		VARCHAR
) RETURNS SETOF tSPaid AS
$BODY$
DECLARE
	_retval		tSPaid%ROWTYPE;

	_RcvNo		VARCHAR;
	_RefNo		VARCHAR;
	_RefDesc	VARCHAR;
	_RefType	VARCHAR;
	_SPaidAmt	NUMERIC(19,4);
	_transDate	DATE;
	_Total		NUMERIC(19,4);
	_JT		DATE;
	_DeptId		VARCHAR;
	_PrjId		VARCHAR;

	cSPaid		REFCURSOR;
BEGIN
	_RcvNo:=__RcvNo;
	OPEN cSPaid FOR
		SELECT refno, reftype, spaidamt, deptid, prjid, spaidnote 
		FROM spaid 
		WHERE rcvno=_RcvNo AND (__DeptId IS NULL OR deptid=__DeptId) AND (__PrjId IS NULL OR prjid=__PrjId);
	LOOP
		FETCH cSPaid INTO _RefNo, _RefType, _SPaidAmt, _DeptId, _PrjId, _RefDesc;
		EXIT WHEN NOT FOUND;

		_transDate:=NULL;
		_Total:=NULL;
		_JT:=NULL;

		IF(_RefType='SALE') THEN
			SELECT INTO _transDate, _Total, _JT saledate, total, saledate+duedays 
			FROM sale 
			WHERE saleno=_RefNo;
			IF (_RefDesc IS NULL OR _RefDesc='') THEN
				_RefDesc:='Penjualan#'||_RefNo;
			END IF;
		ELSIF(_RefType='SRET') THEN
			SELECT INTO _transDate, _Total sretdate, total 
			FROM sret 
			WHERE sretno=_RefNo;
			IF (_RefDesc IS NULL OR _RefDesc='') THEN
				_RefDesc:='Retur Penjualan#'||_RefNo;
			END IF;
		ELSIF(_RefType='FCN') THEN
			SELECT INTO _transDate, _Total fcndate, fcnamt 
			FROM fcn
			WHERE fcnno=_RefNo;
			IF (_RefDesc IS NULL OR _RefDesc='') THEN
				_RefDesc:='Nota Potong Penjualan#'||_RefNo;
			END IF;
		END IF;

		_retval.refno:=_RefNo;
		_retval.refdesc:=_RefDesc;
		_retval.spaidamt:=_SPaidAmt;
		_retval.transDate:=_transDate;
		_retval.total:=_Total;
		_retval.jt:=_JT;
		_retval.deptid:=_DeptId;
		_retval.prjid:=_PrjId;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cSPaid;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fRcvD (
	_RcvNo		VARCHAR,
	_Excrate 		NUMERIC(19,4)
) RETURNS SETOF tRcvD AS
$BODY$
DECLARE
	_retval		tRcvD%ROWTYPE;

	cRcvD		REFCURSOR;

	_CashDesc	VARCHAR;
	_RcvdAmt	NUMERIC(19,4);
	_CbgID		VARCHAR;
	_CashID		VARCHAR;
	_RcvdMtd	VARCHAR;
	_CbgDueDate	DATE;
	_CbgNo		VARCHAR;
	_BankName	VARCHAR;
	_BankID		VARCHAR;
	_RcvDNote	VARCHAR;
BEGIN
	OPEN cRcvD FOR
		SELECT cashid, rcvdmtd, (rcvdamt*excrate)/_Excrate, cbgid, rcvdnote FROM rcvd WHERE rcvno=_RcvNo;
	LOOP
		FETCH cRcvD INTO _CashID, _RcvdMtd, _RcvdAmt, _CbgID, _RcvDNote;
		EXIT WHEN NOT FOUND;

		SELECT INTO _CashDesc cashdesc FROM cash WHERE cashid=_CashID;
		SELECT INTO _CbgDueDate, _CbgNo, _BankName cbgduedate, cbgno, bankname 
		FROM cbg LEFT JOIN bank ON bank.bankid=cbg.bankid WHERE cbgid=_CbgID;
	
		IF(_CbgID IS NOT NULL) THEN
			_retval.cashdesc:=_BankName;
		ELSE
			_retval.cashdesc:=_CashDesc;
		END IF;

		_retval.rcvdamt:=_RcvdAmt;
		_retval.cbgid:=_CbgNo;
		_retval.cbgduedate:=_CbgDueDate;
		_retval.rcvdnote:=_RcvDNote;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cRcvD;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStockLocation (
	_ItemID		VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	_temp		VARCHAR;

	_first		BOOLEAN;

	cStock		REFCURSOR;
BEGIN
	_retval:='';
	_first:=TRUE;

	OPEN cStock FOR
		SELECT location FROM stock WHERE itemid=_ItemID;
	LOOP
		FETCH cStock INTO _temp;
		EXIT WHEN NOT FOUND;

		IF (_first<>TRUE) THEN
			_retval:=_retval||','||_temp;
		ELSE
			_retval:=_retval||_temp;
		END IF;
	END LOOP;
	CLOSE cStock;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fUnitSubtotal(
	_ItemID		VARCHAR(20),
	_WhID		VARCHAR(3),
	_PID		VARCHAR(25),
	_Unit		VARCHAR(5)
) RETURNS NUMERIC(19,4) AS 
$BODY$
DECLARE
	_ItemType	VARCHAR(5);
	_isStock	BOOLEAN;
	_isAsm		BOOLEAN;
	_BoMID		VARCHAR(20);
	_Unit1		VARCHAR(5);
	_SubTotal	NUMERIC(19,4);
	_CostUnit1	NUMERIC(19,4);
	_mulFac		NUMERIC(15,4);	/* Multiply Factor from UnitX to Unit1 */
	_BoMDItemID	VARCHAR(15);
	_BoMDWhID	VARCHAR(3);
	_BoMDQty	NUMERIC(15,4);
	_BoMDUnit	VARCHAR(5);
	_BoMD_ItemCost	NUMERIC(19,4);	/*-- Cost 1PCS item detail BoM*/
	_BoM_ItemsCost	NUMERIC(19,4);	/*-- Total cost untuk 1 set(UNIT1) Item BoM*/
	_retval		NUMERIC(19,4);
	_Qty		NUMERIC(15,4);
	cBoMD		REFCURSOR;
BEGIN
	SELECT INTO 
		_ItemType, _IsAsm, _Unit1, _IsStock 
		ItemType, isAsm, Unit1, isStock
	FROM Item 
	WHERE ItemID=_ItemID;

	IF (_ItemType='SERV') THEN 
		_CostUnit1:=0;
	ELSIF (_ItemType='PACK' OR _IsAsm=TRUE) THEN	 /** Backward compatibilty **/
		SELECT INTO _BoMID BoMID FROM BoM WHERE ItemID=_ItemID;
		IF NOT FOUND THEN
			RAISE EXCEPTION 'B.o.M. untuk % tidak ditemukan (fUnitSubTotal)', _ItemID;
		END IF;

		_BoM_ItemsCost:=0;
		OPEN cBoMD FOR
			SELECT ItemID, WhID, Qty, Unit FROM BoMD WHERE BoMID=_BoMID;
		LOOP
			FETCH cBoMD INTO _BoMDItemID, _BoMDWhID, _BoMDQty, _BoMDUnit;
			EXIT WHEN NOT FOUND;

			_BoMD_ItemCost:=fUnitCost(_BoMDItemID, _WhID, _PID, _BoMDUnit);
			IF (_BoMD_ItemCost IS NULL) THEN
				RAISE EXCEPTION 'Item cost untuk % IS NULL (fUnitSubTotal)', _ItemID;
			END IF;
			_BoM_ItemsCost=_BoM_ItemsCost+(_BoMDQty*_BoMD_ItemCost);
		END LOOP;
		CLOSE cBoMD;
		_CostUnit1:=_BoM_ItemsCost;

	ELSIF(_IsStock=TRUE) THEN
		SELECT INTO 
			_SubTotal, _Qty
			Subtotal, Qty
		FROM Stock 
		WHERE 
			ItemID=_ItemID 
			AND WhID=_WhID 
			AND (_PID IS NULL OR PID=_PID);

		IF (_Qty=0) THEN
			_CostUnit1:=0;
		ELSE
			_CostUnit1:=_SubTotal/_Qty;
		END IF;

		IF NOT FOUND THEN
			RETURN NULL;
		END IF;

		IF (_CostUnit1 IS NULL) THEN
			RETURN NULL;
		END IF;
	ELSE
		_CostUnit1:=0;
	END IF;

	IF (_Unit<>_Unit1) THEN
		_mulFac:=fQtyZToQty1(_ItemID, 1, _Unit);

		IF (_mulFac IS NULL) THEN
			RAISE EXCEPTION 'Mulfac IS NULL untuk % ! (fUnitSubTotal)', _ItemID;
		END IF;
	ELSE
		_mulFac:=1;
	END IF;

	IF (_CostUnit1 IS NULL) THEN
		RAISE EXCEPTION 'CostUnit1 IS NULL untuk % ! (fUnitSubTotal)', _ItemID;
	END IF;

	_retval:=(_mulFac*_CostUnit1);
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fMfgM (
	_MfgNo	VARCHAR
) RETURNS NUMERIC(15,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(15,4);
BEGIN
	_retval:=0;

	SELECT INTO _retval SUM(fQtyZToQty1(pd.itemid, pd.qty, pd.unit)) FROM mfgm p, mfgmd pd
	WHERE p.mfgmno=pd.mfgmno AND p.mfgno=_MfgNo;
	IF (_retval IS NULL) THEN
		_retval:=0;
	END IF;

	RETURN _retval; 
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fMfgP (
	_MfgNo	VARCHAR
) RETURNS NUMERIC(15,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(15,4);
BEGIN
	_retval:=0;

	SELECT INTO _retval SUM(fQtyZToQty1(pd.itemid, pd.qty, pd.unit)) FROM mfgp p, mfgpd pd 
	WHERE p.mfgpno=pd.mfgpno AND p.mfgno=_MfgNo;
	IF (_retval IS NULL) THEN
		_retval:=0;
	END IF;

	RETURN _retval; 
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItGrpList (
	_itgrpid	VARCHAR
) RETURNS SETOF VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;

	cVar		REFCURSOR;
BEGIN
	/**RETURN NEXT _itgrpid;
	FOR _retval IN SELECT * FROM fItGrpList(_itgrpid, 0) 
	LOOP
	RETURN NEXT _retval;
	END LOOP; */

	/** ternyata bukan masalah versi postgresql tapi masalah pl/pgsqlnya ndak support for loop*/
	RETURN NEXT _itgrpid;
	OPEN cVar FOR
		SELECT * FROM fItGrpList(_itgrpid, 0);
	LOOP
		FETCH cVar INTO _retval;
		EXIT WHEN NOT FOUND;
		
		RETURN NEXT _retval;

	END LOOP;
	CLOSE cVar;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItGrpList (
	_itgrpid	VARCHAR,
	_depth		INTEGER
) RETURNS SETOF VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	_retval2 	VARCHAR;

	_parent		VARCHAR;

	cVar1		REFCURSOR;
	cVar2		REFCURSOR;
BEGIN
	/** FOR _retval IN SELECT itgrpid FROM itgrp WHERE upitgrpid=_itgrpid
	LOOP
		RETURN NEXT _retval;
		FOR _retval2 IN SELECT * FROM fItGrpList(_retval, _depth+1) 
		LOOP
		RETURN NEXT _retval2;
		END LOOP;
	END LOOP; */

	/** ternyata bukan masalah versi postgresql tapi masalah pl/pgsqlnya ndak support for loop*/
	OPEN cVar1 FOR
		SELECT itgrpid FROM itgrp WHERE upitgrpid=_itgrpid;
	LOOP
		FETCH cVar1 INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
			
		OPEN cVar2 FOR
			SELECT * FROM fItGrpList(_retval, _depth+1);
		LOOP
			FETCH cVar2 INTO _retval2;
			EXIT WHEN NOT FOUND;

			RETURN NEXT _retval2;
		END LOOP;
		CLOSE cVar2;
	END LOOP;
 	CLOSE cVar1;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItGrp2List (
	_itgrp2id	VARCHAR
) RETURNS SETOF VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;

	cVar		REFCURSOR;
BEGIN
	/** RETURN NEXT _itgrp2id;
	FOR _retval IN SELECT * FROM fItGrp2List(_itgrp2id, 0) 
	LOOP
	RETURN NEXT _retval;
	END LOOP; */

	/** ternyata bukan masalah versi postgresql tapi masalah pl/pgsqlnya ndak support for loop */
	RETURN NEXT _itgrp2id;	
	OPEN cVar FOR
		SELECT * FROM fItGrp2List(_itgrp2id, 0);
	LOOP
		FETCH cVar INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cVar;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItGrp2List (
	_itgrp2id	VARCHAR,
	_depth		INTEGER
) RETURNS SETOF VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	_retval2 	VARCHAR;

	_parent		VARCHAR;
	
	cVar1		REFCURSOR;
	cVar2		REFCURSOR;
BEGIN
	/** FOR _retval IN SELECT itgrp2id FROM itgrp2 WHERE upitgrp2id=_itgrp2id
	LOOP
		RETURN NEXT _retval;
		FOR _retval2 IN SELECT * FROM fItGrp2List(_retval, _depth+1) 
		LOOP
		RETURN NEXT _retval2;
		END LOOP;
	END LOOP; */

	/** ternyata bukan masalah versi postgresql tapi masalah pl/pgsqlnya ndak support for loop*/
	OPEN cVar1 FOR
		SELECT itgrp2id FROM itgrp2 WHERE upitgrp2id=_itgrp2id;
	LOOP
		FETCH cVar1 INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
		OPEN cVar2 FOR
			SELECT * FROM fItGrp2List(_retval, _depth+1);
		LOOP
			FETCH cVar2 INTO _retval2;
			EXIT WHEN NOT FOUND;

			RETURN NEXT _retval2; 
		END LOOP;
		CLOSE cVar2;
	END LOOP;
	CLOSE cVar1;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItGrp3List (
	_itgrp3id	VARCHAR
) RETURNS SETOF VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;

	cVar		REFCURSOR;
BEGIN
	/** RETURN NEXT _itgrp3id;
	FOR _retval IN SELECT * FROM fItGrp3List(_itgrp3id, 0) 
	LOOP
	RETURN NEXT _retval;
	END LOOP; */

	/** ternyata bukan masalah versi postgresql tapi masalah pl/pgsqlnya ndak support for loop*/
	RETURN NEXT _itgrp3id;
	OPEN cVar FOR
		SELECT * FROM fItGrp3List(_itgrp3id, 0);
	LOOP
		FETCH cVar INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cVar;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fItGrp3List (
	_itgrp3id	VARCHAR,
	_depth		INTEGER
) RETURNS SETOF VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	_retval2 	VARCHAR;

	_parent		VARCHAR;

	cVar1		REFCURSOR;
	cVar2		REFCURSOR;
BEGIN
	/** FOR _retval IN SELECT itgrp3id FROM itgrp3 WHERE upitgrp3id=_itgrp3id
	LOOP
		RETURN NEXT _retval;
		FOR _retval2 IN SELECT * FROM fItGrp3List(_retval, _depth+1) 
		LOOP
		RETURN NEXT _retval2;
		END LOOP;
	END LOOP; */

	/** ternyata bukan masalah versi postgresql tapi masalah pl/pgsqlnya ndak support for loop*/
	OPEN cVar1 FOR
		SELECT itgrp3id FROM itgrp3 WHERE upitgrp3id=_itgrp3id;
	LOOP
		FETCH cVar1 INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
		OPEN cVar2 FOR
			SELECT * FROM fItGrp3List(_retval, _depth+1);
		LOOP
			FETCH cVar2 INTO _retval2;
			EXIT WHEN NOT FOUND;

			RETURN NEXT _retval2; 
		END LOOP;
		CLOSE cVar2;
	END LOOP;
	CLOSE cVar1;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSaleUseCons (
	_SaleNo VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;
	_ConsCount	INT2;
BEGIN
	SELECT INTO _ConsCount COUNT(consno) FROM saled WHERE saleno=_SaleNo AND (consno IS NOT NULL OR consno<>'');
	IF (_ConsCount<=0) THEN
		_retval:=FALSE;
	ELSE
		_retval:=TRUE;
	END IF;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPOSCheckModal (
	_POSID	VARCHAR
) RETURNS SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_PoSSesNo	VARCHAR;

	_POSCash	VARCHAR;
	_Amount		NUMERIC(19,4);
	_POSAmount	NUMERIC(19,4);
BEGIN	
	_Amount:=COALESCE(CAST(fReg('POS_MODAL') AS NUMERIC(19,4)), 0);

	SELECT INTO _POSCash cashid FROM POS WHERE POSID=_POSID;
	IF (_POSCash IS NULL) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Kas untuk kasir '||_POSID||' belum diset! (fPOSCheckModal)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/* cek apakah kasir bersangkutan masih belum setor pada penjualan sebelumnya*/
	SELECT INTO _PoSSesNo PoSSesNo 
	FROM PoSSes 
	WHERE PoSID=_POSID AND IsCollect=FALSE;
	IF FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Penjualan Kasir '||_POSID||' sebelumnya masih belum disetorkan! (fPOSCheckModal)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** asumsi jika nilai kas kasir >= value di REG POSMODAL dinyatakan sudah ada set modal */
	SELECT INTO _POSAmount cashbal FROM cash WHERE cashid=_POSCash;
	IF NOT FOUND THEN
		_POSAmount:=0;
	END IF;

	IF (_POSAmount!=0 AND _Amount!=0 AND _POSAmount>=_Amount) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Kas awal kasir '||_POSID||' sudah di set (fPOSCheckModal)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAgingPiutang (
	_branchid	VARCHAR,
	_srepid		VARCHAR,
	_canvasid	VARCHAR
) RETURNS SETOF tAgingPiutang AS
$BODY$
DECLARE
	_retval		tAgingPiutang%ROWTYPE;
	_bpid		VARCHAR;
	_arbalance	NUMERIC(19,4);
	_arlimitamt	NUMERIC(19,4);
	_bpname		VARCHAR;
	_bpareaid		VARCHAR;
	_sale1		NUMERIC(19,4);
	_sale2		NUMERIC(19,4);
	_sale3		NUMERIC(19,4);
	_sale4		NUMERIC(19,4);
	_sale5		NUMERIC(19,4);
	_sret1		NUMERIC(19,4);
	_sret2		NUMERIC(19,4);
	_sret3		NUMERIC(19,4);
	_sret4		NUMERIC(19,4);
	_sret5		NUMERIC(19,4);
	_fcn1		NUMERIC(19,4);
	_fcn2		NUMERIC(19,4);
	_fcn3		NUMERIC(19,4);
	_fcn4		NUMERIC(19,4);
	_fcn5		NUMERIC(19,4);
	_rcv1		NUMERIC(19,4);
	_rcv2		NUMERIC(19,4);
	_rcv3		NUMERIC(19,4);
	_rcv4		NUMERIC(19,4);
	_rcv5		NUMERIC(19,4);
	_total1		NUMERIC(19,4);
	_total2		NUMERIC(19,4);
	_total3		NUMERIC(19,4);
	_total4		NUMERIC(19,4);
	_total5		NUMERIC(19,4);
	_totalAll	NUMERIC(19,4);
	_bpgrpid	VARCHAR;
	cBP		REFCURSOR;
	_crcID		VARCHAR;

BEGIN
	OPEN cBP FOR
		SELECT bp.bpid, bpgrpid, arbalance, arlimitamt, bpacc.crcid, a.areaid 
		FROM bpacc 
		JOIN bp ON bpacc.bpid=bp.bpid LEFT JOIN addr a ON bp.bpid=a.bpid WHERE arbalance<>0 order by bp.bpid;
	LOOP
		FETCH cBP INTO _bpid,_bpgrpid,_arbalance,_arlimitamt,_crcid,_bpareaid;
		EXIT WHEN NOT FOUND;

		SELECT INTO _bpname bpname FROM bp WHERE bpid=_bpid;

		SELECT INTO _sale1, _sale2, _sale3, _sale4, _sale5
		SUM(CASE WHEN CURRENT_DATE-s.saledate<=7 THEN (s.total-s.paidamt) ELSE 0 END) AS q1,
		SUM(CASE WHEN CURRENT_DATE-s.saledate<=15 AND CURRENT_DATE-(s.saledate)>=8 THEN (s.total-s.paidamt) ELSE 0 END) AS q2,
		SUM(CASE WHEN CURRENT_DATE-s.saledate<=30 AND CURRENT_DATE-(s.saledate)>=16 THEN (s.total-s.paidamt) ELSE 0 END) AS q3,
		SUM(CASE WHEN CURRENT_DATE-s.saledate<=45 AND CURRENT_DATE-(s.saledate)>=31 THEN (s.total-s.paidamt) ELSE 0 END) AS q4,
		SUM(CASE WHEN CURRENT_DATE-s.saledate>45 THEN (s.total-s.paidamt) ELSE 0 END) AS q5
		FROM sale s
		WHERE s.saledate<=CURRENT_DATE AND custid=_bpid AND isdraft=false AND crcid=_crcid 
		AND (_branchid is null or branchid=_branchid) AND (_srepid is null or srepid=_srepid) AND (_canvasid is null or canvasid=_canvasid);

		SELECT INTO _sret1,_sret2,_sret3,_sret4,_sret5
		SUM(CASE WHEN CURRENT_DATE-(s.sretdate)<=7 THEN s.total ELSE 0 END) AS q1,
		SUM(CASE WHEN CURRENT_DATE-s.sretdate<=15 AND CURRENT_DATE-(s.sretdate)>=8 THEN s.total ELSE 0 END) AS q2,
		SUM(CASE WHEN CURRENT_DATE-s.sretdate<=30 AND CURRENT_DATE-(s.sretdate)>=16 THEN s.total ELSE 0 END) AS q3,
		SUM(CASE WHEN CURRENT_DATE-s.sretdate<=45 AND CURRENT_DATE-(s.sretdate)>=31 THEN s.total ELSE 0 END) AS q4,
		SUM(CASE WHEN CURRENT_DATE-s.sretdate>45 THEN s.total ELSE 0 END) AS q5
		FROM sret s
		WHERE custid=_bpid AND isdraft=false AND iscontra=false AND crcid=_crcid 
		AND (_branchid is null or branchid=_branchid) AND (_srepid is null or srepid=_srepid) AND (_canvasid is null or canvasid=_canvasid);


		SELECT INTO _fcn1,_fcn2,_fcn3,_fcn4,_fcn5
		SUM(CASE WHEN CURRENT_DATE-fcndate<=7 THEN fcnamt ELSE 0 END) AS q1,
		SUM(CASE WHEN CURRENT_DATE-fcndate<=15 AND CURRENT_DATE-fcndate>=8 THEN fcnamt ELSE 0 END) AS q2,
		SUM(CASE WHEN CURRENT_DATE-fcndate<=30 AND CURRENT_DATE-fcndate>=16 THEN fcnamt ELSE 0 END) AS q3,
		SUM(CASE WHEN CURRENT_DATE-fcndate<=45 AND CURRENT_DATE-fcndate>=31 THEN fcnamt ELSE 0 END) AS q4,
		SUM(CASE WHEN CURRENT_DATE-fcndate>45 THEN fcnamt ELSE 0 END) AS q5
		FROM fcn 
		WHERE bpid=_bpid AND iscontra=false AND (_branchid is null or branchid=_branchid);

		/*SELECT INTO _rcv1,_rcv2,_rcv3,_rcv4,_rcv5
		SUM(CASE WHEN CURRENT_DATE-rcvdate<=7 THEN spaidamt ELSE 0 END) AS q1,
		SUM(CASE WHEN CURRENT_DATE-rcvdate<=15 AND CURRENT_DATE-rcvdate>=8 THEN spaidamt ELSE 0 END) AS q2,
		SUM(CASE WHEN CURRENT_DATE-rcvdate<=30 AND CURRENT_DATE-rcvdate>=16 THEN spaidamt ELSE 0 END) AS q3,
		SUM(CASE WHEN CURRENT_DATE-rcvdate<=45 AND CURRENT_DATE-rcvdate>=31 THEN spaidamt ELSE 0 END) AS q4,
		SUM(CASE WHEN CURRENT_DATE-rcvdate>45 THEN spaidamt ELSE 0 END) AS q5
		FROM spaid left join rcv on spaid.rcvno=rcv.rcvno
		WHERE bpid=_bpid AND (spaid.refno IN (SELECT saleno FROM sale WHERE custid=_bpid) OR spaid.refno IN (SELECT sretno FROM sret WHERE custid=_bpid) OR spaid.refno IN (SELECT fcnno FROM fcn WHERE bpid=_bpid));*/

		_total1:=coalesce(_sale1,0)-coalesce(_sret1,0)-coalesce(_fcn1,0);
		_total2:=coalesce(_sale2,0)-coalesce(_sret2,0)-coalesce(_fcn2,0);
		_total3:=coalesce(_sale3,0)-coalesce(_sret3,0)-coalesce(_fcn3,0);
		_total4:=coalesce(_sale4,0)-coalesce(_sret4,0)-coalesce(_fcn4,0);
		_total5:=coalesce(_sale5,0)-coalesce(_sret5,0)-coalesce(_fcn5,0);
		_totalAll:=_total1+_total2+_total3+_total4+_total5;

		_retval.bpid:=_bpid;
		_retval.bpname:=_bpname;
		_retval.total1:=_total1;
		_retval.total2:=_total2;
		_retval.total3:=_total3;
		_retval.total4:=_total4;
		_retval.total5:=_total5;
		_retval.arbalance:=_totalAll;
		_retval.arlimitamt:=_arlimitamt;
		_retval.bpgrpid:=_bpgrpid;
		_retval.bpareaid:=_bpareaid;
		_retval.crcid:=_crcid;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cBP;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

---------------------------------- OVERLOAD tanpa srepid, canvasid --------------------------------------------

CREATE OR REPLACE FUNCTION fAgingPiutang (
	_branchid	VARCHAR
) RETURNS SETOF tAgingPiutang AS
$BODY$
DECLARE
	_retval			tAgingPiutang%ROWTYPE;
	cBP			REFCURSOR;
BEGIN
	OPEN cBP FOR
		SELECT * FROM fAgingPiutang(_branchid, null, null);
	LOOP
		FETCH cBP INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cBP;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fAgingHutang (
	_branchid	VARCHAR
) RETURNS SETOF tAgingHutang AS
$BODY$
DECLARE
	_retval		tAgingHutang%ROWTYPE;
	_bpid		VARCHAR;
	_apbalancebc	NUMERIC(19,4);
	_aplimitamt	NUMERIC(19,4);
	_bpname		VARCHAR;
	_bpareaid	VARCHAR;
	_purc1		NUMERIC(19,4);
	_purc2		NUMERIC(19,4);
	_purc3		NUMERIC(19,4);
	_purc4		NUMERIC(19,4);
	_purc5		NUMERIC(19,4);
	_pret1		NUMERIC(19,4);
	_pret2		NUMERIC(19,4);
	_pret3		NUMERIC(19,4);
	_pret4		NUMERIC(19,4);
	_pret5		NUMERIC(19,4);
	_fcn1		NUMERIC(19,4);
	_fcn2		NUMERIC(19,4);
	_fcn3		NUMERIC(19,4);
	_fcn4		NUMERIC(19,4);
	_fcn5		NUMERIC(19,4);
	_pay1		NUMERIC(19,4);
	_pay2		NUMERIC(19,4);
	_pay3		NUMERIC(19,4);
	_pay4		NUMERIC(19,4);
	_pay5		NUMERIC(19,4);
	_total1		NUMERIC(19,4);
	_total2		NUMERIC(19,4);
	_total3		NUMERIC(19,4);
	_total4		NUMERIC(19,4);
	_total5		NUMERIC(19,4);
	_totalAll	NUMERIC(19,4);
	_bpgrpid	VARCHAR;
	_crcid		VARCHAR;
	cBP		REFCURSOR;
BEGIN
	OPEN cBP FOR
		SELECT bp.bpid, bpgrpid, apbalance, aplimitamt, bpacc.crcid, a.areaid 
		FROM bpacc 
		JOIN bp ON bpacc.bpid=bp.bpid 
		JOIN bpbptype ON bp.bpid=bpbptype.bpid 
		LEFT JOIN addr a ON bp.bpid=a.bpid 
		WHERE apbalance<>0 AND bptype='VEND' 
		ORDER BY bp.bpid;
	LOOP
		FETCH cBP INTO _bpid, _bpgrpid, _apbalancebc, _aplimitamt, _crcid, _bpareaid;
		EXIT WHEN NOT FOUND;

		SELECT INTO _bpname bpname FROM bp WHERE bpid=_bpid;

		SELECT INTO _purc1, _purc2, _purc3, _purc4 ,_purc5
		SUM(CASE WHEN CURRENT_DATE-p.purcdate<=7 THEN (p.total-p.paidamt) ELSE 0 END) AS q1,
		SUM(CASE WHEN CURRENT_DATE-p.purcdate<=15 AND CURRENT_DATE-(p.purcdate)>=8 THEN (p.total-p.paidamt) ELSE 0 END) AS q2,
		SUM(CASE WHEN CURRENT_DATE-p.purcdate<=30 AND CURRENT_DATE-(p.purcdate)>=16 THEN (p.total-p.paidamt) ELSE 0 END) AS q3,
		SUM(CASE WHEN CURRENT_DATE-p.purcdate<=45 AND CURRENT_DATE-(p.purcdate)>=31 THEN (p.total-p.paidamt) ELSE 0 END) AS q4,
		SUM(CASE WHEN CURRENT_DATE-p.purcdate>45 THEN (p.total-p.paidamt) ELSE 0 END) AS q5
		FROM purc p
		WHERE p.purcdate<=CURRENT_DATE 
		AND vendorid=_bpid AND isdraft=FALSE AND crcid=_crcid AND (_branchid IS NULL OR branchid=_branchid);

		SELECT INTO _pret1, _pret2, _pret3, _pret4, _pret5
		SUM(CASE WHEN CURRENT_DATE-(p.pretdate)<=7 THEN p.total ELSE 0 END) AS q1,
		SUM(CASE WHEN CURRENT_DATE-p.pretdate<=15 AND CURRENT_DATE-(p.pretdate)>=8 THEN p.total ELSE 0 END) AS q2,
		SUM(CASE WHEN CURRENT_DATE-p.pretdate<=30 AND CURRENT_DATE-(p.pretdate)>=16 THEN p.total ELSE 0 END) AS q3,
		SUM(CASE WHEN CURRENT_DATE-p.pretdate<=45 AND CURRENT_DATE-(p.pretdate)>=31 THEN p.total ELSE 0 END) AS q4,
		SUM(CASE WHEN CURRENT_DATE-p.pretdate>45 THEN p.total ELSE 0 END) AS q5
		FROM pret p
		WHERE vendorid=_bpid AND isdraft=FALSE AND iscontra=FALSE AND crcid=_crcid AND (_branchid IS NULL OR branchid=_branchid);

		SELECT INTO _fcn1, _fcn2, _fcn3, _fcn4, _fcn5
		SUM(CASE WHEN CURRENT_DATE-fcndate<=7 THEN fcnamt ELSE 0 END) AS q1,
		SUM(CASE WHEN CURRENT_DATE-fcndate<=15 AND CURRENT_DATE-fcndate>=8 THEN fcnamt ELSE 0 END) AS q2,
		SUM(CASE WHEN CURRENT_DATE-fcndate<=30 AND CURRENT_DATE-fcndate>=16 THEN fcnamt ELSE 0 END) AS q3,
		SUM(CASE WHEN CURRENT_DATE-fcndate<=45 AND CURRENT_DATE-fcndate>=31 THEN fcnamt ELSE 0 END) AS q4,
		SUM(CASE WHEN CURRENT_DATE-fcndate>45 THEN fcnamt ELSE 0 END) AS q5
		FROM fcn 
		WHERE bpid=_bpid AND iscontra=FALSE AND (_branchid IS NULL or branchid=_branchid);

		/*SELECT INTO _pay1, _pay2, _pay3, _pay4, _pay5
		SUM(CASE WHEN CURRENT_DATE-paydate<=7 THEN ppaidamt ELSE 0 END) AS q1,
		SUM(CASE WHEN CURRENT_DATE-paydate<=15 AND CURRENT_DATE-paydate>=8 THEN ppaidamt*pay.excrate ELSE 0 END) AS q2,
		SUM(CASE WHEN CURRENT_DATE-paydate<=30 AND CURRENT_DATE-paydate>=16 THEN ppaidamt*pay.excrate ELSE 0 END) AS q3,
		SUM(CASE WHEN CURRENT_DATE-paydate<=45 AND CURRENT_DATE-paydate>=31 THEN ppaidamt*pay.excrate ELSE 0 END) AS q4,
		SUM(CASE WHEN CURRENT_DATE-paydate>45 THEN ppaidamt*pay.excrate ELSE 0 END) AS q5
		FROM ppaid 
		JOIN pay ON ppaid.payno=pay.payno
		WHERE bpid=_bpid AND (ppaid.refno IN (SELECT purcno FROM purc WHERE vendorid=_bpid AND isdraft=FALSE) OR ppaid.refno IN (SELECT pretno FROM pret WHERE vendorid=_bpid AND isdraft=FALSE) OR ppaid.refno IN (SELECT fcnno FROM fcn WHERE bpid=_bpid));*/

		_total1:=COALESCE(_purc1,0)-COALESCE(_pret1,0)-COALESCE(_fcn1,0);
		_total2:=COALESCE(_purc2,0)-COALESCE(_pret2,0)-COALESCE(_fcn2,0);
		_total3:=COALESCE(_purc3,0)-COALESCE(_pret3,0)-COALESCE(_fcn3,0);
		_total4:=COALESCE(_purc4,0)-COALESCE(_pret4,0)-COALESCE(_fcn4,0);
		_total5:=COALESCE(_purc5,0)-COALESCE(_pret5,0)-COALESCE(_fcn5,0);
		_totalAll:=_total1+_total2+_total3+_total4+_total5;

		_retval.bpid:=_bpid;
		_retval.bpname:=_bpname;
		_retval.total1:=_total1;
		_retval.total2:=_total2;
		_retval.total3:=_total3;
		_retval.total4:=_total4;
		_retval.total5:=_total5;
		_retval.apbalancebc:=_totalAll;
		_retval.aplimitamt:=_aplimitamt;
		_retval.bpgrpid:=_bpgrpid;
		_retval.bpareaid:=_bpareaid;
		_retval.crcid:=_crcid;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cBP;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spBegBal_Remove(
	_AccNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	cJnl		REFCURSOR;
	_JnlNo		VARCHAR;
	_spret		SPStatus%ROWTYPE;
	_retval		SPStatus%ROWTYPE;
BEGIN
	OPEN cJnl FOR
		SELECT jnld.jnlno 
		FROM jnld 
		JOIN jnl ON jnld.jnlno=jnl.jnlno 
		WHERE accno=_AccNo AND jnltype='BEG';
	LOOP
		FETCH cJnl INTO _JnlNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spJnl_void(_JnlNo);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cJnl;

	UPDATE begbal SET balance=0 WHERE accno=_AccNo;

	_retval:=(0, 'REMOVE SUKSES');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



/**
	fPositivOrZero() -> fPoZ
	
	ALWAYS RETURN :
		- Positif value of parameter 
		- OR ZERO (if negatif)

**/

CREATE OR REPLACE FUNCTION fPoZ (
	_value	NUMERIC
) RETURNS NUMERIC AS 
$BODY$
DECLARE
	_retval		NUMERIC;
BEGIN
	
	IF (_value<=0) THEN
		_retval:=0;
	ELSE
		_retval:=coalesce(_value, 0);
	END IF;


	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPOList (
	_ItemID	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;

	cPO		REFCURSOR;

	_PONo		VARCHAR;
	_First		BOOLEAN;
BEGIN
	_First:=TRUE;
	_retval:='';

	OPEN cPO FOR
		SELECT DISTINCT m.PONo FROM PO m LEFT JOIN POD d ON d.pono=m.pono WHERE ItemID=_ItemID AND m.active=TRUE AND (fQtyZToQty1(d.itemid,d.qty,d.unit)>(d.qtydelivered+d.qtybo)); 	
	LOOP
		FETCH cPO INTO _PONo;
		EXIT WHEN NOT FOUND;
		RAISE NOTICE '%', _PONo;

		IF (_PONo IS NOT NULL AND char_length(_PONo)>0) THEN
			IF (_First=TRUE) THEN
				_retval:=_retval||_PONo;
				_First:=FALSE;
			ELSE
				_retval:=_retval||','||_PONo;
			END IF;
		END IF;
	END LOOP;
	CLOSE cPO;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSOList (
	_ItemID		VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	
	cSO		REFCURSOR;

	_SONo		VARCHAR;
	_First		BOOLEAN;
BEGIN
	_First:=TRUE;
	_retval:='';

	OPEN cSO FOR
		SELECT DISTINCT m.SONo 
		FROM SO m 
		JOIN SOD d ON d.sono=m.sono 
		WHERE ItemID=_ItemID AND m.active=TRUE AND (fQtyZToQty1(d.itemid,d.qty,d.unit)>(d.qtydelivered+d.qtybo)); 
	LOOP
		FETCH cSO INTO _SONo;
		EXIT WHEN NOT FOUND;

		IF (_SONo IS NOT NULL AND char_length(_SONo)>0) THEN
			IF (_First=TRUE) THEN
				_retval:=_retval||_SONo;
				_First:=FALSE;
			ELSE
				_retval:=_retval||','||_SONo;
			END IF;
		END IF;
	END LOOP;
	CLOSE cSO;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSaleQty (
	_SaleNo		VARCHAR
) RETURNS NUMERIC(15,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(15,4);
BEGIN
	SELECT INTO _retval sum(fQtyZtoQty1(itemid, qty, unit)) 
	FROM saled 
	WHERE saleno=_SaleNo;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tCreateFA_Depr CASCADE;
CREATE TYPE tCreateFA_Depr AS (
	depramt		NUMERIC(19,4),
	deprmonth	INT,
	accumytd	NUMERIC(19,4)
);

CREATE OR REPLACE FUNCTION fCreateFA_Depr(
	_FAID			VARCHAR,
	_DeprID			VARCHAR,
	_MaxDeprMonth		INT,
	__TotalDeprMonth	INT,
	_EcLife			NUMERIC(19,4),
	_MaxAccumCost		NUMERIC(19,4),
	_AcqDate		DATE,
	_AcqCost		NUMERIC(19,4),
	_RsdValue		NUMERIC(19,4),
	_PerStart		DATE
) RETURNS SETOF tCreateFA_Depr AS
$BODY$
DECLARE
	_retval			tCreateFA_Depr%ROWTYPE;

	_AkumulasiPerTahun	NUMERIC(19,4);
	_AkumulasiPerBulan	NUMERIC(19,4);
	_SelisihPerTahun	NUMERIC(19,4);
	_SelisihPerLife		NUMERIC(19,4);

	_PersenPerTahun		NUMERIC(19,4);
	_YearCounter		INT;
	_YearRemain		INT;
	_OldYearCounter		INT;

	_DeprAmt		NUMERIC(19,4);
	_TotalDeprAmt		NUMERIC(19,4);

	_LastDeprMonth		INT;

	_MonthIncrement		INT;

	_FADeprNo		VARCHAR;
	_DeprDate		DATE;
	
	_LockDate		DATE;

	_AccHist		VARCHAR;
	_AccAccum		VARCHAR;
	_AccAsset		VARCHAR;
	_FAName			VARCHAR;
	_FABV			NUMERIC(19,4);
	_BranchID		VARCHAR;
	_DeptID			VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_JnlNo			VARCHAR;
	_JnlNote		VARCHAR;
	_JnlDNo			INT2;

	_TotalDeprMonth		INT;
BEGIN
	/** Inisialisasi */
	_retval.depramt:=0;
	_retval.deprmonth:=0;
	_retval.accumytd:=0;

	_AkumulasiPerTahun:=0;
	_AkumulasiPerBulan:=0;
	_SelisihPerTahun:=0;
	_SelisihPerLife:=0;
	_DeprAmt:=0;
	_TotalDeprAmt:=0;
	_LastDeprMonth:=0;
	_YearCounter:=0;
	_OldYearCounter:=-1;
	_TotalDeprMonth:=__TotalDeprMonth;
	
	SELECT INTO _LastDeprMonth COALESCE(CAST(MAX(accummonth) AS INT), 0) 
	FROM fadepr WHERE faid=_FAID AND accummonth<_TotalDeprMonth;

	SELECT INTO _AccAsset, _AccAccum, _FAName, _BranchID, _DeptID, _FABV 
		accasset, accaccum, faname, branchid, deptid, fabv  
	FROM fa WHERE faid=_FAID; 

	SELECT INTO _CrcID crcid FROM crc WHERE isdefault=true;
	_ExcRate:=1;
	_FisRate:=1;

	SELECT INTO _LockDate lockdate FROM tlock WHERE transtype='JNL';
	IF NOT FOUND THEN
		_LockDate:=NULL;
	END IF;

	IF (_TotalDeprMonth>_MaxDeprMonth) THEN
		_TotalDeprMonth:=_MaxDeprMonth;
	END IF;

	_MonthIncrement:=_TotalDeprMonth-_LastDeprMonth;

	IF (_DeprID='SLM') THEN /** Metode garis lurus */
		_AkumulasiPerTahun:=ROUND(_MaxAccumCost/_EcLife);
		_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
		_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
		_SelisihPerLife:=_MaxAccumCost-(_AkumulasiPerTahun*_EcLife);

		WHILE (_MonthIncrement>0) LOOP
			_LastDeprMonth:=_LastDeprMonth+1;
			_FADeprNo:=fTransNoGet('FADEPR');
			_DeprDate:=CAST(CAST(DATE_TRUNC('month', _AcqDate) AS DATE)+CAST(_LastDeprMonth||'MONTH' AS INTERVAL) AS DATE)-1;
			
			IF (_LockDate IS NOT NULL AND _DeprDate<_LockDate) THEN
				RETURN NEXT _retval;
				RETURN;
			END IF;
			
			IF (MOD(_LastDeprMonth, 12)=0) THEN
				_DeprAmt:=_AkumulasiPerBulan+_SelisihPerTahun;
			ELSE	
				_DeprAmt:=_AkumulasiPerBulan;
			END IF;
			
			IF (_LastDeprMonth=_MaxDeprMonth) THEN /** Bulan Terakhir */
				_DeprAmt:=_DeprAmt+_SelisihPerLife;
			END IF;

			_TotalDeprAmt:=_TotalDeprAmt+_DeprAmt;

			_MonthIncrement:=_MonthIncrement-1;
		END LOOP;
	ELSIF (_DeprID='DBM') THEN /** Metode saldo menurun */
		_PersenPerTahun:=(100/_EcLife)/100;
		_AkumulasiPerTahun:=_MaxAccumCost*_PersenPerTahun;
		
		_PersenPerTahun:=(_AkumulasiPerTahun/_MaxAccumCost)*2;
		IF (_PersenPerTahun>0.5) THEN
			_PersenPerTahun:=0.5;
		END IF;
		
		WHILE (_MonthIncrement>0) LOOP
			_LastDeprMonth:=_LastDeprMonth+1;
			_YearCounter:=(_LastDeprMonth/12);
			_YearRemain:=MOD(_LastDeprMonth, 12);
			
			IF ((_YearCounter>_OldYearCounter AND _YearRemain<>0) OR (_YearCounter=_OldYearCounter AND _YearRemain=1)) THEN
				_OldYearCounter:=_YearCounter;

				IF (_YearCounter<(_EcLife-1)) THEN
					_AkumulasiPerTahun:=_PersenPerTahun*POW((1-_PersenPerTahun), (_YearCounter-1))*(_AcqCost-(_PersenPerTahun*_AcqCost));
--					IF (_LastDeprMonth=25) THEN
--						RAISE EXCEPTION 'GJL 1 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;
				ELSIF (_YearCounter=(_EcLife-1)) THEN
					_AkumulasiPerTahun:=POW((1-_PersenPerTahun), (_YearCounter-1))*(_AcqCost-(_PersenPerTahun*_AcqCost))-_RsdValue;	
--					IF (_LastDeprMonth=25) THEN
--						RAISE EXCEPTION 'GJL 2 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;		
				END IF;

				_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
				_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
				_FABV:=_FABV-_AkumulasiPerTahun;
			ELSIF (_YearCounter>_OldYearCounter AND _YearRemain=0) THEN
				_OldYearCounter:=_YearCounter;
				
				IF (_YearCounter<_EcLife) THEN
					_AkumulasiPerTahun:=_PersenPerTahun*POW((1-_PersenPerTahun), (_YearCounter-2))*(_AcqCost-(_PersenPerTahun*_AcqCost));
--					IF (_LastDeprMonth=36) THEN
--						RAISE EXCEPTION 'GNP 1 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;
				ELSIF (_YearCounter=_EcLife) THEN
					_AkumulasiPerTahun:=POW((1-_PersenPerTahun), (_YearCounter-2))*(_AcqCost-(_PersenPerTahun*_AcqCost))-_RsdValue;
--					IF (_LastDeprMonth=48) THEN
--						RAISE EXCEPTION 'GNP 2 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;					
				END IF;				
				
				_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
				_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
				_FABV:=_FABV-_AkumulasiPerTahun;
			END IF;			
			
--			IF (_LastDeprMonth=48) THEN
--				RAISE EXCEPTION '0 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--			END IF;

			_FADeprNo:=fTransNoGet('FADEPR');
			_DeprDate:=CAST(CAST(DATE_TRUNC('month', _AcqDate) AS DATE)+CAST(_LastDeprMonth||'MONTH' AS INTERVAL) AS DATE)-1;
			
			IF (_LockDate IS NOT NULL AND _DeprDate<_LockDate) THEN
				RETURN NEXT _retval;
				RETURN;
			END IF;

			IF (MOD(_LastDeprMonth, 12)=0) THEN
				_DeprAmt:=_AkumulasiPerBulan+_SelisihPerTahun;
			ELSE	
				_DeprAmt:=_AkumulasiPerBulan;
			END IF;
			
			IF (_LastDeprMonth=_MaxDeprMonth) THEN /** Bulan Terakhir */
				_DeprAmt:=_DeprAmt+_SelisihPerLife;
			END IF;

			_TotalDeprAmt:=_TotalDeprAmt+_DeprAmt;

			_MonthIncrement:=_MonthIncrement-1;
		END LOOP;
	END IF;
	
	_retval.depramt:=_TotalDeprAmt;
	_retval.deprmonth:=_LastDeprMonth;
	_retval.accumytd:=_AkumulasiPerTahun;
	
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



/** 
	SP yang dipanggil waktu buat FA

	Dependensi :
	1. fFixedAsset.sql

	NTH :
	1. Jika sudah tutup buku, tapi bikin saldo awal FA lagi 

*/

CREATE OR REPLACE FUNCTION spFixedAsset_New(_FAID VARCHAR) 
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_AcqCost		NUMERIC(19,4);	/** Harga Beli */
	_AcqDate		DATE;			/** Tanggal Beli */
	_UsageDate		DATE;
	_DeprID			VARCHAR;		/** Metode Depresiasi */
	_EcLife			NUMERIC(19,4);	/** Umur Ekonomis */
	_CurrAccumCost		NUMERIC(19,4);	/** Nilai Akumulasi Saat Ini */
	_RsdValue		NUMERIC(19,4);	/** Nilai Residu */
	_FABV			NUMERIC(19,4);	/** Nilai Buku Fixed Asset */
	_IsPosted		BOOLEAN;
	_FAIType		VARCHAR;
	_IsBegBal		BOOLEAN;
	_DoGenJnl		BOOLEAN;

	_StartMonth		INT;			/** Bulan Beli */
	_StartYear		INT;			/** Tahun Beli */
	_CurrMonth		INT;			/** Bulan Periode Akuntansi */			
	_CurrYear		INT;			/** Tahun Periode Akuntansi */
	_PerStart		DATE;			/** Periode Awal Akuntansi */
	
	_TotalDeprMonth		INT;			/** Jumlah Bulan Depresiasi Saat Ini (_PerStart-_UsageDate) */
	_MaxDeprMonth		INT;			/** Total Jumlah Bulan Depresiasi (_EcLife*12) */
	
	_MaxAccumCost		NUMERIC(19,4);	/** Nilai Maximum Akumulasi Depresiasi (_AcqCost-_RsdValue) */

	_TotalDeprAmt		NUMERIC(19,4);	/** Total Nilai Depresiasi Selama _TotalDeprMonth */

	_FADeprNo		VARCHAR;
BEGIN
	/** Inisialisasi */
	_TotalDeprMonth:=0;
	_MaxDeprMonth:=0;
	_MaxAccumCost:=0;

	_TotalDeprAmt:=0;
	_IsPosted:=FALSE;

	/** Validasi ketersediaan data */
	SELECT INTO _AcqCost, _AcqDate, _DeprID, _EcLife, _CurrAccumCost, _RsdValue, _FAIType, _IsPosted, _UsageDate  
			acqcost, acqdate, deprid, eclife, COALESCE(accumcost, 0), rsdval, faitype, isposted, usagedate 	
	FROM fa WHERE faid=_FAID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Harta Tetap#'||_FAID||' tidak ditemukan (spFixedAsset_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_IsPosted=FALSE) THEN
		RETURN NEXT _retval;
		RETURN;
	END IF;

	/** Ambil data periode akuntansi */
	SELECT INTO _PerStart perstart 
	FROM per WHERE active=TRUE;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Periode Akuntansi belum diset dalam sistem (spFixedAsset_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_FAIType='BB' OR _FAIType='FB') THEN
		_IsBegBal:=TRUE;
		_DoGenJnl:=FALSE;
	ELSE
		_IsBegBal:=FALSE;
		_DoGenJnl:=TRUE;
	END IF;

	_MaxDeprMonth:=CAST(_EcLife AS INT) * 12;
	_MaxAccumCost:=_AcqCost-_RsdValue;
	_FABV:=_AcqCost; /** Nilai Buku Awal = Harga Beli */

	IF (_UsageDate>_PerStart) THEN
		_PerStart:=_UsageDate;
	END IF;

	_StartMonth:=DATE_PART('month', _UsageDate);
	_StartYear:=DATE_PART('year', _UsageDate);
	_CurrMonth:=DATE_PART('month', _PerStart);
	_CurrYear:=DATE_PART('year', _PerStart);

	/** Hitung Jumlah Bulan Depresiasi */
	IF (_CurrYear=_StartYear) THEN
		_TotalDeprMonth:=_CurrMonth-_StartMonth;
	ELSIF (_CurrYear>_StartYear) THEN
		_TotalDeprMonth:=(12-_StartMonth)+((_CurrYear-_StartYear-1)*12)+_CurrMonth;
	ELSE
		_TotalDeprMonth:=0; /** kalo tahun beli > tahun sekarang, berarti belum ada depresiasi */
	END IF;

	/** SET BEBAN PER BULANNYA */
	UPDATE fa SET famc=0, fabv=_FABV, active=TRUE, accumytd=0, accumcost=0, lastdeprdate=NULL  
	WHERE faid=_FAID;
	
	SELECT INTO _spret * FROM spCreateFA_Depr(_FAID, _DeprID, _MaxDeprMonth, _TotalDeprMonth, _EcLife, _MaxAccumCost, 
		_AcqDate, _AcqCost, _RsdValue, _PerStart, _UsageDate, _DoGenJnl, _IsBegBal);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFixedAsset_Void(_FAID VARCHAR) 
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_FAIType		VARCHAR;	
	_CountDeprNonBB		INT;	/** Counter untuk depresiasi non-begbal */	
BEGIN
	/** Inisialisasi */
	
	SELECT INTO _FAIType faitype
	FROM fa WHERE faid=_FAID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Harta Tetap #'||_FAID||' tidak ditemukan (spFixedAsset_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 
	
	/** Diasumsikan Fixed Asset selalu ada jika modul akuntansi aktif */
	SELECT INTO _CountDeprNonBB COALESCE(COUNT(fd.*), 0) 
	FROM fadepr fd JOIN jnl ON jnl.refno=fd.fadeprno WHERE fd.faid=_FAID;

	IF (_CountDeprNonBB>0) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Sudah ada depresiasi sebelumnya, '||E'\n'||'silahkan hapus terlebih dahulu depresiasi untuk Harta Tetap ini (spFixedAsset_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (isAccInstalled()) THEN
		SELECT INTO _spret * FROM spJnl_Void_byRef('FA',_FAID);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM fadepr WHERE faid=_FAID;
	DELETE FROM fa WHERE faid=_FAID;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFixedAsset_Depr_New(_FADeprNo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
BEGIN
	/** Inisialisasi */
	_retval:=(0, 'OK');

	SELECT INTO _spret * FROM spFixedAsset_Depr_New(_FADeprNo, TRUE);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spFixedAsset_Depr_New(_FADeprNo VARCHAR, _DoGenJnl BOOLEAN)
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	
	_PerStart		DATE;
	
	_FAID			VARCHAR;
	_FADeprAmt		NUMERIC(19,4);
	_FADeprDate		DATE;

	_FAName			VARCHAR;
	_AccCost		VARCHAR;
	_AccAccum		VARCHAR;
	_DeptID			VARCHAR;
	_BranchID		VARCHAR;
	_AccumCost		NUMERIC(19,4);
	_FAMC			NUMERIC(19,4);
	_IsActive		BOOLEAN;
	_FABV			NUMERIC(19,4);
	_RsdVal			NUMERIC(19,4);
	_LastDeprDate		DATE;

	_Month			INT4;
	_Year			INT4;
	_Bulan			VARCHAR;

	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_CountFADepr		INT;
	_CurrAccumCost		NUMERIC(19,4);

	_JnlNo			VARCHAR;
	_JnlNote		VARCHAR;
BEGIN
	/** Inisialisasi */

	/** Cek periode Akuntansi */
	SELECT INTO _PerStart perstart FROM per WHERE active=true;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Periode akuntansi belum diset dalam sistem (spFixedAsset_Depr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** Cek data depresiasi */
	SELECT INTO _FAID, _FADeprAmt, _FADeprDate faid, depramt, fadeprdate
	FROM fadepr WHERE fadeprno=_FADeprNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Depresiasi Harta Tetap#'||_FADeprNo||' tidak ditemukan (spFixedAsset_Depr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/** Cek data FA */
	SELECT INTO _FAName, _AccCost, _AccAccum, _DeptID, _BranchID, _AccumCost, _FAMC, _IsActive, _FABV, _RsdVal, _LastDeprDate 
			faname,	acccost, accaccum, deptid, branchid, accumcost, famc, active, fabv, rsdval, lastdeprdate 
	FROM FA WHERE FAID=_FAID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Harta Tetap #'||_FAID||' tidak ditemukan (spFixedAsset_Depr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;	
	
	/** Jika FA tidak aktif, skip proses selanjutnya */
	IF (_isActive=FALSE) THEN
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Month:=DATE_PART('month',_FADeprDate);
	_Year:=DATE_PART('year',_FADeprDate);

	SELECT INTO _CrcID crcid FROM crc WHERE isdefault=true;
	_ExcRate:=1;
	_FisRate:=1;
	
	SELECT INTO _CountFADepr COALESCE(COUNT(*),0) 
	FROM fadepr WHERE faid=_FAID AND DATE_PART('month',fadeprdate)=_Month 
	AND DATE_PART('year',fadeprdate)=_Year;	
	IF(_CountFADepr>1) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Sudah ada depresiasi untuk '||_FAName||' pada bulan ini! (spFixedAsset_Depr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_LastDeprDate IS NOT NULL AND _LastDeprDate>_FADeprDate) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Sudah ada depresiasi lain yang menghandle '||_FAName||' untuk bulan ini (spFixedAsset_Depr_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;
	
	_CurrAccumCost:=_AccumCost+_FADeprAmt;
	_FABV:=_FABV-_FADeprAmt;
	
	IF (_FABV=_RsdVal) THEN
		_IsActive=FALSE;
	END IF;

	/** Jika nilai residu adalah nol dan nilai buku saat ini adalah nol, maka set nilai buku = 1 */	
	IF (_RsdVal=0 AND _FABV=0) THEN
		_FABV:=1;
	END IF;

	UPDATE fa SET accumcost=_CurrAccumCost, famc=(CASE WHEN (_IsActive=TRUE) THEN _FADeprAmt ELSE 0 END), 
	fabv=_FABV, lastdeprdate=_FADeprDate, active=_IsActive, isposted=TRUE WHERE faid=_FAID;
	
	/** AKUNTANSI */
	IF (isAccInstalled() AND _DoGenJnl=TRUE) THEN
		IF(_Month=1) THEN
			_Bulan:='Januari';
		ELSIF(_Month=2) THEN
			_Bulan:='Februari';
		ELSIF(_Month=3) THEN
			_Bulan:='Maret';
		ELSIF(_Month=4) THEN
			_Bulan:='April';
		ELSIF(_Month=5) THEN
			_Bulan:='Mei';
		ELSIF(_Month=6) THEN
			_Bulan:='Juni';
		ELSIF(_Month=7) THEN
			_Bulan:='Juli';
		ELSIF(_Month=8) THEN
			_Bulan:='Agustus';
		ELSIF(_Month=9) THEN
			_Bulan:='September';
		ELSIF(_Month=10) THEN
			_Bulan:='Oktober';
		ELSIF(_Month=11) THEN
			_Bulan:='Nopember';
		ELSIF(_Month=12) THEN
			_Bulan:='Desember';
		END IF;

		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:='Penyusutan '||_FAName||' '||_Bulan||' '||_Year;


		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'FA', _FADeprDate,'FA', _FADeprNo,
			_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
			null, null, null, null,
			null, _BranchID, _DeptID, null, _CrcID, _ExcRate, _FisRate				
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, _AccCost, 'D', _FADeprAmt, _DeptID, null, _FADeprAmt);
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,2, _AccAccum, 'K', _FADeprAmt, _DeptID, null, _FADeprAmt);


		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFixedAsset_Depr_New_Manual(_FADeprNo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_spret			SPStatus%ROWTYPE;
	_retval			SPStatus%ROWTYPE;
	_fret			tCreateFA_Depr%ROWTYPE;		

	_FAID			VARCHAR;
	_FADeprAmt		NUMERIC(19,4);
	_FADeprDate		DATE;
	_FADeprIType		VARCHAR;
	_FAName			VARCHAR;
	_AccAsset		VARCHAR;
	_AccCost		VARCHAR;
	_AccAccum		VARCHAR;
	_AccLoss		VARCHAR;
	_LossAmt		NUMERIC(19,4);
	_DeptID			VARCHAR;
	_BranchID		VARCHAR;
	_Month			INT4;
	_Year			INT4;
	_JnlNo			VARCHAR;
	_JnlNote		VARCHAR;
	_AccumYtd		NUMERIC(19,4);
	_AccumCost		NUMERIC(19,4);
	_AccumCostNow		NUMERIC(19,4);
	_CntFaDepr		INT;
	_LastDeprDate		DATE;

	_DeprID			VARCHAR;
	_AcqDate		DATE;
	_AcqCost		NUMERIC(19,4);
	_EcLife			NUMERIC(19,4);
	_RsdValue		NUMERIC(19,4);
	_FABV			NUMERIC(19,4);
	_TotalDeprAmt		NUMERIC(19,4);
	
	_TotalDeprMonth		INT;
	_MaxDeprMonth		INT;
	_MaxAccumCost		NUMERIC(19,4);

	_StartMonth		INT;
	_StartYear		INT;
	_CurrMonth		INT;
	_CurrYear		INT;

	_Bulan			VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);
	_PerStart		DATE;
BEGIN
	/** Inisialisasi */ 
	
	SELECT INTO _FAID, _FADeprAmt, _FADeprDate, _FADeprIType, _AccLoss 
		faid, depramt, fadeprdate, fadepritype, accloss 
	FROM fadepr WHERE fadeprno=_FADeprNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Depresiasi Harta Tetap #'||_FADeprNo||' tidak ditemukan (spFixedAsset_Depr_New_Manual)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	SELECT INTO _PerStart perstart FROM per WHERE perno=1;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Periode Akuntansi belum diset (spFixedAsset_Depr_New_Manual)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _CrcID crcid FROM crc WHERE isdefault=true;
	_ExcRate:=1;
	_FisRate:=1;

	IF (_FADeprIType<>'AFKIR') THEN
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _FAName, _LastDeprDate, _EcLife, _RsdValue, _AcqCost, _AcqDate, _DeprID, _FABV, _AccAsset, _AccCost, _AccAccum 
		faname, lastdeprdate, eclife, rsdval, acqcost, acqdate, deprid, fabv, accasset, acccost, accaccum  
	FROM fa WHERE faid=_FAID;

	_Month:=DATE_PART('month', _FADeprDate);
	_Year:=DATE_PART('year', _FADeprDate);

	SELECT INTO _CntFADepr COALESCE(COUNT(*),0) 
	FROM fadepr WHERE faid=_FAID AND DATE_PART('month', fadeprdate)=_Month AND DATE_PART('year',fadeprdate)=_Year;	
	IF (_CntFADepr>1) THEN
		_retval.ErrNo:=5001;
		_retval.ErrMsg:='Sudah ada depresiasi untuk '||_FAName||' pada bulan ini! (spFixedAsset_Depr_New_Manual)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_LastDeprDate IS NOT NULL AND _LastDeprDate>_FADeprDate) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Sudah ada depresiasi lain yang menghandle '||_FAName||' untuk bulan ini (spFixedAsset_Depr_New_Manual)';
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;
	
	_MaxDeprMonth:=CAST(_EcLife AS INT) * 12;
	_MaxAccumCost:=_AcqCost-_RsdValue;
		
	_StartMonth:=DATE_PART('month', _AcqDate);
	_StartYear:=DATE_PART('year', _AcqDate);
	_CurrMonth:=DATE_PART('month', _FADeprDate);
	_CurrYear:=DATE_PART('year', _FADeprDate);

	/** Hitung Jumlah Bulan Depresiasi */
	IF (_CurrYear=_StartYear) THEN
		_TotalDeprMonth:=_CurrMonth-_StartMonth+1;
	ELSIF (_CurrYear>_StartYear) THEN
		_TotalDeprMonth:=(12-_StartMonth)+((_CurrYear-_StartYear-1)*12)+_CurrMonth+1;
	ELSE
		_TotalDeprMonth:=0; /** kalo tahun beli > tahun sekarang, berarti belum ada depresiasi */
	END IF;

	SELECT INTO _fret * FROM fCreateFA_Depr(_FAID, _DeprID, _MaxDeprMonth, _TotalDeprMonth, _EcLife, _MaxAccumCost, 
		_AcqDate, _AcqCost, _RsdValue, _PerStart);
	
	_LossAmt:=_FABV-_fret.depramt;
	_TotalDeprAmt:=_AcqCost-_LossAmt;
	_FADeprAmt:=_fret.depramt;

	UPDATE fadepr SET depramt=_fret.depramt, accummonth=_fret.deprmonth, lossamt=_LossAmt 
	WHERE fadeprno=_FADeprNo;	

	UPDATE fa SET accumcost=accumcost+_FADeprAmt, famc=0, fabv=0, lastdeprdate=_FADeprDate, active=false, isposted=TRUE WHERE faid=_FAID;

	IF (isAccInstalled()) THEN
		IF(_Month=1) THEN
			_Bulan:='Januari';
		ELSIF(_Month=2) THEN
			_Bulan:='Februari';
		ELSIF(_Month=3) THEN
			_Bulan:='Maret';
		ELSIF(_Month=4) THEN
			_Bulan:='April';
		ELSIF(_Month=5) THEN
			_Bulan:='Mei';
		ELSIF(_Month=6) THEN
			_Bulan:='Juni';
		ELSIF(_Month=7) THEN
			_Bulan:='Juli';
		ELSIF(_Month=8) THEN
			_Bulan:='Agustus';
		ELSIF(_Month=9) THEN
			_Bulan:='September';
		ELSIF(_Month=10) THEN
			_Bulan:='Oktober';
		ELSIF(_Month=11) THEN
			_Bulan:='Nopember';
		ELSIF(_Month=12) THEN
			_Bulan:='Desember';
		END IF;

/** START : JURNAL PENYUSUTAN ======================================================================= */
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:='Penyusutan '||_FAName||' '||_Bulan||' '||_Year;


		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'FA', _FADeprDate,'FA', _FADeprNo,
			_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
			null, null, null, null,
			null, _BranchID, _DeptID, null, _CrcID, _ExcRate, _FisRate				
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, _AccCost, 'D', _FADeprAmt, _DeptID, null, _FADeprAmt);
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,2, _AccAccum, 'K', _FADeprAmt, _DeptID, null, _FADeprAmt);


		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
/** END : JURNAL PENYUSUTAN ======================================================================= */


/** START : JURNAL PENGAFKIRAN ======================================================================= */
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:='Pengafkiran '||_FAName||' '||_Bulan||' '||_Year;


		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'FA', _FADeprDate,'FA', _FADeprNo,
			_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
			null, null, null, null,
			null, _BranchID, _DeptID, null, _CrcID, _ExcRate, _FisRate				
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, _AccAccum, 'D', _TotalDeprAmt, _DeptID, null, _TotalDeprAmt);
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, _AccLoss, 'D', _LossAmt, _DeptID, null, _LossAmt);
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 3, _AccAsset, 'K', _AcqCost, _DeptID, null, _AcqCost);

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
/** END : JURNAL PENGAFKIRAN ======================================================================= */
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$ 
LANGUAGE plpgsql;




CREATE OR REPLACE FUNCTION spFixedAsset_Depr_Void (_FADeprNo VARCHAR) 
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_FAID			VARCHAR;
	_AccumMonth		INT;
	_FADeprIType		VARCHAR;
	_FADeprAmt		NUMERIC(19,4);
	_LossAmt		NUMERIC(19,4);

	_FAName			VARCHAR;
	_RsdValue		NUMERIC(19,4);
	_FABV			NUMERIC(19,4);
	_IsActive		BOOLEAN;

	_CountUpper		INT;
	_LastDeprDate		DATE;
BEGIN
	/** Inisialisasi */

	SELECT INTO _FAID, _AccumMonth, _FADeprAmt, _LossAmt, _FADeprIType 
		faid, accummonth, depramt, lossamt, fadepritype  
	FROM fadepr WHERE fadeprno=_FADeprNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Depresiasi Harta Tetap#'||_FADeprNo||' tidak ditemukan (spFixedAsset_Depr_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 

	SELECT INTO _FAName, _RsdValue, _FABV, _IsActive 
			faname, rsdval, fabv, active
	FROM fa WHERE faid=_FAID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Harta Tetap #'||_FAID||':'||COALESCE(_FAName, '<kosong>')||' tidak ditemukan (spFixedAsset_Depr_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _CountUpper COUNT(*) FROM fadepr WHERE faid=_FAID AND fadeprno<>_FADeprNo AND accummonth>_AccumMonth;
	IF (_CountUpper>0) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Sudah ada depresiasi lain di atas depresiasi ini (spFixedAsset_Depr_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LastDeprDate MAX(fadeprdate) FROM fadepr 
	WHERE faid=_FAID AND accummonth=(SELECT MAX(accummonth) FROM fadepr WHERE faid=_FAID AND accummonth<_AccumMonth);
	
	IF (_FADeprIType='AFKIR') THEN
		_FABV:=_FABV+_FADeprAmt+_LossAmt;
	ELSIF (_FADeprIType='NORMD') THEN
		IF (_FABV=1 AND _RsdValue=0) THEN
			_FABV:=_FABV+_FADeprAmt-1;
		END IF;
	END IF;

	IF (isAccInstalled()) THEN
		SELECT INTO _spret * FROM spJnl_Void_byRef('FA', _FADeprNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	IF (_FABV>1 AND _IsActive=FALSE ) THEN
		_IsActive:=TRUE;
	END IF;

	UPDATE fa SET accumcost=accumcost-_FADeprAmt, fabv=_FABV, 
		lastdeprdate=_LastDeprDate, active=_IsActive 
	WHERE faid=_FAID;

	DELETE FROM fadepr WHERE fadeprno=_FADeprNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spFixedAsset_Depr(_DateNow DATE)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_PerStart		DATE;

	_IsBegBal		BOOLEAN;
	_DoGenJnl		BOOLEAN;

	_FAID			VARCHAR;
	_AcqCost		NUMERIC(19,4);
	_AcqDate		DATE;
	_UsageDate		DATE;
	_DeprID			VARCHAR;
	_EcLife			NUMERIC(19,4);
	_CurrAccumCost		NUMERIC(19,4);
	_RsdValue		NUMERIC(19,4);

	_TotalDeprMonth		INT;
	_MaxDeprMonth		INT;
	_MaxAccumCost		NUMERIC(19,4);

	_StartMonth		INT;
	_StartYear		INT;
	_CurrMonth		INT;
	_CurrYear		INT;

	cFA			REFCURSOR;
BEGIN
	/** Inisialisasi */
	_IsBegBal:=FALSE;
	_DoGenJnl:=TRUE;

	SELECT INTO _PerStart perstart FROM per WHERE perno=1;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Periode Akuntansi belum diset (spFA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	OPEN cFA FOR
		SELECT faid, acqcost, acqdate, deprid, eclife, COALESCE(accumcost, 0), rsdval, usagedate  	
		FROM fa WHERE active=TRUE;
	LOOP
		FETCH cFA INTO _FAID, _AcqCost, _AcqDate, _DeprID, _EcLife, _CurrAccumCost, _RsdValue, _UsageDate;
		EXIT WHEN NOT FOUND;

		_MaxDeprMonth:=CAST(_EcLife AS INT) * 12;
		_MaxAccumCost:=_AcqCost-_RsdValue;
		
		_StartMonth:=DATE_PART('month', _UsageDate);
		_StartYear:=DATE_PART('year', _UsageDate);
		_CurrMonth:=DATE_PART('month', _DateNow);
		_CurrYear:=DATE_PART('year', _DateNow);

		/** Hitung Jumlah Bulan Depresiasi */
		IF (_CurrYear=_StartYear) THEN
			_TotalDeprMonth:=_CurrMonth-_StartMonth+1;
		ELSIF (_CurrYear>_StartYear) THEN
			_TotalDeprMonth:=(12-_StartMonth)+((_CurrYear-_StartYear-1)*12)+_CurrMonth+1;
		ELSE
			_TotalDeprMonth:=0; /** kalo tahun beli > tahun sekarang, berarti belum ada depresiasi */
		END IF;

		/** Buat Depresiasi */
		SELECT INTO _spret * FROM spCreateFA_Depr(_FAID, _DeprID, _MaxDeprMonth, _TotalDeprMonth, _EcLife, _MaxAccumCost, 
		_AcqDate, _AcqCost, _RsdValue, _PerStart, _UsageDate, _DoGenJnl, _IsBegBal);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cFA;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$ 
LANGUAGE plpgsql;




CREATE OR REPLACE FUNCTION spCreateFA_Depr(
	_FAID			VARCHAR,
	_DeprID			VARCHAR,
	_MaxDeprMonth		INT,
	_TotalDeprMonth	INT,
	_EcLife			NUMERIC(19,4),
	_MaxAccumCost		NUMERIC(19,4),
	_AcqDate		DATE,
	_AcqCost		NUMERIC(19,4),
	_RsdValue		NUMERIC(19,4),
	_PerStart		DATE,
	_UsageDate		DATE,
	_DoGenJnlDepr		BOOLEAN
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
BEGIN
	/** Inisialisasi */

	SELECT INTO _spret * FROM spCreateFA_Depr(_FAID, _DeprID, _MaxDeprMonth, _TotalDeprMonth, _EcLife, 
		_MaxAccumCost, _AcqDate, _AcqCost, _RsdValue, _PerStart, _UsageDate, _DoGenJnlDepr, FALSE);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spCreateFA_Depr(
	_FAID			VARCHAR,
	_DeprID			VARCHAR,
	_MaxDeprMonth		INT,
	__TotalDeprMonth	INT,
	_EcLife			NUMERIC(19,4),
	_MaxAccumCost		NUMERIC(19,4),
	_AcqDate		DATE,
	_AcqCost		NUMERIC(19,4),
	_RsdValue		NUMERIC(19,4),
	_PerStart		DATE,
	_UsageDate		DATE,
	_DoGenJnlDepr		BOOLEAN,
	_DoGenJnlBegBal		BOOLEAN
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_AkumulasiPerTahun	NUMERIC(19,4);
	_AkumulasiPerBulan	NUMERIC(19,4);
	_SelisihPerTahun	NUMERIC(19,4);
	_SelisihPerLife		NUMERIC(19,4);

	_PersenPerTahun		NUMERIC(19,4);
	_YearCounter		INT;
	_YearRemain		INT;
	_OldYearCounter		INT;

	_DeprAmt		NUMERIC(19,4);
	_TotalDeprAmt		NUMERIC(19,4);

	_LastDeprMonth		INT;	

	_MonthIncrement		INT;

	_FADeprNo		VARCHAR;
	_DeprDate		DATE;
	
	_LockDate		DATE;

	_AccHist		VARCHAR;
	_AccAccum		VARCHAR;
	_AccAsset		VARCHAR;
	_FAName			VARCHAR;
	_FABV			NUMERIC(19,4);
	_BranchID		VARCHAR;
	_DeptID			VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_JnlNo			VARCHAR;
	_JnlNote		VARCHAR;
	_JnlDNo			INT2;

	_TotalDeprMonth		INT;
	_Depreciatable		BOOLEAN;
BEGIN
	/** Inisialisasi */
	_AkumulasiPerTahun:=0;
	_AkumulasiPerBulan:=0;
	_SelisihPerTahun:=0;
	_SelisihPerLife:=0;
	_DeprAmt:=0;
	_TotalDeprAmt:=0;
	_LastDeprMonth:=0;
	_YearCounter:=0;
	_OldYearCounter:=-1;
	_TotalDeprMonth:=__TotalDeprMonth;
	
	SELECT INTO _LastDeprMonth COALESCE(CAST(MAX(accummonth) AS INT), 0) 
	FROM fadepr WHERE faid=_FAID;

	SELECT INTO _AccAsset, _AccAccum, _FAName, _BranchID, _DeptID, _FABV, _Depreciatable 
		fa.accasset, fa.accaccum, fa.faname, fa.branchid, fa.deptid, fa.fabv, g.depreciatable 
	FROM fa 
	JOIN fagrp g ON fa.fagrpid=g.fagrpid
	WHERE faid=_FAID; 

	SELECT INTO _CrcID crcid FROM crc WHERE isdefault=true;
	_ExcRate:=1;
	_FisRate:=1;

	SELECT INTO _LockDate lockdate FROM tlock WHERE transtype='JNL';
	IF NOT FOUND THEN
		_LockDate:=NULL;
	END IF;

	IF (_TotalDeprMonth>_MaxDeprMonth) THEN
		_TotalDeprMonth:=_MaxDeprMonth;
	END IF;

	_MonthIncrement:=_TotalDeprMonth-_LastDeprMonth;

	IF (_Depreciatable=TRUE AND _DeprID='SLM') THEN /** Metode garis lurus */
		_AkumulasiPerTahun:=ROUND(_MaxAccumCost/_EcLife);
		_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
		_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
		_SelisihPerLife:=_MaxAccumCost-(_AkumulasiPerTahun*_EcLife);

		WHILE (_MonthIncrement>0) LOOP
			_LastDeprMonth:=_LastDeprMonth+1;
			_FADeprNo:=fTransNoGet('FADEPR');
			_DeprDate:=CAST(CAST(DATE_TRUNC('month', _UsageDate) AS DATE)+CAST(_LastDeprMonth||'MONTH' AS INTERVAL) AS DATE)-1;
			
			IF (_LockDate IS NOT NULL AND _DeprDate<_LockDate) THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Tanggal Pembelian Melebihi LockDate! Mundurkan LockDate Terlebih Dahulu!';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			
			IF (MOD(_LastDeprMonth, 12)=0) THEN
				_DeprAmt:=_AkumulasiPerBulan+_SelisihPerTahun;
			ELSE	
				_DeprAmt:=_AkumulasiPerBulan;
			END IF;
			
			IF (_LastDeprMonth=_MaxDeprMonth) THEN /** Bulan Terakhir */
				_DeprAmt:=_DeprAmt+_SelisihPerLife;
			END IF;

			_TotalDeprAmt:=_TotalDeprAmt+_DeprAmt;
			INSERT INTO fadepr(fadeprno, faid, fadeprdate, depramt, accummonth) 
					VALUES(_FADeprNo, _FAID, _DeprDate, _DeprAmt, _LastDeprMonth);

			SELECT INTO _spret * FROM spFixedAsset_Depr_New(_FADeprNo, _DoGenJnlDepr);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_MonthIncrement:=_MonthIncrement-1;
		END LOOP;

		UPDATE fa SET accumytd=(CASE WHEN active=TRUE THEN _AkumulasiPerTahun ELSE 0 END) WHERE faid=_FAID;
	ELSIF (_Depreciatable=TRUE AND _DeprID='DBM') THEN /** Metode saldo menurun */
		_PersenPerTahun:=(100/_EcLife)/100;
		_AkumulasiPerTahun:=_MaxAccumCost*_PersenPerTahun;
		
		_PersenPerTahun:=(_AkumulasiPerTahun/_MaxAccumCost)*2;
		IF (_PersenPerTahun>0.5) THEN
			_PersenPerTahun:=0.5;
		END IF;
		
		WHILE (_MonthIncrement>0) LOOP
			_LastDeprMonth:=_LastDeprMonth+1;
			_YearCounter:=(_LastDeprMonth/12);
			_YearRemain:=MOD(_LastDeprMonth, 12);
			
			IF ((_YearCounter>_OldYearCounter AND _YearRemain<>0) OR (_YearCounter=_OldYearCounter AND _YearRemain=1)) THEN
				_OldYearCounter:=_YearCounter;

				IF (_YearCounter<(_EcLife-1)) THEN
					_AkumulasiPerTahun:=_PersenPerTahun*POW((1-_PersenPerTahun), (_YearCounter-1))*(_AcqCost-(_PersenPerTahun*_AcqCost));
--					IF (_LastDeprMonth=25) THEN
--						RAISE EXCEPTION 'GJL 1 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;
				ELSIF (_YearCounter=(_EcLife-1)) THEN
					_AkumulasiPerTahun:=POW((1-_PersenPerTahun), (_YearCounter-1))*(_AcqCost-(_PersenPerTahun*_AcqCost))-_RsdValue;	
--					IF (_LastDeprMonth=25) THEN
--						RAISE EXCEPTION 'GJL 2 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;		
				END IF;

				_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
				_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
				_FABV:=_FABV-_AkumulasiPerTahun;
			ELSIF (_YearCounter>_OldYearCounter AND _YearRemain=0) THEN
				_OldYearCounter:=_YearCounter;
				
				IF (_YearCounter<_EcLife) THEN
					_AkumulasiPerTahun:=_PersenPerTahun*POW((1-_PersenPerTahun), (_YearCounter-2))*(_AcqCost-(_PersenPerTahun*_AcqCost));
--					IF (_LastDeprMonth=36) THEN
--						RAISE EXCEPTION 'GNP 1 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;
				ELSIF (_YearCounter=_EcLife) THEN
					_AkumulasiPerTahun:=POW((1-_PersenPerTahun), (_YearCounter-2))*(_AcqCost-(_PersenPerTahun*_AcqCost))-_RsdValue;
--					IF (_LastDeprMonth=48) THEN
--						RAISE EXCEPTION 'GNP 2 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--					END IF;					
				END IF;				
				
				_AkumulasiPerBulan:=ROUND(_AkumulasiPerTahun/12);
				_SelisihPerTahun:=_AkumulasiPerTahun-(_AkumulasiPerBulan*12);
				_FABV:=_FABV-_AkumulasiPerTahun;
			END IF;			
			
--			IF (_LastDeprMonth=48) THEN
--				RAISE EXCEPTION '0 %', _YearCounter||':'||_YearRemain||' -> '||_AkumulasiPerTahun;
--			END IF;

			_FADeprNo:=fTransNoGet('FADEPR');
			_DeprDate:=CAST(CAST(DATE_TRUNC('month', _UsageDate) AS DATE)+CAST(_LastDeprMonth||'MONTH' AS INTERVAL) AS DATE)-1;
			
			IF (_LockDate IS NOT NULL AND _DeprDate<_LockDate) THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Tanggal Pembelian Melebihi LockDate! Mundurkan LockDate Terlebih Dahulu!';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF (MOD(_LastDeprMonth, 12)=0) THEN
				_DeprAmt:=_AkumulasiPerBulan+_SelisihPerTahun;
			ELSE	
				_DeprAmt:=_AkumulasiPerBulan;
			END IF;
			
			IF (_LastDeprMonth=_MaxDeprMonth) THEN /** Bulan Terakhir */
				_DeprAmt:=_DeprAmt+_SelisihPerLife;
			END IF;

			_TotalDeprAmt:=_TotalDeprAmt+_DeprAmt;
			INSERT INTO fadepr(fadeprno, faid, fadeprdate, depramt, accummonth) 
					VALUES(_FADeprNo, _FAID, _DeprDate, _DeprAmt, _LastDeprMonth);

			SELECT INTO _spret * FROM spFixedAsset_Depr_New(_FADeprNo, _DoGenJnlDepr);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_MonthIncrement:=_MonthIncrement-1;
		END LOOP;

		UPDATE fa SET accumytd=(CASE WHEN active=TRUE THEN _AkumulasiPerTahun ELSE 0 END) WHERE faid=_FAID;
	ELSIF (_Depreciatable=FALSE) THEN
		_TotalDeprAmt:=0;
	END IF;

	IF (isAccInstalled() AND _DoGenJnlBegBal=TRUE) THEN
		_AccHist:=fAccDefault('BEGBAL');
		IF (_AccHist IS NULL OR _AccHist='') THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Default Akun Saldo Awal belum diset dalam sistem (spCreateFA_Depr)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:='Saldo Awal Harta Tetap '||_FAName;
			
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, 'BEG', _PerStart,'FA', _FAID,
			_JnlNote, true, true, CAST(0 as NUMERIC(19,4)),
			null, null, null, null,
			null, _BranchID, _DeptID, null, _CrcID, _ExcRate, _FisRate				
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_JnlDNo:=1;
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccAsset, 'D', _AcqCost, _DeptID, null, _AcqCost);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_TotalDeprAmt!=0) THEN
			_JnlDNo:=_JnlDNo+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccAccum, 'D', (_TotalDeprAmt*(-1)), _DeptID, null, (_TotalDeprAmt*(-1)));
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		_JnlDNo:=_JnlDNo+1;
		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccHist, 'K', (_AcqCost-_TotalDeprAmt), _DeptID, null, (_AcqCost-_TotalDeprAmt));
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPurcFA_New(_PurcNo VARCHAR) 
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	cPurc			REFCURSOR;
	cPurcD			REFCURSOR;
	cTax			REFCURSOR;

	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);
	_IsDraft		BOOLEAN;
	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(5);
	_VendorID		VARCHAR(15);
	_PurcDate		DATE;
	_TermType		VARCHAR(1);
	_Term			INT2;
	_CashID			VARCHAR(15);
	_Total			NUMERIC(19,4);
	_PaidAmt		NUMERIC(19,4);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_EmpID			VARCHAR(15);
	_CrcID			VARCHAR(5);
	_AccDisc		VARCHAR(10);
	_MTaxAmt		NUMERIC(19,4);
	_AccFreight		VARCHAR(10);
	_Freight		NUMERIC(19,4);
	_ExcRate		NUMERIC(19,4);
	_MDiscAmt		NUMERIC(19,4);
	_Others			NUMERIC(19,4);
	_PurcType		VARCHAR(5);
	_AccAP			VARCHAR(10);
	_Subtotal		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);
	_IsTaxed		BOOLEAN;
	_IsTaxInc		BOOLEAN;

	_Round			INT;
	_LockDate		DATE;

	_AllowCredit		NUMERIC(19,4);
	_CreditLimit		NUMERIC(19,4);
	_CreditAmount		NUMERIC(19,4);
	_CountCredit		INT;
	_PayNo			VARCHAR(15);
	_BPAAmt			NUMERIC(19,4);
	_BPANote		VARCHAR;

	_FAID			VARCHAR(15);
	_Qty			NUMERIC(15,4);
	_ListPrice		NUMERIC(19,4);
	_BasePrice		NUMERIC(19,4);
	_DiscExp		VARCHAR;
	_DiscAmt		NUMERIC(19,4);
	_BDiscAmt		NUMERIC(19,4); -- base currency detail disc
	_Disc2Amt		NUMERIC(19,4);
	_TaxID			VARCHAR;
	_TaxAmt			NUMERIC(19,4);

	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;
	_JnlType		VARCHAR(4);
	_AccNo			VARCHAR(15);
	_AccINV			VARCHAR(15);
	_AccPurcTax		VARCHAR(15);
	_SubtotalExt		NUMERIC(19,4);
	_TotalTax		NUMERIC(19,4);	
	_Count			INT;

	_RsdVal			NUMERIC(19,4);
	_tmp_va1		VARCHAR(1);

	_BTotal			NUMERIC(19,4); -- base currency total
	_BSubtotal		NUMERIC(19,4); -- base currency subtotal
	_MBFTaxAmt		NUMERIC(19,4); -- base currency taxamt
	_MBDiscAmt		NUMERIC(19,4); -- base currency discamt
	
	_TaxableAmt		NUMERIC(19,4);
	_TotalTaxAmt		NUMERIC(19,4);
	_BTotalTaxAmt		NUMERIC(19,4);
	_TotalDiscAmt		NUMERIC(19,4);
	_TotalDisc2Amt		NUMERIC(19,4);
	_DSubtotal		NUMERIC(19,4);
	_DBSubtotal		NUMERIC(19,4);
	
	_AccRateLR		VARCHAR; -- akun laba/rugi selisih kurs
	_RateLR			NUMERIC(19,4); -- nilai laba/rugi selisih kurs

	_CashCrcID		VARCHAR;
	_InOut			CHAR(1);
BEGIN
	/** Inisialisasi */
	_retval:=(0, 'OK');
	_InOut:='O';
	_DeptID:=NULL;
	_PrjID:=NULL;
	_Count:=0;

	_Subtotal:=0;
	_BSubtotal:=0;

	/** validasi standar */
	SELECT INTO
		_IsDraft, _PerID, _BranchID, _VendorID, _PurcDate, _TermType, _Term, _CashID, _BTotal, _Total, 
		_PaidAmt, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _EmpID, _CrcID, _AccDisc, _MBFTaxAmt, _MTaxAmt,
		_AccFreight, _Freight, _ExcRate, _MBDiscAmt, _MDiscAmt, _Others, _PurcType, _AccAP, _FisRate, _isTaxed, _isTaxInc

		isdraft, perid, branchid, vendorid, purcdate, termtype, duedays, cashid, basetotal, total, 
		paidamt, crtby, crtdate, updby, upddate, empid, crcid, accpdisc, basftaxamt, taxamt,
		accfreight, freight, excrate, basediscamt, discamt, others, purctype, accap, fisrate, istaxed, taxinc 
	FROM purc
	WHERE purcno=_PurcNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Pembelian harta tetap#'||_PurcNo||' tidak ditemukan (spPurcFA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	IF (_IsDraft IS NOT NULL AND _IsDraft=TRUE) THEN
		_retval.ErrMsg:='Draft Saved';
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM tlock WHERE transtype='PURC';
	IF(_LockDate IS NOT NULL AND _LockDate>_PurcDate) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg='Tanggal pembelian harta tetap melebihi batas Tanggal Kunci Transaksi (spPurcFA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');
	
	/** validasi termin */
	IF ((_TermType IS NULL) AND (_CashID IS NOT NULL)) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Termin pembelian harta tetap belum diset (spPurcFA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	ELSEIF (_TermType='C') THEN /** validadi jika terminnya CASH */
		IF ((_CashID IS NULL) OR (_CashID='')) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Kas/Bank untuk pembelian harta tetap tunai belum diset (spPurcFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		END IF;	

		SELECT INTO _CashCrcID crcid FROM cash WHERE cashid=_CashID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' belum diset (spPurcFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
 					
		IF (_CrcID!=_CashCrcID) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' tidak sama dengan mata uang pembelian (spPurcFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;		
		END IF;						
	ELSEIF (_TermType='R') THEN /** validasi jika terminnya CREDIT */
		SELECT INTO _AllowCredit, _CreditLimit, _CreditAmount aplimitfreq, aplimitamt, apbalance FROM BPAcc WHERE BPID=_VendorID AND crcid=_CrcID;

		SELECT INTO _CountCredit COUNT(*) FROM purc WHERE PayStatus<>'F' AND vendorid=_VendorID;

		IF (_AllowCredit=0) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Supplier#'||_VendorID||' tidak diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurcFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;	
		ELSIF (_AllowCredit<>-1 AND _AllowCredit<_CountCredit) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Kredit Supplier#'||_VendorID||' melebihi jumlah yang diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurcFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		END IF;

		/* mestinya Sudah di-Check di Applikasi agar bisa diconfirmasi ke user*/
		IF (_CreditLimit=0) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Supplier#'||_VendorID||' tidak diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurcFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		ELSEIF (_CreditLimit<>-1 AND _CreditLimit<(_CreditAmount+_Total) ) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Pembelian melebihi limit hutang (spPurcFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;								
		END IF;
	END IF;
	

	/** proses hutang/kas */
	IF (_Total=0) THEN	-- Total=0 ? set lunas!

		_tmp_va1:='F';
		UPDATE purc SET paystatus=_tmp_va1 WHERE purcno=_PurcNo;
		IF NOT FOUND THEN
			_retval.ErrNo:=50058;
			_retval.ErrMsg:='Error UPDATE Purc:PayStatus [NettAmt=0] (spPurcFA_New)';
			RAISE EXCEPTION '%', _reval.ErrMsg;
		END IF;

	ELSIF (_Total>0) THEN
		--************************************** TUNAI *************************************
		IF (_TermType='C') THEN	
	
			UPDATE purc SET paidamt=0 WHERE purcno=_PurcNo;

			_retval.ErrNo:=50050;
			_retval.ErrMsg:='ERROR spPay_New (spPurcFA_New)';

			SELECT INTO _PayNo * FROM fTransNoGet('PAY');
			IF (_PayNo IS NULL) THEN
				_retval.ErrNo:=50041;
				_retval.ErrMsg:='Error getting PayNo (spPurcFA_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			--generate data payment tunai
			INSERT INTO pay(payno, paytype, paydate, bpid, empid, totppaid, totpayDisc, totpayamt, reftype, refno, crtby, crtdate, updby, upddate, isautogen, branchid, excrate, fisrate, crcid)
				VALUES(_PayNo, 'PURC', _PurcDate, _VendorID, _EmpId, _Total, 0, _Total, 'PURC', _PurcNo, _CrtBy, _CrtDate, _UpdBy, _UpdDate, true, _BranchID, _ExcRate, _FisRate, _CrcID);
			INSERT INTO payd(payno, paydno, paydmtd, cashid, paydamt, refno, excrate, fisrate, crcid) VALUES(_PayNo, 1, 'CASH', _CashId, _Total, _PurcNo, _ExcRate, _FisRate, _CrcID);
			INSERT INTO ppaid(payno, ppaidno, reftype, refno, ppaidDiscAmt, ppaidamt, excrate, fisrate, crcid) VALUES(_PayNo, 1, 'PURC', _PurcNo, 0, _Total, _ExcRate, _FisRate, _CrcID);

			SELECT INTO _spret * FROM spPay_New(_PayNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			-- tetep harus update Purc nya lagi, sebab Pay yg reftype='PURC' tidak update Purc ???

			_tmp_va1:='F';
			UPDATE purc SET paidamt=_Total, paystatus=_tmp_va1 WHERE purcno=_PurcNo;
						
		--************************************** KREDIT *************************************
		ELSEIF (_TermType='R') THEN		
			_retval.ErrNo:=50051;
			_retval.ErrMsg:='Total: '||_PurcNo;
			
			SELECT INTO 
				_AllowCredit, _CreditLimit, _CreditAmount 
				aplimitfreq, aplimitamt, apbalance 
			FROM bpacc
			WHERE bpid=_VendorID AND crcid=_CrcID;

			SELECT INTO _CountCredit COUNT(*) FROM purc WHERE paystatus<>'F' AND vendorid=_VendorID;

			IF (_AllowCredit=0) THEN
				_retval.ErrNo:=50005;
				_retval.ErrMsg:='Kepada Supplier '||_VendorID||' tidak diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurcFA_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;	
							
			ELSEIF (_AllowCredit<>-1 AND _AllowCredit<_CountCredit) THEN
				_retval.ErrNo:=50006;
				_retval.ErrMsg:='Kepada Supplier '||_VendorID||' melebihi jumlah diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurcFA_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;								
			END IF;

			IF (_CreditLimit=0) THEN
				_retval.ErrNo:=50007;
				_retval.ErrMsg:='Kepada supplier '||_VendorID||' tidak diperkenankan KREDIT. Daftarkan supplier baru, ulangi pembelian KREDIT (spPurcFA_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;								
			ELSEIF (_CreditLimit<>-1 AND _CreditLimit<(_CreditAmount+_Total) ) THEN
				_retval.ErrNo:=50008;
				_retval.ErrMsg:='Pembelian harta tetap melebihi limit hutang (spPurcFA_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;								
			END IF;

			_tmp_va1:='N';
			UPDATE purc SET paidamt=0, paystatus=_tmp_va1 WHERE purcno=_PurcNo;
			_BPAAmt:=_Total*(-1);
			_BPANote:='Pembelian #'||_PurcNo;
			

			-- Posting Account-Payable
			SELECT INTO _spret * FROM spBPA_New(
						_VendorID, 'C',_CrcID, _PurcDate, 'PURC', _PurcNo, _BPAAmt, 
						_BPANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
						_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

		END IF;
	END IF;

	SELECT INTO _DiscAmt, _BDiscAmt COALESCE(SUM(totaldiscamt), 0), COALESCE(SUM(CAST(totaldiscamt*_ExcRate AS NUMERIC(19,4))), 0) FROM purcdfa WHERE purcno=_PurcNo;
	_MDiscAmt:=_MDiscAmt+_DiscAmt;	
	_MBDiscAmt:=_MBDiscAmt+_BDiscAmt;

	OPEN cPurcD FOR
		SELECT d.faid, d.listprice*d.qty, d.rsdval 
		FROM purcdfa d JOIN fa ON d.faid=fa.faid 
		WHERE purcno=_PurcNo;
	LOOP
		FETCH cPurcD INTO _FAID, _ListPrice, _RsdVal;
		EXIT WHEN NOT FOUND;

		IF (_RsdVal>_ListPrice) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Nilai Residu Harta Tetap#'||_FAID||' tidak boleh melebihi nilai beli (spPurcFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		UPDATE fa SET acqdate=_PurcDate, acqcost=CAST(_ListPrice*_ExcRate AS NUMERIC(19,4)), rsdval=CAST(_RsdVal*_ExcRate AS NUMERIC(19,4)), fabv=CAST(_ListPrice*_ExcRate AS NUMERIC(19,4)), isposted=true
		WHERE faid=_FAID;
	END LOOP;
	CLOSE cPurcD;

	/** proses akuntansi */
	IF (isAccInstalled()) THEN
		_JnlNo:=fTransNoGet('JURNAL');
		_JnlNote:= 'Pembelian Harta Tetap '||fBPName(_VendorID)||':'||_PurcNo;
		

		IF(_PurcType='B') THEN
			_JnlNote:= 'Begining Balance Hutang: '||fBPName(_VendorID)||':'||_PurcNo;
			_JnlType:='BEG';
		ELSIF (_PurcType='FA') THEN
			_JnlType:='FA';
		ELSE
			_JnlType:='PURC';
		END IF;

		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, _JnlType, _PurcDate,'PURC',_PurcNo,
			_JnlNote , true, true, CAST(0 AS DECIMAL(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
			);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;


		IF _TermType='C' THEN
			_AccNo:=fAccCash(_CashID, _InOut);
		ELSIF _TermType='R' THEN
			_AccNo:=fBpAcc(_VendorID,_CrcID,'AP');
		ELSIF(_PurcType='B') THEN
			_AccNo:=_AccAP;
		END IF;
		
		/** proses jurnal harta */
		OPEN cPurcD FOR
			SELECT fFAAcc(faid, 'ASSET'), SUM(CAST(d.listprice*d.qty AS NUMERIC(19,4))), SUM(CAST(d.listprice*d.qty*_ExcRate AS NUMERIC(19,4)))
			FROM purcdfa d 
			WHERE d.purcno=_PurcNo
			GROUP BY fFAAcc(faid, 'ASSET');
		LOOP
			FETCH cPurcD INTO _AccINV, _DSubTotal, _DBSubtotal;
			EXIT WHEN NOT FOUND;

			_Subtotal:=_Subtotal+COALESCE(_DSubtotal, 0);
			_BSubtotal:=_BSubtotal+COALESCE(_DBSubtotal, 0);

			_Count:=_Count+1;			
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccINV, 'D', _DBSubTotal, _DeptID, _PrjID, _DSubTotal);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;			
		END LOOP;
		CLOSE cPurcD;


/*************** Jurnal TAX DI GRUP IN SESUAI PAJAK TIAP2 ITEM *********************/
		SELECT INTO _TotalTax SUM(d.taxamt*d.qty*_ExcRate) 
		FROM purcdfa d JOIN tax ON d.taxid=tax.TaxID
		WHERE d.purcno=_PurcNo AND d.taxid IS NOT NULL;

		OPEN cTax FOR
			SELECT SUM(d.basftotaltaxamt), SUM(d.totaltaxamt), accpurctax
			FROM purcdfa d JOIN tax ON d.taxid=tax.taxid
			WHERE d.purcno=_PurcNo AND d.taxid IS NOT NULL
			GROUP BY accpurctax;
		LOOP
			FETCH cTax INTO _BTotalTaxAmt, _TotalTaxAmt, _AccPurcTax, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccPurcTax, 'D', _BTotalTaxAmt, _DeptID, _PrjID, _TotalTaxAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;

/*************** Jurnal Purc-Discount (Header/Total) *********************/

		IF (_MDiscAmt<>0) THEN
			_Count:=_Count+1;
			IF(_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('PURCDISC');
			END IF;
--			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,_Count,_AccDisc,'D',_MDiscAmt*_ExcRate*-1,null,null,_MDiscAmt*-1);
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccDisc, 'K', _MBDiscAmt, NULL, NULL, _MDiscAmt);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		
		/** ==================== LABA RUGI SELISIH KURS ================================ */
		_RateLR:=(_BTotal+_MBDiscAmt)-(_BSubtotal+_MBFTaxAmt);
--RAISE EXCEPTION 'LR: %', _RateLR;
		IF (_RateLR!=0) THEN
			_Count:=_Count+1;
			_AccRateLR:=fAccDefault('PKKURS');
			IF (_AccRateLR IS NULL OR _AccRateLR='') THEN
				_retval.ErrNo:=50555;
				_retval.ErrMsg:='Akun PK Selisih Kurs belum diset.'||E'\n'||'Silahkan disetting pada form Setting Akun';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccRateLR, 'D', _RateLR, _DeptID, _PrjID, NULL, _RateLR/_ExcRate);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;	
		
		IF (_Freight<>0) THEN
			_Count:=_Count+1;

			IF (_AccFreight IS NULL) THEN
				_AccFreight:=fAccDefault('PURCFREIGHT');
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccFreight, 'D', _Freight*_ExcRate, NULL, NULL, _Freight);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		IF (_Others<>0) THEN
			_Count:=_Count+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, fAccDefault('PKROUND'), 'D', _Others*_ExcRate, null, null, _Others);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
		_Count:=_Count+1;
		
/******************  Posting Kas-Keluar / Hutang Usaha **************************/

		SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo, _Count, _AccNo, 'K', _BTotal, null, null, _Total);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

/**^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^***/
/*************************** FINISHED GENERATE JURNAL ****************************/


/***************** Trigger Posting the Jurnal ************************************/
		
		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;	-- IF isAccInstalled()

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPurcFA_Void(__PurcNo VARCHAR, _IsEdit BOOLEAN) 
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	cPurc			REFCURSOR;

	_PurcNo			VARCHAR(15);
	_VendorID		VARCHAR(15);
	_PurcDate		DATE;
	_TermType		VARCHAR(1);
	_Term			INT2;
	_CashID			VARCHAR(15);
	_Total			NUMERIC(19,4);
	_PayStatus		VARCHAR(1);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_IsDraft		BOOLEAN;

	_LockDate		DATE;
	_PayNo			VARCHAR(15);	

	_FAID			VARCHAR(15);
BEGIN
	/** Inisialisasi */
	_retval:=(0, 'OK');
	_PurcNo:=__PurcNo;

	SELECT INTO _VendorID, _PurcDate, _TermType, _Term, _CashID, _Total, _PayStatus, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _IsDraft  			vendorid, purcdate, termtype, duedays, cashid, total, paystatus, crtby, crtdate, updby, upddate, isdraft 
	FROM purc WHERE purcno=_PurcNo;	
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Pembelian harta tetap#'||_PurcNo||' tidak ditemukan (spPurcFA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _LockDate lockdate FROM tlock WHERE transtype='PURC';
	IF(_LockDate IS NOT NULL AND _LockDate>_PurcDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Pembelian Melebihi Batas Tanggal Kunci Transaksi (spPurcFA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;


	IF (_Total>0) AND (_TermType='R') AND (_PayStatus<>'N') THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='Sudah ada transaksi pembayaran untuk Pembelian ini.VOID dulu pembayarannya ! (spPurcFA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		OPEN cPurc FOR
			SELECT faid FROM purcdfa WHERE purcno=_PurcNo;
		LOOP
			FETCH cPurc INTO _FAID;
			EXIT WHEN NOT FOUND;
		
			DELETE FROM purcdfa WHERE purcno=_PurcNo AND faid=_FAID;
		
			IF (_IsEdit=FALSE) THEN		
				SELECT INTO _spret * FROM spFixedAsset_Void(_FAID);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END LOOP;
		CLOSE cPurc;

		DELETE FROM purc WHERE purcno=_PurcNo;
		
		RETURN NEXT _retval;
		RETURN;
	END IF;

	IF (_TermType='C') THEN
		_retval.ErrNo:=21;
		_retval.ErrMsg:='ERROR GET _PayNo FROM Pay (spPurcFA_Void)';
		
		/** VOID Pembayaran-Auto nya */
		SELECT INTO _PayNo payno FROM pay WHERE reftype='PURC' AND refno=_PurcNo;
		IF (_PayNo IS NOT NULL) THEN
			_retval.ErrNo:=22;
			_retval.ErrMsg:='ERROR EXEC spPay_Void for PayNo=' || _PayNo || ' (spPurcFA_Void)';
			
			SELECT INTO _spret * FROM spPay_Void(_PayNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;		
	ELSIF (_TermType='R') THEN
		_retval.ErrNo:=20;
		_retval.ErrMsg:='ERROR Void BPA (spPurcFA_Void)';
		
		SELECT INTO _spret * FROM spBPA_Void_byRef('PURC', _PurcNo);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	OPEN cPurc FOR
		SELECT faid FROM purcdfa WHERE purcno=_PurcNo;
	LOOP
		FETCH cPurc INTO _FAID;
		EXIT WHEN NOT FOUND;
		
		DELETE FROM purcdfa WHERE purcno=_PurcNo AND faid=_FAID;
		
		IF (_IsEdit=FALSE) THEN		
			SELECT INTO _spret * FROM spFixedAsset_Void(_FAID);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cPurc;
		
	IF (isAccInstalled() AND _IsDraft=FALSE) THEN
		SELECT INTO _spret * FROM spJnl_Void_byRef('PURC', _PurcNo);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;
	
	DELETE FROM purc WHERE purcno=_PurcNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tFASale CASCADE;
CREATE TYPE tFASale AS (
	saleamt		NUMERIC(19,4),
	salepl		NUMERIC(19,4)
);


CREATE OR REPLACE FUNCTION fFASale (
	_FAID		VARCHAR,
	_Date		DATE
) RETURNS SETOF tFASale AS
$BODY$
DECLARE
	_retval		tFASale%ROWTYPE;
	
	_FABV		NUMERIC(19,4);
BEGIN
	SELECT INTO _retval.saleamt, _FABV (d.listprice-d.discamt-d.disc2amt+d.taxamt), d.fabv 
	FROM saledfa d JOIN sale m ON d.saleno=m.saleno 
	WHERE faid=_FAID AND m.saledate<=_DATE;
	IF NOT FOUND THEN
		_retval.saleamt:=0;
		_retval.salepl:=0;
	ELSE
		_retval.salepl:=_retval.saleamt-_FABV;	
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fFAAcc (
	_FAID		VARCHAR,
	_AccType	VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO _retval
	CASE _AccType
		WHEN 'ASSET' THEN accasset
		WHEN 'ACCUM' THEN accaccum
		WHEN 'DEPR' THEN acccost
	END
	FROM fa WHERE faid=_FAID;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tFARpt CASCADE;
CREATE TYPE tFARpt AS (
	branchname	VARCHAR,
	fagrpname	VARCHAR,
	faid		VARCHAR,
	faname		VARCHAR,
	acqdate		DATE,
	acqcost		NUMERIC(19,4),
	addition	NUMERIC(19,4),
	subtraction	NUMERIC(19,4),
	subpl		NUMERIC(19,4),
	yeardepramt	NUMERIC(19,4),
	totdeprbefore	NUMERIC(19,4),
	accumbefore	NUMERIC(19,4),
	totdeprnow	NUMERIC(19,4),
	accumnow	NUMERIC(19,4),
	fabvbef		NUMERIC(19,4),
	fabvnow		NUMERIC(19,4),
	rsdval		NUMERIC(19,4)
);


/** 
    tidak perlu parameter untuk mode, 
    karena cek jika date=null -> mode hari ini, 
    namun jika non null -> mode per tanggal 
*/
CREATE OR REPLACE FUNCTION fFARpt (
	_Date		DATE,
	_FAID		VARCHAR,
	_FAGrpID	VARCHAR,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR	
) RETURNS SETOF tFARpt AS 
$BODY$
DECLARE
	_retval		tFARpt%ROWTYPE;

	cData		REFCURSOR;

	_BranchName	VARCHAR;
	_FAGrpName	VARCHAR;
	_FAID		VARCHAR;
	_FAName		VARCHAR;
	_AcqDate	DATE;
	_AcqCost	NUMERIC(19,4);
	_Addition	NUMERIC(19,4);
	_Subtraction	NUMERIC(19,4);
	_SubPL		NUMERIC(19,4);
	_YearDeprAmt	NUMERIC(19,4);
	_TotDeprBefore	NUMERIC(19,4);
	_AccumBefore	NUMERIC(19,4);
	_TotDeprNow	NUMERIC(19,4);
	_AccumNow	NUMERIC(19,4);
	_FABVBef	NUMERIC(19,4);
	_FABVNow	NUMERIC(19,4);
	_RsdVal		NUMERIC(19,4);
	_FAIType	VARCHAR;
	_EcLife		NUMERIC(5,2);
	_DeprID		VARCHAR;

	_DateYear	INT2;
	_AcqYear	INT2;

	_StartDateBef	DATE; /** data tahun sebelumnya */
	_EndDateBef	DATE;
	
	_StartDateNow	DATE; /** data tahun per date */
	_EndDateNow	DATE;

	_CurrDate	DATE;

	_Depreciatable	BOOLEAN;
BEGIN
	/** Inisialisasi */
	IF (_Date IS NOT NULL) THEN
		_DateYear:=DATE_PART('year', _Date);
		_CurrDate:=_Date;
	ELSE
		_DateYear:=DATE_PART('year', CURRENT_DATE);
		_CurrDate:=CURRENT_DATE;
	END IF;

	_StartDateNow:=TO_DATE('01-'||'01-'||_DateYear, 'DD-MM-YYYY');
	_EndDateNow:=TO_DATE('31-'||'12-'||_DateYear, 'DD-MM-YYYY');

	_StartDateBef:=TO_DATE('01-'||'01-'||(_DateYear-1), 'DD-MM-YYYY');
	_EndDateBef:=TO_DATE('31-'||'12-'||(_DateYear-1), 'DD-MM-YYYY');

	OPEN cData FOR
		SELECT br.branchname, g.fagrpname, f.faid, f.faname, f.usagedate, f.acqcost, f.rsdval, f.faitype, f.eclife, f.deprid, g.depreciatable 
		FROM fa f 
		JOIN fagrp g ON f.fagrpid=g.fagrpid
		LEFT JOIN branch br ON f.branchid=br.branchid
		WHERE f.isposted=TRUE AND (_Date IS NULL OR f.acqdate<=_Date) 
		AND (_FAID IS NULL OR f.faid=_FAID) AND (_FAGrpID IS NULL OR f.fagrpid=_FAGrpID) 
		AND (_BranchID IS NULL OR f.branchid=_BranchID) AND (_DeptID IS NULL OR f.deptid=_DeptID)
		ORDER BY br.branchname, g.fagrpname, f.acqdate, f.faid;
	LOOP
		FETCH cData INTO _BranchName, _FAGrpName, _FAID, _FAName, _AcqDate, _AcqCost, _RsdVal, _FAIType, _EcLife, _DeprID, _Depreciatable;
		EXIT WHEN NOT FOUND;

		_AcqYear:=DATE_PART('year', _AcqDate);

		_retval.branchname:=_BranchName;
		_retval.fagrpname:=_FAGrpName;
		_retval.faid:=_FAID;
		_retval.faname:=_FAName;
		_retval.acqdate:=_AcqDate;

		IF (_DateYear=_AcqYear) THEN
			_retval.acqcost:=0;
			_retval.addition:=_AcqCost;
		ELSE
			_retval.acqcost:=_AcqCost;
			_retval.addition:=0;
		END IF;

		SELECT INTO _Subtraction, _SubPL * FROM fFASale(_FAID, _CurrDate);
		IF NOT FOUND THEN
			_Subtraction:=0;
			_SubPL:=0;
		END IF;
		_retval.subtraction:=_Subtraction;
		_retval.subpl:=_SubPL;

		IF (_Depreciatable=TRUE) THEN
			_retval.yeardepramt:=fFACalcAccumYtd(_FAID, _AcqDate, _AcqCost, _EcLife, _RsdVal, _DeprID, _CurrDate);
		ELSE
			_retval.yeardepramt:=0;
		END IF;

		_retval.totdeprbefore:=COALESCE(fFADeprSum(_FAID, _StartDateBef, _EndDateBef), 0);
		_retval.accumbefore:=COALESCE(fFADeprSum(_FAID, NULL, _EndDateBef), 0);
		_retval.totdeprnow:=COALESCE(fFADeprSum(_FAID, _StartDateNow, _EndDateNow), 0);
		_retval.accumnow:=COALESCE(fFADeprSum(_FAID, NULL, _EndDateNow), 0);
		_retval.fabvbef:=COALESCE((_retval.acqcost-_retval.accumbefore), 0);
		_retval.fabvnow:=COALESCE(((_retval.acqcost+_retval.addition)-_retval.accumnow-_retval.subtraction+_retval.subpl), 0);
		_retval.rsdval:=_RsdVal;

		RETURN NEXT _retval; 
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fFADeprSum( 
	_FAID 		VARCHAR, 
	_StartDate	DATE, /** isi null jika ingin menghitung total akumulasi */
	_EndDate	DATE
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	SELECT INTO _retval SUM(depramt) FROM fadepr
	WHERE faid=_FAID AND (_StartDate IS NULL OR fadeprdate>=_StartDate) AND fadeprdate<=_EndDate;
	IF NOT FOUND THEN
		_retval:=0;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fFACalcAccumYtd (
	_FAID		VARCHAR,
	_AcqDate	DATE,
	_AcqCost	NUMERIC(19,4),
	_EcLife		NUMERIC(5,2),
	_RsdValue	NUMERIC(19,4),
	_DeprID		VARCHAR,
	_Date		DATE
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval			NUMERIC(19,4);

	_AkumulasiPerTahun	NUMERIC(19,4);
	_SelisihPerLife		NUMERIC(19,4);

	_PersenPerTahun		NUMERIC(19,4);	

	_AcqYear		INT2;
	_AcqMonth		INT2;
	_DateYear		INT2;
	_DateMonth		INT2;

	_DiffMonth		INT2;
	_Year			INT2;
	_Remain			INT2;
	_MaxAccumCost		NUMERIC(19,4);
		
	_YearCounter		INT;
	_YearRemain		INT;
	_OldYearCounter		INT;
BEGIN
	/** Inisialisasi */
	_DiffMonth:=0;
	_Year:=0;
	_Remain:=0;
	_AkumulasiPerTahun:=0;
	_YearCounter:=0;
	_OldYearCounter:=-1;

	_AcqYear:=DATE_PART('year', _AcqDate);
	_AcqMonth:=DATE_PART('month', _AcqDate);

	_DateYear:=DATE_PART('year', _Date);
	_DateMonth:=DATE_PART('month', _Date);

	IF (_AcqYear=_DateYear) THEN
		_Year:=1;
	ELSIF (_AcqYear>_DateYear) THEN
		_DiffMonth:=(12-_AcqMonth)+((_DateYear-_AcqYear-1)*12)+_DateMonth;
		_Year:=_DiffMonth/12;
		_Remain:=MOD(_DiffMonth, 12);

		IF (_Remain>0) THEN
			_Year:=_Year+1;
		END IF;
	END IF;

	IF (_DeprID='SLM') THEN /** Metode garis lurus */
		_AkumulasiPerTahun:=ROUND(_AcqCost/_EcLife);
		_SelisihPerLife:=_AcqCost-(_AkumulasiPerTahun*_EcLife);
		
		IF (_Year=_EcLife) THEN
			_AkumulasiPerTahun:=_AkumulasiPerTahun+_SelisihPerLife;
		END IF;
	ELSIF (_DeprID='DBM') THEN /** Metode saldo menurun */

		_PersenPerTahun:=(100/_EcLife)/100;
		_AkumulasiPerTahun:=_MaxAccumCost*_PersenPerTahun;
		_PersenPerTahun:=(_AkumulasiPerTahun/_AcqCost)*2;
		IF (_PersenPerTahun>0.5) THEN
			_PersenPerTahun:=0.5;
		END IF;

		WHILE (_Year>0) LOOP
			_YearCounter:=_YearCounter+1;
			_YearRemain:=CAST(_EcLife-_YearCounter AS INT2);
			
			IF ((_YearCounter>_OldYearCounter AND _YearRemain<>0) OR (_YearCounter=_OldYearCounter AND _YearRemain=1)) THEN
				_OldYearCounter:=_YearCounter;

				IF (_YearCounter<(_EcLife-1)) THEN
					_AkumulasiPerTahun:=_PersenPerTahun*POW((1-_PersenPerTahun), (_YearCounter-1))*(_AcqCost-(_PersenPerTahun*_AcqCost));
				ELSIF (_YearCounter=(_EcLife-1)) THEN
					_AkumulasiPerTahun:=POW((1-_PersenPerTahun), (_YearCounter-1))*(_AcqCost-(_PersenPerTahun*_AcqCost))-_RsdValue;
				END IF;	
			ELSIF (_YearCounter>_OldYearCounter AND _YearRemain=0) THEN
				_OldYearCounter:=_YearCounter;
				
				IF (_YearCounter<_EcLife) THEN
					_AkumulasiPerTahun:=_PersenPerTahun*POW((1-_PersenPerTahun), (_YearCounter-2))*(_AcqCost-(_PersenPerTahun*_AcqCost));
				ELSIF (_YearCounter=_EcLife) THEN
					_AkumulasiPerTahun:=POW((1-_PersenPerTahun), (_YearCounter-2))*(_AcqCost-(_PersenPerTahun*_AcqCost))-_RsdValue;				
				END IF;				
			END IF;	
			_Year:=_Year-1;
		END LOOP;
	END IF;

	RETURN _AkumulasiPerTahun;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fFaDeprDate (
	_tgl	DATE
) RETURNS SETOF fadeprdate AS
$BODY$
DECLARE
	_retval		fadeprdate%ROWTYPE;

	_faid	 	VARCHAR(15);
	_faname		VARCHAR(100);
	_acqcost 	NUMERIC(19,4);
	_accumcost 	NUMERIC(19,4);
	_fabv	 	NUMERIC(19,4);
	_fagrpid	VARCHAR(10);
	_fagrpname	VARCHAR(50);
	_branchid	VARCHAR(5);
	_branchname	VARCHAR(50);
	_rsdval		NUMERIC(19,4);
	_famc	 	NUMERIC(19,4);
	_deptid		VARCHAR(5);

	_begdate 	DATE;

	cfa		REFCURSOR;
BEGIN
	OPEN cfa FOR
		SELECT fa.faid, fa.faname, fa.acqcost, fa.fabv, fagrp.fagrpid, fagrp.fagrpname, fa.branchid, 
		branch.branchname, fa.rsdval, fa.famc, fa.deptid 
		FROM fa JOIN fagrp on fa.fagrpid=fagrp.fagrpid 
		LEFT JOIN branch on branch.branchid=fa.branchid
		WHERE fa.acqdate<=_tgl;
	LOOP
		FETCH cfa INTO _faid, _faname, _acqcost, _fabv,_fagrpid, _fagrpname, _branchid, _branchname, _rsdval, _famc, _deptid;
		EXIT WHEN NOT FOUND;

		SELECT INTO _accumcost SUM(depramt) FROM fadepr WHERE fadeprdate<=_tgl AND faid=_faid;
		
		_retval.faid:=_faid;
		_retval.faname:=_faname;
		_retval.acqcost:=_acqcost;
		_retval.accumcost:=_accumcost;
		_retval.fabv:=_acqcost-_accumcost;
		_retval.fagrpid:=_fagrpid;
		_retval.fagrpname:=_fagrpname;
		_retval.branchid:=_branchid;
		_retval.branchname:=_branchname;
		_retval.rsdval:=_rsdval;
		_retval.famc:=_famc;
		_retval.deptid:=_deptid;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cfa;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/************
Kerangka:

	CHECK LOCK, DRAFT, VALIDASI
		check cash, credit limit
	
	GENERATE StockA
		stock
		bonus
		paket

	PROSES Non-Stock


*************/



CREATE OR REPLACE FUNCTION spSaleFA_New(_SaleNo VARCHAR(15))
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CustID			VARCHAR(15);
	_CustID2		VARCHAR(15);
	_SaleType		VARCHAR(3);
	_SaleDate		DATE;
	_TermType		VARCHAR(1);
	_Term			INT;

	_isTaxed		BOOLEAN;
	_isTaxInc		BOOLEAN;

	_SubTotal		DECIMAL(19,4);
	_Total			DECIMAL(19,4);
	_PaidAmt		DECIMAL(19,4);
	_CrtBy			VARCHAR(10);
	_CrtDate		TIMESTAMP;
	_UpdBy			VARCHAR(10);
	_UpdDate		TIMESTAMP;
	_SaleDNo		INT;
	_SaleNo2		VARCHAR(15);

	_isInsert		BOOLEAN:=true;

	_BasePrice		DECIMAL(19,4);
	_ListPrice 		DECIMAL(19,4);
	_DiscExp		VARCHAR(15);
	_DiscAmt		DECIMAL(19,4);
	_TaxID			VARCHAR(5);
	_TaxAmt			DECIMAL(19,4);
	_MTaxAmt		DECIMAL(19,4);
	_Cost			DECIMAL(19,4);
	_CostSubTotal		DECIMAL(19,4);

	_ItemType		VARCHAR(5);
	_isStock		BOOLEAN;
	_BoMID			VARCHAR(7);
	_Qty_Unit1		DECIMAL(15,4);
	_CashID			VARCHAR(15);
	_CashAAmt		DECIMAL(19,4);
	_CashANote		VARCHAR(255);
	_DebtNote		VARCHAR(255);
	_AllowCredit		DECIMAL(19,4);
	_CreditLimit		DECIMAL(19,4);
	_countCredit		DECIMAL(19,4);
	_CreditAmount		DECIMAL(19,4);
	_CrcID			VARCHAR(5);

	_ContBal 		DECIMAL(19,4);
	_RcvNo			VARCHAR;
	_EmpId			VARCHAR;

	_ExcRate		DECIMAL(19,4);

	_AccAsset		VARCHAR(10);
	_AccDisc		VARCHAR(10);
	_AccSale		VARCHAR(10);
	_AccAmt			DECIMAL(19,4);
	_AccAmtForex		NUMERIC(19,4);
	_AccAR			VARCHAR;

	_DiscHeader		DECIMAL(19,4);

	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;
	_CounterJnlD		INT;

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID 			VARCHAR(5);
	_Round			INT;

	cSaleD 			REFCURSOR;
	cSaleAcc		REFCURSOR;
	cSaleCRcv		REFCURSOR;
	cCons			REFCURSOR;
	cSaleD1			REFCURSOR;
	cTax			REFCURSOR;

	_RcvType		VARCHAR;
	_CashAmt		VARCHAR;
	_RcvAmt			VARCHAR;

	_isDraft		BOOLEAN;
	_JnlType		VARCHAR;
	_LockDate		DATE;

	_SumD			NUMERIC(19,4);
	_SumKDeli		NUMERIC(19,4);
	_SumK			NUMERIC(19,4);

	_ItemIDCheck		VARCHAR;

	_SaleDTaxAmt		NUMERIC(19,4);
	_AccSaleTax		VARCHAR;
	_TotalTax		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);
	_SaleDTaxAmtForex	NUMERIC(19,4);

	_CrcIDRcvD		VARCHAR;
	_ExcRateRcvD		NUMERIC(19,4);
	_FisRateRcvD		NUMERIC(19,4);

	_ItemsCost		NUMERIC(19,4);
	_SaleTotalMonth		NUMERIC(19,4);
	_tmp_va1		VARCHAR(1);

	_FAID			VARCHAR;
	_FABV			NUMERIC(19,4);

	_AccAccum		VARCHAR;
	_AccumCost		NUMERIC(19,4);
	_AccFA			VARCHAR;
	_AcqCost		NUMERIC(19,4);
	_DK			VARCHAR(1);

	_CashCrcID		VARCHAR;
	_InOut			CHAR(1);
BEGIN
	/** Inisialisasi */
	_retval:=(0, 'OK');
	_DeptID:=NULL;
	_PrjID:=NULL;
	_InOut:='I';

	SELECT INTO 
		_isDraft, _PerID, _BranchID, _CustID, _SaleType, _SaleDate, _TermType, _Term, _CashID, _SubTotal, _Total, _PaidAmt, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _EmpId, _CrcID, _DiscHeader, _AccDisc, _MTaxAmt, _ExcRate, _AccAR, _FisRate, _isTaxed, _isTaxInc
		isDraft, PerID, BranchID, CustID, SaleType, SaleDate, TermType, DueDays, CashID, SubTotal, Total, PaidAmt, CrtBy, CrtDate, UpdBy, UpdDate, EmpID, CrcID, COALESCE(DiscAmt,0), AccSDisc, TaxAmt, ExcRate, AccAR, FisRate, isTaxed, TaxInc
	FROM Sale WHERE SaleNo=_SaleNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penjualan Harta Tetap '|| _SaleNo||' tidak ditemukan ! (spSaleFA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
/******************************************************************************************************
	CHECK LOCK, DRAFT, VALIDASI
******************************************************************************************************/

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SALE';
	IF(_LockDate IS NOT NULL AND _LockDate>_SaleDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penjualan Harta Tetap melebihi batas tanggal kunci transaksi '|| CAST(_LockDate AS VARCHAR) || '!! (spSaleFA_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	_Round:=COALESCE(fReg('ROUND'), '0');

	---------------- cek CashID terisi utk Tunai -------------
	IF (_TermType='C') THEN
		IF((_CashID IS NULL) OR (_CashID='')) THEN
			_retval.ErrNo:=50004;
			_retval.ErrMsg:='Error, Kas untuk Penjualan Tunai belum diset (spSaleFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _CashCrcID crcid FROM cash WHERE cashid=_CashID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' belum diset (spSaleFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
 
		IF (_CrcID!=_CashCrcID) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Mata uang kas#'||_CashID||' tidak sama dengan mata uang penjualan (spSaleFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	--------------- Cek kredit limit -----------------
	ELSIF (_TermType='R') THEN
		SELECT INTO _AllowCredit, _CreditLimit, _CreditAmount arlimitfreq, arlimitamt, arbalance FROM BPacc WHERE BPID=_CustID;
		SELECT INTO _countCredit count(*) FROM sale WHERE PayStatus<>'F' and custID=_CustID;
		IF (_AllowCredit=0) THEN
			_retval.ErrNo:=50005;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' tidak diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSaleFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSEIF (_AllowCredit<>-1 AND _AllowCredit<_countCredit) THEN
			_retval.ErrNo:=50006;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' melebihi jumlah diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSaleFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		-- Sebaiknya sudah di-Check di Applikasi agar bisa diconfirmasi ke user
		IF (_CreditLimit=0) THEN
			_retval.ErrNo:=50007;
			_retval.ErrMsg:='Pelanggan '||COALESCE(_CustID,'<kosong>')||' tidak diperkenankan KREDIT. Daftarkan pelanggan baru, ulangi penjualan KREDIT (spSaleFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSEIF (_CreditLimit<>-1 AND _CreditLimit<(_CreditAmount+_Total) ) THEN
			_retval.ErrNo:=50008;
			_retval.ErrMsg:='Penjualan Harta Tetap melebihi Limit Hutang (spSaleFA_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

/******************************************************************************************************
		PROSES PAYMENT DAN PIUTANG: 
			- Payment : Cash/Bank
			- CREDIT : Piutang
******************************************************************************************************/
	--- Tunai ? Proses Uang masuk 
	IF (_TermType='C') THEN
		_CashAmt:=_PaidAmt;
		_CashANote:='Penjualan Harta Tetap '||COALESCE(fBPName(_CustID),'BP : <kosong>')||':'||_SaleNo;

		-------- set dulu biar mau di generate penerimaan --------
		_tmp_va1:='N';
		UPDATE Sale SET PayStatus=_tmp_va1, PaidAmt=0 WHERE SaleNo=_SaleNo;
		
		SELECT INTO _RcvNo * FROM fTransNoGet('RCV');
		IF (_RcvNo IS NULL) THEN
			_retval.ErrNo:=50041;
			_retval.ErrMsg:='Error getting RcvNo (spRcv_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		INSERT INTO rcv(rcvno,rcvtype,rcvdate,bpid,empid,totspaid,totrcvdisc,totrcvamt,reftype,refno,crtby,crtdate,updby,upddate,isautogen,crcid,excrate,fisrate) 
			VALUES(_RcvNo,'SALE', _SaleDate, _Custid, _EmpId, _Total, 0, _Total, 'SALE', _SaleNo, _CrtBy, _CrtDate, _UpdBy, _UpdDate,true, _CrcID, _ExcRate, _FisRate);

		INSERT INTO rcvd(rcvno,rcvdno, rcvdmtd, cashid,rcvdcash,rcvdamt,refno,crcid,excrate,fisrate) 
		VALUES(_RcvNo, 1, 'CASH', _CashId, _Total, _Total, _SaleNo,_CrcID,_ExcRate, _FisRate);

		INSERT INTO spaid(rcvno,spaidno, reftype, refno,spaiddiscamt,spaidamt,crcid,excrate,fisrate) 
		VALUES(_RcvNo, 1, 'SALE', _SaleNo, 0, _Total, _CrcID,_ExcRate,_FisRate);

		SELECT INTO _spret * FROM spRcv_New(_RcvNo);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmp_va1:='F';
		UPDATE Sale SET PaidAmt=_Total, PayStatus=_tmp_va1 WHERE SaleNo=_SaleNo;

	---------- CREDIT ? Proses Piutang -------------
	ELSIF (_TermType='R') THEN
		_DebtNote:='Penjualan Harta Tetap '||fBPName(_CustID)||':'||_SaleNo;
		SELECT INTO _spret * FROM spBPA_New(
			_CustID, 'C', _CrcID, _SaleDate, 'SALE', _SaleNo, 
			_Total, _DebtNote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _ExcRate, _FisRate
		);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_tmp_va1:='N';
		UPDATE Sale SET PaidAmt=0, PayStatus=_tmp_va1 WHERE SaleNo=_SaleNo;

	END IF;

	OPEN cSaleD FOR
		SELECT faid FROM saledfa WHERE saleno=_SaleNo;
	LOOP
		FETCH cSaleD INTO _FAID;
		EXIT WHEN NOT FOUND;
		
		UPDATE fa SET fabv=0, active=FALSE WHERE faid=_FAID;
	END LOOP;
	CLOSE cSaleD;

/**************************************************************************************************
	PROSES ACCOUNTING
**************************************************************************************************/
	IF (isAccInstalled()) THEN
		_JnlNo:=fTransNoGet('JURNAL');

		_JnlNote:='Penjualan Harta Tetap '||fBPName(_CustID)||':'||_SaleNo;
		_JnlType:='SALE';
		
		------- Generate Jurnal Header untuk JurnalVoucher(JV) Penjualan
		SELECT INTO _spret * FROM spJnl_Insert(
			_JnlNo, _JnlType, _SaleDate,'SALE', _SAleNo,
			_JnlNote, true, true, CAST(0 AS NUMERIC(19,4)),
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
		);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		------- Fetch Akun Asset-Lancar(KasBank/Piutang) --------
		IF(_SaleType<>'B') THEN
			IF _TermType='C' THEN	--- Kas/Bank
				_AccAsset:=fAccCash(_CashID, _InOut);
			ELSE	 --- Piutang
				_AccAsset:=fBpAcc(_CustID, _CrcID, 'AR');
			END IF;
		END IF;

		_CounterJnlD:=1;

		SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _CounterJnlD, _AccAsset, 'D', ROUND(_Total*_ExcRate, _Round), _DeptID, _PrjID, _Total);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		/** Jurnal Akumulasi */
		OPEN cSaleAcc FOR
			SELECT accaccum, SUM(fa.accumcost) 
			FROM saledfa sf JOIN fa ON sf.faid=fa.faid
			WHERE saleno=_SaleNo
			GROUP BY accaccum; 
		LOOP
			FETCH cSaleAcc INTO _AccAccum, _AccumCost;
			EXIT WHEN NOT FOUND;

			IF (_AccumCost!=0) THEN
				_CounterJnlD:=_CounterJnlD+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _CounterJnlD, _AccAccum, 'D', _AccumCost, _DeptID, _PrjID, _AccumCost);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END LOOP;
		CLOSE cSaleAcc;

		---- Jurnal Discount Header --------
		IF (_DiscHeader<>0) THEN
			IF (_AccDisc IS NULL) THEN
				_AccDisc:=fAccDefault('SALEDISC');
			END IF;

			_CounterJnlD:=_CounterJnlD+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _CounterJnlD, _AccDisc, 'K', ROUND(_DiscHeader*_ExcRate*-1, _Round), NULL, NULL, _DiscHeader*-1);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		--- Ayat Pajak , groupped by item tax per item -----
		_TotalTax:=0;
		OPEN cTax FOR
			SELECT 
				SUM(fTaxCalc(d.taxid, qty*(listprice-fDiscCalc(discexp,listprice)-disc2amt)*_ExcRate, _isTaxed, _isTaxInc, 0)),
				SUM(fTaxCalc(d.taxid, qty*(listprice-fDiscCalc(discexp,listprice)-disc2amt), _isTaxed, _isTaxInc, 0)),
				AccSaleTax
			FROM 
				SaleDFA d 
				JOIN tax ON d.taxid=tax.taxid
			WHERE 
				d.SaleNo=_SaleNo 
				AND (d.taxid IS NOT NULL)
			GROUP BY tax.AccSaleTax;
		LOOP
			FETCH cTax INTO _SaleDTaxAmt, _SaleDTaxAmtForex, _AccSaleTax, _DeptID, _PrjID;
			EXIT WHEN NOT FOUND;

			_CounterJnlD:=_CounterJnlD+1;

			IF (_AccSaleTax IS NULL) THEN
				_retval.ErrNo:=504334;
				_retval.ErrMsg:='Akun Pajak Penjualan belum di set, silahkan set di master Pajak (spSale_New)';
				RETURN NEXT _retval;
			END IF;

			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _CounterJnlD, _AccSaleTax, 'K', _SaleDTaxAmt, _DeptID, _PrjID, _SaleDTaxAmtForex);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cTax;

		/** Jurnal Laba Rugi Penjualan Harta Tetap */
		OPEN cSaleAcc FOR
			SELECT 
				m.accloss, 
				SUM(fTaxCalc(d.TaxID, ((d.ListPrice-fDiscCalc(d.DiscExp, d.ListPrice))-d.fabv)*_ExcRate, _isTaxed, _isTaxInc, 1)) 
			FROM SaleDFA d JOIN Sale m ON d.saleno=m.saleno
			WHERE m.SaleNo=_SaleNo 
			GROUP BY m.accloss;
		LOOP
			FETCH cSaleAcc INTO _AccSale, _AccAmt;
			EXIT WHEN NOT FOUND;

			IF (_AccAmt<>0) THEN
				_DK:='K';
				IF (_AccAmt<0) THEN
					_DK:='D';
					_AccAmt:=_AccAmt*-1;
				END IF;
				_CounterJnlD:=_CounterJnlD+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _CounterJnlD, _AccSale, _DK, _AccAmt, _DeptID, _PrjID, _AccAmt);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END LOOP;
		CLOSE cSaleAcc;

		/** Jurnal Harta Tetap */
		OPEN cSaleAcc FOR
			SELECT accasset, SUM(acqcost) 
			FROM saledfa sf JOIN fa ON sf.faid=fa.faid
			WHERE saleno=_SaleNo
			GROUP BY accasset; 
		LOOP
			FETCH cSaleAcc INTO _AccFA, _AcqCost;
			EXIT WHEN NOT FOUND;

			IF (_AcqCost!=0) THEN
				_CounterJnlD:=_CounterJnlD+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _CounterJnlD, _AccFA, 'K', _AcqCost, _DeptID, _PrjID, _AcqCost);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END IF;
		END LOOP;
		CLOSE cSaleAcc;

		SELECT INTO _spret * FROM spJnl_New(_JnlNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spSaleFA_Void(_SaleNo VARCHAR) 
 RETURNS SETOF spStatus AS 
$BODY$
DECLARE
	_retval 			spStatus%ROWTYPE;
	_spret 				spStatus%ROWTYPE;

	_TermType			VARCHAR(1);
	_PayStatus			VARCHAR(1);
	_RcvNo				VARCHAR;
	_JnlNo				VARCHAR(15);
	
	cSaleD 				REFCURSOR;
	
	_isDraft			BOOLEAN;
	_SaleDate			DATE;
	_LockDate			DATE;
		
	_FAID				VARCHAR;
	_FABV				NUMERIC(19,4);
BEGIN
	/** Inisialisasi */
	_retval:=(0, 'OK');	

	SELECT INTO 
		_isDraft, _TermType, _PayStatus, _SaleDate
		isDraft, TermType, PayStatus, SaleDate
	FROM Sale WHERE SaleNo=_SaleNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Penjualan Harta Tetap# '||_SaleNo||' tidak ditemukan (spSaleFA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft=TRUE) THEN
		DELETE FROM SaleDFA WHERE SaleNo=_SaleNo;
		DELETE FROM Sale WHERE SaleNo=_SaleNo;

		_retval:=(0,'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='SALE';
	IF(_LockDate is not null AND _LockDate>_SaleDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penjualan Harta Tetap Melebihi Batas Tanggal Kunci Transaksi ! (spSaleFA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF((_TermType='R') AND (_PayStatus<>'N')) THEN
		_retval.ErrNo='50002';
		_retval.ErrMsg='Sudah ada transaksi pembayaran untuk Penjualan ini.VOID dulu pembayarannya! (spSaleFA_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;		
	END IF;

	IF (_TermType='C') THEN
		SELECT INTO _RcvNo RcvNo FROM Rcv WHERE RefType='SALE' AND RefNo=_SaleNo;

		IF (_RcvNo IS NOT NULL) THEN
			SELECT INTO _spret * from spRcv_Void(_RcvNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;			
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;	
	ELSIF (_TermType='R') THEN
		SELECT INTO _spret * FROM spBPA_Void_byRef('SALE',_SaleNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;			
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;
		
	OPEN cSaleD FOR
		SELECT faid, fabv FROM saledfa  
		WHERE saleno=_SaleNo;
	LOOP
		FETCH cSaleD INTO _FAID, _FABV;
		EXIT WHEN NOT FOUND;
			
		UPDATE fa SET fabv=_FABV, active=TRUE WHERE faid=_FAID;
		DELETE FROM saledfa WHERE saleno=_SaleNo AND faid=_FAID;
	END LOOP;	
	CLOSE cSaleD;
				
	IF (isAccInstalled()) THEN
		SELECT INTO _spret * from spJnl_Void_byRef('SALE',_SaleNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	DELETE FROM sale WHERE saleno=_SaleNo;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP FUNCTION spCheckDP(VARCHAR);
CREATE OR REPLACE FUNCTION spCheckDP (
	_RefType	VARCHAR,
	_RefNo		VARCHAR
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_DPId		BIGINT;
	_PaidNo		VARCHAR;
BEGIN

	SELECT INTO _DPId, _PaidNo 
		dpid, paidno 
	FROM dp 
	WHERE refno=_RefNo AND reftype=_RefType;
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:=_retval.ErrNo||E'|Sudah ada pembayaran Uang Muka:'||COALESCE(_PaidNo, '<kosong>')||' untuk transaksi ini ! (spCheckDP)'||E'\n'||'Silahkan Void terlebih dahulu pembayaran Uang Muka tersebut !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spFAGrp_New(_FAGrpID VARCHAR)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spFAGrp_Void(_FAGrpID VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_CashID		VARCHAR(20);
	_InOut 		VARCHAR(3);
	_CbgAmt		decimal(19,4);
	_outbal		decimal(19,4);
	_active		Boolean;
	_ClearDate	date;
	total		decimal(19,4);
	
BEGIN

	IF EXISTS (SELECT 1 FROM fa WHERE fagrpid=_FAGrpID) THEN 
		_retval.ErrNo:=50003;
		_retval.ErrMsg:='Kelompok Harta tetap #'||_FAGrpID||' sudah dipakai untuk transaksi Harta Tetap';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	DELETE FROM fagrp WHERE FaGrpID=_FAGrpID;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fPPNKeluaran (
	_StartDate	DATE,
	_EndDate	DATE,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS SETOF tPPNMasukan AS
$BODY$
DECLARE
	_retval		tPPNMasukan%ROWTYPE;
	cAcc		REFCURSOR;
	cLgr		REFCURSOR;
	_PurcDate	DATE;
	_PurcNo		VARCHAR;
	_BPName		VARCHAR;
	_TaxAmt		NUMERIC(19,4);
	_ExcRate	NUMERIC(19,4);
	_PRetDate	DATE;
	_PRetNo		VARCHAR;
	_saldoAwal	NUMERIC(19,4);
	_saldo		NUMERIC(19,4);
	_accSaleTax	VARCHAR;
	_LgrAmt		NUMERIC(19,4);
	_LgrDate	DATE;
	_RefNo		VARCHAR;
	_RefType	VARCHAR;
BEGIN
	/*
	--Pembelian
	OPEN cPurc FOR
		SELECT purcdate, purcno, bpname, taxamt, excrate 
		FROM purc 
		JOIN bp ON purc.vendorid=bp.bpid
		WHERE purcdate>=_StartDate AND purcdate<=_EndDate AND isTaxed=true;
	LOOP
		FETCH cPurc INTO _PurcDate, _PurcNo, _BPName, _TaxAmt, _ExcRate;
		EXIT WHEN NOT FOUND;

		_retval.taxdate:=_PurcDate;
		_retval.reftype:='Pembelian';
		_retval.refno:=_PurcNo;
		_retval.vendor:=_BPName;
		_retval.amount:=_TaxAmt;
		_retval.excrate:=_ExcRate;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cPurc;

	--Retur Pembelian
	OPEN cPRet FOR
		SELECT pretdate,pretno,bpname,taxamt,excrate 
		FROM pret 
		JOIN bp ON pret.vendorid=bp.bpid
		WHERE pretdate>=_StartDate AND pretdate<=_EndDate AND isTaxed=true;
	LOOP
		FETCH cPRet INTO _PretDate, _PretNo, _BPName, _TaxAmt, _ExcRate;
		EXIT WHEN NOT FOUND;

		_retval.taxdate:=_PretDate;
		_retval.reftype:='Retur Pembelian';
		_retval.refno:=_PretNo;
		_retval.vendor:=_BPName;
		_retval.amount:=_TaxAmt;
		_retval.excrate:=_ExcRate;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cPRet;*/

	_saldoAwal:=0;
	OPEN cAcc FOR
		SELECT accsaletax FROM tax;
	LOOP
		FETCH cAcc INTO _AccSaleTax;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Saldo * FROM COALESCE(fAccBal_StartDate(_AccSaleTax,_startdate-1,_branchid,_deptid,_prjid),0);
		_saldoAwal:=_saldoAwal+_Saldo;
	END LOOP;
	CLOSE cAcc;

	_retval.taxdate:=NULL;
	_retval.reftype:='Saldo Awal';
	_retval.refno:=null;
	_retval.vendor:=null;
	_retval.amount:=_SaldoAwal;
	_retval.excrate:=null;

	RETURN NEXT _retval;

	OPEN cLgr FOR
		SELECT lgramt,refno,jnltype,lgrdate 
		FROM lgr 
		JOIN jnl OPN lgr.jnlno=jnl.jnlno 
		WHERE lgr.accno IN (SELECT accsaletax FROM tax) AND lgrdate>=_StartDate AND lgrdate<=_EndDate 
		ORDER BY lgrdate;
	LOOP
		FETCH cLgr INTO _LgrAmt,_RefNo,_RefType,_LgrDate;
		EXIT WHEN NOT FOUND;

		IF(_RefType='BEG' OR _RefType='SALE') THEN
			SELECT INTO _BPName,_ExcRate bpname,excrate 
			FROM sale 
			JOIN bp ON sale.custid=bp.bpid
			WHERE saleno=_RefNo;

			_retval.reftype:='Penjualan';
		ELSIF(_RefType='SRET') THEN
			SELECT INTO _BPName,_ExcRate bpname,excrate 
			FROM sret 
			JOIN bp ON sret.custid=bp.bpid
			WHERE sretno=_RefNo;

			_retval.reftype:='Retur Penjualan';
		ELSIF(_RefType='RCV') THEN
			SELECT INTO _BPName,_ExcRate bpname,excrate 
			FROM rcv 
			JOIN bp ON rcv.bpid=bp.bpid
			WHERE rcvno=_RefNo;

			_retval.reftype:='Penerimaan Pembayaran';
		ELSIF(_RefType='GJ') THEN
			_retval.reftype:='Jurnal';
		END IF;

		_retval.taxdate:=_LgrDate;
		_retval.refno:=_RefNo;
		_retval.vendor:=_BPName;
		_retval.amount:=_LgrAmt;
		_retval.excrate:=_ExcRate;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cLgr;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPPNMasukan (
	_StartDate	DATE,
	_EndDate	DATE,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS SETOF tPPNMasukan AS
$BODY$
DECLARE
	_retval		tPPNMasukan%ROWTYPE;
	cAcc		REFCURSOR;
	cLgr		REFCURSOR;
	_PurcDate	DATE;
	_PurcNo		VARCHAR;
	_BPName		VARCHAR;
	_TaxAmt		NUMERIC(19,4);
	_ExcRate	NUMERIC(19,4);
	_PRetDate	DATE;
	_PRetNo		VARCHAR;
	_saldoAwal	NUMERIC(19,4);
	_saldo		NUMERIC(19,4);
	_accPurcTax	VARCHAR;
	_LgrAmt		NUMERIC(19,4);
	_LgrDate	DATE;
	_RefNo		VARCHAR;
	_RefType	VARCHAR;
BEGIN
	/*
	--Pembelian
	OPEN cPurc FOR
		SELECT purcdate,purcno,bpname,taxamt,excrate 
		FROM purc 
		JOIN bp ON purc.vendorid=bp.bpid
		WHERE purcdate>=_StartDate AND purcdate<=_EndDate AND isTaxed=true;
	LOOP
		FETCH cPurc INTO _PurcDate, _PurcNo, _BPName, _TaxAmt, _ExcRate;
		EXIT WHEN NOT FOUND;

		_retval.taxdate:=_PurcDate;
		_retval.reftype:='Pembelian';
		_retval.refno:=_PurcNo;
		_retval.vendor:=_BPName;
		_retval.amount:=_TaxAmt;
		_retval.excrate:=_ExcRate;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cPurc;


	--Retur Pembelian
	OPEN cPRet FOR
		SELECT pretdate,pretno,bpname,taxamt,excrate 
		FROM pret 
		JOIN bp ON pret.vendorid=bp.bpid
		WHERE pretdate>=_StartDate AND pretdate<=_EndDate AND isTaxed=true;
	LOOP
		FETCH cPRet INTO _PretDate, _PretNo, _BPName, _TaxAmt, _ExcRate;
		EXIT WHEN NOT FOUND;

		_retval.taxdate:=_PretDate;
		_retval.reftype:='Retur Pembelian';
		_retval.refno:=_PretNo;
		_retval.vendor:=_BPName;
		_retval.amount:=_TaxAmt;
		_retval.excrate:=_ExcRate;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cPRet;*/

	_saldoAwal:=0;
	OPEN cAcc FOR
		SELECT accpurctax FROM tax;
	LOOP
		FETCH cAcc INTO _AccPurcTax;
		EXIT WHEN NOT FOUND;

		SELECT INTO _Saldo * FROM COALESCE(fAccBal_StartDate(_AccPurcTax,_startdate-1,_branchid,_deptid,_prjid),0);
		_saldoAwal:=_saldoAwal+_Saldo;
	END LOOP;
	CLOSE cAcc;

	_retval.taxdate:=NULL;
	_retval.reftype:='Saldo Awal';
	_retval.refno:=null;
	_retval.vendor:=null;
	_retval.amount:=_SaldoAwal;
	_retval.excrate:=null;

	RETURN NEXT _retval;

	OPEN cLgr FOR
		SELECT lgramt,refno,jnltype,lgrdate 
		FROM lgr 
		JOIN jnl ON lgr.jnlno=jnl.jnlno 
		WHERE lgr.accno IN (SELECT accpurctax FROM tax) AND lgrdate>=_StartDate AND lgrdate<=_EndDate 
		ORDER BY lgrdate;
	LOOP
		FETCH cLgr INTO _LgrAmt,_RefNo,_RefType,_LgrDate;
		EXIT WHEN NOT FOUND;

		IF(_RefType='BEG' OR _RefType='PURC') THEN
			SELECT INTO _BPName,_ExcRate bpname,excrate 
			FROM purc 
			JOIN bp ON purc.vendorid=bp.bpid
			WHERE purcno=_RefNo;

			_retval.reftype:='Pembelian';
		ELSIF(_RefType='PRET') THEN
			SELECT INTO _BPName,_ExcRate bpname,excrate 
			FROM pret 
			JOIN bp ON pret.vendorid=bp.bpid
			WHERE pretno=_RefNo;

			_retval.reftype:='Retur Pembelian';
		ELSIF(_RefType='PAY') THEN
			SELECT INTO _BPName,_ExcRate bpname,excrate 
			FROM pay 
			JOIN bp ON pay.bpid=bp.bpid
			WHERE payno=_RefNo;

			_retval.reftype:='Pembayaran';
		ELSIF(_RefType='GJ') THEN
			_retval.reftype:='Jurnal';
		END IF;

		_retval.taxdate:=_LgrDate;
		_retval.refno:=_RefNo;
		_retval.vendor:=_BPName;
		_retval.amount:=_LgrAmt;
		_retval.excrate:=_ExcRate;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cLgr;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fRekonsiliasiPPN (
	_StartDate	DATE,
	_EndDate	DATE,
	_BranchID	VARCHAR,
	_DeptID		VARCHAR,
	_PrjID		VARCHAR
) RETURNS SETOF tRekonsiliasiPPN AS
$BODY$
DECLARE
	_retval		tRekonsiliasiPPN%ROWTYPE;
	cKeluaran	REFCURSOR;
	cMasukan	REFCURSOR;
	_taxdate	DATE;
	_reftype	VARCHAR;
	_refno		VARCHAR;
	_vendor		VARCHAR;
	_amount		NUMERIC(19,4);
	_excrate	NUMERIC(19,4);
BEGIN
	/*ppn masukan*/
	OPEN cKeluaran FOR
		SELECT taxdate,reftype,refno,vendor,amount,excrate FROM 
		fPPNMasukan(_StartDate,_EndDate,_BranchID,_DeptID,_PrjID);
	LOOP
		FETCH cKeluaran INTO _taxdate,_reftype,_refno,_vendor,_amount,_excrate;
		EXIT WHEN NOT FOUND;

		_retval.tipe:='Daftar PPN Masukan';
		_retval.taxdate:=_taxdate;
		_retval.reftype:=_reftype;
		_retval.refno:=_refno;
		_retval.vendor:=_vendor;
		_retval.amount:=_amount;
		_retval.excrate:=_excrate;
		_retval.masukan:=_amount;
		_retval.keluaran:=0;
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cKeluaran;


	/*ppn keluaran*/
	OPEN cKeluaran FOR
		SELECT taxdate,reftype,refno,vendor,amount,excrate FROM 
		fPPNKeluaran(_StartDate,_EndDate,_BranchID,_DeptID,_PrjID);
	LOOP
		FETCH cKeluaran INTO _taxdate,_reftype,_refno,_vendor,_amount,_excrate;
		EXIT WHEN NOT FOUND;

		_retval.tipe:='Daftar PPN Keluaran';
		_retval.taxdate:=_taxdate;
		_retval.reftype:=_reftype;
		_retval.refno:=_refno;
		_retval.vendor:=_vendor;
		_retval.amount:=_amount;
		_retval.excrate:=_excrate;
		_retval.masukan:=0;
		_retval.keluaran:=_amount;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cKeluaran;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/**
SADJMtd:
	SIN = stock in
	SOUT = Stock Out
	RD = Rounding (penyesuaian stock dibebankan / di round ke HPP itu sendiri 
	RPL = Replace HPP yg ada dilawankan ke akun
	PL = alias. ACC (dibebankan ke akun keuntungan/kerugian) Profit/Loss
**/

CREATE OR REPLACE FUNCTION spSAdj_New (
	_SAdjNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_spretstockad_new	SPStatus%ROWTYPE;
	
	cSadjD			REFCURSOR;
	cStockAD		REFCURSOR;

	_SadjDate		DATE;
	_SadjType		VARCHAR;
	_SadjMtd		VARCHAR;
	_CrtBy			VARCHAR;
	_CrtDate		DATE;
	_UpdBy			VARCHAR;
	_UpdDate		DATE;
	_SADJAccNo		VARCHAR;
	_SadjTypeDesc		VARCHAR;
	_Cnt			INT;
	_StockADNo		INT2;
	_SadjDNo		INT;
	_ItemID			VARCHAR;
	_WhID			VARCHAR;
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR;
	_Cost			NUMERIC(19,4);
	_PID			VARCHAR;
	_QtyX			NUMERIC(15,4);
	_StockANote		VARCHAR;
	_StockANo		VARCHAR;
	_AvgCost		NUMERIC(19,4);
	_StockADCost		NUMERIC(19,4);
	_JnlNo			VARCHAR;
	_JnlNote		VARCHAR;
	_JnlType		VARCHAR;
	_Round			INT;
	_Count			INT;
	_CountSAdj		INT;
	_total			NUMERIC(19,4);
	_DK			VARCHAR;
	_AccInv			VARCHAR;
	_DeptID			VARCHAR;
	_PrjID			VARCHAR;
	_CostNow		NUMERIC(19,4);
	_LockDate		DATE;
	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);
	_AccNo			VARCHAR;
	_RefType		VARCHAR;
	_EmpID			VARCHAR;
	_SAdjNote		VARCHAR;
	_PerID			VARCHAR;
	_BranchID		VARCHAR;
	_Conv			NUMERIC(19,4);
	_isDraft		BOOL;

	_StockQty		NUMERIC(19,4);
	_StockSubtotal		NUMERIC(19,4);
	_Subtotal		NUMERIC(19,4);
BEGIN
/**
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
	RETURN;
**/
	_retval:=(0, 'OK');

	SELECT INTO _SadjDate,_SAdjType, _SAdjMtd, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _SADJAccNo, _EmpID, _SAdjNote, _PerID, _BranchID, _isDraft
		sadjdate, sadjtype, sadjmtd, crtby, crtdate, updby, upddate, sadjaccno, empid, sadjNote, perid, branchid, isdraft 
	FROM sadj WHERE sadjno=_SAdjNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Transaksi stock #'||_SadjNo|| ' tidak ditemukan! (spSAdj_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockDATE FROM TLock WHERE TransType='SADJ';
	IF(_LockDate IS NOT NULL AND _LockDate>_SadjDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi stock melebihi batas tanggal kunci transaksi ! (spSAdj_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF EXISTS (SELECT 1 FROM sadj m JOIN sadjd d ON d.sadjno=m.sadjno WHERE m.sadjno=_SAdjNo AND sadjmtd<>'RPL' AND qty=0) THEN 
		_retval.ErrNo:=50003;
		_retval.ErrMsg='Qty tidak boleh ada yang nol ! (spSAdj_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _SadjTypeDesc sadjtypedesc FROM sadjtype WHERE sadjtype=_SadjType;

	SELECT INTO _CrcID crcid FROM crc WHERE isdefault=true;
	_ExcRate:=1;
	_FisRate:=1;

	_Cnt:=1;
	_StockADNo:=0;
	OPEN cSadjD FOR
		SELECT sadjdno, d.itemid, d.whid, qty, unit, cost, pid, qtyx, deptid, prjid
		FROM sadjd d JOIN item i ON d.itemid=i.itemid
		WHERE sadjno=_SadjNo AND i.isstock=true;
	LOOP
		FETCH cSadjD INTO _SadjDNo, _ItemID, _WhID, _Qty, _Unit, _Cost, _PID, _QtyX, _DeptID, _PrjID;
		EXIT WHEN NOT FOUND;

		_PID:=COALESCE(_PID, '');

		SELECT INTO _count COUNT(*) 
		FROM stockad d 
		JOIN stocka h ON d.stockano=h.stockano
		WHERE (h.reftype<>'SADJ' AND h.reftype<>'SIN' AND h.reftype<>'SOUT')
		AND d.itemid=_ItemID AND d.whid=_WhID AND d.pid=_PID;

		IF ((_SAdjType='SADJ' OR _SAdjType='SIN' OR _SAdjType='SOUT') AND (_count IS NULL OR _count=0)) THEN
			_retval.ErrNo:=50001;
			_retval.ErrMsg:='Penyesuaian Stock hanya boleh digunakan untuk pembetulan'||E'\n'||'Item '||_ItemID||', PID:'||_PID||', Gudang: '||_WhID||' belum pernah ada transaksi sebelumnya.'||E'\n'||'Untuk set saldo awal stock gunakan menu Set-Saldo-Awal-Stock atau Pembelian! (spSAdj_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF ((_SAdjType='BEG') AND (_count IS NOT NULL AND _count>0)) THEN
			SELECT INTO _countSAdj COUNT(DISTINCT(refno)) 
			FROM stockad d 
			JOIN stocka h ON d.StockANo=h.StockANo
			WHERE (h.reftype<>'SADJ' AND h.reftype<>'SIN' AND h.reftype<>'SOUT')
			AND d.itemid=_ItemID AND d.whid=_WhID AND d.pid=_PID AND h.refno<>_SAdjNo AND (h.branchid IS NULL OR h.branchid=_BranchID);

			IF ((_countSAdj IS NOT NULL AND _countSAdj>0)) THEN
				_retval.ErrNo:=50001;
				_retval.ErrMsg:='Set Saldo Awal Stock hanya boleh digunakan jika belum terjadi transaksi. '||E'\n'||'Item '||_ItemID||', PID:'||_PID||', Gudang: '||_WhID||' Cabang: '||COALESCE(_BranchID,'<kosong>')||' sudah pernah ada transaksi sebelumnya.'||E'\n'||' Gunakan menu Penyesuaian Stock ! (spSAdj_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

		_StockANote:=_SAdjTypeDesc|| ' #' || _SAdjNo || COALESCE(_SAdjNote,'');

		IF(_Cnt=1) THEN
			_StockADNo:=0;

			SELECT INTO _StockANo * FROM fTransNoGet('STOCKA');
			IF (_StockANo IS NULL) THEN
				_retval.ErrNo:=3;
				_retval.ErrMsg:='ERROR getting StockANo (spSAdj_New)';
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			SELECT INTO _spret * FROM spStockA_New(
				_StockANo, NULL, _SadjType, _SadjDate, NULL, _SadjType, _SAdjNo, 
				_StockANote, true, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _EmpID
			);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			_cnt:=_cnt+1;
		END IF;

		IF((_SAdjType='SIN' OR _SAdjType='SOUT') AND _Qty<=0) THEN
			_retval.ErrNo:=3;
			_retval.ErrMsg:='Stock '|| CASE WHEN _SAdjType='SIN' THEN 'Masuk' ELSE 'Keluar' END ||' Item : '||_ItemID||' qty tidak boleh negatif ! (spSAdj_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF(_SAdjType='SOUT') THEN
			_Qty:=_Qty*-1;
		END IF;

		_Conv:=fItemUnit_Conv(_ItemID, _Unit);

		IF(_SadjType<>'BEG') THEN
			_CostNow:=fUnitCost(_ItemID,_WhID,_PID,_Unit);

			IF(_SadjMtd='PL') THEN
				IF(_SAdjType='SIN' AND _Cost<>0) THEN
					_AvgCost:=_Cost;
				ELSE
					_AvgCost:=_CostNow;
				END IF;
			ELSIF(_SadjMtd='RD') THEN
				_AvgCost:=0;
			ELSIF(_SadjMtd='RPL') THEN
				_AvgCost:=_Cost;
			END IF;
	
			IF(_SadjMtd='PL' OR _SadjMtd='RD') THEN
				IF (NOT (_SAdjType='SIN' AND _Cost<>0)) THEN
					UPDATE sadjd SET cost=_AvgCost, conv=_Conv WHERE sadjno=_SAdjNo and sadjdno=_SAdjDNo;
				END IF;
			END IF;

			IF(_SadjMtd='RPL') THEN
				UPDATE sadjd SET qty=0, oldcost=_CostNow WHERE sadjno=_SAdjNo and sadjdno=_SAdjDNo;
				_Qty:=0;
			END IF;
		END IF;

		IF(_SadjType='BEG') THEN
			_AvgCost:=_Cost;
		END IF;

		UPDATE sadjd SET conv=_Conv WHERE sadjno=_SAdjNo and sadjdno=_SAdjDNo;
		_StockADNo:=_StockADNo+1;
		_StockADCost:=_AvgCost;
		IF (_PID IS NULL ) THEN
			_PID:='';
		END IF;

		IF (_Qty<0) THEN
			SELECT INTO _StockQty, _StockSubtotal qty, subtotal FROM stock WHERE itemid=_ItemID AND whid=_WhID AND pid=_PID;
			IF (FOUND AND _StockQty>0) THEN
				_Subtotal:=((_Qty*_Conv)/_StockQty)*_StockSubtotal;
			ELSE
				_Subtotal:=_Qty*_StockADCost;
			END IF;

			IF (_SadjMtd='RD') THEN
				_Subtotal:=0;
			END IF;

			SELECT INTO _spret * FROM spStockAD_New(
				_SadjDate,_SadjType, _StockANo, _StockADNo, _WhID, 
				_ItemID, _PID, _Qty, _Unit, _QtyX, _StockADCost, NULL, NULL, true, _SAdjNo, _Subtotal
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		ELSE
			SELECT INTO _spret * FROM spStockAD_New(
				_SadjDate,_SadjType, _StockANo, _StockADNo, _WhID, 
				_ItemID, _PID, _Qty, _Unit, _QtyX, _StockADCost, NULL, NULL, true, _SAdjNo
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;
	END LOOP;
	CLOSE cSadjD;

/********************************* AKUNTANSI ************************************************/

	IF (isAccInstalled() AND (_SadjMtd='RPL' OR _SadjMtd='PL')) THEN
		_JnlNo:=fTransNoGet('JURNAL');
		IF(_SAdjType='BEG') THEN
			_JnlType:='BEG';
			_JnlNote:='Saldo Awal Stock '||':'||_stockano;
			_AccNo:=fAccDefault('BEGBAL');
		ELSE
			_JnlNote:= _SadjTypeDesc||' :'||_sadjno;
			_JnlType:='SADJ';
			_AccNo:=_SADJAccNo;
		END IF;

		
		SELECT INTO _spretstockad_new * FROM spJnl_Insert(
			_JnlNo, _JnlType, _sADJDATE, 'SADJ', _SADjNo,
			_JnlNote,true,true, CAST(0 as NUMERIC(19,4)),
			_CrtBy,_CrtDate,_UpdBy,_UpdDate,
			_PerID, _BranchID, NULL, NULL, _CrcID, _ExcRate, _FisRate
		);
		IF _spretstockad_new.ErrNo<>0 THEN
			_retval.ErrNo:=_spretstockad_new.ErrNo;
			_retval.ErrMsg:=_spretstockad_new.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_Round:=fReg('ROUND');
		IF(_Round is null) THEN
			_Round:=0;
		END IF;
		_Count:=0;
		_total:=0;

		
		IF(_sadjMtd='PL') THEN
			
			OPEN cStockad FOR
			SELECT fItemAcc(itemid,'INV'),ROUND(SUM(qty*cost),_Round), deptID, prjID
			from sadjd where sadjno=_sadjno
			GROUP BY fItemAcc(itemid,'INV'), deptID, prjID;
			
		ELSIF(_sadjMtd='RPL') THEN
			OPEN cStockad FOR
		SELECT fItemAcc(d.itemid,'INV'),ROUND(SUM(s.qty*avgcost)-SUM(s.qty*oldcost),_Round), deptID, prjID 
			from sadjd d join stock s on d.itemid=s.itemid and d.whid=s.whid and d.pid=s.pid
			where sadjno=_sadjno
			GROUP BY fItemAcc(d.itemid,'INV'), deptID, prjID;
		END IF;

		LOOP
			FETCH cStockad INTO _AccInv,_cost, _deptID, _prjID;
			EXIT WHEN NOT FOUND;

			IF (_SAdjType='SOUT') THEN
				_cost:=_cost*-1;
			END IF;

			_Count:=_Count+1;
			SELECT INTO _spretstockad_new * 
			FROM spJnlD_Insert(_JnlNo,_Count,_AccInv,'D', _cost,_deptID, _prjID, _cost);
				IF _spretstockad_new.ErrNo<>0 THEN
					_retval.ErrNo:=_spretstockad_new.ErrNo;
					_retval.ErrMsg:=_spretstockad_new.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			_total:=_total+_cost;

			_DK:='K';

			_Count:=_Count+1;
			SELECT INTO _spretstockad_new * 
			FROM spJnlD_Insert(_JnlNo,_Count,_AccNo,_DK,_cost,_deptID,_prjID,_cost);

			IF _spretstockad_new.ErrNo<>0 THEN
				_retval.ErrNo:=_spretstockad_new.ErrNo;
				_retval.ErrMsg:=_spretstockad_new.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;
		CLOSE cStockad;

		SELECT INTO _spretstockad_new * FROM spJnl_New(_JnlNo);
		IF _spretstockad_new.ErrNo<>0 THEN
			_retval.ErrNo:=_spretstockad_new.ErrNo;
			_retval.ErrMsg:=_spretstockad_new.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE plpgsql VOLATILE;




CREATE OR REPLACE Function spSAdj_Void(_SAdjNo VARCHAR)
RETURNS SETOF SPSTATUS AS 
$BODY$
DECLARE	
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_spretstockad_void	SPStatus%ROWTYPE;
	cSadjD			REFCURSOR;
	_SadjMtd		VARCHAR;
	_StockANo		VARCHAR;
	_SadjDNo		VARCHAR;
	_WhID			VARCHAR;
	_ItemID			VARCHAR;
	_PID			VARCHAR;
	_Qty			NUMERIC(15,4);
	_Unit			VARCHAR;
	_Conv			NUMERIC(15,4);
	_QtyX			NUMERIC(15,4);
	_OldCost		NUMERIC(19,4);
	_Cost			NUMERIC(19,4);
	_SADjDate		DATE;
	_SADjType		VARCHAR;
	_AvgCost		NUMERIC(19,4);
	_RefType		VARCHAR;
	_LockDate		DATE;
	_isDraft		BOOL;
BEGIN

	SELECT INTO _SadjMtd,_SAdjDate,_SADjType,_isDraft sadjmtd,sadjdate,sadjtype,isdraft FROM sadj 
	WHERE sadjno=_SAdjNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Transaksi-Stock #'||_SAdjNo|| ' Tidak ditemukan! (spSAdj_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		_retval:=(0, 'OK');
		DELETE FROM sadjd WHERE sadjno=_SAdjNo;
		DELETE FROM sadj WHERE sadjno=_SAdjNo;
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockDATE FROM TLock WHERE TransType='SADJ';
	IF(_LockDate IS NOT NULL AND _LockDate>_SAdjDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Transaksi Stock Melebihi Batas Tanggal Kunci Transaksi ! (spSAdj_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	/*KLO TYPE NYA SIN PENGARUH KE HPP, JADINYA G MANGGIL STOCKA_VOID_REF*/
	IF(_SadjMtd='PL' AND _SadjType<>'SIN') THEN
		SELECT into _spret * FROM spStockA_Void_byRef(_SADjType, _SAdjNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
			EXIT;
		END IF;
	ELSE

		SELECT INTO _StockANo stockano from stocka where stockatype=_SADjType and refno=_SadjNo;
		OPEN cSadjD FOR
			SELECT SadjDNo, WhID, ItemID, PID, Qty, Unit, Conv, QtyX, OldCost,Cost
			FROM SadjD WHERE SadjNo=_SadjNo;
			LOOP
				FETCH cSadjD INTO _SadjDNo, _WhID, _ItemID, _PID, _Qty, _Unit, _Conv, _QtyX, _OldCost,_Cost;
				EXIT WHEN NOT FOUND;

				IF(_SadjMtd='RD') THEN
					_AvgCost:=_Cost;
					_Qty=_Qty*(-1);
				ELSIF(_SadjMtd='PL') THEN
					_AvgCost:=_Cost;
					_Qty=_Qty*(-1);
				ELSIF(_SadjMtd='RPL') THEN
					_AvgCost:=_OldCost;
				END IF;
				
				SELECT INTO _spret * FROM spStockBal_Update(_ItemID, _WhID, _PID, _Qty, _Unit, _Conv, _QtyX, _AvgCost, _SADjDate, _SADjType, _SAdjNo);

			END LOOP;
		CLOSE cSadjD;

		DELETE FROM StockAD WHERE StockANo=_StockANo;
		DELETE FROM stocka WHERE stockano=_stockano;	
	END IF;

	IF isAccInstalled() THEN
		SELECT into _spretstockad_void * from spJnl_Void_byRef('SADJ',_SAdjNo);
		IF _spretstockad_void.ErrNo<>0 THEN
			_retval.ErrNo:=_spretstockad_void.ErrNo;
			_retval.ErrMsg:=_spretstockad_void.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
	/*--		EXIT;*/
		END IF;
	END IF;

	DELETE FROM sadjd WHERE sadjno=_sadjno;
	DELETE FROM sadj WHERE sadjno=_sadjno;

	_retval:=(0, 'OK'); 
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCalendar(
	_StartDate	DATE,
	_EndDate	DATE,
	_ShowAR		BOOLEAN,
	_ShowAP		BOOLEAN,
	_ShowCbgIN	BOOLEAN,
	_ShowCbgOut	BOOLEAN,
	_ShowTodoNote	BOOLEAN
) RETURNS SETOF tCalendar AS
$BODY$
DECLARE
	_retval		tCalendar%ROWTYPE;
	
	_CalDate	DATE;
	_CalTime	TIMESTAMP;
	_TodoNo		VARCHAR(15);
	_RefType	VARCHAR(5);  /** (Va5): AR, AP, CBGIN, CBGOUT, STOCK, TODO */
	_RefNo		VARCHAR(15);
	_Subject	TEXT;

	_cursor		REFCURSOR;
BEGIN
	OPEN _cursor FOR
		SELECT CURRENT_DATE, CURRENT_TIME, '', '', '', '', '';
	LOOP
		FETCH _cursor INTO _CalDate, _CalTime, _TodoNo, _RefType, _RefNo, _Subject;
		EXIT WHEN NOT FOUND;

	END LOOP;
	CLOSE _cursor;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fCalendarText(
	_StartDate	DATE,
	_EndDate	DATE,
	_ShowAR		BOOLEAN,
	_ShowAP		BOOLEAN,
	_ShowCbgIN	BOOLEAN,
	_ShowCbgOut	BOOLEAN,
	_ShowTodoNote	BOOLEAN
) RETURNS SETOF VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
	
	_dayLength	INT2;
	_dayCounter	INT2;
	_tempDate	DATE;

	_BPName		VARCHAR;
	_Nominal	NUMERIC(19,0);
	_CalDate	DATE;
	_StartTime	TIME;
	_EndTime	TIME;
	_AlarmDate	VARCHAR;
	_AlarmTime	VARCHAR;
	_TodoNo		VARCHAR;
	_cursor		REFCURSOR;
BEGIN
	_dayCounter:=1;
	_tempDate:=_StartDate;
	
	SELECT INTO _dayLength ((_EndDate-_StartDate)+1);

	WHILE (_dayCounter<=_dayLength) LOOP
		_retval:='';
		/** Proses Piutang */
		OPEN _cursor FOR
			SELECT (saledate+duedays) AS duedate, fBPName(custid) AS vendorname, (total-paidamt) AS hutang
			FROM sale 
			WHERE paystatus<>'F' AND (saledate+duedays)=_tempDate;
		LOOP
			FETCH _cursor INTO _CalDate, _BPName, _Nominal;
			EXIT WHEN NOT FOUND;
			
			_retval:=COALESCE(_retval, '')||_CalDate||',<,'||_BPName||':'||_Nominal||E'\\';
		END LOOP;
		CLOSE _cursor;

		/** Proses Hutang */
		OPEN _cursor FOR
			SELECT (purcdate+duedays) AS duedate, fBPName(vendorid) AS vendorname, (total-paidamt) AS hutang
			FROM purc 
			WHERE paystatus<>'F' AND (purcdate+duedays)=_tempDate;
		LOOP
			FETCH _cursor INTO _CalDate, _BPName, _Nominal;
			EXIT WHEN NOT FOUND;

			_retval:=COALESCE(_retval, '')||_CalDate||',>,'||_BPName||':'||_Nominal||E'\\';
		END LOOP;
		CLOSE _cursor;

		/** Proses Giro Masuk */
		OPEN _cursor FOR
			SELECT cbgduedate, fBPName(frombpid) AS BPname, cbgamt
			FROM cbg 
			WHERE active=true AND inout='I' AND cbgduedate=_tempDate;
		LOOP
			FETCH _cursor INTO _CalDate, _BPName, _Nominal;
			EXIT WHEN NOT FOUND;

			_retval:=COALESCE(_retval, '')||_CalDate||',+,'||_BPName||':'||_Nominal||E'\\';
		END LOOP;
		CLOSE _cursor;

		/** Proses Giro Keluar */
		OPEN _cursor FOR
			SELECT cbgduedate, fBPName(tobpid) AS BPname, cbgamt
			FROM cbg 
			WHERE active=true AND inout='O' AND cbgduedate=_tempDate;
		LOOP
			FETCH _cursor INTO _CalDate, _BPName, _Nominal;
			EXIT WHEN NOT FOUND;

			_retval:=COALESCE(_retval, '')||_CalDate||',-,'||_BPName||':'||_Nominal||E'\\';
		END LOOP;
		CLOSE _cursor;

		/** Proses TodoNote */
		OPEN _cursor FOR
			SELECT todono, startdate, todosubject, starttime, endtime, CAST(alarmdt AS DATE), CAST(alarmdt AS TIME) 
			FROM todo 
			WHERE startdate=_tempDate;
		LOOP
			FETCH _cursor INTO _TodoNo, _CalDate, _BPName, _StartTime, _EndTime, _AlarmDate, _AlarmTime;
			EXIT WHEN NOT FOUND;
			
			_retval:=COALESCE(_retval, '')||COALESCE(_CalDate, '0001-01-01')||',@,'||COALESCE(_BPName, 'No BP')||':'||COALESCE(_StartTime,'00:00:00')||','||COALESCE(_TodoNo,'No Todo')||E'\\';
		END LOOP;
		CLOSE _cursor;

		_dayCounter:=_dayCounter+1;
		_tempDate:=_tempDate+1;

		IF (_retval<>'') THEN
			RETURN NEXT _retval;
		END IF;
	END LOOP;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION fGridActiveGiro(
	_inout		CHAR(1)
)
RETURNS SETOF tGridActiveGiro AS
$BODY$
DECLARE
	_retval		tGridActiveGiro%ROWTYPE;
	
	_DueDate	DATE;
	_BPID		VARCHAR(15);
	_BPName		VARCHAR(50);
	_Nominal	NUMERIC(19,4);
	
	_cursor		REFCURSOR;
BEGIN
	IF (_inout='O') THEN /** MODE PURC */
		OPEN _cursor FOR
			SELECT cbgduedate, tobpid, fBPName(tobpid) AS BPname, cbgamt
			FROM cbg 
			WHERE active=true AND inout=_inout;
		LOOP
			FETCH _cursor INTO _DueDate, _BPID, _BPName, _Nominal;
			EXIT WHEN NOT FOUND;
	
			_retval.duedate:=_DueDate;
			_retval.bpid:=_BPID;
			_retval.bpname:=_BPName;
			_retval.nominal:=_Nominal;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE _cursor;
	ELSIF (_inout='I') THEN /** MODE SALE */
		OPEN _cursor FOR
			SELECT cbgduedate, frombpid, fBPName(frombpid) AS BPname, cbgamt
			FROM cbg 
			WHERE active=true AND inout=_inout;
		LOOP
			FETCH _cursor INTO _DueDate, _BPID, _BPName, _Nominal;
			EXIT WHEN NOT FOUND;
	
			_retval.duedate:=_DueDate;
			_retval.bpid:=_BPID;
			_retval.bpname:=_BPName;
			_retval.nominal:=_Nominal;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE _cursor;
	END IF;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fGridMinStock(
	_ModelID	VARCHAR,
	_WhID		VARCHAR,
	_VendorID	VARCHAR,
	_ItGrpID	VARCHAR,
	_ItGrp2ID	VARCHAR,
	_ItGrp3ID	VARCHAR
)
RETURNS SETOF tGridMinStock AS
$BODY$
DECLARE
	_retval		tGridMinStock%ROWTYPE;
	
	_ItemID		VARCHAR(20);
	_ItemDesc	VARCHAR(50);
	_Unit		VARCHAR(5);
	_Saldo		NUMERIC(15,4);
	
	_cursor		REFCURSOR;
BEGIN
	OPEN _cursor FOR
		SELECT s.itemid, i.itemdesc, i.unit1, sum(s.qty), i.qtymin
		FROM stock s JOIN item i ON s.itemid=i.itemid
		WHERE s.qty>0  
		AND (_ModelID IS NULL OR i.modelid=_ModelID) AND (_WhID IS NULL OR s.whid=_WhID) 
		AND (_VendorID IS NULL OR i.vendorid=_VendorID) AND (_ItGrpID IS NULL OR i.itgrpid=_ItGrpID) 
		AND (_ItGrp2ID IS NULL OR i.itgrp2id=_ItGrp2ID) AND (_ItGrp3ID IS NULL OR i.itgrp3id=_ItGrp3ID) 
		GROUP BY s.itemid, i.itemdesc, i.unit1, i.qtymin
		HAVING (SUM(s.qty)<i.qtymin);	
	LOOP
		FETCH _cursor INTO _ItemID, _ItemDesc, _Unit, _Saldo;
		EXIT WHEN NOT FOUND;

		_retval.itemid:=_ItemID;
		_retval.itemdesc:=_ItemDesc;
		_retval.unit:=_Unit;
		_retval.saldo:=_Saldo;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE _cursor;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fGridHPJatuhTempo(
	_mode		INT2,
	_branchid	VARCHAR(5),
	_date		DATE
)
RETURNS SETOF tGridHPJatuhTempo AS
$BODY$
DECLARE
	_retval		tGridHPJatuhTempo%ROWTYPE;
	
	_DueDate	DATE;
	_VendorID	VARCHAR(15);
	_VendorName	VARCHAR(50);
	_Hutang		NUMERIC(19,4);
	_OverDue	INT2;

	_cursor		REFCURSOR;
BEGIN
	IF (_mode=0) THEN /** MODE PURC */
		OPEN _cursor FOR
			SELECT (purcdate+duedays) AS duedate, vendorid, fBPName(vendorid) AS vendorname, 
				(total-paidamt) AS hutang, CAST((_date-(purcdate+duedays)) AS INT2) AS overdue
			FROM purc 
			WHERE paystatus<>'F' AND (_branchid IS NULL OR branchid=_branchid);
		LOOP
			FETCH _cursor INTO _DueDate, _VendorID, _VendorName, _Hutang, _OverDue;
			EXIT WHEN NOT FOUND;
	
			_retval.duedate:=_DueDate;
			_retval.vendorid:=_VendorID;
			_retval.vendorname:=_VendorName;
			_retval.hutang:=_Hutang;
			_retval.overdue:=_OverDue;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE _cursor;
	ELSIF (_mode=1) THEN /** MODE SALE */
		OPEN _cursor FOR
			SELECT (saledate+duedays) AS duedate, custid, fBPName(custid) AS vendorname, 
				(total-paidamt) AS hutang, CAST((_date-(saledate+duedays)) AS INT2) AS overdue
			FROM sale 
			WHERE paystatus<>'F' AND (_branchid IS NULL OR branchid=_branchid);
		LOOP
			FETCH _cursor INTO _DueDate, _VendorID, _VendorName, _Hutang, _OverDue;
			EXIT WHEN NOT FOUND;
	
			_retval.duedate:=_DueDate;
			_retval.vendorid:=_VendorID;
			_retval.vendorname:=_VendorName;
			_retval.hutang:=_Hutang;
			_retval.overdue:=_OverDue;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE _cursor;
	END IF;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION fCekUnit(
	_ItemID		VARCHAR,
	_Unit 		VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;

	_Unit1		VARCHAR;
	_Unit2		VARCHAR;
	_Unit3		VARCHAR;
BEGIN
	_retval:=FALSE;

	SELECT INTO _Unit1, _Unit2, _Unit3 unit1, unit2, unit3 FROM item WHERE itemid=_ItemID;
		
	IF (_retval=FALSE AND _Unit=_Unit1) THEN
		_retval:=TRUE;
	END IF;

	IF (_retval=FALSE AND _Unit=_Unit2) THEN
		_retval:=TRUE;
	END IF;

	IF (_retval=FALSE AND _Unit=_Unit3) THEN
		_retval:=TRUE;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCheckSaldo (
	_RefType	VARCHAR,
	_RefNo		VARCHAR
) RETURNS SETOF Decimal(19,4) AS
$BODY$
DECLARE
	_retval		DECIMAL(19,4);
	_temp		DECIMAL(19,4);
	_totPaid	DECIMAL(19,4);
	cPPaid		REFCURSOR;
	_ppaidAmt	DECIMAL(19,4);
	cSPaid		REFCURSOR;
	_spaidAmt	DECIMAL(19,4);
	_AccNo		varchar;
	_bpid		varchar;
	_crcid		varchar;
BEGIN
	_totpaid:=0;
	_temp:=0;

	/*Ngitung jumlah dari pembayaran, dikasih..*/
	IF(_RefType='PURC') THEN
		SELECT INTO _totpaid COALESCE(SUM(ppaidamt*excrate),0) 
		FROM ppaid 
		WHERE refno=_RefNo AND reftype='PURC';

		SELECT INTO _bpid,_crcid vendorid,crcid 
		FROM purc
		WHERE purcno=_RefNo;

		_accno:=fBpAcc(_BPID,_CrcID,'AP');
	ELSIF(_RefType='PRET') THEN
		SELECT INTO _totpaid COALESCE(SUM(ppaidamt*excrate),0) 
		FROM ppaid 
		WHERE refno=_RefNo AND reftype='PRET';

		SELECT INTO _bpid,_crcid vendorid,crcid 
		FROM pret 
		WHERE pretno=_RefNo;

		_accno:=fBpAcc(_BPID,_CrcID,'AP');
	ELSIF(_RefType='SALE') THEN
		SELECT INTO _totpaid COALESCE(SUM(spaidamt*excrate),0) 
		FROM spaid 
		WHERE refno=_RefNo AND reftype='SALE';

		SELECT INTO _bpid,_crcid custid,crcid 
		FROM sale 
		WHERE saleno=_RefNo;

		_accno:=fBpAcc(_BPID,_CrcID,'AR');
	ELSIF(_RefType='SRET') THEN
		SELECT INTO _totpaid COALESCE(SUM(spaidamt*excrate),0) 
		FROM spaid 
		WHERE refno=_RefNo AND reftype='SRET';

		SELECT INTO _bpid,_crcid custid,crcid 
		FROM sret 
		WHERE sretno=_RefNo;

		_accno:=fBpAcc(_BPID,_CrcID,'AR');
	END IF;

	/*Ngitung jumlah dari lgr, yang include ntar pembeliannya/penjualannya, revaluasi(revaluasi harus dibuat tiap transaksi)*/
	SELECT INTO _temp COALESCE(SUM(lgramt),0) 
	FROM lgr 
	JOIN jnl ON lgr.jnlno=jnl.jnlno 
	WHERE refno=_RefNo and AccNo=_AccNo;

	_retval:=_temp-_totPaid;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spRevaluasi(_DateNow DATE)
RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	cPurc		refcursor;
	cSale		refcursor;
	cPRet		refcursor;
	cSRet		refcursor;
	cCrc			refcursor;
	_CrcID		varchar;

	_CurExcRate	numeric(19,4);
	_CurFisRate	numeric(19,4);

	_PastExcRate	numeric(19,4);
	_PerID		varchar;
	_SelisihRate	numeric(19,4);
	_purcNo		varchar;
	_Sisa		numeric(19,4);
	_VendorID	varchar;
	_JnlNo		varchar;
	_JnlNote	varchar;
	_saleNo		varchar;
	_CustID		varchar;
	_PretNo		varchar;
	_SretNo		varchar;
BEGIN

RAISE NOTICE 'spRevaluasi started';

	OPEN cCrc FOR
		SELECT Crcid, Excrate, Fisrate FROM crc WHERE IsDefault=FALSE;

		LOOP
			FETCH cCrc INTO _CrcID, _CurExcRate, _CurFisRate;
			EXIT WHEN NOT FOUND;

RAISE NOTICE 'Processing Currency %', _CrcID;

			-- baca rate sebelumnya ---
			SELECT INTO _PastExcRate excrate FROM bpa WHERE crcid=_CrcID and bpadate<=_DateNow ORDER BY bpadate desc LIMIT 1;

			SELECT INTO _PerID perid FROM per WHERE active=true;
			_SelisihRate=_CurExcRate-_PastExcRate;

			-- NGECEK PEMBELIAN -- 
			OPEN cPurc FOR
				SELECT purcno,total-paidamt,vendorid FROM purc WHERE crcid=_CrcID AND paystatus<>'F' ORDER BY purcno;
				LOOP
					FETCH cPurc INTO _purcNo, _Sisa, _VendorID;
					EXIT WHEN NOT FOUND;

RAISE NOTICE 'Loop purc';

					_JnlNo:=fTransNoGet('JURNAL');
					_JnlNote:= 'Revaluasi Kurs Pembelian'||':'||_PurcNo;
					SELECT INTO _spret * FROM spJnl_Insert(_JnlNo,'REVA',_DateNow,'REVA',_PurcNo,
						_JnlNote,true,true,cast(0 as numeric(19,4)),null,null,null,null,
						_PerID, null, null, null,null, _CrcID, _CurExcRate, _CurFisRate
					);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

					IF(_SelisihRate>0) THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fAccDefault('ACCPROFIT'), 'D', _SelisihRate*_Sisa, null, null, _SelisihRate*_Sisa);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,2, fBpAcc(_VendorID,_CrcID,'AP'), 'K', _SelisihRate*_Sisa, null, null, _SelisihRate*_Sisa);

						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
					ELSIF(_SelisihRate<0) THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, fBpAcc(_VendorID,_CrcID,'AP'), 'D', abs(_SelisihRate)*_Sisa, null, null, abs(_SelisihRate)*_Sisa);

						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fAccDefault('ACCPROFIT'), 'K', abs(_SelisihRate)*_Sisa, null, null, abs(_SelisihRate)*_Sisa);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

					END IF;

					SELECT INTO _spret * FROM spJnl_New(_JnlNo);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END LOOP;
			CLOSE cPurc;


			/*NGECEK PENJUALAN*/
			OPEN cSale FOR
				SELECT saleno,total-paidamt,custid FROM sale WHERE crcid=_CrcID AND paystatus<>'F' order by saleno;
				LOOP
					FETCH cSale INTO _saleNo, _Sisa, _CustID;
					EXIT WHEN NOT FOUND;


RAISE NOTICE 'Loop Sale';

					_JnlNo:=fTransNoGet('JURNAL');
					_JnlNote:= 'Revaluasi Kurs Penjualan'||':'||_SaleNo;
					SELECT INTO _spret * FROM spJnl_Insert(_JnlNo,'REVA',_DateNow,'REVA',_SaleNo,
						_JnlNote,true,true,cast(0 as numeric(19,4)),null,null,null,null,
						_PerID, null, null, null,null, _CrcID, _CurExcRate, _CurFisRate
					);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

					IF(_SelisihRate>0) THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fAccDefault('ACCPROFIT'), 'D', _SelisihRate*_Sisa, null, null, _SelisihRate*_Sisa);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fBpAcc(_CustID,_CrcID,'AR'), 'K', _SelisihRate*_Sisa, null, null, _SelisihRate*_Sisa);

						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
					ELSIF(_SelisihRate<0) THEN
						
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, fAccDefault('ACCPROFIT'), 'D', abs(_SelisihRate)*_Sisa, null, null, abs(_SelisihRate)*_Sisa);

						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2,fBpAcc(_CustID,_CrcID,'AR'), 'K', abs(_SelisihRate)*_Sisa, null, null, abs(_SelisihRate)*_Sisa);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

					END IF;

					SELECT INTO _spret * FROM spJnl_New(_JnlNo);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END LOOP;
			CLOSE cSale;


			/*NGECEK RETUR PEMBELIAN*/
			OPEN cPRet FOR
				SELECT pretno,total,vendorid FROM pret WHERE crcid=_CrcID AND iscontra=false order by pretno;
				LOOP
					FETCH cPRet INTO _PretNo, _Sisa, _VendorID;
					EXIT WHEN NOT FOUND;

					_JnlNo:=fTransNoGet('JURNAL');
					_JnlNote:= 'Revaluasi Kurs Retur'||':'||_PretNo;
					SELECT INTO _spret * FROM spJnl_Insert(_JnlNo,'REVA',_DateNow,'REVA',_PretNo,
						_JnlNote,true,true,cast(0 as numeric(19,4)),null,null,null,null,
						_PerID, null, null, null,null, _CrcID, _CurExcRate, _CurFisRate
					);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

					IF(_SelisihRate>0) THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fBpAcc(_VendorID,_CrcID,'AP'), 'D', _SelisihRate*_Sisa, null, null, _SelisihRate*_Sisa);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,2,fAccDefault('ACCPROFIT'), 'K', _SelisihRate*_Sisa, null, null, _SelisihRate*_Sisa);

						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
					ELSIF(_SelisihRate<0) THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1, fAccDefault('ACCPROFIT'), 'D', abs(_SelisihRate)*_Sisa, null, null, abs(_SelisihRate)*_Sisa);

						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2,fBpAcc(_VendorID,_CrcID,'AP') , 'K', abs(_SelisihRate)*_Sisa, null, null, abs(_SelisihRate)*_Sisa);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

					END IF;

					SELECT INTO _spret * FROM spJnl_New(_JnlNo);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END LOOP;
			CLOSE cPRet;
			

			/*NGECEK RETUR PENJUALAN*/
			OPEN cSRet FOR
				SELECT sretno,total,custid FROM sret WHERE crcid=_CrcID AND iscontra=false order by sretno;
				LOOP
					FETCH cSRet INTO _sretNo, _Sisa, _CustID;
					EXIT WHEN NOT FOUND;

					_JnlNo:=fTransNoGet('JURNAL');
					_JnlNote:= 'Revaluasi Kurs Penjualan'||':'||_SRetNo;
					SELECT INTO _spret * FROM spJnl_Insert(_JnlNo,'REVA',_DateNow,'REVA',_SretNo,
						_JnlNote,true,true,cast(0 as numeric(19,4)),null,null,null,null,
						_PerID, null, null, null,null, _CrcID, _CurExcRate, _CurFisRate
					);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;

					IF(_SelisihRate>0) THEN
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 1, fBpAcc(_CustID,_CrcID,'AR'), 'D', _SelisihRate*_Sisa, null, null, _SelisihRate*_Sisa);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2, fAccDefault('ACCPROFIT'), 'K', _SelisihRate*_Sisa, null, null, _SelisihRate*_Sisa);

						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
					ELSIF(_SelisihRate<0) THEN

						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo,1,fBpAcc(_CustID,_CrcID,'AR') , 'D', abs(_SelisihRate)*_Sisa, null, null, abs(_SelisihRate)*_Sisa);

						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;
						SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, 2,fAccDefault('ACCPROFIT'), 'K', abs(_SelisihRate)*_Sisa, null, null, abs(_SelisihRate)*_Sisa);
						IF _spret.ErrNo<>0 THEN
							_retval.ErrNo:=_spret.ErrNo;
							_retval.ErrMsg:=_spret.ErrMsg;
							RAISE EXCEPTION '%', _retval.ErrMsg;
						END IF;

					END IF;

					SELECT INTO _spret * FROM spJnl_New(_JnlNo);
					IF _spret.ErrNo<>0 THEN
						_retval.ErrNo:=_spret.ErrNo;
						_retval.ErrMsg:=_spret.ErrMsg;
						RAISE EXCEPTION '%', _retval.ErrMsg;
					END IF;
				END LOOP;
			CLOSE cSRet;

		END LOOP;
	CLOSE cCrc;
	
_retval:=(0, 'OK');
RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBPBalanceDate (
	_Mode		INT,
	_BPID		VARCHAR, 
	_CrcID		VARCHAR,
	_Month		INT, 
	__PYear		INT
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval			NUMERIC(19,4);
	_PYear			INT2;
	_YearChanged		BOOLEAN;
	_YearChangedValue	NUMERIC(15,4);

	cBPBal			REFCURSOR;
BEGIN
	_YearChanged:=FALSE;

	IF (_Mode=0) THEN /** HUTANG */
		SELECT INTO _PYear MAX(pyear) FROM bpcbal WHERE bpid=_BPID AND (_CrcID IS NULL OR crcid=_CrcID);
		IF (_PYear!=__PYear AND _PYear<__PYear) THEN
			_YearChanged:=TRUE;
		END IF;
	
		IF (_YearChanged=FALSE) THEN
			_PYear:=__PYear;
		END IF;

		_retval:=0;

		IF (_Month=1) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;	
		ELSIF (_Month=2) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=3) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=4) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=5) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=6) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)+COALESCE(bpcbal05, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=7) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)+COALESCE(bpcbal05, 0)+COALESCE(bpcbal06, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=8) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)+COALESCE(bpcbal05, 0)+COALESCE(bpcbal06, 0)+COALESCE(bpcbal07, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=9) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)+COALESCE(bpcbal05, 0)+COALESCE(bpcbal06, 0)+COALESCE(bpcbal07, 0)+COALESCE(bpcbal08, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=10) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)+COALESCE(bpcbal05, 0)+COALESCE(bpcbal06, 0)+COALESCE(bpcbal07, 0)+COALESCE(bpcbal08, 0)+COALESCE(bpcbal09, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=11) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)+COALESCE(bpcbal05, 0)+COALESCE(bpcbal06, 0)+COALESCE(bpcbal07, 0)+COALESCE(bpcbal08, 0)+COALESCE(bpcbal09, 0)+COALESCE(bpcbal10, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=12) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)+COALESCE(bpcbal05, 0)+COALESCE(bpcbal06, 0)+COALESCE(bpcbal07, 0)+COALESCE(bpcbal08, 0)+COALESCE(bpcbal09, 0)+COALESCE(bpcbal10, 0)+COALESCE(bpcbal11, 0)) AS balance 
				FROM bpcbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		END IF;

		IF (_YearChanged=TRUE) THEN
			SELECT INTO _YearChangedValue SUM(COALESCE(bpcbal00, 0)+COALESCE(bpcbal01, 0)+COALESCE(bpcbal02, 0)+COALESCE(bpcbal03, 0)+COALESCE(bpcbal04, 0)+COALESCE(bpcbal05, 0)+COALESCE(bpcbal06, 0)+COALESCE(bpcbal07, 0)+COALESCE(bpcbal08, 0)+COALESCE(bpcbal09, 0)+COALESCE(bpcbal10, 0)+COALESCE(bpcbal11, 0)+COALESCE(bpcbal12, 0)) AS balance 
			FROM bpcbal 
			WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
		END IF;

		_retval:=COALESCE(_retval, 0)+COALESCE(_YearChangedValue, 0);
	
	ELSIF (_Mode=1) THEN
		SELECT INTO _PYear MAX(pyear) FROM bpdbal WHERE bpid=_BPID AND (_CrcID IS NULL OR crcid=_CrcID);
		IF (_PYear!=__PYear AND _PYear<__PYear) THEN
			_YearChanged:=TRUE;
		END IF;
	
		IF (_YearChanged=FALSE) THEN
			_PYear:=__PYear;
		END IF;

		_retval:=0;

		IF (_Month=1) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;	
		ELSIF (_Month=2) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=3) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=4) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=5) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=6) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)+COALESCE(bpdbal05, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=7) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)+COALESCE(bpdbal05, 0)+COALESCE(bpdbal06, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=8) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)+COALESCE(bpdbal05, 0)+COALESCE(bpdbal06, 0)+COALESCE(bpdbal07, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=9) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)+COALESCE(bpdbal05, 0)+COALESCE(bpdbal06, 0)+COALESCE(bpdbal07, 0)+COALESCE(bpdbal08, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=10) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)+COALESCE(bpdbal05, 0)+COALESCE(bpdbal06, 0)+COALESCE(bpdbal07, 0)+COALESCE(bpdbal08, 0)+COALESCE(bpdbal09, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=11) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)+COALESCE(bpdbal05, 0)+COALESCE(bpdbal06, 0)+COALESCE(bpdbal07, 0)+COALESCE(bpdbal08, 0)+COALESCE(bpdbal09, 0)+COALESCE(bpdbal10, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		ELSIF (_Month=12) THEN
			IF (_YearChanged=FALSE) THEN
				SELECT INTO _retval SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)+COALESCE(bpdbal05, 0)+COALESCE(bpdbal06, 0)+COALESCE(bpdbal07, 0)+COALESCE(bpdbal08, 0)+COALESCE(bpdbal09, 0)+COALESCE(bpdbal10, 0)+COALESCE(bpdbal11, 0)) AS balance 
				FROM bpdbal 
				WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
			END IF;
		END IF;

		IF (_YearChanged=TRUE) THEN
			SELECT INTO _YearChangedValue SUM(COALESCE(bpdbal00, 0)+COALESCE(bpdbal01, 0)+COALESCE(bpdbal02, 0)+COALESCE(bpdbal03, 0)+COALESCE(bpdbal04, 0)+COALESCE(bpdbal05, 0)+COALESCE(bpdbal06, 0)+COALESCE(bpdbal07, 0)+COALESCE(bpdbal08, 0)+COALESCE(bpdbal09, 0)+COALESCE(bpdbal10, 0)+COALESCE(bpdbal11, 0)+COALESCE(bpdbal12, 0)) AS balance 
			FROM bpdbal 
			WHERE bpid=_BPID AND pyear=_PYear AND (_CrcID IS NULL OR crcid=_CrcID);
		END IF;

		_retval:=COALESCE(_retval, 0)+COALESCE(_YearChangedValue, 0);
	END IF;

	RETURN COALESCE(_retval, 0);
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fHutangDate (
	_Tipe		INT,
	_Date		DATE,
	_CrcID		VARCHAR,
	_NotEqualZero	BOOLEAN
) RETURNS SETOF tHutangPiutang AS
$BODY$
DECLARE
	_retval		tHutangPiutang%ROWTYPE;

	_BPID		VARCHAR(15);
	_BPName		VARCHAR(50);
	_Balance	NUMERIC(19,4);
	_CrcName	VARCHAR(50);
	_CrcSymbol	VARCHAR(5);
	_CurrCrcID	VARCHAR;

	__Date		DATE;

	_PYear		INT2;
	_Month		INT2;
	_Day		INT2;
	_StartDate	DATE;

	__Balance	NUMERIC(19,4);

	cBal		REFCURSOR;
BEGIN
	IF (_Tipe=0 OR _Date IS NULL) THEN
		__Date:=CURRENT_DATE;
	ELSE
		__Date:=_Date;
	END IF;

	_PYear:=EXTRACT(year FROM __Date);
	_Month:=EXTRACT(month FROM __Date);
	_Day:=EXTRACT(day FROM __Date);
	_StartDate:=TO_DATE((_PYear||'-'||_Month||'-01'), 'YYYY-MM-DD');
	_Month:=_Month;

	OPEN cBal FOR
		SELECT DISTINCT bp.bpid, bpname, fBPBalanceDate(0, bp.bpid, crc.crcid, _Month, _PYear) AS apbalance, crcname, crcsymbol, crc.crcid  
		FROM bp 
		LEFT JOIN bpacc ON bp.bpid=bpacc.bpid 
		JOIN bpbptype bt ON bt.bpid=bp.bpid
		JOIN crc ON crc.crcid=bpacc.crcid
		WHERE (_CrcID IS NULL OR crc.crcid=_CrcID) AND (bt.bptype='VEND' OR (bt.bptype!='VEND' AND bpacc.apbalance<>0));
	LOOP
		FETCH cBal INTO _BPID, _BPName, _Balance, _CrcName, _CrcSymbol, _CurrCrcID;
		EXIT WHEN NOT FOUND;

		SELECT INTO __Balance SUM(bpaamt) AS balance 
		FROM bpa m 
		WHERE m.bpid=_BPID AND m.crcid=_CurrCrcID AND reftype IN ('PURC', 'PAY', 'PRET') 
		AND m.bpadate>=_StartDate AND m.bpadate<=__Date;

		IF NOT FOUND THEN
			__Balance:=0;
		END IF;

		_Balance:=_Balance+COALESCE(__Balance,0);

		IF ((_NotEqualZero=TRUE AND _Balance<>0) OR (_NotEqualZero=FALSE)) THEN
			_retval.bpid:=_BPID;
			_retval.bpname:=_BPName;
			_retval.balance:=abs(_Balance);
			_retval.crcname:=_CrcName;
			_retval.crcsymbol:=_CrcSymbol;
	
			RETURN NEXT _retval;
		END IF;
	END LOOP;
	CLOSE cBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPiutangDate(
	_Tipe		INT,
	_Date		DATE,
	_CrcID 		VARCHAR,
	_NotEqualZero	BOOLEAN
) RETURNS SETOF tHutangPiutang AS
$BODY$
DECLARE
	_retval		tHutangPiutang%ROWTYPE;

	_BPID		VARCHAR(15); 
	_BPName		VARCHAR(50); 
	_Balance	NUMERIC(19,4); 
	_CrcName	VARCHAR(50); 
	_CrcSymbol	VARCHAR(5);
	_CurrCrcID	VARCHAR;

	__Date		DATE;

	_PYear		INT2;
	_Month		INT2;
	_Day		INT2;
	_StartDate	DATE;
	
	__Balance	NUMERIC(19,4);
	
	cBal		REFCURSOR;
BEGIN
	IF (_Tipe=0 OR _Date IS NULL) THEN
		__Date:=CURRENT_DATE;
	ELSE
		__Date:=_Date;
	END IF;

	_PYear:=EXTRACT(year FROM __Date);
	_Month:=EXTRACT(month FROM __Date);
	_Day:=EXTRACT(day FROM __Date);
	_StartDate:=TO_DATE((_PYear||'-'||_Month||'-01'), 'YYYY-MM-DD');
	_Month:=_Month;

	OPEN cBal FOR
		SELECT DISTINCT bp.bpid, bpname, fBPBalanceDate(1, bp.bpid, crc.crcid, _Month, _PYear) AS arbalance, crcname, crcsymbol, crc.crcid  
		FROM bp LEFT JOIN bpacc ON bp.bpid=bpacc.bpid  JOIN bpbptype bt ON bt.bpid=bp.bpid
		JOIN crc ON crc.crcid=bpacc.crcid
		WHERE (_CrcID IS NULL OR crc.crcid=_CrcID) AND (bt.bptype='CUST' OR (bt.bptype!='CUST' AND bpacc.arbalance<>0));
	LOOP
		FETCH cBal INTO _BPID, _BPName, _Balance, _CrcName, _CrcSymbol, _CurrCrcID;
		EXIT WHEN NOT FOUND;

		SELECT INTO __Balance SUM(bpaamt) AS balance FROM bpa m 
		WHERE m.bpid=_BPID AND m.crcid=_CurrCrcID AND reftype IN ('SALE', 'RCV', 'SRET') 
		AND m.bpadate>=_StartDate AND m.bpadate<=__Date;
		
		IF NOT FOUND THEN
			__Balance:=0;
		END IF;

		_Balance:=_Balance+COALESCE(__Balance,0);

		IF ((_NotEqualZero=TRUE AND _Balance<>0) OR (_NotEqualZero=FALSE)) THEN
			_retval.bpid:=_BPID;
			_retval.bpname:=_BPName;
			_retval.balance:=_Balance;
			_retval.crcname:=_CrcName;
			_retval.crcsymbol:=_CrcSymbol;

			RETURN NEXT _retval;
		END IF;
	END LOOP;
	CLOSE cBal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPossSaldo(
	_PossesNo	VARCHAR
) RETURNS SETOF tPossSaldo AS
$BODY$
DECLARE
	_retval		tPossSaldo%ROWTYPE;
	
	_Cash		NUMERIC(19,4);
	_Debit		NUMERIC(19,4);
	_Credit		NUMERIC(19,4);
	_Voucher	NUMERIC(19,4);
	_Change		NUMERIC(19,4);
	
BEGIN
	SELECT INTO _Cash, _Change, _Debit, _Credit, _Voucher 
		SUM(CASE WHEN (rcvtype='CASH') THEN rcvamt ELSE 0 END) AS Cash,
		SUM(CASE WHEN (rcvtype='CHG') THEN rcvamt ELSE 0 END) AS Change,
		SUM(CASE WHEN (rcvtype='DC') THEN rcvamt ELSE 0 END) AS Debit,
		SUM(CASE WHEN (rcvtype='CC') THEN rcvamt ELSE 0 END) AS Credit,
		SUM(CASE WHEN (rcvtype='VOUC') THEN rcvamt ELSE 0 END) AS Voucher
	FROM salecrcv WHERE saleno IN (SELECT saleno FROM sale WHERE possesno=_PossesNo);
	IF NOT FOUND THEN
		_Cash:=0;
		_Change:=0;
		_Debit:=0;
		_Credit:=0;
		_Voucher:=0;
	END IF;
	_retval.cash:=(_Cash-_Change);
	_retval.debit:=_Debit;
	_retval.credit:=_Credit;
	_retval.voucher:=_Voucher;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPossUseCRcv (
	_PossesNo	VARCHAR
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_retval		BOOLEAN;
	_Counter	INT2;
BEGIN
	SELECT INTO _Counter COUNT(*)
	FROM salecrcv WHERE saleno IN (SELECT saleno FROM sale WHERE possesno=_PossesNo);
	IF NOT FOUND THEN
		_Counter:=0;
	END IF;

	IF (_Counter IS NULL OR _Counter=0) THEN
		_retval:=FALSE;
	ELSE
		_retval:=TRUE;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCashA_Reconcile (
	__CashANo		VARCHAR,
	_isReconciled		BOOLEAN,
	_ReconciledBy		VARCHAR,
	__ReconcileNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;

	_CashID			VARCHAR;
	_CashADate		DATE;
	_Amount			NUMERIC(19,4);

	_TotalAsli		NUMERIC(19,4);
	_Total			NUMERIC(19,4);

	arrayCashA		VARCHAR[];
	_CashANo		VARCHAR;
	_arrayIndex		INT2;

	arrayReconNo		VARCHAR[];
	_ReconcileNo		VARCHAR;
	_arrayIndex2		INT2;


	_AccCash		VARCHAR(10);
	_AccUnreconciled	VARCHAR(10);
	_AccNo			VARCHAR(10);
	_Count			INT;
	_JnlNo			VARCHAR(15);
	_JnlNote		VARCHAR;
	_isAutoGen		BOOLEAN;
	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID			VARCHAR(5);
	_PrjID			VARCHAR(5);
	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;
	_RefNo			VARCHAR;
	_RefType		VARCHAR;
	_CrcID			VARCHAR;
	_ExcRate		NUMERIC(19,4);
	_FisRate		NUMERIC(19,4);

	_InOut			VARCHAR(1);
	_DK_CashBank		VARCHAR(1);
	_DK_Unrec		VARCHAR(1);
BEGIN
	/** inisialisasi variabel */
	_CashANo:=NULL;
	_Amount:=0;
	_Total:=0;
	_TotalAsli:=0;
	_CrtBy:=_ReconciledBy;
	_CrtDate:=CURRENT_DATE;
	_UpdBy:=_ReconciledBy;
	_UpdDate:=CURRENT_DATE;
	_RefNo:=NULL;

	arrayCashA:=STRING_TO_ARRAY(__CashANo, ',');
	_arrayIndex:=1;	

	arrayReconNo:=STRING_TO_ARRAY(__ReconcileNo, ',');
	_arrayIndex2:=1;

	LOOP
		_CashANo:=arrayCashA[_arrayIndex];
		_ReconcileNo:=arrayReconNo[_arrayIndex2];
		EXIT WHEN _CashANo IS NULL;

		SELECT INTO _CashID, _CashADate, _Amount, _ExcRate, _CrcID, _FisRate, _RefNo, _RefType 
			cashid, cashadate, amount, excRate, crcID, fisrate, refno, reftype  
		FROM casha WHERE cashano=_CashANo;

RAISE NOTICE 'Processing CashA Reconcile % : % : %', _CashANo, _Amount, _ExcRate;

		IF (_isReconciled=TRUE) THEN
			SELECT INTO _spret * FROM spCashBal_Update(_CashID, _CashADate, _Amount);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_TotalAsli:=_TotalAsli+_Amount;
			_Total:=_Total+(_Amount*_ExcRate);
		ELSE
			SELECT INTO _spret * FROM spCashBal_Update(_CashID, _CashADate, _Amount*(-1));
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_TotalAsli:=_TotalAsli+_Amount;
			_Total:=_Total+(_Amount*_ExcRate);
			
		END IF;

		_RefNo:=COALESCE(_RefNo, _CashANo);
		_RefType:=COALESCE(_RefType, '');

		UPDATE CashA SET isreconciled=_isReconciled,reconcileby=_UpdBy,reconciledate=_UpdDate WHERE cashano=_CashANo;

		_arrayIndex:=_arrayIndex+1;
		_arrayIndex2:=_arrayIndex2+1;
	END LOOP;

/** //////////////////////////// AKUNTANSI //////////////////////////////////// **/
	IF isAccInstalled() THEN
		_AccUnreconciled:=fAccDefault('UNRECBANK');
		IF (_AccUnreconciled IS NULL OR _AccUnreconciled='') THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Akun Default Untuk unreconciled bank belum diset (spCashA_Reconcile)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF (_isReconciled=TRUE) THEN
			_arrayIndex:=1;	
			LOOP
				_CashANo:=arrayCashA[_arrayIndex];
				EXIT WHEN _CashANo IS NULL;

				SELECT INTO 
					_CashID, _Amount, _DeptID, _PrjID, _ExcRate, _InOut, _RefNo, _RefType, _PerID, _BranchID, _CrcID 
					cashid, amount, deptid, prjid, ExcRate, InOut, RefNo, RefType, PerID, BranchID, CrcID  
				FROM CashA WHERE cashano=_CashANo;
				_AccCash:=fAccCash(_CashID);

				_RefNo:=COALESCE(_RefNo, _CashANo);
				_RefType:=COALESCE(_RefType, '<kosong>');
		
				_Count:=0;
				_JnlNo:=fTransNoGet('JURNAL');
				_JnlNote:= 'Rekonsiliasi Bank '||':'||CURRENT_DATE;
				SELECT INTO _spret * FROM spJnl_Insert(
					_JnlNo, _RefType, CURRENT_DATE, _RefType, _RefNo,
					_JnlNote,true,true,cast(0 AS NUMERIC(19,4)),
					_CrtBy,_CrtDate,_UpdBy,_UpdDate,
					_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				_Count:=_Count+1;
				IF (_InOut='I') THEN
					_DK_CashBank:='D';
					_DK_Unrec:='K';
				ELSE
					_DK_CashBank:='K';
					_DK_Unrec:='D';
				END IF;
RAISE NOTICE 'Processing Jurnal Reconcile % : % : %', _CashANo, _Amount, _ExcRate;
					--- Ayat Cash/Bank ----
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccCash, _DK_CashBank, ABS(_Amount)*_ExcRate, _DeptID, _PrjID, ABS(_Amount));
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				--- Ayat Unreconciled-Bank  
				_Count:=_Count+1;
				SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _Count, _AccUnreconciled, _DK_Unrec, ABS(_Amount)*_ExcRate, _DeptID, _PrjID, ABS(_Amount));
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;

				_arrayIndex:=_arrayIndex+1;

				SELECT INTO _spret * FROM spJnl_New(_JnlNo);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF;
			END LOOP;
		ELSE
			-- Void Jurnal Reconcile yang ada

		END IF;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spCashA_Reconcile (
	__CashANo	VARCHAR,
	_isReconciled	BOOLEAN,
	_ReconciledBy	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
BEGIN
	/** Inisialisasi */

	SELECT INTO _spret * FROM spCashA_Reconcile(__CashANo, _isReconciled, _ReconciledBy, NULL);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



/** untuk posting Transaksi CashA yang di-ENTRY USER dari program 
	 i.e.: untuk penyesuian-kas
*/

CREATE OR REPLACE FUNCTION spCAdj_New (
	_CAdjNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_CAdjDate	DATE;
	_Amount		NUMERIC(19,4);
	_cashid		varchar;
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_CrtBy		VARCHAR(10);
	_CrtDate	DATE;
	_UpdBy		VARCHAR(10);
	_UpdDate	DATE;
	_Count		INT;
	_JnlNo		VARCHAR(15);
	_JnlNote	VARCHAR(255);
	_BranchID	VARCHAR(10);
	_ExcRate	NUMERIC(19,4);
	_JnlType	VARCHAR;
	_CAdjAccNo	varchar;
	_LockDate	DATE;
	_BPID		VARCHAR;
	_InOut		varchar;
	_ExtNote	varchar;
	_CrcID		VARCHAR;
	_FisRate	numeric(19,4);
	_CAdjType	varchar;
	_AccNo		varchar;
	_PerID		VARCHAR;
	_DeptID		varchar;
	_PrjID		VARCHAR;
	_IsBeg		BOOLEAN;

	_CAdjTypeDesc	VARCHAR;
	_CAdjNote	VARCHAR;
	_isDraft	BOOL;
BEGIN

	_IsBeg:=FALSE;

		SELECT INTO 
			_CashID, _CAdjDate, _Amount, _CAdjNote, _CrtBy,_CrtDate,_UpdBy,_UpdDate,_ExcRate,_BranchID, _CAdjAccNo, _BPID, _CrcID, _FisRate, _CAdjType, _PerID, _DeptID, _PrjID, _CAdjNote, _isDraft
			cashid, CAdjDate, cadjAmt, CAdjNote,CrtBy, CrtDate, UpdBy, UpdDate,excrate,BranchID,CAdjAccNo, BPID, CrcID, FisRate, CAdjType, PerID, DeptID, PrjID, CAdjNote, isDraft
		FROM CAdj WHERE CAdjNo=_CAdjNo;

		IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
			_retval:=(0, 'OK');
			RETURN NEXT _retval;
			RETURN;
		END IF;

		SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CADJ';
		IF(_LockDate IS NOT NULL AND _LockDate>_CAdjDate) THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg='Penyesuaian kas melebihi batas tanggal kunci transaksi ! (spCAdj_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		IF(_Amount>0) THEN
			_InOut:='I';
		ELSIF(_Amount<0) THEN
			_InOut:='O';
		END IF;

		SELECT INTO _CAdjTypeDesc CAdjTypeDesc FROM CAdjType WHERE CAdjType=_CAdjType;
		_ExtNote=_CAdjTypeDesc|| ' ' ||  COALESCE(_CAdjNote, '');


		SELECT INTO _spret * FROM spCashA_New(
				_CashID, _BPID, _CAdjType, _CadjDate, _InOut, 'CADJ', _CAdjNo, _Amount,
				_ExtNote, false, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				NULL, --PerID
				_BranchID, _DeptID, _PrjID, 
				TRUE, 
				_CrcID, _ExcRate, _FisRate
			);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;


/***********************  AKUNTANSI *****************************************/

	IF isAccInstalled() THEN


			IF(_CAdjType='BEG') THEN
				_JnlType:='BEG';
				_JnlNote:='Saldo Awal Kas/Bank '||':'||_CAdjNo;
				_AccNo:=fAccDefault('BEGBAL');
				_IsBeg:=TRUE;
			ELSE
				_JnlType:='CADJ';
				_AccNo:=_CAdjAccNo;
				IF (_CAdjNote IS NULL) THEN
					_JnlNote:='Jurnal Penyesuaian Kas '||_CAdjNo;
				ELSE
					_JnlNote:=_CAdjNote;
				END IF;
			END IF;
			
			
			_JnlNo:=fTransNoGet('JURNAL');
			SELECT INTO _spret * FROM spJnl_Insert(
				_JnlNo, _JnlType, _CAdjDate, 'CADJ', _CAdjNo,
				_JnlNote, true, true, _Amount*_ExcRate, 
				_CrtBy,_CrtDate,_UpdBy,_UpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
				);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			
			_Count:=1;
			IF (_Amount<0) THEN
					SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccNo, 'D', ABS(_Amount*_ExcRate), _DeptID, _PrjID, _Amount);
					_Count:=_Count+1;
					SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, fAccCash(_CashID, _InOut,_IsBeg), 'K', ABS(_Amount*_ExcRate), _DeptID, _PrjID, _Amount);
			ELSE
					SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, fAccCash(_CashID, _InOut,_IsBeg), 'D', _Amount*_ExcRate, _DeptID, _PrjID, _Amount);
					_Count:=_Count+1;
					SELECT INTO _retval * FROM spJnlD_Insert(_JnlNo,_Count, _AccNo, 'K', _Amount*_ExcRate, _DeptID, _PrjID, _Amount);

			END IF;

			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

	END IF; -- isAccInstalled()
		
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spCAdj_Void(
	_CAdjNo		VARCHAR(15)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_spretstockad_void	SPStatus%ROWTYPE;
	_CashID			VARCHAR(15);
	_CAdjDate		DATE;
	_LockDate		DATE;
	_isDraft		BOOLEAN;
BEGIN
	_retval:=(50001, 'GENERAL ERROR (spCashA_Void)');

	SELECT INTO _CashID, _CAdjDate, _isDraft cashID, CAdjDate, isDraft FROM CAdj WHERE CAdjNo=_CAdjNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg:='CAdjNo ' || _CAdjNo || ' not found (spCAdj_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_isDraft IS NOT NULL AND _isDraft=TRUE) THEN
		DELETE FROM CAdj where CAdjNo=_CAdjNo;

		_retval:=(0, 'OK');
		RETURN NEXT _retval;
		RETURN;
	END IF;

	SELECT INTO _LockDate lockdate FROM TLock WHERE TransType='CADJ';
	IF(_LockDate is not null AND _LockDate>_CAdjDate) THEN
		_retval.ErrNo:=50002;
		_retval.ErrMsg='Penyesuaian Kas Melebihi Batas Tanggal Kunci Transaksi !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spCashA_Void_byRef('CADJ', _CAdjNo, _CashID, null);
	IF _spret.ErrNo	<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF isAccInstalled() THEN
		SELECT into _spretstockad_void * from spJnl_Void_byRef('CADJ',_CAdjNo);
		IF _spretstockad_void.ErrNo<>0 THEN
			_retval.ErrNo:=_spretstockad_void.ErrNo;
			_retval.ErrMsg:=_spretstockad_void.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
	/*--		EXIT;*/
		END IF;
	END IF;

	DELETE FROM CAdj where CAdjNo=_CAdjNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spAcc_Remove (
	_AccNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_jnlNo		VARCHAR(15);
BEGIN
	SELECT into _jnlNo jnlno
	FROM lgr WHERE accNo=_AccNo;
	IF FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Acc#'||COALESCE(_AccNo, '<kosong>')||' sudah dipakai dalam transaksi (spAcc_Remove) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _JnlNo accno FROM acc WHERE upaccno=_AccNo;
	IF FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Acc#'||COALESCE(_AccNo, '<kosong>')||' direferensikan akun lain sebagai header (spAcc_Remove) !';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error DELETE accbal (spAcc_Remove)';
	DELETE FROM accbal WHERE accno=_accno;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error DELETE begbal (spAcc_Remove)';
	DELETE FROM begbal WHERE accno=_accno;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE acc (spAcc_Remove)';
	DELETE FROM acc WHERE accno=_accno;

	_retval:=(0, 'DELETE SUKSES');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fJurnalD_byRef (
	__RefType		VARCHAR(5),
	__RefNo		VARCHAR(15)
) RETURNS SETOF TJurnalD AS
$BODY$
DECLARE
	_retval		TJurnalD%ROWTYPE;
	_cur		REFCURSOR;
BEGIN
	OPEN _cur FOR
		SELECT d.JnlNo,d.JnlDNo,d.AccNo, a.AccName, d.DK, d.Debit, d.Credit, d.JnlDAmt, d.JnlDAmtForex
		FROM JnlD d 
		JOIN Jnl h ON h.JnlNo=d.JnlNo 
		JOIN Acc a ON d.AccNo=a.AccNo
		WHERE (__RefType IS NULL OR h.RefType=__RefType) AND h.RefNo=__RefNo
		ORDER BY JnlNo, JnlDNo;
	LOOP
		FETCH _cur INTO _retval.JnlNo, _retval.JnlDNo, _retval.AccNo, _retval.AccName, _retval.DK, _retval.Debit, _retval.Credit, _retval.JnlDAmt, _retval.JnlDAmtForex;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE _cur;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStockAD_byRef (
	__RefType	VARCHAR(5),
	__RefNo		VARCHAR(15)
) RETURNS SETOF TStockAD AS
$BODY$
DECLARE
	_rv		TStockAD%ROWTYPE;
	_cur		REFCURSOR;
BEGIN
	OPEN _cur FOR
		SELECT 
			d.StockANo, d.StockADNo, d.ItemID, i.ItemDesc, d.Qty, d.Unit, d.Cost, d.SubTotal
		FROM 
			StockAD d JOIN StockA h ON h.StockANo=d.StockANo
			JOIN Item i ON d.ItemID=i.ItemID
		WHERE (__RefType IS NULL OR h.RefType=__RefType) AND h.RefNo=__RefNo
		ORDER BY StockANo, StockADNo;

	LOOP
		FETCH _cur INTO _rv.StockANo, _rv.StockADNo, _rv.ItemID, _rv.ItemDesc, _rv.Qty, _rv.Unit, _rv.Cost, _rv.SubTotal;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _rv;
	END LOOP;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStrYearMonth (
	_Date	DATE
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval	VARCHAR;
BEGIN
	_retval:=SUBSTR(CAST(_Date AS VARCHAR), 1,7);
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fStrYearMonth (
	_Date	TIMESTAMP
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval	VARCHAR;
BEGIN
	_retval:=SUBSTR(CAST(_Date AS VARCHAR), 1,7);
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fStrYearMonth (
	_Date	TIMESTAMP WITH TIME ZONE
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_retval	VARCHAR;
BEGIN
	_retval:=SUBSTR(CAST(_Date AS VARCHAR), 1,7);
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE VIEW vwStockAD_H AS
	SELECT h.stockadate, h.reftype, h.refno, d.*
	--h.StockANo, d.ItemID, d.WhID, d.Qty, d.PID, d.Cost
	FROM stockad d JOIN stocka h ON d.stockano=h.stockano;




CREATE OR REPLACE FUNCTION fSale_ItemCost(
	__ItemID	VARCHAR,
	__WhID		VARCHAR,
	__PID		VARCHAR,
	__SaleNo	VARCHAR,	-- optional
	__SaleDNo	INT		-- optional
) RETURNS DECIMAL(19,4) AS
$BODY$
DECLARE
	cur1					REFCURSOR;
	_retval					DECIMAL(19,4);
	_PID					VARCHAR;
	_force_try_to_match_any_best_cost	BOOLEAN;
BEGIN
	_force_try_to_match_any_best_cost:=TRUE;	-- default=FALSE, Ubah bila perlu

	_retval:=NULL;
	_PID:=__PID;

/**
	IF (_PID IS NULL) THEN 
		_PID:='';
	END IF;
**/

	IF (__ItemID IS NULL) THEN 
		RETURN NULL;
	END IF;

	IF (__SaleNo IS NOT NULL) THEN
		IF (__SaleDNo IS NOT NULL) THEN 
			SELECT INTO _retval Cost FROM SaleD WHERE SaleNo=__SaleNo AND SaleDNo=__SaleDNo;
		ELSE
			SELECT INTO _retval Cost FROM SaleD WHERE SaleNo=__SaleNo AND ItemID=__ItemID AND WhID=__WhID AND PID=_PID;
		END IF;
	ELSE
		IF (__PID IS NOT NULL) THEN
			-- Baca dari SaleD yang terakhir
			SELECT INTO 
				_retval Cost 
			FROM SaleD d
			JOIN Sale h ON h.SaleNo=d.SaleNo
			WHERE 
				d.ItemID=__ItemID 
				AND (__WhID IS NULL OR d.WhID=__WhID)
				AND d.PID=_PID
			ORDER BY h.SaleDate DESC, h.SaleNo DESC LIMIT 1;

			IF (_retval=0) THEN
				SELECT INTO 
					_retval Cost 
				FROM SaleD d
				JOIN Sale h ON h.SaleNo=d.SaleNo
				WHERE 
					d.ItemID=__ItemID 
					AND (__WhID IS NULL OR d.WhID=__WhID)
					AND d.PID=_PID
					AND d.Cost>0
				ORDER BY h.SaleDate DESC, h.SaleNo DESC LIMIT 1;
			END IF;
		ELSE
			SELECT INTO 
				_retval Cost 
			FROM SaleD d
			JOIN Sale h ON h.SaleNo=d.SaleNo
			WHERE 
				d.ItemID=__ItemID 
				AND (__WhID IS NULL OR d.WhID=__WhID)
				-- sembarang PID 
			ORDER BY h.SaleDate DESC, h.SaleNo DESC LIMIT 1;
		END IF;
	END IF;

	-- Misal SRetD ter-set SaleNo, tapi mungkin SaleDNo gak diisi (bug prg lama), atau
	-- SaleNo-nya udah di utik2 (bug prg jg blm di lock)
	-- dan kita mesti ngeyel set HPP Returnya, maka dicariin berdasarkan PID aja
	IF ((_retval IS NULL OR _retval=0) AND (_force_try_to_match_any_best_cost=TRUE)) THEN
		RAISE NOTICE '		RetVal=%, Smart searching Cost now', _retval;

		IF (_PID IS NOT NULL) THEN			-- codeblock dibawah ini ngopi dari block diatas
			SELECT INTO 
				_retval Cost 
			FROM SaleD d
			JOIN Sale h ON h.SaleNo=d.SaleNo
			WHERE 
				d.ItemID=__ItemID 
				AND (__WhID IS NULL OR d.WhID=__WhID)
				AND d.PID=_PID
			ORDER BY h.SaleDate DESC, h.SaleNo DESC LIMIT 1;

			IF (_retval=0) THEN
				SELECT INTO 
					_retval Cost 
				FROM SaleD d
				JOIN Sale h ON h.SaleNo=d.SaleNo
				WHERE 
					d.ItemID=__ItemID 
					AND (__WhID IS NULL OR d.WhID=__WhID)
					AND d.PID=_PID
					AND d.Cost>0
				ORDER BY h.SaleDate DESC, h.SaleNo DESC LIMIT 1;
			END IF;
		END IF;
	END IF;


	IF FOUND THEN
		RETURN _retval;
	ELSE
		RETURN NULL;
	END IF;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


/******************** OVERLOAD *******************/

CREATE OR REPLACE FUNCTION fSale_ItemCost(
	_ItemID		VARCHAR,
	_SaleNo		VARCHAR,
	_SaleDNo	INT
) RETURNS DECIMAL(19,4) AS 
$BODY$
DECLARE
	_Cost	DECIMAL(19,4);
	_retval	DECIMAL(19,4);
BEGIN	
	SELECT INTO 
		_Cost
		Cost
	FROM SaleD
	WHERE 
		SaleNo=_SaleNo 
		AND SaleDNo=_SaleDNo
		AND ItemID=_ItemID;

	IF NOT FOUND THEN
		_retval:=NULL;
	ELSE
		_retval:=_Cost;
	END IF;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION isItemLastTrans (
	__ItemID		VARCHAR(20),
	__WhId			VARCHAR,
	__RefType		VARCHAR(5),
	__RefNo			VARCHAR(15)
) RETURNS BOOLEAN AS
$BODY$
DECLARE
	_StockANo		VARCHAR(15);
	_StockADate		DATE;

	_NextStockANo		VARCHAR(15);
	_NextStockADate		DATE;
	_NextRefType		VARCHAR(5);
	_NextRefNo		VARCHAR(15);

	cur1			REFCURSOR;
	_retval			BOOLEAN;
	_found			BOOLEAN;
BEGIN

	SELECT INTO 
		_StockANo, _StockADate
		StockANo, StockADate
	FROM StockA
	WHERE RefType=__RefType AND RefNo=__RefNo;


	IF NOT FOUND THEN
		RETURN TRUE;
	END IF;

	SELECT INTO 
		_NextStockANo, _NextStockADate, _NextRefType, _NextRefNo
		h.StockANo, h.StockADate, h.RefType, h.RefNo
	FROM StockAD d
	JOIN StockA h ON h.StockANo=d.StockANo
	WHERE ItemID=__ItemID AND d.whid=__WhId
	AND 
	(
		(h.StockADate>_StockADate)
		OR ((h.StockADate>=_StockADate) AND (h.StockANo>_StockANo))
	);

	_found:=FOUND;

	IF _found THEN
		RAISE NOTICE 'Maaf sudah ada transaksi baru %/% Tgl:% (isItemLastTrans)', _NextRefType, _NextRefNo, _NextStockADate;
	END IF;


	_retval:=NOT(_found);
	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



/**
	SP ini untuk mencheck apakah suatu akun sudah digunakan di Jurnal, GL,
	dan atau di pointed oleh defa, bp, item, cash ?	
	
	bisa digunakan utk mencegah perubahan data master akun di UI

	RETURNS: BOOLEAN
		TRUE  : akun sudah terpakai, jangan boleh diedit!
		FALSE : akun masih perawan ting-ting hehehe


	pseudo:
		1. cek exist lgr
		2. cek exist di jnld
		3. cek saldo di acc:accbal
		-- reserved steps (not yet implemented or .... actually not needed ?
		4. cek accbal disemua tahun & bulan, tidak boleh ada bal disana
**/


CREATE OR REPLACE FUNCTION isAccUsedInTrans (
	__AccNo		VARCHAR
) RETURNS BOOLEAN AS 
$BODY$
DECLARE
	_count		INT;
	_accbal		DECIMAL(19,4);
	_retval		BOOLEAN;
BEGIN
	_retval:=FALSE;

	-- 1. cek exist lgr
	IF NOT(_retval) THEN
		SELECT INTO _count COUNT(*) FROM lgr WHERE AccNo=__AccNo;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
		END IF;
	END IF;
	
	-- 2. cek exist di jnld
	IF NOT(_retval) THEN	-- jika sudah TRUE maka block ini akan di bypass, langsung ke return block
		SELECT INTO _count COUNT(*) FROM jnld WHERE AccNo=__AccNo;
		IF (FOUND AND _count>0) THEN
			_retval:=TRUE;
		END IF;
	END IF;

	-- 3. cek saldo di acc:accbal
	IF NOT(_retval) THEN -- jika sudah TRUE maka block ini akan di bypass, langsung ke return block
		SELECT INTO _accbal accbal FROM acc WHERE AccNo=__AccNo;
		IF (FOUND AND _accbal>0) THEN
			_retval:=TRUE;
		END IF;
	END IF;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fStdCost(
	__ItemID	VARCHAR,
	__PID		VARCHAR,
	__Qty		DECIMAL(15,4),
	__Unit		VARCHAR
) RETURNS DECIMAL(19,4) AS
$BODY$
DECLARE
	_retval		DECIMAL(19,4);
	_BoMID		VARCHAR;

	_bom_qty	DECIMAL(15,4);
	_bom_unit	VARCHAR;
	_bom_conv	DECIMAL(15,4);
	_bom_total	DECIMAL(19,4);

	_conv_bom	DECIMAL(15,4);
	_conv_now	DECIMAL(15,4);

	_bom_totalx	DECIMAL(19,4);
BEGIN
	SELECT INTO
		_BoMID, _bom_qty, _bom_unit
		BoMID, Qty, Unit
	FROM BoM 
	WHERE ItemID=__ItemID AND active=TRUE;	-- harusnya di cek versi BOM yg terbaru
	IF ((NOT FOUND) OR (_BoMID IS NULL)) THEN
		RETURN 0;
	END IF;

	-- get BoM TotStdCost (bisa ambil BoM:TotStdCost saja?)
	SELECT INTO
		_bom_total
		SUM(Qty*COALESCE(StdCost, 0))
	FROM BoMD 
	WHERE BoMID=_BoMID;

	/** Dimatikan dulu karena stdcost diambil nilai per item dan di ambil dari bom saja sebagai patokan cost
	_bom_totalx:=_bom_total*(__Qty*fItemUnit_Conv(__ItemID, __Unit)) / (_bom_qty*fItemUnit_Conv(__ItemID, _bom_unit));
	_retval:=_bom_totalx;**/

	_retval:=_bom_total;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fCashCrc (
	__CashID		VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;
BEGIN
	SELECT INTO 
		_retval
		CrcID
	FROM Cash
	WHERE CashID=__CashID;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fQtyStock (
	_itemid		VARCHAR
) RETURNS NUMERIC(15,4) AS
$BODY$
/*-- _ItemID = item id*/
/*-- _Qty1   = banyak item pada unit 1*/
DECLARE
	_qty		NUMERIC(15,4);
	_whid 		VARCHAR(3);
BEGIN
	BEGIN
		SELECT INTO _whid * FROM fReg('WHSALE');
		SELECT INTO _qty SUM(s.qty) FROM stock s WHERE s.itemid=_itemid AND s.whid=_whid;
	END;

	RETURN _qty;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPer_Active()
RETURNS DATE AS
$BODY$
DECLARE
	_perStart DATE;
BEGIN
	SELECT INTO _perStart perstart FROM per WHERE active='t';
	IF NOT FOUND THEN
		RAISE EXCEPTION 'Tidak ada Periode Aktif !!';
	END IF;

	RETURN _perStart;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPer_Check (
	_TransDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_perStart	DATE;
	_retval		SPStatus%ROWTYPE;
BEGIN
	SELECT INTO _perStart fPer_Active();
	IF NOT FOUND THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Tidak ada Periode Aktif !!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 

	IF _TransDate < _perStart THEN
		_retval.ErrNo:=2;
		_retval.ErrMsg:='Tanggal Transaksi diluar Periode Aktif !!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPer_Open (
	_StartDate	DATE,
	_EndDate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_PYear		INT;
	_PerID		VARCHAR;
	_PerNo		INT;
BEGIN
	UPDATE per SET active='f';
	_PYear := date_part('year', _StartDate);
	_PerID := fTransNoGet('PER');

	SELECT INTO _PerNo COALESCE(max(perno),0)+1 FROM per;

	INSERT INTO per(perid,perdesc,perno,perstart,perend,active) 
	VALUES(_PerID,_PYear,1,_StartDate,_EndDate,true);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spTutupTahun()
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_YearStart	INT;
	_YearEnd	INT;
	_perStart	DATE;
	_perEnd		DATE;
	_StartDate	DATE;
	_EndDate	DATE;
BEGIN
	SELECT INTO _perStart, _perEnd perstart, perend FROM per WHERE active='t';
	IF NOT FOUND THEN
		_retval.ErrNo:=1;
		_retval.ErrMsg:='Tidak ada Periode Aktif !!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 

	_YearStart:=date_part('year',_perStart)+1;
	_YearEnd:=date_part('year',_perEnd)+1;

	_StartDate:=_YearStart||'-'||date_part('month',_perStart)||'-'||date_part('day',_perStart);
	_EndDate:=_YearEnd||'-'||date_part('month',_perEnd)||'-'||date_part('day',_perEnd);

	SELECT INTO _spret * FROM spPer_Open(_StartDate, _EndDate);
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
	END IF;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fFloorDate (
	_dateIn		DATE
) RETURNS DATE AS
$BODY$
DECLARE
	_retval		DATE;
	_Month		VARCHAR;
	_Year		VARCHAR;
BEGIN

	SELECT INTO _Month EXTRACT(MONTH FROM _dateIN);
	SELECT INTO _Year EXTRACT(YEAR FROM _dateIN);
	SELECT INTO _retval CAST((_Year||'-'||_Month||'-01') AS DATE);

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spLoginSRep (
	_SRepID		VARCHAR,
	_Password	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_found		VARCHAR;
BEGIN
	_retval.ErrNo:=1;
	_retval.ErrMsg:='Login Failed !';

	SELECT INTO _found srepid FROM srep WHERE srepid=_SRepID AND passwd=_Password;
	IF FOUND THEN
		_retval:=(0, 'OK');
	END IF;

	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spsetpasswordsrep(_srepid varchar, _pass varchar)
  RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval spStatus%ROWTYPE;
BEGIN
	_retval := (1, 'fail to set password for user ' || _srepid);
	UPDATE srep SET passwd = md5(_pass) WHERE srepid=_srepid;
	_retval := (0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spPrcLvl_CreateAll() 
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_count		INT;
	cItem		REFCURSOR;
	cPrcLvl		REFCURSOR;
	_ItemID		VARCHAR(20);
	_CrcID		VARCHAR(5);
BEGIN
	_count:=0;

	_CrcID := fCrcDefault();

	OPEN cItem FOR
		SELECT itemid 
		FROM item
		WHERE IsSale=true AND (_xt!=true OR (itemid<>'0' AND itemid<>'-1'));
	LOOP
		FETCH cItem INTO _ItemID;
		EXIT WHEN NOT FOUND;

		INSERT INTO prc (itemid,prclvlid,crcid) SELECT _ItemID, prclvlid, _CrcID FROM prclvl WHERE prclvlid NOT IN (SELECT prclvlid FROM prc WHERE itemid=_ItemID);
		
	END LOOP;
	CLOSE cItem;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION spPrcLvl_CreateAll(
	_bujukan	VARCHAR(20)
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_spret		SPStatus%ROWTYPE;
	_retval		SPStatus%ROWTYPE;
BEGIN

	SELECT INTO _spret * FROM spPrcLvl_CreateAll();
	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=_spret;
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSODO_Stock(
	_StartDate 		DATE, 
	_EndDate 		DATE,
	_CustID			VARCHAR(20),
	_SRepID			VARCHAR(20),
	_StartReqDate		DATE,
	_EndReqDate		DATE,
	_SOStatus		VARCHAR(2),
	_BranchID		VARCHAR(5)
) RETURNS SETOF sodo_stock AS
$BODY$
DECLARE
	_retval			sodo_stock%ROWTYPE;
	_ItemID			VARCHAR(20);
	_ItemDesc		VARCHAR(50);
	_QtySO			DECIMAL(19,4);
	_QtySODesc		VARCHAR;
	_QtyStock		VARCHAR;

	cItem			REFCURSOR;
BEGIN
	OPEN cItem FOR
		SELECT itemid, itemdesc 
		FROM item 
		WHERE active='t' AND isstock='t' AND itemid IN (
			SELECT distinct(itemid) 
			FROM sod sd, so s 
			WHERE s.sono=sd.sono AND 
				(_StartDate IS NULL OR s.sodate>=_StartDate) AND 
				(_EndDate IS NULL OR s.sodate<=_EndDate) AND 
				(_CustID IS NULL OR s.custid=_CustID) AND 
				(_SRepID IS NULL OR s.srepid=_SRepID) AND 
				(_StartReqDate IS NULL OR sd.reqdate>=_StartReqDate) AND 
				(_EndReqDate IS NULL OR sd.reqdate<=_EndReqDate) AND 
				(fQtyZToQty1(sd.itemid, sd.qty, sd.unit)-sd.qtydelivered)>0 AND 
				s.active=TRUE AND 
				(_SOStatus IS NULL OR (CASE WHEN _SOStatus='UF' THEN sostatus<>'F' ELSE sostatus=_SOStatus END)) AND 
				(_BranchID IS NULL OR s.branchid=_BranchID))
		ORDER BY itemid;
	LOOP
		FETCH cItem INTO _ItemID, _ItemDesc;
		EXIT WHEN NOT FOUND;

		SELECT INTO _QtySO COALESCE(SUM(fQtyZToQty1(sd.itemid, sd.qty, sd.unit)-sd.qtydelivered),0) 
		FROM so s, sod sd 
		WHERE s.sono=sd.sono 
		AND s.sodate>=_StartDate AND s.sodate<=_EndDate 
		AND s.active=TRUE 
		AND s.isdraft=false 
		AND sd.itemid=_ItemID 
		AND (_CustID IS NULL OR s.custid=_CustID) 
		AND (_SRepID IS NULL OR s.srepid=_SRepID) 
		AND (_StartReqDate IS NULL OR sd.reqdate>=_StartReqDate)
		AND (_EndReqDate IS NULL OR sd.reqdate<=_EndReqDate) 
		AND (_SOStatus IS NULL OR (CASE WHEN _SOStatus='UF' THEN sostatus<>'F' ELSE sostatus=_SOStatus END)) 
		AND (_BranchID IS NULL OR s.branchid=_BranchID);

		SELECT INTO _QtySODesc * FROM fqty1desc(_ItemID,_QtySO);

		SELECT INTO _QtyStock fqty1desc(_ItemID,COALESCE(SUM(qty),0)) FROM stock WHERE itemid=_ItemID;
		
		_retval.itemid:=_ItemID;
		_retval.itemdesc:=_ItemDesc;
		_retval.qtyso:=_QtySODesc;
		_retval.qtystock:=_QtyStock;
			
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cItem;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


/******************************************************************************************/
/*************************************** OVERLOAD-1 ***************************************/
/******************************************************************************************/

CREATE OR REPLACE FUNCTION fSODO_Stock(
	_StartDate 		DATE, 
	_EndDate 		DATE,
	_CustID			VARCHAR(20),
	_SRepID			VARCHAR(20),
	_StartReqDate	DATE,
	_EndReqDate		DATE
) RETURNS SETOF sodo_stock AS 
$BODY$
DECLARE
	_retval			sodo_stock%ROWTYPE;
BEGIN

	SELECT INTO _retval * FROM fSODO_Stock(
		_StartDate, _EndDate, _CustID, _SRepID, _StartReqDate, _EndReqDate, CAST(NULL AS VARCHAR), CAST(NULL AS VARCHAR));

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPriceListPurc()
RETURNS SETOF tPriceListPurc AS
$BODY$
DECLARE
	_retval			tPriceListPurc%ROWTYPE;
	_lastpurcdate		DATE;
	_lastpurcprice		NUMERIC(19,4);
	_lastpurctaxid		VARCHAR(5);
	_lastpurctaxamt		NUMERIC(19,4);
	_lastpurcdiscexp	VARCHAR(15);
	_lastpurcdiscamt	NUMERIC(19,4);
	_lastpurcunit		VARCHAR;
	_itemid			VARCHAR;

	_cItem			REFCURSOR;
BEGIN
	OPEN _cItem FOR
		SELECT itemid FROM item;
	LOOP
		FETCH _cItem INTO _itemid;
		EXIT WHEN NOT FOUND;

		SELECT INTO  
			_LastPurcDate, _LastPurcPrice, _LastPurcTaxID, _LastPurcTaxAmt, _LastPurcDiscExp, _LastPurcDiscAmt,_LastPurcUnit
			LastPurcDate, LastPurcPrice, LastPurcTaxID, LastPurcTaxAmt, LastPurcDiscExp, LastPurcDiscAmt, LastPurcUnit
		FROM fLastPurc(_itemid);

		_retval.ItemID=_itemid;
		_retval.LastPurcDate=_LastPurcDate;
		_retval.LastPurcPrice=_LastPurcPrice;
		_retval.LastPurcTaxID=_LastPurcTaxID;
		_retval.LastPurcTaxAmt=_LastPurcTaxAmt;
		_retval.LastPurcDiscExp=_LastPurcDiscExp;
		_retval.LastPurcDiscAmt=_LastPurcDiscAmt;
		_retval.LastPurcUnit=_LastPurcUnit;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE _cItem;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spAcc_Reset (
	_bujukan	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_count		INT;
BEGIN
	SELECT into _count COUNT(*) FROM lgr;
	IF (FOUND AND _count>0) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Reset Akun Gagal, Sudah ada Transaksi(spAcc_Reset)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
		EXIT;
	END IF;

	_retval.ErrNo:=50002;
	_retval.ErrMsg:='Error DELETE setting akun (spAcc_Reset)';
	UPDATE defa SET defaval=null;

	_retval.ErrNo:=50003;
	_retval.ErrMsg:='Error DELETE akun mitra bisnis (spAcc_Reset)';
	UPDATE bpacc SET accap=null, accar=null;

	_retval.ErrNo:=50004;
	_retval.ErrMsg:='Error DELETE akun item (spAcc_Reset)';
	UPDATE item SET accitemdisc=null, accprcv=null, acccost=null, accinv=null, accpret=null, accsale=null, accsret=null, accexp=null;

	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error DELETE akun kas (spAcc_Reset)';
	UPDATE cash SET accno=null;

	_retval.ErrNo:=50005;
	_retval.ErrMsg:='Error DELETE akun pajak (spAcc_Reset)';
	UPDATE tax SET accpurctax=null, accsaletax=null;

	_retval.ErrNo:=50006;
	_retval.ErrMsg:='Error DELETE accbal (spAcc_Reset)';
	DELETE FROM accbal;

	_retval.ErrNo:=50007;
	_retval.ErrMsg:='Error DELETE begbal (spAcc_Reset)';
	DELETE FROM begbal;

	_retval.ErrNo:=50008;
	_retval.ErrMsg:='Error DELETE acc (spAcc_Reset)';
	DELETE FROM acc WHERE accno<>'*';

	_retval:=(0, 'RESET SUKSES');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fMasterReportPenagihan(
	_CustID		VARCHAR,
	_JT		INT,
	_JTDate		DATE,
	_StartDate	DATE,
	_EndDate	DATE
) RETURNS SETOF tMasterReportPenagihan AS
$BODY$
DECLARE
	_retval		tMasterReportPenagihan%ROWTYPE;
	_bpname		VARCHAR;
	_cityname	VARCHAR;
	_addr		VARCHAR;
	_phone		VARCHAR;
	_Now		DATE;
	_TotSale	DECIMAL(19,4);
	_TotFcn		DECIMAL(19,4);
	_TotSRet	DECIMAL(19,4);
BEGIN
	SELECT INTO _bpname, _cityname, _addr, _phone 
		bpname, cityname, addr, phone 
	FROM bp 
	LEFT JOIN addr ON addr.bpid=bp.bpid 
	LEFT JOIN city ON city.cityid=addr.cityid 
	WHERE bp.bpid=_CustID;

	IF(_JT IS NULL) THEN
		SELECT INTO _TotSale COALESCE(SUM(total-paidamt),0) 
		FROM sale 
		WHERE CustID=_CustID AND SaleDate>=_StartDate AND SaleDate<=_EndDate;
	ELSIF(_JT=1) THEN
		SELECT INTO _TotSale COALESCE(SUM(total-paidamt),0) 
		FROM sale 
		WHERE CustID=_CustID AND SaleDate>=_StartDate AND SaleDate<=_EndDate AND duedays+saledate>_JTDate;
	ELSIF(_JT=0) THEN
		SELECT INTO _TotSale COALESCE(SUM(total-paidamt),0) 
		FROM sale 
		WHERE CustID=_CustID AND SaleDate<=_StartDate AND SaleDate<=_EndDate AND duedays+saledate>_JTDate;
	END IF;

	SELECT INTO _TotFcn COALESCE(SUM(FcnAmt),0) FROM FCN WHERE bpid=_CustID AND FcnDate>=_StartDate 
	AND FcnDate<=_EndDate 
	AND iscontra=false;

	SELECT INTO _TotSRet COALESCE(SUM(total),0) FROM sret WHERE custid=_CustID AND SretDate>=_StartDate 
	AND SretDate<=_EndDate 
	AND iscontra=false;

	_retval.NOW:=CURRENT_DATE;
	_retval.BPNAME:=_bpname;
	_retval.cityname:=_cityname;
	_retval.addr:=_addr;
	_retval.phone:=_phone;
	_retval.dueamt:=_TotSale-_TotFcn-_TotSRet;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fSODeO (
	_DeONo		VARCHAR
) RETURNS VARCHAR AS 
$BODY$
DECLARE
	_cso	REFCURSOR;
	_SONo	VARCHAR;
	_retval	VARCHAR;
BEGIN
	_retval:='';
	OPEN _cso FOR
		/*--SELECT SONo FROM SODeli WHERE DeliNo=_DeliNo;*/
		SELECT DISTINCT(SONo) FROM DeOD WHERE DeONo=_DeONo;
	LOOP
		FETCH _cso INTO _SONo;
		EXIT WHEN NOT FOUND;

		_retval:=_retval||_SONo||',';
	END LOOP;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;





DROP FUNCTION spLimitSaleCost(date);
CREATE OR REPLACE FUNCTION spLimitSaleCost (
	_saledate	DATE
) RETURNS SETOF NUMERIC(19,4) AS
$BODY$
DECLARE
	_cost		DECIMAL(19,4);
	_retval		SPStatus%ROWTYPE;
BEGIN
	_cost:=0;

	SELECT INTO _cost SUM(total)
	FROM SALE WHERE date_part('month',saledate)=date_part('month',_saledate) AND date_part('year',saledate)=date_part('year',_saledate);

	/**IF(_cost>=__subtotal) THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Penjualan Melebihi Batas Limit '|| CAST(__subtotal AS DECIMAL(19,4)) || '!!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	**/

	--_retval:=(0, 'OK');
	RETURN NEXT _cost;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;






CREATE OR REPLACE FUNCTION spLogTrans_New(
	_refType 	VARCHAR, 
	_refNo 		VARCHAR, 
	_crtBy 		VARCHAR, 
	_status 	VARCHAR, 
	_crtDate 	TIMESTAMP WITH TIME ZONE
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_LogNo		VARCHAR;
BEGIN	
	_retval:=(0, 'OK');

	SELECT INTO _spret * FROM spLogTrans_New(_refType, _refNo, _crtBy, _status, CAST(_crtDate AS TIMESTAMP));
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spLogTrans_New(
	_refType 	VARCHAR, 
	_refNo 		VARCHAR, 
	_crtBy 		VARCHAR, 
	_status 	VARCHAR, 
	_crtDate 	TIMESTAMP
) RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_LogNo		VARCHAR;
BEGIN	
	_retval:=(2, 'Gagal buat log !');

	_LogNo := fTransNoGet('LOGTRANS');

	INSERT INTO logtrans VALUES (_LogNo, _refType, _refNo, _crtDate, _crtBy, _status);

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spProcs_New (
	__ProcsID	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval	SPStatus%ROWTYPE;
BEGIN
	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spProcs_Void (
	__ProcsID	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
BEGIN
	_retval:=(1,'Proses '||__ProcsID ||' sudah digunakan di transaksi lain!');

	DELETE FROM ProcsBoM WHERE ProcsID=__ProcsID;
	DELETE FROM Procs WHERE ProcsID=__ProcsID;

	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spRou_Void (
	_RouID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval	SPStatus%ROWTYPE;
	_spret	SPStatus%ROWTYPE;
BEGIN
	DELETE FROM roud WHERE rouid = _RouID ;
	DELETE FROM rou WHERE rouid = _RouID ;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spWO_New (
	_WONO		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	_i		INT;

	cData 		REFCURSOR;
	_JobCardNo	VARCHAR;
BEGIN
	_i:=0;

	OPEN cData FOR 
		SELECT jobcardno FROM jobcard WHERE wono=_WONo;
	LOOP
		FETCH cData INTO _JobCardNo; 
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spJobCard_Void(_JobCardNo);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END LOOP;
	CLOSE cData;


	LOOP

/**
		--INI AKAN INFINITE LOOP alias ERROR
	
		IF (_i>3) THEN 
			RAISE NOTICE 'diatas teluuu aeiii';
		ELSIF (_i>5) THEN
			EXIT;
		END IF;
**/

/**
		--SOLUSI-1

		IF (_i>3) THEN 
			RAISE NOTICE 'diatas teluuu aeiii';
		END IF;

		IF (_i>5) THEN
			EXIT;
		END IF;
**/

		IF (_i>5) THEN
			EXIT;
		ELSIF (_i>3) THEN 
			RAISE NOTICE 'diatas teluuu aeiii';
		END IF;

		_i:=_i+1;
		RAISE NOTICE 'i=%   %', _i, 'OK';
	END LOOP;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spWO_Close (
	__WONo		VARCHAR
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_WOStatus	VARCHAR;
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
BEGIN
	SELECT INTO
		_WOStatus
		WOStatus
	FROM WO WHERE WONo=__WONo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Work Order #% tidak ditemukan (spWO_Close)', __WONo;
	END IF;

	/** 
	IF (_WOStatus<>'F') THEN
		RAISE EXCEPTION 'Status Work Order belum selesai, tidak bisa di Re-Open (spWO_Close)';
	END IF;
	**/

	UPDATE WO SET 
		WOStatus='F'
	WHERE WONo=__WONo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE plpgsql;




CREATE OR REPLACE FUNCTION spWO_Void (
	_WONo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	/*tambahan*/
	_WOStatus	VARCHAR;
	_Active		BOOLEAN;
	_MfgNo		VARCHAR;
BEGIN
	-- cek apakah sudah ada Mfg yang memakai WO ini ?
	SELECT INTO _MfgNo MfgNo
	FROM MFG WHERE WONo=_WONO;

	-- jiak ketemu, jgangan boleh void
	IF FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error, WO '||_WONo ||' masih direferensikan di Produksi '||_MfgNo||' (spWO_Void)'; 
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO 
		_Active, _WOStatus 
		Active, WOStatus
	FROM WO WHERE WONo=_WONo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error, WO '||_WONo ||' tidak ditemukan! (spWO_Void)'; 
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _spret * FROM spJobCard_Void_byRef(_WONo);

	IF _spret.ErrNo<>0 THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	DELETE FROM wod WHERE wono = _WONO;
	DELETE FROM wo WHERE wono = _WONO;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spWO_Open(
	__WONo 	VARCHAR
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_WOStatus	VARCHAR;
	_MfgNo		VARCHAR;
	_MfgStatus	VARCHAR;

	cMfg		REFCURSOR;
BEGIN
	SELECT INTO _WOStatus wostatus
	FROM wo WHERE wono=__WONo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Work Order #% tidak ditemukan (spWO_Open)', __WONo;
	END IF;

	/**
	IF (_WOStatus<>'F') THEN
		RAISE EXCEPTION 'Status Work Order belum selesai, tidak bisa di Re-Open (spWO_Open)';
	END IF;
	**/

	OPEN cMfg FOR
		SELECT mfgno, mfgstatus
		FROM mfg 
		WHERE wono=__WONo;
	LOOP
		FETCH cMfg INTO _MfgNo, _MfgStatus;
		EXIT WHEN NOT FOUND;

		IF (_MfgStatus='F') THEN 
			RAISE EXCEPTION 'Status Produksi#% untuk WO #% sudah dalam keadaan Finish, Buka kembali Produksi (spWO_Open)', _MfgNo, __WONo;
		END IF;
	END LOOP;
	CLOSE cMfg;
	

	UPDATE wo SET wostatus='D' WHERE wono=__WONo;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spWO_Open(
	__WONo 	VARCHAR,	
	__MfgNo VARCHAR
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_WOStatus	VARCHAR;
	_MfgStatus	VARCHAR;
BEGIN
	SELECT INTO _WOStatus wostatus
	FROM wo WHERE wono=__WONo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Work Order #% tidak ditemukan (spWO_Open)', __WONo;
	END IF;

	/**
	IF (_WOStatus<>'F') THEN
		RAISE EXCEPTION 'Status Work Order belum selesai, tidak bisa di Re-Open (spWO_Open)';
	END IF;
	**/

	SELECT INTO _MfgStatus mfgstatus
	FROM mfg WHERE wono=__WONo AND mfgno=__MfgNo;

	IF (_MfgStatus='F') THEN 
		RAISE EXCEPTION 'Status Produksi WO #% sudah dalam keadaan Finish, Buka kembali Produksi (spWO_Open)', __WONo;
	END IF;

	UPDATE wo SET wostatus='D' WHERE wono=__WONo;
	
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fWOItemID (
	_WONO	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_itemid		VARCHAR;
	_pid		VARCHAR;

	_retval		VARCHAR;
BEGIN
	/** Inisialisasi */
	_retval:='';

	SELECT INTO _itemid,_pid itemid,pid
	FROM WOD WHERE wono=_WONO  
	ORDER BY wodno LIMIT 1;

	--_pid:=COALESCE(_pid,'');

	--_retval:=''|| _itemid ||' '|| _pid ||'';

	_retval:=_itemid;
	
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJobCard_New(__JobCardNo VARCHAR) 
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_JobCardNo	VARCHAR;
BEGIN
	/** Inisialisasi */
	
	SELECT INTO _JobCardNo jobcardno FROM jobcard WHERE jobcardno=__JobCardNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Job Card#'||__JobCardNo||' tidak ditemukan (spJobCard_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spJobCard_Void(__JobCardNo VARCHAR) 
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_JobCardNo	VARCHAR;
	_MfgNo		VARCHAR;
BEGIN
	/** Inisialisasi */
	
	SELECT INTO _JobCardNo, _MfgNo jobcardno, mfgno FROM jobcard WHERE jobcardno=__JobCardNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Job Card#'||__JobCardNo||' tidak ditemukan (spJobCard_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_MfgNo IS NOT NULL AND _MfgNo<>'') THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Job Card#'||__JobCardNo||' sudah dipakai dalam produksi#'||COALESCE(_MfgNo, '<kosong>')||' (spJobCard_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	DELETE FROM jobcardm WHERE jobcardno=_JobCardNo;
	DELETE FROM jobcardp WHERE jobcardno=_JobCardNo;
	DELETE FROM jobcard WHERE jobcardno=_JobCardNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION spJobCard_Void_byRef (
	__WONo		VARCHAR
) RETURNS SETOF spStatus AS 
$BODY$
DECLARE
	_retval			spStatus%ROWTYPE;
	_spret			spStatus%ROWTYPE;

	_JobCardNo		VARCHAR;
	cJobCard		REFCURSOR;
BEGIN
	OPEN cJobCard FOR
		SELECT JobCardNo
		FROM JobCard 
		WHERE WONo=__WONo;
	LOOP
		FETCH cJobCard INTO _JobCardNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spJobCard_Void(_JobCardNo);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	END LOOP;
	CLOSE cJobCard;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fBOMProductMultiplier (
	_MfgNo		VARCHAR,
	_BoMID 		VARCHAR 
) RETURNS NUMERIC(19, 4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
	_BoMItemID	VARCHAR;
	_BoMPID		VARCHAR;
	_BoMQty		NUMERIC(15,4);
	_BoMUnit	VARCHAR;

	_ItemID		VARCHAR;
	_PID		VARCHAR;
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR;
	_TotalQty	NUMERIC(15,4);

	cOutput		REFCURSOR;
	_cItemID	VARCHAR;
	_cPID		VARCHAR;
	_cQty		NUMERIC(15,4);
	_cUnit		VARCHAR;
BEGIN
	/** Inisialisasi */
	_TotalQty:=0;
	_retval:=0;

	/** LIMITASI : output produksi hanya 1 boleh item */
	OPEN cOutput FOR 
		SELECT d.itemid, d.pid, SUM(d.qty), d.unit
		FROM mfgpd d JOIN mfgp m ON d.mfgpno=m.mfgpno
		WHERE m.mfgno=_MfgNo AND m.mfgpitype='P' 
		GROUP BY d.itemid, d.pid, d.unit; /** OUTPUT PRODUKSI BUKAN WASTE */
	LOOP
		FETCH cOutput INTO _cItemID, _cPID, _cQty, _cUnit;
		EXIT WHEN NOT FOUND;
		
		_ItemID:=_cItemID;
		_PID:=_cPID;
		_Qty:=_cQty;
		_Unit:=_cUnit;

		_TotalQty:=_TotalQty+fQtyZToQty1(_ItemID, _Qty, _Unit);
	END LOOP;
	CLOSE cOutput;
	IF (_TotalQty=0) THEN
		_retval:=0; /** masih dalam proses secara keseluruhan */
		RETURN _retval;
	END IF;

	SELECT INTO _BoMItemID, _BoMPID, _BoMQty, _BoMUnit 
			itemid, pid, qty, unit
	FROM bom WHERE bomid=_BoMID;
	IF NOT FOUND THEN
		_retval:=-1; /** sudah terproses semua */
	ELSE
		IF (_BoMPID='') THEN
			_BoMQty:=fQtyZToQty1(_BoMItemID, _BoMQty, _BoMUnit);
			_Qty:=_TotalQty;
			_retval:=_Qty/_BoMQty; /** terproses sekian x */
		ELSIF (_BoMItemID=_ItemID AND _BoMPID=_PID) THEN
			_BoMQty:=fQtyZToQty1(_BoMItemID, _BoMQty, _BoMUnit);
			_Qty:=fQtyZToQty1(_ItemID, _Qty, _Unit);
			_retval:=_Qty/_BoMQty; /** terproses sekian x */
		END IF;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tEfisiensiBahan CASCADE;
CREATE TYPE tEfisiensiBahan AS (
	mfgpqty			NUMERIC(19,4),
	itemdesc		VARCHAR,
	pid			VARCHAR,
	unit			VARCHAR,
	bomqty			NUMERIC(19,4),
	stdqty			NUMERIC(19,4),
	qty			NUMERIC(19,4),
	stdcost			NUMERIC(19,4),
	cost			NUMERIC(19,4)
);


CREATE OR REPLACE FUNCTION fEfisiensiBahan(
	__MfgNo			VARCHAR,
	__WONo			VARCHAR
) RETURNS SETOF tEfisiensiBahan AS
$BODY$
DECLARE
	_retval		tEfisiensiBahan%ROWTYPE;
	
	_MfgNo		VARCHAR;
	_WONo		VARCHAR;

	cQuery		REFCURSOR;
BEGIN
	/** Inisialisasi */
	_MfgNo:=COALESCE(__MfgNo,'');
	_WONo:=COALESCE(__WONo,'');

	OPEN cQuery FOR
		SELECT fProdukQty(m.mfgno),i.itemdesc,d.pid,d.unit,b.qty AS bomqty,SUM(d.stdqty) AS stdqty,SUM(d.qty) AS qty,SUM(d.stdcost) AS stdcost,SUM(d.cost) AS cost 
		FROM mfgmd d 
		JOIN mfgm p ON p.mfgmno=d.mfgmno   
		JOIN mfg m ON m.mfgno=p.mfgno
		JOIN bom b ON b.bomid=m.bomid
		JOIN item i ON d.itemid=i.itemid
		WHERE (_MfgNo='' OR _MfgNo IS NULL OR m.mfgno=_MfgNo)
		AND (_WONo='' OR _WONo IS NULL OR m.wono=_WONo)
		GROUP BY m.mfgno,i.itemdesc,d.pid,d.unit,b.qty,d.stdqty,d.stdcost;
	LOOP
		FETCH cQuery INTO _retval.mfgpqty, _retval.itemdesc, _retval.pid, _retval.unit,_retval.bomqty, _retval.stdqty, _retval.qty, _retval.stdcost, _retval.cost;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cQuery;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fProdukQty(
	__MfgNo			VARCHAR
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
	
	_MfgNo		VARCHAR;
BEGIN
	/** Inisialisasi */
	_MfgNo:=COALESCE(__MfgNo,'');

	SELECT into _retval sum(d.qty)
	FROM mfgpd d JOIN mfgp p ON p.mfgpno=d.mfgpno
	JOIN mfg m ON m.mfgno=p.mfgno
	WHERE (_MfgNo='' OR _MfgNo IS NULL OR m.mfgno=_MfgNo);

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tEfisiensiBahanBaku CASCADE;
CREATE TYPE tEfisiensiBahanBaku AS (
	mfgpqty			NUMERIC(19,4),
	produkitem		VARCHAR,
	itemdesc		VARCHAR,
	pid			VARCHAR,
	unit			VARCHAR,
	bomqty			NUMERIC(19,4),
	stdqty			NUMERIC(19,4),
	qty			NUMERIC(19,4),
	stdcost			NUMERIC(19,4),
	cost			NUMERIC(19,4)
);


CREATE OR REPLACE FUNCTION fProdukName(
	__MfgNo			VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;

	_MfgNo		VARCHAR;
BEGIN
	/** Inisialisasi */
	_MfgNo:=COALESCE(__MfgNo,'');

	SELECT into _retval i.itemdesc
	FROM mfgpd d JOIN mfgp p ON p.mfgpno=d.mfgpno
	JOIN mfg m ON m.mfgno=p.mfgno
	JOIN item i ON i.itemid=d.itemid
	WHERE (_MfgNo='' OR _MfgNo IS NULL OR m.mfgno=_MfgNo);
	
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fEfisiensiBahanBaku(
	__MfgNo			VARCHAR,
	__WONo			VARCHAR,
	__Tipe			VARCHAR
) RETURNS SETOF tEfisiensiBahanBaku AS
$BODY$
DECLARE
	_retval		tEfisiensiBahanBaku%ROWTYPE;
	
	_MfgNo		VARCHAR;
	_WONo		VARCHAR;
	_Tipe		VARCHAR;

	cQuery		REFCURSOR;
BEGIN
	/** Inisialisasi */
	_MfgNo:=COALESCE(__MfgNo,'');
	_WONo:=COALESCE(__WONo,'');
	_Tipe:=COALESCE(__Tipe,'');


	OPEN cQuery FOR
		SELECT fProdukQty(m.mfgno),fProdukName(m.mfgno),i.itemdesc,d.pid,d.unit,b.qty AS bomqty,d.stdqty AS stdqty,
		d.qty AS qty,d.stdcost AS stdcost,d.cost AS cost 
		FROM mfgmd d 
		JOIN mfgm p ON p.mfgmno=d.mfgmno   
		JOIN mfg m ON m.mfgno=p.mfgno
		JOIN bom b ON b.bomid=m.bomid
		JOIN bomd bd ON bd.itemid=d.itemid
		JOIN item i ON d.itemid=i.itemid
		WHERE (_MfgNo='' OR _MfgNo IS NULL OR m.mfgno=_MfgNo)
		AND (_WONo='' OR _WONo IS NULL OR m.wono=_WONo)
		AND (_Tipe='' OR _Tipe IS NULL OR bd.matclass=_Tipe)
		GROUP BY m.mfgno,i.itemdesc,d.pid,d.unit,b.qty,d.stdqty,d.qty,d.stdcost,d.cost;
	LOOP
		FETCH cQuery INTO _retval.mfgpqty, _retval.produkitem, _retval.itemdesc, _retval.pid, _retval.unit,_retval.bomqty, _retval.stdqty, 
		_retval.qty, _retval.stdcost, _retval.cost;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cQuery;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tEfisiensiProduk CASCADE;
CREATE TYPE tEfisiensiProduk AS (
	itemdesc		VARCHAR,
	unit			VARCHAR,
	stdqty			NUMERIC(19,4),
	qty			NUMERIC(19,4),
	stdcost			NUMERIC(19,4),
	cost			NUMERIC(19,4)
);


CREATE OR REPLACE FUNCTION fEfisiensiProduk(
	__MfgNo			VARCHAR,
	__WONo			VARCHAR
) RETURNS SETOF tEfisiensiProduk AS
$BODY$
DECLARE
	_retval		tEfisiensiProduk%ROWTYPE;
	
	_MfgNo		VARCHAR;
	_WONo		VARCHAR;

	cQuery		REFCURSOR;
BEGIN
	/** Inisialisasi */
	_MfgNo:=COALESCE(__MfgNo,'');
	_WONo:=COALESCE(__WONo,'');

	OPEN cQuery FOR
		SELECT i.itemdesc,d.unit,b.qty AS stdqty,SUM(d.qty) AS qty,b.totstdcost AS stdcost,d.cost AS cost 
		FROM mfgpd d 
		JOIN mfgp p ON p.mfgpno=d.mfgpno   
		JOIN mfg m ON m.mfgno=p.mfgno
		JOIN bom b ON b.bomid=m.bomid
		JOIN item i ON d.itemid=i.itemid
		WHERE (_MfgNo='' OR _MfgNo IS NULL OR m.mfgno=_MfgNo)
		AND (_WONo='' OR _WONo IS NULL OR m.wono=_WONo)
		GROUP BY i.itemdesc,d.unit,b.qty,b.totstdcost,d.cost;
	LOOP
		FETCH cQuery INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cQuery;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tRptCostOfMfg CASCADE;
CREATE TYPE tRptCostOfMfg AS (
	wono		VARCHAR,
	sono		VARCHAR,
	cust		VARCHAR,
	groupid		INT2,
	groupdesc	VARCHAR,
	itemid		VARCHAR,
	pid		VARCHAR,
	itemdesc	VARCHAR,
	cost		NUMERIC(19,4)
);

DROP TYPE tMfgTree CASCADE;
CREATE TYPE tMfgTree AS (
	mfgno		VARCHAR,
	dmfgno		VARCHAR	
);


/**CREATE OR REPLACE FUNCTION fMfgTree(
	_MfgNo		VARCHAR
) RETURNS SETOF tMfgTree AS
$BODY$
DECLARE
	_retval		tMfgTree%ROWTYPE;
	
	_DMfgNo		VARCHAR;
	
	cData		REFCURSOR;	
BEGIN
	OPEN cData FOR
	
	LOOP

	END LOOP;	
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;
*/

CREATE OR REPLACE FUNCTION fRptCostOfMfg (
	__SONo		VARCHAR,
	__WONo		VARCHAR,
	__MfgNo		VARCHAR,
	__ItemID	VARCHAR,
	__PID		VARCHAR,
	__PerWO		BOOLEAN
) RETURNS SETOF tRptCostOfMfg AS 
$BODY$
DECLARE
	_retval		tRptCostOfMfg%ROWTYPE;
	
	cData		REFCURSOR;
	cInput		REFCURSOR;
	cWIP		REFCURSOR;
	cOutput		REFCURSOR;

	_RawFound	BOOLEAN;
	_LaborFound	BOOLEAN;
	_OverhFound	BOOLEAN;
	_WIPFound	BOOLEAN;

	_Cust		VARCHAR;
	_SONo		VARCHAR;
	_UpWONo		VARCHAR;
	_WONo		VARCHAR;
	_WOLevel	INT2;
	_LastCust	VARCHAR;
	_LastSONo	VARCHAR;
	_LastWONo	VARCHAR;
	
	_MfgNo		VARCHAR;
	_BoMID		VARCHAR;
	_ItemID		VARCHAR;
	_ItemDesc	VARCHAR;
	_ItemType	VARCHAR;
	_PID		VARCHAR;
	_Qty		NUMERIC(15,4);
	_Unit		VARCHAR;
	_Cost		NUMERIC(19,4); 

	_Query		TEXT;
	_SubQuery	TEXT;
	_MfgList	TEXT;

	_LastMfgNo	VARCHAR;
BEGIN
	IF (__SONo IS NOT NULL) THEN
		OPEN cInput FOR
			SELECT wono FROM wo WHERE sono=__SONo AND wolevel=0;
		LOOP
			FETCH cInput INTO _WONo;
			EXIT WHEN NOT FOUND;

			OPEN cData FOR
				SELECT * FROM fRptCostOfMfg(NULL, _WONo, NULL, NULL, NULL, __PerWO);
			LOOP
				FETCH cData INTO _retval;
				EXIT WHEN NOT FOUND;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cData;
		END LOOP;
		CLOSE cInput;
	ELSIF (__WONo IS NOT NULL) THEN
		OPEN cInput FOR
			SELECT mfgno FROM mfg WHERE wono=__WONo;
		LOOP
			FETCH cInput INTO _MfgNo;
			EXIT WHEN NOT FOUND;

			OPEN cData FOR
				SELECT * FROM fRptCostOfMfg(NULL, NULL, _MfgNo, NULL, NULL, __PerWO);
			LOOP
				FETCH cData INTO _retval;
				EXIT WHEN NOT FOUND;

				RETURN NEXT _retval;
			END LOOP;
			CLOSE cData;
		END LOOP;
		CLOSE cInput;
	ELSIF (__MfgNo IS NOT NULL) THEN
		_RawFound:=FALSE;
		_LaborFound:=FALSE;
		_OverhFound:=FALSE;
		_WIPFound:=FALSE;

		--_SubQuery:=QUOTE_LITERAL(__MfgNo);
		_MfgList:='';
		SELECT INTO _WONo, _UpWONo, _WOLevel m.wono, wo.upwono, wo.wolevel  
		FROM mfg m JOIN wo ON m.wono=wo.wono WHERE m.mfgno=__MfgNo;
		IF FOUND THEN
			OPEN cData FOR
				SELECT m.mfgno, wo.wolevel FROM mfg m JOIN wo ON m.wono=wo.wono WHERE (_UpWONo IS NULL OR upwono=_UpWONo) AND wo.wolevel>_WOLevel ORDER BY wo.wolevel DESC;
			LOOP
				FETCH cData INTO _MfgNo, _WOLevel;
				EXIT WHEN NOT FOUND;
				
				_SubQuery:=_SubQuery||','||QUOTE_LITERAL(_MfgNo);
				_MfgList:=_MfgList||','||_MfgNo;
			END LOOP;
			CLOSE cData;
		END IF;
		_MfgList:=_MfgList||','||__MfgNo;

		_Query:='SELECT bp.bpname, m.sono, m.wono, i.itemtype, m.itemid, i.itemdesc, m.pid, m.cost  
			FROM (
				SELECT ms.custid, wo.sono, ms.wono, CASE WHEN d.itemidsub IS NULL OR d.itemidsub='' THEN d.itemid ELSE d.itemidsub END AS itemid, d.pid, SUM(d.qty*d.cost) AS cost
				FROM mfgmd d JOIN mfgm m ON d.mfgmno=m.mfgmno
				JOIN mfg ms ON m.mfgno=ms.mfgno 
				LEFT JOIN wo ON ms.wono=wo.wono 
				WHERE ms.mfgno=__MfgNo
				GROUP BY ms.custid, wo.sono, ms.wono, d.itemid, d.pid
			) m JOIN item i ON m.itemid=i.itemid 
			LEFT JOIN bp ON m.custid=bp.bpid';

		/** Proses Input */
		OPEN cInput FOR
			SELECT bp.bpname, m.sono, m.wono, i.itemtype, m.itemid, i.itemdesc, m.pid, m.cost  
			FROM (
				SELECT ms.custid, wo.sono, ms.wono, CASE WHEN d.itemidsub IS NULL OR d.itemidsub='' THEN d.itemid ELSE d.itemidsub END AS itemid, d.pid, SUM(d.qty*d.cost) AS cost
				FROM mfgmd d JOIN mfgm m ON d.mfgmno=m.mfgmno
				JOIN mfg ms ON m.mfgno=ms.mfgno 
				LEFT JOIN wo ON ms.wono=wo.wono 
				WHERE ms.mfgno=__MfgNo
				GROUP BY ms.custid, wo.sono, ms.wono, d.itemid, d.itemidsub, d.pid
			) m JOIN item i ON m.itemid=i.itemid 
			LEFT JOIN bp ON m.custid=bp.bpid;  
		LOOP
			FETCH cInput INTO _Cust, _SONo, _WONo, _ItemType, _ItemID, _ItemDesc, _PID, _Cost;
			EXIT WHEN NOT FOUND;
			
			IF (_ItemType='RAW') THEN
				_retval.groupid:=0;
				_retval.groupdesc:='Bahan Baku';
				_RawFound:=TRUE;
			ELSIF (_ItemType='WIP' OR _ItemType='GOOD' OR _ItemType='PACK') THEN
				/**
				_retval.groupid:=3;
				_retval.groupdesc:='WIP Terpakai';
				_WIPFound:=TRUE;
				*/
				CONTINUE;
			ELSIF (_ItemType='LABOR') THEN
				_retval.groupid:=1;
				_retval.groupdesc:='Biaya Tenaga Kerja';
				_LaborFound:=TRUE;
			ELSE
				_retval.groupid:=2;
				_retval.groupdesc:='Biaya Operasional';
				_OverhFound:=TRUE;
			END IF;

			_retval.wono:=_WONo;
			_retval.sono:=_SONo;
			_retval.cust:=_Cust;
			_retval.itemid:=_ItemID;
			_retval.pid:=_PID;
			_retval.itemdesc:=_ItemDesc;
			_retval.cost:=_Cost;

			_LastWONo:=_WONo;
			_LastSONo:=_SONo;
			_LastCust:=_Cust;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cInput;

		IF (_RawFound=FALSE) THEN
			_retval.groupid:=0;
			_retval.groupdesc:='Bahan Baku';
			_retval.wono:=_LastWONo;
			_retval.sono:=_LastSONo;
			_retval.cust:=_LastCust;
			_retval.itemid:=NULL;
			_retval.pid:=NULL;
			_retval.itemdesc:=NULL;
			_retval.cost:=NULL;
			
			RETURN NEXT _retval;
		END IF;

		IF (_LaborFound=FALSE) THEN
			_retval.groupid:=1;
			_retval.groupdesc:='Biaya Tenaga Kerja';
			_retval.wono:=_LastWONo;
			_retval.sono:=_LastSONo;
			_retval.cust:=_LastCust;
			_retval.itemid:=NULL;
			_retval.pid:=NULL;
			_retval.itemdesc:=NULL;
			_retval.cost:=NULL;

			RETURN NEXT _retval;
		END IF;

		IF (_OverhFound=FALSE) THEN
			_retval.groupid:=2;
			_retval.groupdesc:='Biaya Operasional';
			_retval.wono:=_LastWONo;
			_retval.sono:=_LastSONo;
			_retval.cust:=_LastCust;
			_retval.itemid:=NULL;
			_retval.pid:=NULL;
			_retval.itemdesc:=NULL;
			_retval.cost:=NULL;
		
			RETURN NEXT _retval;
		END IF;

		/** PROSES WIP */
		OPEN cWIP FOR
			SELECT * FROM fWIPUsed(_MfgList, ',');
		LOOP
			FETCH cWIP INTO _ItemID, _PID, _ItemDesc, _Cost;
			EXIT WHEN NOT FOUND;
			
			_WIPFound:=TRUE;

			_retval.groupid:=3;
			_retval.groupdesc:='WIP Terpakai';
			_retval.wono:=_LastWONo;
			_retval.sono:=_LastSONo;
			_retval.cust:=_LastCust;
			_retval.itemid:=_ItemID;
			_retval.pid:=_PID;
			_retval.itemdesc:=_ItemDesc;
			_retval.cost:=_Cost;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cWIP;

		IF (_WIPFound=FALSE) THEN
			_retval.groupid:=3;
			_retval.groupdesc:='WIP Terpakai';
			_retval.wono:=_LastWONo;
			_retval.sono:=_LastSONo;
			_retval.cust:=_LastCust;
			_retval.itemid:=NULL;
			_retval.pid:=NULL;
			_retval.itemdesc:=NULL;
			_retval.cost:=NULL;

			RETURN NEXT _retval;
		END IF;

		/** Proses Output */
		OPEN cOutput FOR
			SELECT bp.bpname, m.sono, m.wono, i.itemtype, m.itemid, i.itemdesc, m.pid, m.cost  
			FROM (
				SELECT ms.custid, wo.sono, ms.wono, d.itemid, d.pid, SUM(d.qty*d.cost) AS cost
				FROM mfgpd d JOIN mfgp m ON d.mfgpno=m.mfgpno
				JOIN mfg ms ON m.mfgno=ms.mfgno 
				LEFT JOIN wo ON ms.wono=wo.wono
				WHERE ms.mfgno=__MfgNo AND m.mfgpitype='P' 
				GROUP BY ms.custid, wo.sono, ms.wono, d.itemid, d.pid
			) m JOIN item i ON m.itemid=i.itemid 
			LEFT JOIN bp ON m.custid=bp.bpid;
		LOOP
			FETCH cOutput INTO _Cust, _SONo, _WONo, _ItemType, _ItemID, _ItemDesc, _PID, _Cost;
			EXIT WHEN NOT FOUND;

			_retval.wono:=_WONo;
			_retval.sono:=_SONo;
			_retval.cust:=_Cust;
			_retval.groupid:='4';
			_retval.groupdesc:='Barang Jadi';
			_retval.itemid:=_ItemID;
			_retval.pid:=_PID;
			_retval.itemdesc:=_ItemDesc;
			_retval.cost:=_Cost;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cOutput;
	ELSE

	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION fRptCostOfMfg (
	__SONo		VARCHAR,
	__WONo		VARCHAR,
	__MfgNo		VARCHAR,
	__ItemID	VARCHAR,
	__PID		VARCHAR 
) RETURNS SETOF tRptCostOfMfg AS 
$BODY$
DECLARE
	_retval		tRptCostOfMfg%ROWTYPE;
	
	cData		REFCURSOR;
BEGIN
	OPEN cData FOR
		SELECT * FROM fRptCostOfMfg(__SONo, __WONo, __MfgNo, __ItemID, __PID, FALSE);
	LOOP
		FETCH cData INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fUnitIndex (
	_ItemID VARCHAR, 
	_Unit VARCHAR
) RETURNS INT2 AS
$BODY$
DECLARE
	_Unit1		VARCHAR;
	_Unit2		VARCHAR;
	_Unit3		VARCHAR;
BEGIN
	SELECT INTO _Unit1, _Unit2, _Unit3 unit1, unit2, unit3 FROM item WHERE itemid=_ItemID;

	IF (_Unit=_Unit1) THEN
		RETURN 1;
	ELSIF (_Unit=_Unit2) THEN
		RETURN 2;
	ELSIF (_Unit=_Unit3) THEN
		RETURN 3;
	END IF;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPriceList(
	_ItemID		VARCHAR,
	_Unit		VARCHAR,
	_PrcLvlID	VARCHAR,
	_CrcID		VARCHAR
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
	_PriceIndex	INTEGER;
BEGIN
	_retval:=0;

	_PriceIndex:=fUnitIndex(_ItemID, _Unit);
	_PriceIndex:=COALESCE(_PriceIndex, 0);

	SELECT INTO _retval (CASE WHEN _PriceIndex=1 THEN price1 WHEN _PriceIndex=2 THEN price2 WHEN _PriceIndex=3 THEN price3 ELSE 0 END) 
	FROM prc 
	WHERE itemid=_ItemID AND prclvlid=_PrcLvlID AND crcid=_CrcID;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spMfg_Close (
	__MfgNo		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	
	_PerID			VARCHAR;
	_BranchID		VARCHAR;
	_DeptID			VARCHAR;
	_PrjID			VARCHAR;


	_Jnl1No			VARCHAR;
	_Jnl1Note		VARCHAR;
	_JnlD1No		SMALLINT;
	_JnlD1Amt		DECIMAL(19,4);
	_JnlD1AmtForex		DECIMAL(19,4);


	_MfgM_TotCost		DECIMAL(19,4);
	_MfgP_TotQty1		DECIMAL(15,4);
	_MfgP_Cost_Unit1	DECIMAL(19,4);

	_MfgPNo			VARCHAR;
	_MfgPDNo		SMALLINT;
	_MfgPDate		DATE;
	
	_ItemID			VARCHAR;
	_WhID			VARCHAR;
	_PID			VARCHAR;
	_Qty			DECIMAL(15,4);
	_MfgP_Qty1		DECIMAL(15,4);
	_QtyX			DECIMAL(15,4);
	_Unit			VARCHAR;
	_Cost			DECIMAL(19,4);
	_MfgPDPercent		DECIMAL(5,2);
	
	_AccItemInv		VARCHAR;
	_AccWIPInv		VARCHAR;

	_JnlDAmt		DECIMAL(19,4);
	_JnlDAmtForex		DECIMAL(19,4);

	_JnlNo			VARCHAR;
	_JnlDNo			SMALLINT;
	_JnlNote		VARCHAR;

	_ExcRate		DECIMAL(19,4);
	_FisRate		DECIMAL(19,4);
	_CrcID			VARCHAR;

	_CrtBy			VARCHAR(10);
	_CrtDate		TIMESTAMP;
	_UpdBy			VARCHAR(10);
	_UpdDate		TIMESTAMP;

	_MfgPDCrtBy		VARCHAR(10);
	_MfgPDCrtDate		TIMESTAMP;
	_MfgPDUpdBy		VARCHAR(10);
	_MfgPDUpdDate		TIMESTAMP;
	_MfgPIType		VARCHAR(5);

	_StockANo		VARCHAR;
	_StockANote		VARCHAR;
	_StockADNo		SMALLINT;

	_WONo			VARCHAR;
	_WOStatus		VARCHAR;

	cMfgP			REFCURSOR;
	cMfgPD			REFCURSOR;
	cWO			REFCURSOR;
	cWOD			REFCURSOR;

	_WODNo			SMALLINT;
	_WOD_ItemID		VARCHAR;
	_WOD_Qty		DECIMAL(19,4);
	_WOD_Qtydone		DECIMAL(19,4);

	_MfgType		VARCHAR;

	_Mfg_CloseDate		DATE;
	_MfgPD_Conv		DECIMAL(15,4);
	_MfgPD_StdCost		DECIMAL(19,4);
	_tmpQty			DECIMAL(15,4);

	_BoMID			VARCHAR;
	_BoM_StdQty		NUMERIC(15,4);
	_BoM_StdUnit		VARCHAR;
BEGIN
	_ExcRate:=1;
	_FisRate:=1;
	_CrcID:=fCrcDefault();
	_Mfg_CloseDate:=CURRENT_DATE;

	SELECT INTO 
		_PerID, _BranchID, _DeptID, _PrjID, _AccWIPInv, _BomID
		perid, branchid, deptid, prjid, accwip, bomid 
	FROM Mfg WHERE MfgNo=__MfgNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Produksi #'||__MfgNo||' tidak ditemukan! (spMfg_Close)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	-- Hitung total nominal bahan produksi
	SELECT INTO 
		_MfgM_TotCost 
		SUM(Qty*Cost) 
	FROM MfgMD d JOIN MfgM h ON h.MfgMNo=d.MfgMNo
	WHERE h.MfgNo=__MfgNo;

	-- Hitung total qty hasil produksi
	SELECT INTO
		_MfgP_TotQty1
		SUM(fQtyZToQty1(d.ItemID, d.Qty, d.Unit))
	FROM MfgPD d JOIN MfgP h ON h.MfgPNo=d.MfgPNo
	WHERE h.MfgNo=__MfgNo;

	-- Hitung cost per item hasil
	_MfgP_Cost_Unit1:=_MfgM_TotCost/_MfgP_TotQty1;

	-- Generate stock Output hasil Produksi ---
	OPEN cMfgP FOR
		SELECT MfgPNo FROM MfgP
		WHERE MfgNo=__MfgNo;
	LOOP -- looping sejumlah mfgp
		FETCH cMfgP INTO _MfgPNo;
		EXIT WHEN NOT FOUND;

		-- ambil data user ybs
		SELECT INTO
			_MfgPDate,_MfgPDCrtBy,_MfgPDCrtDate,_MfgPDUpdBy,_MfgPDUpdDate,_MfgPIType
			MfgPDate,CrtBy,CrtDate,UpdBy,UpdDate,mfgpitype
		FROM MfgP
		WHERE MfgPNo=_MfgPNo;


		-- proses item2 MfgPD di dalam tiap MfgP
		_StockADNo:=0;
		OPEN cMfgPD FOR
			SELECT 
				d.MfgPDNo, d.ItemID, d.WhID, d.Qty, d.Unit, d.PID, d.QtyX, fQtyZToQty1(d.ItemID, d.Qty, d.Unit), d.MfgPDPercent
			FROM MfgPD d JOIN Item i ON d.ItemID=i.ItemID
			WHERE d.MfgPNo=_MfgPNo AND i.IsStock=TRUE;
		LOOP
			FETCH cMfgPD INTO _MfgPDNo, _ItemID, _WhID, _Qty, _Unit, _PID, _QtyX, _MfgP_Qty1, _MfgPDPercent;
			EXIT WHEN NOT FOUND;


			_StockADNo:=_StockADNo+1;
			---- Jika row pertama, maka Generate header!
			IF (_StockADNo=1) THEN
				_StockANo:=fTransNoGet('STOCKA');
				_StockANote:='Hasil Produksi #'||__MfgNo;

				SELECT INTO _spret * FROM spStockA_New(
					_StockANo, NULL, 'MFGP', _MfgPDate, 'I', 'MFGP', _MfgPNo, _StockANote,
					TRUE, _MfgPDCrtBy, _MfgPDCrtDate, _MfgPDUpdBy, _MfgPDUpdDate, _PerID, _BranchID, _DeptID, _PrjID
				);
				IF _spret.ErrNo<>0 THEN
					_retval.ErrNo:=_spret.ErrNo;
					_retval.ErrMsg:=_spret.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMSg;
				END IF;
			END IF;

			_MfgPD_Conv:=fItemUnit_Conv(_ItemID, _Unit);
			_tmpQty:=_Qty;	-- positif (masuk)
			IF(_MfgPIType='P') THEN
				-- Hitung cost per item hasil sesuai persentase
				_Cost:=((_MfgM_TotCost*(_MfgPDPercent/100))/_MfgP_Qty1)*_MfgPD_Conv;
			ELSE
				_Cost:=0;
			END IF;
			-- insert stockad
			SELECT INTO _spret * FROM spStockAD_New(
				_MfgPDate, 'MFGP', _StockANo, _StockADNo, _WhID, 
				_ItemID, _PID, _tmpQty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, TRUE);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			IF(_MfgPIType='P') THEN
				_MfgPD_StdCost:=fStdCost(_ItemID, _PID, _Qty, _Unit);
			ELSE
				_MfgPD_StdCost:=0;
			END IF;
			
			IF (_MfgPD_StdCost IS NULL OR _MfgPD_StdCost=-1) THEN
				RAISE EXCEPTION 'Error menghitung Standard cost Produksi:%, MfgP:% (spMfg_Close)', __MfgNo, _MfgPNo;
			END IF;

			---- get standard cost dari BoMD dengan item yang bersesuaian ---
			SELECT INTO 
				_BoM_StdQty, _BoM_StdUnit
				Qty, Unit 
			FROM BoM 
			WHERE BoMID=_BoMID AND ItemID=_ItemID;

			-- Update Cost item hasil produksi di MfgP
			UPDATE MfgPD SET 
				Cost=_Cost,
				StdCost=_MfgPD_StdCost,
				StdQty=_BoM_StdQty,
				StdUnit=_BoM_StdUnit
			WHERE MfgPNo=_MfgPNo AND MfgPDNo=_MfgPDNo;

		END LOOP; -- cMfgPD (detail hasil produksi)
		CLOSE cMfgPD;

	END LOOP; -- cMfgP (daftar hasil produksi)
	CLOSE cMfgP;


	------------------- Accounting here -------------------------
	IF (isAccInstalled()) THEN
		IF (_AccWIPInv IS NULL) THEN
			_AccWIPInv:=fAccDefault('WIPINVENTORY');
		END IF;

		/** Last Checking */
		IF (_AccWIPInv IS NULL) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Akun WIP belum diset atau setting default akun WIP belum diset ! '||E'\n'||'Silahkan mensetting akun tersebut di master produksi#'||_MfgNo||' atau form setting default akun (spMfg_Close)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		_Jnl1No:=fTransNoGet('JURNAL');
		_Jnl1Note:='Hasil Produksi #'||__MfgNo;

		OPEN cMfgPD FOR
			SELECT d.MfgPNo, fItemAcc(d.ItemID, 'INV'), SUM(d.Qty*d.Cost) 
			FROM MfgPD d
			JOIN MfgP p ON d.MfgPNo=p.MfgPNo
			JOIN Mfg h ON p.MfgNo=h.MfgNo
			WHERE h.MfgNo=__MfgNo
			GROUP BY d.MfgPNo, fItemAcc(d.ItemID, 'INV');

		LOOP
			FETCH cMfgPD INTO _MfgPNo, _AccItemInv, _JnlDAmt;
			EXIT WHEN NOT FOUND;


			SELECT INTO
				_MfgPDCrtBy,_MfgPDCrtDate,_MfgPDUpdBy,_MfgPDUpdDate
				CrtBy,CrtDate,UpdBy,UpdDate
			FROM MfgP
			WHERE MfgPNo=_MfgPNo;

			_JnlDAmtForex:=_JnlDAmt;
			_JnlNo:=fTransNoGet('JURNAL');
			_JnlNote:='Hasil Produksi#'||__MfgNo||', MfgP#'||_MfgPNo;

			
			------- Generate Jurnal Header untuk JurnalVoucher(JV) Penjualan
			SELECT INTO _spret * FROM spJnl_Insert(
				_JnlNo, 'MFGP', _MfgPDate, 'MFGP', _MfgPNo,
				_Jnl1Note, true, true, CAST(0 AS NUMERIC(19,4)),
				_MfgPDCrtBy, _MfgPDCrtDate, _MfgPDUpdBy, _MfgPDUpdDate,
				_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate
			);

			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			_JnlDNo:=0;
			------- Jurnal Ayat Item(produk) Inventory debet-------------
			_JnlDNo:=_JnlDNo+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccItemInv, 'D', _JnlDAmt, _DeptID, _PrjID, _JnlDAmtForex);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			------- Jurnal Ayat WIP kredits-------------
			_JnlDNo:=_JnlDNo+1;
			SELECT INTO _spret * FROM spJnlD_Insert(_JnlNo, _JnlDNo, _AccWIPInv, 'K', _JnlDAmt, _DeptID, _PrjID, _JnlDAmtForex);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
			
			--- trigger posting jurnal
			SELECT INTO _spret * FROM spJnl_New(_JnlNo);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;

	END IF;

	SELECT INTO
		_MfgType
		MfgType
	FROM Mfg WHERE MfgNo=__MfgNo;

	-- check apakah mfg nya simple atau advance
	IF _MfgType<>'S' THEN

		--Update qtydone dan tutup WO
		OPEN cWOD FOR
		SELECT wod.wono,wod.wodno,wod.itemid,fQtyZToQty1(wod.ItemID, wod.Qty, wod.Unit) as qty_wo 
		FROM mfg m 
		JOIN mfgp mp ON m.mfgno=mp.mfgno 
		JOIN mfgpd mpd on mpd.mfgpno=mp.mfgpno 
		JOIN wod ON m.wono=wod.wono AND mpd.itemid=wod.itemid
		WHERE m.mfgno=__MfgNo;
		LOOP
			FETCH cWOD INTO _WONo,_WODNo,_WOD_ItemID,_WOD_Qty;
			EXIT WHEN NOT FOUND;

			UPDATE WOD SET QTYDONE=_MfgP_TotQty1
			WHERE wono=_WONo AND wodno=_WODNo AND itemid=_WOD_ItemID;

			SELECT INTO _WOD_Qtydone qtydone FROM WOD
			WHERE wono=_WONo AND wodno=_WODNo AND itemid=_WOD_ItemID;

			IF (_WOD_Qtydone>=_WOD_Qty) THEN

				SELECT INTO _spret * FROM spWO_Close(_WONo);
				IF (_spret.ErrNo<>0) THEN
					_retval.ErrNo:=spWO_Close.ErrNo;
					_retval.ErrMsg:=spWO_Close.ErrMsg;
					RAISE EXCEPTION '%', _retval.ErrMsg;
				END IF; 
			END IF;
		END LOOP;
		CLOSE cWOD;
	END IF;

	-- Update status produksi = F (Finished)
	UPDATE Mfg SET MfgStatus='F' WHERE MfgNo=__MfgNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;



/**
	sp utk membuka kembali status produksi yg sudah ditutup
	pseudo:
	1. void transaksi stok utk output produksinya
	2. void posting jurnal
	3. update status = 'P'
**/

CREATE OR REPLACE FUNCTION spMfg_ReOpen(
	__MfgNo		VARCHAR
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_MfgStatus	VARCHAR;
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;
	
	_WONo		VARCHAR;

	cWO		REFCURSOR;
	cMfgP		REFCURSOR;

	_MfgPNo		VARCHAR;
	_MfgType	VARCHAR;

	_WODNo		SMALLINT;
	_WOD_ItemID	VARCHAR;

BEGIN
	SELECT INTO
		_MfgStatus, _MfgType
		MfgStatus, MfgType
	FROM Mfg WHERE MfgNo=__MfgNo;

	IF NOT FOUND THEN
		RAISE EXCEPTION 'Produksi #% tidak ditemukan (spMfg_ReOpen)';
	END IF;
--RAISE EXCEPTION '%',_MfgStatus;
	IF (_MfgStatus<>'F') THEN
		RAISE EXCEPTION 'Status Produksi belum selesai, tidak bisa di Re-Open (spMfg_ReOpen)';
	END IF;

	-- 1. void transaksi stok utk output produksinya
	OPEN cMfgP FOR
	SELECT MfgPNo FROM MfgP WHERE MfgNo=__MfgNo;
	LOOP
		FETCH cMfgP INTO _MfgPNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spStockA_Void_byRef('MFGP', _MfgPNo);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=spretspStockA_Void_byRef.ErrNo;
			_retval.ErrMsg:=spretspStockA_Void_byRef.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF; 

		-- 2. void posting jurnal
		IF (isAccInstalled()) THEN
			SELECT INTO _spret * from spJnl_Void_byRef('MFGP', _MfgPNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=spretSisaCons.ErrNo;
				_retval.ErrMsg:=spretSisaCons.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END IF;

	END LOOP;
	CLOSE cMfgP;
	
	-- 3. update status = 'P'
	UPDATE Mfg SET 
		MfgStatus='P'
	WHERE MfgNo=__MfgNo;

	-- check apakah mfg nya simple atau advance
	IF _MfgType<>'S' THEN

	--Update qtydone dan tutup WO
		OPEN cWO FOR
		SELECT wod.wono,wod.wodno,wod.itemid 
		FROM mfg m 
		JOIN mfgp mp ON m.mfgno=mp.mfgno 
		JOIN mfgpd mpd on mpd.mfgpno=mp.mfgpno 
		JOIN wod ON m.wono=wod.wono AND mpd.itemid=wod.itemid
		WHERE m.mfgno=__MfgNo;
		LOOP
			FETCH cWO INTO _WONo,_WODNo,_WOD_ItemID;
			EXIT WHEN NOT FOUND;

			UPDATE WOD SET QTYDONE=0
			WHERE wono=_WONo AND wodno=_WODNo AND itemid=_WOD_ItemID;

			SELECT INTO _spret * FROM spWO_Open(_WONo,__MfgNo);
			IF (_spret.ErrNo<>0) THEN
				_retval.ErrNo:=spWO_Close.ErrNo;
				_retval.ErrMsg:=spWO_Close.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF; 
		END LOOP;
		CLOSE cWO;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spMfg_ReOpen_Close_Force()
RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_MfgNo		VARCHAR;
	_MfgStatus	VARCHAR;
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	cMfgP		REFCURSOR;
BEGIN
	OPEN cMfgP FOR
	SELECT m.mfgno,m.mfgstatus FROM mfgp p JOIN mfg m ON m.mfgno=p.mfgno WHERE mfgstatus='F' order by mfgno;
	LOOP
		FETCH cMfgP INTO _MfgNo,_MfgStatus;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spMfg_ReOpen(_MfgNo);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=spretspMfg_ReOpen.ErrNo;
			_retval.ErrMsg:=spretspMfg_ReOpen.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF; 

		SELECT INTO _spret * FROM spMfg_Close(_MfgNo);
		IF (_spret.ErrNo<>0) THEN
			_retval.ErrNo:=spretspMfg_Close.ErrNo;
			_retval.ErrMsg:=spretspMfg_Close.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF; 
	END LOOP;
	CLOSE cMfgP;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spMfgM_New(__MfgMNo VARCHAR)
RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;
	_spret			SPStatus%ROWTYPE;
	_found			BOOLEAN;

	_MfgMDate		DATE;
	_MfgNo			VARCHAR(15);
	_MfgStatus		VARCHAR(5);
	_MfgMNote		VARCHAR;

	_MfgMDNo		SMALLINT;

	_StockANo		VARCHAR;
	_StockANote		VARCHAR(255);
	_StockADNo		SMALLINT;

	_PerID			VARCHAR(5);
	_BranchID		VARCHAR(10);
	_DeptID     		VARCHAR(5);
   	_PrjID      		VARCHAR(5);
	_CrcID			VARCHAR;
	_ExcRate		DECIMAL(19,4);
	_FisRate		DECIMAL(19,4);


	_CrtBy			VARCHAR(10);
	_CrtDate		DATE;
	_UpdBy			VARCHAR(10);
	_UpdDate		DATE;

	_ItemID			VARCHAR(20);
	_ItemIDSub		VARCHAR(20);
	_WhID			VARCHAR(3);
	_PID			VARCHAR;
	_Qty			NUMERIC(15,4);	
	_QtyX			NUMERIC(15,4);	
	_Unit			VARCHAR(5);
	_Cost			NUMERIC(19,4);
	_ItemType		VARCHAR(5);
	_isStock		BOOLEAN;

	_tmpQty			NUMERIC(15,4);

	cMfgM			REFCURSOR;
	_tmp			VARCHAR;
	_tmp2			VARCHAR;
	_tmp3			INT;

	_BoMID			VARCHAR;
	_StdUnit		VARCHAR(5);
	_StdCost		DECIMAL(19,4);
	_TotCost		DECIMAL(19,4);
	_TotStdCost		DECIMAL(19,4);

	_BoMD_StdCost		DECIMAL(19,4);
	_BoMD_Unit		VARCHAR;
	_BoMD_StdQty		NUMERIC(15,4);


	_AccWIPInv		VARCHAR;
	_AccMatInv		VARCHAR;

	_Jnl1No			VARCHAR;
	_Jnl1Note		VARCHAR;
	_JnlD1No		SMALLINT;
	_JnlD1Amt		DECIMAL(19,4);
	_JnlD1AmtForex		DECIMAL(19,4);

	cMfgMD			REFCURSOR;

	_AccNo			VARCHAR;
	_AccType2		VARCHAR;	

	_RealCost		NUMERIC(19,4);
	_CostSubtotal		NUMERIC(19,4);

	_JnlWIPDNo		INT2;
	_JnlWIPAmt		NUMERIC(19,4);
BEGIN
/**
	_retval:=(0, 'OK');
	RETURN NEXT _retval;
	RETURN;

**/

	_TotCost:=0;
	_TotStdCost:=0;
	_CrcID:=fCrcDefault();
	_ExcRate:=0;
	_FisRate:=0;
	_DeptID:=NULL;
	_PrjID:=NULL;

	SELECT INTO 
		_MfgNo, _MfgMDate, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _MfgMNote
		MfgNo, MfgMDate, CrtBy, CrtDate, UpdBy, UpdDate, mfgmnote 
	FROM MfgM 
	WHERE MfgMNo=__MfgMNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Manufacturing Materials# ('||__MfgMNo||') tidak ditemukan (spMfgM_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;	

	-- baca data parent (mfg)
	SELECT INTO
		_MfgStatus, _PerID, _BranchID, _DeptID, _PrjID, _BomID, _AccWIPInv  
		mfgstatus, perid, branchid, deptid, prjid, bomid, accwip  
	FROM Mfg WHERE MfgNo=_MfgNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Produksi #'||_MfgNo||' tidak ditemukan (spMfgM_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;	

	IF (_MfgStatus='F') THEN
		RAISE EXCEPTION 'Status Produksi % sudah selesai, tidak diperkenankan penggunaan bahan untuk produksi ini (spMfgM_New)', _MfgNo;
	END IF;
	
	IF (isAccInstalled()) THEN
		SELECT INTO _AccNo, _AccType2 fItemAcc(itemid, 'INV'), acctype2 
		FROM mfgmd JOIN acc a ON a.accno=fItemAcc(itemid, 'INV') 
		WHERE mfgmno=__MfgMNo AND acctype2='530';  
		IF FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Akun inventori bahan baku produksi tidak boleh dalam tipe biaya produksi tidak langsung';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;  
	
	-- Generate Pengeluaran Bahan Produksi ---
	_StockADNo:=0;
	OPEN cMfgM FOR
		SELECT 
			d.MfgMDNo, d.ItemID, d.ItemIDSub,  i.itemtype, d.WhID, d.Qty, d.Unit, d.PID, d.QtyX, d.cost, i.isstock
		FROM MfgMD d JOIN Item i ON d.ItemID=i.ItemID
		--iSStock dihilangkan karena produksi ada item jasa		
		--WHERE d.MfgMNo=__MfgMNo AND i.IsStock=TRUE;
		WHERE d.MfgMNo=__MfgMNo
		ORDER BY d.MfgMDNo;
	LOOP
		FETCH cMfgM INTO _MfgMDNo, _ItemID, _ItemIDSub, _ItemType, _WhID, _Qty, _Unit, _PID, _QtyX, _RealCost, _isStock;
		EXIT WHEN NOT FOUND;		

		_StockADNo:=_StockADNo+1;


		---- Jika row pertama, maka Generate header!
		IF (_StockADNo=1) THEN
			_StockANo:=fTransNoGet('STOCKA');		
			_StockANote:='Bahan Produksi #'||_MfgNo;

			SELECT INTO _spret * FROM spStockA_New(
				_StockANo, NULL, 'MFGM', _MfgMDate, 'O', 'MFGM', __MfgMNo, _StockANote,
				TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate, _PerID, _BranchID, _DeptID, _PrjID
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMSg;
			END IF;
		END IF;

		---- get standard cost dari BoMD dengan item yang bersesuaian ---
		SELECT INTO 
			_BoMD_StdCost, _BoMD_Unit
			StdCost, Unit 
		FROM BoMD 
		WHERE BoMID=_BoMID AND ItemID=_ItemID;
		IF FOUND THEN
			_StdCost:=_BoMD_StdCost*fItemUnit_Conv(_ItemID, _Unit)/fItemUnit_Conv(_ItemID, _BoMD_Unit);
		ELSE
			_StdCost:=_RealCost;		
		END IF;
		_tmpQty:=_Qty*(-1);	-- ubah jadi minus -> potong stock
		IF (_ItemType='SERV' OR _isStock=FALSE) THEN
			--_Cost:=_StdCost; ganti ambil dari harga jual
			_Cost:=fPriceList(_ItemID, _Unit, '0', _CrcID);
		ELSE
			IF _ItemIDSub IS NULL THEN
				_CostSubTotal:=fUnitCost(_ItemID, _WhID, _PID, _Qty, _Unit);
				_Cost:=_CostSubTotal/_Qty;
			ELSE
				_CostSubTotal:=fUnitCost(_ItemIDSub, _WhID, _PID, _Qty, _Unit);
				_Cost:=_CostSubTotal/_Qty;
			END IF;
		END IF;

--RAISE NOTICE 'itemid:% whid:% pid:% unit:%', _ItemID, _WhID, _PID, _Unit;

		IF _ItemIDSub IS NULL THEN
			SELECT INTO _spret * FROM spStockAD_New(
				_MfgMDate, 'MFGM', _StockANo, _StockADNo, _WhID, 
				_ItemID, _PID, _tmpQty, _Unit, _QtyX*-1, _Cost, _DeptID, _PrjID, TRUE, _MfgNo, _CostSubTotal*-1);
		ELSE
			SELECT INTO _spret * FROM spStockAD_New(
				_MfgMDate, 'MFGM', _StockANo, _StockADNo, _WhID, 
				_ItemIDSub, _PID, _tmpQty, _Unit, _QtyX*-1, _Cost, _DeptID, _PrjID, TRUE, _MfgNo, _CostSubTotal*-1);
		END IF;

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;


		---- get standard cost dari BoMD dengan item yang bersesuaian ---
		SELECT INTO 
			_BoMD_StdQty, _BoMD_StdCost, _BoMD_Unit
			Qty, StdCost, Unit 
		FROM BoMD 
		WHERE BoMID=_BoMID AND ItemID=_ItemID;

		_StdCost:=_BoMD_StdCost;

		_TotCost:=_TotCost+(_Qty*_Cost);
		_TotStdCost:=_TotStdCost+_StdCost;


		UPDATE MfgMD SET 
			Cost=_Cost,
			StdCost=_StdCost,
			StdQty=_BoMD_StdQty,
			StdUnit=_BoMD_Unit
		WHERE MfgMNo=__MfgMNo AND MfgMDNo=_MfgMDNo;

	END LOOP;
	CLOSE cMfgM;	

/********************************************* ACCOUNTING ******************************************/

	------------------- Accounting here -------------------------
	IF (isAccInstalled()) THEN
		_Jnl1No:=fTransNoGet('JURNAL');
		_Jnl1Note:='Bahan Produksi #'||_MfgNo||COALESCE(':'||_MfgMNote, '');
		------- Generate Jurnal Header untuk JurnalVoucher(JV) Penjualan
		SELECT INTO _spret * FROM spJnl_Insert(
			_Jnl1No, 'MFGM', _MfgMDate, 'MFGM', __MfgMNo,
			_Jnl1Note, TRUE, TRUE, CAST(0 AS NUMERIC(19,4)),
			_CrtBy, _CrtDate, _UpdBy, _UpdDate,
			_PerID, _BranchID, _DeptID, _PrjID, _CrcID, _ExcRate, _FisRate				
		);

		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;


		_JnlD1No:=0;
		------- Jurnal Ayat WIP debet-------------
		IF (_AccWIPInv IS NULL) THEN
			_AccWIPInv:=fAccDefault('WIPINVENTORY');
		END IF;

		/** Last Checking */
		IF (_AccWIPInv IS NULL) THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Akun WIP belum diset atau setting default akun WIP belum diset ! '||E'\n'||'Silahkan mensetting akun tersebut di master produksi#'||_MfgNo||' atau form setting default akun (spMfgM_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		
		_JnlD1No:=_JnlD1No+1;
		_JnlWIPDNo:=_JnlD1No; /** tampung dulu untuk dipakai nanti */		
		_JnlWIPAmt:=0;		
	
		------- Jurnal Ayat Material/Bahan: kredit -------------
		OPEN cMfgMD FOR
			SELECT fItemAcc(ItemID, 'INV'), SUM(Qty*Cost) FROM MfgMD
			WHERE MfgMNo=__MfgMNo
			GROUP BY fItemAcc(ItemID, 'INV');
		LOOP
			FETCH cMfgMD INTO _AccMatInv, _JnlD1Amt;
			EXIT WHEN (NOT FOUND OR _AccMatInv IS NULL OR _AccMatInv='');

			_JnlWIPAmt:=_JnlWIPAmt+_JnlD1Amt;
			_JnlD1AmtForex:=_JnlD1Amt;

			_JnlD1No:=_JnlD1No+1;		
			SELECT INTO _spret * FROM spJnlD_Insert(_Jnl1No, _JnlD1No, _AccMatInv, 'K', _JnlD1Amt, _DeptID, _PrjID, _JnlD1AmtForex);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;
		END LOOP;

		_JnlD1AmtForex:=_JnlWIPAmt;		
		SELECT INTO _spret * FROM spJnlD_Insert(_Jnl1No, _JnlWIPDNo, _AccWIPInv, 'D', _JnlWIPAmt, _DeptID, _PrjID, _JnlD1AmtForex);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		--- trigger posting jurnal
		SELECT INTO _spret * FROM spJnl_New(_Jnl1No);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	END IF;			-- isAccInstalled()


	UPDATE Mfg SET 
		TotCost=_TotCost,
		TotStdCost=_TotStdCost,
		MfgStatus='P'
	WHERE MfgNo=_MfgNo;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;





CREATE OR REPLACE FUNCTION spMfgM_Void (
	__MfgMNo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval	SPStatus%ROWTYPE;
	_spret	SPStatus%ROWTYPE;

	_MfgNo		VARCHAR;
	_MfgStatus	VARCHAR;
BEGIN

	SELECT INTO
		_MfgNo, _MfgStatus
		m.MfgNo, h.MfgStatus
	FROM MfgM m JOIN Mfg h ON m.MfgNo=h.MfgNo
	WHERE MfgMNo=__MfgMNo;

	IF (_MfgStatus='F') THEN
		RAISE EXCEPTION 'Produksi #% sudah selesai, tidak boleh Void Bahan Produksi. Buka dulu kembali Produksi (spMfgM_Void)', _MfgNo;
	END IF;

	SELECT INTO _spret * FROM spStockA_Void_ByRef('MFGM', __MfgMNo);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=''||_spret.ErrMsg|| '(spMfgM_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;

	SELECT INTO _spret * from spJnl_Void_byRef('MFGM', __MfgMNo);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	DELETE FROM mfgmd WHERE mfgmno = __MfgMNo ;
	DELETE FROM mfgm WHERE mfgmno = __MfgMNo ;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spMfgP_New (
	__MfgPNo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_MfgPDate	DATE;
	_MfgNo		VARCHAR(15);
	_StockANo	VARCHAR;
	_StockANote	VARCHAR(255);
	_StockADNo	SMALLINT;
	_PerID		VARCHAR(5);
	_BranchID	VARCHAR(10);
	_DeptID		VARCHAR(5);
	_PrjID		VARCHAR(5);
	_CrtBy		VARCHAR(10);
	_CrtDate	DATE;
	_UpdBy		VARCHAR(10);
	_UpdDate	DATE;

	_ItemID		VARCHAR(20);
	_WhID		VARCHAR(3);
	_PID		VARCHAR;
	_Qty		NUMERIC(15,4);
	_QtyX		NUMERIC(15,4);
	_Unit		VARCHAR(5);
	_Cost		NUMERIC(19,4);
	_IsMainPrd	BOOLEAN;

	cMfgP		REFCURSOR;
	_isStock	BOOLEAN;

	_AccNo		VARCHAR;
	_AccType2	VARCHAR;
BEGIN

	SELECT INTO 
		_MfgNo, _MfgPDate, _CrtBy, _CrtDate, _UpdBy, _UpdDate
		MfgNo, MfgPDate, CrtBy, CrtDate, UpdBy, UpdDate
	FROM MfgP WHERE MfgPNo=__MfgPNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Hasil Produksi #'||__MfgPNo||' tidak ditemukan ! (spMfgP_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO 
		_PerID, _BranchID, _DeptID, _PrjID
		PerID, BranchID, DeptID, PrjID
	FROM Mfg WHERE MfgNo=_MfgNo;

	IF NOT FOUND THEN
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Hasil Produksi #'||COALESCE(_MfgNo, '<kosong>')||' tidak ditemukan ! (spMfgP_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (isAccInstalled()) THEN
		SELECT INTO _AccNo, _AccType2 fItemAcc(itemid, 'INV'), acctype2 
		FROM mfgpd 
		JOIN acc a ON a.accno=fItemAcc(itemid, 'INV') 
		WHERE mfgpno=__MfgPNo AND acctype2='530';  
		IF FOUND THEN
			_retval.ErrNo:=50555;
			_retval.ErrMsg:='Akun inventori ouput produksi tidak boleh dalam tipe biaya produksi tidak langsung ! (spMfgP_New)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;  
/**
	_StockADNo=0;
	OPEN cMfgP FOR
		SELECT 
			d.ItemID, d.WhID, d.Qty, d.Unit, d.IsMainPrd, d.PID, d.QtyX,d.Cost, i.isStock
		FROM MfgPD d JOIN Item i ON d.ItemID=i.ItemID
		WHERE d.MfgPNo=__MfgPNo;
	LOOP
		FETCH cMfgP INTO _ItemID, _WhID, _Qty, _Unit, _IsMainPrd, _PID, _QtyX, _Cost, _isStock;
		EXIT WHEN NOT FOUND;

		_StockADNo:=_StockADNo+1;

		IF (_StockADNo=1) THEN
			_StockANo:=fTransNoGet('STOCKA');
			_StockANote:='Hasil untuk Produksi #'||_MfgNo;

			SELECT INTO _spret * FROM spStockA_New(
				_StockANo, NULL, 'MFGP', _MfgPDate, 'I', 'MFGP', __MfgPNo, _StockANote,
				TRUE, _CrtBy, _CrtDate, _UpdBy, _UpdDate,
				 _PerID, _BranchID, _DeptID, _PrjID
			);
			IF _spret.ErrNo<>0 THEN
				_retval.ErrNo:=_spret.ErrNo;
				_retval.ErrMsg:=_spret.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMSg;
			END IF;
		END IF;

		--Sekarang Cost diisi oleh hpp--
		--_Cost:=0;

		SELECT INTO _spret * FROM spStockAD_New(
			_MfgPDate, 'MFGP', _StockANo, _StockADNo, _WhID, 
			_ItemID, _PID, _Qty, _Unit, _QtyX, _Cost, _DeptID, _PrjID, TRUE);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

	END LOOP;
	CLOSE cMfgP;
**/

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spMfgP_Void (
	__MfgPNo	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_spret		SPStatus%ROWTYPE;

	_MfgNo		VARCHAR;
	_MfgStatus	VARCHAR;
BEGIN
	SELECT INTO
		_MfgNo, _MfgStatus
		p.MfgNo, h.MfgStatus
	FROM MfgP p JOIN Mfg h ON p.MfgNo=h.MfgNo
	WHERE MfgPNo=__MfgPNo;

	IF (_MfgStatus='F') THEN
		RAISE EXCEPTION 'Produksi #% sudah selesai, tidak boleh Void Hasil Produksi. Buka dulu kembali Produksi (spMfgP_Void)', _MfgNo;
	END IF;

/**
	-- void transasksi stock
	SELECT INTO _spret * FROM spStockA_Void_ByRef('MFGP', __MfgPNo);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=''||_spret.ErrMsg|| '(spMfgP_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;	
	END IF;

	--- void jurnal
	SELECT INTO _spret * from spJnl_Void_byRef('MFGP', __MfgPNo);
	IF (_spret.ErrNo<>0) THEN
		_retval.ErrNo:=_spret.ErrNo;
		_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
**/

	-- hapus tabel2
	DELETE FROM mfgpd WHERE mfgpno = __MfgPNo ;
	DELETE FROM mfgp WHERE mfgpno = __MfgPNo ;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fPossSaldoSRet(
	_PossesNo	VARCHAR
) RETURNS DECIMAL(19,4) AS
$BODY$
DECLARE
	_totalSRet DECIMAL(19,4);
BEGIN
	SELECT INTO _totalSRet COALESCE(sum((sr.qty*(sr.baseprice-sr.discamt-sr.disc2amt))+sr.totaltaxamt),0) as totalsret 
	FROM sretd sr
	JOIN sale s ON s.saleno=sr.saleno
	WHERE s.possesno=_PossesNo;

	IF NOT FOUND THEN
		_totalSRet:=0;
	END IF;

	RETURN _totalSRet;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tWIPUsed CASCADE;
CREATE TYPE tWIPUsed AS (
	itemid		VARCHAR,
	pid		VARCHAR,
	itemdesc	VARCHAR,
	cost		NUMERIC(19,4)
); 


CREATE OR REPLACE FUNCTION fWIPUsed (
	_MfgList	VARCHAR,
	_Delimiter	VARCHAR
) RETURNS SETOF tWIPUsed AS
$BODY$
DECLARE
	_retval		tWIPUsed%ROWTYPE;

	_MfgArray	VARCHAR[];
	_Counter	INT;

	_LastMfgNo	VARCHAR;
	_MfgNo		VARCHAR;

	cOutput		REFCURSOR;
	cInput		REFCURSOR;

	_OutItemID	VARCHAR;
	_OutPID		VARCHAR;
	_OutQty		NUMERIC(15,4);
	_OutUnit	VARCHAR;
	_OutCost	NUMERIC(19,4);

	_Out1Qty	NUMERIC(15,4);
	_Out1Cost	NUMERIC(19,4);
	_Out1Conv	NUMERIC(15,4);

	_InItemID	VARCHAR;
	_InPID		VARCHAR;
	_InQty		NUMERIC(15,4);
	_InUnit		VARCHAR;
	_InCost		NUMERIC(19,4);

	_In1Qty		NUMERIC(15,4);
	_In1Cost	NUMERIC(19,4);
	_In1Conv	NUMERIC(15,4);

	_WIPCost	NUMERIC(19,4);
	_DiffQty	NUMERIC(15,4);
BEGIN
	/** Inisialisasi */
	_LastMfgNo:=NULL;

	_MfgArray:=STRING_TO_ARRAY(_MfgList, _Delimiter);

	_Counter:=ARRAY_LOWER(_MfgArray, 1);
	WHILE (_Counter <= ARRAY_UPPER(_MfgArray, 1)) LOOP
		_MfgNo:=_MfgArray[_Counter];

		IF (_LastMfgNo IS NULL) THEN
			_LastMfgNo:=_MfgNo;
		ELSE
			OPEN cOutput FOR
				SELECT d.itemid, d.pid, d.qty, d.unit, d.cost 
				FROM mfgpd d JOIN mfgp m ON d.mfgpno=m.mfgpno
				WHERE m.mfgno=_LastMfgNo;  
			LOOP
				FETCH cOutput INTO _OutItemID, _OutPID, _OutQty, _OutUnit, _OutCost;
				EXIT WHEN NOT FOUND;

				_Out1Qty:=fQtyZToQty1(_OutItemID, _OutQty, _OutUnit);
				_Out1Conv:=fItemUnit_Conv(_OutItemID, _OutUnit);
				_Out1Cost:=_OutCost/_Out1Conv;

				OPEN cInput FOR
					SELECT d.itemid, d.pid, d.qty, d.unit, d.cost 
					FROM mfgmd d JOIN mfgm m ON d.mfgmno=m.mfgmno
					WHERE m.mfgno=_MfgNo AND d.itemid=_OutItemID AND d.pid=_OutPID;
				LOOP
					FETCH cInput INTO _InItemID, _InPID, _InQty, _InUnit, _InCost;
					EXIT WHEN NOT FOUND;

					_In1Qty:=fQtyZToQty1(_InItemID, _InQty, _InUnit);
					_In1Conv:=fItemUnit_Conv(_InItemID, _InUnit);
					_In1Cost:=_InCost/_In1Conv;

					IF (_In1Qty>_Out1Qty) THEN
						_retval.itemid:=_InItemID;
						_retval.pid:=_InPID;
						_retval.itemdesc:=fItemDesc(_retval.itemid);
						IF (_Out1Cost<>_In1Cost) THEN
							_DiffQty:=_In1Qty-_Out1Qty;
							_WIPCost:=((_In1Cost*(_Out1Qty+_DiffQty))-(_Out1Qty*_Out1Cost))/(_Out1Qty+_DiffQty);
							_retval.cost:=_DiffQty*_WIPCost;
						ELSE
							_retval.cost:=(_In1Qty-_Out1Qty)*_Out1Cost;
						END IF;

						RETURN NEXT _retval;
					END IF;
				END LOOP;
				CLOSE cInput;
			END LOOP;
			CLOSE cOutput;

			_LastMfgNo:=_MfgNo;
		END IF;

		_Counter:=_Counter+1;
	END LOOP;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fGetRoute(
	_ItemID 		VARCHAR,
	_PID 			VARCHAR
) RETURNS SETOF VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;

	_NoUrut		INT;

	cData		REFCURSOR;
BEGIN
	_NoUrut:=0;
	
	OPEN cData FOR 
		SELECT * FROM fGetRoute(_ItemID, _PID, _NoUrut);
	LOOP
		FETCH cData INTO _retval;
		EXIT WHEN NOT FOUND;
		
		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;



CREATE OR REPLACE FUNCTION fGetRoute(
	_ItemID 		VARCHAR,
	_PID 			VARCHAR,
	_NoUrut			INT
) RETURNS SETOF VARCHAR AS 
$BODY$
DECLARE
	_ProcsID		VARCHAR(15);
	_ProcsIDx		VARCHAR(15);
	_BOMID			VARCHAR(15);
	_ItemID_BOM		VARCHAR(20);
	_PID_BOM		VARCHAR(25);
	_NoUrutBaru		INT;
	_Count			INT;

	cWO			REFCURSOR;
	cData			REFCURSOR;
BEGIN
	OPEN cWO FOR
		SELECT
			p.procsid, pb.bomid
		FROM rou r
		JOIN roud rd ON rd.rouid=r.rouid
		JOIN procs p ON p.procsid=rd.procsid
		JOIN procsbom pb ON pb.procsid=p.procsid
		WHERE r.itemid=_ItemID AND r.pid=_PID AND r.active=TRUE
		ORDER BY rd.roudno;
	LOOP
		FETCH cWO INTO _ProcsID, _BOMID;

		EXIT WHEN NOT FOUND;

		SELECT INTO _ItemID_BOM, _PID_BOM itemid, pid FROM bom WHERE bomid=_BOMID;
		
		SELECT INTO _Count count(*) FROM rou WHERE itemid=_ItemID_BOM AND pid=_PID_BOM;

		_NoUrutBaru:=_NoUrutBaru+1;

		IF _Count>0 AND _ItemID_BOM<>_ItemID THEN
			OPEN cData FOR
				SELECT * FROM fGetRoute(_ItemID_BOM, _PID_BOM, _NoUrutBaru);
			LOOP
				FETCH cData INTO _ProcsIDx;
				EXIT WHEN NOT FOUND;
				
				RETURN NEXT _ProcsIDx;
			END LOOP;
			CLOSE cData;
			
		ELSE
			RETURN NEXT _ProcsID;
		END IF;

	END LOOP;
	CLOSE cWO;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fWOCard (
	_WONo 		VARCHAR
) RETURNS SETOF wocard AS
$BODY$
DECLARE
	_retval			wocard%ROWTYPE;
	_ItemID			VARCHAR(20);
	_PID			VARCHAR(25);
	/*
	WONo			wo.wono%TYPE,
	WODate			wo.wodate%TYPE,
	BPName			bp.bpname%TYPE,
	WOType			wo.wotype%TYPE,
	SONo			wo.sono%TYPE,
	Active			wo.active%TYPE,
	ItemID			wod.itemid%TYPE,
	ItemDesc		item.itemdesc%TYPE,
	JobCardNo		jobcard.jobcardno%TYPE,
	JobCardDate		jobcard.jobcarddate%TYPE,
	JobNote			jobcard.jobnote%TYPE,
	Used			jobcard.used%TYPE,
	RouName			rou.rouname%TYPE,
	ProcsName		procs.procsname%TYPE,
	BomName			bom.bomname%TYPE,
	WorkDeptName		workdept.workdeptname%TYPE,
	WorkcName		workc.workcname%TYPE,
	WOTypeDesc		wotype.wotypedesc%TYPE,
	MfgNo			mfg.mfgno%TYPE,
	MfgDate			mfg.mfgdate%TYPE,
	EmpName			emp.empname%TYPE,
	MachName		mach.machname%TYPE,
	MfgStatusDesc		mfgstatus.mfgstatusdesc%TYPE,
	SaleNo			sale.saleno%TYPE
	*/

	cWO			REFCURSOR;
	cCard			REFCURSOR;

BEGIN
	OPEN cWO FOR
		SELECT wd.itemid, wd.pid
		FROM wo w
		JOIN wod wd ON wd.wono=w.wono
		WHERE w.wono=_WONo;
	LOOP
		FETCH cWO INTO _ItemID, _PID;

		EXIT WHEN NOT FOUND;

		OPEN cCard FOR
			SELECT w.wono, w.wodate, b.bpname, w.wotype, w.sono, w.active, wd.itemid, i.itemdesc, j.jobcardno, 
					j.jobcarddate, j.jobnote, j.used, r.rouname, p.procsname, bm.bomname, wp.workdeptname, 
					wc.workcname, wt.wotypedesc, m.mfgno, m.mfgdate, e.empname, mc.machname, ms.mfgstatusdesc, sd.saleno
			FROM jobcard j
			JOIN rou r ON r.rouid=j.rouid
			JOIN procs p ON p.procsid=j.procsid
			JOIN bom bm ON bm.bomid=j.bomid
			LEFT JOIN workdept wp ON wp.workdeptid=j.workdeptid
			JOIN workc wc ON wc.workcid=j.workcid
			JOIN wo w ON w.wono=j.wono
			JOIN wod wd ON wd.wono=w.wono
			JOIN item i ON i.itemid=wd.itemid
			LEFT JOIN wotype wt ON wt.wotype=w.wotype
			LEFT JOIN mfg m ON m.jobcardno=j.jobcardno
			LEFT JOIN emp e ON e.empid=m.empid
			LEFT JOIN mach mc ON mc.machid=m.machid
			LEFT JOIN mfgstatus ms ON ms.mfgstatus=m.mfgstatus
			LEFT JOIN so s ON s.sono=w.sono
			LEFT JOIN saled sd ON sd.sono=s.sono
			LEFT JOIN bp b ON b.bpid=w.custid
			WHERE j.wono=_WONo
			ORDER BY j.JOBCARDNO;
		LOOP
			/*_WONo, _WODate, _BPName, _WOType, _SONo, _Active, _JobCardNo, _JobCardDate, _JobNote, _Used,
			_RouName, _ProcsName, _BomName, _WorkDeptName, _WorkcName, _WOTypeDesc, _MfgNo, _MfgDate, 
			_EmpName, _MachName, _MfgStatusDesc, _SaleNo;*/

			FETCH cCard INTO _retval;
			EXIT WHEN NOT FOUND;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cCard;
	END LOOP;
	CLOSE cWO;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fWOCardMain(
	_StartDate 		DATE,
	_EndDate 		DATE,
	_WONo 			VARCHAR,
	_SONo 			VARCHAR,
	_SaleNo			VARCHAR,
	_CustID			VARCHAR
) RETURNS SETOF wocard AS
$BODY$
DECLARE
	_retval		wocard%ROWTYPE;
	__WONo		VARCHAR(15);
	cWO		REFCURSOR;
	cCard		REFCURSOR;
BEGIN
	OPEN cWO FOR
		SELECT
			w.wono
		FROM wo w
		LEFT JOIN so ON so.sono=w.sono
		LEFT JOIN saled sd ON sd.sono=so.sono
		WHERE w.wodate>=_StartDate AND w.wodate<=_EndDate AND
			(_WONo IS NULL OR w.wono=_WONo) AND 
			(_CustID IS NULL OR w.custid=_CustID) AND 
			(_SONo IS NULL OR w.sono=_SONo) AND 
			(_SaleNo IS NULL OR sd.saleno=_SaleNo);
	LOOP
		FETCH cWO INTO __WONo;
		EXIT WHEN NOT FOUND;

		OPEN cCard FOR
			SELECT * FROM fWOCard(__WONo);
		LOOP
			FETCH cCard INTO _retval;
			EXIT WHEN NOT FOUND;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cCard;

	END LOOP;
	CLOSE cWO;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE Function spRou_New (
	_RouID		VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE	
	_retval		SPStatus%ROWTYPE;

	_ItemID		VARCHAR;
	_ItemID2	VARCHAR;
	_PID		varchar;
BEGIN

	SELECT INTO _ItemID,_PID itemid,pid FROM rou WHERE rouid=_RouID;
	IF (NOT FOUND OR _ItemID IS NULL) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Routing# '||_RouID||' tidak ditemukan (spRou_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF; 

	SELECT INTO _ItemID2 itemid FROM rou 
	WHERE rouid<>_RouID 
	AND itemid=_ItemID
	AND pid=_PID
	AND Active=TRUE;

	IF (_ItemID2=_ItemID) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Ada Routing lain yang masih Aktif. In-Aktifkan Routing yang lama terlebih dahulu !!';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;
	
	_retval.ErrNo:=0;
	_retval.ErrMsg:='OK';
	RETURN NEXT _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDP_New(
	_BPID		VARCHAR,
	_DPDate		DATE,
	_RefNo		VARCHAR,
	_RefType	VARCHAR,
	_CrcID		VARCHAR,
	_ExcRate	NUMERIC(19,4),
	_FisRate	NUMERIC(19,4),
	_DPAmt		NUMERIC(19,4),
	_AccDP		VARCHAR,
	_PaidNo		VARCHAR,
	_PaidType	VARCHAR
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_DPAmtBC	NUMERIC(19,4);
BEGIN
	_DPAmtBC:=_DPAmt*_ExcRate;	

	INSERT INTO dp 
		(bpid, dpdate, dpid, refno, reftype, crcid, excrate, fisrate, dpamt, usedamt, dpbal, dpamtbc, usedamtbc, dpbalbc, accdp, paidno, paidtype)
	VALUES 
		(_BPID, _DPDate, nextval('dpidgenerator'), _RefNo, _RefType, _CrcID, _ExcRate, _FisRate, _DPAmt, 0, _DPAmt, _DPAmtBC, 0, _DPAmtBC, _AccDP, _PaidNo, _PaidType);
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Gagal buat down payment (spDP_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDP_Void(
	_DPID	BIGINT
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_RefNo		VARCHAR;
BEGIN

	SELECT INTO _RefNo payno FROM ppaid WHERE refno=CAST(_DPID AS VARCHAR);
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Uang Muka#'||_DPID||' sudah digunakan dalam Pembayaran#'||_RefNo||' (spDP_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RefNo rcvno FROM spaid WHERE refno=CAST(_DPID AS VARCHAR);
	IF FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Uang Muka#'||_DPID||' sudah digunakan dalam Penerimaan Pembayaran#'||_RefNo||' (spDP_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	DELETE FROM dp WHERE dpid=_DPID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Uang Muka#'||_DPID||' tidak ditemukan (spDP_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_retval:=(0, 'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spDPBal_Update(
	_DPID		BIGINT,
	_AdjAmt		NUMERIC(19,4),
	_AdjAmtBC	NUMERIC(19,4)
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_DPBal		NUMERIC(19,4);
	_DPBalBC	NUMERIC(19,4);
BEGIN

	SELECT INTO _DPBal, _DPBalBC dpbal, dpbalbc FROM dp WHERE dpid=_DPID;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Pembayaran Uang Muka#'||COALESCE(_DPID, -1)||' tidak ditemukan (spDPBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	_DPBal:=COALESCE(_DPBal, 0)+COALESCE(_AdjAmt, 0);
	_DPBalBC:=COALESCE(_DPBalBC, 0)+COALESCE(_AdjAmtBC, 0);

	IF (_DPBal<0) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Saldo Uang Muka#'||COALESCE(_DPID, -1)||' sudah habis digunakan (spDPBal_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	UPDATE dp SET dpbal=_DPBal, dpbalbc=_DPBalBC, usedamt=usedamt-COALESCE(_AdjAmt, 0), usedamtbc=usedamtbc-COALESCE(_AdjAmtBC, 0)  
	WHERE dpid=_DPID;

	_retval:=(0,'OK');
	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spProfile_New (
	_cmpname	VARCHAR,
	_cmpaddr	VARCHAR,
	_zipcode	VARCHAR,
	_cityid		VARCHAR,
	_stateid	VARCHAR,
	_phone		VARCHAR,
	_fax		VARCHAR,
	_fiscalyear	INT,
	_formserno	VARCHAR,
	_taxregno	VARCHAR,
	_vatregno	VARCHAR,
	_vatregdate	DATE,
	_startdate	DATE,
	_crcid		VARCHAR,
	_enddate	DATE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;
	_PYear		INT;
	_PerID		VARCHAR;
	_PerNo		INT;
BEGIN
	UPDATE cmp SET cmpname=_cmpname, cmpaddr=_cmpaddr, zipcode=_zipcode, 
	cityid=_cityid, stateid=_stateid, phone=_phone, fax=_fax, 
	fiscalyear=_fiscalyear, formserno=_formserno, taxregno=_taxregno, vatregno=_vatregno,
	vatregdate=_vatregdate, startdate=_startdate, crcid=_crcid;

	UPDATE crc SET isdefault=true WHERE crcid=_crcid;

	UPDATE crc SET isdefault=false WHERE crcid<>_crcid;

	SELECT INTO _retval * FROM spPer_Open(_startdate, (SELECT CAST(_enddate + INTERVAL '1 month'- INTERVAL '1 day' AS DATE)));

	_retval:=(0, 'OK');
	RETURN NEXT _retval;	
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spSAdj_Reset(_date Date)
	RETURNS SETOF spstatus AS
$BODY$
DECLARE
	_retval SPStatus%ROWTYPE;
	_spret SPStatus%ROWTYPE;
	_SAdjNo VARCHAR;
	cSAdj REFCURSOR;
BEGIN
	OPEN cSAdj FOR
		SELECT SAdjNo 
		FROM SAdj
		WHERE SAdjDate=_date;
		LOOP 
		FETCH cSAdj INTO _SAdjNo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _spret * FROM spSAdj_Void(_SAdjNo);
		IF _spret.ErrNo<>0 THEN
			_retval.ErrNo:=_spret.ErrNo;
			_retval.ErrMsg:=_spret.ErrMsg;
		RAISE EXCEPTION '%', _retval.ErrMSg;
		END IF;

		END LOOP; 
	CLOSE cSAdj;

	_retval.ErrNo:=0;
	_retval.ErrMsg:='ALL DELETE SUCCESS!!';

	RETURN NEXT _retval;

END;
$BODY$
  LANGUAGE 'plpgsql' VOLATILE;


CREATE OR REPLACE FUNCTION spStockAD_New_BoM_SRet(
	_StockADate 		DATE, 
	_StockAType		VARCHAR(5),
	_StockANo 		VARCHAR(15), 
	_StockADNo 		SMALLINT,
	_WhID 			VARCHAR(3), 
	_ItemID 		VARCHAR(20), 
	_PID 			VARCHAR(25), 
	_Qty 			DECIMAL(15,4),
	_Unit 			VARCHAR, 
	_QtyX 			DECIMAL(15,4), 
	_Cost 			DECIMAL(19,4),
	_DeptID			VARCHAR(5),
	_PrjID			VARCHAR(5),
	_CrcID			VARCHAR,
	_SaleNo			VARCHAR,
	_SaleDNo		SMALLINT,
	_SRetNo			VARCHAR
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval 			SPStatus%ROWTYPE;
	_spretStockAD_New_BoM_SRet	SPStatus%ROWTYPE;
	_spretStockAD_New 		SPStatus%ROWTYPE;

	_ItemType			VARCHAR(5);
	_IsAsm				BOOLEAN;
	_BoMID				VARCHAR(20);
	_IsStock			BOOLEAN;

	_BoMD_ItemID			VARCHAR(20);
	_BoMD_WhID			VARCHAR(3);
	_BoMD_Qty			DECIMAL(15,4);
	_BoMD_Unit			VARCHAR(5);
	_BoMD_PID			VARCHAR(25);

	_BoM_Mul			DECIMAL(15,4);
	_tmpQty				DECIMAL(15,4);
	_tmpQtyX			DECIMAL(15,4);
	_tmpCost			DECIMAL(19,4);
	cBoMD				REFCURSOR;
	__StockADNo 			SMALLINT;
BEGIN
	__StockADNo:=_StockADNo;
	_retval:=(0, 'OK');

	SELECT INTO 
		_ItemType, _IsAsm, _IsStock 
		itemtype, isasm, isstock 
	FROM item WHERE itemid=_ItemID;

	IF NOT FOUND THEN 
		_retval.ErrNo:=50001;
		_retval.ErrMsg:='Error ItemID:'||COALESCE(_ItemID, '<kosong>')||' tidak ditemukan ! (spStockAD_New_BoM_SRet)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF ((_ItemType='PACK') OR (_IsAsm=TRUE)) AND (_IsStock=FALSE) THEN
		SELECT INTO _BoMID bomid FROM bom WHERE itemid=_ItemID;
		IF NOT FOUND THEN
			_retval.ErrNo:=50001;
			_retval.ErrMsg:='Error B.o.M. untuk ItemID:'||COALESCE(_ItemID, '<kosong>')||' tidak ditemukan ! (spStockAD_New_BoM_SRet)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		SELECT INTO _BoM_Mul * FROM fQtyZToQty1(_ItemID, 1, _Unit);
		IF (_BoM_Mul IS NULL) THEN
			_retval.ErrNo:=50002;
			_retval.ErrMsg:='Error konversi Item:'||COALESCE(_ItemID, '<kosong>')||' Unit:'||COALESCE(_Unit, '<kosong>')||' ke Unit1 (spStockAD_New_BoM_SRet)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;

		OPEN cBoMD FOR
			SELECT d.itemid, d.whid, d.qty, d.unit, d.pid, i.itemtype, i.isstock 
			FROM bomd d JOIN item i ON d.itemid=i.itemid 
			WHERE bomid=_BoMID;
		LOOP
			FETCH cBoMD INTO _BoMD_ItemID, _BoMD_WhID, _BoMD_Qty, _BoMD_Unit, _BoMD_PID, _ItemType, _IsStock;
			EXIT WHEN NOT FOUND;

			_tmpQty:=_Qty*_BoM_Mul*_BomD_Qty;
			_tmpQtyX:=0;

			IF (_ItemType='SERV' OR _IsStock=FALSE) THEN
				_tmpCost:=fPriceList(_BoMD_ItemID, _BoMD_Unit, '0', _CrcID);
			ELSE
				SELECT INTO _tmpCost COALESCE(cost,0)
				FROM StockA s, StockAD d 
				WHERE s.StockANo=d.StockANo AND d.ItemID=_BoMD_ItemID AND d.WhID=_WhID AND (_BoMD_PID IS NULL OR d.PID=_BoMD_PID) AND s.RefType='SALE' AND s.RefNo=_SaleNo;
			END IF;

			SELECT INTO _spretStockAD_New_BoM_SRet * FROM spStockAD_New_BoM_SRet (
				_StockADate, _StockAType, _StockANo, __StockADNo, _WhID, _BoMD_ItemID,
			_BoMD_PID, _tmpQty, _BoMD_Unit, _tmpQtyX, _tmpCost, _DeptID, _PrjID, _CrcID, _SaleNo, _SaleDNo, _SRetNo
			); 

			__StockADNo:=_spretStockAD_New_BoM_SRet.ErrNo;

			IF _spretStockAD_New_BoM_SRet.ErrNo<>0 AND _spretStockAD_New_BoM_SRet.ErrMsg<>'OK' THEN
				_retval.ErrNo:=_spretStockAD_New_BoM_SRet.ErrNo;
				_retval.ErrMsg:=_spretStockAD_New_BoM_SRet.ErrMsg;
				RAISE EXCEPTION '%', _retval.ErrMsg;
			END IF;

			--__StockADNo:=__StockADNo+1;
--			__StockADNo:=_spretStockAD_New_BoM_SRet.ErrNo;
		END LOOP;		
		CLOSE cBoMD;
	ELSIF (_IsStock=true) THEN
		_tmpQty=_Qty;
		_tmpQtyX=_QtyX;
		__StockADNo:=__StockADNo+1;

		SELECT INTO _spretStockAD_New * FROM spStockAD_New(
			_StockADate, _StockAType, _StockANo, __StockADNo, 
			_WhID, _ItemID, _PID, _tmpQty, _Unit, _tmpQtyX, 
			_Cost, _DeptID, _PrjID, true
		);
		
		IF _spretStockAD_New.ErrNo<>0 THEN 
			_retval.ErrNo:=_spretStockAD_New.ErrNo;
			_retval.ErrMsg:=_spretStockAD_New.ErrMsg;
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
		
	END IF;

	_retval.ErrNo:=__StockADNo;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;





DROP TYPE tJobCardM CASCADE;
CREATE TYPE tJobCardM AS (
	numberrow	VARCHAR,
	jobcardmdno	SMALLINT,
	itemid		VARCHAR,
	itemdesc	VARCHAR,
	pid		VARCHAR,
	qty		DECIMAL(19,4),
	unit		VARCHAR,
	qtydesc		VARCHAR	
);


CREATE OR REPLACE FUNCTION fJobCardM(_JobCardNo VARCHAR,
	__MaxRow	SMALLINT
) RETURNS SETOF tJobCardM AS
$BODY$
DECLARE
	_retval			tJobCardM%ROWTYPE;

	_Counter		SMALLINT;
	_EmptyRow		SMALLINT;
	_PageCounter		SMALLINT;
	
	cQuery			REFCURSOR;
BEGIN
	/** Inisialisasi */
	_Counter:=0;
	_PageCounter:=1;

	OPEN cQuery FOR
		SELECT jobcardmdno,itemid,itemdesc,pid,qty,unit,fQty1Desc(itemid, fQtyZToQty1(itemid, qty, unit)) AS qtydesc FROM jobcardm 
		WHERE jobcardno=_JobCardNo ORDER BY jobcardmdno;
	LOOP
		FETCH cQuery INTO _retval.jobcardmdno,_retval.itemid,_retval.itemdesc,_retval.pid,_retval.qty,_retval.unit,_retval.qtydesc;
		EXIT WHEN NOT FOUND;
		_Counter=_Counter+1;
		/**_retval.page = _PageCounter;
		_retval.pagegroup = _PageCounter;*/

		RETURN NEXT _retval;
/*		IF ((_Counter / __MaxRow) >=1 AND (_Counter % __MaxRow) = 0 ) THEN
			_PageCounter = _PageCounter + 1;
		END IF;*/
	END LOOP;
	CLOSE cQuery;

	_EmptyRow = _Counter%__MaxRow;

	FOR i IN 1..(__MaxRow-_EmptyRow) LOOP
		_retval.numberrow:=NULL;
		_retval.jobcardmdno:=NULL;
		_retval.itemid:=NULL;
		_retval.itemdesc:=NULL;
		_retval.pid:=NULL;
		_retval.qty:=NULL;
		_retval.unit:=NULL;
		/**_retval.page:= _PageCounter;
		_retval.pagegroup:= _PageCounter;*/
		RETURN NEXT _retval;
	END LOOP;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tJobCardP CASCADE;
CREATE TYPE tJobCardP AS (
	numberrow	VARCHAR,
	jobcardpdno	SMALLINT,
	itemid		VARCHAR,
	itemdesc	VARCHAR,
	pid		VARCHAR	
);


CREATE OR REPLACE FUNCTION fJobCardP(_JobCardNo VARCHAR,
	__MaxRow	SMALLINT
) RETURNS SETOF tJobCardP AS
$BODY$
DECLARE
	_retval			tJobCardP%ROWTYPE;

	_Counter		SMALLINT;
	_EmptyRow		SMALLINT;
	_PageCounter		SMALLINT;

	cQuery			REFCURSOR;
BEGIN
	/** Inisialisasi */
	_Counter:=0;
	_PageCounter:=1;

	OPEN cQuery FOR
		SELECT jobcardpdno,itemid,itemdesc,pid /*fQty1Desc(itemid, fQtyZToQty1(itemid, qty, unit)) AS qtydesc*/ FROM jobcardp 
		WHERE jobcardno=_JobCardNo ORDER BY jobcardpdno;
	LOOP
		FETCH cQuery INTO _retval.jobcardpdno,_retval.itemid,_retval.itemdesc,_retval.pid;
		EXIT WHEN NOT FOUND;
		_Counter=_Counter+1;
		/**_retval.page = _PageCounter;
		_retval.pagegroup = _PageCounter;*/

		RETURN NEXT _retval;
/*		IF ((_Counter / __MaxRow) >=1 AND (_Counter % __MaxRow) = 0 ) THEN
			_PageCounter = _PageCounter + 1;
		END IF;*/
	END LOOP;
	CLOSE cQuery;

	_EmptyRow = _Counter%__MaxRow;
	
	FOR i IN 1..(__MaxRow-_EmptyRow) LOOP
		_retval.numberrow:=NULL;
		_retval.jobcardpdno:=NULL;
		_retval.itemid:=NULL;
		_retval.itemdesc:=NULL;
		_retval.pid:=NULL;
		/**_retval.page:= _PageCounter;
		_retval.pagegroup:= _PageCounter;*/
		RETURN NEXT _retval;
	END LOOP;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE tRptWO CASCADE;
CREATE TYPE tRptWO AS (
	wono			VARCHAR,
	upwono		VARCHAR,
	bpid			VARCHAR,
	pono			VARCHAR,
	wolevel		INT,
	itemdesc		VARCHAR,
	pid			VARCHAR,
	qty			NUMERIC(19,4),
	unit			VARCHAR,
	io			VARCHAR
);

CREATE OR REPLACE FUNCTION fRptWO(
	__WONo			VARCHAR,
	__UpWONo		VARCHAR
) RETURNS SETOF tRptWO AS
$BODY$
DECLARE
	_retval		tRptWO%ROWTYPE;
	
	_WONo		VARCHAR;
	_UpWONo		VARCHAR;
	_BPID		VARCHAR;
	_PONo		VARCHAR;
	_IO		VARCHAR;
	_Level		INT;
	_Itemdesc	VARCHAR;
	_PID		VARCHAR;
	_Qty		NUMERIC(19,4);
	_Unit		VARCHAR;
	_BOMID		VARCHAR;
	_ItemID		VARCHAR;
	_QtyBom		NUMERIC(19,4);
	_QtyWO		NUMERIC(19,4);
	_Multiplier	NUMERIC(19,4);

	cWO		REFCURSOR;
	cBOM		REFCURSOR;
BEGIN
	/** Inisialisasi */
	_WONo:=COALESCE(__WONo,'');
	_UpWONo:=COALESCE(__UpWONo,'');
	
	SELECT INTO _UpWONo,_Level upwono,wolevel FROM WO
	WHERE wono=_WONo; 

	/**
	_retval.wono=_WONo;
	_retval.upwono=_UpWONo;
	_retval.wolevel=_Level;
	**/
	
	OPEN cWO FOR
		SELECT wono,upwono,wolevel,bomid,bp.bpname,s.docno 
		FROM WO
		JOIN SO s ON wo.sono=s.sono
		JOIN BP ON wo.custid=bp.bpid
		WHERE wono=_WONo OR upwono=_UpWONo OR upwono=_WONo;
	LOOP
		FETCH cWO INTO _WONo,_UpWONO, _Level, _BOMID, _BPID,_PONo;
		EXIT WHEN NOT FOUND;

		SELECT INTO _QtyBom fQtyZToQty1(itemid,qty,unit) FROM bom WHERE bomid=_BOMID;

		SELECT INTO _QtyWO fQtyZToQty1(itemid,qty,unit) FROM wod WHERE wono=_WONo;

		_Multiplier = _QtyWO/_QtyBom;
					
		IF (_Level=0 OR _WONo=__WONo) THEN

			SELECT INTO _Level,_Itemdesc, _PID, _Qty, _Unit
			w.wolevel,i.itemdesc,wd.pid,wd.qty,wd.unit 
			FROM wo w
			JOIN wod wd ON w.wono=wd.wono
			JOIN item i ON wd.itemid=i.itemid
			WHERE w.wono=_WONo;
			
			_retval.wono=_WONo;
			_retval.bpid=_BPID;
			_retval.pono=_PONo;
			_retval.wolevel=_Level;
			_retval.itemdesc=_Itemdesc;
			_retval.pid=_PID;
			_retval.qty=_Qty;
			_retval.unit=_Unit;
			_retval.io='Output';

			RETURN NEXT _retval;
		END IF;

		OPEN cBOM FOR
			SELECT w.upwono,w.wolevel,bd.itemid,i.itemdesc,COALESCE(bd.pid,''),(bd.qty*_Multiplier) AS qty,bd.unit 
			FROM wo w
			JOIN bom b ON w.bomid=b.bomid 
			JOIN bomd bd ON b.bomid=bd.bomid
			JOIN item i ON bd.itemid=i.itemid
			WHERE w.wono=_WONo /*OR w.upwono=_UPWONo*/;
		LOOP
			FETCH cBOM INTO _UpWONO, _Level, _ItemID, _Itemdesc, _PID, _Qty, _Unit;
			EXIT WHEN NOT FOUND;

			SELECT INTO _BOMID bomid FROM bom WHERE itemid=_ItemID AND pid=_PID AND active=TRUE;		
			IF FOUND THEN
				CONTINUE;
			END IF;
			
			/**IF(_Level=0) THEN
				_IO = 'Output';
			ELSE**/
				_IO = 'Input';
			/**END IF;**/

			_retval.wono=_WONo;
			_retval.upwono=_UpWONo;
			_retval.bpid=_BPID;
			_retval.pono=_PONo;
			_retval.wolevel=_Level;
			_retval.itemdesc=_Itemdesc;
			_retval.pid=_PID;
			_retval.qty=_Qty;
			_retval.unit=_Unit;
			_retval.io=_IO;

			RETURN NEXT _retval;
		END LOOP;
		CLOSE cBOM;
	END LOOP;
	CLOSE cWO;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fProfitSaleD (
	_SaleNo		VARCHAR,
	_SaleDNo	SMALLINT
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	SELECT INTO 
		_retval 
		sd.qty*(sd.baseprice-sd.discamt-(sd.cost/s.excrate)) 
	FROM saled sd
	JOIN sale s ON s.saleno=sd.saleno
	WHERE sd.saleno=_SaleNo AND sd.saledno=_saledno;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fProfitSRetD (
	_SRetNo		VARCHAR,
	_SRetDNo	SMALLINT
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		NUMERIC(19,4);
BEGIN
	SELECT INTO 
		_retval 
		sd.qty*(sd.baseprice-sd.discamt-(sd.cost/s.excrate)) 
	FROM sretd sd
	JOIN sret s ON s.sretno=sd.sretno
	WHERE sd.sretno=_SRetNo AND sd.sretdno=_SRetDNo;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spUpdateVersion () 
RETURNS TRIGGER AS
$BODY$
DECLARE
BEGIN
	NEW.version=OLD.version+1;

	RETURN NEW;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;

DROP TRIGGER updateversion_preq ON preq;
DROP TRIGGER updateversion_po ON po;
DROP TRIGGER updateversion_prcv ON prcv;
DROP TRIGGER updateversion_purc ON purc;
DROP TRIGGER updateversion_pret ON pret;
DROP TRIGGER updateversion_sadj ON sadj;
DROP TRIGGER updateversion_sttr ON sttr;
DROP TRIGGER updateversion_cadj ON cadj;
DROP TRIGGER updateversion_cstr ON cstr;
DROP TRIGGER updateversion_so ON so;
DROP TRIGGER updateversion_deo ON deo;
DROP TRIGGER updateversion_deli ON deli;
DROP TRIGGER updateversion_sale ON sale;
DROP TRIGGER updateversion_sret ON sret;
DROP TRIGGER updateversion_pay ON pay;
DROP TRIGGER updateversion_rcv ON rcv;
DROP TRIGGER updateversion_cons ON cons;
DROP TRIGGER updateversion_cret ON cret;
DROP TRIGGER updateversion_fcn ON fcn;
DROP TRIGGER updateversion_jnl ON jnl;
DROP TRIGGER updateversion_jnl ON jnl;


CREATE TRIGGER updateversion_preq BEFORE UPDATE ON preq FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_po BEFORE UPDATE ON po FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_prcv BEFORE UPDATE ON prcv FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_purc BEFORE UPDATE ON purc FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_pret BEFORE UPDATE ON pret FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_sadj BEFORE UPDATE ON sadj FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_sttr BEFORE UPDATE ON sttr FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_cadj BEFORE UPDATE ON cadj FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_cstr BEFORE UPDATE ON cstr FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_so BEFORE UPDATE ON so FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_deo BEFORE UPDATE ON deo FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_deli BEFORE UPDATE ON deli FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_sale BEFORE UPDATE ON sale FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_sret BEFORE UPDATE ON sret FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_pay BEFORE UPDATE ON pay FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_rcv BEFORE UPDATE ON rcv FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_cons BEFORE UPDATE ON cons FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_cret BEFORE UPDATE ON cret FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_fcn BEFORE UPDATE ON fcn FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_jnl BEFORE UPDATE ON jnl FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();
CREATE TRIGGER updateversion_recur BEFORE UPDATE ON recur FOR EACH ROW EXECUTE PROCEDURE spUpdateVersion();



CREATE TYPE tLogTrans AS (
	logno	 	VARCHAR,
	reftype	 	VARCHAR,
	reftypedesc	VARCHAR,
	refno 		VARCHAR,
	crtby 		VARCHAR,
	crtbyname		VARCHAR,
	crtdate	 	TIMESTAMP,
	status		VARCHAR
);




CREATE OR REPLACE FUNCTION fLogTransUser() 
RETURNS SETOF tlogtrans AS
$BODY$
DECLARE
	_retval			tlogtrans%ROWTYPE;

	_logNo			logtrans.logno%TYPE;
	_refType		logtrans.reftype%TYPE;
	_tmpRefTypeDesc		VARCHAR;
	_refTypeDesc		VARCHAR;
	_refNo 			logtrans.refno%TYPE;
	_crtBy 			logtrans.crtby%TYPE;
	_crtByName		VARCHAR;
	_crtDate		logtrans.crtdate%TYPE;
	_status			logtrans.status%TYPE;

	cLog			REFCURSOR;
BEGIN
	OPEN cLog FOR
		SELECT logno, reftype, refno, crtby, crtdate, status 
		FROM logtrans lo ORDER BY reftype,refno,crtdate;
	LOOP
		FETCH cLog INTO _logNo, _refType, _refNo, _crtBy, _crtDate, _status;
		EXIT WHEN NOT FOUND;

		SELECT INTO _tmpRefTypeDesc * FROM fRefTypeDesc(_refType);
		_tmpRefTypeDesc:=COALESCE(_tmpRefTypeDesc,'Undefined');

		_refTypeDesc:='Aktifitas '|| _tmpRefTypeDesc ||'';

		SELECT INTO _crtByName usrname FROM usr WHERE usrid=_crtby;

		_retval.logno:=_logno;
		_retval.reftype:=_reftype;
		_retval.refTypeDesc:=_refTypeDesc;
		_retval.refno:=_refno;
		_retval.crtBy:=_crtBy;
		_retval.crtByName:=_crtByName;
		_retval.crtdate:=_crtdate;
		_retval.status:=_status;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cLog;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fRefTypeDesc (
	_RefType	VARCHAR
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval 		VARCHAR;
BEGIN
	_retval:=NULL;

	SELECT INTO _retval reftypedesc
	FROM reftype WHERE reftype=_RefType;

	RETURN _RetVal;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spRecurring_New (
	_RecurNo	recur.recurno%TYPE
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_RefType	reftype.reftype%TYPE;
	_RefNo		VARCHAR(15);
	_RecurDNo	recurd.recurdno%TYPE;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _RefType, _RefNo reftype, refno FROM recur WHERE recurno=_RecurNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Rekuring #'||COALECE(_RecurNo, '<kosong>')||' tidak ditemukan ! (spRecurring_New)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _RecurDNo recurdno
	FROM recurd WHERE recurno=_RecurNo AND recurdref=_RefNo;

	IF (_RefType='PURC') THEN
		UPDATE purc SET isrecurring=TRUE, recurno=_RecurNo, recurdno=_RecurDNo WHERE purcno=_RefNo;
	ELSIF (_RefType='STTR') THEN
		UPDATE sttr SET isrecurring=TRUE, recurno=_RecurNo, recurdno=_RecurDNo WHERE sttrno=_RefNo;
	ELSIF (_RefType='CSTR') THEN
		UPDATE cstr SET isrecurring=TRUE, recurno=_RecurNo, recurdno=_RecurDNo WHERE cstrno=_RefNo;
	ELSIF (_RefType='SALE') THEN
		UPDATE sale SET isrecurring=TRUE, recurno=_RecurNo, recurdno=_RecurDNo WHERE saleno=_RefNo;
	ELSIF (_RefType='PAY') THEN
		UPDATE pay SET isrecurring=TRUE, recurno=_RecurNo, recurdno=_RecurDNo WHERE payno=_RefNo;
	ELSIF (_RefType='RCV') THEN
		UPDATE rcv SET isrecurring=TRUE, recurno=_RecurNo, recurdno=_RecurDNo WHERE rcvno=_RefNo;
	ELSIF (_RefType='JNL') THEN
		UPDATE jnl SET isrecurring=TRUE, recurno=_RecurNo, recurdno=_RecurDNo WHERE jnlno=_RefNo;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spRecurring_Void (
	_RecurNo	recur.recurno%TYPE
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	cData		REFCURSOR;

	_RefType	reftype.reftype%TYPE;
	_RefNo		VARCHAR(15);
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _RefType, _RefNo reftype, refno FROM recur WHERE recurno=_RecurNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Rekuring #'||COALECE(_RecurNo, '<kosong>')||' tidak ditemukan (spRecurring_Void)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_RefType='PURC') THEN
		UPDATE purc SET isrecurring=FALSE, recurno=NULL, recurdno=NULL WHERE purcno=_RefNo;
	ELSIF (_RefType='STTR') THEN
		UPDATE sttr SET isrecurring=FALSE, recurno=NULL, recurdno=NULL WHERE sttrno=_RefNo;
	ELSIF (_RefType='CSTR') THEN
		UPDATE cstr SET isrecurring=FALSE, recurno=NULL, recurdno=NULL WHERE cstrno=_RefNo;
	ELSIF (_RefType='SALE') THEN
		UPDATE sale SET isrecurring=FALSE, recurno=NULL, recurdno=NULL WHERE saleno=_RefNo;
	ELSIF (_RefType='PAY') THEN
		UPDATE pay SET isrecurring=FALSE, recurno=NULL, recurdno=NULL WHERE payno=_RefNo;
	ELSIF (_RefType='RCV') THEN
		UPDATE rcv SET isrecurring=FALSE, recurno=NULL, recurdno=NULL WHERE rcvno=_RefNo;
	ELSIF (_RefType='JNL') THEN
		UPDATE jnl SET isrecurring=FALSE, recurno=NULL, recurdno=NULL WHERE jnlno=_RefNo;
	END IF;

	OPEN cData FOR 
		SELECT recurdref 
		FROM recurd
		WHERE recurno=_RecurNo AND executed=TRUE;
	LOOP
		FETCH cData INTO _RefNo;
		EXIT WHEN NOT FOUND;

		IF (_RefType='PURC') THEN
			UPDATE purc SET recurno=NULL, recurdno=NULL WHERE purcno=_RefNo;
		ELSIF (_RefType='STTR') THEN
			UPDATE sttr SET recurno=NULL, recurdno=NULL WHERE sttrno=_RefNo;
		ELSIF (_RefType='CSTR') THEN
			UPDATE cstr SET recurno=NULL, recurdno=NULL WHERE cstrno=_RefNo;
		ELSIF (_RefType='SALE') THEN
			UPDATE sale SET recurno=NULL, recurdno=NULL WHERE saleno=_RefNo;
		ELSIF (_RefType='PAY') THEN
			UPDATE pay SET recurno=NULL, recurdno=NULL WHERE payno=_RefNo;
		ELSIF (_RefType='RCV') THEN
			UPDATE rcv SET recurno=NULL, recurdno=NULL WHERE rcvno=_RefNo;
		ELSIF (_RefType='JNL') THEN
			UPDATE jnl SET recurno=NULL, recurdno=NULL WHERE jnlno=_RefNo;
		END IF;
	END LOOP;
	CLOSE cData;

	DELETE FROM recurd WHERE recurno=_RecurNo;
	DELETE FROM recur WHERE recurno=_RecurNo;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spRecurring_Update (
	_P_NewNo	VARCHAR(15),
	_P_NewDesc	recurd.recurddesc%TYPE,
	_P_RecurNo	recur.recurno%TYPE,
	_P_RecurDNo	recurd.recurdno%TYPE,
	_P_Executed	recurd.executed%TYPE
) RETURNS SETOF SPStatus AS 
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;

	_RecurNo		recurd.recurno%TYPE;
	_Executed		recurd.executed%TYPE;
	_Finished		recur.finished%TYPE;
	_RecurCount		recur.recurcount%TYPE;

	_CountExecuted		INT;
BEGIN
	_retval:=(0, 'OK');

	SELECT INTO _RecurNo, _Executed, _Finished, _RecurCount
		d.recurno, d.executed, m.finished, m.recurcount
	FROM recurd d 
	JOIN recur m ON d.recurno=m.recurno 
	WHERE d.recurno=_P_RecurNo AND d.recurdno=_P_RecurDNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Rekuring #'||COALESCE(_P_RecurNo, '<kosong>')||' tidak ditemukan ! (spRecurring_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_Finished=TRUE) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Keseluruhan perulangan dari rekuring #'||COALESCE(_P_RecurNo, '<kosong>')||' sudah tereksekusi ! (spRecurring_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	IF (_P_Executed=TRUE AND _Executed=TRUE) THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Rekuring #'||COALESCE(_P_RecurNo, '<kosong>')||' detail#'||COALESCE(_P_RecurDNo, -1)||E' sudah dieksekusi.\n Tidak diperkenankan untuk dieksekusi lebih dari 1 (satu) kali ! \n (spRecurring_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	UPDATE recurd SET executed=_P_Executed, recurdref=_P_NewNo, recurddesc=_P_NewDesc 
	WHERE recurno=_P_RecurNo AND recurdno=_P_RecurDNo;
	IF NOT FOUND THEN
		_retval.ErrNo:=50555;
		_retval.ErrMsg:='Proses update rekuring #'||COALESCE(_P_RecurNo, '<kosong>')||' detail #'||COALESCE(_P_RecurDNo, -1)||E' gagal !\n (spRecurring_Update)';
		RAISE EXCEPTION '%', _retval.ErrMsg;
	END IF;

	SELECT INTO _CountExecuted SUM(CASE WHEN executed=TRUE THEN 1 ELSE 0 END)
	FROM recurd 
	WHERE recurno=_P_RecurNo;

	IF (_CountExecuted=_RecurCount) THEN
		_Finished=TRUE;
	ELSE
		_Finished=FALSE;
	END IF;

	UPDATE recur SET finished=_Finished WHERE recurno=_P_RecurNo;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION isCashUsedInCashier (
	__CashID VARCHAR
) RETURNS BOOLEAN AS 
$BODY$
DECLARE
	_count		INT;
	_retval		BOOLEAN;
	_isStock	BOOLEAN;
BEGIN
	_retval:=FALSE;

	SELECT INTO _count COUNT(*) FROM pos WHERE cashid=__CashID;
	IF (FOUND AND _count>0) THEN
		_retval:=TRUE;
	END IF;

	RETURN _retval;
END;
$BODY$ 
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fGetRefType (
	_CashANo casha.cashano%TYPE
) RETURNS VARCHAR AS
$BODY$
DECLARE
	_retval		VARCHAR;

	_RefNo		casha.refno%TYPE;
	_RefType	casha.reftype%TYPE;
	_UpRefType	casha.reftype%TYPE;
BEGIN

	SELECT INTO _RefNo, _RefType 
		refno, reftype
	FROM casha 
	WHERE cashano=_CashANo;

	IF (_RefType='RCV') THEN
		SELECT INTO _UpRefType reftype 
		FROM rcv 
		WHERE rcvno=_RefNo;

		_RefType:=COALESCE(_UpRefType, _RefType);
	ELSIF (_RefType='PAY') THEN
		SELECT INTO _UpRefType reftype 
		FROM pay 
		WHERE payno=_RefNo;

		_RefType:=COALESCE(_UpRefType, _RefType);
	END IF;

	_retval:=_RefType;
	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spMemorizedUtil (
	_RefType	VARCHAR,
	_RefColumn	VARCHAR,
	_RefNo		VARCHAR,
	_IsMemorized	BOOLEAN,
	_MemorizedNote	TEXT
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_Query		TEXT;
BEGIN
	_retval:=(0, 'OK');

	_Query:='UPDATE '||QUOTE_IDENT(_RefType)||' SET ismemorized='||_IsMemorized||', memorizednote='||QUOTE_LITERAL(_MemorizedNote)||' WHERE '||QUOTE_IDENT(_RefColumn)||'='||QUOTE_LITERAL(_RefNo);

	EXECUTE _Query;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION fLastPurcCrcUnit1 (
	_ItemId		item.itemid%TYPE,
	_CrcId		crc.crcid%TYPE
) RETURNS NUMERIC(19,4) AS
$BODY$
DECLARE
	_retval		prc.price1%TYPE;

	_Unit1		item.unit1%TYPE;
	_Unit2		item.unit1%TYPE;
	_Conv2		item.conv2%TYPE;
	_Unit3		item.unit1%TYPE;
	_Conv3		item.conv3%TYPE;

	_LastPrice	NUMERIC(19,4);
	_LastUnit	item.unit1%TYPE;
BEGIN
	_retval:=0;

	SELECT INTO _Unit1, _Unit2, _Conv2, _Unit3, _Conv3
		unit1, unit2, conv2, unit3, conv3
	FROM item
	WHERE itemid=_ItemId;

	SELECT INTO _LastPrice, _LastUnit f.purcprice, f.purcunit 
	FROM flastpurccrc(_ItemId, _CrcId) f;

	IF (_LastPrice!=0) THEN
		IF (_LastUnit=_Unit1) THEN
			_retval:=_LastPrice;
		ELSIF (_LastUnit=_Unit2) THEN
			_retval:=_LastPrice/_Conv2;
		ELSIF (_LastUnit=_Unit3) THEN
			_retval:=_LastPrice/_Conv3;
		END IF;
	END IF;

	RETURN _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




/**
 *	cek jika stock tidak ditemukan -> buatkan
 */
CREATE OR REPLACE FUNCTION spCreateStockCard (
	_P_ItemId	item.itemid%TYPE,
	_P_WhId		wh.whid%TYPE,
	_P_PId		stock.pid%TYPE
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval		SPStatus%ROWTYPE;

	_PId		stock.pid%TYPE;
BEGIN
	_retval:=(0, 'OK');

	IF (_P_PId IS NULL) THEN
		_PId:='';
	ELSE
		_PId:=_P_PId;
	END IF;

	IF NOT EXISTS (SELECT 1 FROM stock WHERE itemid=_P_ItemId AND whid=_P_WhId AND pid=_PId) THEN
		INSERT INTO stock (itemid, whid, pid) VALUES (_P_ItemId, _P_WhId, _PId);
		IF NOT FOUND THEN
			_retval.ErrNo:=1000005;
			_retval.ErrMsg:=_retval.ErrNo||E'|Gagal buat kartu stok untuk\nitem:'||COALESCE(_P_ItemId, '<kosong>')||' di gudang:'||COALESCE(_WhId, '<kosong>')||'dengan pid:'||_PId||'! (spCreateStockCard)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		END IF;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spUpdatePReq (
	_P_PReqNo	preq.preqno%TYPE,
	_P_PReqDNo	preqd.preqdno%TYPE,
	_P_ItemId	item.itemid%TYPE,
	_P_PId		stock.pid%TYPE,
	_P_Qty1		NUMERIC(15,4)
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;

	_Qty1			NUMERIC(15,4);
	_Qty1_Outstanding	NUMERIC(15,4);
	_Qty1_Ordered		NUMERIC(15,4);
	_Qty1_Rejected		NUMERIC(15,4);
	_Qty1_Tolerance		NUMERIC(15,4);
BEGIN
	_retval:=(0, 'OK');
	_Qty1_Tolerance:=0;

	SELECT INTO _Qty1_Ordered, _Qty1_Rejected, _Qty1 
		qtyordered, qtyreject, fQtyZToQty1(itemid, qty, unit) 
	FROM preqd WHERE preqno=_P_PReqNo AND preqdno=_P_PReqDNo;
	IF FOUND THEN
		_Qty1_Outstanding=(_Qty1+_Qty1_Tolerance)-_Qty1_Ordered-_Qty1_Rejected;
		IF (_P_Qty1>_Qty1_Outstanding) THEN
			_retval.ErrNo:=1000008;
			_retval.ErrMsg:=_retval.ErrNo||E'|Qty Order Pembelian item:'||COALESCE(_P_ItemId, '<KOSONG>')||' pid:'||COALESCE(_P_PId, '')||' melebihi qty permintaan pembelian ! (spUpdatePReq)';
			RAISE EXCEPTION '%', _retval.ErrMsg;
		ELSE
			UPDATE preqd SET qtyordered=qtyordered+_P_Qty1 WHERE preqno=_P_PReqNo AND preqdno=_P_PReqDNo;
		END IF;
	END IF;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




CREATE OR REPLACE FUNCTION spUpdatePReq_Status (
	_P_PReqNo	preq.preqno%TYPE
) RETURNS SETOF SPStatus AS
$BODY$
DECLARE
	_retval			SPStatus%ROWTYPE;

	cData			REFCURSOR;

	_Qty1			NUMERIC(15,4);
	_Qty1_Outstanding	NUMERIC(15,4);
	_PReqStatus		preq.preqstatus%TYPE;

	_First			BOOLEAN;
BEGIN
	_retval:=(0, 'OK');
	_First:=TRUE;

	OPEN cData FOR
		SELECT fQtyZToQty1(itemid, qty, unit), fQtyZToQty1(itemid, qty, unit)-qtyordered-qtyreject
		FROM preqd 
		WHERE preqno=_P_PReqNo;
	LOOP
		FETCH cData INTO _Qty1, _Qty1_Outstanding;
		EXIT WHEN NOT FOUND;

		IF (_First=TRUE) THEN
			IF (_Qty1_Outstanding>0 AND _Qty1!=_Qty1_Outstanding) THEN
				_PReqStatus:='P';
				EXIT;
			ELSIF (_Qty1=_Qty1_Outstanding) THEN
				_PReqStatus:='N';
			ELSIF (_Qty1_Outstanding=0) THEN
				_PReqStatus:='F';
			END IF;

			_First:=FALSE;
		ELSE
			IF (_Qty1_Outstanding>0 AND _Qty1!=_Qty1_Outstanding) THEN
				_PReqStatus:='P';
				EXIT;
			ELSE
				IF (_PReqStatus='N' AND ((_Qty1_Outstanding>0 AND _Qty1!=_Qty1_Outstanding) OR (_Qty1_Outstanding=0))) THEN
					_PReqStatus:='P';
					EXIT;
				ELSIF (_PReqStatus='N' AND (_Qty1_Outstanding=_Qty1)) THEN
					_PReqStatus:='N';
				ELSIF (_PReqStatus='F' AND ((_Qty1_Outstanding>0 AND _Qty1!=_Qty1_Outstanding) OR (_Qty1=_Qty1_Outstanding))) THEN
					_PReqStatus:='P';
					EXIT;
				ELSIF (_PReqStatus='F' AND _Qty1_Outstanding=0) THEN
					_PReqStatus:='F';
				END IF;
			END IF;
		END IF;
	END LOOP;
	CLOSE cData;

	UPDATE preq SET preqstatus=_PReqStatus, active=(CASE WHEN _PReqStatus='F' THEN FALSE ELSE TRUE END) WHERE preqno=_P_PReqNo;

	RETURN NEXT _retval;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;




DROP TYPE DPUsage CASCADE;
CREATE TYPE DPUsage AS (
	usagedate	DATE,
	reference	VARCHAR,
	usageamt	NUMERIC(19,4),
	usagenote	TEXT	
);

CREATE OR REPLACE FUNCTION fDPUsage(
	_DPID BIGINT
) RETURNS SETOF DPUsage AS
$BODY$
DECLARE
	_retval		DPUsage%ROWTYPE;

	_PaidType	VARCHAR;

	_Query		TEXT;
	cData		REFCURSOR;
BEGIN
	_Query:='';
	
	SELECT INTO _PaidType paidtype FROM dp WHERE dpid=_DPID;
	IF (_PaidType='RCV') THEN
		_Query='SELECT m.rcvdate, '||QUOTE_LITERAL('RCV:')||'||m.rcvno, d.spaidamt*-1, m.rcvnote 
			FROM spaid d JOIN rcv m ON d.rcvno=m.rcvno 
			WHERE d.refno='||QUOTE_LITERAL(_DPID)||' AND d.reftype='||QUOTE_LITERAL('DP')||' AND m.rcvtype!='||QUOTE_LITERAL('DP');
	ELSIF (_PaidType='PAY') THEN
		_Query='SELECT m.paydate, '||QUOTE_LITERAL('PAY:')||'||m.payno, d.ppaidamt*-1, m.paynote 
			FROM ppaid d JOIN pay m ON d.payno=m.payno 
			WHERE d.refno='||QUOTE_LITERAL(_DPID)||' AND d.reftype='||QUOTE_LITERAL('DP')||' AND m.paytype!='||QUOTE_LITERAL('DP');
	END IF;

	OPEN cData FOR
		EXECUTE _Query;
	LOOP
		FETCH cData INTO _retval;
		EXIT WHEN NOT FOUND;

		RETURN NEXT _retval;
	END LOOP;
	CLOSE cData;
END;
$BODY$
LANGUAGE 'plpgsql' VOLATILE;


ALTER TABLE JOBCARD ALTER COLUMN USED SET DEFAULT 'FALSE';


ALTER TABLE JNLD ALTER COLUMN JNLDNOTE TYPE TEXT;
ALTER TABLE LGR ALTER COLUMN LGRNOTE TYPE TEXT;
ALTER TABLE LGR ALTER COLUMN REFTXT TYPE TEXT;


INSERT INTO OBJACL (objid, acstype, acsenable) 
SELECT objid,'REPRN',false FROM obj WHERE objid in ('215001','215002','215004','215005','215006','215007','215107','715002','715004','517003','517004','517005',
'715001','715003','715311','715312','815002','815003','815004','815005','815006','815009','815306','835104','825104','845005','835101','527001'
,'517002','716005','818700','218700');

INSERT INTO OBJACL (objid, acstype, acsenable) 
SELECT objid,'REPRN',false FROM obj WHERE objid in ('815007','825005','835105','825105');


ALTER TABLE REG ALTER COLUMN RegDesc TYPE VARCHAR(60);

INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('PURC', 'PPRCV_RPT', 'Text Report Nota Pengiriman Retur Pembelian', 'PPRCV');
INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('PURC', 'PPRCV_RPTD', 'Text Report Nota Pengiriman Retur Pembelian Detail', 'PPRCVD');

INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('PURC', 'PPRCV_RKP_RPT', 'Text Report Nota Rekap Pengiriman Retur Pembelian', 'PPRCV');
INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('PURC', 'PPRCVD_RKP_RPT', 'Text Report Nota Rekap Pengiriman Retur Pembelian Detail', 'PPRCVD');

DELETE FROM RVAR WHERE RPTID='PPRCVD';
DELETE FROM RVAR WHERE RPTID='PPRCV';

DELETE FROM RBAND WHERE RPTID='PPRCVD';
DELETE FROM RBAND WHERE RPTID='PPRCV';

DELETE FROM RPT WHERE RPTID='PPRCV';
DELETE FROM RPT WHERE RPTID='PPRCVD';

insert into rpt(rptid,rptdesc,paperwidth,paperheight,pagecols,pagerows,marginleft,marginright,margintop,marginbottom,font,fontsize,ullastdet,detailmin,detailmax,sendff,nulldtltxt,nulldtlfill) values('PPRCV','Pengiriman Retur Pembelian',200,33,200,33,1,1,1,1,'DRAFT',12,FALSE,'8','11',TRUE,' ',TRUE);

insert into rpt(rptid,rptdesc) values('PPRCVD','Pengiriman Retur Pembelian (detail)');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','A','PRCVNO','PRCVNO','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','B','TERM','TERM','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','C','POSID','POSID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','D','PRCVDATE','PRCVDATE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','E','CRTBY','CRTBY','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','F','TOTAL','TOTAL','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','G1','BILLFROM[1]','BILLFROM[1]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','G2','BILLFROM[2]','BILLFROM[2]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','G3','BILLFROM[3]','BILLFROM[3]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','G4','BILLFROM[4]','BILLFROM[4]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','K','TOTRCVAMT','TOTRCVAMT','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','K2','TOTCHGAMT','TOTCHGAMT','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','L','SREPID','SREPID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','M','SHIPID','SHIPID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCV','0','PONO','PONO','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','A','ITEMID','ITEMID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','B','QTY','QTY','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','B2','Qty Retur','QTYDELIDESC','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','C','UNIT','UNIT','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','D','LISTPRICE','LISTPRICE','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','E','DISCEXP','DISCEXP','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','F','SUBTOTAL','SUBTOTAL','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','G','ITEMDESC','ITEMDESC','L');
insert into rband values('PPRCV','2','DETAIL','PT. BEE ACCOUNTING');
insert into rband values('PPRCV','4','DETAIL','Jl. Raya Klampis No.29 J                 Kepada Yth.: ');
insert into rband values('PPRCV','6','DETAIL','Telp : 031-592.8175 Fax : 031-592.8175            &G1........................');
insert into rband values('PPRCV','10','DETAIL','                                          &G2..............................');
insert into rband values('PPRCV','11','DETAIL','                                                  &G3.............,&G4..........');
insert into rband values('PPRCV','12','DETAIL','                       $0E$1B-1SJ PENGIRIMAN RETUR PEMBELIAN$1B-0$14');
insert into rband values('PPRCV','13','DETAIL','Tanggal: &D..........       No.&A..........           Opr : &E.....');
insert into rband values('PPRCV','15','DETAIL','$1B-1                                                                            $1B-1');
insert into rband values('PPRCV','16','DETAIL','$1B-1  Kode            Nama Barang                                      Banyaknya $1B-0 $1B-0');
insert into rband values('PPRCV','17','SUBREPORT','0');
insert into rband values('PPRCV','18','DETAIL','-----------------------------------------------------------------------------');
insert into rband values('PPRCV','19','DETAIL','$0FKeterangan:$12');
insert into rband values('PPRCV','20','DETAIL','$0FPerhatian:                                                      $12');
insert into rband values('PPRCV','22','DETAIL','$0F1.Harap dicheck jumlah \& kondisi barang sebelum diterima       $12');
insert into rband values('PPRCV','24','DETAIL','$0F2.Komplain setelah barang diterima tidak dilayani               $12     No. Mobil : &M.............');
insert into rband values('PPRCV','26','DETAIL','       Gudang,              di Check oleh,                 Tanda Terima,');
insert into rband values('PPRCVD','1','DETAIL','&A................. &G.........................                        &B..........$20');

ALTER TABLE item DROP CONSTRAINT ITMACCCOST;


INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('SYS', 'FULLSCREEN_POS', 'Fullscreen Form POS', '0');
INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('SYS', 'RPRINT_ENABLED', 'Enabled input baris nota jasper', '0');
INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('SYS', 'RPRINT_DEFAULT', 'Baris Default untuk nota jasper per halaman', '30');



UPDATE pay SET overpay=0 WHERE overpaybc=0;
UPDATE rcv SET overrcv=0 WHERE overrcvbc=0;



INSERT INTO obj(objid,upobjid,modulid,objname,objlevel,enabled) VALUES('189112','189100','MST','Impor Promo',3,true);

INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'ENB',true FROM obj WHERE objid='189112';

INSERT INTO grpprv (grpid, objid, acstype, acsval, acsenable) SELECT 'ADM',objid,acstype,acsval,true FROM objacl WHERE objid='189112';



INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('SALE', 'POSCRCV_RPT', 'Text Report P.o.S Detail Credit Card', 'SALECRCV');



INSERT INTO OBJACL (objid, acstype, acsenable) values( '815005', 'POS_EDIT', false);
INSERT INTO grpprv (grpid, objid, acstype, acsval, acsenable) SELECT 'ADM',objid,acstype,acsval,true FROM objacl WHERE acstype='POS_EDIT';



ALTER TABLE barcode ADD DescSize INT2 NOT NULL DEFAULT 25;


ALTER TABLE prc ADD usrid VARCHAR(10) REFERENCES usr (usrid) ON DELETE RESTRICT ON UPDATE CASCADE;

CREATE SEQUENCE pricelogid_seq;
CREATE TABLE pricelog (
	pricelogid		BIGINT NOT NULL DEFAULT NEXTVAL('pricelogid_seq'),
	pricelogdate		TIMESTAMP,
	itemid			VARCHAR(20),
	usrid			VARCHAR(10),
	prclvlid		VARCHAR(3),
	crcid			VARCHAR(5),
	price1old		NUMERIC(19,4),
	disc1expold		VARCHAR(15),
	price1new		NUMERIC(19,4),
	disc1expnew		VARCHAR(15),
	price2old		NUMERIC(19,4),
	disc2expold		VARCHAR(15),
	price2new		NUMERIC(19,4),
	disc2expnew		VARCHAR(15),
	price3old		NUMERIC(19,4),
	disc3expold		VARCHAR(15),
	price3new		NUMERIC(19,4),
	disc3expnew		VARCHAR(15)
);




CREATE SEQUENCE bgtid_seq;
ALTER TABLE bgt ADD bgtid BIGINT NOT NULL DEFAULT NEXTVAL('bgtid_seq');
ALTER TABLE bgt DROP CONSTRAINT pk_bgt;
ALTER TABLE bgt ALTER prjid DROP NOT NULL;
ALTER TABLE bgt ALTER deptid DROP NOT NULL;
ALTER TABLE bgt ADD constraint pk_bgt primary key (bgtid);
DROP INDEX bgt_pk;
DROP INDEX bgt_pk_2;
CREATE UNIQUE INDEX bgt_pk ON bgt (accno, perid, prjid, deptid);
CREATE UNIQUE INDEX bgt_pk_2 ON bgt (bgtid);




UPDATE jnl SET jnldate=s.saledate FROM sale s WHERE jnl.refno=s.saleno AND saletype='B';
UPDATE jnl SET jnldate=p.purcdate FROM purc p WHERE jnl.refno=p.purcno AND purctype='B';


INSERT INTO obj(objid,upobjid,modulid,objname,objlevel,enabled) VALUES('518009','518000','INV','Rekap Penyesuaian Stock',3,true);
INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'ENB',true FROM obj WHERE objid='518009';
INSERT INTO grpprv (grpid, objid, acstype, acsval, acsenable) SELECT 'ADM',objid,acstype,acsval,true FROM objacl WHERE objid='518009';


INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('SALE', 'CRET_RPT', 'Text Report Retur Konsinyasi', 'CRET');
INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('SALE', 'CRETD_RPT', 'Text Report Retur Konsinyasi Detail', 'CRETD');

INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('SALE', 'RKP_CRET_RPT', 'Text Report Rekap Retur Konsinyasi', 'RKPCRET');
INSERT INTO Reg (modulid, RegID, RegDesc, RegVal) VALUES ('SALE', 'RKP_CRETD_RPT', 'Text Report Rekap Retur Konsinyasi Detail', 'RKPCRETD');


insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPPRET','N','PRETNOTE','PRETNOTE','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPSRET','N','SRETNOTE','SRETNOTE','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALED','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALED','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALED','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALED','N','SALEDNOTE','SALEDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALED','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALEBNS','E','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALEBNS','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD','N','SALEDNOTE','SALEDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD_RKP','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD_RKP','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD_RKP','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD_RKP','N','NOTE','NOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALETAXD_RKP','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='SALEDPID' AND RVARID='N';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALEDPID','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALEDPID','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALEDPID','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALEDPID','N','NOTE','NOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SALEDPID','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='PRCVD' AND RVARID='B2';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRCVD','H','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRCVD','I','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRCVD','N','PRCVDNOTE','PRCVDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRCVD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','N','PRCVDNOTE','PRCVDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPRCVD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DOD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DOD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DOD','N','DEODNOTE','DEODNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DOD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','B1','QTYDO','QTYDO','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','B2','QTYDELIVERED','QTYDELIVERED','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','J1','QTYXDO','QTYXDO','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','J2','QTYXDELIVERED','QTYXDELIVERED','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','N','SODNOTE','SODNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DND','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DND','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DND','N','SALEDNOTE','SALEDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DND','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='DND_PID' AND RVARID='N';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DND_PID','P','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DND_PID','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DND_PID','N','NOTE','NOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DND_PID','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID','B1','QTYDRET','QTYDRET','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID','B2','QTYDELI','QTYDELI','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID','J1','QTYXDRET','QTYXDRET','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID','J2','QTYXDELI','QTYXDELI','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID','N','DELIDNOTE','DELIDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='DELID_RKP' AND RVARID='N';
DELETE FROM RVAR WHERE RPTID='DELID_RKP' AND RVARID='B1';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID_RKP','B1','QTYDRET','QTYDRET','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID_RKP','B2','QTYDELI','QTYDELI','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID_RKP','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID_RKP','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID_RKP','J1','QTYXDRET','QTYXDRET','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID_RKP','J2','QTYXDELI','QTYXDELI','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID_RKP','N','NOTE','NOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELID_RKP','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='POSD' AND RVARID='N';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSD','N','SALEDNOTE','SALEDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SADJD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SADJD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SADJD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SADJD_RKP','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SADJD_RKP','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SADJD_RKP','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SIND','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SIND','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SIND','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SIND_RKP','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SIND_RKP','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SIND_RKP','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOUTD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOUTD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOUTD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOUTD_RKP','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOUTD_RKP','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SOUTD_RKP','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='STTRD' AND RVARID='N1';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('STTRD','N1','STTRDNOTE','STTRDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('STTRD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('STTRD','J','QTYX','QTYX','L');

DELETE FROM RVAR WHERE RPTID='STTRD_RKP' AND RVARID='N1';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('STTRD_RKP','N','NOTE','NOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('STTRD_RKP','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('STTRD_RKP','J','QTYX','QTYX','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','B1','QTYSELLOUT','QTYSELLOUT','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','B2','QTYSELLCRET','QTYSELLCRET','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','J','QTYXSELLIN','QTYXSELLIN','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','J1','QTYXSELLOUT','QTYXSELLOUT','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','J2','QTYXCRET','QTYXCRET','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','N','CONSDNOTE','CONSDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='CONSD_RKP' AND RVARID='N';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','B1','QTYSELLOUT','QTYSELLOUT','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','B2','QTYSELLCRET','QTYSELLCRET','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','J','QTYXSELLIN','QTYXSELLIN','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','J1','QTYXSELLOUT','QTYXSELLOUT','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','J2','QTYXCRET','QTYXCRET','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','N','CONSDNOTE','CONSDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSD_RKP','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','B1','QTYSELLOUT','QTYSELLOUT','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','B2','QTYSELLCRET','QTYSELLCRET','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','J','QTYXSELLIN','QTYXSELLIN','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','J1','QTYXSELLOUT','QTYXSELLOUT','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','J2','QTYXCRET','QTYXCRET','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','N','CONSDNOTE','CONSDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONSTAXD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONSD','B1','QTYSELLOUT','QTYSELLOUT','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONSD','B2','QTYSELLCRET','QTYSELLCRET','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONSD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONSD','J','QTYXSELLIN','QTYXSELLIN','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONSD','J1','QTYXSELLOUT','QTYXSELLOUT','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONSD','J2','QTYXSELLCRET','QTYXSELLCRET','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONSD','N','CONSDNOTE','CONSDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONSD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SPAID','J','SPAIDNOTE','SPAIDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SPAID','K','SPAIDDISCEXP','SPAIDDISCEXP','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SPAID','L','SPAIDDISCAMT','SPAIDDISCAMT','R');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPAID','J','PPAIDNOTE','PPAIDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPAID','K','PPAIDDISCEXP','PPAIDDISCEXP','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PPAID','L','PPAIDDISCAMT','PPAIDDISCAMT','R');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETD','N','SRETDNOTE','SRETDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETCONSD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETCONSD','J','QTYX','QTYX','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETCONSD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETCONSD','N','SRETDNOTE','SRETDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('SRETCONSD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POD','B1','QTYDELIVERED','QTYDELIVERED','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POD','B2','QTYBO','QTYBO','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POD','N','PODNOTE','PODNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('TAXFORMD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('TAXFORMD','J','QTYX','QTYX','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('TAXFORMD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('TAXFORMD','N','SALEDNOTE','SALEDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('TAXFORMD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRETD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRETD','J','QTYX','QTYX','R');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRETD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRETD','N','PRETDNOTE','PRETDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PRETD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('INVPFMD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('INVPFMD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('INVPFMD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('INVPFMD','N','SALEDNOTE','SALEDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('INVPFMD','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='RKPPRETD' AND RVARID='N';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPPRETD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPPRETD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPPRETD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPPRETD','N','NOTE','NOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPPRETD','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='RKPSRETD' AND RVARID='N';
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPSRETD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPSRETD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPSRETD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPSRETD','N','NOTE','NOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('RKPSRETD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSWTLD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSWTLD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSWTLD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSWTLD','N','SALEDNOTE','SALEDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('POSWTLD','W','WHID','WHID','L');

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PURCD','I','PID','PID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PURCD','J','QTYX','QTYX','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PURCD','K','TAXID','TAXID','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PURCD','N','PURCDNOTE','PURCDNOTE','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('PURCD','W','WHID','WHID','L');

DELETE FROM RVAR WHERE RPTID='DN' AND RVARID='G1';
DELETE FROM RVAR WHERE RPTID='DN' AND RVARID='G2';
DELETE FROM RVAR WHERE RPTID='DN' AND RVARID='G3';
DELETE FROM RVAR WHERE RPTID='DN' AND RVARID='G4';


insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DN','G1','SHIPTO[1]','SHIPTO[1]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DN','G2','SHIPTO[2]','SHIPTO[2]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DN','G3','SHIPTO[3]','SHIPTO[3]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DN','G4','SHIPTO[4]','SHIPTO[4]','L');

DELETE FROM RVAR WHERE RPTID='DN_PID' AND RVARID='G1';
DELETE FROM RVAR WHERE RPTID='DN_PID' AND RVARID='G2';
DELETE FROM RVAR WHERE RPTID='DN_PID' AND RVARID='G3';
DELETE FROM RVAR WHERE RPTID='DN_PID' AND RVARID='G4';

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DN_PID','G1','SHIPTO','SHIPTO[1]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DN_PID','G2','SHIPTO','SHIPTO[2]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DN_PID','G3','SHIPTO','SHIPTO[3]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DN_PID','G4','SHIPTO','SHIPTO[4]','L');

DELETE FROM RVAR WHERE RPTID='DELI' AND RVARID='G1';
DELETE FROM RVAR WHERE RPTID='DELI' AND RVARID='G2';
DELETE FROM RVAR WHERE RPTID='DELI' AND RVARID='G3';
DELETE FROM RVAR WHERE RPTID='DELI' AND RVARID='G4';

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELI','G1','SHIPTO[1]','SHIPTO[1]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELI','G2','SHIPTO[2]','SHIPTO[2]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELI','G3','SHIPTO[3]','SHIPTO[3]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELI','G4','SHIPTO[4]','SHIPTO[4]','L');

DELETE FROM RVAR WHERE RPTID='DELI_RKP' AND RVARID='G1';
DELETE FROM RVAR WHERE RPTID='DELI_RKP' AND RVARID='G2';
DELETE FROM RVAR WHERE RPTID='DELI_RKP' AND RVARID='G3';
DELETE FROM RVAR WHERE RPTID='DELI_RKP' AND RVARID='G4';

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELI_RKP','G1','SHIPTO[1]','SHIPTO[1]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELI_RKP','G2','SHIPTO[2]','SHIPTO[2]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELI_RKP','G3','SHIPTO[3]','SHIPTO[3]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DELI_RKP','G4','SHIPTO[4]','SHIPTO[4]','L');

DELETE FROM RVAR WHERE RPTID='DO' AND RVARID='G1';
DELETE FROM RVAR WHERE RPTID='DO' AND RVARID='G2';
DELETE FROM RVAR WHERE RPTID='DO' AND RVARID='G3';
DELETE FROM RVAR WHERE RPTID='DO' AND RVARID='G4';

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DO','G1','SHIPTO[1]','SHIPTO[1]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DO','G2','SHIPTO[2]','SHIPTO[2]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DO','G3','SHIPTO[3]','SHIPTO[3]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DO','G4','SHIPTO[4]','SHIPTO[4]','L');

DELETE FROM RVAR WHERE RPTID='CONS_RKP' AND RVARID='G1';
DELETE FROM RVAR WHERE RPTID='CONS_RKP' AND RVARID='G2';
DELETE FROM RVAR WHERE RPTID='CONS_RKP' AND RVARID='G3';
DELETE FROM RVAR WHERE RPTID='CONS_RKP' AND RVARID='G4';

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONS_RKP','G1','SHIPTO','SHIPTO[1]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONS_RKP','G2','SHIPTO','SHIPTO[2]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONS_RKP','G3','SHIPTO','SHIPTO[3]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('CONS_RKP','G4','SHIPTO','SHIPTO[4]','L');

DELETE FROM RVAR WHERE RPTID='DNCONS' AND RVARID='G1';
DELETE FROM RVAR WHERE RPTID='DNCONS' AND RVARID='G2';
DELETE FROM RVAR WHERE RPTID='DNCONS' AND RVARID='G3';
DELETE FROM RVAR WHERE RPTID='DNCONS' AND RVARID='G4';

insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONS','G1','SHIPTO','SHIPTO[1]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONS','G2','SHIPTO','SHIPTO[2]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONS','G3','SHIPTO','SHIPTO[3]','L');
insert into rvar(rptid,rvarid,rvardesc,rvarfield,rvaralign) values('DNCONS','G4','SHIPTO','SHIPTO[4]','L');


ALTER TABLE crc ADD usrid VARCHAR(10) REFERENCES usr (usrid) ON DELETE RESTRICT ON UPDATE CASCADE;

CREATE SEQUENCE crclogid_seq;
CREATE TABLE crclog (
	crclogid		BIGINT NOT NULL DEFAULT NEXTVAL('crclogid_seq'),
	crclogdate	TIMESTAMP,
	crcid		VARCHAR(5),
	usrid		VARCHAR(10),
	excrateold	NUMERIC(19,4),
	fisrateold	NUMERIC(19,4),
	excratenew	NUMERIC(19,4),
	fisratenew	NUMERIC(19,4)
);




ALTER TABLE preq ADD printcount BIGINT DEFAULT 0;
ALTER TABLE po ADD printcount BIGINT DEFAULT 0;
ALTER TABLE prcv ADD printcount BIGINT DEFAULT 0;
ALTER TABLE purc ADD printcount BIGINT DEFAULT 0;
ALTER TABLE pret ADD printcount BIGINT DEFAULT 0;

ALTER TABLE sadj ADD printcount BIGINT DEFAULT 0;
ALTER TABLE sttr ADD printcount BIGINT DEFAULT 0;

ALTER TABLE cadj ADD printcount BIGINT DEFAULT 0;
ALTER TABLE cstr ADD printcount BIGINT DEFAULT 0;

ALTER TABLE so ADD printcount BIGINT DEFAULT 0;
ALTER TABLE deo ADD printcount BIGINT DEFAULT 0;
ALTER TABLE deli ADD printcount BIGINT DEFAULT 0;
ALTER TABLE sale ADD printcount BIGINT DEFAULT 0;
ALTER TABLE sret ADD printcount BIGINT DEFAULT 0;

ALTER TABLE pay ADD printcount BIGINT DEFAULT 0;
ALTER TABLE rcv ADD printcount BIGINT DEFAULT 0;

ALTER TABLE cons ADD printcount BIGINT DEFAULT 0;
ALTER TABLE cret ADD printcount BIGINT DEFAULT 0;

ALTER TABLE fcn ADD printcount BIGINT DEFAULT 0;
ALTER TABLE jnl ADD printcount BIGINT DEFAULT 0;




ALTER TABLE preq ADD version BIGINT DEFAULT 0;
ALTER TABLE po ADD version BIGINT DEFAULT 0;
ALTER TABLE prcv ADD version BIGINT DEFAULT 0;
ALTER TABLE purc ADD version BIGINT DEFAULT 0;
ALTER TABLE pret ADD version BIGINT DEFAULT 0;

ALTER TABLE sadj ADD version BIGINT DEFAULT 0;
ALTER TABLE sttr ADD version BIGINT DEFAULT 0;

ALTER TABLE cadj ADD version BIGINT DEFAULT 0;
ALTER TABLE cstr ADD version BIGINT DEFAULT 0;

ALTER TABLE so ADD version BIGINT DEFAULT 0;
ALTER TABLE deo ADD version BIGINT DEFAULT 0;
ALTER TABLE deli ADD version BIGINT DEFAULT 0;
ALTER TABLE sale ADD version BIGINT DEFAULT 0;
ALTER TABLE sret ADD version BIGINT DEFAULT 0;

ALTER TABLE pay ADD version BIGINT DEFAULT 0;
ALTER TABLE rcv ADD version BIGINT DEFAULT 0;

ALTER TABLE cons ADD version BIGINT DEFAULT 0;
ALTER TABLE cret ADD version BIGINT DEFAULT 0;

ALTER TABLE fcn ADD version BIGINT DEFAULT 0;
ALTER TABLE jnl ADD version BIGINT DEFAULT 0;


ALTER TABLE po ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE po ADD memorizednote TEXT;
ALTER TABLE purc ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE purc ADD isrecurring BOOLEAN DEFAULT FALSE;
ALTER TABLE purc ADD recurno VARCHAR(15) NULL;
ALTER TABLE purc ADD recurdno INT2 NULL;
ALTER TABLE purc ADD memorizednote TEXT;

ALTER TABLE sadj ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE sadj ADD memorizednote TEXT;
ALTER TABLE sttr ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE sttr ADD isrecurring BOOLEAN DEFAULT FALSE;
ALTER TABLE sttr ADD recurno VARCHAR(15) NULL;
ALTER TABLE sttr ADD recurdno INT2 NULL;
ALTER TABLE sttr ADD memorizednote TEXT;

ALTER TABLE cadj ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE cadj ADD memorizednote TEXT;
ALTER TABLE cstr ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE cstr ADD isrecurring BOOLEAN DEFAULT FALSE;
ALTER TABLE cstr ADD recurno VARCHAR(15) NULL;
ALTER TABLE cstr ADD recurdno INT2 NULL;
ALTER TABLE cstr ADD memorizednote TEXT;

ALTER TABLE so ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE so ADD memorizednote TEXT;
ALTER TABLE sale ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE sale ADD isrecurring BOOLEAN DEFAULT FALSE;
ALTER TABLE sale ADD recurno VARCHAR(15) NULL;
ALTER TABLE sale ADD recurdno INT2 NULL;
ALTER TABLE sale ADD memorizednote TEXT;

ALTER TABLE pay ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE pay ADD isrecurring BOOLEAN DEFAULT FALSE;
ALTER TABLE pay ADD recurno VARCHAR(15) NULL;
ALTER TABLE pay ADD recurdno INT2 NULL;
ALTER TABLE pay ADD memorizednote TEXT;
ALTER TABLE rcv ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE rcv ADD isrecurring BOOLEAN DEFAULT FALSE;
ALTER TABLE rcv ADD recurno VARCHAR(15) NULL;
ALTER TABLE rcv ADD recurdno INT2 NULL;
ALTER TABLE rcv ADD memorizednote TEXT;

ALTER TABLE cons ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE cons ADD memorizednote TEXT;

ALTER TABLE fcn ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE fcn ADD memorizednote TEXT;
ALTER TABLE jnl ADD ismemorized BOOLEAN DEFAULT FALSE;
ALTER TABLE jnl ADD isrecurring BOOLEAN DEFAULT FALSE;
ALTER TABLE jnl ADD recurno VARCHAR(15) NULL;
ALTER TABLE jnl ADD recurdno INT2 NULL;
ALTER TABLE jnl ADD memorizednote TEXT;

CREATE TABLE recur (
	recurno		VARCHAR(15),
	recurtype	VARCHAR(5),
	refno		VARCHAR(15),
	reftype		VARCHAR(5),
	recurcount	INT2,
	recurdesc	TEXT,
	active		BOOLEAN,
	finished	BOOLEAN,
	version		BIGINT DEFAULT 0,
	CONSTRAINT pk_recur PRIMARY KEY (recurno)
);

CREATE UNIQUE INDEX recur_pk ON recur (
	recurno
);

CREATE TABLE recurd (
	recurno		VARCHAR(15),
	recurdno	INT2,
	recurddate	DATE,
	recurdref	VARCHAR(15),
	recurddesc	TEXT,
	recurdtotal	NUMERIC(19,4),
	executed	BOOLEAN DEFAULT false,
	CONSTRAINT pk_recurd PRIMARY KEY (recurno, recurdno),
	CONSTRAINT fk_recurd_rel_recur FOREIGN KEY (recurno) REFERENCES recur (recurno) ON DELETE RESTRICT ON UPDATE CASCADE
);

CREATE UNIQUE INDEX recurd_pk ON recurd (
	recurno, recurdno
);

INSERT INTO ncount(noid, transcode, curno, nolen) VALUES ('RECUR', 'RCR', 1000, 8);

DROP TABLE recurtype CASCADE;
CREATE TABLE recurtype (
	recurtype 	VARCHAR(5) NOT NULL,
	recurtypedesc 	VARCHAR(225) NOT NULL,
	uprecurtype 	VARCHAR(5),
	upmultiplier 	INT2,
	CONSTRAINT pk_recurtype PRIMARY KEY (recurtype),
	CONSTRAINT fk_recurtype_rel_recurtype FOREIGN KEY (uprecurtype) REFERENCES recurtype (recurtype) ON DELETE RESTRICT ON UPDATE CASCADE
);

CREATE UNIQUE INDEX recurtype_pk ON recurtype (
	recurtype
);

INSERT INTO recurtype VALUES ('DLY', 'Harian', null, 1);
INSERT INTO recurtype VALUES ('WLY', 'Mingguan', null, 1);
INSERT INTO recurtype VALUES ('MLY', 'Bulanan', null, 1);
INSERT INTO recurtype VALUES ('YLY', 'Tahunan', null, 1);

CREATE TABLE deptprv (
	grpid		VARCHAR(10) NOT NULL,
	deptid		VARCHAR(5) NOT NULL,
	acsenable	BOOLEAN NULL DEFAULT false,
	acsval		VARCHAR(10) NULL,
CONSTRAINT deptprv_identifier UNIQUE (grpid, deptid),
CONSTRAINT FK_DEPTPRV_REL_GRP FOREIGN KEY (grpid) REFERENCES grp (grpid) ON DELETE RESTRICT ON UPDATE CASCADE,
CONSTRAINT FK_DEPTPRV_REL_DEPT FOREIGN KEY (deptid) REFERENCES dept (deptid) ON DELETE restrict ON UPDATE CASCADE 
);

CREATE INDEX deptprv_grpidx ON deptprv (
	grpid
);

CREATE INDEX deptprv_deptidx ON deptprv (
	deptid
);

--CREATE TABLE pphbadan (
--	pphbadanid	BIGINT,
--	startperiod	DATE,
--	endperiod	DATE,
--	pphbadantype	VARCHAR(5)
--);




ALTER TABLE fagrp ADD depreciatable BOOLEAN DEFAULT TRUE;



INSERT INTO obj(objid,upobjid,modulid,objname,objlevel,enabled) VALUES('517006','500000','INV','Stock Opname',2,true);

INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'NEW',true FROM obj WHERE objid IN ('517006');
INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'ENB',true FROM obj WHERE objid IN ('517006');
INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'OPN',true FROM obj WHERE objid IN ('517006');
INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'UPD',true FROM obj WHERE objid IN ('517006');
INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'DEL',true FROM obj WHERE objid IN ('517006');
INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'PRN',true FROM obj WHERE objid IN ('517006');
INSERT INTO OBJACL (objid, acstype, acsenable) SELECT objid,'VOI',true FROM obj WHERE objid IN ('517006');

INSERT INTO grpprv (grpid, objid, acstype, acsval, acsenable) 
SELECT 'ADM',objid,acstype,acsval,true FROM objacl WHERE objid IN ('517006');



ALTER TABLE tax ADD taxexpunreg VARCHAR(15) NOT NULL DEFAULT '';
UPDATE tax SET taxexpunreg=taxexp;

ALTER TABLE pay ADD taxmonth DATE DEFAULT NULL;



INSERT INTO OBJACL (objid, acstype, acsenable) VALUES('110001', 'BPACC', true);
INSERT INTO grpprv (grpid, objid, acstype, acsval, acsenable) SELECT 'ADM','110001','BPACC',acsval,true FROM objacl;

INSERT INTO OBJACL (objid, acstype, acsenable) VALUES('130002', 'QTYSETTING', true);
INSERT INTO grpprv (grpid, objid, acstype, acsval, acsenable) SELECT 'ADM','130002','QTYSETTING',acsval,true FROM objacl;


CREATE UNIQUE INDEX pk_taxsaleno ON sale (
	taxsaleno
);

CREATE UNIQUE INDEX pk_taxpurcno ON purc (
	taxpurcno
);




/** PReq */
UPDATE obj SET upobjid='200000', objlevel=2 WHERE objid='215101';
UPDATE obj SET upobjid='215101', objlevel=3 WHERE objid='215201';
UPDATE obj SET upobjid='215101', objlevel=3 WHERE objid='215001';

/** PO */
UPDATE obj SET upobjid='200000', objlevel=2 WHERE objid='215102';
UPDATE obj SET upobjid='215102', objlevel=3 WHERE objid='215202';
UPDATE obj SET upobjid='215102', objlevel=3 WHERE objid='215002';

/** PRcv */
UPDATE obj SET upobjid='200000', objlevel=2 WHERE objid='215103';
UPDATE obj SET upobjid='215103', objlevel=3 WHERE objid='215203';
UPDATE obj SET upobjid='215103', objlevel=3 WHERE objid='215004';

/** Purc */
UPDATE obj SET upobjid='200000', objlevel=2 WHERE objid='215104';
UPDATE obj SET upobjid='215104', objlevel=3 WHERE objid='215204';
UPDATE obj SET upobjid='215104', objlevel=3 WHERE objid='215005';

/** PRet */
UPDATE obj SET upobjid='200000', objlevel=2 WHERE objid='215107';
UPDATE obj SET upobjid='215107', objlevel=3 WHERE objid='215207';
UPDATE obj SET upobjid='215107', objlevel=3 WHERE objid='215006';

/** Purc FA */
UPDATE obj SET upobjid='215025', objlevel=3 WHERE objid='215105';
UPDATE obj SET upobjid='215025', objlevel=3 WHERE objid='215205';

/** PRcv Ret */
UPDATE obj SET upobjid='215010', objlevel=3 WHERE objid='215106';
UPDATE obj SET upobjid='215010', objlevel=3 WHERE objid='215206';

DELETE FROM grpprv WHERE objid IN ('215100', '215200');
DELETE FROM objacl WHERE objid IN ('215100', '215200');
DELETE FROM obj WHERE objid IN ('215100', '215200');

/** Rcv */
UPDATE obj SET upobjid='700000', objlevel=2 WHERE objid='715101';
UPDATE obj SET upobjid='715101', objlevel=3 WHERE objid='715201';
UPDATE obj SET upobjid='715101', objlevel=3 WHERE objid='715001';

/** Pay */
UPDATE obj SET upobjid='700000', objlevel=2 WHERE objid='715102';
UPDATE obj SET upobjid='715102', objlevel=3 WHERE objid='715202';
UPDATE obj SET upobjid='715102', objlevel=3 WHERE objid='715002';

DELETE FROM grpprv WHERE objid IN ('715100', '715200');
DELETE FROM objacl WHERE objid IN ('715100', '715200');
DELETE FROM obj WHERE objid IN ('715100', '715200');

/** SO */
UPDATE obj SET upobjid='800000', objlevel=2 WHERE objid='815301';
UPDATE obj SET upobjid='815301', objlevel=3 WHERE objid='815201';
UPDATE obj SET upobjid='815301', objlevel=3 WHERE objid='815002';

/** DEO */
UPDATE obj SET upobjid='800000', objlevel=2 WHERE objid='815302';
UPDATE obj SET upobjid='815302', objlevel=3 WHERE objid='815202';
UPDATE obj SET upobjid='815302', objlevel=3 WHERE objid='815003';

/** Deli */
UPDATE obj SET upobjid='800000', objlevel=2 WHERE objid='815303';
UPDATE obj SET upobjid='815303', objlevel=3 WHERE objid='815203';
UPDATE obj SET upobjid='815303', objlevel=3 WHERE objid='815004';

/** Sale */
UPDATE obj SET upobjid='800000', objlevel=2 WHERE objid='815304';
UPDATE obj SET upobjid='815304', objlevel=3 WHERE objid='815204';
UPDATE obj SET upobjid='815304', objlevel=3 WHERE objid='815005';

/** Sale FA */
UPDATE obj SET upobjid='800000', objlevel=2 WHERE objid='815010';
UPDATE obj SET upobjid='815010', objlevel=3 WHERE objid='815205';
UPDATE obj SET upobjid='815010', objlevel=3 WHERE objid='815305';

/** SRet */
UPDATE obj SET upobjid='800000', objlevel=2 WHERE objid='815306';
UPDATE obj SET upobjid='815306', objlevel=3 WHERE objid='815206';
UPDATE obj SET upobjid='815306', objlevel=3 WHERE objid='815006';

DELETE FROM grpprv WHERE objid IN ('815200', '815300');
DELETE FROM objacl WHERE objid IN ('815200', '815300');
DELETE FROM obj WHERE objid IN ('815200', '815300');




INSERT INTO OBJACL (objid, acstype, acsenable) values( '845005', 'DROW', true);
INSERT INTO grpprv (grpid, objid, acstype, acsval, acsenable) SELECT 'ADM',objid,acstype,acsval,true FROM objacl WHERE objid='845005' AND acstype='DROW';


ALTER TABLE prc ADD lastpurcprice NUMERIC(19,4) DEFAULT 0;
ALTER TABLE prc ADD lastavgcost NUMERIC(19,4) DEFAULT 0;
UPDATE prc SET lastpurcprice=price1, lastavgcost=price1;

ALTER TABLE cbg ADD useddate DATE;
UPDATE cbg SET useddate=pay.paydate FROM pay JOIN payd ON payd.payno=pay.payno WHERE payd.cbgid=cbg.cbgid;
UPDATE cbg SET useddate=rcv.rcvdate FROM rcv JOIN rcvd ON rcvd.rcvno=rcv.rcvno WHERE rcvd.cbgid=cbg.cbgid;


